package roboforge;

import java.io.*;
import javax.vecmath.*;
import javax.media.j3d.*;
import java.util.*;
//import javax.vecmath.*;
import java.awt.*;
//import roboforge.geometry.*;
import roboforge.sound.*;

import roboforge.animation.*;


public class B2dAnimation implements Serializable, BAnimation


{
  // Persistent members..

  private String name;
  private Vector frameNames = null;

  private boolean useLight;
  private Color3f lightColor;
  private boolean playSound;
  private float transparency;
  double scaleWeight;

  int currentFrame = START;

  //BComponent screenC;


  // Transient members..

  private transient OrientedShape3D screen = null;
  private transient Appearance projector = null;
  private transient BSound thisExpSound = null;

  private transient Texture[] frames = null;
  private transient TextureAttributes textureA;

  private transient Group parent;
  private transient BranchGroup myLocale;
  private transient TransformGroup scaleTrans;
  private transient Transform3D scale;

  private transient boolean readyForNextFrame = false;
  private transient int movementDirection;
  private transient boolean isLive;
  private transient BComponent parentComponent;

  // Constants..
  //private static final double LIGHTRADIUS = 5.0;
  private static final double SCREEN_WIDTH = 36.0;


  static final long serialVersionUID = -1L;
  //private transient BoundingSphere bounds;
  //private static Point3f ATTENUATION =  new Point3f(0,0.03f,0) ;

  private static final int START = -1;
  private static final int FORWARD = 666;
  private static final int BACKWARD = 888;

  private static int FRAME_TO_REMOVE_COMPONENT = 3;

  // 1/3 - used for converting volume to animation scale [see attach()].
  protected final static float ONE_THIRD = 1.0f / 3.0f;


  public B2dAnimation
  (
    String newName,
    float newTransparency,
    boolean addLight,
    Color3f newLightColor,
    double scaleWeight
  )
  {
    // newTransparency --> 0 == completely opaque apart from the transparent gif areas.

    this.lightColor   = newLightColor;
    this.useLight     = addLight;
    this.transparency = newTransparency;
    this.name         = newName;
    this.scaleWeight  = scaleWeight;

    init();
    reset();
  }


  public void init()
  {
    if (scaleWeight == 0.0)
      scaleWeight = 1.0;

    if ( screen == null)
    {
      //bounds = new BoundingSphere(BGlobal.INFINITE_BOUNDS);

      isLive = false;
      readyForNextFrame = false;
      ///

      projector = new Appearance();

      Material mat = new Material();
      mat.setDiffuseColor(lightColor);
      mat.setAmbientColor(BGlobal.WHITE_3F);
      mat.setSpecularColor(BGlobal.WHITE_3F);

      Color3f l = lightColor;

      //float r = l.x * 0.5f;
      //float g = l.y * 0.5f;
      //float b = l.z * 0.5f;

      //Color3f emittedColor = new Color3f(r, g, b);
      mat.setEmissiveColor(l);
      mat.setLightingEnable(false);

      textureA = new TextureAttributes();
      textureA.setTextureMode(TextureAttributes.MODULATE);

      TransparencyAttributes TA  = new TransparencyAttributes(TransparencyAttributes.BLENDED ,
                              transparency,
                              TransparencyAttributes.BLEND_SRC_ALPHA,
                              TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA);

      RenderingAttributes renderAttr = new RenderingAttributes();
	    renderAttr.setDepthBufferWriteEnable( false );
      projector.setRenderingAttributes(renderAttr);

      //new TransparencyAttributes(TransparencyAttributes.BLEND_SRC_ALPHA, transparency);
 

      projector.setTextureAttributes(textureA);
      projector.setTransparencyAttributes(TA);
      projector.setMaterial( mat );
      projector.setCapability(Appearance.ALLOW_TEXTURE_WRITE);

      //screen.setAppearance(projector);


      myLocale = new BranchGroup();
      myLocale.setCapability(BranchGroup.ALLOW_DETACH);

      scale = new Transform3D();
      scaleTrans = new TransformGroup(scale);
      myLocale.addChild(scaleTrans);

      Shape3D stub =  new BRectangle3D(SCREEN_WIDTH, SCREEN_WIDTH);
      screen = new OrientedShape3D(stub.getGeometry(), projector, OrientedShape3D.ROTATE_ABOUT_POINT, BPhysics.CENTEROFUNIVERSE);

      //HACK FOR j3d1.2.1 Beta1; (textures do not always update, -reset Appearance to force update
      //screen.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);

      scaleTrans.addChild(screen);



      //  INITIALISE  FRAMES
      if (frameNames != null)
        if (frameNames.size() > 0 )
          for (int i = 0; i < frameNames.size() ; i++ )
            addFrame((String) frameNames.get(i), false);


      }
  }

  public void reset()
  {
    currentFrame = START;
  }

  public void setPosition(Group toAttach, BComponent parentComponentnew, boolean playSoundnew)
  {
    if ( !isLive  )
    {
      playSound = playSoundnew;
      parent = toAttach;
      parentComponent = parentComponentnew;
    }
  }

  private void attach()
  {
    try
    {
      isLive = true;

      if (parentComponent != null)
        scaleToComponent(parentComponent);
      else
        scaleDefault();

      parent.addChild(myLocale);

      if ((thisExpSound != null) && playSound)
        thisExpSound.play();
    }
    catch (Exception e)
    {
      BDebug.print ("B2dAnimation.attach ERROR " + e.getMessage() );
      BDebug.print ("component " + parentComponent);
      
    }
  }

  protected void scaleToComponent(BComponent c)
  {
    double s = Math.pow(c.getVolume(), ONE_THIRD) * scaleWeight;
    scale.set(s);
    scaleTrans.setTransform(scale);
  }

  protected void scaleDefault()
  {
    scale.set(scaleWeight);
    scaleTrans.setTransform(scale);
  }


  public void addFrame(String frameName)
    { addFrame(frameName, true); }

  public void addFrame(String frameName, boolean saveFrameName)
  {
    Texture tempT = BTextures.getTexture(BGlobal.ANIMATIONPATH + frameName);
    addFrame(tempT,  frameName, saveFrameName);
  }

  public void addFrame(Texture nextFrame, String frameName, boolean saveFrameName)
  {
    if (frames == null)
    {
      frames = new Texture[1]; // new Vector();
      frames[0] = nextFrame;
    }
    else
    {
      Texture[] old = frames;
      int n = frames.length;

      frames = new Texture[n+1];
      for (int i=0; i<n; i++)
        frames[i] = old[i];

      frames[n] = nextFrame;
    }

    if (saveFrameName)
    {
      if (frameNames == null)
        frameNames = new Vector();
      frameNames.add(frameName);
    }
  }

  public void playNext()
  {
    // We need to set a flag here, because we can only swap appearances in postSwap
    currentFrame = java.lang.Math.min(currentFrame + 1, frameNames.size()-1) ;
    movementDirection = FORWARD;
    readyForNextFrame = true;


  }

  public void playPrevious()
  {
    // We need to set a flag here, because we can only swap appearances in postSwap
    currentFrame = java.lang.Math.max(currentFrame - 1 ,START);
    movementDirection = BACKWARD;
    readyForNextFrame = true;
  }


  public void skipTo(int i)
  {
    int old = currentFrame;
    currentFrame = i;
    movementDirection = (old < i) ? FORWARD : BACKWARD;
    readyForNextFrame = true;

    swapFrames();
  }



  public void setName(String name)
    { this.name = name; }

  public String getName()
    { return name; }


  public void setUseLight(boolean useLight)
    { this.useLight = useLight; }

  public boolean isUseLight()
    { return useLight; }

  public void setLightColor(Color c)
  {
    float r = c.getRed()   / 255.0f;
    float g = c.getGreen() / 255.0f;
    float b = c.getBlue()  / 255.0f;

    this.lightColor = new Color3f(r, g, b);
  }

  public Color getLightColor()
  {
    if (lightColor != null)
      return (new Color(lightColor.x, lightColor.y, lightColor.z));
    else
      return null;
  }


  public void setPlaySound(boolean playSound)
    { this.playSound = playSound; }

  public boolean isPlaySound()
    { return playSound; }

  public void setSound(BSound newSound)
    { thisExpSound = newSound; }

  public BSound getSound()
    { return thisExpSound; }


  public void setScaleWeight(double scaleWeight)
    { this.scaleWeight = scaleWeight; }

  public double getScaleWeight()
    { return scaleWeight; }


  public void setTransparency(float transparency)
    { this.transparency = transparency; }

  public float getTransparency()
    { return transparency; }


  // Implementation of roboforge.animationBAnimation
  //==========================================================

  public void start()
  {
    // BDebug.print("B2dAnimation.start()");
    currentFrame = 0;
    projector.setTexture(frames[0]);
    attach();
  }

  public void stop()
  {
    // BDebug.print("B2dAnimation.stop()");
    remove();
  }

  public boolean animate()

  {
    // BDebug.print("B2dAnimation.animate()");

    if (currentFrame == frames.length/* -1 */ )
      return false;
    if (projector.isLive() )
    {
      projector.setTexture(frames[currentFrame]);
      currentFrame ++;

      //HACK FOR j3d1.2.1 Beta1; (textures do not always update, -reset Appearance to force update
      //screen.setAppearance(null);
      //screen.setAppearance(projector);


      if (currentFrame == FRAME_TO_REMOVE_COMPONENT)
        if (parentComponent != null)
        {
          BranchGroup n = parentComponent.getLiveNode();
          n.detach();
        }

    }

    return true;



  }


  //==========================================================




  public boolean swapFrames()
  // OLD ANIMATION CALL
  // DEPRECATED
  // DO NOT CALL THIS ANYWHERE EXCEPT Canvas3d.postswap.
  // There is a bug, that will cause the prog to hangs if you change textures of a object rapidly outside postswap

  {
    // DEPRECATED
       // BDebug.print("B2dAnimation.swapFrames currentFrame " +  currentFrame + " DIRECTION "  + movementDirection );
        //BDebug.print("B2dAnimation.swapFrames " +   parentComponent );

        //BDebug.print("B2dAnimation.swapFrames  noOfFrames -1  " +   currentFrame  );

    // DEPRECATED
        boolean played = false;

//        if ( (currentFrame == frameNames.size() -1 ) && ( movementDirection == FORWARD ) )
        if ( (currentFrame == frameNames.size() ) && ( movementDirection == FORWARD ) )
                remove();
        else if ( (currentFrame == START) && ( movementDirection == BACKWARD ) )
                remove();
        else
        {


                if (readyForNextFrame)
                {

                        if (((currentFrame >= START + 1 ) &&   (movementDirection == FORWARD)) &&  !isLive )
                          attach();

                        if (projector.isLive() )
                        {
                          //BDebug.print ("B2dAnimation.plaNextFrame playing " + currentFrame);
                          projector.setTexture(null);
                          projector.setTexture(frames[currentFrame]);

                          //HACK FOR j3d1.2.1 Beta1; (textures do not always update, -reset Appearance to force update
                          //screen.setAppearance(null);
                          //screen.setAppearance(projector);

                          readyForNextFrame = false;
                          played = true;

                        }
                        else
                          remove();

                }

        }

        return played;


  }

  public void saveToFile(String filename )
  {
    try
    {
      if (!BGlobal.B2DANIMATIONSUFFIX.equalsIgnoreCase(filename.substring(filename.length() - 4 )))
        filename = filename +  BGlobal.B2DANIMATIONSUFFIX;

      FileOutputStream fout = new FileOutputStream(filename);
      ObjectOutputStream out = new ObjectOutputStream(fout);
      out.writeObject(this);
      out.close();
      System.out.println(filename+" saved.");
    }
    catch (Exception e)
      { BDebug.print("B2dAnimation.writeFile "+ e); }
  }

  public boolean getIsLive()
  { return isLive; }

  public void remove()
  {
    isLive = false;
    myLocale.detach();


  }

  public void cleanse()
    { screen.setAppearance(null); }

  public static B2dAnimation loadFromFile(String filename)
  {
    B2dAnimation newIn = null;

    try
    {
      FileInputStream fin = new FileInputStream(filename);
      ObjectInputStream in = new ObjectInputStream(fin);


      newIn = (B2dAnimation)in.readObject();
      newIn.init();
      newIn.reset();

      in.close();
      fin.close();


    }
    catch (Exception e)
      { BDebug.print("B2dAnimation.loadFromFile ERROR " + e); }

    return newIn;
  }

  public Vector getFrameNames()
  {
    return frameNames;
  }

  public boolean getUseLight()
  {
    return useLight;
  }


}