package roboforge;

import com.sun.j3d.utils.behaviors.mouse.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.j3d.*;
import javax.vecmath.*;
//import com.sun.j3d.utils.behaviors.picking.*;


public class BCameraMouseMove extends MouseBehavior{


   double x_factor = .5;
   double y_factor = .5;
   Vector3d translation = new Vector3d();

   // the constriants of movement for this behavior
   private float ymin;
   private float ymax;
   private float xmin;
   private float xmax;
   private float zmin;
   private float zmax;

   //private static float WALK_TOP = 0.7f;
   private static float WALK_BOTTOM = 0.1f;
   //private static float WALK_DIFF = 0.04f;


   boolean up = true;
   float lasty = WALK_BOTTOM;

   private BArenaCanvas parentCanvas = null;


   boolean useWalk;
   // useWalk indicates the walking bounce

  private MouseBehaviorCallback callback = null;

  /**
   * Creates a mouse translate behavior given the transform group.
   * @param transformGroup The transformGroup to operate on.
   */
  public BCameraMouseMove(TransformGroup transformGroup) {
    super(transformGroup);
  }


  /**
   * Creates a default translate behavior.
   */
  public BCameraMouseMove(int flags, float newXmin, float newXmax, float newYmin, float newYmax, float newZmin, float newZmax, boolean newUseWalk,
                          BArenaCanvas parentCanvasnew)
  {
    super(flags);

    parentCanvas = parentCanvasnew;

    ymin =  newYmin;
    ymax = newYmax;
    xmin =  newXmin;
    xmax =  newXmax;
    zmin =  newZmin;
    zmax = newZmax;

    useWalk = newUseWalk;




  }

  /**
   * Creates a translate behavior.
   * Note that this behavior still needs a transform
   * group to work on (use setTransformGroup(tg)) and
   * the transform group must add this behavior.
   * @param flags
   */
   /*
  public BCameraMouseMove(int flags) {
    super(flags);
  }  */

  public void initialize() {
    super.initialize();
    if ((flags & INVERT_INPUT) == INVERT_INPUT) {
       invert = true;
       x_factor *= -1;
       y_factor *= -1;
    }
  }

  /**
   * Return the x-axis movement multipler.
   **/

  public double getXFactor() {
    return x_factor;
  }

  /**
   * Return the y-axis movement multipler.
   **/

  public double getYFactor() {
    return y_factor;
  }

  /**
   * Set the x-axis amd y-axis movement multipler with factor.
   **/

  public void setFactor( double factor) {
    x_factor = y_factor = factor;

  }

  /**
   * Set the x-axis amd y-axis movement multipler with xFactor and yFactor
   * respectively.
   **/

  public void setFactor( double xFactor, double yFactor) {
    x_factor = xFactor;
    y_factor = yFactor;
  }


  // Local Variable
  // DO NOT USE ANYWHERE EXCEPT processStim
  private Vector3f processStimulusDisplacement = new Vector3f();

  public void processStimulus (Enumeration criteria) {
    WakeupCriterion wakeup;
    AWTEvent[] event;
    int id;
    int dx, dy;

    while (criteria.hasMoreElements()) {
      wakeup = (WakeupCriterion) criteria.nextElement();

      if (wakeup instanceof WakeupOnAWTEvent) {
	      event = ((WakeupOnAWTEvent)wakeup).getAWTEvent();
	    for (int i=0; i<event.length; i++) {
	      processMouseEvent((MouseEvent) event[i]);

	    if (((buttonPress)&&((flags & MANUAL_WAKEUP) == 0)) ||
	      ((wakeUp)&&((flags & MANUAL_WAKEUP) != 0)))
      {
       id = event[i].getID();
       //if ((id == MouseEvent.MOUSE_DRAGGED) &&
		   //    !((MouseEvent)event[i]).isAltDown() &&
		   //     ((MouseEvent)event[i]).isMetaDown())
       if ((id == MouseEvent.MOUSE_DRAGGED) &&
		        ((MouseEvent)event[i]).isMetaDown())
       {
	      x = ((MouseEvent)event[i]).getX();
	      y = ((MouseEvent)event[i]).getY();

	      dx = x - x_last;
	      dy = y - y_last;

        // Dg: Added to use ALT as a multiplier
        if ( ((MouseEvent)event[i]).isAltDown() )
        {
          dx = dx * 8;
          dy = dy * 8;
        }

	      if ((!reset) && ((Math.abs(dy) < 400) && (Math.abs(dx) < 400))){
		    //System.out.println("dx " + dx + " dy " + dy);
		    transformGroup.getTransform(currXform);

		    translation.x = -dx*x_factor;
        translation.z = -dy*y_factor;

		    transformX.set(translation);

		    if (invert) {
		      currXform.mul(currXform, transformX);
		    } else {
		      currXform.mul(transformX, currXform);
		    }


      currXform.get(processStimulusDisplacement);


      //cap the movements
      processStimulusDisplacement.x = java.lang.Math.max(processStimulusDisplacement.x, xmin);
      processStimulusDisplacement.x = java.lang.Math.min(processStimulusDisplacement.x, xmax);

      if (!BGlobal.DEVELOPER_MODE )
      {
        processStimulusDisplacement.y = java.lang.Math.max(processStimulusDisplacement.y, ymin);
        processStimulusDisplacement.y = java.lang.Math.min(processStimulusDisplacement.y, ymax);
      }
      
      processStimulusDisplacement.z = java.lang.Math.max(processStimulusDisplacement.z, zmin);
      processStimulusDisplacement.z = java.lang.Math.min(processStimulusDisplacement.z, zmax);

      currXform.setTranslation(processStimulusDisplacement);



      transformGroup.setTransform(currXform);
      transformChanged( currXform );

      if (parentCanvas != null)
        if (parentCanvas.getMode() == BGlobal.CANVAS_MODE_ARENA_AUTOCENTER)
         parentCanvas.lookAt(false);


	    	if (callback!=null)
		    callback.transformChanged( MouseBehaviorCallback.TRANSLATE,
					       currXform );

	      }
	      else
        {
		      reset = false;
	      }

	      x_last = x;
	      y_last = y;

	    }
	    else if (id == MouseEvent.MOUSE_PRESSED)
      {
	      x_last = ((MouseEvent)event[i]).getX();
	      y_last = ((MouseEvent)event[i]).getY();
	    }

	  }
	    }
      }
    }

    wakeupOn (mouseCriterion);
  }

  /**
    * Users can overload this method  which is called every time
    * the Behavior updates the transform
    *
    * Default implementation does nothing
    */
  public void transformChanged( Transform3D transform ) {
  }

  /**
    * The transformChanged method in the callback class will
    * be called every time the transform is updated
    */
  public void setupCallback( MouseBehaviorCallback callback ) {
      this.callback = callback;
  }
}


