package roboforge;

import javax.vecmath.*;
import java.util.*;
import javax.media.j3d.*;
import javax.swing.tree.*;

//import com.sun.j3d.loaders.ParsingErrorException;
//import com.sun.j3d.loaders.IncorrectFormatException;
import com.sun.j3d.loaders.Scene;
//import com.sun.j3d.utils.geometry.*;
import java.io.*;
import java.awt.Color;
//import java.awt.Panel;
//import com.sun.j3d.utils.image.TextureLoader;

//import roboforge.animation.*;
import roboforge.attributes.*;
import roboforge.btree.*;
import roboforge.math.*;
//import roboforge.geometry.*;
//import roboforge.sound.*;
import roboforge.arenaSystems.*;
import roboforge.ai.worldstate.BWorldState;

import roboforge.util.text.BStrings;

public class BComponent extends Object 
  implements Serializable, Cloneable, BArenaEvent, BPaintable
{
/*

* Description:
Created from Visio Diagram, Component Superclass.

* Example:
* Modification History ( Name, Date, Notes ):

*/

  static final long serialVersionUID = -1L;

  // Component Members

  // IF YOU CHANGE THIS YOU MUST RE_CALCULATE THE COLLISION HASH TABLE IN BBOT
  protected String name;                    // Short description

  private Integer serialNum;                // Internal Serial Number
  private float app_version;                // Game Version Number
  private String details;                   // 200 Character Blurb
  private boolean uniqueItem = false;

  public float weight;                      // = Throwing Mass (KG)
  //public float energyUsage;               // Watts used per Second (W/S) (No More Static Usage)
  public float price;                       // price of the component ($)
  public int damageMax;                     // Damage units - typical component around 1000+
  //public float damageDealStriking;        // Damage units per KG/M/S (Mass * Velocity)
  //private float maxLimbLoad;                // The Max Limb load that this limb can take.
  private float volume;
  private boolean chassisLock;              // Component is part of Chassis. Cannot be deleted.
  protected String componentFilePath;       // Path to original component (used by Upgrade Process)

  protected Vector connectPoints;
  protected BConnectPoint connectPointParent = null;
  protected BConnectPoint connectPointBase = null;
  private double zAlignment = 0;            // Z the rotate around the connection
  protected String structure3DFileName;

  protected Color componentColor;
  private float componentShine;
  private String textureName;
  protected float transparency;
  protected int colourDepth = 100;
  private boolean isConvex = false;
  protected transient BBot ownerBot; // Bot this component belongs to
  private boolean autoTextureGen = false;


  private String materialCode = "";

  private String ambientSoundName = null;
  //private transient BPointSound3D ambientSound;


  private transient BComponentNode treeNode;
  // XXXXXX  Game Engine transient fields
  transient public float limbLoad;   // Torque  (newton metres)  The direction of the torque is always assumed to be tangental to the acceleration.
                                    // this is just for a gross load calculation.
  transient public float limbWeight; // actual weight of objects on a limb
  transient public Vector3f limbRadius3D; // Torque in 3 dimensions used for angular acceleration torque calculation
  transient public Vector3f limbRadius2D;
  transient public Vector3f angVelocity; // Direction and distance of rotation
  transient public Vector3f prevAngVelocity; // previous direction and distance of rotation
  transient protected int damageCurrent;      // Accumulated damage units
  transient public boolean childComponentCollided = false;
  transient public Vector3f currentInertia;   // Current Velocity and Direction of Component
  transient public Vector3f childInertia;  // Inertia of children used in force/damage calculation
  transient public Vector3f parentInertia;  // Inertia of parent used in force/damage calculation
  transient public Vector3f collisionInertia; // Last Collision Inertia
 // transient public BComponent collisionComponent; // Component collided with
  transient public boolean inResultantMove = false;     // flag to indicate the move is in a resultant move, so process collisions
                                                // set in BBot create ResultantMove, Cleared in BMove.execute

  transient private Group directParent;  // This is set in BBot.loadComponent.
                                         // It's the Node that this.liveNode is a direct child to.
                                         // This is use in the ArenaCanvas for re-connecting, and explosions
  transient private Point3d lastCollidedp1 =  null;
  transient private Point3d lastCollidedp2 =  null;
  transient protected Shape3D lastCollidedMemberShape = null;
  transient protected BShape3D lastCollidedShape =  null;

  transient public BComponentMaterial componentmaterial;

  // Transient Graphic fields
  transient protected BranchGroup liveBotNode = null;
  transient BranchGroup myShapesNode = null;

  // SJS [17/02/2000] - Added BG nodes for selection.
  transient BranchGroup   myConnectPtsNode    = null;
  transient BranchGroup   mySelectionBoxNode = null;
  transient BSelectionBox mySelectionBox     = null;

  transient protected Appearance normalAppearance;
  transient protected BranchGroup structure3D;
  transient private BConnectPoint connectPointSelected = null;
  transient private TransformGroup connectRotateNode = null;
  transient private BShape3D arbitraryShape = null;
  transient private BShape3D arbitraryShape_live = null;
  transient protected Material componentMaterial; // the appearance material (Not Component Material)
  transient protected ColoringAttributes cAtrib;
  transient protected RenderingAttributes rAttrib;


  // volume calcuation transient fields
  static transient protected float vMinX;
  static transient protected float vMinY;
  static transient protected float vMinZ;

  static transient protected float vMaxX;
  static transient protected float vMaxY;
  static transient protected float vMaxZ;
  // CONSTANTS

  private static int TRAVERSE_MODE_EXTRACT = 1;
  private static int TRAVERSE_MODE_SETAPP = 2;
  private static int TRAVERSE_MODE_EXTRACT_FORLIVE = 3;
  protected static int TRAVERSE_MODE_VOLUME = 4;
  private static int TRAVERSE_MODE_CONVERT_TO_LIVE = 5;
  private static int TRAVERSE_MODE_CLEANSE = 7;
  private static int TRAVERSE_MODE_POLYCOUNT= 8;

  public static final Color DEFAULTCOLOR = new Color(0.5f,0.5f,0.5f);
  public static final float DEFAULTSHINE = 10f;
  public static final int DEFAULTDEPTH = 100;

  //protected static transient final LineAttributes componentLine = new LineAttributes(1.0f, LineAttributes.PATTERN_SOLID ,false);
  //protected static transient final PolygonAttributes pa = new PolygonAttributes(PolygonAttributes.POLYGON_LINE, PolygonAttributes.CULL_BACK, 0.0f) ;
  public static transient String stateCode = "components";

  transient Vector liveShapes = new Vector();

  transient int polyCount; // initialised in loadShapes, calculated in traverseshapes // MODE_EXTRACT

  // Calc Limb Load/Inertia transients
  transient private Vector3f limbTorque;
  transient private Vector3f totalRadius3D;
  transient protected Vector3f prevRadius3D;
  transient public Vector3f parentRadius3D;
  transient public Vector3f prevParentRadius3D;
  transient public Vector3f parentAngVelocity;
  transient public Vector3f prevParentAngVelocity;
  transient protected float angAcceleration;
  transient protected float directAcceleration;
  transient public int childCount;
  transient protected int prevChildCount;
  transient private Vector3f tempV3D;
  transient private Transform3D tempT;
  transient protected Vector3f currentVWorld;
  transient protected Vector3f prevVWorld;
  transient protected Vector3f tempVWorld;

  transient private boolean isTopOfStableLimb = false;

  //transient private boolean inHighlight         = false;
  transient protected boolean isSelected = false;
  private transient boolean showingConnectPoints = false;

    //---DJG---
  // 0 = Not Exceeded - accelerate as normal
  // 1 = Stop Acceleration of limb
  // 2 onwards = deceleration scale
  transient public int maxVelocityExceededCtr = 0;
  transient public float maxVelocityForLimb = 0; // M/Sec
  transient public float componentVelocity = 0; // M/Sec
  //---DJG---

  // DJG June 2001
  transient public boolean hasMoved = false; // Has component moved in the last timeslice
  transient public boolean collisionCandidate = false; // Likely to collide with enemy in next CALC_TIMESLICE
  transient public float boundingRadius = 0.0f; // For collision optimisation
  transient public float currentX = 0.0f;
  transient public float currentY = 0.0f;
  transient public float currentZ = 0.0f;
  transient public double currentAbsRadius = 0.0d;



  public BComponent(String newName)
  {

    name = newName;
    connectPoints = new Vector();
    app_version = BGlobal.APP_VERSION;

  }


  public static String COMPONENT_TYPE = "Component";
  public static String UNDEFINED_COMPONENT_TYPE = "Undefined";

  public String getComponentType()
  { return COMPONENT_TYPE; }


  public BComponent(String newName, String filename)
  /*

  * Description:
        Constructs Bot with Structure3D.
  * Parameters:
	* Results:
  * Example:
  * Modification History ( Name, Date, Notes ):
                      YLY, 6 Apr implememted cloning
                      HSC, 6 Apr created Set and Get methods for all primatives.


  */


  {

   this(newName);
   structure3DFileName = filename;
   Scene s = loadShape();
   parseLights(s);
  }   // BComponent



  public boolean isChassisLockLeaf()
  {

      DefaultMutableTreeNode treeNode =  getTreeNode();

      if (treeNode == null)
      {
        BDebug.print( "BComponent.isChassisLockLeaf DETECTED invalid component FOUND "   );
        return false;
      }

      boolean noChildrenAreChassisLock = true;



      if (treeNode.isLeaf() )
        noChildrenAreChassisLock = true;
      else if ( getChassisLock() )
      {



        Enumeration immediateChildren = treeNode.children();
        while (immediateChildren.hasMoreElements())
        {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)immediateChildren.nextElement();

            //BDebug.print ("BComponent isChassisLockLeaf " + treeNode );

            if (node.getUserObject() != null)
              noChildrenAreChassisLock = noChildrenAreChassisLock && !((BComponent)node.getUserObject()).getChassisLock();
            

        }



      }

       //BDebug.print ("BComponent noChildrenAreChassisLock " + noChildrenAreChassisLock  );
      // BDebug.print ("BComponent getChassisLock() " + getChassisLock()  );


      return  noChildrenAreChassisLock && getChassisLock()  ;



  }


   public boolean getDestroyed()
  {
    return damageMax <= damageCurrent;

  }

  public void destroy()
  {
     damageCurrent = damageMax ;

  }


  public void init(boolean optimise)
  {
    loadShape();
    damageCurrent = 0;
    componentmaterial = BComponentMaterial.getMaterial(materialCode);

    // fill in default weight to stop divide by zero errors
    if (this.weight == 0)
      this.weight = 0.00000001f;


    if (volume == 0.0)
      volume = guessVolume();

    //inHighlight = false;


  }

  protected void setAppearance()
  /*

  * Description:
        Initailises all the Transient Appearance Objects
  */

    {

    cAtrib =new ColoringAttributes();
    cAtrib.setCapability(ColoringAttributes.ALLOW_SHADE_MODEL_WRITE);
    cAtrib.setCapability(ColoringAttributes.ALLOW_COLOR_WRITE);

    rAttrib = new RenderingAttributes();
    rAttrib.setCapability(RenderingAttributes.ALLOW_VISIBLE_WRITE);


    componentMaterial = new Material();
    componentMaterial.setCapability(Material.ALLOW_COMPONENT_WRITE);
    TextureAttributes newTexA = new TextureAttributes();
    newTexA.setTextureMode(TextureAttributes.MODULATE);

    normalAppearance = new Appearance();
    normalAppearance.setMaterial(componentMaterial);
    normalAppearance.setTextureAttributes(newTexA);
    normalAppearance.setRenderingAttributes(rAttrib);
    //normalAppearance.setLineAttributes(componentLine);

    if (autoTextureGen && BGlobal.USE_AUTOGEN)
    {

      TexCoordGeneration t_gen = new TexCoordGeneration(TexCoordGeneration.SPHERE_MAP, TexCoordGeneration.TEXTURE_COORDINATE_2  ) ;
      normalAppearance.setTexCoordGeneration(t_gen);
    }

    normalAppearance.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
    normalAppearance.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_READ);
    normalAppearance.setCapability(Appearance.ALLOW_MATERIAL_READ);
    normalAppearance.setCapability(Appearance.ALLOW_LINE_ATTRIBUTES_READ);
    normalAppearance.setCapability(Appearance.ALLOW_POINT_ATTRIBUTES_READ);
    normalAppearance.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_READ);
    normalAppearance.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_WRITE);
    normalAppearance.setCapability(Appearance.ALLOW_RENDERING_ATTRIBUTES_READ);
    normalAppearance.setCapability(Appearance.ALLOW_TEXGEN_READ);
    normalAppearance.setCapability(Appearance.ALLOW_TEXTURE_READ);
    normalAppearance.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
    normalAppearance.setCapability(Appearance.ALLOW_TEXTURE_WRITE);
    normalAppearance.setCapability(Appearance.ALLOW_TEXTURE_ATTRIBUTES_READ);
    normalAppearance.setCapability(Appearance.ALLOW_TEXGEN_WRITE);
    normalAppearance.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
    normalAppearance.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_WRITE);
    normalAppearance.setCapability(Appearance.ALLOW_LINE_ATTRIBUTES_WRITE);
    normalAppearance.setCapability(Appearance.ALLOW_POINT_ATTRIBUTES_WRITE);
    normalAppearance.setCapability(Appearance.ALLOW_RENDERING_ATTRIBUTES_WRITE);

    if (componentColor == null)
      this.setColor(DEFAULTCOLOR, DEFAULTSHINE, DEFAULTDEPTH);

   }


  public void setAppearance(String newTexture, Color newColor, float newShine , int newDepth)
    { setAppearance(newTexture, newColor, newShine, transparency, newDepth); }

  public void setAppearance(String newTexture, Color newColor, float newShine, float newTrans, int newColourDepth  )
  {
     this.setTexture(newTexture);
     this.setColor(newColor, newShine, newColourDepth);
     this.setTransparency(newTrans );
  }

  public void setAppearance(BComponentAppearance ca)
    { setAppearance(ca, true); }

  public void setAppearance(BComponentAppearance ca, boolean applyTransparency)
  {
    setTexture(ca.getTexture());
    setColor(ca.getColor(), ca.getShine(), ca.getColorDepth());
    
    if (applyTransparency)
      setTransparency(ca.getTransparency());
  }
  
  public void setEnableLighting(boolean on)
  {
    normalAppearance.getMaterial().setLightingEnable(on);
  }

  public BComponentAppearance getAppearance()
  {
    return new BComponentAppearance(
      componentColor,
      colourDepth,
      componentShine,
      textureName, 
      transparency);
  }

  public void replaceStructureANDConnectPoints(String newVRML)
   {
    structure3DFileName = newVRML;
    connectPoints = new Vector();

    Scene s = loadShape();
    parseLights(s);

    setNormalAppearance();


   }

   public int getPolyCount()
   {
      polyCount = 0;
      traverseShapes(structure3D, structure3D, TRAVERSE_MODE_POLYCOUNT);
      return polyCount;
   }

  protected Scene loadShape()
  {

    //BDebug.print ("structure3DFileName " + structure3DFileName );
    //String suffix =  structure3DFileName.substring( structure3DFileName.length()-4, structure3DFileName.length());

    Scene s = B3dShapeManager.getScene( BGlobal.VRMLPATH +  structure3DFileName);
    structure3D = new BranchGroup();

    setAppearance();
    traverseShapes(s.getSceneGroup(), structure3D, TRAVERSE_MODE_EXTRACT);
    structure3D.setCapability(Group.ALLOW_CHILDREN_READ);



    String fileName = structure3DFileName.substring(0, structure3DFileName.length()-4) + BGlobal.J3DSUFFIX;
    File tester = new File (BGlobal.VRMLPATH + fileName);
    
    if (!tester.exists() )
    {
     // BDebug.print ( fileName + " does not exist ");

      if (B3dShapeManager.saveBranchGroup( BGlobal.VRMLPATH  + fileName, structure3D ) )
        BDebug.print (BGlobal.VRMLPATH  + fileName + " converted and saved" );
      else
        BDebug.print (BGlobal.VRMLPATH  + fileName + " ERROR IN SAVE" );
    }

    setAppearance(textureName, componentColor, componentShine, transparency, colourDepth );
    return s;

  }



protected void traverseShapes ( Node oldBranch, Group newBranch, int mode)
{

  if ( oldBranch instanceof Group )
  {
        Group currentGroup = (Group) oldBranch;

        for (Enumeration e = currentGroup.getAllChildren(); e.hasMoreElements();)
                {
                Node newGroup =(Node)e.nextElement();

                if ( (newGroup instanceof TransformGroup) &&
                      ( (mode == TRAVERSE_MODE_EXTRACT)  || (mode == TRAVERSE_MODE_EXTRACT_FORLIVE)  )            )
                {
                        TransformGroup oldTrans = (TransformGroup) newGroup;
                        Transform3D tempTrans = new Transform3D();
                        oldTrans.getTransform(tempTrans);
                        TransformGroup newTrans = new TransformGroup( tempTrans);
                        newTrans.setUserData(getName() + "Imported TransformGroup ");


                        traverseShapes(oldTrans,  newTrans, mode );
                        newBranch.addChild(newTrans);

                }
                else
                     traverseShapes(newGroup,  newBranch, mode );

                }   // loop
        }   // if
  else  if ( (oldBranch instanceof Shape3D )  )
  {
                if ((mode == TRAVERSE_MODE_EXTRACT )  || (mode == TRAVERSE_MODE_EXTRACT_FORLIVE) )
                {

                        Shape3D tempS = (Shape3D)oldBranch;

                        if (!tempS.getGeometry().isLive()  && !tempS.getGeometry().isCompiled() )
                        {
                          tempS.getGeometry().setCapability(Geometry.ALLOW_INTERSECT );
                          tempS.getGeometry().setCapability(GeometryArray.ALLOW_FORMAT_READ);
                        }

                        // remove texture if BGlobal.SHOW_TEXTURES is false.. This is for the reloading after
                        // a preferencedialog has been set
                        //this.setTexture(textureName);

                        BShape3D tempB;
                        Appearance theApp = this.normalAppearance;
                        tempB  = new BShape3D( tempS.getGeometry(), theApp, this);

                        // DJG June 2001
                        boundingRadius = java.lang.Math.max(boundingRadius,BShape3D.boundingRadius);
                        //BDebug.print("BComponent.traverseShapes: "+this.toString()+" radius="+boundingRadius);

                        if  (mode == TRAVERSE_MODE_EXTRACT_FORLIVE )
                       {

                          //tempB  = new BShape3D( tempS.getGeometry(), (Appearance)this.normalAppearance.cloneNodeComponent(), this);

                          arbitraryShape_live = tempB;
                          tempB.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
                          tempB.setCapability(Node.ALLOW_BOUNDS_READ);

                          if (liveShapes ==  null)
                             liveShapes = new Vector();

                          liveShapes.add(tempB);

                       }
                       else  //(mode == TRAVERSE_MODE_EXTRACT )
                       {


                          arbitraryShape = tempB;
                          tempB.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);



                            GeometryArray geo = (GeometryArray)arbitraryShape.getGeometry();

                            if (!geo.isLive()  && !geo.isCompiled() )
                            {
                              geo.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
                              geo.setCapability(GeometryArray.ALLOW_COUNT_READ);


                            }

                        }


                        // add
                        newBranch.addChild( tempB );


                        //BDebug.print( "Got Shape: " + oldBranch.toString());
                }  // MODE EXTRACT
                else if (mode == TRAVERSE_MODE_CONVERT_TO_LIVE)
                {

                           if (liveShapes ==  null)
                             liveShapes = new Vector();

                           Shape3D tempS = (Shape3D)oldBranch;

                           tempS.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
                           tempS.setCapability(Node.ALLOW_BOUNDS_READ);

                           if (!tempS.getGeometry().isLive() && !tempS.getGeometry().isCompiled())
                            tempS.getGeometry().setCapability(Geometry.ALLOW_INTERSECT );

                           arbitraryShape_live = (BShape3D)tempS;
                           liveShapes.add(tempS);


                }
                else if (mode == TRAVERSE_MODE_CLEANSE )
                {
                        BShape3D tempS = (BShape3D)oldBranch;
                        //tempS.setComponent(null);
                        tempS.setAppearance(null);
                }

                else if (mode == TRAVERSE_MODE_SETAPP)
                {

                        BShape3D tempS = (BShape3D)oldBranch;
                        tempS.setAppearance(normalAppearance);

                }
                else if (mode == TRAVERSE_MODE_POLYCOUNT)
                {

                        BShape3D tempS = (BShape3D)oldBranch;

                        int shapesPoly  =  tempS.getPolyCount();
                        //BDebug.print( "BComponent.travershapes . shapesPoly" + shapesPoly);

                        polyCount = polyCount + shapesPoly;

                }


               else if (mode == TRAVERSE_MODE_VOLUME)
               {

                        Vector parentTransforms = new Vector();
                        //TransformGroup tG;
                        Transform3D tempTransform;

                        Node currentNode = oldBranch;

                        while ( currentNode.getParent() != null)
                        {
                            currentNode = currentNode.getParent();

                            if (currentNode instanceof TransformGroup)
                            {
                              // BDebug.print ("BComponent.Traverseshapes (GUESS_VOLUME) :: adding transform to list ");
                              tempTransform = new Transform3D();
                              ((TransformGroup)currentNode).getTransform(tempTransform);
                              parentTransforms.add(0, tempTransform);

                            }

                        }


                        Transform3D totalTransform = new Transform3D();
                        totalTransform.setIdentity();

                        for (int i = 0; i < parentTransforms.size(); i++)
                          totalTransform.mul(  (Transform3D)parentTransforms.get(i)  );


                         BShape3D tempS = (BShape3D)oldBranch;
                         Geometry tempG = tempS.getGeometry();

                        if ( tempG instanceof GeometryArray)
                        {

                          GeometryArray theGA = (GeometryArray) tempG;
                          int count = theGA.getVertexCount();

                          Point3f point = new Point3f();

                         // Point3f pointOut = new Point3f();

                          // for every point in the Object
                          for (int i = 0; i < count ; i++)
                          {

                            theGA.getCoordinate(i,point);
                            totalTransform.transform(point);

                            vMinY = java.lang.Math.min(vMinY,point.y);
                            vMaxY = java.lang.Math.max(vMaxY,point.y);

                            vMinX = java.lang.Math.min(vMinX,point.x);
                            vMaxX = java.lang.Math.max(vMaxX,point.x);

                            vMinZ = java.lang.Math.min(vMinZ,point.z);
                            vMaxZ = java.lang.Math.max(vMaxZ,point.z);

                            //  BDebug.print(" vertex " +  pointOut.toString() );

                            }


                          } // if geoArray
                        else
                          BDebug.print (" BComponent.traverseShapes.MODE_VOLUME assersion failure  geometry not point array ");

               }

  } // if node = shape

}


  private static Point3f parseLights_position= new Point3f();
  private static Vector3f parseLights_direction = new Vector3f();

  protected void parseLights(Scene s)
  {

        Light[] lightsv = s.getLightNodes();

        if (lightsv != null)
          for (int i = lightsv.length-1 ; i>0; i-- )
          {
                Light tempL = lightsv[i];

                if ( tempL instanceof SpotLight )
                {

                        SpotLight tempP = (SpotLight)tempL;
                        tempP.setCapability(PointLight.ALLOW_POSITION_READ);
                        tempP.setCapability(SpotLight.ALLOW_DIRECTION_READ);

                        tempP.getPosition(parseLights_position);
                        tempP.getDirection(parseLights_direction);

                        //BDebug.print (" Direction " + direction);

                        //BDebug.print("Converted" +  tempP.toString() );
                        connectPoints.add(new BConnectPoint( new Vector3d( parseLights_position),
                                          new Vector3d(parseLights_direction)   )
                                        ) ;

                } // if  is spotlight

          } // loop

  }


public void setName(String newName)
{

  name = newName;

}


public String getName()
  { return name; }

public String getToolTipText()
  { return name; }

  /*
    + " [$"+ BMath.format(price)+ ", "
    + damageMax+ " hp, "
    + BMath.format(weight)+ " kg]";
  */



public float getVersion ()
{
  return app_version;
}

public void setVersion (float versionIn)
{
  app_version = versionIn;
}

  public String toString()
  {
    // IF YOU CHANGE THIS YOU MUST RE_CALCULATE THE COLLISION HASH TABLE IN BBOT
    return name;
  }

  public float getPrice()
    { return price; }

  public float getWeight()
    { return weight; }

  public float getDisplayWeight()   // added here to allow BChassis to override, giving the stats total weight
    { return weight; }

  public void setPrice(float price)
    { this.price = price; }

  public void setWeight(float weight)
    { this.weight = weight; }

  public void setTreeNode(BComponentNode treeNodeToSet)
  /*

  * Description:
    Sets the treeNode member.

	* Parameters:
	* Results:
  * Example:
  * Modification History ( Name, Date, Notes ):
    HSC, 25-March-1999

  */


  {
    treeNode = treeNodeToSet;
  }

  public BComponentNode getTreeNode()
  /*

  * Description:
    Returns the treeNode member.

	* Parameters:
	* Results:
    treeNode member.

  * Example:
  * Modification History ( Name, Date, Notes ):
    HSC, 25-March-1999

  */


  {
    return treeNode;
  }



public Object clone()
 {
  /*

  * Description:
    Overrides the Object cloneable method.
    Performs a deep clone by calling the clone method of all it's members,
    who in turn call clone of all their members.
    *** IMPORTANT NOTE ***
    Must implement clone in a for/in all new members.

	* Parameters:
	* Results:
    treeNode member.

  * Example:
  * Modification History ( Name, Date, Notes ):
    HSC, 29-March-1999
    YLY,  8th MAr 99, Implemented Clone

  */

 // BDebug.print(" ----- CLONING ------" + this.name);

  BComponent o = null;
  try
  {
    o = (BComponent)super.clone();
  }
  catch (CloneNotSupportedException e)
  {
    BDebug.print( e.getMessage() );
  }
  // Must close all handles of member objects.

 // o.structure3D = (BranchGroup)this.structure3D.cloneTree(true);
  o.structure3D = new BranchGroup();
  o.connectPoints = new Vector();

  o.setAppearance();
  o.setAppearance(o.textureName, o.componentColor, o.componentShine, o.transparency, o.colourDepth );

  o.traverseShapes(structure3D, o.structure3D, TRAVERSE_MODE_EXTRACT );
  o.traverseShapes(o.structure3D,new Group(), TRAVERSE_MODE_SETAPP);
  o.liveShapes = new Vector();


  // TODO: should these connect points be cloned?

  if (this.connectPointBase != null)
      o.connectPointBase = (BConnectPoint) this.connectPointBase.clone();

       //o.setConnectPointBase(this.connectPointBase, false );

  if (this.connectPointParent  != null)
       o.setConnectPointParent(this.connectPointParent, false);

  o.connectPointSelected = null;


  BConnectPoint current;
  for (int i = 0 ; i < this.connectPoints.size() ; i++ )
    {
    current = (BConnectPoint)connectPoints.get(i);

    BConnectPoint cloned = (BConnectPoint)current.clone();
    o.connectPoints.add(cloned);
  }


  return o;

  }

  public void setConnectPointBase( BConnectPoint newPoint)
    { setConnectPointBase(  newPoint, true); }

 public void setConnectPointBase( BConnectPoint newPoint, boolean releaseOld)
 {


  boolean found = false;

  if (releaseOld)
    if (connectPointBase != null)
     connectPointBase.setActive(true);

  if ( newPoint == null)
    connectPointBase = null;
  else
     {

    for (int i = 0 ; i < this.connectPoints.size() ; i++ )
      {
         BConnectPoint current = (BConnectPoint)connectPoints.get(i);

         // testing with position, because the active flag may be different
         if ( newPoint.equals(current) )
         {
              current.setActive(false);
              connectPointBase = current;
              found = true;
         } // if


        } // for

      if (!found)
        BDebug.print("BComponent.setConnectPointBase: NEWPOINT NOT FOUND IN VECTOR" );
   } // else null  ;


 }


  public void setConnectPointsActive(boolean b)
  {
     for (int i = 0 ; i < this.connectPoints.size() ; i++ )
     {
         BConnectPoint current = (BConnectPoint)connectPoints.get(i);
         current.setActive(b);

     } // for

  }


  public void setConnectPointParent( BConnectPoint newPoint)
  {
    setConnectPointParent( newPoint, true);
  }

 public void setConnectPointParent( BConnectPoint newPoint, boolean releaseOld)
 {
    if (releaseOld)
      if (connectPointParent != null)
        connectPointParent.setActive(true);

    if (newPoint != null)
        newPoint.setActive(false);

    connectPointParent = newPoint;
 }


 public BConnectPoint getConnectPointParent()
 {

  return connectPointParent;

 }

 public void setConnectPointActive(BConnectPoint cp, boolean b)
 {
   // BDebug.print ("BComponent.setConnectPointActive(BConnectPoint cp, boolean b) called "+ b);

    int index = connectPoints.indexOf(cp);

    if (index != -1)
    {
      ((BConnectPoint)connectPoints.get(index)).setActive(b);

    }
  //  else
  //    BDebug.print ("BComponent.setActiveConnectPoint Connect Point not found on Parent " + this );

    cp.setActive(b);
    return;

 }

 public BConnectPoint getConnectPointBase()
 {

  return connectPointBase;

 }

 public boolean isStablePartOfChassis()
 {
    boolean isStable = true;

    BComponent parent = this;

    while( (parent != null) && isStable )
    {
      if (parent instanceof BJoint)
        isStable = false;

      parent = parent.getParent();

    }

    return isStable;

 }



private static Vector3d  zAxis = new Vector3d(0.0, 0.0, -1.0);
private static AxisAngle4d addConnectPointsToScene_rot = new AxisAngle4d();

public Vector addConnectPointsToScene(boolean showInactiveConnectPoints)
  { return addConnectPointsToScene(getLiveNode(), showInactiveConnectPoints); }

public Vector addConnectPointsToScene(Group attachNode, boolean showInactiveConnectPoints)
{
  showingConnectPoints = true;

  if (getChassisLock() )
    if (!isStablePartOfChassis() )
      return null;
  // If connect points are already in scene, do nothing.
  if (myConnectPtsNode != null)
    return null;

  myConnectPtsNode = new BranchGroup();
  myConnectPtsNode.setCapability(BranchGroup.ALLOW_DETACH);

  // variables
  BConnectPointShape sph;
  TransformGroup connectTrans;
  BConnectPoint current;
  Vector tempVector = new Vector();

  Transform3D t;

  // Orient code variables..
  Transform3D trans = new Transform3D();
  Vector3d   facing = new Vector3d();
  Vector3d  rotAxis = new Vector3d();
  double      theta = 0.0;
  //double        dot = 0.0;
  double        tol = 0.01;
  //Vector3d   offset = new Vector3d(0.0, 0.0, 1.5);


  for (int i=0; i<connectPoints.size(); i++ )
  {
    current = (BConnectPoint) connectPoints.get(i);

    if (current.isActive() || showInactiveConnectPoints)
    {
      t = new Transform3D();

      /* ORIENT CODE BEGINS */
     facing.normalize(current.getFacing());

      if (facing.z <= -(1.0 - tol))
      {
        // If facing is +z, rotate 180 degrees.
        trans.rotX(BPhysics.PI);
        t.mul(trans, t);
      }
      else if (facing.z <= (1.0 - tol))
      {
        // Reverse z direction.
        facing.z = -facing.z;

        // Get a vector perpendicular to z & facing.
        rotAxis.cross(facing, zAxis);
        rotAxis.normalize();

        // Get angle between the two vectors.
        theta = facing.angle(zAxis);

        // Rotate about the derived vector.
        addConnectPointsToScene_rot.set(rotAxis.x, rotAxis.y, rotAxis.z, theta);

        trans.setIdentity();
        trans.setRotation(addConnectPointsToScene_rot);
        t.mul(trans, t);
      }

      /* ORIENT CODE ENDS */

      // displace connectpoints
      trans.set(current.position);
      t.mul(trans, t);

      connectTrans = new TransformGroup(t);
      myConnectPtsNode.addChild(connectTrans);

      // Create Point
      sph = new BConnectPointShape(current, this);
      sph.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);

      connectTrans.addChild(sph);

      // SJS [17/02/2000]: Link connectPt back to its shape...
      current.setShape(sph);
     // current.setLine(line);

      tempVector.add(sph);
    }
  }

  attachNode.addChild(myConnectPtsNode);

  return tempVector;
}

  public void removeConnectPointsFromScene()
  {
    showingConnectPoints = false;

    if (myConnectPtsNode != null)
    {
      if (connectPointSelected != null)
        connectPointSelected.removeAnimation();

       // Remove the connect point BG.
      myConnectPtsNode.detach();

      // variables

      BConnectPoint c = null;

      int n = connectPoints.size();

      for (int i=0; i<n; i++)
      {
        c = (BConnectPoint) connectPoints.get(i);
        c.cleanse();

      }                 

      // Forget the connect pt. BG.
      myConnectPtsNode = null;
    }


  }

  public boolean isShowingConnectPoints()
    { return showingConnectPoints; }


  public void setConnectPointSelected (BConnectPoint set)
  {
    // exit if the connect point is inactive
    if (set != null)
      if (!set.isActive() )
        return;

    if (connectPointSelected != null)
    {
      // SJS [12/10/2000]
      // Bail out if we already have the requested connect-point selected.
      if (set != null)
        if (set.equals(connectPointSelected))
          return;
      
      connectPointSelected.setSelected(false);
    }

    boolean found = false;
    connectPointSelected = null;

    if (set != null)
    {
      for (int i = 0 ; i < this.connectPoints.size() ; i++ )
      {
        BConnectPoint current = (BConnectPoint)connectPoints.get(i);

        // Test equality with position, since clones might exist.

        if ( set.equals(current) )
        {
          connectPointSelected = current;
          connectPointSelected.setSelected(true);
          found = true;
        }
      }

      if (!found)
        BDebug.print("BComponent.setConnectPointSelected: NEWPOINT NOT FOUND IN CONNECTPOINTS VECTOR" );
    }
  }



  public BConnectPoint getConnectPointSelected ()
  {
    return connectPointSelected;
  }

  public void setSerialNo (Integer in)
  {
    serialNum = in;
  }

  public Integer getSerialNo ()
  {
    return serialNum;
  }

  public void setZAlignment (double in)
  {
    zAlignment = in;
  }

  public double getZAlignment ()
  {
    return zAlignment;
  }

  public void setDetails (String in)
  {
    details = in;
  }

  public boolean getChassisLock()
  {
    return chassisLock;
  }

  public void setChassisLock(boolean lock)
  {
    chassisLock = lock;
  }

  public void setMaterialType(BComponentMaterial material)
  {
    //BDebug.print ("BComponent.setMaterial called with Code"+ material.getCode());
    materialCode = material.getCode();
    componentmaterial = material;

  }

  public BComponentMaterial getMaterial()
  {
    if ( componentmaterial == null)
      componentmaterial = BComponentMaterial.getMaterial(materialCode) ;

    return componentmaterial;

  }

  public String getMaterialDesc()
  {
    if ( componentmaterial == null)
      componentmaterial = BComponentMaterial.getMaterial(materialCode) ;

    return componentmaterial.getName();

  }

  public int getMaterialType()
  {
  if ( componentmaterial == null)
      componentmaterial = BComponentMaterial.getMaterial(materialCode) ;

    return componentmaterial.type;

  }


  public  void setFilePath(String fn)
  // strips off header
  {

    componentFilePath = fn.substring(BGlobal.COMPONENTPATH.length(), fn.length());


  }

  public static BComponent loadFromFile(String filename)
  {
    return  loadFromFile( filename,true);
  }

  public static BComponent loadFromFile(String filename, boolean init)
  {
    BComponent newIn = null;

    try
    {
      filename =  BStrings.convertToFilePath(filename);

      FileInputStream fin = new FileInputStream(filename);
      ObjectInputStream in = new ObjectInputStream(fin);
      newIn = (BComponent)in.readObject();



      if (init)
        newIn.init(false);

      in.close();
      fin.close();

      newIn.setFilePath(filename);


    } catch (Exception e) {BDebug.print(e.getMessage());}

    return newIn;

  }


  public String getComponentFilePath()
  {
    return componentFilePath;
  }

  public String getType()
  {
    int t = componentFilePath.indexOf("/");

    if (t == -1)
      return null;
    else
    {
      String capType =   componentFilePath.substring(0, t);
      return capType.toLowerCase();

    }
  }

  public String getGenre()
  {
    int t = componentFilePath.indexOf("/");
    int g = componentFilePath.indexOf("/", t+1);
    return  componentFilePath.substring(t+1, g).toLowerCase();
  }

  public String getFileName()
  {
    int g = componentFilePath.lastIndexOf("/");
    return  componentFilePath.substring(g+1);
  }

  public boolean isInternal()
  {
    String type = getType();
    boolean b = false;

    if (type != null)
      b =  type.equalsIgnoreCase(BGlobal.INTERNAL_ITEM);

    return b;
  }


   public void saveToFile(String filename )
  {
    try{
     if (this instanceof BChassis)
     {
        if (  ! BGlobal.CHASSISSUFFIX.equalsIgnoreCase(     filename.substring(filename.length() - 4 )  )  )
                filename = filename +  BGlobal.CHASSISSUFFIX;
     }
     else
     {
        if (  ! BGlobal.COMPONENTSUFFIX.equalsIgnoreCase(     filename.substring(filename.length() - 4 )  )  )
                filename = filename +  BGlobal.COMPONENTSUFFIX;
     }

      FileOutputStream fout = new FileOutputStream(filename);
      ObjectOutputStream out = new ObjectOutputStream(fout);
      out.writeObject(this);
      out.close();
      System.out.println(this.toString() +"Save completed");
    }
    catch (Exception e) {
      BDebug.print("BComponent.saveToFile " + e.getMessage() );
    }
  }

  public Color getColor()
    { return componentColor; }

  public int getColorDepth()
    { return colourDepth; }

  public void setColor(Color theColor, float shine, int newDepth)
  {
    // Globally used colors
    componentColor = theColor;
    componentShine = shine;
    colourDepth = newDepth;

    displayColor(theColor, shine);
  }

  public void setColor(Color theColor)
  {
    componentColor = theColor;
    displayColor(theColor, componentShine);
  }

  public void setTexture (String fileName)
    { setTexture (fileName, true); }


  protected Texture getTexture()
  {
    if (textureName == null)
      return null;
    else
      return BTextures.getTexture(BGlobal.TEXTUREPATH + textureName);
  }


  public void setTexture (String fileName, boolean storeName)
  {
    if (BGlobal.SHOW_TEXTURES)
    {
      //try
     // {
        if (( fileName == null ) || (fileName.equals("") ) )
        //Remove texture if the textureName = null
        {
          if (storeName)
            textureName = null;
          normalAppearance.setTexture(null );
        }
        else
        {
          if (storeName)
            textureName = fileName;
          //fileName =  BGlobal.TEXTUREPATH + fileName ;

          normalAppearance.setTexture(getTexture());


         // traverseShapes(structure3D,structure3D, TRAVERSE_MODE_SETAPP);



        }
     // }
     // catch (Exception e)
     //   { BDebug.print("BComponent.setTexture ERROR " + getName() + " "+ e.getMessage() ); }
    }
  }

  public String getTextureName()
  {
    return textureName;
  }


  public void setPickable(boolean p)
  {
    getLiveNode().setPickable(p);


  }

  public void setLiveBotNode(BranchGroup newLive)
  {
    if (!newLive.isLive())
    {
       newLive.setCapability(Group.ALLOW_BOUNDS_READ);
       newLive.setCapability(Node.ALLOW_PICKABLE_WRITE);
       newLive.setUserData(getName() + " LiveBotNode ");
    }
    liveBotNode = newLive;
  }


  public BComponent getParent()
  {
    BComponent p = null;

    if ( getTreeNode() != null )
      if (getTreeNode().getParent() != null)
        p =   ((BComponentNode)getTreeNode().getParent()).getComponent();

    return p;

  }


  public BranchGroup getLiveNode( )
 {
    return getLiveNode(false);
 }

 public BranchGroup getLiveNode(boolean centerOfObject)
 {
    // centre of object not used in this implementation of BArenaObject
    return liveBotNode;
 }

 public void setHighlight()
 {

    // Determine highlight color of this component.
    //Color highlightColor = chassisLock ? BGlobal.LOCKED_COLOR : getColor(); // BGlobal.colors.getLighterShade();

    Color highlightColor = chassisLock ? BGlobal.LOCKED_COLOR : BGlobal.SELECTED_COLOR; // BGlobal.colors.getLighterShade();


    // BGlobal.SELECTED_COLOR;
    setHighlight(highlightColor);

 }


 public void setHighlight(BComponentAppearance ca)
  {
    setTexture(ca.getTexture(), false);
    setHighlight(ca.getColor());
    setTransparency(ca.getTransparency(), false);
  }

 public void setHighlight(Color highlightColor)
 {
    Color3f highlightColor3f = new Color3f(
    (float)highlightColor.getRed()   / 256,
    (float)highlightColor.getGreen() / 256,
    (float)highlightColor.getBlue()  / 256);

    setHighlight(highlightColor3f);
 }

 public void setHighlight(Color3f c)
 {
    // Mark this component as being highlighted...
    //inHighlight = true;

    Color3f highlightColor3f = new Color3f(c.x/2f, c.y/2f, c.z/2f);

    // Set material properties for highlight mode.
    componentMaterial.setDiffuseColor(highlightColor3f); // BGlobal.BLACK_3F);
    componentMaterial.setSpecularColor(BGlobal.BLACK_3F);
    componentMaterial.setEmissiveColor(highlightColor3f);
    componentMaterial.setShininess(1.0f);
 // componentMaterial.setAmbientColor(highlightColor3f);

    /*
    // Set transparency.
    if (BGlobal.USE_TRANSPARENCY)
        setTransparency(BGlobal.SELECTED_TRANSPARENCY, false);
    else
        setWireFrame();
    */

    // Set color attributes.
    cAtrib.setColor(highlightColor3f);
    cAtrib.setShadeModel(ColoringAttributes.SHADE_GOURAUD);

    // Set appearance node properties.
    normalAppearance.setMaterial(componentMaterial);
    normalAppearance.setColoringAttributes(cAtrib);
    // normalAppearance.setTexture(null);
 }


 public void setNormalAppearance()
 {
      //if (inHighlight )
       // {
    //inHighlight = false;
    setAppearance(textureName, componentColor, componentShine, transparency, colourDepth );
    //normalAppearance.setPolygonAttributes(null);
       // }
 }

  public void setSelected(boolean selected)
    // Default selection behavior.
    { setSelected(selected, true, true, true, false, BGlobal.SELECTED_COLOR_3F ); }

  public void setSelected
  (
    boolean selected,
    boolean showSelectBox,
    boolean showSelectColor,
    boolean showConnectPts,
    boolean showInactiveConnectPoints,
    Color3f selectColor
  )
  {
    //inHighlight = selected;

    if (selected )
    {
      if (showConnectPts)
        addConnectPointsToScene(showInactiveConnectPoints);
      else
        removeConnectPointsFromScene();

      if (showSelectBox)
        addSelectionBoxToScene(selectColor);
      else
        removeSelectionBoxFromScene();

      // YLY Highlight differently if you are constructing a wizardBot
      if (getBot() != null)
        if (getBot().getType() == BWizardTemplateBot.BOT_TYPE)
        {
          setHighlight(selectColor);
          setTransparency(0.6f, false);
          myShapesNode.setPickable(false);
        }
    }
    else
    {
      connectPointSelected = null;
      // setConnectPointSelected(null);
      
      removeConnectPointsFromScene();
      removeSelectionBoxFromScene();

      if (getBot() != null)
        if (getBot().getType() == BWizardTemplateBot.BOT_TYPE)
        {
          myShapesNode.setPickable(true);
          setNormalAppearance();
        }
    }
  }

  protected void addSelectionBoxToScene(Color3f boxColor)
  {
    // If selection box is already in scene, do nothing.
    if (mySelectionBoxNode != null)
      return;

    try
    {
      BranchGroup attachNode = getLiveNode();

      mySelectionBoxNode = new BranchGroup();
      mySelectionBoxNode.setCapability(BranchGroup.ALLOW_DETACH);
      mySelectionBoxNode.setPickable(false);

      mySelectionBox = getSelectionBox(boxColor, 2.0f, 3.0f, 1.5f, 0.4f);
      mySelectionBoxNode.addChild(mySelectionBox);

      attachNode.addChild(mySelectionBoxNode);
    }
    catch (Exception e)
      { BDebug.print("BComponent.addSelectionBoxToScene ERROR: "+ e); }
  }

  protected void removeSelectionBoxFromScene()
  {
    if (mySelectionBoxNode != null)
    {
      mySelectionBoxNode.detach();
      mySelectionBoxNode = null;

      mySelectionBox.cleanse();
      mySelectionBox = null;
    }
  }

  public String getAmbientSoundName()
  { return ambientSoundName; }

  public void setAmbientSoundName(String name )
  { ambientSoundName = name; }


  public BranchGroup getStructure3D()
  {
     return getStructure3D(true);
  }

  public BranchGroup getStructure3D(boolean deleteStructure3D)
  {

    if (structure3D == null)
      loadShape();
      
    traverseShapes ( structure3D, new Group(), BComponent.TRAVERSE_MODE_CONVERT_TO_LIVE);


    structure3D.setCapability(BranchGroup.ALLOW_DETACH);

    myShapesNode = structure3D;
    myShapesNode.setCapability(Node.ALLOW_BOUNDS_READ);
    myShapesNode.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);


    BranchGroup tempBranch = new BranchGroup();
    tempBranch.setCapability(Node.ALLOW_BOUNDS_READ);
    tempBranch.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
    tempBranch.setCapability(BranchGroup.ALLOW_DETACH);
    tempBranch.addChild(myShapesNode);


    if (deleteStructure3D)
      structure3D = null;

    return tempBranch;





}



public BranchGroup cloneStructure3D( )
{
   return cloneStructure3D( false, true);
}

public BranchGroup cloneStructure3D( boolean cloneAdditions, boolean forLive)
{
  return cloneStructure3D( cloneAdditions, forLive, false );

}



public BranchGroup cloneStructure3D( boolean cloneAdditions, boolean forLive, boolean cloneAppearance)

{
  if (structure3D == null)
    loadShape();

  BranchGroup tempBranch = new BranchGroup();
  myShapesNode =  new BranchGroup();
  myShapesNode.setUserData(getName() + ".myshapesNode");
  myShapesNode.setCapability(Node.ALLOW_BOUNDS_READ);
  myShapesNode.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
  myShapesNode.setCapability(Node.ALLOW_PICKABLE_WRITE);


  if (forLive)
    traverseShapes(structure3D, myShapesNode, TRAVERSE_MODE_EXTRACT_FORLIVE);
  else
    traverseShapes(structure3D, myShapesNode, TRAVERSE_MODE_EXTRACT);

  tempBranch.addChild(myShapesNode);
  return tempBranch;


}

 public void displayColor(Color theColor)
 {
    displayColor(theColor, componentShine);
 }

 protected void displayColor(Color theColor,  float shine)
 {

    try
    {

      float[] hSB = Color.RGBtoHSB(
        theColor.getRed(),theColor.getGreen(),theColor.getBlue(),null);

      float highlight_brightness =
        hSB[2] + ((1f - hSB[2]) * (float) colourDepth / 100.0f);

      Color highlight =
        Color.getHSBColor(hSB[0], hSB[1], highlight_brightness);

      Color3f objColor   = new Color3f(theColor);
      Color3f objHigh    = new Color3f(highlight);
      Color3f objAmbient = new Color3f(objColor);

      componentMaterial.setAmbientColor(objAmbient);
      componentMaterial.setDiffuseColor(objColor);
      componentMaterial.setSpecularColor(objHigh);
      componentMaterial.setShininess(shine);
      componentMaterial.setEmissiveColor(BGlobal.BLACK_3F);

      normalAppearance.setMaterial(componentMaterial);

      cAtrib.setColor(objColor);
      cAtrib.setShadeModel(ColoringAttributes.SHADE_GOURAUD);

      normalAppearance.setColoringAttributes(cAtrib);
    }
    catch(Exception e)
    {
      BDebug.print ("BComponent.displayColor error " + e.getMessage() );

    }
 }

 public TransformGroup getConnectRotateNode()
 {
 return connectRotateNode;
 }

 public void setConnectRotateNode(TransformGroup newNode)
 {
  connectRotateNode = newNode;

 }


 public BShape3D getArbitraryShape()
 // GETS an Arbitary shape of this component
 {
    return arbitraryShape;

 }


 public void setVisible(boolean v)
 {
   // SJS 05/04/2001: Added ability to show/hide a component.
   rAttrib.setVisible(v);
 }

 public void setTransparency(float newT)
 {
   // SJS 19/01/2000: Added boolean flag - whether to update default appearance or not.
   setTransparency(newT, true);
 }


public void setTransparency(float newT, boolean updateDefault)
{

  //BDebug.print(this + ".setTransparency " + newT);
  if (updateDefault)
    transparency = newT;

 // try {
  if (BGlobal.USE_TRANSPARENCY)
  {
    if (newT == 0.0f)
    {
      normalAppearance.setTransparencyAttributes(null );

    }
    else
    {
      //if (BGlobal.USE_TRANSPARENCY)
      //{
        TransparencyAttributes tranA;
        tranA = normalAppearance.getTransparencyAttributes();

        if (tranA == null)
        {
          tranA =  new TransparencyAttributes(TransparencyAttributes.BLENDED , newT, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA) ;
          tranA.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
          normalAppearance.setTransparencyAttributes(tranA);

          // HACK for j3dBeta1


        }
        else
        {
          tranA.setTransparency(newT);

        }

      //}
    }
  } else {
    // Force Disabling of Transparency
    TransparencyAttributes tranA = new TransparencyAttributes(TransparencyAttributes.NONE,1.0f);
    //tranA.setCapability(TransparencyAttributes.ALLOW_VALUE_WRITE);
    normalAppearance.setTransparencyAttributes(tranA);
  }
//  }
//  catch (Exception e){ BDebug.print("BComponent.setTransparency :Error " + e.getMessage()); }

}



public float getShine()
{

  return componentShine;

}

public float getTransparency()
{
 return transparency;
}


public void setWireFrame()
  {


      if (normalAppearance.getMaterial() != null )
        {
        //normalAppearance.setPolygonAttributes(pa);
        normalAppearance.setMaterial(null);
        normalAppearance.setTexture(null);
        normalAppearance.getColoringAttributes().setShadeModel(ColoringAttributes.SHADE_FLAT);
       // normalAppearance.setLineAttributes(new LineAttributes(1, LineAttributes.PATTERN_SOLID , true  ));


        }


  }


  public void setConnectPoints(Vector connectPointsIn)
  {
    connectPoints = connectPointsIn;
  }

  public Vector getConnectPoints()
  {
    return connectPoints;
  }


  // gets the next free connect point on this component
  public BConnectPoint getFreeConnectPoint()
  {
    if (getChassisLock() )
      if (!isStablePartOfChassis() )
        return null;

    Vector v = getConnectPoints();
    if(v != null)
    {
      for (int i = 0; i < v.size(); i++)
      {
        BConnectPoint cp = (BConnectPoint)v.elementAt(i);
        if(cp.isActive())
          return cp;
      }
    }  
    return null;
  }

  public void attach(BComponent parent, BranchGroup subTreeToAttach, boolean useDefaultJoint, boolean optimise)
  {

    // set the live node
    setLiveBotNode(subTreeToAttach);

    TransformGroup componentDisplace = new TransformGroup();

    // if optimse then this is the only node above the liveNode, so need to be able to detach
    if (optimise)
       componentDisplace.setCapability(Group.ALLOW_CHILDREN_WRITE);

                // Position Variables
    Vector3d parentFacing =   new Vector3d (getConnectPointParent().facing );
    Vector3d baseFacing =   new Vector3d (getConnectPointBase().facing ) ;
    Vector3d parentLength = new Vector3d (getConnectPointParent().position);
    //Vector3d baseLength =   new Vector3d (getConnectPointBase().position);

    // --TRANSLATE FROM CENTRE OF PARENT'S UNIVERSE TO PARENT CONNECT POINT POSITION
    Transform3D total = new Transform3D();
    total.set( parentLength );

    Transform3D trans = new Transform3D();

    // --POSITION ROTATE ( ROTATE SO THAT THE BOTTOM OF THE CHILD OBJECT ALIGNS WITH THE PARENT CONNECT POINT

   // BDebug.print ( "BComponent.attach baseFacing " + baseFacing );
   // BDebug.print ( "BComponent.attach parentFacing " + parentFacing );


    parentFacing.normalize();
    baseFacing.normalize();

    // Get a vector perpendicular to z & facing.
    Vector3d  rotAxis = new Vector3d();
    Vector3d negbase = new Vector3d(baseFacing);
    negbase.negate();
    //BDebug.print ( "BComponent.attach  negbase " + negbase );


    if ( negbase.equals(parentFacing) )
    {
     // BDebug.print ( "BComponent.attach  neg Base equals "  );
    }
    else if (baseFacing.equals (parentFacing) )
    {

      if ( BMath3D.isAxis(parentFacing) )
      {
        //BDebug.print ("parentFacing is axis");

        Vector3d yAxis = new Vector3d(0,1.0,0);
        Vector3d xAxis = new Vector3d(1.0,0,0);

        double axisTheta = yAxis.angle(negbase);
        trans.rotZ(axisTheta);
        trans.transform(xAxis, rotAxis);
        rotAxis.normalize();

        double theta = BPhysics.PI;

        // Rotate about the derived vector.
        AxisAngle4d rot = new  AxisAngle4d();
        rot.set(rotAxis.x, rotAxis.y, rotAxis.z, -theta);

        trans.setIdentity();
        trans.setRotation(rot);
        total.mul(trans);
      }
      else
      {
        negbase.x = negbase.x + 0.0000000001;
        //BDebug.print("Same but not axis");
        rotAxis.cross(parentFacing, negbase);
        rotAxis.normalize();

        // Get angle between the two vectors.
        double theta;

        theta = java.lang.Math.PI;


        AxisAngle4d rot = new  AxisAngle4d();
        rot.set(rotAxis.x, rotAxis.y, rotAxis.z, -theta);

        trans.setIdentity();
        trans.setRotation(rot);
        total.mul(trans);
      }

    }
    else
    {
      rotAxis.cross(parentFacing, negbase);
      rotAxis.normalize();

      // Get angle between the two vectors.
      double theta;

      theta= parentFacing.angle(negbase);

      //BDebug.print ( "theta " + theta );
      // Rotate about the derived vector.
      AxisAngle4d rot = new  AxisAngle4d();
      rot.set(rotAxis.x, rotAxis.y, rotAxis.z, -theta);

      trans.setIdentity();
      trans.setRotation(rot);
      total.mul(trans);
    }





    // JOINT MOVEMENT
    // THIS MUST GO AFTER THE ALIGN
    TransformGroup jointRotateG = new TransformGroup();

    if (optimise)
    {
      AxisAngle4d connectRotate = new AxisAngle4d(baseFacing.x, baseFacing.y, baseFacing.z, getZAlignment() );
      Transform3D alignT = new Transform3D();
      alignT.setRotation(connectRotate);
                // combine connectpoint rotate
      total.mul(alignT);



      // ALIGN CHILD COMPONENT
      Vector3d newLength = new Vector3d (getConnectPointBase().position);
      newLength.scale(-1.0);

      alignT = new Transform3D();
      alignT.setTranslation(newLength);
      total.mul(alignT);

      componentDisplace.setTransform(total);


      // Do NOT Change to getComponentType = Bjoint.getComponentType
      // Because We need all sub classes of BJoint
       if (this instanceof BJoint)
       {

                        jointRotateG = new TransformGroup( ((BJoint)this).getTempTransform(true) );
                        jointRotateG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
                        jointRotateG.setUserData("jointRotateGroup ");

                        ((BJoint)this).setJointTransformNode(jointRotateG);
                        componentDisplace.addChild(jointRotateG);
                        jointRotateG.setCapability(Group.ALLOW_CHILDREN_WRITE);

                        //BDebug.print(((BJoint)newComponent).toString());
                        jointRotateG.addChild(subTreeToAttach);
                        jointRotateG.setCapability(Group.ALLOW_CHILDREN_EXTEND);

                        setDirectLiveParentNode(jointRotateG);
       }
       else
       {
                        componentDisplace.setCapability(Group.ALLOW_CHILDREN_EXTEND);
                        setDirectLiveParentNode(componentDisplace);
                        componentDisplace.addChild(subTreeToAttach);

       }


    }
    else
    {
      componentDisplace.setTransform(total);

      AxisAngle4d connectRotate = new AxisAngle4d(baseFacing.x, baseFacing.y, baseFacing.z, getZAlignment() );
      Transform3D connectBaseRotateT = new Transform3D();
      connectBaseRotateT.setRotation(connectRotate);
      TransformGroup connectBaseRotateG = new TransformGroup(connectBaseRotateT);


      connectBaseRotateG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
      setConnectRotateNode(connectBaseRotateG);
      componentDisplace.addChild(connectBaseRotateG );


      // ALIGN CHILD COMPONENT
      Vector3d newLength = new Vector3d (getConnectPointBase().position);
      newLength.scale(-1.0);
      Transform3D alignT = new Transform3D();
      alignT.set(newLength);
      TransformGroup alignGroup = new TransformGroup(alignT);
      connectBaseRotateG.addChild(alignGroup);
      alignGroup.setCapability(Group.ALLOW_CHILDREN_WRITE);


       // Do NOT Change to getComponentType = Bjoint.getComponentType
      // Because We need all sub classes of BJoint
      if (this instanceof BJoint)
      {


        jointRotateG = new TransformGroup( ((BJoint)this).getTempTransform(useDefaultJoint) );
        jointRotateG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        ((BJoint)this).setJointTransformNode(jointRotateG);
        alignGroup.addChild(jointRotateG);
        jointRotateG.setCapability(Group.ALLOW_CHILDREN_WRITE);
        jointRotateG.addChild(subTreeToAttach);

        //BDebug.print ("BComponent.attach setting Joint parent " +  jointRotateG );
        setDirectLiveParentNode(alignGroup);
        alignGroup.setCapability(Group.ALLOW_CHILDREN_WRITE);
        alignGroup.setCapability(Group.ALLOW_CHILDREN_EXTEND);
      }
      else
        alignGroup.addChild(subTreeToAttach);
   }

    // from BBot
    /*


   */

    BranchGroup wrapperBranch = new BranchGroup();
    wrapperBranch.addChild(componentDisplace);
    //wrapperBranch.compile();

    parent.getLiveNode().addChild(wrapperBranch);


  }

  public boolean getIsTopOfStableLimb()
  {
    return isTopOfStableLimb;
  }

  public void calculateIsTopOfStableLimb()
  // PRE:  All Children have have been calculated
  {
    DefaultMutableTreeNode tree = this.getTreeNode();

    if (tree.isLeaf())
      isTopOfStableLimb = true;
    else
    {
      boolean stableTester = true;

      Enumeration immediateChildren = tree.children();
      DefaultMutableTreeNode node;
      BComponent  nodeComponent;

      // recurse chidren of b;
      while (immediateChildren.hasMoreElements() && stableTester)
      {
        node = (DefaultMutableTreeNode)immediateChildren.nextElement();
        nodeComponent =  (BComponent)node.getUserObject();

        stableTester = stableTester && nodeComponent.getIsTopOfStableLimb() && !( nodeComponent.isJoint() );

      }


      isTopOfStableLimb = stableTester;

    }

   // BDebug.print ("BComponent.calculateIsTopOfStableLimb " + getName() + " " + isTopOfStableLimb);


  }

  public boolean isLive()
  {
    if (liveBotNode == null)
      return false;
    else
      return liveBotNode.isLive();
  }



   public void cleanseLiveNodes()
  {
        removeSelectionBoxFromScene();
        
        if (liveBotNode != null)
          if (liveBotNode.isLive())
            liveBotNode.detach();



        liveBotNode = null;
        connectRotateNode = null;
        directParent = null;
        arbitraryShape_live = null;

        // the cleanup of the shapes appearances are performed in the canvas
        if (liveShapes != null)
        {
          //BShape3D shape;
          for (int i = 0; i < liveShapes.size(); i++)
            ((BShape3D)liveShapes.get(i)).cleanse();

          liveShapes.clear();

        }

        limbRadius3D = null;
        limbRadius2D = null;
        limbTorque = null;
        totalRadius3D = null;
        angVelocity = null;
        prevAngVelocity = null;
        currentInertia = null;
        childInertia = null;
        parentInertia = null;
        prevRadius3D = null;
        parentRadius3D = null;
        prevParentRadius3D = null;
        parentAngVelocity = null;
        prevParentAngVelocity = null;
        tempV3D = null;
        tempT = null;
        prevVWorld = null;
        currentVWorld = null;
        tempVWorld = null;
        /* old stuff that was nulled
        limbTorque = null;
        limbRadius3D = null;
        limbRadius2D = null;
        angVelocity = null;
        prevAngVelocity = null;
        parentAngVelocity = null;
        prevParentAngVelocity = null;
        */

        lastCollidedp1 = null;
        lastCollidedp2 = null;
        lastCollidedMemberShape = null;
        lastCollidedShape = null;

        currentInertia = null;   // Current Velocity and Direction of Component
        childInertia = null;  // Inertia of children used in force/damage calculation
        parentInertia = null;  // Inertia of parent used in force/damage calculation
        collisionInertia = null; // Last Collision Inertia

        directParent = null;

        lastCollidedp1 =  null;
        lastCollidedp2 =  null;
        lastCollidedMemberShape = null;
        lastCollidedShape =  null;


        // SJS [18/02/2000] - Cleanse the selection stuff...
        myConnectPtsNode    = null;
        mySelectionBoxNode  = null;
        mySelectionBox      = null;

        //calculateLimbInertiaTempT = null;

        // DJG June 2001
  			hasMoved = false;
  			collisionCandidate = false;
  			//boundingRadius = 0.0f;
  			currentX = 0.0f;
  			currentY = 0.0f;
  			currentZ = 0.0f;
  			currentAbsRadius = 0.0d;
        maxVelocityExceededCtr = 0;
  			maxVelocityForLimb = 0;
  			componentVelocity = 0;
        childCount = 0;
    		prevChildCount = 0;


  }


  public void cleanse()
  {

    //BDebug.print("BComponent.cleanse" + getName());
    cleanseLiveNodes();

    traverseShapes(structure3D, null, TRAVERSE_MODE_CLEANSE);

    normalAppearance = null;
    structure3D = null;
    connectPointSelected = null;
    connectRotateNode = null;
    arbitraryShape = null;
    arbitraryShape_live = null;
    componentMaterial = null;
    ownerBot = null;

    connectPoints = null;
    directParent =null;
    treeNode = null;
    connectPointBase = null;

    componentColor = null;
    connectPointSelected = null;
    connectRotateNode = null;
    arbitraryShape = null;
    arbitraryShape_live = null;
    componentMaterial = null;
    liveShapes =null;

    //BDebug.print("BComponent: "+this.toString()+" cleansed.");

  }


  public boolean isPartOf(BShape3D theShape)
 {
    boolean isTrue = false;
    try{

        isTrue =  (theShape.getComponent() == this);


       } catch(Exception e) { }

    return isTrue;


 }

 transient private static Vector3f calculateLimbLoad_tempV = new Vector3f();
 transient private static Vector3f childLimbLoad_tempV = new Vector3f();

 public void calculateLimbLoad(float timeSlice, boolean worldStateCycle)
 //
 // PRE!!!!  All The Limb Loads for all this components children must be calculated before this.
 //
 // Calculates the current Limb Load of the component   (Effect on parent)
 // sets previous values, so should only be called once per cycle
 {
 
  if ( limbTorque == null ) // First Time Through - init all
  {
     limbRadius3D = new Vector3f();
     limbRadius2D = new Vector3f();
     limbTorque = new Vector3f();
     totalRadius3D = new Vector3f();
     angVelocity = new Vector3f();
     prevAngVelocity = new Vector3f();
     currentInertia = new Vector3f();
     childInertia = new Vector3f();
     parentInertia = new Vector3f();
     prevRadius3D = new Vector3f();
     parentRadius3D = new Vector3f();
     prevParentRadius3D = new Vector3f();
     parentAngVelocity = new Vector3f();
     prevParentAngVelocity = new Vector3f();
     angAcceleration = 0f;
     directAcceleration = 0f;
     childCount = 0;
     prevChildCount = 0;
     tempV3D = new Vector3f();
     tempT = new Transform3D();
     prevVWorld = new Vector3f();
     currentVWorld = new Vector3f();
     tempVWorld = new Vector3f();

     this.getLiveNode().getLocalToVworld(tempT);
     tempT.get(currentVWorld);

  }

  if ( damageMax == 0f )
     damageMax = 1;


  if( getDestroyed() || !this.getLiveNode().isLive() )
  {
   
   limbLoad = 0.0f;
   limbWeight = 0.0f;
   limbTorque.set(0f,0f,0f);
   limbRadius3D.set(0f,0f,0f);
   limbRadius2D.set(0f,0f,0f);
   prevAngVelocity.set(0f,0f,0f);
   angVelocity.set(0f,0f,0f);
   angAcceleration = 0f;
   directAcceleration = 0f;
   childCount = 0;
   prevChildCount = 0;

  }
  else
  {

   // DJG June 2001
   prevVWorld.set(currentVWorld);
   this.getLiveNode().getLocalToVworld(tempT);
   tempT.get(currentVWorld);

   currentX = currentVWorld.x * BGlobal.CANVAS_TO_WORLD_SCALE;
   currentY = currentVWorld.y * BGlobal.CANVAS_TO_WORLD_SCALE;
   currentZ = currentVWorld.z * BGlobal.CANVAS_TO_WORLD_SCALE;
   currentAbsRadius = java.lang.Math.sqrt((double)(( currentX * currentX ) + (currentZ * currentZ)));

   calculateLimbInertia(timeSlice);

   //---DJG---
   if ( componentVelocity > BPhysics.MAX_VELOCITY_CAP && BPhysics.CAP_VELOCITY_FLAG )
   {
     maxVelocityExceededCtr += 1;
   } else {
     maxVelocityExceededCtr = 0;
   }
   maxVelocityForLimb = componentVelocity;
   //---DJG---

   DefaultMutableTreeNode myNode = getTreeNode();

   //float tX = 0f;
   //float tY = 0f;
   //float tZ = 0f;

   if ( worldStateCycle )
   {

     // Do Limb Torque Calculations

	   limbTorque.set(0f,0f,0f);
	   totalRadius3D.set(0f,0f,0f);
	   limbRadius2D.set(0f,0f,0f);
   	 childInertia.set(0f,0f,0f);
	   angAcceleration = 0f;
	   directAcceleration = 0f;
	   childCount = 0;
	   limbWeight = 0.0f;
	   collisionInertia = null;
	   parentAngVelocity.set(0f,0f,0f); // will be set when parent is processed after
	   prevParentAngVelocity.set(0f,0f,0f);

	   // Get the current position of the Component in 3D space
	   // and calculate its velocity

	   //this.getLiveNode().getLocalToVworld(tempT);
	   //tempT.get(calculateLimbLoad_tempV);
	   //tX = calculateLimbLoad_tempV.x;  // This X Co-Ordinate
	   //tY = calculateLimbLoad_tempV.y;  // This Y Co-Ordinate
	   //tZ = calculateLimbLoad_tempV.z;  // This Z Co-Ordinate

   }

   /* DJG June 2001 - Moved Up
   calculateLimbInertia(timeSlice);

   //---DJG---
   if ( componentVelocity > BPhysics.MAX_VELOCITY_CAP && BPhysics.CAP_VELOCITY_FLAG ) {
   	maxVelocityExceededCtr += 1;
   } else {
   	maxVelocityExceededCtr = 0;
   }
   //---DJG---
   */


   // Now process all child (attached) components and accumulate torque load
   for (Enumeration componentC = myNode.children(); componentC.hasMoreElements() ;)
   {

   		BComponentNode child = (BComponentNode)componentC.nextElement();
      BComponent current = child.getComponent();

      if ( current.getLiveNode().isLive() && !current.getDestroyed() )
      {
        if ( worldStateCycle )
    		{

	         // DG 01/09/2000
	         // Calculate the torque of the child and its children individually
	         // Calculate the angular velocity of each child individually and accumulate as non-vectors
	         // to evaluate "work done" by a given joint

	         //float cX = 0f;
	         //float cY = 0f;
	         //float cZ = 0f;

	         // UNEEDED // tempT.setIdentity();
	         // UNEEDED //calculateLimbLoad_tempV.set(0f,0f,0f);
	         //current.getLiveNode().getLocalToVworld(tempT);
	         //tempT.get(calculateLimbLoad_tempV);

           //BDebug.print ("BComponent.calculateLimbLoad " + this  + " raw calculateLimbLoad_tempV " + calculateLimbLoad_tempV );


	         //cX = calculateLimbLoad_tempV.x;  // Child X Co-Ordinate
	         //cY = calculateLimbLoad_tempV.y;  // Child Y Co-Ordinate
	         //cZ = calculateLimbLoad_tempV.z;  // Child Z Co-Ordinate

	         float dX = current.currentX - this.currentX;  // Difference X
	         float dY = current.currentY - this.currentY;  // Difference X
	         float dZ = current.currentZ - this.currentZ;  // Difference Z

	         // Create new 2D vector to get Length from (0,0,0). Used for instantaneous Torque due to gravity
	         calculateLimbLoad_tempV.set(dX,0f,dZ);
	         //calculateLimbLoad_tempV.scale(1 / BGlobal.HEIGHT_SCALE);
	         //calculateLimbLoad_tempV.scale(BGlobal.CANVAS_TO_WORLD_SCALE);

	        // BDebug.print ("BComponent.calculateLimbLoad " + this  + " zero y calculateLimbLoad_tempV " + calculateLimbLoad_tempV );

	         childLimbLoad_tempV.set(calculateLimbLoad_tempV); // distance from child to its children
	         childLimbLoad_tempV.add(current.limbRadius2D); // total distance to childs children

	         // update direction/distance vector
  	       limbRadius2D.add(childLimbLoad_tempV);

	         // Now scale to realize actual torque of childs incumbant load
	         childLimbLoad_tempV.scale(current.limbWeight);

	         // Calculate torque of child itself
	         // Calculate Torque being (MASS * ACCELERATION) * DISTANCE  (F = MA and T = Fd at 90 degrees)
	         float torque = (float)current.weight * BPhysics.GRAVITY * calculateLimbLoad_tempV.length();

	         if ( calculateLimbLoad_tempV.length() != 0f)
	         {
	          calculateLimbLoad_tempV.normalize();
	          calculateLimbLoad_tempV.scale(torque);
	         }

	         // BDebug.print ("BComponent.calculateLimbLoad " + this  + " norm/scale calculateLimbLoad_tempV " + calculateLimbLoad_tempV );


		         // Add to total torque
	         if ( calculateLimbLoad_tempV.length() == 0f) // Normalising a 0,0,0 Vector returns a NaN Vector;
		           calculateLimbLoad_tempV.set(0f,0f,0f);

	         if ( childLimbLoad_tempV.length() != 0f)
	           calculateLimbLoad_tempV.add(childLimbLoad_tempV);


	         // BDebug.print ("BComponent.calculateLimbLoad " + this  + " pre add limbTorque " + limbTorque );
	         //BDebug.print ("BComponent.calculateLimbLoad " + this  + " pre add calculateLimbLoad_tempV " + calculateLimbLoad_tempV );

	         limbTorque.add(calculateLimbLoad_tempV);

	         // Add to vector used to calculate gravitational effects
	         tempV3D.set(dX, dY, dZ);
	         // tempV3D.scale(1 / BGlobal.HEIGHT_SCALE);
	         //tempV3D.scale(BGlobal.CANVAS_TO_WORLD_SCALE);
	         tempV3D.scale(current.weight);
	         tempV3D.add(current.limbRadius3D);
	         totalRadius3D.add(tempV3D);

	         // Update child distance to parent to calculate angular velocity of child
	         // Use these to calculate child angular acceleration
	         tempV3D.set(dX, dY, dZ);
	         //tempV3D.scale(1 / BGlobal.HEIGHT_SCALE);
	         //tempV3D.scale(BGlobal.CANVAS_TO_WORLD_SCALE);

	         // Set Child/Parent Distances
	         if ( current.parentRadius3D != null )
	           current.prevParentRadius3D.set(current.parentRadius3D);
	         else
	           current.prevParentRadius3D.set(tempV3D);

	         current.parentRadius3D.set(tempV3D);

	         // Set Child/Parent Velocities
	         if ( current.parentAngVelocity != null )
	           current.prevParentAngVelocity.set(current.parentAngVelocity);
	         else
	           current.prevParentAngVelocity.set(current.parentAngVelocity);

	         current.parentAngVelocity.set(current.parentRadius3D);
	         current.parentAngVelocity.sub(current.prevParentRadius3D);

           // DJG June 2001
           // Don't scale as now only executed once every CALC_TIMESLICE
	         //current.parentAngVelocity.scale(BGlobal.CALC_TIMESLICE / timeSlice);

	         // Accumulate angular acceleration
  	       tempV3D.set(current.parentAngVelocity);
	         tempV3D.sub(current.prevParentAngVelocity);
	         angAcceleration += ( tempV3D.length() * ( current.weight + current.limbWeight ) );

	         // Accumulate direct acceleration
	         directAcceleration = current.parentAngVelocity.length() - current.prevParentAngVelocity.length();
	         directAcceleration = ( directAcceleration * ( current.weight + current.limbWeight ) );

	         // Sundry accumulators
	         limbWeight += current.weight;
	         limbWeight += current.limbWeight;
	         childCount += current.childCount;
	         childCount += 1;

	         // Effective Inertia of Children
	         float scalar = 0.8f; // Proportion of Inertia from child to effect this component
	         // Hack which hardsets influence of attached components on total inertia.

	         calculateLimbLoad_tempV.set(current.currentInertia);
	         calculateLimbLoad_tempV.add(current.childInertia);
	         calculateLimbLoad_tempV.scale(scalar);
	         childInertia.add(calculateLimbLoad_tempV);
	         //childInertia += java.lang.Math.min((( current.weight + current.childInertia ) * scalar)
	         //                                         , current.maxLimbLoad * scalar);

  	       //BDebug.print("BComp.calcLimbLoad: "+this.toString()+" rad="+radius+" tq="+torque+" ttq="+limbTorque.length());

        }

        //---DJG---
	      if ( current.maxVelocityExceededCtr > this.maxVelocityExceededCtr )
	        this.maxVelocityExceededCtr = current.maxVelocityExceededCtr;
          
        maxVelocityForLimb = java.lang.Math.max(maxVelocityForLimb,current.maxVelocityForLimb);

        if ( this.hasMoved )
          current.hasMoved = true;
	      //---DJG---

   		}

	 } // Next Child Component



   if ( worldStateCycle )
   {

     if ( childCount > prevChildCount )
   	   prevChildCount = childCount;
       
     // Calculate torque direction of rotation and distance (scale) for this timeslice
     if ( limbRadius3D != null )
	     prevRadius3D.set(limbRadius3D);
     else
	     prevRadius3D.set(totalRadius3D);

     // Update members
     limbRadius3D.set(totalRadius3D);     // 3 Dimensional Vector of instantaneous torque

     // BDebug.print ("BComponent.calculateLimbLoad " + this  + " limbTorque " + limbTorque );

	   limbLoad = limbTorque.length();  // 1 Dimensional Scalar of instantaneous torque

	   // BDebug.print ("BComponent.calculateLimbLoad " +  this + " limbLoad " + limbLoad );

  	 prevAngVelocity.set(angVelocity); // required to calculate velocity and acceleration
	   angVelocity.set(totalRadius3D);
	   angVelocity.sub(prevRadius3D);   // Net Velocity and Direction
	   angVelocity.scale( 1 / this.limbWeight );

     // DJG June 2001
     // Don't scale as now only executed once every CALC_TIMESLICE
	   //angVelocity.scale(BGlobal.CALC_TIMESLICE / timeSlice); // Normalize to CALC_TIMESLICE


	   //if ( this instanceof BJoint )
	   //{
	   //  BDebug.print("BComponent.calcLimbLoad: "+this.toString()+" ts="+(BGlobal.CALC_TIMESLICE / timeSlice)+" thisRad="+totalRadius3D.length()+" prevRad="+prevRadius3D.length()+" diff="+(totalRadius3D.length()-prevRadius3D.length()));
	   //  BDebug.print("Limb Load: "+this.toString()+" tq="+limbTorque.length()+" V="+((BJoint)this).currentVelocity);
	   //  BDebug.print("           Prev="+prevAngVelocity.y+" this="+angVelocity.y);
	   //}
     
   }


  }// else Destroyed or part of Chassis

 }


 public float getLimbLoadScalar()
  {   return limbLoad; }

 public void getVelocity(Vector3f theVelocity)
 // return vector in meters/sec

 {
    /*if (getBot().getName().equals("0 Bashed 2"))
    {
      BDebug.print (" getVelocity " + getName() +  " + currentInertia " + currentInertia );
      BDebug.print (" getVelocity " + getName() +  " + this.weight " + this.weight );

    }     */

    if (this.weight == 0)
      this.weight = 0.0000001f;

    theVelocity.set(currentInertia);
    theVelocity.scale(1/this.weight);
 }


 transient private static Vector3f calculateLimbInertia_tempV = new Vector3f();

 //transient private static Transform3D calculateLimbInertiaTempT = new Transform3D();

 private void calculateLimbInertia(float timeSlice)
 {


   // Recalculate current inertia
   if ( this.getLiveNode() != null && !getDestroyed() )
   {
     //if (calculateLimbInertiaTempT == null)
     //  calculateLimbInertiaTempT =new Transform3D();

     //prevVWorld.set( currentVWorld);

     //this.getLiveNode().getLocalToVworld(tempT);
     //tempT.get(currentVWorld);

     calculateLimbInertia_tempV.sub(currentVWorld, prevVWorld); // Net direction and distance

     calculateLimbInertia_tempV.scale(BGlobal.CANVAS_TO_WORLD_SCALE); // Length converted to Meters

     /*if (getBot().getName().equals("0 Bashed 2"))
     {
      BDebug.print( getName() +  " vel " + calculateLimbInertia_tempV.length() + " timeSlice " + timeSlice );
      BDebug.print( getName() +  " prevVWorld " +prevVWorld );
      BDebug.print( getName() +  " currentVWorld " + currentVWorld );
      BDebug.print( getName() +  " this.weight " + this.weight );


     }*/

     // Predict position in next timeslice
     //nextX = currentX + calculateLimbInertia_tempV.x;
     //nextY = currentY + calculateLimbInertia_tempV.y;
     //nextZ = currentZ + calculateLimbInertia_tempV.z;

     calculateLimbInertia_tempV.scale(1 / timeSlice);   // Converted to Meters/Sec

     /*if (getBot().getName().equals("0 Bashed 2"))
     {
      BDebug.print( getName() +  " length " + calculateLimbInertia_tempV.length() + " calculateLimbInertia_tempV " + calculateLimbInertia_tempV );
     }*/

     if ( ownerBot.getOpponentBot() != null )
     {

     		float diffX = java.lang.Math.abs(currentX - (ownerBot.getOpponentBot().getArenaPosition().x * BGlobal.CANVAS_TO_WORLD_SCALE));
     		float diffZ = java.lang.Math.abs(currentZ - (ownerBot.getOpponentBot().getArenaPosition().z * BGlobal.CANVAS_TO_WORLD_SCALE));
     		float colDistance = boundingRadius + (ownerBot.getOpponentBot().getBotReach()) + 1.0f;

     		/*
     		if ( toString().startsWith("C338") || toString().startsWith("C64") )
       		BDebug.print("---- "+toString()+" x="+currentX+" z="+currentZ+" rad="+boundingRadius+" oppX="+(ownerBot.getOpponentBot().getArenaPosition().x * BGlobal.CANVAS_TO_WORLD_SCALE)+" oppZ="+(ownerBot.getOpponentBot().getArenaPosition().z * BGlobal.CANVAS_TO_WORLD_SCALE)+" oppRad="+(ownerBot.getOpponentBot().getBoundingRadius() * BGlobal.CANVAS_TO_WORLD_SCALE));
     		*/

     		if ( diffX <= colDistance && diffZ <= colDistance)
       		collisionCandidate = true;
     		else
       		collisionCandidate = false;
     } else {
     		collisionCandidate = false;
     }

     //---DJG---
     componentVelocity = calculateLimbInertia_tempV.length(); // Current actual Velocity
     if ( ! getChassisLock() )
     {
       if ( collisionCandidate )
       	ownerBot.fastestInboundsComponentSpeed = Math.max(ownerBot.fastestInboundsComponentSpeed, componentVelocity );

       ownerBot.fastestComponentSpeed = Math.max(ownerBot.fastestComponentSpeed, componentVelocity );
     }

     this.hasMoved = (this.componentVelocity > 0.0001);

     if ( this.isJoint() && !hasMoved )
       this.hasMoved = ((BJoint)this).hasMoved();

     ownerBot.movedInLastTimeslice = (ownerBot.movedInLastTimeslice || this.hasMoved);

     //---DJG---

     // DJG June 2001
     if ( this.currentAbsRadius + this.boundingRadius > BBot.arenaRadius && this.hasMoved)
       ownerBot.checkArenaCollisions = true;
     if ( !this.getChassisLock() && this.currentY - this.boundingRadius <= 0 && this.hasMoved)
       ownerBot.checkArenaCollisions = true;

     // Scale back to Max Velocity
     if ( componentVelocity > BPhysics.MAX_VELOCITY_CAP )
        calculateLimbInertia_tempV.scale( BPhysics.MAX_VELOCITY_CAP / componentVelocity );

     calculateLimbInertia_tempV.scale(this.weight); // Converted to Inertia (KG/M/Sec)

     currentInertia.set(calculateLimbInertia_tempV);

      /*if (getBot().getName().equals("0 Bashed 2"))
     {
      BDebug.print( getName() +  " currentInertia " + currentInertia );
     }    */

     //ownerBot.setArenaComments(toString()+" Inertia="+tempV.length());
   }
   else
   {
     currentInertia.set(0f,0f,0f);  // No Inertia
   }



 }

 // Used to get recalculated inertia after rolling back
 private transient float inertiaRollbackScalar = 1f;

 public void calcAdjustedInertia(float timeSlice)
 {

   inertiaRollbackScalar = 1f;

   if ( this.getLiveNode() != null && !getDestroyed() )
   {
     this.getLiveNode().getLocalToVworld(tempT);
     tempT.get(tempVWorld);

     calculateLimbInertia_tempV.sub(tempVWorld, prevVWorld);
     calculateLimbInertia_tempV.scale(BGlobal.CANVAS_TO_WORLD_SCALE);
     calculateLimbInertia_tempV.scale(1 / timeSlice);

     float thisVelocity = calculateLimbInertia_tempV.length();

     //BDebug.print("BComp.getAdjInertia: thisVel="+thisVelocity+" currVel="+componentVelocity+" inertia="+currentInertia.length());

     if ( thisVelocity > 0.0f || componentVelocity > 0.0f )
     {
       //float velScalar = ((thisVelocity + componentVelocity) / 2) / componentVelocity;
       //currentInertia.scale(velScalar);
       if ( componentVelocity <= 0.0f )
         inertiaRollbackScalar = java.lang.Math.max(thisVelocity / 2, 1.0f);
       else
         inertiaRollbackScalar = java.lang.Math.max( ((thisVelocity + componentVelocity) / 2) / componentVelocity, 1.0f);
     }
     else
     {
       //currentInertia.scale(0.00001f); // So not NaN
       inertiaRollbackScalar = 0.00001f;
     }

     if ( Float.isNaN(inertiaRollbackScalar) )
     {
       inertiaRollbackScalar = 0.00001f;
     }

   }

 }

 public BShape3D getArbitraryShape_live()
 {
    if (arbitraryShape_live != null)
      return arbitraryShape_live;
    else
      return arbitraryShape;
 }

 public float getVolume()
   { return volume; }

 
 public void setVolume(float newVol)
 {
   volume = newVol;
 }

 public float guessVolume()
 // guesses the volume of the compnent based on a bounding box.
 //  This will be based on the orientation imported from 3ds, not any other orientations.
 {

  float vol = 0.0f;

 // try
 // {
    vMinX = Float.MAX_VALUE;
    vMinY = Float.MAX_VALUE;
    vMinZ = Float.MAX_VALUE;

    vMaxX = Float.MIN_VALUE;
    vMaxY = Float.MIN_VALUE;
    vMaxZ = Float.MIN_VALUE;

    traverseShapes(structure3D, new Group(), TRAVERSE_MODE_VOLUME);

    vol = ( ( vMaxX -  vMinX ) * BGlobal.CANVAS_TO_WORLD_SCALE) *
          ( ( vMaxY - vMinY ) * BGlobal.CANVAS_TO_WORLD_SCALE ) *
          ( ( vMaxZ - vMinZ ) * BGlobal.CANVAS_TO_WORLD_SCALE );

   return vol;
 }

  public BSelectionBox getSelectionBox(
    Color3f color,
    float boundingOffset,
    float cornerLength,
    float lineWidth,
    float alpha )
  {
    vMinX = Float.MAX_VALUE;
    vMinY = Float.MAX_VALUE;
    vMinZ = Float.MAX_VALUE;

    vMaxX = Float.MIN_VALUE;
    vMaxY = Float.MIN_VALUE;
    vMaxZ = Float.MIN_VALUE;

    traverseShapes(structure3D, new Group(), TRAVERSE_MODE_VOLUME);

    return new BSelectionBox(
      vMinX - boundingOffset,
      vMinY - boundingOffset,
      vMinZ - boundingOffset,
      vMaxX + boundingOffset,
      vMaxY + boundingOffset,
      vMaxZ + boundingOffset,
      color,
      cornerLength,
      lineWidth,
      alpha
      );
  }



private static Transform3D getCollisionMomentum_tempTran = new Transform3D() ;
private static Vector3f getCollisionMomentum_tempVec = new Vector3f()  ;
private static Point3d getCollisionMomentum_tempPoint = new Point3d()  ;
private static Vector3d getCollisionMomentum_momDir = new Vector3d()  ;

public Vector3f getCollisionMomentum()
  {
    BComponentNode myNode = getTreeNode();

    float momentumScalar = this.currentInertia.length();

    // getlocal

    this.getLiveNode().getLocalToVworld(getCollisionMomentum_tempTran);
    //Vector3f tempVec = new Vector3f();

    //convert to point
    getCollisionMomentum_tempTran.get(getCollisionMomentum_tempVec);

    getCollisionMomentum_tempPoint.set(getCollisionMomentum_tempVec);

    //invert currentInertia
    getCollisionMomentum_momDir.set(currentInertia);
    getCollisionMomentum_momDir.negate();
    getCollisionMomentum_momDir.normalize();


    while ( (myNode.getParent() != null)  )
    {



                myNode = (BComponentNode) myNode.getParent();
                BComponent c = myNode.getComponent();

                if (BCollision.boundsIntersect(c, getCollisionMomentum_tempPoint, getCollisionMomentum_momDir) )
                {
                        momentumScalar = momentumScalar + c.currentInertia.length();
                      //  BDebug.print("BComponent.getCollisionMomentum ray intersects " + c );

                }

    }



    getCollisionMomentum_tempVec.set ( currentInertia );
    if (getCollisionMomentum_tempVec.length() != 0)
    {
                getCollisionMomentum_tempVec.normalize();

                float stableM = this.getStableNeigbourMomentum();

                if (momentumScalar < stableM)
                        momentumScalar = stableM;



                getCollisionMomentum_tempVec.scale(momentumScalar);
    }

    // DJG July 2001
    //BDebug.print("InertiaRollbackScalar="+inertiaRollbackScalar);
    getCollisionMomentum_tempVec.scale(inertiaRollbackScalar);

    //BDebug.print("Collision Momentum="+getCollisionMomentum_tempVec.length());

    return new Vector3f(getCollisionMomentum_tempVec);



 }

  public float getStableNeigbourWeight()
 {
    float weight = this.weight;

    BComponent[] components = getBot().getBreathComponents();

    for (int i = components.length-1; i >=0; i--)
    {
      if (components[i] != this)
        if (!getBot().notInConnectedWithoutJointsCache(this, components[i] ) )
          weight = weight + components[i].weight;
    }

    return weight;

 }


 public float getStableNeigbourMomentum()
 {
    float momentumScalar = this.currentInertia.length();

    BComponent[] components = getBot().getBreathComponents();

    for (int i = components.length-1; i >=0; i--)
    {
      if (components[i] != this)
        if (!getBot().notInConnectedWithoutJointsCache(this, components[i] ) )
        {

          momentumScalar = momentumScalar + components[i].currentInertia.length();
          //BDebug.print (getName() + " getStableNeigbourMomentum adding " +  components[i] + " " + components[i].currentInertia.length() );

        }
    }

    return momentumScalar;

 }

 public float getStableParentMomentum()
 // returns the weight of this limb + all parents up until the first joint

 {
         DefaultMutableTreeNode myNode = getTreeNode();
         float momentumScalar = this.currentInertia.length();

         boolean jointHit = false;

         while ( (myNode.getParent() != null)  && !jointHit )
         {

                myNode = (DefaultMutableTreeNode) myNode.getParent();
                BComponent c = (BComponent)myNode.getUserObject();
                momentumScalar = momentumScalar + c.currentInertia.length();

                if ( c instanceof BJoint)
                  jointHit = true;

         }

         return momentumScalar;


 }


 public float getStableParentWeight()
 // returns the weight of this limb + all parents up until the first joint

 {

         DefaultMutableTreeNode myNode = getTreeNode();
         float sWeight = this.weight;
         boolean jointHit = false;

         while ( (myNode.getParent() != null)  && !jointHit )
         {

                myNode = (DefaultMutableTreeNode) myNode.getParent();
                BComponent c = (BComponent)myNode.getUserObject();
                sWeight = sWeight + c.weight;

                if ( c.isJoint() )
                        jointHit = true;




         }

         return sWeight;


 }


 public float getLimbWeight()
 {
   DefaultMutableTreeNode rootNode = getTreeNode();
   Enumeration enumerator = rootNode.depthFirstEnumeration();
   float limbWeight = this.weight;
   while (enumerator.hasMoreElements())
   {
     DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
     BComponent c = (BComponent)node.getUserObject();
     if (!c.getDestroyed())
      limbWeight = limbWeight + c.weight;
    }

   return limbWeight;

 }


 public int getDamageCurrent()
   { return damageCurrent; }

 public int getHealthCurrent()
   { return damageMax - damageCurrent; }


 public int getDamageMax()
  { return damageMax; }

 public void addToDamageCurrent(int x)
 {


  if (getChassisLock() )
  {
    ownerBot.getChassis().addToDamageCurrent( x);
    //BDebug.print(ownerBot+" takes chassis damage.");
  }
  else
  {
    ownerBot.addToBotDamage( java.lang.Math.min( x, damageMax - damageCurrent) );
    damageCurrent = java.lang.Math.min( damageCurrent + x, damageMax );
    //BDebug.print(ownerBot+" takes "+x+" damage.");


  }
  //BDebug.print ( "BComponent addToDamage " + getName() + " damage current " + damageCurrent + " damageMax " +   damageMax);

 }

 public double getHealthPercent()
 {
 // BDebug.print ( "BComponent getHealthPercent " + getName() + " damage current " + damageCurrent + " damageMax " +   damageMax);

  return 100.0 - ((double)damageCurrent / (double)damageMax )  * 100.0 ;
 }

 public BBot getBot()
 {
  return ownerBot;
 }

 public void setBot(BBot owner)
 {
  ownerBot = owner;
 }

 public void  clearDamageCurrent()
 {
  damageCurrent = 0;
 }



 public void reConnect()
 {
  if (directParent != null)
    directParent.addChild(getLiveNode());

 }

 protected void setDirectLiveParentNode(Group dParent)
 {
        directParent = dParent;
 }

 public Group getDirectLiveParentNode()
 {
        return directParent;
 }



 /*
 public BAnimation getDestructionAnimation(BranchGroup world)
 {
    BAnimationSet set = new BAnimationSet(2);

    set.add(componentmaterial.getDestructionAnimation(this, world));
    set.add(new BDetachAnimation(this));

    return set;
 }


 protected class BDetachAnimation extends BAbstractAnimation
 {
   protected BComponent c;

    public BDetachAnimation(BComponent c)
      { this.c = c; }

    public boolean animate()
      { c.detach(); return false; }
 }

 */

 protected void detach()
   { liveBotNode.detach(); }



 public void upgrade()
 {
  BComponent upgradeTwin =   BComponent.loadFromFile( BGlobal.COMPONENTPATH+getComponentFilePath(), false );
  upgrade(upgradeTwin);

  //BDebug.print ("BComponent.upgrade() super called for upgrade of"  +  getName());

 }



 protected void upgrade (BComponent upgradeTwin)
 {

            upgradeTwin.componentmaterial = BComponentMaterial.getMaterial(upgradeTwin.materialCode);

            //BDebug.print ("BComponent.upgrade(upgradeTwin) super called for upgrade of"  +  getName());


            setSerialNo(upgradeTwin.getSerialNo());
            setDetails(upgradeTwin.getDetails());
            setWeight(upgradeTwin.getWeight());
            setPrice(upgradeTwin.getPrice());
            this.setMaterialType(upgradeTwin.componentmaterial );
            this.transparency = upgradeTwin.transparency;
           // BDebug.print (this + ".uprade Transpanrency " +  transparency);

            //energyUsage = upgradeTwin.energyUsage;
            setVersion(BGlobal.APP_VERSION);
            setConvex(upgradeTwin.getIsConvex());
            ambientSoundName = upgradeTwin.ambientSoundName;
            this.setAutoTextureGen( upgradeTwin.getAutoTextureGen() );

            if (upgradeTwin.damageMax == 0)
              BDebug.print("WARNING ASSERTION FAILURE " +getName() +" UPGRADE SETTING DAMAGE TO 0 " + upgradeTwin.damageMax);

            damageMax = upgradeTwin.damageMax;
            //damageDealStriking = upgradeTwin.damageDealStriking;

            Vector oldConnectPoints = connectPoints;
            connectPoints = upgradeTwin.connectPoints;

            for (int i = oldConnectPoints.size() - 1; i >=0 ; i-- )
            {
              BConnectPoint oldC = (BConnectPoint)oldConnectPoints.get(i);
              int index =  connectPoints.indexOf(oldC);
              if (index != -1)
              {
                BConnectPoint newC =  (BConnectPoint)connectPoints.get(index);
                //BDebug.print ("Setting connectPoint " );
                newC.setActive(oldC.isActive());
              }

            }
            // need to set active on the inactive connectpoints
            //connectPoints = upgradeTwin.connectPoints
            //;





 }


 public void makeLight()
 // deletes the stuff not needed for the battleplayer
 {
        details = null;
        connectPoints = null;

        serialNum = null;                // Internal Serial Number
        componentFilePath = null;       // Path to original component (used by Upgrade Process)


 }

 public boolean isUnique()
  {
    return uniqueItem;
  }

  public void setUnique(boolean unique)
  {
    uniqueItem = unique;
  }

 public String getDetails()
 // returns the 200 char blurb separately
 {
    return details;

 }

 public boolean getIsConvex()
 {
  return isConvex;
 }

 public void setConvex(boolean b)
 {
  isConvex = b;

 }

 private transient long lastCacheTime;

 public long getCollisionCacheTime()
 {
    return lastCacheTime;
 }

 public void setCollisionCache(Point3d lastCollidedp1, Point3d lastCollidedp2, Shape3D lastCollidedMemberShape,BShape3D lastCollidedShape  )
 {
  if ( this.lastCollidedp1 == null)
       this.lastCollidedp1 =  new Point3d();

  if ( this.lastCollidedp2 == null)
       this.lastCollidedp2 =  new Point3d();

  this.lastCollidedp1.set(lastCollidedp1);

  if (lastCollidedp2 == null)
    this.lastCollidedp2 = null;
  else
    this.lastCollidedp2.set(lastCollidedp2);

  this.lastCollidedMemberShape = lastCollidedMemberShape;
  this.lastCollidedShape =  lastCollidedShape;

  lastCacheTime = System.currentTimeMillis();


 }


 public Point3d getCollisionPoint()
 {

    Point3d midPoint = new Point3d();

    if ((lastCollidedp1 == null) && (lastCollidedp2 == null))
    {
      // do nothing
    }
    else if ( (lastCollidedp1 == null) && (lastCollidedp2 != null) )
      midPoint.set(lastCollidedp2);
    else if ( (lastCollidedp1 != null) && (lastCollidedp2 == null) )
      midPoint.set(lastCollidedp1);
    else if ((lastCollidedp1 != null) && (lastCollidedp1 != null))
      midPoint.interpolate(lastCollidedp1, lastCollidedp2, 0.5);

    if (tempT == null)
      tempT = new Transform3D();

    if ( lastCollidedMemberShape != null  )
      lastCollidedMemberShape.getLocalToVworld(tempT);
    else
      getLiveNode().getLocalToVworld(tempT);
      
    tempT.transform(midPoint);

    return midPoint;


 }

 public Point3d getLastCollidedp1()
 {
  return lastCollidedp1;

 }

  public Point3d getLastCollidedp2()
 {
  return lastCollidedp2;

 }


 protected void addDeveloperAttributes(Vector theList)
 {
  Vector tempV;

  tempV = new Vector(2);
  tempV.add("material");
  tempV.add(componentmaterial.getName() + "");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("Polygon count");
  tempV.add(getPolyCount() + " stripified triangles");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("Volume");
  tempV.add(getVolume()+ " m3");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("Is Convex");
  tempV.add(isConvex + "");
  tempV.trimToSize();
  theList.add(tempV);


 }


 public Vector getAttributeStrings()
 // returns a Vector of tuples  (heading, value )
 // 2-d Vector
 {

  Vector theList = new Vector();

  Vector tempV;

  tempV = new Vector(2);
  tempV.add("name");
  tempV.add(name);
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("cost");
  tempV.add(price +" "+ BGlobal.MONEY_UNIT);
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("weight");
  tempV.add(BMath.format(getDisplayWeight(), 1) + " kg");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("damage points");
  tempV.add( damageMax + " " + BGlobal.HEALTH_UNIT);
  tempV.trimToSize();
  theList.add(tempV);

  if (BGlobal.DEVELOPER_MODE)
    addDeveloperAttributes(theList);


  return theList;

 }


 public void initForArena(BChassis theChassis, BBot owner)
 {
    ownerBot = owner;
    damageCurrent = 0;
    clearDamageCurrent();
    inResultantMove = false;

    if (this.weight == 0)
      this.weight = 0.00000001f;



 }

 protected static final int DAM_THRESHOLD = 5;

 public float resistDamage(float dam )
 {
  if (dam < DAM_THRESHOLD )
    dam = 0;


  return  dam;

 }

  public float applyDamage(float dam )
  {
    // DJG July 2001
    // Change of Philsophy
    // Apply resistance first
    //return java.lang.Math.min(getHealthCurrent(), dam );
    return dam;
  }


  public int getStateModifier()
  { return 1500; }

  public String getArenaEventType()
  { return BArenaEvent.DESTRUCTION ; }

  // SJS [28/09/2000]: Bot Attributes...
  // -------------------------------------------------------------

  protected static final float SPEED_MIN = 0.0f;
  protected static final float SPEED_MAX = 5.0f;

 
  public BAttributeSet getAttributes()
  {
    BAttributeSet as = new BAttributeSet();

    as.add( new BAttribute(
      "Cost",
      new BAbsoluteValue(BMath.format(price)+ " "+ BGlobal.MONEY_UNIT),
      BAttribute.HIGH));

    as.add( new BAttribute(
      "Weight",
      new BAbsoluteValue(BMath.format( getDisplayWeight() )+ " "+ BGlobal.WEIGHT_UNIT),
      BAttribute.HIGH));

    as.add( new BAttribute(
      "Hit points",
      new BAbsoluteValue(BMath.format(damageMax)+ " "+ BGlobal.HEALTH_UNIT),
      BAttribute.HIGH));

    return as;
  }

  public Vector3f getCachedVWorld()
  // return the v world vector that is caculated 1 per execution cycle in
  // BComponent.caculateInertia
    { return currentVWorld; }


 // accessor methods
  public void setAutoTextureGen(boolean b)
  {
    //BDebug.print ("BComponent set Aucogen called" + b);
    autoTextureGen = b;
  }

  public boolean getAutoTextureGen()
    { return autoTextureGen; }


  public String getWorldStateCode()
  {
    char dot = BWorldState.DELIMITER;

    return  BBot.stateCode+ dot+
            BComponent.stateCode+ dot+
            getComponentType().toLowerCase()+ dot+
            getName();

  }


  public boolean isParentOf(BComponent other)
  {
    return this.getTreeNode().isNodeDescendant(other.getTreeNode() );
  }


  public BConnectPoint selectClosestConnectPoint(Vector3f position)
  {
    if (getChassisLock() )
      if (!isStablePartOfChassis() )
        return null;


    Point3f viewPos = new Point3f(position);
    BConnectPoint closest = null;
    float closestPos = Float.MAX_VALUE;

    for (int i=0; i<connectPoints.size(); i++ )
    {
      BConnectPoint current = (BConnectPoint) connectPoints.get(i);

      if (current.isActive())
      {
        Point3f currentPos = new Point3f( current.getLocation());
        float dist =  viewPos.distance(currentPos);
        if (dist < closestPos)
        {
          closest = current;
          closestPos = dist;
        }

      }


    }

    return closest;


  }

  public void snapRotateConnect(int deg)
  {
    double currentDeg = getZAlignment() * BPhysics.DEGREE_CONVERT;
   // BDebug.print ("----BComponent.snapRotateConnect currentDeg " + currentDeg);
    double absDeg = Math.abs(deg);

   // BDebug.print ("BComponent.snapRotateConnect absDeg " + absDeg);
    double degDiff;
    if (currentDeg > 0 )
    {
     // BDebug.print ("POSITIVE CALLED deg > 0");

      if (deg < 0)
        degDiff = Math.abs( (currentDeg % absDeg) );
      else
        degDiff = Math.abs( (currentDeg % absDeg) - absDeg);
    }
    else
    {

      if (deg > 0)
      {
       // BDebug.print ("NEG CALLED deg > 0");
        degDiff = Math.abs( (Math.abs(currentDeg) % absDeg) );
      }
      else
      {
       // BDebug.print ("NEG CALLED deg > 0");
        degDiff = Math.abs( (Math.abs(currentDeg) % absDeg) - absDeg);

      }

    }

   // BDebug.print ("BComponent.snapRotateConnect degDiff " + degDiff);

    double rotAmount;
    if (degDiff < 0.001)
      rotAmount = absDeg;
    else
      rotAmount = degDiff;

    //BDebug.print ("BComponent.snapRotateConnect rotAmount " + rotAmount);


    if (deg < 0)
      rotateConnect(getZAlignment() + ( -rotAmount * BPhysics.RAD_CONVERT));
    else
      rotateConnect(getZAlignment() + ( rotAmount * BPhysics.RAD_CONVERT));


  }

  private static AxisAngle4d connectRotate  = new  AxisAngle4d();
  private static Transform3D curTrans = new Transform3D();

  public void rotateConnect(double zRot)
  {
    TransformGroup rotateNode = getConnectRotateNode();
    Vector3d facing = getConnectPointBase().facing;

    //double zRot  = getZAlignment();

    //zRot +=  dz;
    setZAlignment(zRot);

    connectRotate.set(facing.x, facing.y, facing.z, zRot);

    curTrans.setRotation(connectRotate);
    rotateNode.setTransform(curTrans);
  }


  private static double y_factor = 0.01;

  public void rotateConnect( int dx, int dy)
  // Only rotates around the connect point
  // This method should be moved to BComponent

  {
    rotateConnect (getZAlignment() + (dy * y_factor));

  }

  // DJG June 2001
  public boolean isJoint()
  {
    return ( this.getComponentType() == BJoint.COMPONENT_TYPE || this.getComponentType() == BAutoRotateJoint.COMPONENT_TYPE);
  }

  public void resetForSim()
  {
    hasMoved = false;
    collisionCandidate = false;
    currentX = 0.0f;
    currentY = 0.0f;
    currentZ = 0.0f;
    currentAbsRadius = 0.0d;
    maxVelocityExceededCtr = 0;
    maxVelocityForLimb = 0;
    componentVelocity = 0;
    childCount = 0;
    prevChildCount = 0;
  }



}

















