package roboforge;

import roboforge.ui.console.*;
import roboforge.ui.events.*;

import java.awt.GraphicsConfiguration;
//import java.util.Vector;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.util.*;

import com.sun.j3d.utils.universe.*;

public class BConsoleCanvas
  extends
    Canvas3D
  implements
    BActivateable,
    BRenderer
{

  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;


// Constants...
  // -------------------------------------------------------------

  protected static final double Y_PERSPECTIVE_CORRECTION  = 8.0;


  // Internal variables...
  // -------------------------------------------------------------

  protected SimpleUniverse universe;
  // protected BUniverse universe;

  // TGs for placing console geometry...
  protected TransformGroup  consoleTranslateGroup;
  protected TransformGroup  consoleScaleGroup;
  private TransformGroup consolePosition;
  private BranchGroup universeNode;
  private ViewingPlatform vp;


  protected OrderedGroup orderedGroup;

  protected BConsoleRootContainer root;
    // Contains all console components.


  // Clipping distances for canvas.
  protected double frontClip;
  protected double backClip;
  protected double fov;

  protected double canvasWidth;
  protected double canvasHeight;
  protected double consoleWidth ;
  protected double consoleHeight;
  protected double consoleZ;

  // List of renderable objects; rendered during renderField().
  protected ArrayList renderables = new ArrayList(); 

  // Support class for handling activation listeners...
  protected BActivationSupport activationSupport
    = new BActivationSupport(this);



  // Constructor.
  // -------------------------------------------------------------

  public BConsoleCanvas
  (
    GraphicsConfiguration gc,
    double frontClipDistance,
    double backClipDistance,
    double fov
  )
  {
    super(gc);

    frontClip          = frontClipDistance;
    backClip           = backClipDistance;
    this.fov           = fov;

    consoleZ           = frontClip * 9.0;

    vp = new ViewingPlatform();
    // set capability for the 3d sound manager to attenutae sounds relative to the view platform
    //vp.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
    vp.getViewPlatformTransform().setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
    vp.getViewPlatform().setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);

    Viewer v           = new Viewer(this);

    View view          = v.getView();
    view.setBackClipDistance(backClip);
    view.setFrontClipDistance(frontClip);
    view.setFieldOfView(fov);

    //universe = new BUniverse(vp, v);
     universe = new SimpleUniverse(vp, v);

    initConsole();
  }


  public void calculateConsoleSize()
  {
    calculateConsoleSize(fov);
  }


  private Transform3D setConsoleVisiblet = new Transform3D();
  private Vector3d setConsoleVisibleV = new Vector3d();

  public void setConsoleVisible(boolean b)
  {
   if (b)
     calculateConsoleSize();
   else
   {

      setConsoleVisibleV.set(0,0,0);
      setConsoleVisiblet.set(setConsoleVisibleV);
      consoleTranslateGroup.setTransform(setConsoleVisiblet);
    }
  }


  // Initialization helper methods...
  // -------------------------------------------------------------

  private Transform3D calculateConsoleSizet = new Transform3D();
  private Transform3D calculateConsoleSizes = new Transform3D();
  private Vector3d calculateConsoleSizeV = new Vector3d();

  protected void calculateConsoleSize(double newFov)
  {
    consoleWidth  = (java.lang.Math.tan (newFov / 2.0) * consoleZ) ;
    consoleWidth  = consoleWidth * 2.0;
    consoleHeight = (canvasHeight / canvasWidth) * consoleWidth;

    // Calculate console translation vector...
    double   x = (java.lang.Math.tan (newFov / 2.0) * consoleZ);
    double   y = (canvasHeight / canvasWidth) * x;
    calculateConsoleSizeV.set(-x, y, -consoleZ);

    double scale = consoleWidth / canvasWidth;


    calculateConsoleSizet.set(calculateConsoleSizeV);
    calculateConsoleSizes.setScale(scale);

    consoleTranslateGroup.setTransform(calculateConsoleSizet);
    consoleScaleGroup.setTransform(calculateConsoleSizes);

  }


  protected void addToUniverse(BranchGroup b)
  {
    universeNode.addChild(b);
  }

  protected void initConsole()
  {
    universeNode = new BranchGroup();
    universeNode.setCapability(BranchGroup.ALLOW_DETACH);
    universeNode.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);

    consoleTranslateGroup = new TransformGroup();
    consoleTranslateGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

    consoleScaleGroup     = new TransformGroup();
    consoleScaleGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    //consoleScaleGroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
    //consoleScaleGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
    //consoleScaleGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);



    orderedGroup = new OrderedGroup();
    // orderedGroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
    // orderedGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
    // orderedGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);

    consolePosition = new TransformGroup();
    consolePosition.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

    consoleScaleGroup.addChild(orderedGroup);
    consoleTranslateGroup.addChild(consoleScaleGroup);

    consolePosition.addChild(consoleTranslateGroup);
    universeNode.addChild(consolePosition);


    // Set up the console root container...
    root = new BConsoleRootContainer(this);
    root.setLocation(0, 0);

    orderedGroup.addChild(root);


    universe.addBranchGraph(universeNode);

    //consoleTranslateGroup.addChild(consoleScaleGroup);

    //universe.getViewingPlatform().setPlatformGeometry(pg);

  /*
    PlatformGeometry pg = new PlatformGeometry();

    consoleTranslateGroup = new TransformGroup();
    consoleTranslateGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);

    consoleScaleGroup     = new TransformGroup();
    consoleScaleGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
    //consoleScaleGroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
    //consoleScaleGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
    //consoleScaleGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);



    orderedGroup = new OrderedGroup();
    orderedGroup.setCapability(TransformGroup.ALLOW_CHILDREN_EXTEND);
    orderedGroup.setCapability(TransformGroup.ALLOW_CHILDREN_READ);
    orderedGroup.setCapability(TransformGroup.ALLOW_CHILDREN_WRITE);

    consoleScaleGroup.addChild(orderedGroup);
    pg.addChild(consoleTranslateGroup);
    consoleTranslateGroup.addChild(consoleScaleGroup);

    universe.getViewingPlatform().setPlatformGeometry(pg);       */
  }



  // Accessor methods...
  // -------------------------------------------------------------

  public void setSize(int w, int h)
  {
    super.setSize(w, h);

    // Update the root container...
    root.setSize(w, h);

    canvasWidth  = (double) w;
    canvasHeight = (double) h;

    calculateConsoleSize(fov);
  }

  public double getScreenToConsoleX(double x)
    { return x; }

  public double getScreenToConsoleY(double y)
  {
    double centreY = canvasHeight / 2;
    double diff = centreY - y;
    double diffRatio = diff / centreY;

    return -y + (Y_PERSPECTIVE_CORRECTION * diffRatio);
  }


  // Console component methods...
  // -------------------------------------------------------------

  public void add(BConsoleComponent c)
    { root.add(c); }

  public void remove(BConsoleComponent c)
    { root.remove(c); }

  public void removeAll()
    { root.removeAll(); }

  public void clear()
    { removeAll(); }



  // YLY -- Made it static with cleanse function because j3d was not releasing the node.

  protected static class BConsoleRootContainer extends BConsoleContainer
  {
    protected BConsoleCanvas parentCanvas;

    public BConsoleRootContainer(BConsoleCanvas parentCanvas)
      { this.parentCanvas = parentCanvas; }

    protected void transformChildLocationVector(BConsoleComponent child, Vector3d v)
      { v.y = parentCanvas.getScreenToConsoleY(v.y); }

    public void cleanse()
    {
      super.cleanse();
      parentCanvas = null;

    }
  }




  //private boolean setFovOnPostSwap = false;

  public void setFov(double fov)
  {

    this.fov = fov;
    //calculateConsoleSize();
    getView().setFieldOfView(fov);


    /*
      if (!setFovOnPostSwap)
      {
        this.fov = fov;
        setFovOnPostSwap = true;
      }   */
  }

  public double getFov()
  {
    return fov ;
  }


  // Activation methods...
  // -------------------------------------------------------------

  public boolean isActive()
    { return activationSupport.isActive(); }

  public boolean isClosed()
    { return activationSupport.isClosed(); }


  public void requestActivate()
    { activationSupport.requestActivate(); }

  public void requestDeactivate()
    { activationSupport.requestDeactivate(); }

  public void requestClose()
    { activationSupport.requestClose(); }


  public void onActivate()
    { activationSupport.fireActivate(); }

  public void onDeactivate()
    { activationSupport.fireDeactivate(); }

  public void onClose()
  {
    //BDebug.print("BConsoleCanvas.onClose Called");
    activationSupport.fireClose();

    //universe.cleanse();
    if (root != null)
      root.cleanse();
    root = null;

    renderables.clear();
    renderables = null;

    // SJS [05/04/2001] This line was crashing J3D.
    // universe.getLocale().removeBranchGraph(universeNode) ;

    stopRenderer();
    universe.getViewer().getView().stopView();
    universe.getViewer().getView().removeCanvas3D(this);

    // Clean up all J3D threads...
    universe.getLocale().removeBranchGraph(universeNode);
    universeNode = null;
    universe.removeAllLocales();
    universe.getViewer().setViewingPlatform(null);
    universe = null;


    /*
    universe.getViewer().setViewingPlatform(null);
    //universe.getViewer().getView().stopView();
    universe.getViewer().getView().removeCanvas3D(this);

    // YLY, Try removing after rendere has stopped
    universe.getLocale().removeBranchGraph(universeNode) ;
    universe.removeAllLocales();

    universe = null; */

    consoleTranslateGroup = null;
    consoleScaleGroup = null;
    orderedGroup = null;
    activationSupport = null;

  }


  public void addActivationListener(BActivationListener l)
    { activationSupport.addActivationListener(l); }

  public void removeActivationListener(BActivationListener l)
    { activationSupport.addActivationListener(l); }


  Transform3D consoleT = new Transform3D();

  /*public void updateConsole(Transform3D tranC)
  {
    consolePosition.setTransform(tranC);
  }   */


  public synchronized void updateView(Transform3D tranC)
  {
    vp.getViewPlatformTransform().setTransform(tranC);
    consolePosition.setTransform(tranC);
  }



  // BRenderer implementation...
  // -------------------------------------------------------------

  public void addRenderable(BRenderable r)
    { renderables.add(r); }

  public boolean removeRenderable(BRenderable r)
    { return renderables.remove(r); }

  public void getCameraTransform(Transform3D t)
    { vp.getViewPlatformTransform().getTransform(t); }



  // RenderField() override (for BRenderables)...
  // -------------------------------------------------------------

  // SJS [06/04/2001] - If BRenderable is ever used,
  // will need to reinstate this function.
  
  /*
  public void renderField(int description)
  {
    super.renderField(description);

    // If we've closed the canvas, abort.
    if (renderables == null) return;

    // Obtain a graphics context...
    GraphicsContext3D g = getGraphicsContext3D();

    // Remember camera transform...
    // getCameraTransform(camera);

    // Set model transform to camera...
    // g.setModelTransform(camera);

    // Render all renderables...
    BRenderable r;

    for (int i=renderables.size()-1; i>=0; i--)
    {
      r = (BRenderable) renderables.get(i);
      r.render(g);
    }
  }
  */

}
