package roboforge;

import java.util.*;
import java.io.*;


import roboforge.ai.worldstate.*;
import roboforge.ai.values.*;
import roboforge.botanalyst.*;

public class BMoveList extends BMonitoredList
  implements Serializable
{
/*

* Description:
  A Collection
  which holds BMoves.
  Should really be called BMoveRepertory
  but the simple majority prefer BMoveList.

* Example:
* Modification History ( Name, Date, Notes ):
  HSC, 4-April-1999
  SJS [03/03/2000]: Added convenience methods for accessing moves by name.

*/



  static final long serialVersionUID = -1L;

  // private Vector list;

  private BMove startMove;
  private BMove victoryMove;

  private transient boolean anyMoveCompleted = false;
  //private transient boolean anyMoveActive = false;

  private transient BWorldState lastWorldState;


  // Chassis Move Type Constants.
  public static final int CHASSIS_STAND = 0;
  public static final int CHASSIS_MOVE_FORWARD = 1;
  public static final int CHASSIS_MOVE_BACKWARDS = 2;
  public static final int CHASSIS_MOVE_LEFT = 3;
  public static final int CHASSIS_MOVE_RIGHT = 4;
  public static final int CHASSIS_ROTATE_LEFT = 5;
  public static final int CHASSIS_ROTATE_RIGHT = 6;
  public static final int CHASSIS_RESULTANT_MOVE = 7;

  public static String stateCode = "moves";


  private boolean noSideWaysMovement = false;

  // SJS [13/03/2000]: Constants for chassis move units...
  public static final String METERS       = "meters";
  public static final String DEGREES      = "degrees";
  public static final String METER        = "meter";
  public static final String METER_SHORT  = "m";
  public static final String DEGREE       = "degree";
  public static final String DEGREE_SHORT = "d";
  public static final String UNITS        = "units";


  public  BMoveList()
    { super(); }

  public void init()
  {

    for (int i = list.size() - 1; i >=0; i--)
      ((BMove)list.get(i)).init(this);
    // SJS [29/05/2000]
    //   - Conversion code for old moves -> new.
    //   - TODO: Remove!

    /*if (moveCollection != null)
    {
      BDebug.print("BMoveList.init(): Converting move-list representation...");
      list = new Vector();
      list.addAll(moveCollection);
      moveCollection = null;
    }   */
  }





  /*
  public void add(BMove move)
    { super.add(move); }

    int i = list.size();

    list.add(move);
    fireIntervalAdded(i, i);
  }

  public int size()
    { return list.size(); }

  public int indexOf(BMove move)
    { return list.indexOf(move); }
  */

  // THIS SHOULD ONLY BE USED IN THE UI
  /*public int indexOf_ForUI(BMove move)
  {
    int i = list.indexOf(move);

    if (noSideWaysMovement && ((i== CHASSIS_ROTATE_LEFT ) ||  (i== CHASSIS_ROTATE_RIGHT )  )  )
      i = i-2;

    return  i;
  } */

  public int getChassisMoveType(BMove move)
    { return list.indexOf(move); }


  public void removeThumbnails()
  {
    BMove current;
    for (int i=0; i<list.size(); i++)
    {
      current = (BMove) list.get(i);
      current.removeThumbnails();
    }

  }

  public void cleanse()
  {
   // BDebug.print ("BMoveList.cleanse called ");
    
    lastWorldState = null;

    BMove current;
    for (int i=0; i<list.size(); i++)
    {
      current = (BMove) list.get(i);
      current.cleanse();
    }



    super.cleanse();

  }

  public void cleanseLiveNodes()
  {
    //BDebug.print ("BMoveList: CleanseLiveNodes");
    
    lastWorldState = null;

    BMove current;
    for (int i=0; i<list.size(); i++)
    {
      current = (BMove) list.get(i);
      current.cleanseLiveNodes();
    }


  }


  public void validate(Vector warnings, boolean onlyFatal)
  {
    BMove current;

    if (!onlyFatal)
      for (int i=0; i<list.size(); i++)
      {
        current = (BMove) list.get(i);

        if (current.getSnapshotCount() == 1) // 1 means that only the default snapshot is there
           warnings.add(new BBotWarning( "Move: " + current.getName() + " does not contain snapshots." ,
                                    "The Bot will do nothing if it tries to perform this move") );
      }


  }

  public void cleanseJointsNotInBot(BBot theBot)
  {
    BMove current;

    //BDebug.print (" BMoveList.cleanseJointsNotInBot started ");


    for (int i=0; i<list.size(); i++)
    {
      current = (BMove) list.get(i);
      current.cleanseJointsNotInBot(theBot);
    }

    //BDebug.print (" BMoveList.cleanseJointsNotInBot finished ");
  }


  /*
  public BMove getMove(String moveName)
  {
    BMove current;

    for (int i=0; i<size(); i++)
    {
      current = (BMove) list.get(i);
      if (moveName.equals(current.getName()))
        return current;
    }

    return null;
  }
  */


  public BMove getStartMove()
    { return startMove; }

  public BMove getFirstMove()
  {
    if (size() > 0)
      return (BMove) list.get(0);
    else
      return null;
  }

  public void setStartMove (BMove startMoveIn)
  {
    BDebug.print ("BMoveList.setStartMove " + startMoveIn);
    startMove = startMoveIn;
  }

  public BMove getVictoryMove()
    { return victoryMove; }

  public void setVictoryMove (BMove victoryMoveIn )
    { victoryMove = victoryMoveIn; }

  public boolean getCanMoveSideWays()
  { return !noSideWaysMovement; }

  public void setCanMoveSideWays( boolean b)
  { noSideWaysMovement = !b; }


  public BMove getSelectedMove()
    { return (BMove) selected; }


  public BMove getMove(String name)
  {
    for (int i = list.size() -1 ; i >=0; i-- )
    {
      BMove m = (BMove)list.get(i);

      if ( m.getName().equals(name)  )
        return m;
    }

    return null;
  }

  public BMove getMove(int i)
  {
    if (noSideWaysMovement&& ( (i== CHASSIS_MOVE_LEFT ) ||  (i== CHASSIS_MOVE_RIGHT )  )  )
      i = i + 2;

    return (BMove) list.get(i);
  }

  public Vector getMoves()
    { return list; }


  public void setMoves(Vector movesIn)
  {
    int n = list.size();

    list = movesIn;
    fireContentsChanged(0, n-1);
  }


  public void addJoint(BJoint bJoint)
  {
    for (int i=0; i < list.size(); i++ )
    {
      Vector snapshots = ((BMove) list.get(i)).getSnapshots();

      if (snapshots != null)
        for (int j=0; j < snapshots.size(); j++ )
        {
          BSnapshot snapshot = (BSnapshot) snapshots.get(j);
          snapshot.add(new BComponentTransform((BJoint) bJoint));
        }
    }
  }

  public void removeJoint(BJoint bJoint)
  {
    // Loop through all the Moves
    ListIterator itMoves = list.listIterator();
    while (itMoves.hasNext())
    {
      Vector snapshotVector = ((BMove)itMoves.next()).getSnapshots();
      ListIterator itSnapshots = snapshotVector.listIterator();

      // Loop through all the Snapshots
      while (itSnapshots.hasNext())
      {
        Vector componentTransformsVector = ((BSnapshot)itSnapshots.next()).getComponentTransforms();
        ListIterator itComponentTransforms = componentTransformsVector.listIterator();

        // Loop through all the ComponentTransforms
        boolean jointNotFound = true;
        while (itComponentTransforms.hasNext() && jointNotFound)
        {
          BComponentTransform bComponentTransform = (BComponentTransform)itComponentTransforms.next();
          if (bComponentTransform.getJoint() == bJoint)
          {
            // Delete the component
            itComponentTransforms.remove();
            jointNotFound = false;
          }
        }
      }
    }
  }


public void preProcessMoves(BChassis newChassis)
{
  // Created 24-June-1999 HSC

  ListIterator itMoves = list.listIterator();
  BMove current;

  while (itMoves.hasNext())
  {
    current = (BMove) itMoves.next();
    current.initForSim(newChassis);
  }
}





  public void remove(Object o)
  {
    super.remove(o);

    char dot = BWorldState.DELIMITER;

    if (lastWorldState != null)
      lastWorldState.remove(BBot.stateCode+ dot+ stateCode+ dot + o.toString() );

    if ( o != null)
    {
      if ( o == startMove )
        startMove = null;

      if (o == victoryMove)
        victoryMove = null;

      BMove m = (BMove) o;
      m.setParentList(null);
      //m.setName(m.getName() + " (deleted)");

    }

  }

  public void moveNameChanged(BMove m, String oldName)
  {
    char dot = BWorldState.DELIMITER;

    if (lastWorldState != null)
    {
      lastWorldState.remove(BBot.stateCode+ dot+ stateCode+ dot + oldName );
      m.addWorldItems(lastWorldState);
    }
  }

   public void add(Object o)
  {

    super.add(o);

    if (lastWorldState != null)
    {
      BMove m = (BMove) o;
      m.init(this);
      m.addWorldItems(lastWorldState);

    }

  }

  public void addWorldItems(BWorldState root)
  // TODO
  // XXXX TO BE MOVED TO  BMOveList When Steve Checks it In
  {
    lastWorldState = root;

    Vector moves = getMoves();

    char dot = BWorldState.DELIMITER;

    for (int i = 0; i < moves.size(); i++ )
    {
      BMove m = (BMove) moves.get(i);
      m.addWorldItems(root);
    }


    // anyMove Complete
    String key = BBot.stateCode+ dot+ stateCode ;

    BMoveBooleanWorldItem anyComplete = new BMoveBooleanWorldItem(key + dot+ "hasCompleted", "Any completed", new BBoolean() , "One of my Bots moves completed this AI cycle", null)
      {
        public BValue evaluate()
        {
            setValue( getAnyMoveCompleted() );
            return getValue();
        }
      };

    root.put( anyComplete);

    BMoveBooleanWorldItem anyActive = new BMoveBooleanWorldItem(key + dot+ "anyActive", "Any active", new BBoolean() ,"'yes' of any of my bots moves are currently executing", null)
      {
        public BValue evaluate()
        {
            setValue( getAnyMoveActive() );
            return getValue();
        }
    };

    root.put( anyActive);


  }

  public void setAnyMoveCompleted(boolean b)
  {  anyMoveCompleted = b ; }

  public boolean getAnyMoveCompleted()
  {
    boolean wasComplete = anyMoveCompleted;
    anyMoveCompleted = false;
    return wasComplete;
  }

  private transient int activeMoves = 0;

  public void setAnyMoveActive( boolean b)
  {
    if (b)
      activeMoves++;
    else
      activeMoves-- ;

  }

  public boolean getAnyMoveActive()
  {
    return activeMoves > 0;
  }

  public void replaceBot (BBot bot)
  {
    BMove m;
    for (int i = list.size() - 1; i >=0; i--)
    {
      m = (BMove) list.get(i);
      m.replaceBot(bot);

    }
  }

  public void upgrade(BMoveList other, BBot bot)
  {
    BMove m;
    for (int i = list.size() - 1; i >=0; i--)
    {
      m = (BMove) list.get(i);
      m.upgrade( (BMove) other.list.get(i), bot );

    }
  }


  public String getChassisMoveUnits(BMove move, boolean abbreviated)
  {
    int moveType = getChassisMoveType(move);

    String result = "";

    switch (moveType)
    {
      case BMoveList.CHASSIS_MOVE_FORWARD:
      case BMoveList.CHASSIS_MOVE_BACKWARDS:
      case BMoveList.CHASSIS_MOVE_LEFT:
      case BMoveList.CHASSIS_MOVE_RIGHT:
        if (abbreviated)
          result = METER_SHORT;
        else
          result = METERS;
        break;

      case BMoveList.CHASSIS_ROTATE_LEFT:
      case BMoveList.CHASSIS_ROTATE_RIGHT:
        if (abbreviated)
          result = DEGREE_SHORT;
        else
          result = DEGREES;
        break;
    }

    return result;

  }





}///:~