package roboforge;

import com.sun.j3d.utils.geometry.Cone;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.geometry.*;
import java.util.*;
//import javax.swing.tree.*;

import javax.media.j3d.*;
import javax.vecmath.*;
//import com.sun.j3d.loaders.Scene;
import java.awt.Color;

public class BSensor extends BComponent  implements Comparable
/*

* Description:
  The Sensors of the Bot

* Example:
* Modification History ( Name, Date, Notes ):
  HSC, 25-March-1999
  YLY, 26-May -99 - Implemented Range Display

  */

{
  static final long serialVersionUID = -1L;

  public int sensorRange;          // Sensor detection range (Metres)
  public int sensorScope;          // in Degrees from centre/normal
                                   // Anything above 90 is considered a Sphere Sensor
  public int sensorAccuracy;       // Percentage Accuracy (100% always correct, 0% never correct)
  public boolean glow;             // If this is set, a light is placed in the sensor

  private static float SENSOR_TRANSPARENCY =0.8f;
  // transient Graphic fields

  private transient BranchGroup rangeShapeGroup = null;
  private transient Primitive activePrimitive = null;

  transient private  Appearance sensorRangeAppearance = null;
  transient private  ColoringAttributes sensorRangecAtrib = null;
  transient private SpotLight sensorLight;
  //transient private boolean addedGlow = false;

  private transient boolean immobile;  // true if the sensor cannot move relative to the chassis

  public static String COMPONENT_TYPE = "Sensor";

  //private TexCoordGeneration TEXT_GEN = new TexCoordGeneration(TexCoordGeneration.SPHERE_MAP, TexCoordGeneration.TEXTURE_COORDINATE_2  ) ;
  //private TransparencyAttributes RANGE_TRANSPARENCY_ATTRIBUTES = new TransparencyAttributes(TransparencyAttributes.BLENDED ,
  //                                                                            SENSOR_TRANSPARENCY,
  //                                                                            TransparencyAttributes.BLEND_SRC_ALPHA,
  //                                                                            TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA);
  //private PolygonAttributes RANGE_POLYGON_ATTRIBUTES = new PolygonAttributes(PolygonAttributes.POLYGON_FILL, PolygonAttributes.CULL_NONE, 0.0f        );


  public String getComponentType()
  { return COMPONENT_TYPE; }




  public BSensor (String nameIn, String filename)
  {
      super(nameIn, filename);
  }

 

  public Shape3D getSensorDetectShape()

  {

    if (activePrimitive == null)
      BDebug.print (" BSensor.getSensorDetectShape() IS NULL in component " + toString());
    else if (activePrimitive instanceof Cone )
      return activePrimitive.getShape(Cone.BODY );
    else if (activePrimitive instanceof Sphere )
      return activePrimitive.getShape( Sphere.BODY);
    else
      BDebug.print (" BSensor.getSensorDetectShape() assertion error :: active primitive NOT a sphere or Cone in component :: " + toString());

    return null;
  }


  private void addLight(BranchGroup addGroup)
  {
      //BDebug.print("BSensor.addLight :: " +  BGlobal.SHOW_SENSORLIGHTS );

    //  if (BGlobal.SHOW_SENSORLIGHTS)
    //  {

       Color3f lightC =new Color3f((float)componentColor.getRed() / 256,(float)componentColor.getGreen() / 256,(float)componentColor.getBlue() / 256);

       sensorLight = new SpotLight(     lightC,
                                        new Point3f(0,0,0),  // position
                                        new Point3f(0,0.03f,0),  // Attenuation (constant, linear, quadratic)
                                        new Vector3f(0, 0, 1), // DIRECTION
                                        sensorScope * BPhysics.RAD_CONVERT ,
                                        0.0f);


       sensorLight.setInfluencingBounds(new BoundingSphere( new Point3d(0,0,0),sensorRange * BGlobal.WORLD_TO_CANVAS_SCALE));
       sensorLight.setCapability(PointLight.ALLOW_COLOR_WRITE);

       addGroup.addChild(sensorLight);
    //  }

 }



 public BranchGroup cloneStructure3D( boolean cloneAdditions, boolean forLive, boolean cloneAppearance)
 {
     BranchGroup tempB = super.cloneStructure3D(cloneAdditions,forLive, cloneAppearance);

    if (glow )
    {
       addLight( tempB);
    }

    return tempB;

 }


  public BranchGroup cloneStructure3D()
  {

    BranchGroup tempB = super.cloneStructure3D();

    if (glow )
    {
       addLight( tempB);
    }

    return tempB;
 }


 public void removeRange()
 {

       //BDebug.print( "Sensor Range Removed");

       if (rangeShapeGroup != null)
          rangeShapeGroup.detach();



 }

  public void cleanseLiveNodes()
  {


     super.cleanseLiveNodes();
     removeRange();

     if (activePrimitive != null)
        if (!activePrimitive.isLive() )
          activePrimitive.setAppearance(null);


     activePrimitive = null;
     rangeShapeGroup = null;
     sensorRangecAtrib = null;


  }


 public void addRangeDisplay()
 {
  addRangeDisplay(liveBotNode);


 }

 public boolean detects (BZone theZone)
 {
    //BDebug.print ("BSensor detect " + this + " " + theZone );
    return  detects (theZone.getShape(), false);;

 }

 // used for sorting sensors to be the best sensor to test first
 public int compareTo(Object o)
 {
    int result = 0;

    if (o instanceof  BSensor)
    {
      BSensor other = (BSensor) o;

      if ( this.sensorScope > other.sensorScope )
        result = -1;
      else if ( this.sensorScope < other.sensorScope )
        result = 1;
      else
      {
        if (this.sensorRange > other.sensorRange )
          result = -1;
        else  
          result = 1;


      }


    }

   //BDebug.print (this + "compareTo" + o + " result " +result);


    return result;
 }


 private boolean detects(BComponent triggerComponent )
 {
  //BDebug.print ("BSensor detects called for " + triggerComponent );

  if (triggerComponent.getDestroyed() )
    return false;
  else
    return detects (triggerComponent.getLiveNode(), true );
 }


 private static Transform3D detects_sensorTransform = new Transform3D();
 private static Vector3f detects_sensorDisplacement = new Vector3f();
 private static Transform3D detects_triggerTransform = new Transform3D();
 private static Vector3f detects_triggerDisplacement = new Vector3f();
 private static Vector3f detects_resultant= new Vector3f();
 private static Vector3f zAxis= new Vector3f(0,0,1f);

 private boolean detects(Node centreOfArenaObject, boolean fuzzydetect)
 {
  boolean detected = false;

  if (!getDestroyed() )
  {

    // get sensor displacement
    //Transform3D sensorTransform =  new Transform3D();
    // sensorDisplacement = new Vector3f();
    getLiveNode().getLocalToVworld(detects_sensorTransform);
    detects_sensorTransform.get(detects_sensorDisplacement);
   // BDebug.print ( "BSensor.detect sensorDisplacement " +  detects_sensorDisplacement);

    // get trigger displacement
    centreOfArenaObject.getLocalToVworld(detects_triggerTransform);
    detects_triggerTransform.get(detects_triggerDisplacement);
    //BDebug.print ( " BSensor.detect triggerDisplacement " +  detects_triggerDisplacement);


    detects_resultant.sub(detects_triggerDisplacement, detects_sensorDisplacement);
    //BDebug.print ( " BSensor.detect resultant vector " +  detects_resultant);

    zAxis.set(0,0,1f);
    detects_sensorTransform.transform(zAxis);
   // BDebug.print ( " BSensor.detect resultant zAxis " +  zAxis);


    float deltaAngle = zAxis.angle(detects_resultant);
    //BDebug.print ( "BSensor.detect Angle " +  deltaAngle);
    boolean inScope;

    float distance =   detects_resultant.length()   * BGlobal.CANVAS_TO_WORLD_SCALE ;
    float nearFudgePercent  =  (sensorRange - distance) / sensorRange;

   // BDebug.print ( "BSensor.nearFudgePercent " +  nearFudgePercent);
    if (!fuzzydetect)
      nearFudgePercent = 0;
    else if (distance > sensorRange / 2)
      nearFudgePercent = nearFudgePercent /2;

    deltaAngle = deltaAngle - ( BGlobal.SENSOR_ERRORMARGIN * nearFudgePercent);


    if (sensorScope < 90)
    {
      float rad = sensorScope * BPhysics.RAD_CONVERT ;
      //BDebug.print ( "BSensor.detect rad " +  rad);

      inScope = (deltaAngle <= rad);


    }
    else
      inScope = true;

    if (inScope)
    {

      if( distance <  sensorRange )
        detected = true;
    }

  } // if destroyed

  return detected;


 }


 public boolean detects (BArenaObject triggerObject)
 {

  boolean detected = false;

  //BDebug.print("BSensor.detect " + this);

  if(!getDestroyed())
  {

    if (triggerObject instanceof BBot )
    {

      BComponent[] botComponents;

      if (BGlobal.getCollisionCode() == BGlobal.COLLISION_BOUNDS)
        botComponents =  ((BBot)triggerObject).getChassisAndLeaves();
      else
        botComponents =  ((BBot)triggerObject).getBreathComponents();


      for (int i =  botComponents.length- 1;  i >= 0 ; i--)
      {
        detected = detects(  botComponents[i]);

        // break loop for fastest processing
        if (detected )
                return true;
      }




  }  // instance of bbot

  else
    detected = detects ( (BComponent)triggerObject );


  }  // !destroyed



  return detected;

  }

 private void addRangeDisplay(Group addGroup)
 {

   BranchGroup tempB = new BranchGroup();
   tempB.setCapability(BranchGroup.ALLOW_DETACH);

   if (liveBotNode == null)
     BDebug.print("BSensor.displayRange ERROR : liveBotNode == null ");
   else
   {

       if (sensorRangeAppearance == null)
        initSensorRangeAppearance();

        if  (rangeShapeGroup != null)
        {
          addGroup.addChild(rangeShapeGroup);
          return;

        }
        else if (sensorScope == 90 )
          BDebug.print("BSensor.displayRange :: 90 degree angle not implemented, choose angle " );
        else if (sensorScope < 90)
        {
          float coneAngle;
          coneAngle = sensorScope;

          // orthogonal projection of the cone gives a triangle that can be split into 2 right angle triangles.
          // The cone radius is the Opposite side to the angle
          float radius = ((float)sensorRange * BGlobal.WORLD_TO_CANVAS_SCALE) * (float)Math.tan(coneAngle * BPhysics.RAD_CONVERT );

          float coneHeight = ((float)sensorRange * BGlobal.WORLD_TO_CANVAS_SCALE);// * (float)Math.cos(coneAngle * BPhysics.RAD_CONVERT );

          //Cone tempC = new Cone(radius  , ((float)sensorRange * BGlobal.WORLD_TO_CANVAS_SCALE));
          // | Cone.GENERATE_TEXTURE_COORDS
          Cone tempC = new Cone(radius  , coneHeight,Cone.GENERATE_NORMALS | Cone.GENERATE_TEXTURE_COORDS, sensorRangeAppearance  );

          //sensorRangeAppearance.setTexCoordGeneration(TEXT_GEN);
          //tempC.setAppearance(sensorRangeAppearance);
          activePrimitive = tempC;
          if ( BGlobal.USE_AUTOGEN )
            sensorRangeAppearance.setTexture(BTextures.getTexture(BGlobal.TEXTUREPATH + BGlobal.SENSORTEXTURE));

         // activePrimitive.getShape(Cone.BODY ).setAppearance(sensorRangeAppearance);


           if (!((GeometryArray)activePrimitive.getShape(Cone.BODY ).getGeometry()).isLive())
          {
            ((GeometryArray)activePrimitive.getShape(Cone.BODY ).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);
            ((GeometryArray)activePrimitive.getShape(Cone.BODY ).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);
           }

          // set pickable to false
          tempC.setPickable(false);

          // Align the cone to the Z axis
          // Liquid Edge standard to have all sensors facing the Z axis
          Transform3D alignToZ = new Transform3D();
          alignToZ.rotX(-java.lang.Math.PI / 2.0f);

          // transpose in the y axis because we are combining the transpose with the rotate
          // so this y transpose will actually result in a z transpose
          Transform3D moveCentre = new Transform3D();
          moveCentre.set(new Vector3f(0.0f,-((float)sensorRange * BGlobal.WORLD_TO_CANVAS_SCALE) / 2.0f,0.0f)  );
          alignToZ.mul(alignToZ, moveCentre);


          TransformGroup alignG = new TransformGroup (alignToZ);
          alignG.addChild(tempC);
          tempB.addChild(alignG);






        }
        else  // if the angle is larger that 90, use a sphere
        {

          Sphere rangeSphere = new Sphere(((float)sensorRange * BGlobal.WORLD_TO_CANVAS_SCALE),Cone.GENERATE_NORMALS | Cone.GENERATE_TEXTURE_COORDS,  sensorRangeAppearance);

          if ( BGlobal.USE_AUTOGEN )
            sensorRangeAppearance.setTexture(BTextures.getTexture(BGlobal.TEXTUREPATH + BGlobal.SENSORTEXTURE));

          rangeSphere.setPickable(false);
          tempB.addChild(rangeSphere);
          activePrimitive =  rangeSphere;

          //activePrimitive.getShape(Sphere.BODY ).setCapability(Node.ALLOW_BOUNDS_READ);
          //activePrimitive.getShape(Sphere.BODY ).setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
          //activePrimitive.getShape(Sphere.BODY ).setCapability(Shape3D.ALLOW_GEOMETRY_READ);

          if (!((GeometryArray)activePrimitive.getShape(Sphere.BODY ).getGeometry()).isLive())
          {
            ((GeometryArray)activePrimitive.getShape(Sphere.BODY ).getGeometry()).setCapability(GeometryArray.ALLOW_COUNT_READ);
            ((GeometryArray)activePrimitive.getShape(Sphere.BODY ).getGeometry()).setCapability(GeometryArray.ALLOW_COORDINATE_READ);
           }

        }


        rangeShapeGroup = tempB;

        addGroup.addChild(rangeShapeGroup);









   }



 }


 public Object clone()
 {
        BSensor o = (BSensor) super.clone();
        o.rangeShapeGroup = null;
        o.activePrimitive = null;
        return o;


 }


// private static SENSOR_RANGE_COLOR3F  = BGlobal.colors.getDarkShade3f();


 private void initSensorRangeAppearance()
 {

    sensorRangeAppearance = new Appearance();

    sensorRangecAtrib = new ColoringAttributes();
    sensorRangecAtrib.setColor(BGlobal.SENSOR_RANGE_COLOR3F);
    sensorRangecAtrib.setCapability(ColoringAttributes.ALLOW_COLOR_WRITE);
    sensorRangecAtrib.setShadeModel(ColoringAttributes.SHADE_FLAT);
    sensorRangeAppearance.setColoringAttributes(sensorRangecAtrib);

    //componentMaterial = new Material();
    TextureAttributes newTexA = new TextureAttributes();
    newTexA.setTextureMode(TextureAttributes.MODULATE);

    TransparencyAttributes tranA =  new TransparencyAttributes(TransparencyAttributes.BLENDED , SENSOR_TRANSPARENCY, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA) ;

    TexCoordGeneration texGen = null;
    if ( BGlobal.USE_AUTOGEN )
      texGen= new TexCoordGeneration(TexCoordGeneration.SPHERE_MAP, TexCoordGeneration.TEXTURE_COORDINATE_2  ) ;
    //else
    //  texGen= new TexCoordGeneration(TexCoordGeneration.OBJECT_LINEAR, TexCoordGeneration.TEXTURE_COORDINATE_2  ) ;

    PolygonAttributes polyAtt = new PolygonAttributes(PolygonAttributes.POLYGON_FILL, PolygonAttributes.CULL_NONE, 0.0f        );


    sensorRangeAppearance.setTextureAttributes(newTexA);
    //sensorRangeAppearance.setMaterial(componentMaterial);
    sensorRangeAppearance.setTransparencyAttributes( tranA );
    if ( BGlobal.USE_AUTOGEN )
      sensorRangeAppearance.setTexCoordGeneration(texGen);
    sensorRangeAppearance.setPolygonAttributes(polyAtt);

    Material rangeMaterial = new Material();
    rangeMaterial.setLightingEnable(false);
    sensorRangeAppearance.setMaterial(rangeMaterial);

     // normalAppearance.setCapability(Appearance.ALLOW_TEXTURE_WRITE);


 /*
  // Appearance
        sensorRangeAppearance = new Appearance();
        TextureAttributes newTexRange = new TextureAttributes();
        newTexRange.setTextureMode(TextureAttributes.MODULATE);

        // Material
        Material rangeMaterial = new Material();
        rangeMaterial.setLightingEnable(false);
        rangeMaterial.setAmbientColor(BGlobal.colors.getDarkerShade3f());
        rangeMaterial.setDiffuseColor(BGlobal.colors.getDarkerShade3f());
        rangeMaterial.setSpecularColor(BGlobal.colors.getDarkerShade3f());

        //RenderingAttributes renderAttr = new RenderingAttributes();
        //renderAttr.setDepthBufferWriteEnable( false );

        // Colour
        if (sensorRangecAtrib == null)
        {
                sensorRangecAtrib = new ColoringAttributes();
                sensorRangecAtrib.setColor(BGlobal.SENSOR_RANGE_COLOR);
                sensorRangecAtrib.setShadeModel(ColoringAttributes.SHADE_FLAT);
        }

        // Setting Apperaces
        sensorRangeAppearance.setTransparencyAttributes( RANGE_TRANSPARENCY_ATTRIBUTES );
        //sensorRangeAppearance.setRenderingAttributes(renderAttr);
        sensorRangeAppearance.setColoringAttributes(sensorRangecAtrib);
        sensorRangeAppearance.setPolygonAttributes( RANGE_POLYGON_ATTRIBUTES);
        sensorRangeAppearance.setMaterial(rangeMaterial);
        sensorRangeAppearance.setTextureAttributes(newTexRange);
        sensorRangeAppearance.setTexCoordGeneration(TEXT_GEN);

        sensorRangeAppearance.setTexture(BTextures.getTexture(BGlobal.TEXTUREPATH + BGlobal.SENSORTEXTURE));
  */
 }

 /*
  private void initSensorRangeAppearance()
 {

  // Appearance
        sensorRangeAppearance = new Appearance();
        TextureAttributes newTexRange = new TextureAttributes();
        newTexRange.setTextureMode(TextureAttributes.MODULATE);

        // Material
        Material rangeMaterial = new Material();
        rangeMaterial.setLightingEnable(false);
        rangeMaterial.setAmbientColor(BGlobal.colors.getDarkerShade3f());
        rangeMaterial.setDiffuseColor(BGlobal.colors.getDarkerShade3f());
        rangeMaterial.setSpecularColor(BGlobal.colors.getDarkerShade3f());

        RenderingAttributes renderAttr = new RenderingAttributes();
        renderAttr.setDepthBufferWriteEnable( false );

        // Colour
        if (sensorRangecAtrib == null)
        {
                sensorRangecAtrib = new ColoringAttributes();
                sensorRangecAtrib.setColor(BGlobal.SENSOR_RANGE_COLOR);
                sensorRangecAtrib.setShadeModel(ColoringAttributes.SHADE_FLAT);
        }

        // Setting Apperaces
        sensorRangeAppearance.setTransparencyAttributes( RANGE_TRANSPARENCY_ATTRIBUTES );
        sensorRangeAppearance.setRenderingAttributes(renderAttr);
        sensorRangeAppearance.setTexture(BTextures.getTexture(BGlobal.TEXTUREPATH + BGlobal.SENSORTEXTURE));
        sensorRangeAppearance.setColoringAttributes(sensorRangecAtrib);
        sensorRangeAppearance.setPolygonAttributes( RANGE_POLYGON_ATTRIBUTES);
        sensorRangeAppearance.setMaterial(rangeMaterial);
        sensorRangeAppearance.setTexCoordGeneration(TEXT_GEN);
        sensorRangeAppearance.setTextureAttributes(newTexRange);



 }
*/


  public void setColor(Color theColor, float shine, int newDepth)
  {
    super.setColor(theColor, shine, newDepth);

    if (sensorLight != null )
    {
      sensorLight.setColor( new Color3f((float)theColor.getRed() / 256f,(float)theColor.getGreen() / 256f,(float)theColor.getBlue() / 256f) );

    }


  }


   public void upgrade()
   {

    BSensor upgradeTwin =  (BSensor)BComponent.loadFromFile( BGlobal.COMPONENTPATH+getComponentFilePath(), false );

    super.upgrade(upgradeTwin);

    sensorRange = upgradeTwin.sensorRange;
    sensorScope = upgradeTwin.sensorScope;
    sensorAccuracy = upgradeTwin.sensorAccuracy;

   }

 public Vector getAttributeStrings()
 // returns a Vector of tuples  (heading, value )
 // 2-d Vector
 {

  Vector theList = super.getAttributeStrings();


  Vector tempV;

  tempV = new Vector(2);
  tempV.add("range");
  tempV.add(sensorRange + " m");
  tempV.trimToSize();
  theList.add(tempV);

  tempV = new Vector(2);
  tempV.add("field of view");
  tempV.add(sensorScope * 2 + " degrees");
  tempV.trimToSize();
  theList.add(tempV);


  return theList;


  }

   public void initForArena(BChassis theChassis, BBot owner)
  {
    super.initForArena(theChassis, owner);
    immobile = !owner.notInConnectedWithoutJointsCache(theChassis, this);

    //BDebug.print (this + " BSensor.initForArena immobile " + immobile);

  }

  public boolean isImmobile()
  // PRE:: Only valid after initForArena
  // returns true if the sensor  cannot move relative to the chassis
    {  return immobile; }




}


