package roboforge.ai.actions;

import roboforge.*;
import roboforge.ai.orient.*;
import roboforge.ai.values.*;
import roboforge.ai.worldstate.*;

import java.util.Vector;
import java.io.*;



public class BOrientAction implements BAction, Serializable
{

  static final long serialVersionUID = -1L;

  BBot myBot;
  BOrient orient;
  BOrientSearchPattern search;
  float timeOut;

  //working variables
  transient float timeSpent;
  transient private BBoolean oppDet;
  transient private float timeSinceLastKnown;

  public BOrientAction(BBot bot)
    { this(bot, new BOrientDistanceFacing(5f, 0, 1f), new BSearch_UseLastPos(), 10); }

  public BOrientAction(BOrientAction node )
  {
    this(node.myBot, node.orient, node.search, node.timeOut);
  }

  public BOrientAction(BBot theBot, BOrient orient, BOrientSearchPattern search , float timeOut )
  {
    myBot = theBot;
    this.orient = orient;
    this.search = search;
    this.timeOut = timeOut;

  }

  public static Vector getSearchOptions( )
  {
    Vector tempV = new Vector();
    tempV.add( new BSearch_StopOrient() );
    tempV.add( new BSearch_Nothing() );
    tempV.add( new BSearch_RotateTowardsCenter() );
    tempV.add( new BSearch_UseLastPos() );

    return tempV;

  }

  public static Vector getOrientOptions()
  {
    Vector tempV = new Vector(5);
    tempV.add( new  BOrientDistanceFacing(5f, 0, 1f) );
    tempV.add( new BOrientDistanceOnly(5f,1f)  );
    tempV.add( new BOrientLocationOnly() );
    tempV.add( new BOrientFacingOnly(0f)     );
    tempV.add( new BOrientFull()          );
    
    return tempV;

  }

  public void start()
  {
    timeSpent = 0f;
    timeSinceLastKnown = 1000;
  }

  public String getName()
    { return orient.getName(); }

  public String toString()
    { return orient.getName(); }


  public boolean execute(float s)
  {
    timeSpent += s;
    if (timeSpent >= timeOut)
    {

      return false;
    }
    boolean executed = false;
    BWorldState state = myBot.getWorldState();

    BWorldItem oppDetectedItem = state.get( BSimulation.WSTATE_OPPDETECTED  );
    BBoolean oppDet = (BBoolean)oppDetectedItem.getValue();

    boolean botDetected = oppDet.getValue();
    boolean falseLocation = false;


    // Orient
    // first search if not found
    if ( !botDetected && !search.executeOrient() )
    {
      search.executeSearch(s, myBot);
      if (search.stopOrient() )
      {
        myBot.getChassis().terminateChassisMove(false);
        executed = false;
      }
      else
        executed = true;
    }
    else
    { 
      executed = !orient.executeOrient( myBot);
    }

   return executed;



  }

  public boolean isValid()
  {
    return true;
  }

  public void stop()
  {
    if (myBot != null)
      if (myBot.getChassis() != null)
        myBot.getChassis().terminateChassisMove(false);
  }

  public boolean conflictsWith(BAction a)
  {
    boolean c = false;

    if (a == this)
      c = true;
    if (a instanceof BOrientAction)
      c =  true;
    else if (a instanceof BMove)
    {
      BMove m = (BMove) a;
      if ( m.getContainsChassisMove() )
        c = true;
      else if ( m.getIsResultantMove() )
        if (m.containsChassisJoints() )
          c = true;

    }
    else if (a instanceof BChassisAction)
      c =  true;
    else if (a instanceof BBotFallingAction)
      c = true;

    return c;
  }

  public boolean overrides(BAction a)
  { return false;}

  public BOrientSearchPattern getSearch()
  { return search; }

  public void setSearch(BOrientSearchPattern s)
  { search = s; }

  public BOrient getOrient()
  { return orient; }

  public void setOrient(BOrient o)
  { orient = o; }

  public float getTimeout()
  { return timeOut; }

  public void setTimeout(float t)
  { this.timeOut = t; }
  
  public void setBot(BBot bot)
  { myBot = bot; }






}

