package roboforge.ai.decisions;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BBot;

import roboforge.ai.*;
import roboforge.ai.expressions.*;
import roboforge.ai.actions.*;
import roboforge.ai.memory.*;
import roboforge.ai.values.*;
import roboforge.ui.components.HSBImageIcon;
import roboforge.util.parser.*;

import javax.swing.Icon;

public class BAssignmentNode extends BLeafNode
{
  // Class variables...
  // -------------------------------------------------------------

  protected static Icon stringIcon;
  protected static Icon booleanIcon;
  protected static Icon numberIcon;
  protected static Icon selectedIcon;

  // Internal variables..
  // -------------------------------------------------------------

  protected BMemoryItem item;
  protected BExpression expr;
  protected boolean expressionValid = true;

  protected boolean usesSimpleAssignment = true;
  protected BValue simpleValue;

  private String nullifiedActorString;



  // Constructor.
  // -------------------------------------------------------------

  public BAssignmentNode()
  {
    super();
  }

  public BAssignmentNode(BMemoryItem item)
  {
    super();
    this.item = item;
  }

  public BAssignmentNode(BAssignmentNode node)
  {
    super(node);
    this.item = node.item;
    this.expr = node.expr;

    this.expressionValid = node.expressionValid;

    this.usesSimpleAssignment = node.usesSimpleAssignment;
    this.simpleValue = node.simpleValue;
    this.nullifiedActorString = node.nullifiedActorString;


  }

  public Object clone()
    { return new BAssignmentNode(this); }


  // Initialization method.
  // -------------------------------------------------------------

  public void init()
  {
    if (expr != null)
      expr.init();

    super.init();
  }


  // Accessor Methods...
  // -------------------------------------------------------------

  public void setMemoryItem(BMemoryItem item)
  {
    if (item != null)
    {
      this.item = item;
      updateVariables();
    }
  }

  private void updateVariables()
  {
    setDefaultName();

    // BDebug.print("BAssignmentNode updateVariables() this.item " + this.item );
    // BDebug.print("BAssignmentNode expr.item).getType() " + this.item.getType() );
    //BDebug.print("BAssignmentNode this.item.getType() " + this.item.getType() );


    if (getUsesSimpleAssingment() )
      expressionValid = true;
    else if (  (this.expr != null)  && (this.item != null)   )
      expressionValid = ( expr.evaluate().getType() == this.item.getType() );
    else
      expressionValid = false;

  }

  public BMemoryItem getMemoryItem()
  { return this.item; }

  public void setExpression(BExpression e)
  {
    // if (item.getType() != expr.getType())
    //   throw new IllegalArgumentException("Type mismatch");

    this.expr = e;
    updateVariables();


  }

  public BExpression getExpression()
    { return this.expr; }

  public Icon getIcon()
  {
   // BDebug.print ("BAssignmentNode.getIcon Called item " + item);
    Icon icon = null;

    if (item == null)
    {
      selectedIcon = createTreeIcon("ai_memory_icon.gif",  BGlobal.colors.getAIMemoryColor());
      icon = selectedIcon;

    //  BDebug.print ("BAssignmentNode.getIcon null item " );

    }
    else if (item.getType() == BType.NUMBER )
    {
      if (numberIcon == null)
        numberIcon = createTreeIcon("ai/mem_item_number_18.gif",  BGlobal.colors.getAIMemoryColor());
      icon = numberIcon;
     // BDebug.print ("BAssignmentNode.getIcon  number" );
    }
    else if (item.getType() == BType.STRING )
    {
      if (stringIcon == null)
        stringIcon = createTreeIcon("ai/mem_item_string_18.gif",  BGlobal.colors.getAIMemoryColor());
      icon = stringIcon;
      //BDebug.print ("BAssignmentNode.getIcon string " );
    }
    else if (item.getType() == BType.BOOLEAN )
    {
      if (booleanIcon == null)
        booleanIcon = createTreeIcon("ai/mem_item_yn_18.gif",  BGlobal.colors.getAIMemoryColor());
      icon = booleanIcon;
      //BDebug.print ("BAssignmentNode.getIcon  boolean " );
    }
    else
    {
      selectedIcon = createTreeIcon("ai_memory_icon.gif",  BGlobal.colors.getAIMemoryColor());
      icon = selectedIcon;

     // BDebug.print ("BAssignmentNode.getIcon null item type " + item.getType() );

    }

    return icon;

  }

  public Icon getSelectedIcon()
    { return getIcon(); }

  protected String generateDefaultName()
  {
    if (item == null)
      return "Update Memory Item";
    else if (usesSimpleAssignment)
      return item.toString() + " = " + simpleValue ;
    else
      return item.toString()+ " = "+
        ((expr == null) ? "?" : expr.toString());
  }


  public boolean isValid()
  {
    if (item == null)
      return false;
    else if (usesSimpleAssignment)
      return (simpleValue != null);
    else if (expr == null)
      return false;
    else if ( item.getType() == BType.UNDEFINED )
      return false;
    else
      return expressionValid;

  }

  // Visit Method.
  // -------------------------------------------------------------

  public void visit()
  {
    if (item != null)
    {
        if (usesSimpleAssignment)
        {
          if (simpleValue != null)
            item.setValue(simpleValue);
        }
        else if (expr != null)
        {
          BValue value = expr.evaluate();
          item.setValue(value);
        }

    }
    


  }


  public void setActor(BActor a)
  {
    super.setActor(a);

    if ( (a == null) )
    {
      if (expr == null)
        nullifiedActorString = null;
      else
        nullifiedActorString = expr.toString();

      expr = null;
     //BDebug.print ("BAssignmentNode.setActor nullifiedActorString " + nullifiedActorString);

    }
    else if ( nullifiedActorString != null)
    {
      BBot bot = (BBot) a;
      BDecisionTreeAI ai = (BDecisionTreeAI) bot.getAI();
      BMemoryBank bank = ai.getMemoryBank();
      item = (BMemoryItem) bank.get(item.getName() );
     // BDebug.print ("BAssignmentNode.setActor new parsing ... " + nullifiedActorString);

      try
        { expr = ai.getParser().parse(nullifiedActorString);
        }
      catch ( BParseException ex ){}

    }

  }

  // State storage / retrieval...
  // -----------------------------------------------------------

  public void setState(Object o)
  {
    super.setState(o);

    AssignmentNodeState state =
      (AssignmentNodeState) o;

    this.item = state.item;
    this.expr = state.expr;
    this.usesSimpleAssignment = state.usesSimpleAssignment;
    this.simpleValue = state.simpleValue;
    this.nullifiedActorString = state.nullifiedActorString;


  }


  public void setUsesSimpleAssignment(boolean b)
    { usesSimpleAssignment = b; }

  public boolean getUsesSimpleAssingment()
    { return usesSimpleAssignment; }

  public void setSimpleValue(BValue v)
    {simpleValue = v;}

  public BValue getSimpleValue()
    { return simpleValue; }



  public void prune()
  {
    updateVariables();
  }

  public Object getState()
    { return new AssignmentNodeState(this); }

  protected static class AssignmentNodeState extends DecisionTreeNodeState
  {
    static final long serialVersionUID = -1L;

    protected BMemoryItem item;
    protected BExpression expr;
    protected boolean expressionValid;

    protected boolean usesSimpleAssignment;
    protected BValue simpleValue;
    protected String nullifiedActorString;

    public AssignmentNodeState(BAssignmentNode node)
    {
      super(node);
      this.item = node.item;
      this.expr = node.expr;
      this.usesSimpleAssignment = node.usesSimpleAssignment;
      this.simpleValue = node.simpleValue;
      this.nullifiedActorString = node.nullifiedActorString;

    }
  }

}