package roboforge.ai.decisions;

import roboforge.BBot;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BMove;
import roboforge.BChassis;
import roboforge.BMoveList;

import roboforge.ai.actions.*;
import roboforge.ai.expressions.*;
import roboforge.ai.values.*;
import roboforge.ai.*;
import roboforge.ui.components.HSBImageIcon;

import javax.swing.Icon;


public class BChassisActionNode extends BActionNode
{

  // Class variables...
  // -------------------------------------------------------------

  protected static Icon icon =
    createTreeIcon("ai/chassis_move_18.gif", BGlobal.colors.getAIActionColor());

 // protected static Icon selectedIcon =
  //  createTreeIcon("ai/chassis_move_18.gif", BGlobal.colors.getAIActionColor());


  // Member variables...
  // -------------------------------------------------------------

  protected BExpression distance;
  protected boolean usesSimpleDistance = true;
  protected float simpleDistance = 1;
  protected int chassisMoveIndex;
  protected String nullifiedActorDistance;

  // Constructor.
  // -------------------------------------------------------------

  public BChassisActionNode()
    { super(); }

  public BChassisActionNode(BActor a)
    { super(a); }

  public BChassisActionNode(BChassisActionNode node)
  {
    super(node);
    usesSimpleDistance = node.usesSimpleDistance;
    simpleDistance = node.simpleDistance;
    chassisMoveIndex = node.chassisMoveIndex;
    distance = node.distance;
    nullifiedActorDistance = node.nullifiedActorDistance;

    if (node.getAction() != null)
    {
      BChassisAction ca = new BChassisAction( (BChassisAction)node.getAction()    );
      setAction(ca);
    }

  }

  public Object clone()
    { return new BChassisActionNode(this); }


  // Accessor functions
  // -------------------------------------------------------------

  public void setChassisAction(BChassisAction m)
  {
    this.action = m;
    setDefaultName();
  }

  public BChassisAction getChassisAction()
    { return (BChassisAction) action; }


  public void setDistance(BExpression d)
  {
    this.distance = d;
    setDefaultName();
  }

  public void setDistance(float d)
  {
    this.simpleDistance = d;
    setDefaultName();
  }


  public BExpression getDistance()
    { return distance; }

  public Icon getIcon()
    { return icon; }

  public Icon getSelectedIcon()
    { return getIcon(); }

  protected String generateDefaultName()
  {
    if (action == null)
      return "Chassis move  ";
    else
      return action.toString() + "    ";
  }


  // Initialization method.
  // -------------------------------------------------------------

  public void init()
  {
    if (distance != null)
      distance.init();

    super.init();
  }


  // Visit method...
  // -----------------------------------------------------------

  public void visit()
  {


    if (action != null)
    {
      BChassisAction cAction = (BChassisAction)  action;

      if (!cAction.getActive() )
      {
        if (usesSimpleDistance)
          cAction.setDistance(simpleDistance);
        else
        {
          BValue d = BValue.UNDEFINED;

          if (distance != null)
            d = distance.evaluate();

          if (d.isNumber())
            cAction.setDistance((float) ((BNumber) d).getValue());
        }
      }
      super.visit();
    }
  }


  public void setUsesSimpleDistance (boolean b)
    { this.usesSimpleDistance = b ;}

  public boolean getUsesSimpleDistance ()
    { return usesSimpleDistance; }

  public float getSimpleDistance()
    { return simpleDistance; }


  /*
      super.setActor(a);

    if ( (a == null) )
    {
      if (question == null)
        nullifiedActorString = null;
      else
        nullifiedActorString = question.toString();

      question = null;
      //BDebug.print ("BDecisionNode.setActor nullifiedActorString " + nullifiedActorString);

    }
    else if ( nullifiedActorString != null)
    {
      BBot bot = (BBot) a;
      BDecisionTreeAI ai = (BDecisionTreeAI) bot.getAI();
      //BDebug.print ("BDecisionNode.setActor new parsing ... " + nullifiedActorString);

      try
        { question = ai.getParser().parse(nullifiedActorString);
        }
      catch ( BParseException ex ){}

    }*/


  public void setActor(BActor a)
  {


    if (a == null)
    {
      BChassis oldChassis = ((BBot)getActor() ).getChassis();
      if (getAction() == null)
        chassisMoveIndex = -1;
      else
      {
        BMove cMove = ((BChassisAction)getAction()).getChassisMove();

        chassisMoveIndex  = oldChassis.getChassisMoveList().indexOf( cMove );

        if (distance == null)
          nullifiedActorDistance = null;
        else
          nullifiedActorDistance = distance.toString();

        distance = null;

      }
      //BDebug.print ("BChassisActionNode.setActorchassisMoveIndex " + chassisMoveIndex);

    }
    else if (getAction() == null)
    {
      BBot b = (BBot) a;
      BChassis newChassis = b.getChassis();

      boolean setMove = true;

      if (  (chassisMoveIndex == BMoveList.CHASSIS_MOVE_LEFT) || (BMoveList.CHASSIS_MOVE_RIGHT ==  chassisMoveIndex ))
        setMove = newChassis.getCanMoveSideWays();

      if (chassisMoveIndex == -1)
        setMove = false;

      if (setMove)
      {

        try
          {
            if (nullifiedActorDistance == null)
              if (distance != null)
                nullifiedActorDistance = distance.toString();
                
            setDistance(  ((BDecisionTreeAI)b.getAI()).getParser().parse( nullifiedActorDistance )  );
          }
        catch
          ( Exception ex )
          { BDebug.print ("BChassisActionNode.setActor exception " + ex.getMessage());  }

        BChassisAction ca = new BChassisAction(newChassis.getChassisMoveList().getMove(chassisMoveIndex ), newChassis, 0);
        this.setAction(ca);

      }
    }

    super.setActor(a);

  }

  public boolean isValid()
  {
    boolean actionValid = super.isValid();

    if (actionValid)
      if (!usesSimpleDistance)
      {
        if(distance == null)
          actionValid = false;
        else
          actionValid = distance.getType() == BType.NUMBER;

      }



    return actionValid;


  }

  // State storage / retrieval...
  // -----------------------------------------------------------

  public void setState(Object o)
  {
    super.setState(o);

    ChassisActionNodeState state =
      (ChassisActionNodeState) o;

    this.distance = state.distance;
    this.usesSimpleDistance = state.usesSimpleDistance ;
    this.simpleDistance = state.simpleDistance;
    this.chassisMoveIndex = state.chassisMoveIndex;
    this.nullifiedActorDistance = state.nullifiedActorDistance;

  }

  public Object getState()
    { return new ChassisActionNodeState(this); }

  protected static class ChassisActionNodeState extends ActionNodeState
  {
    static final long serialVersionUID = -1L;

    protected BExpression distance;
    protected boolean usesSimpleDistance = true;
    protected float simpleDistance = 1;
    protected int chassisMoveIndex;
    protected String nullifiedActorDistance;



    public ChassisActionNodeState(BChassisActionNode node)
    {
      super(node);
      this.distance = node.distance;
      this.usesSimpleDistance = node.usesSimpleDistance ;
      this.simpleDistance = node.simpleDistance;
      this.chassisMoveIndex = node.chassisMoveIndex;
      this.nullifiedActorDistance = node.nullifiedActorDistance;

    }
  }

}
