package roboforge.ai.orient;

import roboforge.*;
import java.util.Vector;

import javax.media.j3d.*;
import javax.vecmath.*;

import roboforge.ai.worldstate.*;
import roboforge.ai.values.*;


public class BOrientLocationOnly extends BOrient
{
  static final long serialVersionUID = -1L;

  private static String name =  "Orient location only";
  private float errorMargin;
  private Vector3f orientVector;

  // working variables
  transient private BNumber oppDstItem = null;
  transient private BNumber oppAngItem= null;
  transient private BNumber oppFacingItem= null;
  transient private BNumber oppReachItem= null;
  transient private BNumber myReachItem= null;


  private float distToGo;
  private float angleToTurn;

  private static float STOPANDTURN_THRESHOLD   = 10f;
  private static float ANGLE_EQUALS_THRESHOLD   = 2f;
  private static float MAX_FORWARD_BEND = 10f;


  public BOrientLocationOnly()
    { this(new Vector3f(0, 0, 5), 1); }


  public BOrientLocationOnly(Vector3f location, float margin)
  {
    this.orientVector = location;
    orientVector.y = 0;
    this.errorMargin = margin;
  }

    public Object clone()
  {
    return new BOrientLocationOnly(
      new Vector3f(orientVector),
      this.errorMargin);
  }

  public boolean equals(Object o)
    { return (o instanceof BOrientLocationOnly); }




  public void setXZ(float x, float z)
    { orientVector.x = x;  orientVector.z = z; }

  public float getX()
    { return orientVector.x; }

  public float getZ()
    { return orientVector.z; }

  public void setErrorMargin(float errorMargin)
    { this.errorMargin = errorMargin;}

  public float getErrorMargin()
    { return this.errorMargin;}


    private static Vector3f transposeAngleDist_oppPos = new Vector3f();
  private static Matrix4f transposeAngleDist_turnAffine = new Matrix4f();
  private static Vector3f transposeAngleDist_oppFacingVector = new Vector3f();
  private static Matrix4f transposeAngleDist_oppFacingAffine = new Matrix4f();
  private static Vector3f zVector = new Vector3f(0,0,1f);
  private static Vector3f transposeAngleDist_crossdirectionV = new Vector3f();
  private static Matrix4f transposeAngleDist_orientDispAffine = new Matrix4f();
  private static Vector3f transposeAngleDist_orientDispVector  = new Vector3f();
  private static Vector3f transposeAngleDist_sideOfTargetA = new Vector3f();
  private static Vector3f transposeAngleDist_sideOfTargetB = new Vector3f();
  private static Matrix4f transposeAngleDist_tangentAffine = new Matrix4f();
  private static Vector3f transposeAngleDist_newTargetPos = new Vector3f();




  private void transposeAngleDist()
  // calculate the displacement orient
  // Bot Coordinates is where the bot is the locale, and +z is the facing of the bot
  {


    // the opponents Position to bot-coordinates
    // Bot Coordinates is where the bot is the locale, and +z is the facing of the bot
    transposeAngleDist_oppPos.set(0,0, distToGo);
    transposeAngleDist_turnAffine.rotY(angleToTurn * BPhysics.RAD_CONVERT);
    transposeAngleDist_turnAffine.transform(transposeAngleDist_oppPos);
    //BDebug.print( " oppPos "  + transposeAngleDist_oppPos );

    // opponent facing
    // creat a vector to represent the oppsnents facing in bot coordinates
    float oppFacing = (float)oppFacingItem.getValue();

    transposeAngleDist_oppFacingVector.set(transposeAngleDist_oppPos);
    transposeAngleDist_oppFacingVector.negate();

    //BDebug.print("BCpu.transposeAngleDist transposeAngleDist_oppFacingVectorFront :" + transposeAngleDist_oppFacingVector);

    //transposeAngleDist_oppFacingAffine.setIdentity();
    transposeAngleDist_oppFacingAffine.rotY(-oppFacing  * BPhysics.RAD_CONVERT);
    transposeAngleDist_oppFacingAffine.transform(transposeAngleDist_oppFacingVector);
    //BDebug.print( " transposeAngleDist_oppFacingVector "  + transposeAngleDist_oppFacingVector );


    //BDebug.print("BCpu.transposeAngleDist Modified transposeAngleDist_oppFacingVector :" + transposeAngleDist_oppFacingVector);

   // get angle between opponentfacing and bot front.
   // this will be used to transpose the orient displacement
    float coordMapAngle = transposeAngleDist_oppFacingVector.angle(  zVector  )    ;
   // BDebug.print("BCpu.transposeAngleDist coordMapAngle :" + coordMapAngle * BPhysics.DEGREE_CONVERT );

    transposeAngleDist_crossdirectionV.cross( zVector , transposeAngleDist_oppFacingVector );

    if (transposeAngleDist_crossdirectionV.y > 0)
    {
     // BDebug.print("BCpu.transposeAngleDist crossdirectionV is left" );
     //

    }

    else
    {
      coordMapAngle = -coordMapAngle;



    }

    //transposeAngleDist_orientDispAffine.setIdentity();
    transposeAngleDist_orientDispAffine.rotY(coordMapAngle );
   // BDebug.print("BCpu.transposeAngleDist coordMapAngle"  + coordMapAngle);
    // get the orient dfisplacement vector from the move and convert to bot coordinates
    transposeAngleDist_orientDispVector.set( orientVector );
    //BDebug.print("BCpu.transposeAngleDist original orientDispVector"  + transposeAngleDist_orientDispVector);
    transposeAngleDist_orientDispAffine.transform(transposeAngleDist_orientDispVector);

   // BDebug.print("BCpu.transposeAngleDist orientDispVector"  + transposeAngleDist_orientDispVector);

    // the the position you want to go is on the other side of the bot
    if ( transposeAngleDist_orientDispVector.dot(transposeAngleDist_oppPos) > 0 )
    {
      //BDebug.print("BCpu.print new Transposed Vector is Backfacing to me ");

      transposeAngleDist_sideOfTargetA.set(transposeAngleDist_oppPos);
      transposeAngleDist_sideOfTargetA.normalize();
      transposeAngleDist_sideOfTargetB.set(transposeAngleDist_sideOfTargetA );

      transposeAngleDist_tangentAffine.setIdentity();
      transposeAngleDist_tangentAffine.rotY(BPhysics.PI / 2f );

      transposeAngleDist_tangentAffine.transform(transposeAngleDist_sideOfTargetA);

      transposeAngleDist_tangentAffine.rotY(-BPhysics.PI / 2f );
      transposeAngleDist_tangentAffine.transform(transposeAngleDist_sideOfTargetB);

      float oppReach = (float)oppReachItem.getValue()  * 2.0f;
      float myReach =  (float) myReachItem.getValue()  * 2.0f;


      transposeAngleDist_sideOfTargetA.scale(oppReach  + myReach);
      transposeAngleDist_sideOfTargetB.scale(oppReach + myReach);

      Vector3f tangentTarget;

      float aAngle =  BPhysics.round(transposeAngleDist_sideOfTargetA.angle(transposeAngleDist_orientDispVector) , 4 );
      float bAngle =  BPhysics.round(transposeAngleDist_sideOfTargetB.angle(transposeAngleDist_orientDispVector) , 4 );


      if ( aAngle  < bAngle )
      {
        tangentTarget =  transposeAngleDist_sideOfTargetA;


      }
      else
        tangentTarget =  transposeAngleDist_sideOfTargetB;


      Vector3f newTargetPos = new Vector3f(transposeAngleDist_oppPos);
      newTargetPos.add(tangentTarget);

      distToGo = newTargetPos.length();
      angleToTurn = newTargetPos.angle(   zVector  )   * BPhysics.DEGREE_CONVERT ;

      transposeAngleDist_crossdirectionV.cross(newTargetPos ,  zVector);

      if (transposeAngleDist_crossdirectionV.y > 0)
        angleToTurn = (  180f - angleToTurn  ) + 180f;


    }
    else
    {

      transposeAngleDist_newTargetPos.set(transposeAngleDist_oppPos);
      //BDebug.print("BCpu.transposeAngleDist oldPos :" + newTargetPos);
      //BDebug.print("BCpu.transposeAngleDist orientDispVector:" + orientDispVector);

      // add the orient displacement into the opponentpos
      transposeAngleDist_newTargetPos.add(transposeAngleDist_orientDispVector);


    //  BDebug.print("BCpu.transposeAngleDist newdirectTargetPos:" + newTargetPos);

      distToGo = transposeAngleDist_newTargetPos.length();
      angleToTurn = transposeAngleDist_newTargetPos.angle(  zVector )   * BPhysics.DEGREE_CONVERT ;

     // BDebug.print("BCpu.transposeAngleDist Modified smallAngleToTurn:" + angleToTurn);

      // convert the new point in bot-coordinates back to the 2 channels dist and angle
      transposeAngleDist_crossdirectionV.cross(transposeAngleDist_newTargetPos ,  zVector );

      if (transposeAngleDist_crossdirectionV.y > 0)
        angleToTurn = (  180f - angleToTurn  ) + 180f;


    }


  }



   public boolean executeOrient( BBot bot)
  {
    BWorldState worldState = bot.getWorldState();

    oppDstItem = (BNumber)worldState.get(BSimulation.WSTATE_OPPDISTANCE).getValue();
    oppAngItem = (BNumber)worldState.get(BSimulation.WSTATE_OPPANGLE).getValue();
    oppFacingItem = (BNumber)worldState.get(BSimulation.WSTATE_OPPFACING).getValue() ;
    oppReachItem = (BNumber)worldState.get(BSimulation.WSTATE_OPPREACH).getValue();
    myReachItem = (BNumber)worldState.get(BSimulation.WSTATE_MYREACH).getValue();;


    float oppBotAngle = (float)oppAngItem.getValue();
    distToGo = (float) oppDstItem.getValue() ;
    angleToTurn =(float) oppAngItem.getValue();


   // BDebug.print ("BOrientFull.print pre-transpose angleToTurn " + angleToTurn );

    // create new target area to go to
    transposeAngleDist();

    float smallestAngle;
    if (angleToTurn < 180 )
        smallestAngle = angleToTurn;
    else
        smallestAngle = 360 - angleToTurn ;


    // precalculate the target pos so we can use it to test orient achieved
    float targetAngleToOpp = 0;
    float smallTargetAngle;
    int targetRotateDir;

    if (oppBotAngle >  targetAngleToOpp )
        smallTargetAngle =  oppBotAngle -  targetAngleToOpp;
    else
        smallTargetAngle =  targetAngleToOpp - oppBotAngle;

    if (smallTargetAngle > 180 )
    {
        smallTargetAngle = 360 - smallTargetAngle;
        targetRotateDir = BMoveList.CHASSIS_ROTATE_LEFT;

    }
    else
        targetRotateDir = BMoveList.CHASSIS_ROTATE_RIGHT;

    // testing orient achieved

     if ( distToGo <= errorMargin )
     {
         // BDebug.print(" Orient achieved " +getName() + ".orientBot orient achieved ");
          return true;
     }


     if (distToGo >  errorMargin )
     {


        if (smallestAngle > 180 - STOPANDTURN_THRESHOLD * 2 )
        {
          float backBend = java.lang.Math.min(MAX_FORWARD_BEND, 180 - smallestAngle);

          if ( angleToTurn < 180 )
            backBend = backBend;
          else
            backBend = -backBend;

          bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(BMoveList.CHASSIS_MOVE_BACKWARDS), distToGo,   BChassis.ADD_MODE_UPDATE_BUT_NOT_OVERIDE , backBend, null);

        }
        else if ( bot.getChassis().getCanMoveSideWays() &&  (smallestAngle > 90 - STOPANDTURN_THRESHOLD )  && (smallestAngle < 90 + STOPANDTURN_THRESHOLD ) )
        {
          int sideWaysDir;

          if ( angleToTurn < 180 )
            sideWaysDir =  BMoveList.CHASSIS_MOVE_RIGHT;
          else
            sideWaysDir =  BMoveList.CHASSIS_MOVE_LEFT;

          bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(sideWaysDir), distToGo,   BChassis.ADD_MODE_UPDATE_BUT_NOT_OVERIDE , 0, null);

        }

        else if (smallestAngle > STOPANDTURN_THRESHOLD )
        {
         // BDebug.print ("BOrientFull.print rotate angleToTurn " + angleToTurn );
          //BDebug.print ("BOrientFull.print rotate smallestAngle " + smallestAngle );

          int rotateDir;

          if ( angleToTurn < 180 )
            rotateDir =  BMoveList.CHASSIS_ROTATE_RIGHT;
          else
            rotateDir =  BMoveList.CHASSIS_ROTATE_LEFT;

          bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(rotateDir), smallestAngle,   BChassis.ADD_MODE_OVERRIDE, null );

         }
        else
        {
          smallestAngle = java.lang.Math.min(MAX_FORWARD_BEND, smallestAngle);

          if ( angleToTurn < 180 )
            smallestAngle = -smallestAngle;
          else
            smallestAngle = smallestAngle;

          bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(BMoveList.CHASSIS_MOVE_FORWARD), distToGo,   BChassis.ADD_MODE_UPDATE_BUT_NOT_OVERIDE , smallestAngle, null);
          //setArenaComments("BBot.orientBot: Set Move Forward "+distToGo+" metres ");
        }

      }
      else
      {
         // achieved the position
        //
        return true;


      }

   return false;
  }



  public String getName()
    { return name; }


}

/*
package roboforge.ai.orient;

import roboforge.*;

// import java.awt.Point2D;
import java.util.Vector;

import javax.media.j3d.*;
import javax.vecmath.*;

import roboforge.ai.worldstate.*;
import roboforge.ai.values.*;


public class BOrientFull extends BOrient
{
  static final long serialVersionUID = -1L;

  private static String name =  "Orient to location and facing";
  private float errorMargin;
  private Vector3f orientVector;
  private float orientAngleFacingOpponent;  // the Angle of the bot vs the angle of the apponent

  // working variables
  transient private BNumber oppDstItem = null;
  transient private BNumber oppAngItem= null;
  transient private BNumber oppFacingItem= null;
  transient private BNumber oppReachItem= null;
  transient private BNumber myReachItem= null;


  private float distToGo;
  private float angleToTurn;

  private static float STOPANDTURN_THRESHOLD   = 10f;
  private static float ANGLE_EQUALS_THRESHOLD   = 2f;
  private static float MAX_FORWARD_BEND = 10f;



  public Object clone()
  {
    return new BOrientFull(
      new Vector3f(orientVector),
      this.orientAngleFacingOpponent,
      this.errorMargin);
  }

  public boolean equals(Object o)
    { return (o instanceof BOrientFull); }




  public void setXZ(float x, float z)
    { orientVector.x = x;  orientVector.z = z; }

  public float getX()
    { return orientVector.x; }

  public float getZ()
    { return orientVector.z; }


  public void setFacing(float facing)
    { this.orientAngleFacingOpponent = facing; }

  public float getFacing( )
    { return this.orientAngleFacingOpponent; }

  public void setErrorMargin(float errorMargin)
    { this.errorMargin = errorMargin;}

  public float getErrorMargin()
    { return this.errorMargin;}


  private static Vector3f transposeAngleDist_oppPos = new Vector3f();
  private static Matrix4f transposeAngleDist_turnAffine = new Matrix4f();
  private static Vector3f transposeAngleDist_oppFacingVector = new Vector3f();
  private static Matrix4f transposeAngleDist_oppFacingAffine = new Matrix4f();
  private static Vector3f zVector = new Vector3f(0,0,1f);
  private static Vector3f transposeAngleDist_crossdirectionV = new Vector3f();
  private static Matrix4f transposeAngleDist_orientDispAffine = new Matrix4f();
  private static Vector3f transposeAngleDist_orientDispVector  = new Vector3f();
  private static Vector3f transposeAngleDist_sideOfTargetA = new Vector3f();
  private static Vector3f transposeAngleDist_sideOfTargetB = new Vector3f();
  private static Matrix4f transposeAngleDist_tangentAffine = new Matrix4f();
  private static Vector3f transposeAngleDist_newTargetPos = new Vector3f();




  private void transposeAngleDist()
  // calculate the displacement orient
  // Bot Coordinates is where the bot is the locale, and +z is the facing of the bot
  {


    // the opponents Position to bot-coordinates
    // Bot Coordinates is where the bot is the locale, and +z is the facing of the bot
    transposeAngleDist_oppPos.set(0,0, distToGo);
    transposeAngleDist_turnAffine.rotY(angleToTurn * BPhysics.RAD_CONVERT);
    transposeAngleDist_turnAffine.transform(transposeAngleDist_oppPos);
    //BDebug.print( " oppPos "  + transposeAngleDist_oppPos );

    // opponent facing
    // creat a vector to represent the oppsnents facing in bot coordinates
    float oppFacing = (float)oppFacingItem.getValue();

    transposeAngleDist_oppFacingVector.set(transposeAngleDist_oppPos);
    transposeAngleDist_oppFacingVector.negate();

    //BDebug.print("BCpu.transposeAngleDist transposeAngleDist_oppFacingVectorFront :" + transposeAngleDist_oppFacingVector);

    //transposeAngleDist_oppFacingAffine.setIdentity();
    transposeAngleDist_oppFacingAffine.rotY(-oppFacing  * BPhysics.RAD_CONVERT);
    transposeAngleDist_oppFacingAffine.transform(transposeAngleDist_oppFacingVector);
    //BDebug.print( " transposeAngleDist_oppFacingVector "  + transposeAngleDist_oppFacingVector );


    //BDebug.print("BCpu.transposeAngleDist Modified transposeAngleDist_oppFacingVector :" + transposeAngleDist_oppFacingVector);

   // get angle between opponentfacing and bot front.
   // this will be used to transpose the orient displacement
    float coordMapAngle = transposeAngleDist_oppFacingVector.angle(  zVector  )    ;
   // BDebug.print("BCpu.transposeAngleDist coordMapAngle :" + coordMapAngle * BPhysics.DEGREE_CONVERT );

    transposeAngleDist_crossdirectionV.cross( zVector , transposeAngleDist_oppFacingVector );

    if (transposeAngleDist_crossdirectionV.y > 0)
    {
     // BDebug.print("BCpu.transposeAngleDist crossdirectionV is left" );
     //

    }

    else
    {
      coordMapAngle = -coordMapAngle;



    }

    //transposeAngleDist_orientDispAffine.setIdentity();
    transposeAngleDist_orientDispAffine.rotY(coordMapAngle );
   // BDebug.print("BCpu.transposeAngleDist coordMapAngle"  + coordMapAngle);
    // get the orient dfisplacement vector from the move and convert to bot coordinates
    transposeAngleDist_orientDispVector.set( orientVector );
    //BDebug.print("BCpu.transposeAngleDist original orientDispVector"  + transposeAngleDist_orientDispVector);
    transposeAngleDist_orientDispAffine.transform(transposeAngleDist_orientDispVector);

   // BDebug.print("BCpu.transposeAngleDist orientDispVector"  + transposeAngleDist_orientDispVector);

    // the the position you want to go is on the other side of the bot
    if ( transposeAngleDist_orientDispVector.dot(transposeAngleDist_oppPos) > 0 )
    {
      //BDebug.print("BCpu.print new Transposed Vector is Backfacing to me ");

      transposeAngleDist_sideOfTargetA.set(transposeAngleDist_oppPos);
      transposeAngleDist_sideOfTargetA.normalize();
      transposeAngleDist_sideOfTargetB.set(transposeAngleDist_sideOfTargetA );

      transposeAngleDist_tangentAffine.setIdentity();
      transposeAngleDist_tangentAffine.rotY(BPhysics.PI / 2f );

      transposeAngleDist_tangentAffine.transform(transposeAngleDist_sideOfTargetA);

      transposeAngleDist_tangentAffine.rotY(-BPhysics.PI / 2f );
      transposeAngleDist_tangentAffine.transform(transposeAngleDist_sideOfTargetB);

      float oppReach = (float)oppReachItem.getValue()  * 2.0f;
      float myReach =  (float) myReachItem.getValue()  * 2.0f;


      transposeAngleDist_sideOfTargetA.scale(oppReach  + myReach);
      transposeAngleDist_sideOfTargetB.scale(oppReach + myReach);

      Vector3f tangentTarget;

      float aAngle =  BPhysics.round(transposeAngleDist_sideOfTargetA.angle(transposeAngleDist_orientDispVector) , 4 );
      float bAngle =  BPhysics.round(transposeAngleDist_sideOfTargetB.angle(transposeAngleDist_orientDispVector) , 4 );


      if ( aAngle  < bAngle )
      {
        tangentTarget =  transposeAngleDist_sideOfTargetA;


      }
      else
        tangentTarget =  transposeAngleDist_sideOfTargetB;


      Vector3f newTargetPos = new Vector3f(transposeAngleDist_oppPos);
      newTargetPos.add(tangentTarget);

      distToGo = newTargetPos.length();
      angleToTurn = newTargetPos.angle(   zVector  )   * BPhysics.DEGREE_CONVERT ;

      transposeAngleDist_crossdirectionV.cross(newTargetPos ,  zVector);

      if (transposeAngleDist_crossdirectionV.y > 0)
        angleToTurn = (  180f - angleToTurn  ) + 180f;


    }
    else
    {

      transposeAngleDist_newTargetPos.set(transposeAngleDist_oppPos);
      //BDebug.print("BCpu.transposeAngleDist oldPos :" + newTargetPos);
      //BDebug.print("BCpu.transposeAngleDist orientDispVector:" + orientDispVector);

      // add the orient displacement into the opponentpos
      transposeAngleDist_newTargetPos.add(transposeAngleDist_orientDispVector);


    //  BDebug.print("BCpu.transposeAngleDist newdirectTargetPos:" + newTargetPos);

      distToGo = transposeAngleDist_newTargetPos.length();
      angleToTurn = transposeAngleDist_newTargetPos.angle(  zVector )   * BPhysics.DEGREE_CONVERT ;

     // BDebug.print("BCpu.transposeAngleDist Modified smallAngleToTurn:" + angleToTurn);

      // convert the new point in bot-coordinates back to the 2 channels dist and angle
      transposeAngleDist_crossdirectionV.cross(transposeAngleDist_newTargetPos ,  zVector );

      if (transposeAngleDist_crossdirectionV.y > 0)
        angleToTurn = (  180f - angleToTurn  ) + 180f;


    }


  }





  public boolean executeFacing( BBot bot, BNumber oppAng )
  {


    float angleToTurn = (float)oppAng.getValue() - orientAngleFacingOpponent;
    float smallestAngle;

    if (angleToTurn < -180 )
      smallestAngle = 360 + angleToTurn;
    else  if (angleToTurn < 0 )
       smallestAngle = -angleToTurn;
    else if (angleToTurn < 180 )
        smallestAngle = angleToTurn;
    else
        smallestAngle = 360 - angleToTurn ;

    if ( smallestAngle < ANGLE_EQUALS_THRESHOLD )
    {
      bot.setArenaComments(" BCpu.orient achieved " +getName() + ".orientBot orient achieved ");
      bot.getChassis().terminateChassisMove(false);
      return true;
    }
    else
    {
       int rotateDir;

       if (angleToTurn < -180 )
        rotateDir =  BMoveList.CHASSIS_ROTATE_RIGHT;
       else if (angleToTurn < 0 )
        rotateDir =  BMoveList.CHASSIS_ROTATE_LEFT;

       else if ( angleToTurn < 180 )
        rotateDir =  BMoveList.CHASSIS_ROTATE_RIGHT;
       else
        rotateDir =  BMoveList.CHASSIS_ROTATE_LEFT;

        bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(rotateDir), smallestAngle,   BChassis.ADD_MODE_UPDATE_BUT_NOT_OVERIDE, null );

    }

     return false;


  }



   public boolean executeOrient( BBot bot)
  {
    BWorldState worldState = bot.getWorldState();

    oppDstItem = (BNumber)worldState.get(BSimulation.WSTATE_OPPDISTANCE).getValue();
    oppAngItem = (BNumber)worldState.get(BSimulation.WSTATE_OPPANGLE).getValue();
    oppFacingItem = (BNumber)worldState.get(BSimulation.WSTATE_OPPFACING).getValue() ;
    oppReachItem = (BNumber)worldState.get(BSimulation.WSTATE_OPPREACH).getValue();
    myReachItem = (BNumber)worldState.get(BSimulation.WSTATE_MYREACH).getValue();;


    float oppBotAngle = (float)oppAngItem.getValue();
    distToGo = (float) oppDstItem.getValue() ;
    angleToTurn =(float) oppAngItem.getValue();


   // BDebug.print ("BOrientFull.print pre-transpose angleToTurn " + angleToTurn );

    // create new target area to go to
    transposeAngleDist();

    float smallestAngle;
    if (angleToTurn < 180 )
        smallestAngle = angleToTurn;
    else
        smallestAngle = 360 - angleToTurn ;


    // precalculate the target pos so we can use it to test orient achieved
    float targetAngleToOpp = orientAngleFacingOpponent;
    float smallTargetAngle;
    int targetRotateDir;

    if (oppBotAngle >  targetAngleToOpp )
        smallTargetAngle =  oppBotAngle -  targetAngleToOpp;
    else
        smallTargetAngle =  targetAngleToOpp - oppBotAngle;

    if (smallTargetAngle > 180 )
    {
        smallTargetAngle = 360 - smallTargetAngle;
        targetRotateDir = BMoveList.CHASSIS_ROTATE_LEFT;

    }
    else
        targetRotateDir = BMoveList.CHASSIS_ROTATE_RIGHT;

    // testing orient achieved

     if (  ( distToGo <= errorMargin )
          &&   ( smallTargetAngle < ANGLE_EQUALS_THRESHOLD )  )
       {
         // BDebug.print(" Orient achieved " +getName() + ".orientBot orient achieved ");
          return true;
       }


     if (distToGo >  errorMargin )
     {


        if (smallestAngle > 180 - STOPANDTURN_THRESHOLD * 2 )
        {
          float backBend = java.lang.Math.min(MAX_FORWARD_BEND, 180 - smallestAngle);

          if ( angleToTurn < 180 )
            backBend = backBend;
          else
            backBend = -backBend;

          bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(BMoveList.CHASSIS_MOVE_BACKWARDS), distToGo,   BChassis.ADD_MODE_UPDATE_BUT_NOT_OVERIDE , backBend, null);

        }
        else if ( bot.getChassis().getCanMoveSideWays() &&  (smallestAngle > 90 - STOPANDTURN_THRESHOLD )  && (smallestAngle < 90 + STOPANDTURN_THRESHOLD ) )
        {
          int sideWaysDir;

          if ( angleToTurn < 180 )
            sideWaysDir =  BMoveList.CHASSIS_MOVE_RIGHT;
          else
            sideWaysDir =  BMoveList.CHASSIS_MOVE_LEFT;

          bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(sideWaysDir), distToGo,   BChassis.ADD_MODE_UPDATE_BUT_NOT_OVERIDE , 0, null);

        }

        else if (smallestAngle > STOPANDTURN_THRESHOLD )
        {
         // BDebug.print ("BOrientFull.print rotate angleToTurn " + angleToTurn );
          //BDebug.print ("BOrientFull.print rotate smallestAngle " + smallestAngle );

          int rotateDir;

          if ( angleToTurn < 180 )
            rotateDir =  BMoveList.CHASSIS_ROTATE_RIGHT;
          else
            rotateDir =  BMoveList.CHASSIS_ROTATE_LEFT;

          bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(rotateDir), smallestAngle,   BChassis.ADD_MODE_OVERRIDE, null );

         }
        else
        {
          smallestAngle = java.lang.Math.min(MAX_FORWARD_BEND, smallestAngle);

          if ( angleToTurn < 180 )
            smallestAngle = -smallestAngle;
          else
            smallestAngle = smallestAngle;

          bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(BMoveList.CHASSIS_MOVE_FORWARD), distToGo,   BChassis.ADD_MODE_UPDATE_BUT_NOT_OVERIDE , smallestAngle, null);
          //setArenaComments("BBot.orientBot: Set Move Forward "+distToGo+" metres ");
        }

      }
      else
      {
         // achieved the position
        BDebug.print(" BCpu.orient position achieved, rotating bot ");
        return executeFacing(bot,oppAngItem);


      }

   return false;
  }



  public String getName()
    { return name; }




}*/