package roboforge.ai.values;

import java.io.Serializable;
import java.io.*;

import roboforge.BDebug;


public abstract class BValue
  implements BType, Cloneable, Serializable
{

  // Serialization ID.
  // -----------------------------------------------------------
  static final long serialVersionUID = -1L;



  // Class variables...
  // -------------------------------------------------------------

  public static final BUndefined UNDEFINED = new BUndefined();     

  // YLY - 11 Jul 00 Changed 'Enumerated' to 'Set' AND 'String' to 'Text'
  // Data set removed
  protected static String[] TYPE_NAMES =
    { "Undefined", "Number", "Logical", "Text" };

  public static String[] getTypeNames()
    { return TYPE_NAMES; }


  // Member variables
  // -------------------------------------------------------------

  protected int type;


  // Accessor methods...
  // -------------------------------------------------------------

  public int getType()
    { return type; }

  public boolean isCompatible(BValue v)
    { return this.type == v.type; }

  public boolean isUndefined()
    { return type == BType.UNDEFINED; }

  public boolean isNumber()
    { return type == BType.NUMBER; }

  public boolean isBoolean()
    { return type == BType.BOOLEAN; }

  public boolean isString()
    { return type == BType.STRING; }

  public boolean isEnumerated()
    { return type == BType.ENUMERATED; }




  // Factory method for BValues.
  // -------------------------------------------------------------

  public static BValue createValue(int type)
  {
    //BDebug.print("BValue.CreateValue " + type );

    if (type == BType.UNDEFINED)
      return new BUndefined();
    else if (type == BType.NUMBER)
      return new BNumber();
    else if (type == BType.BOOLEAN)
      return new BBoolean();
    else if (type == BType.STRING)
      return new BString();
    else if (type == BType.ENUMERATED)
      return new BEnumerated();
    else
      return null;
  }


  // IO Methods...
  // -------------------------------------------------------------

  public void writeDataType(DataOutput dout) throws Exception
  {
    //BDebug.print("BValue.writing " + type);
    dout.writeInt(type);
  }

  public abstract void readValue(DataInput din) throws IOException;
  public abstract void writeValue(DataOutput dout) throws IOException;

  public abstract void setValue(BValue v);


  // Clone method...
  // -------------------------------------------------------------

  public Object clone()
  {
    try
      { return super.clone(); }
    catch (CloneNotSupportedException e)
      { return null; }
  }

}
