package roboforge.ai.worldstate;

import java.util.*;

import roboforge.BLookup;
import roboforge.ai.expressions.*;


public interface BWorldState extends BLookup
{

  // KEY NAMING CONVENTION:
  // -----------------------------------------------------------

  // Keys follow a dotted naming convention:
  //   e.g.  world.me.zones.attack zone.isActive

  // Accessor methods...
  // -----------------------------------------------------------

  public BWorldItem get(String key);
    // Retrieve a world-item with ID 'key' from the state.
    // Returns null if item was not found.

  public void put(String key, BWorldItem item);
    // Place a world-item with ID 'key' into the state.
    // Returns null if item was not found.

  public void put(BWorldItem item);
    // Convenience method.

  public void remove(String key);
    // Remove the  ID 'key' and all descendant ID's.

  public Enumeration getWorldItems();
    // Enumeration of all world items.

  public static final char DELIMITER = '.';
    // Delimiter used to separate items in the key.

  // Evaluation method...
  // -----------------------------------------------------------

  public void evaluate();
    // Evaluates all items in the world-state, by calling
    // evaluate() on each one.



}