package roboforge.ai.worldstate;

import roboforge.*;
import java.util.*;
import javax.swing.tree.*;
import roboforge.ai.values.*;
import roboforge.ai.memory.*;

public class BWorldStateNode extends DefaultMutableTreeNode
  implements BWorldState
{

  public static int NODE_TYPE_CATEGORY = 1;
  public static int NODE_TYPE_ITEM = 2;
  public static int NODE_TYPE_ROOT = 3;
  public static int NODE_TYPE_MEMORY = 4;


  private int nodeType;
    // the type of node

  private String keyPart;
    // the partOfTheKey


  public BWorldStateNode()
    { this("" , new BWorldCategory("", "World"), NODE_TYPE_ROOT ); }

  public char getKeyDelimeter()
  { return DELIMITER ; }


  public BWorldStateNode(String keyPart, Object item, int type)
  {
    this.userObject = item;
    this.keyPart = keyPart;
    this.nodeType = type;

    /*
    if (item instanceof BWorldCategory)
      nodeType =  NODE_TYPE_CATEGORY;
    else if (item instanceof BWorldItem)
      nodeType = NODE_TYPE_ITEM;
    else
      throw new IllegalArgumentException(
        "item: expected BWorldCategory or BWorldItem.");   */
  }

  private String getNextCategory(String key )
  {
    if ( key.indexOf(DELIMITER) != -1)
      return key.substring(0,key.indexOf(DELIMITER)  ) ;
    else
      return key;
  }

   private String getRemainder(String key )
  {
    if ( key.indexOf(DELIMITER) != -1)
      return key.substring(key.indexOf(DELIMITER)  + 1) ;
    else
      return "";
  }


  public int getNodeType()
  { return nodeType; }

  private String getKeyPart()
  { return keyPart; }


  public void put (BWorldCategory newItem )
  {
    putInternal(newItem.getKey(), newItem, NODE_TYPE_CATEGORY);

  }

  public void put (BWorldItem newItem )
  {
    putInternal(newItem.getKey(), newItem, NODE_TYPE_ITEM);

  }

  public void put (BMemoryItem item )
  {
    //BDebug.print ("BDecisionNode put(Memory) me.memory." + item.getName() );
    putInternal ("me.memory." + item.getName(), item, NODE_TYPE_MEMORY );
  }

  public void evaluate()
  {
    Enumeration depth = depthFirstEnumeration();

    while (depth.hasMoreElements())
    {
      BWorldStateNode node = (BWorldStateNode)depth.nextElement();

      if (node.getNodeType()  == BWorldStateNode.NODE_TYPE_ITEM)
        ((BWorldItem)node.getUserObject()).evaluate();

    }
  }

  public Object get(Object key)
    { return this.get((String) key); } 

  public BWorldItem get(String key)
  {
     String keyPart =  getNextCategory( key );
     String remainder = getRemainder(key);

     if (this.isLeaf())
     {
        if  ( this.getNodeType()== NODE_TYPE_ITEM )
          return (BWorldItem) this.getUserObject();
        else
          return null;
     }
     else
     {

        boolean foundKeyPart = false;
        Vector immediateChildren = this.children;
        for(int i = 0; i< immediateChildren.size(); i++ )
          if (  ((BWorldStateNode)immediateChildren.get(i)).getKeyPart().equalsIgnoreCase(keyPart) )
          {

            BWorldStateNode pathToTraverse =  (BWorldStateNode)immediateChildren.get(i);
            return pathToTraverse.get(remainder);

          }

        // if not found
         return null;
     }


  }

  public void cleanse()
  {
    this.removeAllChildren();
  }

   public void remove( String key  )
  {
     String keyPart =  getNextCategory( key );
     String remainder = getRemainder (key);

     //BDebug.print ("remove testing " + key + "   ::   " + this.keyPart);
     if (key.equalsIgnoreCase(""))
     {
          //BDebug.print ("removing " + this.getUserObject());
          this.removeFromParent();

          Enumeration e = this.breadthFirstEnumeration();
          while (e.hasMoreElements())
          {
            BWorldStateNode n = (BWorldStateNode) e.nextElement();
            if (n.getNodeType() == NODE_TYPE_ITEM)
            {
              BWorldItem item = (BWorldItem)n.getUserObject();
              item.setValue(BValue.UNDEFINED);

            }
          }
     }
     else
     {

        boolean foundKeyPart = false;
        Vector immediateChildren = this.children;

        if (immediateChildren != null)
          for(int i = 0; i< immediateChildren.size(); i++ )
            if (  ((BWorldStateNode)immediateChildren.get(i)).getKeyPart().equalsIgnoreCase(keyPart) )
            {

              BWorldStateNode pathToTraverse =  (BWorldStateNode)immediateChildren.get(i);
              pathToTraverse.remove(remainder);

            }


     }


  }

  public void put (String key, BWorldItem newItem )
  {
    putInternal(key, newItem, NODE_TYPE_ITEM);
  }

  public Enumeration getWorldItems()
  {
    Vector treeState = new Vector();
    Enumeration e = this.depthFirstEnumeration();

    while (e.hasMoreElements() )
    {
      BWorldStateNode n = (BWorldStateNode) e.nextElement();
      if (n.getNodeType() == BWorldStateNode.NODE_TYPE_ITEM )
        treeState.add(n.getUserObject());

    }

    return treeState.elements();


  }



  public void putInternal (String key, Object newItem, int type )
  // PRE : Ordering IS CASE SENSITIVE
  {

    String keyPart =  getNextCategory( key );
    String remainder = getRemainder (key);

    boolean foundKeyPart = false;

    int insertIndex = 0;  // for alphabetical order
    String displayName = newItem.toString();

    Vector immediateChildren = this.children;

    if (immediateChildren != null)
      for(int i = 0; i< immediateChildren.size() && !foundKeyPart; i++ )
      {
        BWorldStateNode node = (BWorldStateNode)immediateChildren.get(i);

        // ordering
        // remainder > 0 means that it is a category
        if (remainder.length() > 0)
        {
         // BDebug.print ("Category Insert testing " + displayName + " and " + node.toString() + " result " + insertIndex );

           if (node.getNodeType() == NODE_TYPE_CATEGORY)
            if (keyPart.compareTo(node.toString()) > 0 )
              insertIndex++;

        }
        else //(type == NODE_TYPE_ITEM)
        {
          if (node.getNodeType() == NODE_TYPE_CATEGORY)
            insertIndex++;
          else if (displayName.compareTo(node.toString()) > 0 )
            insertIndex++;

        }



        if (  node.getKeyPart().equalsIgnoreCase(keyPart) )
        {
          foundKeyPart = true;
          BWorldStateNode pathToTraverse =  (BWorldStateNode)immediateChildren.get(i);
          pathToTraverse.putInternal(remainder, newItem , type);

          if (displayName.compareTo(immediateChildren.get(i).toString()) > 0 )
            insertIndex++;

        }
        
      }

    if (!foundKeyPart )
    {
      if (remainder.equals("") )
      {
        if (newItem != getUserObject() )
        {
          BWorldStateNode tempNode = new  BWorldStateNode(keyPart, newItem, type);
          this.insert(tempNode, insertIndex);
        }

        //this.add(tempNode);
      }
      else
      {
        BWorldStateNode tempNode = new  BWorldStateNode(keyPart, new BWorldCategory(keyPart , keyPart ), NODE_TYPE_CATEGORY );
         this.insert(tempNode, insertIndex);
        //this.add(tempNode);
        tempNode.putInternal (remainder, newItem , type);
      }

    }


  }


}