package roboforge.animation.alpha;

public class BKeyedChannel implements BAlphaChannel
{

  protected BAlphaValue[] keys;
    // A set of alpha value keys.

  protected float[] alphas;
    // The alpha values of each key.


  public BKeyedChannel()
  {
    keys    = new BAlphaValue[0];
    alphas  = new float[0];
  }


  public void addKey(float alpha, BAlphaValue key)
  {
    int n = keys.length;

    // Locate insertion index.
    int i = 0;
    while ((i < n) && (alpha > alphas[i])) i++;

    // Allocate space for the new key.
    BAlphaValue[] okeys   = keys;
    float[]       oalphas = alphas;

    keys   = new BAlphaValue[n+1];
    alphas = new float[n+1];

    // Copy elements prior to insertion point across.
    if (i > 0)
    {
      System.arraycopy(okeys,   0, keys,   0, i);
      System.arraycopy(oalphas, 0, alphas, 0, i);
    }

    // Copy elements past insertion point across.
    if (i < n)
    {
      System.arraycopy(okeys,   i, keys,   i+1, n-i);
      System.arraycopy(oalphas, i, alphas, i+1, n-i);
    }

    // Insert the new key.
    keys[i]   = key;
    alphas[i] = alpha;
  }



  public void evaluate(BAlphaValue value, float alpha)
  {
    // Get key count.
    int n = keys.length;

    // If no keys, return null.
    if (n == 0)
      { return; }

    // If one key, return that key.
    if (n == 1)
      { value.set(keys[0]); return; }

    // Low range-check on alphas.
    if (alpha <= alphas[0])
      { value.set(keys[0]); return; }

    // High range-check on alphas.
    int last = n-1;
    if (alpha >= alphas[last])
      { value.set(keys[last]); return; }

    // Find appropriate keypair.
    int i = 0;
    while ((i < n) && (alpha > alphas[i])) i++;

    // Normalize alpha for the keypair (into 't').
    float lo = alphas[i-1];
    float hi = alphas[i];
    float t  = (alpha - lo) / (hi - lo);

    // Perform interpolation.
    value.set(keys[i-1]);
    value.interpolate(keys[i], t);
  }


  public BAlphaValue getStartValue()
  {
    if (keys.length <= 0)
      throw new RuntimeException(
        "Can't get starting value - no keys have been set.");

    return keys[0].copy();
  }


}