package roboforge.animation.alpha.map;


public class BLinearMap implements BAlphaMap
{
  // BLinearMap linearly maps a time interval
  // onto an alpha interval.


  public static final boolean INFINITE = true;
    // Constant for infinite valid interval.

  public static final boolean FINITE = false;
    // Constant for finite valid interval.


  protected float t0, t1;
    // Time interval for the linear map.

  protected float a0, a1;
    // Alpha values at either end of time interval.

  protected boolean isInfinite;
    // Whether times outside t0 and t1 are valid.



  public BLinearMap(float duration)
    { this(0, duration, 0, 1); }

  public BLinearMap(float t0, float t1)
    { this(t0, t1, 0, 1); }

  public BLinearMap(float t0, float t1, float a0, float a1)
    { this(t0, t1, a0, a1, FINITE); }

  public BLinearMap(float duration, boolean isInfinite)
    { this(0, duration, 0, 1, isInfinite); }

  public BLinearMap(float t0, float t1, boolean isInfinite)
    { this(t0, t1, 0, 1, isInfinite); }
    
  public BLinearMap(float t0, float t1, float a0, float a1, boolean isInfinite)
  {
    setInfinite(isInfinite);
    setTimeInterval(t0, t1);
    setAlphaInterval(a0, a1);
  }

  public void setInfinite(boolean isInfinite)
    { this.isInfinite = isInfinite; }

  public boolean isInfinite()
    { return isInfinite; }


  public void setTimeInterval(float t0, float t1)
  {
    // Check time interval parameters...
    if (t1 < t0)
      throw new IllegalArgumentException("t1 must be >= t0");

    this.t0 = t0;
    this.t1 = t1;
  }

  public void setAlphaInterval(float a0, float a1)
  {
    // Check a0 parameter value...
    if (a0 < 0 || a0 > 1)
      throw new IllegalArgumentException("Illegal alpha value for a0: "+ a0);

    // Check a1 parameter value...
    if (a1 < 0 || a1 > 1)
      throw new IllegalArgumentException("Illegal alpha value for a1: "+ a1);

    this.a0 = a0;
    this.a1 = a1;
  }


  public float alpha(float t)
  {
    if (t <= t0)
      return a0;
    else if (t >= t1)
      return a1;
    else
    {
      // Normalize t to [0..1]
      t = (t - t0) / (t1 - t0);

      // Return interpolated alpha.
      return a0 * (1-t) + a1 * t;
    }
  }

  public boolean isValidTime(float t)
  {
    if (isInfinite)
      return true;
    else
      return (t >= t0) && (t <= t1);
  }

}