package roboforge.animation.time;

import roboforge.animation.*;
import roboforge.util.timing.*;



public class BScheduledAnimation extends BTimeAnimation
{

  // A BScheduledAnimation controls the animation speed of a child animation.
  // The child receives animate() calls at a rate no higher than the
  // specified 'delay' in ms.

  // NOTE: if the scheduled animation itself receives animate() calls
  //       infrequently, the child rate may be slower than expected.


  protected static float DEFAULT_FPS = 10f;
    // Default delay, in seconds.


  protected BAnimation child;
    // The animation to be scheduled.

  protected float period;
    // Delay in seconds desired between child animate() calls.

  protected float scheduled;
    // Scheduled time for next child frame.



  public BScheduledAnimation()
    { this(null, DEFAULT_FPS); }

  public BScheduledAnimation(BAnimation child)
    { this(child, DEFAULT_FPS); }

  public BScheduledAnimation(long periodMillis)
    { this(null, periodMillis); }

  public BScheduledAnimation(BAnimation child, long periodMillis)
    { this(child, 1f / ((float) periodMillis / 1000f)); }


  public BScheduledAnimation(BAnimation child, float fps)
  {
    super();
    this.child = child;
    this.period = 1f / fps;
  }


  public void setChild(BAnimation a)
    { this.child = a; }

  public void setFPS(float fps)
    { this.period = 1f / fps; }


  public void start(float t)
  {
    if (child != null)
      child.start();

    scheduled = 0f;
  }


  public boolean animate(float t, float dt)
  {
    if (child == null)
      return false;

    if (t >= scheduled)
    {
      scheduled = t + period;
      return child.animate();
    }

    return true;
  }


  public void stop(float t)
  {
    if (child != null)
      child.stop();
  }

}