package roboforge.math;

import roboforge.BGlobal;
import roboforge.BDebug;
import java.text.*;
import javax.vecmath.*;



public class BMath
{

  // String conversion routines...
  // -------------------------------------------------------------

  protected static final NumberFormat number
    // Static formatting object.
    = NumberFormat.getInstance(BGlobal.getLocale());

  public static Number parse(String s)
  {
    try
      { return number.parse(s); }
    catch (ParseException e)
      { return null; }
  }

  public static String format(long n)
    { return number.format(n); }

  public static String format(double n)
    { return number.format(n); }

  public static String format(double n, int dp)
    { return number.format(round(n, dp)); }



  // Number manipulation...
  // -------------------------------------------------------------

  protected static final double[] POWERS;
    // Powers of 10 to the i, where i is array index.

  protected static final int NUM_POWERS = 20;
    // Number of entries in powers array.

  static
    // Static initializer for POWERS.
  {
    POWERS = new double[NUM_POWERS];
    POWERS[0] = 1;

    for (int i=1; i<NUM_POWERS; i++)
      POWERS[i] = POWERS[i-1] * 10;
  }


  public static double round(double number, int decimalPlaces)
  {
    // Check for bad argument...
    if ((decimalPlaces < 0) || (decimalPlaces >= NUM_POWERS))
      throw new IllegalArgumentException(
        "decimalPlaces: "+ decimalPlaces+ " requested, "+
        "[0.."+ (NUM_POWERS-1)+ "] allowed.");

    double m = POWERS[decimalPlaces];
    return Math.round(number * m) / m;
  }

  public static long round(double number)
    { return Math.round(number); }




  // Vector math...
  // -------------------------------------------------------------

   public static double[] base2 =
   	{ 0, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, 65536 };

   public static double getNearestEnclosingPowerOf2(double n)
   {
      if (n > base2[base2.length - 1])
        BDebug.assert(false, "BMath getNearestEnclosingPowerOfTwo does not support n = " + n);

       for (int i = base2.length-1; i >=0; i-- )
       {
          if (base2[i] < n)
            return base2[i + 1] ;
       }

       return 0;

   }



  // Random number generation...
  // -------------------------------------------------------------

  public static double random(double min, double max)
    { return min + (Math.random() * (max-min)); }

  public static float random(float min, float max)
    { return min + (float) (Math.random() * (max-min)); }

  public static int random(int min, int max)
    { return min + (int) Math.round(Math.random() * (max-min)); }

  public static long random(long min, long max)
    { return min + Math.round(Math.random() * (max-min)); }

}
