package roboforge.math;

import javax.vecmath.*;
import javax.media.j3d.PickRay;

public class BPlane3d
{
  // Internal variables...
  // -------------------------------------------------------------

  protected Point3d  p;   // Point on plane.
  protected Vector3d n;   // Normal to plane.


  // Working variables...
  // -------------------------------------------------------------

  protected Vector3d r0;  // Vector from origin to p.


  // Constants
  // -------------------------------------------------------------

  protected static final double TOLERANCE = 0.000001;


  // Constructor.
  // -------------------------------------------------------------

  public BPlane3d(Point3d p, Vector3d n)
  {
    this.p = p;
    this.n = n;

    r0 = new Vector3d(p);
  }

  public BPlane3d(Vector3d n)
    { this(new Point3d(), n); }

  public BPlane3d()
    { this(new Point3d(), new Vector3d(0,1,0)); }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setReferencePoint(Point3d p)
  {
    this.p = p;
    r0 = new Vector3d(p);
  }

  public void setReferencePoint(Vector3d r0)
  {
    this.r0 = r0;
    p = new Point3d(r0);
  }

  public Point3d getReferencePoint()
    { return p; }


  public void setNormal(Vector3d n)
    { this.n = n; }

  public Vector3d getNormal()
    { return n; }

  public String toString()
    { return "BPlane3d(p:"+p+ ", n:"+ n+ ")"; }


  // Intersection functions...
  // -------------------------------------------------------------

  public boolean intersects(PickRay ray)
  {
    double dist = 0.0;
    Point3d loc = new Point3d();
    return intersects(ray, dist, loc);
  }

  public boolean intersects(PickRay ray, double dist, Point3d loc)
  {
    Point3d  o = new Point3d();
    Vector3d d = new Vector3d();

    ray.get(o, d);

    double n_dot_d = n.dot(d);

    // Handle cases where ray is parallel to plane...
    // ..............................................

    double delta = n_dot_d;

    if (delta < 0) delta = -delta;
    if (delta < TOLERANCE)
    {
      Vector3d r = new Vector3d(o);

      // Pick ray is parallel...
      delta = n.dot(r) - n.dot(r0);

      if (delta < 0) delta = -delta;
      if (delta < TOLERANCE)
      {
        // Ray is collinear with plane.
        // Return ray-origin as intersect pt.
        loc.set(o);
        dist = 0.0;
        return true;
      }
      else
        return false;
    }

    // ..............................................

    Vector3d po = new Vector3d(o.x - p.x, o.y - p.y, o.z - p.z);

    dist = (n.dot(r0) - n.dot(po)) / n_dot_d;

    loc.x = o.x + dist * d.x;
    loc.y = o.y + dist * d.y;
    loc.z = o.z + dist * d.z;

    return true;
  }

}
