package roboforge.net.client;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.net.*;

import java.io.*;
import java.net.*;

import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;

import iaik.security.random.SecRandom;



public class BServerTarget extends BTarget
{

  protected URL url;

  // The Original URL that was set when the server target was created
  // Allows us to refer to the original URL when adding the type of request
  // to the URL as used for server side processing of the request.
  protected URL urlOrig;

  public BServerTarget() throws BNetException
  {
    this(BGlobal.URLSGATEWAY);

    /*super(BGlobal.PUBLIC_KEY_SERVER);
      super(BGlobal.PUBLIC_KEY_CLIENT);

      BServerTarget(BGlobal.URLSGATEWAY);
      this.url = getURL(BGlobal.URLSGATEWAY);

      urlOrig = getURL(BGlobal.URLSGATEWAY);

      getURL("http://localhost:8080/servlet/roboforge.net.server.BLoopbackServlet");
    */
  }

  public BServerTarget(String urlString) throws BNetException
  {
    super(BGlobal.PUBLIC_KEY_SERVER);
    // super(BGlobal.PUBLIC_KEY_CLIENT);

    this.url =
      getURL(urlString);
      // getURL("http://localhost:8080/servlet/roboforge.net.server.BLoopbackServlet");

    urlOrig =
      getURL(urlString);
  }

  public URL getURL()
    { return url; }

  protected URL getURL(String url)
  {
    try
      { return new URL(url); }
    catch (MalformedURLException e)
      { throw new IllegalArgumentException("Malformed URL: "+ url); }
  }

  public void setURL (URL url)
    { this.url = url;}

  public void resetURLToType (String type)
  {
    // Sets the URL to be the original URL with the type
    // Appended to the end as the beginning of a query

    url = this.getURL(urlOrig.toExternalForm() + "?" + type);
  }

  public Key getSymmetricKey()
  {
    if (symmetricKey == null)
      try
      {

        // If the Key Exists then load it from disk
        File keyFile = new File(BGlobal.SYMMETRIC_KEY_CLIENT);
        if (keyFile.exists())
        {
          BDebug.print("Loading Key from disk");
          //BDebug.print("Loading Start Time: " + System.currentTimeMillis());
          ObjectInputStream i = new ObjectInputStream(new
          FileInputStream(BGlobal.SYMMETRIC_KEY_CLIENT));

          symmetricKey = (Key)i.readObject();
          BDebug.print("Finished loading key from disk");
          //BDebug.print("Finished Loading Time: " + System.currentTimeMillis());
        }
        else
        {
          // Key File Doesn't exist so generate it and store it
          BDebug.print("Start Gen Time: " + System.currentTimeMillis());
          symmetricKey = genSymmetricKey();
          BDebug.print("End Gen Time: " + System.currentTimeMillis());

          ObjectOutputStream o = new ObjectOutputStream(new
          FileOutputStream(BGlobal.SYMMETRIC_KEY_CLIENT));
          o.writeObject(symmetricKey);
          o.flush();
          o.close();
        }

      }
      catch (Exception e)
      {
        BDebug.print("Error: BServerTarget.getSymmetricKey: " + e.getMessage());
      }
    return symmetricKey; 
  }

  protected Key genSymmetricKey() throws BNetException
  {
    BDebug.seedTimer();
    
    try
    {
      BDebug.print("Generating Triple-DES key...");
      KeyGenerator kg = 
        KeyGenerator.getInstance("3DES");

      kg.init(168);
      
      Key key = kg.generateKey();
      
      BDebug.printTime("Triple-DES key generated.");
      return key;
    }
    catch (NoSuchAlgorithmException e)
      { throw new BNetException("Can't generate Triple-DES key."); }
  }


  public byte[] getSymmetricIV()
  { 
    if (symmetricIV == null)
      try
      {

        // If the Key Exists then load it from disk
        File keyFile = new File(BGlobal.SYMMETRICIV_KEY_CLIENT);
        if (keyFile.exists())
        {
          BDebug.print("Loading KeyIV from disk");
          //BDebug.print("Loading Start Time: " + System.currentTimeMillis());
          ObjectInputStream i = new ObjectInputStream(new
          FileInputStream(BGlobal.SYMMETRICIV_KEY_CLIENT));

          symmetricIV = (byte[])i.readObject();
          BDebug.print("Finished loading keyIV from disk");
          //BDebug.print("Finished Loading Time: " + System.currentTimeMillis());
        }
        else
        {
          // Key File Doesn't exist so generate it and store it
          //BDebug.print("Start GenIV Time: " + System.currentTimeMillis());
          symmetricIV = genSymmetricIV();
          //BDebug.print("End GenIV Time: " + System.currentTimeMillis());

          ObjectOutputStream o = new ObjectOutputStream(new
          FileOutputStream(BGlobal.SYMMETRICIV_KEY_CLIENT));
          o.writeObject(symmetricIV);
          o.flush();
          o.close();
        }
      }
      catch (Exception e)
      {
        BDebug.print("Error: BServerTarget.getSymmetricKey: " + e.getMessage());
      }

    //if (symmetricIV == null)
      //symmetricIV = genSymmetricIV();
    
    return symmetricIV; 
  }
  
  protected byte[] genSymmetricIV()
  {
    BDebug.print("Generating SymmetricIV...");
    byte[] iv = new byte[8];
    
    SecureRandom random = 
      SecRandom.getDefault();
    
    random.nextBytes(iv);
    BDebug.print("Finished Generating SymmetricIV...");
    return iv;
  }
  
} 