package roboforge.net.data;

import roboforge.BBattleRecorder;
import roboforge.BDebug;
import roboforge.BStats;
import roboforge.SServerAccess;
import roboforge.util.data.*;

import java.io.*;
import java.util.zip.*;



public class BFightRecordData extends BAbstractAtom
{

  // Serialization ID.
  // -----------------------------------------------------------

  static final long serialVersionUID = -1L;


  // Constructor...
  // -------------------------------------------------------------

  public BFightRecordData(long id)
    { super(id); }


  // Type definition...
  // -------------------------------------------------------------

  public int getType()
    { return BDataTypes.FIGHT_RECORD; }


  // Data fields...
  // -------------------------------------------------------------

  public byte[] fightRecording;
  public byte[] fightStats;

  
  // Accessor methods...
  // -------------------------------------------------------------

  public void save(String filename) throws IOException
  {
    BBattleRecorder recording = getRecording();
        
    if (recording != null)
    {
      recording.saveToFile(filename +".rec", null, null);
      recording.cleanse();
      recording = null;
    }

    
    if (fightStats == null)
      return;

    byte[] stats = getStats();
    
    FileOutputStream fo =
      new FileOutputStream(filename+ ".sta");
    
    BufferedOutputStream bo =
      new BufferedOutputStream(fo);
    
    bo.write(stats);
    bo.close();
  }
  
  
  public BBattleRecorder getRecording() throws IOException
  {
    if (fightRecording == null)
      return null;

    try
    {
      ByteArrayInputStream bytes =
        new ByteArrayInputStream(fightRecording);

      GZIPInputStream unzip =
        new GZIPInputStream(bytes);
          
      ObjectInputStream in = 
        new ObjectInputStream(unzip);

      BBattleRecorder result =
        (BBattleRecorder) in.readObject();
       
      in.close();
      
      return result;
    }
    catch (IOException e)
      { throw e; }
    catch (Exception e)
      { throw new RuntimeException(""+ e); }
  }
  
  
  public byte[] getStats() throws IOException
  {
    if (fightStats == null)
      return null;
    
    try
    {
      return SServerAccess.gzipInflateArray(fightStats);
      
      /*
      ByteArrayInputStream bytes =
        new ByteArrayInputStream(fightStats);

      GZIPInputStream unzip =
        new GZIPInputStream(bytes);
          
      ObjectInputStream in = 
        new ObjectInputStream(unzip);

      byte[] result =
        (byte[]) in.readObject();
       
      in.close();
      
      return result;
      */
    }
    catch (IOException e)
      { throw e; }
    catch (Exception e)
      { throw new RuntimeException(""+ e); }
  }
  
  
} 