package roboforge.particles.systems;

import roboforge.BComponent;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BTextures;

import roboforge.particles.*;
import roboforge.particles.emitters.*;
import roboforge.particles.forces.*;
import roboforge.particles.forms.*;
import roboforge.particles.looks.*;
import roboforge.particles.physics.*;

import roboforge.scalability.*;

import java.util.*;
import javax.vecmath.*;
import javax.media.j3d.*;




public class BComponentSystem extends BParticleSystemScalable
{

  protected BComponent component;
    // The component that this system's emitter attaches to.

  protected BSystemEmitter3D emitter;
    // The emitter, attached to component.


  public BComponentSystem(int min, int target, int max)
    { this(null, null, min, target, max); }

  public BComponentSystem(BScalabilityMetric metric, int min, int target, int max)
    { this(null, metric, min, target, max); }

  public BComponentSystem(BComponent c, BScalabilityMetric metric, int min, int target, int max)
  {
    super(min, target, max);

    emitter = new BSystemEmitter3D(this);
    addAspect(emitter);

    if (c != null)
      setComponent(c);
  }


  public void setComponent(BComponent c)
  {
    this.component = c;

    if (c.getChassisLock())
      emitter.setParent(c.getLiveNode());
    else
      emitter.setParent(c.getDirectLiveParentNode());
  }


  protected static final float ONE_THIRD = 1f / 3f;

  protected float getComponentSize()
  {
    // Returns a sizing value for this component.
    // Should give a number somewhere between the
    // shortest and longest dimension of the component.
    // Used to help set an appropriate size for
    // particles.

    if (component == null)
      return 0f;

    return (float)
      Math.pow(component.getVolume(), ONE_THIRD);
  }

}