package roboforge.particles.systems;

import roboforge.BComponent;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BTextures;

import roboforge.math.BMath;

import roboforge.particles.*;
import roboforge.particles.emitters.*;
import roboforge.particles.forces.*;
import roboforge.particles.forms.*;
import roboforge.particles.looks.*;
import roboforge.particles.physics.*;

import roboforge.scalability.*;

import javax.vecmath.*;
import javax.media.j3d.*;



public class BDebrisSystem extends BComponentSystem
{

  // Textures used in this particle system...

  protected static Texture spark
    = BTextures.getTexture(BGlobal.TEXTUREPATH +
      "internal/particles/particle_spark.gif");

  protected static Texture starburst
    = BTextures.getTexture(BGlobal.TEXTUREPATH +
      "internal/particles/particle_starburst.gif");

  protected static Texture[] textures
    = { spark, starburst };



  protected static float SCALE
    = BGlobal.WORLD_TO_CANVAS_SCALE;

  protected float intensity = 1f;
  protected float lifespan  = 1f;

  protected BEmissionPatternPeriodic pattern;



  public BDebrisSystem(BranchGroup parent, BScalabilityMetric metric)
  {
    super(metric, 1, 5, BGlobal.isParticleQualityHigh() ? 20 : 10);

    pattern = new BEmissionPatternPeriodic(1f);
    emitter.setEmissionPattern(pattern);
    emitter.setSharedEmitter(new BDebrisEmitter(emitter));

    BSystemPhysicsForces physics = new BSystemPhysicsForces();
    physics.setForces(new BDragForce(1f));

    BSystemForm form;

    if (BGlobal.isParticleQualityHigh())
      form = new BSystemFormCard(parent);
    else
      form = new BSystemFormCluster(parent);


    BSystemLookShared look = new BSystemLookShared();
    look.setSharedLook(new BDebrisLook());

    addAspect(physics);
    addAspect(form);
    addAspect(look);

    // Preload particles into system...
    preloadParticles();

    setIntensity(1f);
  }

  public String getName()
  	{ return "BDebrisSystem"; }


  public void setIntensity(float intensity)
  {
    this.intensity = intensity;

    // Determine how many particles to release...
    int n = 2 + (int) Math.round(8 * intensity);
    pattern.setN(n);

    // Determine particle lifespan...
    lifespan = 0.5f + (intensity / 2f);
  }

  public void setDuration(float d)
    { pattern.setDuration(d); }


  protected class BDebrisEmitter
    extends BParticleEmitter3D
  {
    protected float BASE_SPEED = 2f;

    public BDebrisEmitter(BSystemEmitter3D parent)
      { super(parent); }

    public void start(BParticle p)
    {
      Transform3D t = getEmitterTransform();

      p.p.set(0f, 0f, 0f);
      t.transform(p.p);
      p.oldp.set(p.p);

      float v = (2f + intensity * BASE_SPEED) * SCALE;

      p.v.set(rnd(v), rnd(v), rnd(v));
      // t.transform(p.v);

      p.setMass(1f);
      p.setLifeSpan(lifespan);
    }

    protected float rnd(float range)
      { return ((float) Math.random() - 0.5f) * range; }
  }



  protected class BDebrisLook
    extends    BParticleAspectAdapter
    implements BParticleLook
  {
    protected float size;

    public void start(BParticle p)
    {
      size = (0.1f + intensity * 0.2f) * SCALE;
      p.form.setSize(size);

      int n = BMath.random(1, textures.length) - 1;
      p.form.setTexture(textures[n]);
      p.form.setColor(BGlobal.WHITE_4F);
    }

    public void update(BParticle p, float t, float dt)
      { p.form.setSize(size * (1f - p.alpha)); }
  }


  // public boolean isAlive()
  //  { return (t < pattern.getDuration()); }

}
