package roboforge.particles.systems;

import roboforge.BComponent;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BTextures;

import roboforge.particles.*;
import roboforge.particles.emitters.*;
import roboforge.particles.forces.*;
import roboforge.particles.forms.*;
import roboforge.particles.looks.*;
import roboforge.particles.physics.*;

import roboforge.scalability.*;

import java.util.*;
import javax.vecmath.*;
import javax.media.j3d.*;



public class BFlameBallSystem extends BComponentSystem
{

  protected static float SCALE
    = BGlobal.WORLD_TO_CANVAS_SCALE;

  protected static List flameTextures =
    BTextures.getTextureSet(BGlobal.IMAGEPATH +
      "animation/explo_5_fire/5", 4, ".gif");


  protected BEmissionPatternPeriodic pattern;



  public BFlameBallSystem(BranchGroup parent, BScalabilityMetric metric)
    {  this(parent, metric, null); }

  public BFlameBallSystem(BranchGroup parent, BScalabilityMetric metric, BComponent c)
  {
    super(metric, 1, 1, 1);

    pattern = new BEmissionPatternPeriodic(0.1f, 1f);
    emitter.setEmissionPattern(pattern);
    emitter.setSharedEmitter(new BFlameBallEmitter(emitter));

    BSystemPhysicsForces physics = new BSystemPhysicsForces();
    physics.setForces(new BDragForce(1.0f));

    BSystemForm form;

    if (BGlobal.isParticleQualityHigh())
      form = new BSystemFormCard(parent);
    else
      form = new BSystemFormCluster(parent);


    BSystemLook look = new BFlameBallSystemLook();

    addAspect(physics);
    addAspect(form);
    addAspect(look);

    // Preload particles into system...
    preloadParticles();
    
    if (c != null)
      setComponent(c);
  }

  public String getName()
  	{ return "BFlameballSystem"; }


  public void setDuration(float d)
    { pattern.setDuration(d); }


  protected class BFlameBallEmitter extends BParticleEmitter3D
  {
    protected float BASE = 1.5f * SCALE; // 2 * SCALE;

    public BFlameBallEmitter(BSystemEmitter3D parent)
      { super(parent); }

    public void start(BParticle p)
    {
      Transform3D t = getEmitterTransform();

      p.p.set(0f, 0f, 0f);
      t.transform(p.p);

      p.oldp.set(p.p);

      p.v.set(rnd(BASE), rnd(BASE*2f), rnd(BASE));
      t.transform(p.v);

      p.setMass(1f);
      p.setLifeSpan(1f);
    }

    protected float rnd(float range)
      { return ((float) Math.random() - 0.5f) * range; }
  }




  protected class BFlameBallLook
    extends BParticleLookAnimated
  {

    public BFlameBallLook(List ts)
      { super(ts); }

    protected float scale;

    public void start(BParticle p)
    {
      super.start(p);

      // Calculate a good size for the explosion...
      scale = Math.min(getComponentSize(), 1f) * 5f * SCALE;
      p.form.setSize(scale);
      p.form.setColor(BGlobal.WHITE_4F);
    }
  }


  protected class BFlameBallSystemLook
    extends    BSystemAspectAdapter
    implements BSystemLook
  {

    public BParticleAspect createParticleAspect(BParticle p)
      throws BParticleSystemException
      { return new BFlameBallLook(flameTextures); }
  }


  // public boolean isAlive()
  //   { return t < pattern.getDuration(); }

}