package roboforge.particles.systems;

import roboforge.BComponent;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BTextures;

import roboforge.particles.*;
import roboforge.particles.emitters.*;
import roboforge.particles.forces.*;
import roboforge.particles.forms.*;
import roboforge.particles.looks.*;
import roboforge.particles.physics.*;

import roboforge.math.*;
import roboforge.scalability.*;

import javax.vecmath.*;
import javax.media.j3d.*;



public class BVortexSystem extends BComponentSystem
{

  protected static Texture spark
    = BTextures.getTexture(BGlobal.TEXTUREPATH +
      "internal/particles/particle_spark.gif");

  protected static float SCALE
    // = 0.5f;
    = BGlobal.WORLD_TO_CANVAS_SCALE;

  protected float duration  = 4f;
  protected float halftime  = 2f;
  protected float size			= 2.5f;


  // Color scheme...
  protected Color4f base = new Color4f(0.4f, 0.8f, 1, 1);
  protected Color4f vary = new Color4f(0.0f, 0.4f, 0, 0);


  protected BEmissionPatternPeriodic pattern;
  protected BVortexForces forces;


  public BVortexSystem
    (BranchGroup parent, BScalabilityMetric metric)
  {
    super(metric, 1, 10, 20);

    pattern = new BEmissionPatternPeriodic(2, 0.1f, duration);
    emitter.setEmissionPattern(pattern);
    emitter.setSharedEmitter(new BVortexEmitter(emitter));

    BSystemPhysicsForces physics = new BSystemPhysicsForces();
    forces = new BVortexForces();
    physics.setForces(forces);

    BSystemForm form = new BSystemFormCluster(parent);

    BSystemLookShared look = new BSystemLookShared();
    look.setSharedLook(new BVortexLook());

    addAspect(physics);
    addAspect(form);
    addAspect(look);
    addAspect(emitter);

    // Preload particles into system...
    preloadParticles();

    setDuration(4f);
  }

  public String getName()
  	{ return "BVortexSystem"; }



  /*
  public void setComponent(BComponent c)
  {
  	super.setComponent(c);

    // Calculate a good size for the vortex...
    setSize(Math.max(1.5f, Math.min(getComponentSize(), 2f)));
  }
  */


  public void setSize(float size)
  {
  	this.size = size;
    forces.setSize(size);
  }

  public void setDuration(float d)
  {
  	this.duration = duration;
    this.halftime = duration / 2f;
    pattern.setDuration(duration);
  }

  public void setColors(Color4f base, Color4f vary)
  {
  	this.base.set(base);
    this.vary.set(vary);
  }



  protected class BVortexEmitter
    extends BParticleEmitter3D
  {
    protected float KILL_RADIUS = 0.5f * SCALE;

    protected Vector3f radius
    	= new Vector3f();


    public BVortexEmitter(BSystemEmitter3D parent)
      { super(parent); }

    public void start(BParticle p)
    {
    	Point3f origin = getOrigin();

      BMath3D.randomPtOnSphere(origin, size * SCALE, p.p);
      p.oldp.set(p.p);

      // Calculate initial velocity.
      p.v.sub(origin, p.p);
      p.v.cross(p.v, BMath3D.UP);
      p.v.scale(2f);

      // p.v.set(spherev);

      p.setMass(1f);
      // p.setLifeSpan(duration);
      p.setLifeSpan(1f);
    }

    public void update(BParticle p, float t, float dt)
    {
	   	radius.sub(getOrigin(), p.p);

    	if (radius.length() < KILL_RADIUS)
      	p.kill();
    }
  }



  protected class BVortexLook
    extends    BParticleAspectAdapter
    implements BParticleLook
  {
    protected float size;

    protected Color4f color
    	= new Color4f();

    public void start(BParticle p)
    {
      size = 0.5f * SCALE;
      p.form.setSize(size);
      p.form.setTexture(spark);

      BMath3D.randomize(base, vary, color);
      p.form.setColor(color);
    }

    public void update(BParticle p, float t, float dt)
      { p.form.setSize(size * (1f - p.alpha)); }
  }



  protected class BVortexForces extends BGravityPointForce
  {
    public BVortexForces()
    {
      super(1, 1f);
      setSize(size);
    }

    public void setSize(float f)
    {
    	float m = size * 450f;
    	setMass(m*m);
    }

    public void update(float t)
    {
    	super.update(t);
    	setOrigin(emitter.getOrigin());
    }
  }

  public boolean isAlive()
   { return (time <= duration) ? true : super.isAlive(); }
}

