package roboforge.ui;

import roboforge.*;
import roboforge.ui.components.BUIResources;
import roboforge.ui.dnd.*;
import roboforge.ui.events.*;
import roboforge.ui.modes.*;
import roboforge.util.graphics.*;

import java.util.*;
import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;

import javax.vecmath.*;
import javax.media.j3d.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import javax.swing.event.SwingPropertyChangeSupport;

import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.picking.*;




public class B3DCanvas
  extends
    Canvas3D
  implements
    BActivateable
{

  // Class variables.
  // -------------------------------------------------------------

  protected final BoundingSphere BOUNDS = new BoundingSphere(BGlobal.INFINITE_BOUNDS);

  protected static final double   BACKCLIPDIST  = 1000.0;
  protected static final double   FRONTCLIPDIST = 1.0;

  protected static final int      NUM_VIEW_TRANSFORMS = 4;
  protected static final int      TRANSFORM_BOTORIGIN = 0;
  protected static final int      TRANSFORM_ROTABOUTY = 1;
  protected static final int      TRANSFORM_ROTABOUTX = 2;
  protected static final int      TRANSFORM_TRANSLATE = 3;

  protected static final double   VIEW_TRANSLATION_MIN   = 20.0;
  protected static final double   VIEW_TRANSLATION_MAX  = 2000.0;


  
  // Member variables.
  // -------------------------------------------------------------

  protected BMouseMultiBehavior   mouseBehavior;

  protected BComponent              selectedComponent = null;
  protected BConnectPoint           selectedConnectPt = null;
  protected BZone                   selectedZone      = null;

  // Support class for handling activation listeners...
  protected BActivationSupport activation
    = new BActivationSupport(this);

  // Support class for handling property-change listeners...
  protected SwingPropertyChangeSupport propertyManager =
    new SwingPropertyChangeSupport(this);
  
  
  // Universe Nodes
  protected SimpleUniverse        universe;
  protected View                  view;
  protected BranchGroup           root;
  protected BranchGroup           pickGroup;

  protected Background            background;

  protected BranchGroup           behaviorGroup;
  protected Hashtable             behaviorTable = new Hashtable(5);

  // Universe Appearance
  protected AmbientLight          ambientLight;
  protected DirectionalLight      dirLight;

  // Temporary variables
  protected BranchGroup           tempLiveNode;
  protected BranchGroup           scene;

  // Collections for properly deallocating shapes/components.
  protected Vector componentList = new Vector();
  protected Vector shapeList     = new Vector();

  // Bot orientation fields.
  protected Vector3d viewOrigin      = null;
  protected Vector3d viewRotation    = null;
  protected Vector3d viewTranslation = null;

  // DnD helpers.
  protected DragGestureRecognizer  recognizer;
  protected DragSource              dragSource;

  protected DnDGestureListener  dragGestureListener;
  protected DnDSourceListener   dragSourceListener;

  // Picking helper.
  protected PickCanvas pickCanvas;
  protected boolean dragActive = false;


  // Cursors...

  protected static Cursor defaultCursor =
    new Cursor(Cursor.DEFAULT_CURSOR);

  private static Cursor linkCursor =
    BGraphics.getCursor("link_cursor.gif", 0, 0);




  // Constructor.
  // -------------------------------------------------------------

  public B3DCanvas(BUIResources res, String resourceKey)
  {
    super(
      GraphicsEnvironment
        .getLocalGraphicsEnvironment()
        .getDefaultScreenDevice()
        .getBestConfiguration(new GraphicsConfigTemplate3D()));


    // Initialize the canvas component's ui...
    initializeUI(res, resourceKey);

    // Create the canvas' scene-graph.
    createSceneGraph();

    // Create the 'pick-group' - the BG for adding pickable components.
    pickGroup = new BranchGroup();
    pickGroup.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    pickGroup.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);

    // Add the final node, the 'pick group', to the scene.
    Group g = getPickGroupParent();
    g.addChild(pickGroup);

    // Set up the view.
    ViewingPlatform newP = new ViewingPlatform(NUM_VIEW_TRANSFORMS);
    newP.getViewPlatform().setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ) ; // to allow BBotCanvas to pick closest connect Point

    Viewer newV = new Viewer(this);

    view = newV.getView();
    //view.setDepthBufferFreezeTransparent(true);
    view.setFrontClipDistance(FRONTCLIPDIST);
    view.setBackClipDistance(BACKCLIPDIST);

    universe = new SimpleUniverse(newP, newV);
    universe.addBranchGraph(root);

    
    // Set up tooltips...
    // .........................................
    
    // HACK: Add key relay listener to make Swing work.
    //       See KeyRelay listener class for details.
    addKeyListener(new KeyRelayListener());

    initToolTipSystem();

    
    // Set up DnD...
    // .........................................

    dragSource = DragSource.getDefaultDragSource();
    //dragSource = new DragSource();

    dragGestureListener = new DnDGestureListener();
    dragSourceListener = new DnDSourceListener();

  
    recognizer = dragSource.createDefaultDragGestureRecognizer
    (
         this, // component where drag originates
         DnDConstants.ACTION_MOVE, // actions
         dragGestureListener
    );
    //recognizer = new BDragGestureRecognizer
    //  (dragSource, this, DnDConstants.ACTION_MOVE, dragGestureListener);

    //recognizer.setEnabled(false);
    // recognizer.setDragAction(DnDConstants.ACTION_MOVE);

    
    // Set up picking...
    // .........................................

    pickCanvas = new PickCanvas(this, getPickGroup());
    pickCanvas.setMode(PickCanvas.GEOMETRY);
    pickCanvas.setTolerance(0f);
    
  }

  protected void finalize()
    { BDebug.print( this.getClass().toString() + ".finalize()"); }


  protected DirectionalLight createDirLight()
  {
    // Create the directional lights
    Vector3f dir = new Vector3f(0f, -1.0f, -0.4f);
    DirectionalLight dirLight = new DirectionalLight(BGlobal.CANVASLIGHT, dir);
    dirLight.setInfluencingBounds(BOUNDS);

    return dirLight;
  }


  protected void createSceneGraph()
  {
    // Root
    root = new BranchGroup();

    background = new Background(getBackground3f());
    background.setCapability(Background.ALLOW_COLOR_WRITE);
    background.setApplicationBounds(BOUNDS);
    root.addChild(background);

    // Ambient Light
    ambientLight = new AmbientLight(BGlobal.AMBIENT_DIRLIGHT);
    ambientLight.setInfluencingBounds(BOUNDS);

    // Create the directional lights
    dirLight = createDirLight();

    // Create the behavior group.
    behaviorGroup = new BranchGroup();
    behaviorGroup.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    behaviorGroup.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);

    // Create scene graph from nodes.
    root.addChild(behaviorGroup);
    root.addChild(ambientLight);
    root.addChild(dirLight);

    setupMouse();
  }

  protected void setupMouse()
  {
    // Setup behaviors...
    mouseBehavior = new BMouseListenerBehavior(this);
    addBehavior(mouseBehavior);

  }

  protected void removeMouse()
  {

    if (mouseBehavior instanceof BMouseListenerBehavior)
    {
      BMouseListenerBehavior castMouse = (BMouseListenerBehavior) mouseBehavior;

      this.removeMouseListener(castMouse);
      this.removeMouseMotionListener(castMouse);

    }

    removeBehavior(mouseBehavior);
    mouseBehavior.requestClose();
    mouseBehavior.cleanse();
    mouseBehavior = null;


  }

  protected Group getPickGroupParent()
    { return root; }


  // Initialization helper methods.
  // -------------------------------------------------------------

  public void initializeUI(BUIResources res, String key)
  {
    setBackground(Color.black); // BGlobal.colors.getDarkestShade());
    setForeground(BGlobal.colors.getLightShade());

    if (res != null)
      res.initComponent(this, key);
  }

  protected Color3f getBackground3f()
    { return BGlobal.colors.getExtraDarkShade3f(); }

  public void setBackground(Color3f newbackground)
    { background.setColor(newbackground); }


  // Activation methods...
  // -------------------------------------------------------------

  public boolean isActive()
    { return activation.isActive(); }

  public boolean isClosed()
    { return activation.isClosed(); }


  public void requestActivate()
    { activation.requestActivate(); }

  public void requestDeactivate()
    { activation.requestDeactivate(); }

  public void requestClose()
    { activation.requestClose(); }


  public void onActivate()
  {
    mouseBehavior.requestActivate();
    setViewFromParameters();
    activation.fireActivate();
  }

  public void onDeactivate()
  {
    mouseBehavior.requestDeactivate();
    activation.fireDeactivate();
  }

  public void setVisible(boolean vis)
  {
    if ( vis )
      startRenderer();
    else
      stopRenderer();

    super.setVisible(vis);
  }

  public void onClose()
  {
   // BDebug.print ("*" + this.getClass().toString() + ".onClose Called ");

    lock  = null;

    stopRenderer();
    view.stopView();
    view.removeCanvas3D(this);
    view = null;


    // Clean up all J3D threads...
    universe.removeAllLocales();
    universe.getViewer().setViewingPlatform(null);
    universe = null;

    propertyManager   = null;

    removeMouse();

    // Cleanup behaviors...
    behaviorTable.clear();
    behaviorTable = null;


    if (relayListener != null)
    {
      this.removeMouseListener(relayListener);
      this.removeMouseMotionListener(relayListener);
      relayListener = null;
    }

    selectedComponent = null;
    selectedConnectPt = null;
    selectedZone      = null;

    defaultCursor = null;
    universe = null;
    view = null;
    root = null;
    pickGroup = null;

    background = null;
    behaviorGroup = null;
    ambientLight = null;
    dirLight = null;
    tempLiveNode = null;
    scene = null;

    componentList = null;
    shapeList     = null;

    // Bot orientation fields.
    viewOrigin      = null;
    viewRotation    = null;
    viewTranslation = null;

    activation.fireClose();
    activation = null;

    // DnD helpers.
    //recognizer = null;
    dragSource = null;

    dragGestureListener = null;
    dragSourceListener = null;

    pickCanvas = null;


  }

  public void addActivationListener(BActivationListener l)
    { activation.addActivationListener(l); }

  public void removeActivationListener(BActivationListener l)
    { activation.addActivationListener(l); }


  // This method can be overridden by subclasses to return
  // alternative 3d scene graphs...

  protected BranchGroup cloneStructure3D(BComponent c)
    { return c.cloneStructure3D(); }



  // Component management methods.
  // -------------------------------------------------------------

  public void addComponent(BComponent c)
  {
    // Place component in the clearance list.
    componentList.add(c);

    // Cache Shapes for disposal (BUG HACK)
    tempLiveNode = cloneStructure3D(c);
    tempLiveNode.setCapability(Group.ALLOW_CHILDREN_WRITE);
    tempLiveNode.setCapability(BranchGroup.ALLOW_DETACH);
    tempLiveNode.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
    tempLiveNode.setCapability(Group.ALLOW_CHILDREN_EXTEND);
    tempLiveNode.setCapability(Group.ALLOW_BOUNDS_READ);

    // Get the Shape3D's, for garbage collection later.
    extractShapes(tempLiveNode);

    // New Live Node
    c.setLiveBotNode(tempLiveNode);

    // Add component to the scene.
    pickGroup.addChild(tempLiveNode);

    // Remember the component's BG [for detaching]...
    scene = tempLiveNode;

    //YLY null out tempLiveNode
    tempLiveNode = null;
  }

  public void removeComponent(BComponent c)
  {
    if (c == null)
      return;
      
    // Reset selection if 'c' is currently selected...
    if (c == selectedComponent)
      setSelectedComponent(null);

    componentList.remove(c);
    c.getLiveNode().detach();
    BShape3D s;

    for (int a = shapeList.size()-1; a>=0; a--)
    {
      s = (BShape3D) shapeList.get(a);
      if (s.getComponent() == c)
      {
        s.setAppearance(null);
        s.setGeometry(null);
        s.setComponent(null);
        shapeList.remove(s);
      }
    }

    c.cleanseLiveNodes();

  }

  protected void extractShapes(Node oldBranch)
  {
    if (oldBranch instanceof Group)
    {
      Group currentGroup = (Group) oldBranch;
      for (Enumeration e = currentGroup.getAllChildren(); e.hasMoreElements();)
      {
        Node newGroup = (Node) e.nextElement();
        extractShapes(newGroup);
      }
    }
    else if ((oldBranch instanceof BShape3D))
    {
      shapeList.add(oldBranch);
      oldBranch.setCapability(Shape3D.ALLOW_BOUNDS_READ);
    }
  }

  // Property change listeners
  // -------------------------------------------------------------

  public void addPropertyChangeListener(PropertyChangeListener listener)
    { propertyManager.addPropertyChangeListener(listener); }

  public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
    { propertyManager.addPropertyChangeListener(propertyName, listener); }

  public void removePropertyChangeListener(PropertyChangeListener listener)
    { propertyManager.removePropertyChangeListener(listener); }

  public void removePropertyChangeListener( String propertyName, PropertyChangeListener listener)
   { propertyManager.removePropertyChangeListener(propertyName, listener); }



  // Selection stuff
  // -------------------------------------------------------------

  public void setSelectedComponent(BComponent c)
    { setSelectedComponent(c, true, true, true, false, BGlobal.SELECTED_COLOR_3F); }

  public void setSelectedComponent(BComponent c, Color3f color)
    { setSelectedComponent(c, true, true, true, false, color); }
  
  public void setSelectedComponent(
    BComponent c,
    boolean showSelectBox,
    boolean showSelectColor,
    boolean showConnectPts,
    boolean showInactiveConnectPoints,
    Color3f selectBoxColor)
  {

    BComponent old = selectedComponent;
    selectedComponent = c;

    if ((old != c) && (old != null))
      old.setSelected(
        false,
        showSelectBox,
        showSelectColor,
        showConnectPts,
        showInactiveConnectPoints,
        selectBoxColor);

    if (c != null)
      c.setSelected(
        true,
        showSelectBox,
        showSelectColor,
        showConnectPts,
        showInactiveConnectPoints,
        selectBoxColor);

    // Tell interested parties about the change in selection.
    propertyManager.firePropertyChange("selectedComponent", old, c);
  }


  public BComponent getSelectedComponent()
  {
    return selectedComponent;
  }

  public void setSelectedConnectPt(BConnectPoint c)
  {
    if (c.isActive())
    {
      BConnectPoint old = getSelectedConnectPt();

      if (selectedComponent != null)
        selectedComponent.setConnectPointSelected(c);

      // Tell interested parties about the change in selection.
      propertyManager.firePropertyChange("selectedConnectPt", old, c);
    }
  }

  public BConnectPoint getSelectedConnectPt()
  {
    if (selectedComponent != null)
      return selectedComponent.getConnectPointSelected();
    else
      return null;
  }

  public void setSelectedZone(BZone z)
  {
    BZone old = selectedZone;
    selectedZone = z;

    BDebug.print("Canvas.setSelectedZone(OLD: "+old+ ", NEW: "+ z+ ")");

    if (old != null)
      old.setHighlight(false);

    if (z != null)
      z.setHighlight(true);

    // Tell interested parties about the change in selection.
    propertyManager.firePropertyChange("selectedZone", old, z);
  }

  public BZone getSelectedZone()
    { return selectedZone; }

  public BranchGroup getPickGroup()
    { return pickGroup; }


  // Accessor methods...
  // ----------------------------------------------------

  public void setMode(BModeMouse mode)
  {
    BModeMouse old = 
      mouseBehavior.getMode();

    mouseBehavior.setMode(mode);
    propertyManager.firePropertyChange("mode", old, mode);
  }


  public BModeMouse getMode()
    { return mouseBehavior.getMode(); }

  public void setCursor()
    { setCursor(defaultCursor); }



  // Accessors for bot orientation..
  // ----------------------------------------------------

  public javax.media.j3d.Locale getLocale3d()
    { return universe.getLocale(); }

  public void setViewRotation(Vector3d r)
    { viewRotation = r; }

  public Vector3d getViewRotation()
  {
    if (viewRotation == null)
      viewRotation = new Vector3d();
    return viewRotation;
  }


  public void setViewTranslation(Vector3d t)
  {
    viewTranslation = t;

    if ( viewTranslation != null)
    {
      if (viewTranslation.z > getViewTranslationMax())
        viewTranslation.z = getViewTranslationMax();

      if (viewTranslation.z < getViewTranslationMin())
        viewTranslation.z = getViewTranslationMin();
    }
  }

  public Vector3d getViewTranslation()
  {
    // TODO: Fix distance hack.
    if (viewTranslation == null)
      viewTranslation = new Vector3d(0.0, 0.0, -200.0);

    return viewTranslation;
  }
 
   // zoom accesor Methods
  public double getViewTranslationMax()
    { return VIEW_TRANSLATION_MAX; }

  public double getViewTranslationMin()
    { return VIEW_TRANSLATION_MIN; }

  public Vector3d getViewOrigin()
  {
    // TODO: Fix distance hack.
    if (viewOrigin == null)
      viewOrigin = new Vector3d(0.0, 0.0, 0.0);

    return viewOrigin;
  }


  private transient Transform3D o = new Transform3D();
  private transient Transform3D t = new Transform3D();
  private transient Transform3D x = new Transform3D();
  private transient Transform3D y = new Transform3D();

  protected void setViewFromParameters()
  {
    // BDebug.print("B3DCanvas.setViewFromParameters()");

    ViewingPlatform platform = getViewingPlatform();
    MultiTransformGroup mtg = platform.getMultiTransformGroup();

    TransformGroup botOrigin = mtg.getTransformGroup(TRANSFORM_BOTORIGIN);
    TransformGroup rotAboutY = mtg.getTransformGroup(TRANSFORM_ROTABOUTY);
    TransformGroup rotAboutX = mtg.getTransformGroup(TRANSFORM_ROTABOUTX);
    TransformGroup translate = mtg.getTransformGroup(TRANSFORM_TRANSLATE);


    o.setTranslation(getViewOrigin());
    t.setTranslation(getViewTranslation());

    Vector3d rotation = getViewRotation();
    x.rotX(rotation.x);  

    y.rotY(rotation.y);

    botOrigin.setTransform(o);
    rotAboutY.setTransform(y);
    rotAboutX.setTransform(x);
    translate.setTransform(t);
  }

  protected TransformGroup getViewTG(int i)
  {
    ViewingPlatform platform = getViewingPlatform();
    MultiTransformGroup mtg = platform.getMultiTransformGroup();

    return mtg.getTransformGroup(i);
  }

  public TransformGroup getViewOriginTG()
    { return getViewTG(TRANSFORM_BOTORIGIN); }

  public TransformGroup getViewRotateXTG()
    { return getViewTG(TRANSFORM_ROTABOUTX); }

  public TransformGroup getViewRotateYTG()
    { return getViewTG(TRANSFORM_ROTABOUTY); }

  public TransformGroup getViewTranslateTG()
    { return getViewTG(TRANSFORM_TRANSLATE); }

  public ViewingPlatform getViewingPlatform()
    { return universe.getViewingPlatform(); }


  
  // Accessor methods for behavior management.
  // -------------------------------------------------------------

  protected void addBehavior(Behavior b)
  {
    BranchGroup bg = (BranchGroup) behaviorTable.get(b);

    if (bg == null)
    {
      // TODO: set b's parent to null.
      b.setSchedulingBounds(BOUNDS);

      bg = new BranchGroup();
      bg.setCapability(BranchGroup.ALLOW_DETACH);
      bg.addChild(b);

      behaviorTable.put(b, bg);
    }

    behaviorGroup.addChild(bg);
  }

  protected void removeBehavior(Behavior b)
  {
    if (behaviorTable != null)
    {
      BranchGroup bg =
        (BranchGroup) behaviorTable.get(b);

      if (bg != null)
        bg.detach();
    }
  }


  // Image capture methods...
  // -------------------------------------------------------------

  protected Object  lock                = new Object() {};
  protected boolean isCaptureRequested  = false;
  protected boolean doCapture           = false;
  
  protected BufferedImage captured; 

  public BufferedImage getImage()
  {
    synchronized (lock)
    {
      isCaptureRequested = true;

      try
      {
        repaint();
        lock.wait();
      }
      catch (InterruptedException ex)
        { }

      isCaptureRequested = false;
    }

    return captured;
  }


  protected BufferedImage capture()
  {
    Dimension d = getSize();

    BufferedImage img =
      new BufferedImage(d.width, d.height, BufferedImage.TYPE_INT_ARGB);

    ImageComponent2D comp =
      new ImageComponent2D(ImageComponent2D.FORMAT_RGB, img);

    Raster raster =
      new Raster(new Point3f(), Raster.RASTER_COLOR,
        0, 0, d.width, d.height, comp, null);

    getGraphicsContext3D().readRaster(raster);

    return raster.getImage().getImage();
  }


  public void preRender()
  {
    super.preRender();
    
    if (isCaptureRequested)
      doCapture = true;
  }
  
  public void postSwap()
  {
    super.postSwap();

    if (doCapture)
      synchronized (lock)
      {
        captured = capture();
        doCapture = false;

        lock.notifyAll();
      }
  }





  // HACK: Tooltip display functions...
  // -------------------------------------------------------------

  protected void initToolTipSystem()
    { mouseRelayEnabled = false; }

  protected boolean mouseInCanvas;
  protected boolean mouseRelayEnabled;
  protected BMouseRelayListener relayListener;


  public void setRelayListener(BMouseRelayListener l)
    { relayListener = l; }

  public void setMouseRelayEnabled(boolean enabled)
  {
    if (enabled && !mouseRelayEnabled)
    {
      addMouseListener(relayListener);
      addMouseMotionListener(relayListener);

      if (mouseInCanvas)
        relayListener.generateEvent(MouseEvent.MOUSE_ENTERED, 0, 0);
    }
    else if (!enabled && mouseRelayEnabled)
    {
      removeMouseListener(relayListener);
      removeMouseMotionListener(relayListener);

      if (mouseInCanvas)
        relayListener.generateEvent(MouseEvent.MOUSE_EXITED, 0, 0);
    }
    mouseRelayEnabled = enabled;
  }

  public String getToolTipText(int x, int y)
  {
    BModeMouse m = mouseBehavior.getMode();

    return (m == null) ?
      null : m.getToolTipText(x, y);
  }

  
  // Key event relay method...
  // -------------------------------------------------------------

  public class KeyRelayListener implements KeyListener
  {
    // HACK: Since Canvas3D's are heavyweight components,
    // they do not behave correctly wrt. Swing InputMaps.
    // One specific problem was the Save hotkey (Ctrl-S)
    // not getting to BWorkshopScreen, thus preventing
    // users from quickly saving.  The current (hacky)
    // solution is to relay keypresses from a Canvas3D to
    // its immediate parent.

    public void keyPressed(KeyEvent e)
      { relay(e); }

    public void keyReleased(KeyEvent e)
      { relay(e); }

    public void keyTyped(KeyEvent e)
      { relay(e); }

    protected void relay(KeyEvent e)
    {
      Component parent = getParent();

      if (parent != null)
        parent.dispatchEvent(e);
    }
  }



  // Picking methods...
  // -------------------------------------------------------------

  public Shape3D pickClosestShape(Point p)
  {
    try
    {
      pickCanvas.setShapeLocation(p.x, p.y);
      PickResult picked = pickCanvas.pickClosest();

      return (picked == null) ?
        null : (Shape3D) picked.getObject();
    }
    catch (Exception e)
      { return null; }
  }
  
  
  protected BComponent pickComponent(Point p)
    { return pickComponent(p, false); }
  
  protected BComponent pickComponent(Point p, boolean select)
  {
    Shape3D s = pickClosestShape(p);
    
    if (s == null)
      return null;

    if (s instanceof BConnectPointShape)
    {
      BConnectPointShape cps = (BConnectPointShape) s;
      BComponent c = cps.getParentComponent();

      if (select)
        c.setConnectPointSelected(cps.getConnectPoint());

      return c;
    }

    if (s instanceof BShape3D)
    {
      BComponent c = ((BShape3D) s).getComponent();

      //isShowingConnectPoints
     // if (select && (c != getSelectedComponent()))
     if (select && ( !c.isShowingConnectPoints() ))
     {
       // BDebug.print ( "B3DCanvas.pickComponent " + c );

        Color3f selectColor =
          c.getChassisLock() ?
            BGlobal.LOCKED_COLOR_3F :
            BGlobal.SELECTED_COLOR_3F;

        setSelectedComponent(c, selectColor);

      }

      return c;
    }

    return null;
  }


  protected BConnectPoint pickConnectPoint(Point p, boolean select)
  {
    Shape3D s = pickClosestShape(p);
    
    if (s == null)
      return null;
      
    if (s instanceof BConnectPointShape)
    {
      BConnectPointShape cps = (BConnectPointShape) s;
      
      if (select)
      {
        BComponent c = cps.getParentComponent();
        c.setConnectPointSelected(cps.getConnectPoint());
      }

      return cps.getConnectPoint();
    }
    
    return null;
  }
  
  
  
  // Drag 'n Drop methods...
  // -------------------------------------------------------------

 // public void setDnDEnabled(boolean enabled)
 //   { recognizer.setEnabled(enabled); }

  protected class DnDGestureListener implements DragGestureListener
  {
    public void dragGestureRecognized(DragGestureEvent e)
    {
     // BDebug.print("boo");
       if (dragActive)
        if (!e.getTriggerEvent().isMetaDown() )
        {
          Transferable t = getDragData(e);

          // If drag is not valid, abort.
          if (t == null) return;

          dragSource.startDrag(e, linkCursor, t,
          dragSourceListener);
        }
    }
  }
  

   protected Transferable getDragData(DragGestureEvent e)
  {
    BComponent old =
      pickComponent(e.getDragOrigin(), true);

    if (old == null)
      return null;

    BConnectPoint cp =
      old.getConnectPointSelected();

    if (cp == null)
      return null;

    if (!cp.isActive() )
      return null;

    setSelectedComponent(null);

    BComponent c =
      (BComponent) old.clone();

    setSelectedComponent(old);
    old.setConnectPointSelected(cp);

    c.setConnectPointSelected(cp);
    lastDragData =  new CleansableObjectSelection(c);
    return lastDragData;
  }


  //XXXXXXX
  // VERY NASTY DRAG AND DROP CLASS TO STOP THE DND JNI FROM HOLDING BCOMPONENT OBJECTS
  // CLEANSED IN B3DCanvas.getDropped
  //  TODO, see if we can get rid of it when a new JDK is released
  //XXXXXX
  protected transient static CleansableObjectSelection lastDragData = null;

  protected static class CleansableObjectSelection extends ObjectSelection
  {
    public CleansableObjectSelection(Object o)
    {
      super(o);
    }

    public void cleanse()
    {
        //BDebug.print ("CleansableObjectSelection Called ");
        flavors = null;
        data = null;
    }
  }


  public void setDragActive(boolean b)
  {
    dragActive = b;
  }


  Transform3D setDefaultSelectedConnectPoint_Tran = new Transform3D();
  Vector3f setDefaultSelectedConnectPoint_v = new Vector3f();

  protected void setDefaultSelectedConnectPoint(BComponent c)
  {
        getViewingPlatform().getViewPlatform().getLocalToVworld( setDefaultSelectedConnectPoint_Tran );
        setDefaultSelectedConnectPoint_Tran.get(setDefaultSelectedConnectPoint_v);

        BConnectPoint cp = c.selectClosestConnectPoint(setDefaultSelectedConnectPoint_v);

        if(cp != null)
          c.setConnectPointSelected(cp);
  }


  
  // -------------------------------------------------------------
  // Drag Source Listener Class...
  // -------------------------------------------------------------

  private MouseEvent mouseReleaseEvent = new MouseEvent(this, MouseEvent.MOUSE_RELEASED, 0, 0, 0, 0, 1, false);

  protected class DnDSourceListener implements DragSourceListener
  {
    public void dropActionChanged (DragSourceDragEvent e) { }
    public void dragExit          (DragSourceEvent e)     { }
    public void dragDropEnd(DragSourceDropEvent e)
    {
      // clear the mode event
      mouseBehavior.getMode().mouseReleased( mouseReleaseEvent );

      lastDragData.cleanse();
      lastDragData = null;
    }

    public void dragEnter(DragSourceDragEvent e) 
    { 
      DragSourceContext context =
        e.getDragSourceContext();

      context.setCursor(DragSource.DefaultMoveNoDrop);
    }
    
    public void dragOver (DragSourceDragEvent e)
    {
      DragSourceContext context =
        e.getDragSourceContext();

      context.setCursor(null);
      context.setCursor(linkCursor);
    }
  }

}







