package roboforge.ui;

import roboforge.BBot;
import roboforge.BChassis;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BProgressMonitorable;
import roboforge.BWizardTemplateBot;

import roboforge.attributes.*;

import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.components.border.*;
import roboforge.ui.events.*;
import roboforge.ui.layers.*;
import roboforge.ui.net.*;

import roboforge.util.graphics.*;
import roboforge.util.io.*;
import roboforge.util.processors.*;
import roboforge.util.text.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.Font;
import java.io.*;

import java.util.Vector;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import javax.media.j3d.VirtualUniverse;

public class BArchiveScreen extends BSystemScreen {

  String backgroundicon = "background_archive.jpg";

  protected BUIResources res = new BUIResources( new Object[]
  {
    "titleIcon.icon",             BIconCache.getHSB("archives_icon.gif"),
    "titleLabel.icon",            BIconCache.getHSB("archives_title.gif"),

    "background.icon",            BGraphics.getHSBIcon(backgroundicon),
    "background.foreground",      colors.getMidShade(),
    "background.bounds",          new Rectangle(0, 1, 800, 450),

    "prompt.text",                "Please select an archived battle to view...",
    "prompt.bounds",              new Rectangle(75, 50, 300, 20),
    // "prompt.foreground",          colors.getMidShade(),

    "view.name",                  "View",
    "view.tooltip",               "View selected battle",
    "view.bounds",                new Rectangle(225, 345, 80, 30),

    "viewAction.name",            "View",

    "delete.tooltip",             "Delete battle from archive",
    "delete.bounds",              new Rectangle(315, 345, 80, 30),

    "deleteAction.name",          "Delete"


  });


  // Visual elements ...
  // -------------------------------------------------------------
  
  protected JLabel background           = new BLabel(res, "background");

  protected BLabel prompt               = new BLabel(res, "prompt");
  protected BFileList   items           = new BFileList();
  protected BScrollPane itemScroller    = new BScrollPane(items);
  protected BButton view                = new BButton(res, "view");
  protected BButton delete              = new BButton(res, "delete");

  public BArchiveScreen()
  {
    super(true);

    VirtualUniverse.setJ3DThreadPriority(Thread.MIN_PRIORITY );

    layoutArchiveScreenChildren();
    configArchiveScreenChildren();

    res.cleanse();
    res = null;
  }

  protected void layoutArchiveScreenChildren()
  {
    titleIcon.setIcon(res.getIcon("titleIcon.icon"));
    titleLabel.setIcon(res.getIcon("titleLabel.icon"));

    body.setLayout(null);
    body.addOnTop(background);

    body.addOnTop(prompt);
    
    itemScroller.setBounds(75, 75, 320, 260);
    body.addOnTop(itemScroller);

    body.addOnTop(view);
    body.addOnTop(delete);

  }

  protected void configArchiveScreenChildren()
  {

    view.setAction (new ViewAction(res, "viewAction") );
    delete.setAction (new DeleteAction(res, "deleteAction") );

    try
    {
      items.setSourcePath(BGlobal.BATTLERECORDINGSPATH);
      items.setFilter(new BRoboforgeFilter(BGlobal.RECORDINGSUFFIX ) );

    } catch (Exception e)
    {
      e.printStackTrace();
    }

    //BDebug.seedTimer();
   // background.setIcon(BGraphics.getHSBIcon(backgroundicon));
   // BDebug.printTime("BArchive iconTime ");

  }

    protected class DeleteAction extends BThreadedAction
  {
    boolean isRemoving;
    protected BPromptDialog deleteDialog =
      BPromptDialog.createYesNoDialog("Delete Battle?",
        "Do you wish to delete the selected battle?");

    public DeleteAction (BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
    {
    }

    public void threadedAction(ActionEvent e)
    {
      String filename = items.getSelectedFileWithPath();

      if  ( (filename != null) && (! filename.equals("")  ))
      {
       // BDebug.print (filename);
        BApplication.showDialog(deleteDialog);
        isRemoving = deleteDialog.getResult();

        if (isRemoving)
        {
          File d = new File(filename);
          d.delete();

          String statsFileName =filename.substring(0, filename.length() - 3) + "sta";
          File s = new File(statsFileName);
          if (s.exists() )
            s.delete();

          // refresh Box
          try {  items.setSourcePath(BGlobal.BATTLERECORDINGSPATH);} catch (Exception ex)  {ex.printStackTrace();  }

        }
      }

    }

    public void swingCleanupAction(ActionEvent e)
    {
      setStatus(isRemoving ?
        ("Battle deleted") :
        "Remove cancelled.");
    }
  }


  /*
  protected class DeleteAction extends BAction
  {
    public DeleteAction (BUIResources res, String key)
      { super(res, key); }

    public void actionPerformed(ActionEvent e)
    {
      String filename = items.getSelectedFileWithPath();

      if (filename != null)
      {
        File d = new File(filename);
        d.delete();

        // refresh Box
        try {  items.setSourcePath(BGlobal.BATTLERECORDINGSPATH);} catch (Exception ex)  {ex.printStackTrace();  }

      }
    }
  }
  */


  protected class ViewAction extends BAction
  {
    public ViewAction (BUIResources res, String key)
      { super(res, key); }

    public void actionPerformed(ActionEvent e)
    {
      String filename = items.getSelectedFileWithPath();

      if (filename != null)
        BBattleScreen.load(filename);
    }
  }


  public void onDeactivate()
  {
    background.setIcon(null);
    super.onDeactivate();
    
  }

  public void onActivate()
  {
    super.onActivate();

    if (background.getIcon() == null)
      background.setIcon(BGraphics.getHSBIcon(backgroundicon));

  }




} 