package roboforge.ui;

import java.awt.*;
import java.awt.event.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.lang.Math.*;
import java.util.*;

import roboforge.ui.events.*;
import roboforge.ui.modes.*;
import roboforge.*;


public class BInputHandlerBehavior extends BMouseMultiBehavior implements BActivateable
{

  protected BActivationSupport activation
    = new BActivationSupport(this);

  protected BModeMouse mode;
  protected boolean isMouseInCanvas;
  protected boolean isMousePressed;
  
  protected WakeupCondition  wakeUpConditions;
  protected WakeupOnAWTEvent mouseMotionCritereon;
  protected WakeupOnAWTEvent mouseCritereon;
  protected WakeupOnAWTEvent keyCritereon;


  // Constructor...
  // -------------------------------------------------------------

  public BInputHandlerBehavior()
  {
    super();
    
    this.setSchedulingBounds(BGlobal.INFINITE_BOUNDS);
    this.setMode(null);
    this.isMouseInCanvas = false;
    this.isMousePressed  = false;
  }

  public void initialize()
  { 
    mouseMotionCritereon =
      new WakeupOnAWTEvent(AWTEvent.MOUSE_MOTION_EVENT_MASK);
    
    mouseCritereon =
      new WakeupOnAWTEvent(AWTEvent.MOUSE_EVENT_MASK);

    keyCritereon =
      new WakeupOnAWTEvent(AWTEvent.KEY_EVENT_MASK);
    
    wakeUpConditions =
      new WakeupOr(new WakeupCriterion[]
        { mouseMotionCritereon, mouseCritereon, keyCritereon });
    
    this.wakeupOn(wakeUpConditions); 
  }

  
  
  // Accessor methods...
  // -------------------------------------------------------------
  
  public boolean isMouseInCanvas()
    { return isMouseInCanvas; }

  public BModeMouse getMode()
    { return mode; }

  public void setMode(BModeMouse m)
  {
    BModeMouse old = mode;

    if (old != null)
      old.requestDeactivate();
    
    this.mode = m;
    
    if (isActive())
    {
      boolean isAMode = 
        mode != null;
      
      setEnable(isAMode);
      
      if (isAMode) 
        mode.requestActivate();
    }
  }
  
  // Activation methods...
  // -------------------------------------------------------------

  public boolean isActive()
    { return activation.isActive(); }

  public boolean isClosed()
    { return activation.isClosed(); }

  public void requestActivate()
    { activation.requestActivate(); }

  public void requestDeactivate()
    { activation.requestDeactivate(); }

  public void requestClose()
    { activation.requestClose(); }

  public void onActivate()
  {
    if (mode != null)
    {
      setEnable(true);
      mode.requestActivate();
    }
   
    activation.fireActivate();
  }

  public void onDeactivate()
  {
    if (mode != null)
    {
      setEnable(false);
      mode.requestDeactivate();
    }
    
    activation.fireDeactivate();
  }

  public void onClose()
    { activation.fireClose(); }

  public void addActivationListener(BActivationListener l)
    { activation.addActivationListener(l); }

  public void removeActivationListener(BActivationListener l)
    { activation.addActivationListener(l); }
  
  
  
  // Implementation methods...
  // -------------------------------------------------------------
  
  public void handleMouseEvent(MouseEvent m)
  {
    int id = m.getID();

    switch (id)
    {
      case MouseEvent.MOUSE_DRAGGED:
        if (isMousePressed)
          mode.mouseDragged(m);
        break;

      case MouseEvent.MOUSE_PRESSED:
        isMousePressed = true;
        mode.mousePressed(m);
        break;

      case MouseEvent.MOUSE_RELEASED:
        isMousePressed = false;
        mode.mouseReleased(m);
        break;

      case MouseEvent.MOUSE_ENTERED:
        isMouseInCanvas = true;
        break;

      case MouseEvent.MOUSE_EXITED:
        isMouseInCanvas = false;
        break;

      case MouseEvent.MOUSE_MOVED:
        break;
    }
  }

  public void handleKeyEvent(KeyEvent k)
  {
    int id = k.getID();

    switch (id)
    {
      case KeyEvent.KEY_PRESSED:
        mode.keyPressed(k);
        break;

      case KeyEvent.KEY_RELEASED:
        mode.keyReleased(k);
        break;
    }
  }


  public void processStimulus(Enumeration criteria)
  {
    AWTEvent[] es;

    if (mode == null)
      return;
    
    if (mouseCritereon.hasTriggered())
    {
      es = mouseCritereon.getAWTEvent();
        
      int n = es.length;
      for (int i=0; i<n; i++)
        handleMouseEvent((MouseEvent) es[i]);
    }
    
    if (mouseMotionCritereon.hasTriggered())
    {
      es = mouseMotionCritereon.getAWTEvent();

      int n = es.length;
      for (int i=0; i<n; i++)
        handleMouseEvent((MouseEvent) es[i]);
    }

    if (keyCritereon.hasTriggered())
    {
      es = keyCritereon.getAWTEvent();

      int n = es.length;
      for (int i=0; i<n; i++)
        handleKeyEvent((KeyEvent) es[i]);
    }
    
    wakeupOn(wakeUpConditions);
  }


  public void cleanse()
  {
      activation = null;

      mode = null;


      wakeUpConditions = null;
      mouseMotionCritereon = null;
      mouseCritereon = null;


  }

} 