package roboforge.ui;

import roboforge.BGlobal;
import roboforge.ui.*;
import roboforge.ui.components.*;
import roboforge.util.graphics.*;
import roboforge.sound.BSound;
import roboforge.math.*;
import roboforge.BDebug;
import roboforge.util.processors.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.border.BevelBorder;
import roboforge.ui.fusion.FusionLookAndFeel;



public class BPreferenceScreen extends BSystemScreen {

protected static final Integer LABEL_WIDTH = 
    new Integer(100);
    
  protected BUIResources res = new BUIResources( new Object[]
  {
    "cb_mipMap.text",               "Smooth textures for distant objects",
    "cb_mipMap.h_align",            new Integer(LEFT),

    "cb_fog.text",                  "Use fog in the battle player",
    "cb_fog.h_align",               new Integer(LEFT),

    "cb_lowMap.text",               "Use low texture detail",
    "cb_lowMap.h_align",      			new Integer(LEFT),

    "cb_autogen.text",              "Use special texture maps",
    "cb_autogen.h_align",           new Integer(LEFT),

    "particleLabel.text",           "Special effects quality:",
    "particleLabel.height",         new Integer(20),

    "particleQuality.width",        new Integer(75),

    "cb_disableFlashing.text",      "Disable flashing objects",
    "cb_disableFlashing.h_align",   new Integer(LEFT),

    "cb_solidZones.text",      			"Display zones as solid objects",
    "cb_solidZones.h_align",    		new Integer(LEFT),

    "cb_useTrans.text",      				"Display transparencies",
    "cb_useTrans.h_align",    			new Integer(LEFT),

    "cb_interpolation.text",        "Smooth animation in battle player",

    "videoOptions.text",            "Video options",
    // graphics
    "cb_useSound.text",             "Use Sound",
    "cb_useSound.h_align",          new Integer(LEFT),

    "foreSliderLabel.text",         "Foreground sound volume (min-max)",
    "foreSliderLabel.width",        new Integer(200),

    "foreSlider.max",               new Integer((int)BSound.FOREGROUND_SOUND_MAX),
    "foreSlider.ticks",             new Boolean(true),
    "foreSlider.min",               new Integer((int)BSound.FOREGROUND_SOUND_MIN),

    "backSliderLabel.text",         "Background sound volume (min-max)",
    "backSliderLabel.width",        new Integer(200),

    "backSlider.max",               new Integer((int)BSound.BACKGROUND_SOUND_MAX),
    "backSlider.min",               new Integer((int)BSound.BACKGROUND_SOUND_MIN),
    "backSlider.ticks",             new Boolean(true),

    "interfaceSoundLabel.text",    "Interface sound volume (min-max)",
    "interfaceSoundLabel.width",    new Integer(200),

    "interfaceSlider.max",          new Integer((int)BSound.INTERFACE_SOUND_MAX),
    "interfaceSlider.min",          new Integer((int)BSound.INTERFACE_SOUND_MIN),
    "interfaceSlider.ticks",        new Boolean(true),


    // general label
    "cb_invertMouse.text",          "Invert mouse Y axis in battle player",
    "cb_invertMouse.h_align",       new Integer(LEFT),

    "cb_useProxy.text",             "Use proxy server settings",
    "cb_useProxy.h_align",          new Integer(LEFT),

    "proxyHostLabel.text",          "Proxy host name:  ",
    "proxyHostLabel.width",         LABEL_WIDTH,
    "proxyHostLabel.h_align",       BLabel.ALIGN_LEFT,

    "proxyPortLabel.text",          "Proxy port:  ",
    "proxyPortLabel.width",         LABEL_WIDTH,
    "proxyPortLabel.h_align",       BLabel.ALIGN_LEFT,

    "proxyPasswordLabel.text",      "Proxy password:  ",
    "proxyPasswordLabel.width",     LABEL_WIDTH,
    "proxyPasswordLabel.h_align",   BLabel.ALIGN_LEFT,

    "cb_autosave.text",             "Autosave in workshop",
    "cb_autosave.h_align",          new Integer(LEFT),

    "saveButton.text",              "Save preferences and exit",

    // group
    "groupPanel.bounds",            new Rectangle(0, 1, 800, 450),
    "groupPanel.v_align",           new Integer(TOP),

    "prefChildGroup.width",         new Integer(210),
    "prefChildGroup.height",        new Integer(320),
    "prefChildGroup.h_align",       new Integer(LEFT),

   /* "subTitleLabel.text",           "Roboforge v" + BMath.format(BGlobal.APP_VERSION),
    "subTitleLabel.font",           new Font("Eurostile", Font.PLAIN, 12),
    "subTitleLabel.antialiased",    new Boolean(true),
    "subTitleLabel.border",         new EmptyBorder(0, 0, 15, 0),    */

    "color1.width",            new Integer(24),
    "color1.height",           new Integer(24),

    "color2.width",            new Integer(24),
    "color2.height",           new Integer(24),

    "color3.width",            new Integer(24),
    "color3.height",           new Integer(24),

    "color4.width",            new Integer(24),
    "color4.height",           new Integer(24),

    "b_advanced.text",         "Advanced",
    "b_advanced.height",       new Integer(14),
    "b_simple.text",           "Back",
  });

  BGraphicsPanel graphicsPanel          = new BGraphicsPanel(res);
  BSoundPanel soundPanel                = new BSoundPanel(res);
  BGeneralPanel generalPanel            = new BGeneralPanel(res);
 // protected JLabel  subTitleLabel       = new BLabel(res, "subTitleLabel");

  // variables to detect if the program needs to be re-started.


  public BPreferenceScreen()
  {
    super();

    layoutWorkshopChildren();
    configWorkshopChildren();

    res.cleanse();
    res = null;

  }


  protected void savePreferences()
  {
    setStatus("Please wait ... changing preferences.");

    graphicsPanel.saveToBGlobal();
    soundPanel.saveToBGlobal();
    generalPanel.saveToBGlobal();



    BGlobal.save();
    closeScreen();
    //setStatus("Preferences saved.");
  }

  protected void layoutWorkshopChildren()
  {
    titleIcon.setIcon(BIconCache.getHSB("options_icon.gif"));
    titleLabel.setIcon(BIconCache.getHSB("options_title.gif"));
    //subTitle.addHVGlued(subTitleLabel);
    
  }

  public void onActivate()
  {
    super.onActivate();
    BDebug.print("BPreferencePAnel.onActivate");
  }


  public void onClose()
  {
    graphicsPanel.onClose();
    soundPanel.onClose();
    generalPanel.onClose();
    super.onClose();

  }

  protected void configWorkshopChildren()
  {
    body.setLayout(null);

    BBoxPanel groupPanel = new BBoxPanel(res, "groupPanel", BoxLayout.X_AXIS);
    body.addOnTop( groupPanel);

    groupPanel.addGlue();

    groupPanel.add( graphicsPanel );
    groupPanel.addMargin();
    groupPanel.add( soundPanel );
    groupPanel.addMargin();
    groupPanel.add( generalPanel );
    groupPanel.addGlue();

  }


  class BGraphicsPanel extends BBoxPanel implements ActionListener
  {

    protected BTitlebarLabel graphicsLabel  = new BTitlebarLabel("3D Graphics");
    BCheckBox cb_mipMap                     = new BCheckBox(res, "cb_mipMap");
    BCheckBox cb_fog                        = new BCheckBox(res, "cb_fog");
    BCheckBox cb_lowMap                     = new BCheckBox(res, "cb_lowMap");
    BCheckBox cb_autogen                    = new BCheckBox(res, "cb_autogen");
    BCheckBox cb_interpolation              = new BCheckBox(res, "cb_interpolation");
    BCheckBox cb_disableFlashing 						= new BCheckBox(res, "cb_disableFlashing");
    BCheckBox cb_solidZones 						  	= new BCheckBox(res, "cb_solidZones");
    BCheckBox cb_useTrans 						  		= new BCheckBox(res, "cb_useTrans");

    BLabel    particleLabel                 = new BLabel(res, "particleLabel");

    Object[] particleItems
      = { "Low", "Medium", "High" };

    BComboBox particleQuality
      = new BComboBox(res, "particleQuality", particleItems);
      

    BButton b_advanced                      = new BButton(res, "b_advanced");
    BButton b_simple                        = new BButton(res, "b_simple");

    BRadioButton rb_openGL = new BRadioButton("OpenGL");
    BRadioButton rb_directX = new BRadioButton("DirectX");
    BRadioButton rb_slowComp = new BRadioButton("Low (64Mb Pentium II)");
    BRadioButton rb_midComp  = new BRadioButton("Medium (128Mb Pentium III or better)");
    BRadioButton rb_fastComp = new BRadioButton("High (PIII 750 MHz, 32Mb video card)");
    BRadioButton rb_advanced = new BRadioButton("");

    BMultiLabel desc = new BMultiLabel();
    BLabel desc2 = new BLabel();

    BLabel compSpeed = new BLabel("Computer speed:");
    BLabel vidMode = new BLabel("Video mode");

    BLine line = new BLine();
    BBoxPanel gSimpleBox = new BBoxPanel(BoxLayout.Y_AXIS);
    BBoxPanel gTrickyBox = new BBoxPanel(BoxLayout.Y_AXIS);

    String oldVideoMode;
    boolean changedVideoModes = false;

    boolean oldZonesMode;
    boolean changedZonesModes = false;

    public BGraphicsPanel(BUIResources localRes)

    {
      super(localRes, "prefChildGroup", BoxLayout.Y_AXIS);
      this.addHGlued(graphicsLabel);
      this.addSmallMargin();

      oldVideoMode = BGlobal.VIDEOMODE;
      oldZonesMode = BGlobal.SOLID_ZONES;

      ButtonGroup specsGroup = new ButtonGroup();
      ButtonGroup vidGroup = new ButtonGroup();

      vidGroup.add(rb_openGL);
      vidGroup.add(rb_directX);

      specsGroup.add(rb_slowComp);
      specsGroup.add(rb_midComp);
      specsGroup.add(rb_fastComp);
      specsGroup.add(rb_advanced);

      desc.setText("OpenGL is preferable, unless you have a Voodoo based graphics card or are running under Windows 2000. Please ensure that you have Direct X 8 or higher");
      desc2.setText("Set graphics based on computer specs.");
      gTrickyBox.addLinear(cb_mipMap, Box.createHorizontalGlue());
      gTrickyBox.addTinyMargin();
      gTrickyBox.addLinear(cb_fog, Box.createHorizontalGlue());
      gTrickyBox.addTinyMargin();
      gTrickyBox.addLinear(cb_lowMap, Box.createHorizontalGlue());
      gTrickyBox.addTinyMargin();
      gTrickyBox.addLinear(cb_autogen, Box.createHorizontalGlue());
      gTrickyBox.addTinyMargin();
      gTrickyBox.addLinear(cb_interpolation, Box.createHorizontalGlue());
      gTrickyBox.addTinyMargin();
      gTrickyBox.addLinear(cb_disableFlashing, Box.createHorizontalGlue());
      gTrickyBox.addTinyMargin();
      gTrickyBox.addLinear(cb_solidZones, Box.createHorizontalGlue());
      gTrickyBox.addTinyMargin();
      gTrickyBox.addLinear(cb_useTrans, Box.createHorizontalGlue());
      gTrickyBox.addTinyMargin();
      gTrickyBox.setHGlue(particleLabel);
      gTrickyBox.addLinear(particleLabel, particleQuality);
      gTrickyBox.addTinyMargin();
      gTrickyBox.addGlue();
      gTrickyBox.addLinear(b_simple, Box.createHorizontalGlue());
      gTrickyBox.setVisible(false);

      compSpeed.setForeground(BGlobal.colors.getMidShade());
      vidMode.setForeground(BGlobal.colors.getMidShade());
      gSimpleBox.addHGlued(compSpeed);
      gSimpleBox.addHGlued(desc2);
      gSimpleBox.addTinyMargin();
      gSimpleBox.addHGlued(rb_slowComp);
      gSimpleBox.addHGlued(rb_midComp);
      gSimpleBox.addHGlued(rb_fastComp);
      gSimpleBox.addLinear(rb_advanced, b_advanced, Box.createHorizontalGlue());
      gSimpleBox.addSmallMargin();
      gSimpleBox.addHGlued(line);
      gSimpleBox.addSmallMargin();
      gSimpleBox.addHGlued(vidMode);
      gSimpleBox.addLinear(rb_openGL, gSimpleBox.getHGlue(), rb_directX);
      gSimpleBox.addTinyMargin();
      gSimpleBox.addHGlued(desc);
      gSimpleBox.addGlue();

      this.add(gTrickyBox);
      this.add(gSimpleBox);

      rb_slowComp.addActionListener(this);
      rb_midComp.addActionListener(this);
      rb_fastComp.addActionListener(this);
      rb_advanced.addActionListener(this);

      b_advanced.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          gSimpleBox.setVisible(false);
          gTrickyBox.setVisible(true);
        }
      });
      b_simple.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          gTrickyBox.setVisible(false);
          gSimpleBox.setVisible(true);
        }
      });

      updateFromBGlobal();
    }

    public void actionPerformed(ActionEvent e)
    {
      setAdvanced();
    }

    public void setAdvanced()
    {
      if(rb_advanced.isSelected())
        b_advanced.setEnabled(true);
      else
        b_advanced.setEnabled(false);
    }

    public void updateFromBGlobal()
    {
      cb_mipMap.setSelected(BGlobal.USE_MIPMAP);
      cb_fog.setSelected(BGlobal.USE_FOG);
      cb_lowMap.setSelected(BGlobal.USE_LOW_TEXTURE_DETAIL);
      cb_autogen.setSelected(BGlobal.USE_AUTOGEN);
      cb_interpolation.setSelected(BGlobal.USE_INTERPOLATION);
      cb_disableFlashing.setSelected(BGlobal.DISABLE_FLASHING_OBJECTS);
      cb_solidZones.setSelected(BGlobal.SOLID_ZONES);
      cb_useTrans.setSelected(BGlobal.USE_TRANSPARENCY);
      particleQuality.setSelectedIndex(BGlobal.PARTICLE_QUALITY);

      if(BGlobal.GRAPHICS_SPEC == BGlobal.LOW_SPEC)
        rb_slowComp.setSelected(true);
      if(BGlobal.GRAPHICS_SPEC == BGlobal.MID_SPEC)
        rb_midComp.setSelected(true);
      else if(BGlobal.GRAPHICS_SPEC == BGlobal.HIGH_SPEC)
        rb_fastComp.setSelected(true);
      else if(BGlobal.GRAPHICS_SPEC == BGlobal.ADVANCED_SPEC)
        rb_advanced.setSelected(true);


      if (BGlobal.VIDEOMODE.equals(BGlobal.VIDEOMODE_DIRECTX))
        rb_directX.setSelected(true);
      else
        rb_openGL.setSelected(true);

      setAdvanced();
    }

    public void saveToBGlobal()
    {
        BGlobal.USE_MIPMAP = cb_mipMap.isSelected();
        BGlobal.USE_FOG = cb_fog.isSelected();
        BGlobal.setLowTextureDetail(cb_lowMap.isSelected());
        BGlobal.USE_AUTOGEN  = cb_autogen.isSelected();
        BGlobal.USE_INTERPOLATION = cb_interpolation.isSelected();
        BGlobal.DISABLE_FLASHING_OBJECTS = cb_disableFlashing.isSelected();
        BGlobal.SOLID_ZONES = cb_solidZones.isSelected();
        BGlobal.USE_TRANSPARENCY = cb_useTrans.isSelected();

        BGlobal.PARTICLE_QUALITY  = particleQuality.getSelectedIndex();
        
        if(rb_slowComp.isSelected())
          BGlobal.GRAPHICS_SPEC = BGlobal.LOW_SPEC;
        else if(rb_midComp.isSelected())
          BGlobal.GRAPHICS_SPEC = BGlobal.MID_SPEC;
        else if(rb_fastComp.isSelected())
          BGlobal.GRAPHICS_SPEC = BGlobal.HIGH_SPEC;
        else if(rb_advanced.isSelected())
          BGlobal.GRAPHICS_SPEC = BGlobal.ADVANCED_SPEC;

        BGlobal.setGraphicsSpec();

        if (rb_directX.isSelected() )
          BGlobal.VIDEOMODE = BGlobal.VIDEOMODE_DIRECTX;
        else
          BGlobal.VIDEOMODE = BGlobal.VIDEOMODE_OPENGL;

        changedVideoModes = !BGlobal.VIDEOMODE.equals(oldVideoMode);
        changedZonesModes = !(BGlobal.SOLID_ZONES == oldZonesMode);

        try
        {
          if (changedVideoModes)
          {
            BGlobal.swapInVideoMode();

            BProcessor p = new BProcessor()
            {
              public void process()
                { notifyAndExit(); }
            };

            p.start();
          } else if ( changedZonesModes ) {
            BProcessor p = new BProcessor()
            {
              public void process()
                { notifyAndExit(); }
            };

            p.start();
          }
        }
        catch (Exception e)
        {
          e.printStackTrace();
        }
    }

    public void notifyAndExit()
    {
      BPromptDialog restartD = BPromptDialog.createOKDialog("Video Options Changed",
                                            "Critical Video Options have been modified, Roboforge will now restart..." );
      BApplication.showDialog(restartD);
      BApplication.restart();

    }

    public void onClose()
    {
       super.onClose();
    }
  }


  class BSoundPanel extends BBoxPanel
  {

    protected BTitlebarLabel soundLabel = new BTitlebarLabel("Sound");
    BCheckBox cb_useSound               = new BCheckBox(res, "cb_useSound");
    BSlider foreSlider                  = new BSlider(res, "foreSlider");
    BSlider backSlider                  = new BSlider(res, "backSlider");
    BSlider interfaceSlider             = new BSlider(res, "interfaceSlider");


    BLabel foreSliderLabel              = new BLabel(res, "foreSliderLabel");
    BLabel backSliderLabel              = new BLabel(res, "backSliderLabel");
    BLabel interfaceSoundLabel          = new BLabel(res, "interfaceSoundLabel");


    public BSoundPanel(BUIResources localRes)

    {
      super(localRes, "prefChildGroup", BoxLayout.Y_AXIS);
      this.addHGlued(soundLabel);
      this.addSmallMargin();
      
      if (BGlobal.DEVELOPER_MODE)
      {

        //this.addSmallMargin();

        cb_useSound.addActionListener(new ActionListener()
          {
            public void actionPerformed(ActionEvent e)
            {
              BSoundPanel.this.configWorkshopChildren();
            }
          }
          );

        this.addLinear(cb_useSound, Box.createHorizontalGlue());
        //this.addSmallMargin();

      }

      this.addLinear(foreSliderLabel, Box.createHorizontalGlue());
      this.addLinear(foreSlider, Box.createHorizontalGlue());
      this.addTinyMargin();

      this.addLinear(backSliderLabel, Box.createHorizontalGlue());
      this.addLinear(backSlider, Box.createHorizontalGlue());
      this.addTinyMargin();

      this.addLinear(interfaceSoundLabel, Box.createHorizontalGlue());
      this.addLinear(interfaceSlider, Box.createHorizontalGlue());
      this.addTinyMargin();

      updateFromBGlobal();
      this.configWorkshopChildren();


    }

    protected void configWorkshopChildren()
    {
      foreSliderLabel.setEnabled(cb_useSound.isSelected() );
      backSliderLabel.setEnabled(cb_useSound.isSelected() );
      foreSlider.setEnabled(cb_useSound.isSelected() );
      backSlider.setEnabled(cb_useSound.isSelected() );

    }


    public void updateFromBGlobal()
    {
      cb_useSound.setSelected(BGlobal.USE_SOUND);
      foreSlider.setValue((int)BGlobal.FOREGROUND_SOUND_VOLUME);
      backSlider.setValue((int)BGlobal.BACKGROUND_SOUND_VOLUME);
      interfaceSlider.setValue((int)BGlobal.INTERFACE_SOUND_VOLUME);
    }

    public void saveToBGlobal()
    {
       BGlobal.USE_SOUND = cb_useSound.isSelected();
       BGlobal.FOREGROUND_SOUND_VOLUME = foreSlider.getValue();
       BGlobal.BACKGROUND_SOUND_VOLUME = backSlider.getValue();
       BGlobal.INTERFACE_SOUND_VOLUME = interfaceSlider.getValue();

    }






  }


   class BGeneralPanel extends BBoxPanel
  {

    protected BTitlebarLabel generalLabel = new BTitlebarLabel("General");
    BCheckBox cb_invertMouse              = new BCheckBox(res, "cb_invertMouse");

    BCheckBox cb_useProxy                 = new BCheckBox(res, "cb_useProxy");

    protected BLabel proxyHostLabel       = new BLabel(res, "proxyHostLabel");
    protected BTextField proxyHost        = new BTextField();

    protected BLabel proxyPortLabel       = new BLabel(res, "proxyPortLabel");
    protected BTextField proxyPort        = new BTextField();

    protected BLabel proxyPasswordLabel   = new BLabel(res, "proxyPasswordLabel");
    protected BTextField proxyPassword    = new BTextField();

    BCheckBox cb_autosave                 = new BCheckBox(res, "cb_autosave");

    BButton saveButton                    = new BButton(res, "saveButton");
    BTextField systemPath                 = new BTextField(res, "systemPath");

    //BButton color1 = new BButton(res, "color1");
    //BButton color2 = new BButton(res, "color2");
    //BButton color3 = new BButton(res, "color3");
    //BButton color4 = new BButton(res, "color4");

    //BevelBorder border = (BevelBorder)BorderFactory.createBevelBorder(BevelBorder.LOWERED, BGlobal.colors.getDarkerShade(), BGlobal.colors.getBlackShade());


    public BGeneralPanel(BUIResources localRes)

    {
      super(localRes, "prefChildGroup", BoxLayout.Y_AXIS);
      this.addHGlued(generalLabel);

      this.addSmallMargin();
      this.addLinear(cb_invertMouse, Box.createHorizontalGlue());
      this.addSmallMargin();

      this.addLinear(cb_useProxy, Box.createHorizontalGlue());
      this.addSmallMargin();

      this.addLinear(proxyHostLabel, proxyHost);
      this.addSmallMargin();

      this.addLinear(proxyPortLabel, proxyPort);
      this.addSmallMargin();

      this.addLinear(proxyPasswordLabel, proxyPassword);
      this.addSmallMargin();

      this.addLinear(cb_autosave, Box.createHorizontalGlue());
      this.addSmallMargin();

      if (BGlobal.DEVELOPER_MODE)
      {
        this.addHGlued(systemPath);
        this.addSmallMargin();
      }

//      this.addHGlued(new BLabel("Screen colors"));
//      this.addHGlued(new BLine());
//      this.addSmallMargin();

//      BGlobal.UI_BASE_COLOR = new Color(200, 100, 200);
//      BGlobal.colors = new BUIColorScheme(BGlobal.UI_BASE_COLOR);
//      BGlobal.clearAllHardCaches();
//      FusionLookAndFeel.install();

//      color1.setBorder(border);
//      color2.setBorder(border);
//      color3.setBorder(border);
//      color4.setBorder(border);

//      color1.setBackground(new Color(20, 20, 50));
//      color2.setBackground(new Color(50, 20, 20));
//      color3.setBackground(new Color(20, 50, 20));
//      color4.setBackground(new Color(50, 20, 50));

//      BBoxPanel bp = new BBoxPanel(BoxLayout.X_AXIS);
//      bp.add(color1);
//      bp.addSmallMargin();
//      bp.add(color2);
//      bp.addSmallMargin();
//      bp.add(color3);
//      bp.addSmallMargin();
//      bp.add(color4);
//      bp.addGlue();
//      this.add(bp);

//      this.addSmallMargin();
//      this.addHGlued(new BLine());

      this.addGlue();

      saveButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          savePreferences();

        }
      });

      this.addLinear(Box.createHorizontalGlue(), saveButton );

      updateFromBGlobal();
    }

     public void updateFromBGlobal()
    {
      cb_invertMouse.setSelected(BGlobal.INVERT_PLAYER_Y);

      cb_useProxy.setSelected(BGlobal.USE_PROXY);
      proxyHost.setText(BGlobal.PROXY_HOST);
      proxyPort.setText(BGlobal.PROXY_PORT);
      proxyPassword.setText(BGlobal.PROXY_PASSWORD);

      cb_autosave.setSelected(BGlobal.AUTOSAVE);


      systemPath.setText(BGlobal.SYSTEMPATH);

    }

    public void saveToBGlobal()
    {
       BGlobal.INVERT_PLAYER_Y = cb_invertMouse.isSelected();

       BGlobal.USE_PROXY      = cb_useProxy.isSelected();
       BGlobal.PROXY_HOST     = proxyHost.getText();
       BGlobal.PROXY_PORT     = proxyPort.getText();
       BGlobal.PROXY_PASSWORD = proxyPassword.getText();

       BGlobal.AUTOSAVE       = cb_autosave.isSelected();

       if (BGlobal.DEVELOPER_MODE)
       {
        BGlobal.SYSTEMPATH = systemPath.getText();
        BGlobal.updatePaths();
       }

    }




  }

/*
  class VideoOptionsDialog extends BSystemDialog
  {

    BRadioButton rb_openGL = new BRadioButton("OpenGL");
    BRadioButton rb_directX = new BRadioButton("DirectX");

    String oldVideoMode;
    boolean changedVideoModes = false;

    public VideoOptionsDialog()
    {
      super("Video options", true);

      int dialogHeight = 200;
      int dialogWidth = 200;
      setSize(new Dimension(dialogWidth, dialogHeight));
      setLocation(207,355);

      modal    = false;
      blocking = false;

      layoutVideoChildren();
      configVideoChildren();

    }

    protected void configVideoChildren()
    {
      ButtonGroup group = new ButtonGroup();
      group.add(rb_openGL);
      group.add(rb_directX);

      oldVideoMode = BGlobal.VIDEOMODE;

      if (BGlobal.VIDEOMODE.equals(BGlobal.VIDEOMODE_DIRECTX))
        rb_directX.setSelected(true);
      else
        rb_openGL.setSelected(true);


    }

    protected void layoutVideoChildren()
    {
      content.addLinear(rb_openGL, Box.createHorizontalGlue());
      content.addMargin();
      content.addLinear(rb_directX, Box.createHorizontalGlue());
      content.addGlue();
    }

    public void saveToBGlobal()
    {

      if (rb_directX.isSelected() )
        BGlobal.VIDEOMODE = BGlobal.VIDEOMODE_DIRECTX;
      else
        BGlobal.VIDEOMODE = BGlobal.VIDEOMODE_OPENGL;

      changedVideoModes= !BGlobal.VIDEOMODE.equals(oldVideoMode);

    }

    public boolean getChangedVideoModes()
    {
      return changedVideoModes;
    }
*/



 // }





}