package roboforge.ui;

import roboforge.util.processors.*;
import java.io.*;

public class BSystemLogger extends PrintStream
{

  // Amount of time to wait for error prints before aborting.
  protected static int ABORT_TIMEOUT = 1000;

  // The logfile for printed output.
  protected static OutputStream logfile;

  // Pre-existing standard out/error streams..
  protected static PrintStream oldStdout;
  protected static PrintStream oldStderr;

  // Booleans to handle aborting..
  protected boolean abortOnWrite;
  protected boolean aborting;


  public BSystemLogger(PrintStream ps, boolean abortOnWrite)
  {
    super(ps);
    this.abortOnWrite = abortOnWrite;
  }

  public static void start(String f)
    throws IOException
  {
  	// Save old settings.
  	oldStdout = System.out;
  	oldStderr = System.err;

  	// Create/Open logfile.
  	logfile = new PrintStream(
      new BufferedOutputStream(
 	    new FileOutputStream(f)));

  	// Start redirecting the output.
    System.setOut(new BSystemLogger(System.out, false));
    System.setErr(new BSystemLogger(System.err, true));

  }

  // Restores the original settings.
  public static void stop()
  {
  	System.setOut(oldStdout);
  	System.setErr(oldStderr);

    try
      { logfile.close(); }
    catch (Exception e)
      { e.printStackTrace(); }
  }

  // PrintStream override.
  public void write(int b)
  {
    try { logfile.write(b); }
    catch (Exception e)
    {
      e.printStackTrace();
      setError();
    }
  	super.write(b);

    if (abortOnWrite && !aborting)
      abort();
  }

  // PrintStream override.
  public void write
    (byte buf[], int off, int len)
  {
    try
      { logfile.write(buf, off, len); }
    catch (Exception e)
    {
      e.printStackTrace();
      setError();
    }
   	super.write(buf, off, len);

    if (abortOnWrite && !aborting)
      abort();
  }


  protected void abort()
  {
    aborting = true;

    // Spawn a new thread to do the abort.  This is done to allow
    // time for further system.err messages (e.g. a stack trace).

    BProcessor p = new BProcessor()
    {
      public void process()
      {
        try
          { Thread.currentThread().sleep(ABORT_TIMEOUT); }
        catch (Exception e)
          { }
        finally
          { BApplication.abort(); }
      }
    };

    p.start();
  }
}
