package roboforge.ui;

/*
* File:
*   BSystemScreen.java
*
* Description:
*   Base screen for major sub-areas of the application.
*   Contains:
*     - area title (workshop, battleground, options...)
*     - client area panel
*     - status bar
*
* Modification History: ( Name, Date, Notes )
*   Created SJS, 28/01/2000.
*
*/

import roboforge.BDebug;
//import roboforge.BGlobal;
import roboforge.BProgressMonitorable;

import roboforge.ui.action.*;
import roboforge.ui.components.*;
//import roboforge.ui.components.border.*;
//import roboforge.ui.events.*;
import roboforge.ui.layers.*;
import roboforge.util.graphics.*;

//import java.awt.Event;
//import java.awt.Font;
import java.awt.Dimension;
import java.awt.event.*;

import java.util.Enumeration;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.*;



public class BSystemScreen extends BBasicScreen
  implements SwingConstants
{

  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "titleIcon.foreground",    colors.getMidShade(),
    "titleIcon.size",          new Dimension(70, 70),

    "masthead.opaque",         Boolean.FALSE,
    "title.opaque",            Boolean.FALSE,
    "titleLine.height",        new Integer(1),

    "subTitle.opaque",         Boolean.FALSE,
    "subTitle.height",         new Integer(45),

    "systemButtons.opaque",    Boolean.FALSE,

    "titleLabel.foreground",   colors.getMidShade(),
    "titleLabel.border",       new EmptyBorder(8, 0, 0, 0),

    "back.icon",               BIconCache.getHSB("exit_icon.gif"),
    "back.tooltip",            "Back to main menu.",

    "exit.icon",               BIconCache.getHSB("exit_icon.gif"),
    "exit.tooltip",            "Exit Roboforge.",

    "statusBar.opaque",        Boolean.FALSE,
    "statusBar.height",        new Integer(20),

    "working.icon",            BGraphics.getHSBIcon("anim_status.gif", colors.getMidShade()),
    "working.delay",           new Integer(50),
    "working.size",            new Dimension(20, 20),

    "status.light",            colors.getMidShade(),
    "status.dark",             colors.getDarkShade(),
    "status.cycle",            new Integer(20),
    "status.frames",           new Integer(20),
    "status.delay",            new Integer(50),
    
    "progress.width",          new Integer(200),
    "progress.opaque",         Boolean.FALSE,
  });



  // Visual elements...
  // -------------------------------------------------------------

  protected BLabel          titleIcon         = new BFilledLabel(res, "titleIcon");
  protected BBoxPanel       masthead          = new BBoxPanel(res, "masthead", BBoxPanel.Y_AXIS);
  protected BBoxPanel       title             = new BBoxPanel(res, "title", BBoxPanel.X_AXIS);
  protected BBoxPanel       subTitle          = new BBoxPanel(res, "subTitle", BBoxPanel.X_AXIS);
  protected BBoxPanel       systemButtons     = new BBoxPanel(res, "systemButtons", BBoxPanel.X_AXIS);

  protected BLine           titleLine         = new BLine(res, "titleLine");
  protected BLabel          titleLabel        = new BLabel(res, "titleLabel");

  protected BSystemButton   closeButton       = new BSystemButton();

  protected BBoxPanel       statusBar         = new BBoxPanel(res, "statusBar", BoxLayout.X_AXIS);

  protected BAnimatedIconLabel  working       = new BAnimatedIconLabel(res, "working");
  protected BAnimatedTextLabel  status        = new BAnimatedTextLabel(res, "status");
  protected BProgressBar        progress      = new BProgressBar(res, "progress");


  // Dialog boxes...
  // -------------------------------------------------------------

  protected BPromptDialog closeDialog =
    BPromptDialog.createYesNoDialog ("exit roboforge",
      "Are you sure you wish to quit?");


  // Action...
  // -------------------------------------------------------------

  protected BackAction backAction            = new BackAction(res, "back");
  protected ExitAction exitAction            = new ExitAction(res, "exit");


  // Member variables...
  // -------------------------------------------------------------

  protected BLayerStack    panels           = new BSingleLayerStack(body);
  protected ButtonGroup    panelButtons     = new ButtonGroup();
  protected boolean        canGoBack;
  protected int            workCounter      = 0;
  protected WorkMonitor    workMonitor      = new WorkMonitor();


  // for removal onClose
  Vector buttons = new Vector();
  Vector actionListeners = new Vector();
  boolean isClosed = false;


  

  // Constructor.
  // -------------------------------------------------------------

  public BSystemScreen()
    { this(true); }

  public BSystemScreen(boolean canGoBack)
  {
    super();

    this.canGoBack = canGoBack;

    layoutSystemChildren();
    configSystemChildren();

    res.cleanse();
    res = null;
  }


  // Initialization helper methods...
  // -------------------------------------------------------------

  protected void layoutSystemChildren()
  {
    header.addMargin();
    header.add(titleIcon);
    header.add(masthead);

    masthead.add(title);
    masthead.addHGlued(titleLine);
    masthead.add(subTitle);

    title.addMargin();
    title.addHVGlued(titleLabel);
    title.addVGlued(systemButtons);
    title.addMargin();

    subTitle.addBigMargin();

    systemButtons.addMargin();
    systemButtons.add(closeButton);
    systemButtons.addTinyMargin();

    footer.setBorder(new EmptyBorder(0, 15, 20, 15));
    footer.addHGlued(statusBar);

    statusBar.add(working);
    statusBar.addSmallMargin();
    statusBar.addHVGlued(status);
    statusBar.add(progress);
    statusBar.addSmallMargin();
  }

  protected void configSystemChildren()
  {
    if (canGoBack)
      closeButton.setAction(backAction);
    else
      closeButton.setAction(exitAction);

    working.setVisible(false);
    progress.setVisible(false);
  }


  // UI Listeners...
  // -------------------------------------------------------------

  protected class BackAction extends BThreadedAction
  {
    protected boolean isClosing;

    public BackAction(BUIResources res, String key)
      { super(res, key); }

    public void threadedAction(ActionEvent e)
    {
      isClosing = confirmBackWithUser();

      if (isClosing)
        onBackPressed();
    }

    public void swingCleanupAction(ActionEvent e)
    {
      if (isClosing)
        closeScreen();
    }
  }


  protected class ExitAction extends BThreadedAction
  {
    protected boolean isClosing;

    public ExitAction(BUIResources res, String key)
      { super(res, key); }

    public void threadedAction(ActionEvent e)
    {
      isClosing = confirmExitWithUser();

      if (isClosing)
        onExitPressed();
    }

    public void swingCleanupAction(ActionEvent e)
    {
      if (isClosing)
        BApplication.shutdown();
    }
  }


  // System button hooks...
  // -------------------------------------------------------------

  protected boolean confirmBackWithUser()
    { return true; }

  protected void onBackPressed()
    { }

  protected void closeScreen()
    { BApplication.closeScreen(); }
  
  protected boolean confirmExitWithUser()
  {
    BApplication.showDialog(closeDialog);
    return closeDialog.getResult();
  }

  protected void onExitPressed()
    { }

  


  protected void setCloseButtonEnabled(boolean b)
  {
    closeButton.setEnabled(b);
  }


  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();
    panels.requestActivate();
  }

  public void onDeactivate()
  {
    panels.requestDeactivate();
    super.onDeactivate();
  }

  public void onClose()
  {
    isClosed = true;
    panels.requestClose();

    titleIcon         = null;
    masthead          = null;
    title             = null;
    subTitle          = null;
    systemButtons     = null;

    titleLine         = null;
    titleLabel        = null;

    if (closeButton != null)
      closeButton.removeActionListener(backAction);
    closeButton       = null;

    statusBar         = null;

    working           = null;
    status            = null;
    progress          = null;

    closeDialog       = null;

    backAction            = null;
    exitAction            = null;
    panels                = null;

    for (int i = buttons.size() - 1; i >= 0; i-- )
    {
      AbstractButton b = (AbstractButton) buttons.get(i);
      OpenSystemPanelListener oL = (OpenSystemPanelListener)actionListeners.get(i);
      b.removeActionListener(oL);
    }

    buttons = null;
    actionListeners = null;


    panelButtons          = null;
    workMonitor           = null; 

    super.onClose();
  }




  // Accessor methods...
  // -------------------------------------------------------------

  protected void addSystemPanel(BSystemPanel p, AbstractButton b)
  {
    OpenSystemPanelListener bLis = new OpenSystemPanelListener(p);

    b.addActionListener(bLis);

    buttons.add(b);
    actionListeners.add(bLis) ;
    panelButtons.add(b);
    subTitle.add(b);

    panels.push(p);
  }

  protected void setPanelButtonsEnabled(boolean enabled)
  {
    Enumeration e = panelButtons.getElements();

    while (e.hasMoreElements())
    {
      AbstractButton b =
        (AbstractButton) e.nextElement();

      b.setEnabled(enabled);
    }
  }


  protected class OpenSystemPanelListener implements ActionListener
  {
    protected BSystemPanel panel = null;

    public OpenSystemPanelListener(BSystemPanel panel)
      { this.panel = panel; }

    public void actionPerformed(ActionEvent e)
      { setSystemPanel(panel); }
  }


  public BSystemPanel getSystemPanel()
    { return (BSystemPanel) panels.peek(); }

  public void setSystemPanel(BSystemPanel panel)
  {
    setBusyWorking(true);
    
    if (panels.contains(panel))
      panels.moveToTop(panel);
    else
      panels.push(panel);
    
    setBusyWorking(false);
  }


  public void setStatus(String text)
  {
    try
    {
      if (status != null)
      {
        status.setText(text);
        status.start();
      }
    }
    catch (Exception e)
    {
      BDebug.print ("BSystemScreen.setStatus error " + e.getMessage() );
    }

  }

  public String getStatus()
  {
    String text = "";
    if (status != null)
      text = status.getText();

    return text;
  }


  public synchronized void setWorking(boolean isWorking)
  {
    workCounter += 
      (isWorking ? 1 : -1);

    boolean isChanging =
      ((workCounter == 1) && isWorking)  ||
      ((workCounter == 0) && !isWorking);
                                       
    if (isChanging)
    {
      working.setVisible(isWorking);
        
      if (isWorking)
        working.start();
      else
        working.stop();
    }
  }
  

  public void setBusy(boolean isBusy)
    { BApplication.setBusy(isBusy); }

  public void setBusyWorking(boolean b)
  {
    setWorking(b);
    setBusy(b);
  }
  
  public void setMonitored(BProgressMonitorable source)
    { workMonitor.setMonitored(source); }
  
  

  
  // Work Monitor class...
  // -------------------------------------------------------------
  
  protected class WorkMonitor implements ActionListener
  {
    protected Timer timer;
    protected BProgressMonitorable source;
    
    
    public WorkMonitor()
      { timer = new Timer(50, this); }
    
    public void setMonitored(BProgressMonitorable source)
    { 
      synchronized (this)
        { this.source = source; }
        
      boolean isMonitoring = 
       (this.source != null);
      
      setWorking(isMonitoring);
      progress.setVisible(isMonitoring);
      
      if (isMonitoring)
        timer.start();
      else
        timer.stop();
    }
    
    public synchronized void actionPerformed(ActionEvent e)
    {
      if (isClosed)
      {
        timer.stop();
      }
      else if (source != null)
      {
        String old = getStatus();
        String cur = source.getProgressMessage();
        
        if (!old.equals(cur))
          setStatus(cur);
        
        progress.setValue(source.getProgressPercent());
      }
    }
  }

}
