package roboforge.ui;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.ui.components.*;
import roboforge.ui.layers.*;

import java.awt.*;
import java.util.Vector;

import javax.swing.*;


public class BTooledPanel extends BSystemPanel
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BLayerStack stack;


  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "toolbar.width",           new Integer(30),
    "toolPanels.width",        new Integer(310),
  });


  // Visual elements...
  // -------------------------------------------------------------

  protected BBoxPanel toolbar       = new BBoxPanel(res, "toolbar", BoxLayout.Y_AXIS);
  protected BBoxPanel contentArea   = new BBoxPanel(BoxLayout.Y_AXIS);
  protected BTabbedPane toolPanels  = new BTabbedPane(res, "toolPanels");



  // Constructor.
  // -------------------------------------------------------------

  public BTooledPanel(BSystemScreen parent, String name)
  {
    super(parent, name);

    stack = new BMultiLayerStack(toolPanels);

    layoutTooledPanelChildren();
    configTooledPanelChildren();

    res.cleanse();
    res = null;
  }


  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutTooledPanelChildren()
  {
    setEmptyBorder(MARGIN);

    add(toolbar);
    addMargin();

    add(contentArea);
    addMargin();

    // toolPanels.setLayout(new BorderLayout());
    add(toolPanels);
  }

  protected void configTooledPanelChildren()
  {
  }


  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();
    stack.requestActivate();
  }

  public void onDeactivate()
  {
    stack.requestDeactivate();
    super.onDeactivate();
  }

  public void onClose()
  {
    stack.onClose();

    toolPanels.removeAll();
    remove(toolPanels);
    stack = null;
    toolbar = null;
    contentArea   = null;
    toolPanels  = null;

    super.onClose();

  }


  // ToolPanel windowing methods...
  // -------------------------------------------------------------

  public void selectToolPanel(BToolPanel p)
  { 
    toolPanels.setSelectedComponent(p);

    /*
    // TODO: Required?
    toolPanels.invalidate();
    toolPanels.validate();
    toolPanels.repaint();
    */
  }

  public void moveToolPanelToFront(BToolPanel p)
    { toolPanels.insertTab(p.getTitle(), null, p, null, 0); }
  
  public void openToolPanel(BToolPanel p)
  {
    p.setParent(this);
    stack.push(p);
  }

  public void hideToolPanel(BToolPanel p)
    { stack.remove(p); }

  public boolean isToolPanelOpen(BToolPanel p)
    { return stack.isOpen(p); }
}