package roboforge.ui;

import roboforge.BBot;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BPhysics;
import roboforge.BWizardTemplateBot;
import roboforge.BWizardLimbOptions;
import roboforge.BWizardLimb;
import roboforge.BWizardMemoryOptions;
import roboforge.BWizardBlockOptions;
import roboforge.BLimb;
import roboforge.BComponent;
import roboforge.ai.BAIBlock;

import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.ui.layers.*;
import roboforge.ui.modes.*;

import roboforge.math.*;
import roboforge.util.graphics.*;
import roboforge.util.io.*;
import roboforge.util.processors.*;
import roboforge.util.text.*;

import roboforge.animation.*;
import roboforge.animation.animation.*;
import roboforge.animation.animation.b3d.*;

import roboforge.BComponent;
import roboforge.btree.*;

import java.awt.*;
import java.awt.event.*;


import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.io.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.table.*;

import javax.vecmath.*;
import javax.media.j3d.*;


public class BWizardScreen extends BSystemScreen
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BWizardTemplateBot bot;
  
  protected Vector panels = new Vector();
  protected BSleepingThreadAnimator animator;
  protected BDemoAutoJoints aJoints;
  protected boolean botVisChange;

  protected boolean enteringWorkshop = false;
  
  protected Vector limbVector = new Vector();

  // UI Resources ...
  // -------------------------------------------------------------

  protected Dimension WIZARD_BUTTON_SIZE = new Dimension(75, 20);

  protected BUIResources res = new BUIResources( new Object[]
  {
    "titleIcon.icon",       BIconCache.getHSB("wizard_icon.gif"),
    "titleLabel.icon",      BIconCache.getHSB("wizard_title.gif"),

    "3dcanvas.background",  BGlobal.colors.getDarkestShade(),
    "3dcanvas.foreground",  BGlobal.colors.getWhiteShade(),
    "3dcanvas.bounds",      new Rectangle(30, 135, 340, 390),
    "3dcanvas.showfloor",   new Boolean(false),
  });



  // Visual elements...
  // -------------------------------------------------------------

  protected B3DWizardBotCanvas canvas;



  // Constructor.
  // -------------------------------------------------------------

  public BWizardScreen(BWizardTemplateBot bot)
  {
    super();

    VirtualUniverse.setJ3DThreadPriority(Thread.MIN_PRIORITY );
    BGlobal.J3DENGINESTARTED = true;
    
    this.bot = bot;

    titleIcon.setIcon(res.getIcon("titleIcon.icon"));
    titleLabel.setIcon(res.getIcon("titleLabel.icon"));

    init3DCanvas();

    BWizardPanel start = getWizPanel(0);
    setSystemPanel(start);
  }


  class BAutoSize implements BAnimation
  {

    private final double BASEZOOM = 80;

    Transform3D zoomTran = new Transform3D();
    Vector3d vec = new Vector3d();
    double delta  = 2;

    public BAutoSize ()
      { }

    public void start()
      {  vec.z = bot.getBoundingRadius() * B3DWizardBotCanvas.ZOOMFACTOR; }

    public void stop()
      {}

    public boolean animate()
    {

      TransformGroup tg = canvas.getViewTranslateTG();

      if (bot == null)
        return false;
      if (bot.getChassis() ==null)
        return false;

      if (bot.getChassis().isLive())
      {
        double r = BASEZOOM + (bot.getBoundingRadius() * B3DWizardBotCanvas.ZOOMFACTOR);

        if (r > vec.z)
          vec.z = java.lang.Math.min(r, vec.z + delta);
        else
          vec.z = java.lang.Math.max(r, vec.z - delta);

        zoomTran.setTranslation(vec);
        tg.setTransform(zoomTran);

        return true;
      }
      else
        return false;

    }


  }

  protected void init3DCanvas()
  {
    canvas = new B3DWizardBotCanvas(res, "3dcanvas");

    addOnTop(canvas);
    canvas.addBot(bot);

    animator = new BSleepingThreadAnimator(50);
    BRotY yrot = new BRotY( canvas.getViewRotateYTG(), 0.05f );
    //B3DCanvasYRotator yrot =   new B3DCanvasYRotator( 0.05f );
    aJoints = new BDemoAutoJoints (bot);

    BAutoSize autoSize = new BAutoSize();

    //BModeViewRot rot = new BModeViewRot( canvas);
    //canvas.setMode(rot);


    animator.add(autoSize);
    animator.add(yrot);
    animator.add(aJoints);
    animator.start();
  }

  protected BWizardPanel getWizPanel(int i)
  {
    if (i < panels.size())
      return (BWizardPanel) panels.get(i);

    else
    {
      BWizardPanel p = createWizPanel(i);
      panels.add(p);
      return p;
    }
  }

  
  public void setVisible(boolean v)
  {
    super.setVisible(v);

    if (v)
    {
      canvas.invalidate();
      canvas.validate();
      canvas.repaint();
    }
  }
  
  public void onClose()
  {
    if (animator != null)
     {
      animator.stop();
      animator.clear();
      animator = null;
     }


     if (!enteringWorkshop)
     {
      bot.cleanse();
      BGlobal.clearAllHardCaches();
     }

     canvas.onClose();
     canvas.removeBot();
     BSerializableTree.clearCache();

     bot = null;
     panels = null;
     

     super.onClose();
  }

  protected BWizardPanel createWizPanel(int i)
  {
    BWizardPanel result = null;

    /*
    if (i == 0)
      result = new BChooseBotPanel(this, i);
    else
    {
    */
    
      int n = getLimbPanelCount();
      int m = getAIPanelCount();

      // new AI

      if (i < n)
        result = new BChooseLimbPanel(this, bot.getLimbOptions(i), i);
      else if (i < (n+m))
//        result = new BChooseAIPanel(this, bot.getMemoryOptionAt(i-n), i);
        result = new BChooseAIPanel(this, bot.getBlockOption(i-n), i);
      else // if (i == (n+m))
        result = new BChooseNamePanel(this, i);
      
    // }

    return result;
  }

  protected int getWizPanelCount()
  {
    if (bot == null)
      return 0;
    else
      return getLimbPanelCount()+ getAIPanelCount()+ 1; // 2;
  }

  protected int getLimbPanelCount()
  {
    if (bot == null)
      return 0;
    else
      return bot.getLimbOptions().size();
  }

  protected int getAIPanelCount()
  {
    if (bot == null || bot.getBlockOptions() == null)
      return 0;
    else
 //     return bot.getMemoryOptionsCount();
      return bot.getBlockOptions().size();
  }

  public void setStatus(String text)
    { /* Ignore status messages */ }



  // -------------------------------------------------------------
  // Wizard Panel Internal Class...
  // -------------------------------------------------------------

  protected class BWizardPanel extends BSystemPanel
    implements PropertyChangeListener
  {

    // Internal variables...
    // -------------------------------------------------------------

    protected boolean isBotVisible = false;
    protected int index = 0;


    // UI Resources ...
    // -------------------------------------------------------------

    protected Dimension WIZARD_BUTTON_SIZE = new Dimension(100, 30);

    protected BUIColorScheme colors = BGlobal.colors;

    protected BUIResources res = new BUIResources( new Object[]
    {
      "canvasArea.width",           new Integer(340),
      "clientArea.width",           new Integer(330),

      "panelTitle.font",            new Font(BGlobal.FONT_NAME, Font.BOLD, 24),
      "panelTitle.foreground",      colors.getMidShade(),
      "panelTitle.height",          new Integer(50),
      "panelTitle.v_align",         new Integer(BOTTOM),

      "progress.font",              new Font("Dialog", Font.PLAIN, 14),
      "progress.foreground",        colors.getDarkShade(),
      "progress.size",              new Dimension(100, 50),
      "progress.v_align",           new Integer(BOTTOM),
      "progress.h_align",           new Integer(RIGHT),

      "costLabel.text",             "Total cost:  ",
      "costLabel.font",             new Font("Dialog", Font.PLAIN, 14),
      "costLabel.foreground",       colors.getDarkShade(),
      "costLabel.h_align",          new Integer(RIGHT),
      "costLabel.size",             new Dimension(100, 30),

      "cost.text",                  "0 " + BGlobal.MONEY_UNIT,
      "cost.size",                  new Dimension(90, 30),
      "cost.font",                  new Font("Dialog", Font.BOLD, 14),
      "cost.h_align",               new Integer(RIGHT),
      "cost.foreground",            colors.getMidShade(),

      "back.size",                  WIZARD_BUTTON_SIZE,
      "back.icon",                  BIconCache.getHSB("wiz_back_icon.gif"),
      "back.text",                  "Back",

      "next.size",                  WIZARD_BUTTON_SIZE,
      "next.icon",                  BIconCache.getHSB("wiz_next_icon.gif"),
      "next.text",                  "Next",

      "cancel.size",                WIZARD_BUTTON_SIZE,
      "cancel.text",                "Cancel",


      "botsLabel.text",             "Please choose a basic bot design:",
      "botImage.border",            BUIResources.getLineBorder(colors.getDarkerShade()),

      "AIOptionsLabel.height",      new Integer(15),

      "stateDesc.border",           new EmptyBorder(5,5,5,5),

      "limbsLabel.text",            "Please choose a limb:",
      "limbs.height",               new Integer(90),

      "limbDesc.border",            new EmptyBorder(5,5,5,5),
      "limbDesc.opaque",            Boolean.TRUE,
      "limbDesc.background",        colors.getDarkestShade(),

      "nameLabel.text",             "Please enter a name for your bot:",
      "botName.fieldsize",          new Integer(BBot.MAX_NAME_LENGTH),

      "optionDesc.border",          new EmptyBorder(5,5,5,5),
      "optionDesc.opaque",          Boolean.TRUE,
      "optionDesc.background",      colors.getDarkestShade(),

      "optDescScroll.height",       new Integer(40),

      "blockDesc.border",           new EmptyBorder(5,5,5,5),
    });


    // Visual elements...
    // -------------------------------------------------------------

    protected BBoxPanel canvasArea        = new BBoxPanel(res, "canvasArea", BoxLayout.Y_AXIS);
    protected BBoxPanel clientArea        = new BBoxPanel(res, "clientArea", BoxLayout.Y_AXIS);
    protected BBoxPanel buttons           = new BBoxPanel(BoxLayout.X_AXIS);

    protected BLabel panelTitle           = new BLabel(res, "panelTitle");
    protected BLabel progress             = new BLabel(res, "progress");
    protected BLabel costLabel            = new BLabel(res, "costLabel");
    protected BLabel cost                 = new BLabel(res, "cost");

    protected BButton back                = new BButton(res, "back");
    protected BButton next                = new BButton(res, "next");
    protected BButton cancel              = new BButton(res, "cancel");


    // Constructor.
    // -------------------------------------------------------------

    public BWizardPanel
    (
      BSystemScreen parent,
      String name,
      int index,
      boolean botVisible
    )
    {
      super(parent, name);

      this.isBotVisible = botVisible;

      this.index = index;

      layoutWizardPanelChildren();
      configWizardPanelChildren();
    }

    // Children initialization methods...
    // -------------------------------------------------------------

    protected void layoutWizardPanelChildren()
    {
      setEmptyBorder(BIG_MARGIN);

      // setVGlue(clientArea);
      BBoxPanel right = createVBoxPanel();

      add(canvasArea);
      addStrut(55);
      add(right);
      addMargin();

      right.add(clientArea);

      if (isBotVisible)
      {
        BBoxPanel costArea = createHBoxPanel();

        costArea.addGlue();
        costArea.add(costLabel);
        costArea.addSmallMargin();
        costArea.add(cost);

        right.addBigMargin();
        right.addHGlued(new BLine());
        right.addSmallMargin();
        right.add(costArea);
      }

      right.addBigMargin();
      right.add(buttons);

      setHGlue(panelTitle);

      clientArea.addLinear(panelTitle, progress);
      clientArea.addMargin();

      buttons.add(back);
      buttons.addGlue();
      buttons.add(next);
      buttons.addGlue();
      buttons.add(cancel);
    }

    protected void configWizardPanelChildren()
    {
      setPanelTitle(this.getName());

      int n = getWizPanelCount();

      if (n > 0)
        progress.setText("step "+ (index+1)+ " of "+ n);
      else
        progress.setText("step "+ (index+1));

      if (index > 0)
        back.addActionListener(new BackListener());
      else
        back.setEnabled(false);

      next.addActionListener(new NextListener());

      if (index == (getWizPanelCount()-1))
      {
        next.setText("Finish");
        next.setIcon(null);
      }

      cancel.addActionListener(new CancelListener());

      if (bot != null)
        bot.addPropertyChangeListener(this);

      next.setNextFocusableComponent(back);
      back.setNextFocusableComponent(cancel);

    }

    protected void setPanelTitle(String t)
      { panelTitle.setText(t); }

    protected void updateBotInfoUI()
      { cost.setText(BMath.format(bot.getTotalPrice()) + " "+ BGlobal.MONEY_UNIT); }


    // UI Listeners...
    // -------------------------------------------------------------

    protected class BackListener implements ActionListener
    {
      public void actionPerformed(ActionEvent e)
      {
        BDebug.print("Back button pressed.");
        onBackPressed();
      }
    }

    protected class NextListener implements ActionListener
    {
      public void actionPerformed(ActionEvent e)
      {
        BDebug.print("Next button pressed.");
        onNextPressed();
      }
    }

    protected class CancelListener implements ActionListener
    {
      public void actionPerformed(ActionEvent e)
      {
        BDebug.print("Cancel button pressed.");
        onCancelPressed();
      }
    }

    public void propertyChange(PropertyChangeEvent e)
      { updateBotInfoUI(); }


    // Activation methods...
    // -------------------------------------------------------------

    public void onActivate()
    {
      super.onActivate();

      if (bot != null)
        updateBotInfoUI();
    }

    protected void onBackPressed()
    {
      BWizardPanel panel =
        getWizPanel(index-1);

      botVisChange =
        (panel.isBotVisible != this.isBotVisible);

      setSystemPanel(panel);
    }

    protected void onNextPressed()
    {
      BWizardPanel panel =
        getWizPanel(index+1);

      botVisChange =
        (panel.isBotVisible != this.isBotVisible);

      setSystemPanel(panel);
    }



    protected void onCancelPressed()
      { BApplication.closeScreen(); }

    public void setVisible(boolean v)
    {
      super.setVisible(v);

      if (canvas != null)
      {
        if (!botVisChange)
          return;

        if (!v || !isBotVisible)
          canvas.setVisible(false);

        else if (!canvas.isVisible())
          canvas.setVisible(true);
      }
    }

  }


  // -------------------------------------------------------------
  // 1. Choose Bot Panel ...
  // -------------------------------------------------------------

  /*
  protected class BChooseBotPanel extends BWizardPanel
  {

    protected LoadBotAction loader =
      new LoadBotAction();

    protected String loadedBotFileName = null;

    protected BLabel botImage  = new BLabel(this.res, "botImage");
    protected BLabel botsLabel = new BLabel(this.res, "botsLabel");
    protected BFileComboBox bots;

    public BChooseBotPanel(BSystemScreen parent, int i)
    {
      super(parent, "Choose basic design", i, false);

      bots = new BFileComboBox();
      bots.setFilter(new BRoboforgeFilter(BGlobal.WIZSUFFIX));

      layoutChooseBotChildren();
      configChooseBotChildren();
    }

    protected void layoutChooseBotChildren()
    {
      canvasArea.addHVGlued(botImage);

      clientArea.addHGlued(botsLabel);
      clientArea.addTinyMargin();
      clientArea.add(bots);
      clientArea.addGlue();
    }

    protected void configChooseBotChildren()
    {
      try
        { bots.setSourcePath(BGlobal.WIZARDPATH); }
      catch (FileNotFoundException e)
        { } //

      bots.addActionListener(new BotsListener());

      if (!bots.isEmpty())
        bots.setSelectedIndex(0);
      else
      {
        next.setEnabled(false);
        bots.setEnabled(false);
        BApplication.notifyUser("There are no wizard bots available.");
      }
    }

    protected class BotsListener implements ActionListener
    {
      public void actionPerformed(ActionEvent e)
      {
        botImage.setIcon(
          new ImageIcon(bots.getSelectedFileWithPathNoExt()+ ".jpg"));
      }
    }

    protected void onNextPressed()
      { loader.perform(); }

    protected class LoadBotAction extends BThreadedAction
    {
      protected String filename;

      public void swingPrepAction(ActionEvent e)
      {
        filename = bots.getSelectedFileWithPath();

        if (filename.equals(loadedBotFileName))
          filename = null;
        else
        {
          parent.setStatus(
            "Loading bot '"
            + bots.getSelectedItem()
            + "', please wait...");
            
          setBusyWorking(true);
        }
      }

      public void threadedAction(ActionEvent e)
      {
        if (filename != null)
        {
          File f = new File(filename);
          if (f.canRead())
          {
            bot = BWizardTemplateBot.loadWizFromFile(filename);
            init3DCanvas();

            loadedBotFileName = filename;
          }
        }
      }

      public void swingCleanupAction(ActionEvent e)
      {
        if (filename != null)
        {
          parent.setStatus("Bot '"+ bots.getSelectedItem()+ "' loaded.");
          setBusyWorking(false);

          panels.clear();
          panels.add(BChooseBotPanel.this);
        }

        BChooseBotPanel.super.onNextPressed();
      }
    }
  }
  */

  private class LimbGlowAnimation extends BSinColorCycler
  {
    BComponent[] limbComponents;

    public LimbGlowAnimation(BLimb limb)
    {
      super(Color.black, BGlobal.ALT_HIGHLIGHT_COLOR, 30, 255);

      Vector tempV = new Vector();
      Enumeration e = limb.getBreathComponents();

      while (e.hasMoreElements() )
        tempV.add(e.nextElement() );

      limbComponents = new BComponent[tempV.size() ];

      for (int i = 0; i < tempV.size(); i++)
      {
        limbComponents[i] = ((BComponentNode)tempV.get(i)).getComponent();
        limbComponents[i].setNormalAppearance();
      }
    }

    public void start()
      { }

    public void stop()
      { }

    public boolean animate()
    {
      super.animate();

      float transparency  = super.getCycletValue();

      for (int i = 0; i < limbComponents.length; i++)
      {
        if (limbComponents[i].isLive())
        {
          limbComponents[i].setTexture(null, false);
          limbComponents[i].displayColor(thisCyclesColor);
          // limbComponents[i].setTransparency(transparency, false);
        }
      }

      return true;
    }
  }


  // -------------------------------------------------------------
  // 2. Choose Limbs ...
  // -------------------------------------------------------------

  protected class BChooseLimbPanel extends BWizardPanel
  {

    protected int lastSelectedIndex = 0;

    protected BWizardLimbOptions limbOptions;
    protected LimbTableModel model;

    // protected BLabel limbsLabel         = new BLabel(this.res, "limbsLabel");
    protected BTable limbs              = new BTable();
    protected BScrollPane limbScroller  = new BScrollPane(this.res, "limbs", limbs);

    protected BMultiLabel description   = new BMultiLabel(this.res, "limbDesc");
    protected BScrollPane descScroller  = new BScrollPane(description);

    protected LimbGlowAnimation currentGlow;

    public BChooseLimbPanel
    (
      BSystemScreen parent,
      BWizardLimbOptions options,
      int i
    )
    {
      super(parent, options.getName(), i, true);

      this.limbOptions   = options;

      layoutChooseLimbChildren();
      configChooseLimbChildren();
    }

    protected void layoutChooseLimbChildren()
    {
      // limbs.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

      // clientArea.addHGlued(limbsLabel);
      // clientArea.addTinyMargin();

      clientArea.add(limbScroller);

      clientArea.addMargin();
      clientArea.add(descScroller);

      descScroller.setBorder(null);
    }

    protected void configChooseLimbChildren()
    {
      model = new LimbTableModel(limbOptions);
      limbs.setModel(model);

      TableColumnModel columns =
        limbs.getColumnModel();

      TableColumn cost =
        columns.getColumn(1);

      cost.setMaxWidth(100);

      ListSelectionModel smodel =
        limbs.getSelectionModel();

      smodel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
      smodel.addListSelectionListener(new LimbsListener());
    }

    public void onActivate()
    {
      super.onActivate();

      BDebug.print ("onActivate pressed lastSelectedIndex " + lastSelectedIndex);

      // TODO: Assumes limb options contains 1+ limbs.
      limbs.setRowSelectionInterval(lastSelectedIndex, lastSelectedIndex);

      setBotTransparent();

      selectLimb(limbOptions.getSelectedLimb(), false);
    }

    public void onDeactivate()
    {
      BWizardLimb limb = limbOptions.getSelectedLimb();

      if (limb != null)
      {
        animator.remove(currentGlow);
        limb.setNormalAppearance();
        setLimbTransparent(limb);
        limbVector.add(limb);
        currentGlow = null;
      }

      super.onDeactivate();
    }

    protected void setBotTransparent()
    {
      BComponent [] bc = bot.getBreathComponents(true);

      for (int i = 0; i < bc.length; i++)
      {
         Color c = bc[i].getColor();
         bc[i].setTransparency(0.6f, false);
        // bc[i].displayColor(c.darker());
      }
    }

    protected void setLimbTransparent(BWizardLimb lm)
    {
      Enumeration e = lm.getBreathComponents();

      while (e.hasMoreElements())
      {
         BComponent l = ((BComponentNode)e.nextElement()).getComponent();
         Color c = l.getColor();
         l.setTransparency(0.6f, false);
      //   l.displayColor(c.darker());
      }
    }


    protected void selectLimb(BWizardLimb limb, boolean updateOptions)
    {
     BWizardLimb old =
        limbOptions.getSelectedLimb();

      if (updateOptions)
        limbOptions.setSelected(limb);

      if (old != null)
        if (old.isLive())
        {
          canvas.removeLimb(old);
          aJoints.removeLimbFromEnd(old);
        }

      if (limb != null)
      {
        if (!limb.isLive())
        {
          canvas.addLimb(limb);
          aJoints.addLimb(limb);
        }
        description.setText(limb.getDescription());

        if (currentGlow != null)
          animator.remove(currentGlow);

        if (!BGlobal.DISABLE_FLASHING_OBJECTS)
        {
          currentGlow =  new LimbGlowAnimation(limb);
          animator.add(currentGlow);
        }
        else
        {
          limb.displayColor(BGlobal.ALT_HIGHLIGHT_COLOR);
          limb.setTransparency(0f, false);
          limb.setTexture(null, false);
        }

      }
      else
        description.setText("");

    }

    protected class LimbTableModel extends AbstractTableModel
    {
      protected BWizardLimbOptions options;

      public LimbTableModel(BWizardLimbOptions options)
      {
        super();
        this.options = options;
      }

      public int getRowCount()
      {
        if (options.getAllowNoSelection())
          return options.size() + 1;
        else
          return options.size();
      }

      public int getColumnCount()
        { return 2; }

      public String getColumnName(int column)
        { return (column == 0) ? "Available options" : "Cost"; }

      public Object getValueAt(int row, int column)
      {
        if (row < options.size())
        {
          // Valid limb option.
          BWizardLimb limb =
            options.getLimb(row);

          if (column == 0)
            return limb;
          else
            return BMath.format(limb.getPrice())+ " "+ BGlobal.MONEY_UNIT;
        }
        else
        {
          // 'None' limb option.
          if (column == 0)
            return "None";
          else
            return "-";
        }
      }

    }

    protected class LimbsListener implements ListSelectionListener
    {
      public void  valueChanged(ListSelectionEvent e)
      {
        int row = limbs.getSelectedRow();
        BWizardLimb limb;

        if (row < limbOptions.size())
          limb = limbOptions.getLimb(row);
        else
          limb = null;

        selectLimb(limb, true);
        lastSelectedIndex = row;
      }
    }

    protected void onBackPressed()
    {
      selectLimb(null, false);
      super.onBackPressed();
    }
  }







  // -------------------------------------------------------------
  // 3. Choose AI ...
  // -------------------------------------------------------------

  protected class BChooseAIPanel extends BWizardPanel implements ActionListener
  {
    protected BWizardBlockOptions blockOptions;
    protected BComboBoxModel model;

    protected BMultiLabel optionDesc    = new BMultiLabel(this.res, "optionDesc");
    protected BMultiLabel blockDesc     = new BMultiLabel(this.res, "blockDesc");


    protected BScrollPane optDescScroll    = new BScrollPane(this.res, "optDescScroll", optionDesc);
    protected BScrollPane blockDescScroll  = new BScrollPane(blockDesc);

    protected BComboBox blockCombo = new BComboBox();


    public BChooseAIPanel
    (
      BSystemScreen parent,
      BWizardBlockOptions options,
      int i
    )
    {
      super(parent, options.getName(), i, true);

      this.blockOptions = options;

      model = new BComboBoxModel(blockOptions.getBlocks());
      blockCombo.setModel(model);

      layoutChooseAIChildren();
      configChooseAIChildren();
    }

    protected void layoutChooseAIChildren()
    {
      clientArea.add(optDescScroll);
      clientArea.addMargin();
      clientArea.add(blockCombo);
      clientArea.addSmallMargin();
      clientArea.addVGlued(blockDescScroll);

      blockDescScroll.setBorder(null);
      optDescScroll.setBorder(null);
    }

    protected void configChooseAIChildren()
    {
      optionDesc.setText(blockOptions.getDescription());
      blockCombo.addActionListener(this);
    }

    public void actionPerformed(ActionEvent e)
    {
      if(e.getSource() == blockCombo)
        selectBlock( (BAIBlock)blockCombo.getSelectedItem(), true );
    }

    public void onActivate()
    {
      super.onActivate();
      if(blockOptions.getSelectedBlock() == null)
        blockOptions.setSelected( (BAIBlock)blockOptions.getBlocks().elementAt(0));

      selectBlock(blockOptions.getSelectedBlock(), false);
    }

    protected void selectBlock(BAIBlock block, boolean updateOptions)
    {
      if (updateOptions)
        blockOptions.setSelected(block);

      if (block != null)
        blockDesc.setText(block.getDescription());
      else
        blockDesc.setText("");
    }

    protected void onBackPressed()
    {
      selectBlock(null, false);
      super.onBackPressed();
    }
  }


  // -------------------------------------------------------------
  // 4. Choose Name ...
  // -------------------------------------------------------------

  protected class BChooseNamePanel extends BWizardPanel
  {

    protected BLabel nameLabel  = new BLabel(this.res, "nameLabel");
    protected BTextField name   = new BFilenameField(this.res, "botName");

    public BChooseNamePanel
    (
      BSystemScreen parent,
      int i
    )
    {
      super(parent, "Enter bot name", i, true);

      layoutChooseNameChildren();
      configChooseNameChildren();
    }

    protected void layoutChooseNameChildren()
    {
      clientArea.addHGlued(nameLabel);
      clientArea.addTinyMargin();
      clientArea.add(name);
      clientArea.addGlue();
    }

    protected void configChooseNameChildren()
    {
      name.setNextFocusableComponent(next);
    }

    protected void onNextPressed()
    {
      bot.setName(name.getText());

      BProcessor p = new BProcessor()
      {
        public void process()
          { saveWizardBot(); }
      };

      p.start();
    }

    public void saveWizardBot()
    {
      BDebug.print("saveWizardBot()");

      String name =
        BStrings.convertToFilename(bot.getName());

      String fileName =
        BGlobal.BOTPATH+ name+ BGlobal.BOTSUFFIX;

      if (bot.getName().equals(""))
      {
        BPromptDialog msgDialog =
          BPromptDialog.createOKDialog
          (
            "bot has no name...",
            "You must enter a name for your bot."
          );

        BApplication.showDialog(msgDialog);
        return;
      }

      File f = new File(fileName);

      if (!f.exists())
        saveBot(fileName);
      else
      {
        BPromptDialog replaceDialog =
          BPromptDialog.createYesNoDialog
          (
            "replace bot...",
            "A bot called '"+ bot.getName()+ "' already exists."+ "\n"+
            "Replace it with the bot you've just created?"
          );

        BApplication.showDialog(replaceDialog);

        boolean isReplacing =
          replaceDialog.getResult();

        if (isReplacing)
          saveBot(fileName);
      }
    }

   protected BWorkshopScreen workshopScreen;
   private transient boolean finishedonce = false;

   public boolean saveBot(String fileName)
   {
    if (!finishedonce )
    {
      finishedonce = true;

      setBusyWorking(true);
      enteringWorkshop = true;

      if (animator != null)
        animator.stop();


      bot.convertToBaseBot();
      BBot saveBot = new BBot(bot);

      try
        { saveBot.saveToFile(fileName); }
      catch (IOException e)
        { return false; }

      canvas.removeBot();
      // bot.cleanse();

      workshopScreen =
        new BWorkshopScreen(saveBot);

      Runnable closeScreen = new Runnable()
      {
        public void run()
        {
          // Go to workshop screen.
          BApplication.closeScreen();
          BApplication.openScreen(workshopScreen);
          
          workshopScreen = null;
        }
      };

      setBusyWorking(false);
      SwingUtilities.invokeLater(closeScreen);
    }

    return true;
  }
 }
}

