package roboforge.ui;

//import roboforge.B3dShapeManager;
import roboforge.BBot;
import roboforge.BChassis;
import roboforge.BDebug;
import roboforge.BGhost;
import roboforge.BGlobal;
//import roboforge.BTextures;
import roboforge.BWizardTemplateBot;
import roboforge.BConnectPointShape;

import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.btree.*;
import roboforge.ui.modes.*;

import roboforge.util.graphics.*;
//import roboforge.util.processors.*;

import roboforge.animation.*;
import roboforge.math.*;

//import roboforge.ui.events.*;
import roboforge.util.io.*;
import roboforge.util.text.*;
//import roboforge.animation.*;

import java.awt.*;
import java.awt.event.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.io.*;
import javax.swing.*;
import javax.media.j3d.*;
//import java.net.*;






public class BWorkshopScreen extends BSystemScreen
  implements SwingConstants, PropertyChangeListener
{

  // Internal variables...
  // -----------------------ener--------------------------------------

  // The bot being created / edited in the workshop.
  protected BBot bot = null;

  // Whether or not bot has been altered in workshop.
  protected boolean isBotChanged = true;

  // A Animator for all the workshops children to use
  protected BSleepingThreadAnimator animator =
    new BSleepingThreadAnimator(40, true);


  // Modes...
  // -------------------------------------------------------------

  protected BModeMouse modeSelect;
  protected BModeMouse modeViewRotate;
  protected BModeMouse modeViewPan;
  protected BModeMouse modeViewZoom;
  protected BModeMouse modeRotateComponent;




  // UI Resources ...
  // -------------------------------------------------------------

  protected static BUIColorScheme colors = BGlobal.colors;

  protected BUIResources res = new BUIResources( new Object[]
  {
    "titleIcon.icon",       BIconCache.getHSB("workshop_icon.gif"),
    "titleLabel.icon",      BIconCache.getHSB("workshop_title.gif"),

    "design.text",          "design",
    "design.tooltip",       "Design your bot's body.",

    "general.text",         "profile",
    "general.tooltip",      "The profile for your bot",

    "moves.text",           "moves",
    "moves.tooltip",        "Make your bot move.",

    "ai.text",              "a.i.",
    "ai.tooltip",           "Give your bot artificial intelligence.",

    "test.text",            "test",
    "test.tooltip",         "Test your bot against others.",

    "costtext.text",        "cost:",
    "costtext.fontstyle",   new Integer(Font.BOLD),
    "costtext.h_align",     new Integer(SwingConstants.RIGHT),
    "costtext.width",       new Integer(50),
    "costlabel.width",      new Integer(70),
    "costlabel.foreground", colors.getMidShade(),

    "weighttext.text",     "weight:",
    "weighttext.fontstyle", new Integer(Font.BOLD),
    "weighttext.h_align",   new Integer(SwingConstants.RIGHT),
    "weighttext.width",     new Integer(50),
    "weightlabel.width",    new Integer(50),
    "weightlabel.foreground", colors.getMidShade(),

    "healthtext.text",      "health:",
    "healthtext.fontstyle", new Integer(Font.BOLD),
    "healthtext.h_align",   new Integer(SwingConstants.RIGHT),
    "healthtext.width",     new Integer(50),
    "healthlabel.width",    new Integer(50),
    "healthlabel.foreground", colors.getMidShade(),

    "3dcanvas.bounds",      new Rectangle(61, 151, 398, 388),
    "3dcanvas.background",  colors.getExtraDarkShade(),
    "3dcanvas.foreground",  colors.getMidShade(),
    "3dcanvas.showfloor",   new Boolean(true),

    "save.name",            "Save",
    "save.tooltip",         "Save your bot to file.",
    "save.accelerator",     KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK),

    "help.name",            "Help",
    "help.icon",            BIconCache.getHSB("help_icon.gif"),
    "help.tooltip",         "Help",
    "help.accelerator",     KeyStroke.getKeyStroke(KeyEvent.VK_H, Event.CTRL_MASK),

    "manual.name",           "Manual",
    "manual.icon",           BIconCache.getHSB("manual_icon.gif"),
    "manual.tooltip",        "Manual",

    "saveas.name",          "Save As",
    "saveas.icon",          BIconCache.getHSB("save_icon.gif"),
    "saveas.accelerator",   KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK | Event.SHIFT_MASK),
    "saveas.tooltip",       "Save your bot to file.",
  });



  // Visual elements ...
  // -------------------------------------------------------------

  protected BConnectorButton designButton = new BConnectorButton(res, "design");
  protected BConnectorButton generalButton = new  BConnectorButton(res, "general");
  protected BConnectorButton movesButton  = new BConnectorButton(res, "moves");
  protected BConnectorButton aiButton     = new BConnectorButton(res, "ai");
  protected BConnectorButton testButton   = new BConnectorButton(res, "test");

  protected BBodyPanel designPanel        = null;
  protected BGeneralPanel generalPanel    = null;
  protected BMovesPanel movesPanel        = null;
  protected BAIPanel aiPanel              = null;
  protected BTestPanel testPanel          = null;

  protected JLabel costTextLabel          = new BLabel(res, "costtext");
  protected JLabel costLabel              = new BLabel(res, "costlabel");
  protected JLabel weightTextLabel        = new BLabel(res, "weighttext");
  protected JLabel weightLabel            = new BLabel(res, "weightlabel");
  protected JLabel healthTextLabel        = new BLabel(res, "healthtext");
  protected JLabel healthLabel            = new BLabel(res, "healthlabel");

  protected BSystemButton saveas          = new BSystemButton(res, "saveas");
  protected BSystemButton help            = new BSystemButton(res, "help");
  protected BSystemButton manual          = new BSystemButton(res, "manual");


  // Dialogs...
  // -------------------------------------------------------------

  protected BPromptDialog saveDialog =
    BPromptDialog.createYesNoDialog("save bot...",
      "Do you wish to save the changes to your bot?");


  // Visual elements ...
  // -------------------------------------------------------------

  protected HelpAction   helpAction       = new HelpAction(res, "help");
  protected ManualAction manualAction     = new ManualAction(res, "manual");
  protected SaveAction   saveAction       = new SaveAction(res, "save");
  protected SaveAsAction saveAsAction     = new SaveAsAction(res, "saveas");



  // 3D Canvas stuff.
  // -------------------------------------------------------------

  protected B3DBotCanvas canvas = null;


  // Constructor.
  // -------------------------------------------------------------

  public BWorkshopScreen(BBot bot)
  {
    super();
    //MDF disable sound
    BDebug.print("entering workshop - disable sound") ;
    BGlobal.USE_SOUND = false;
    //end MDF
    
    VirtualUniverse.setJ3DThreadPriority(Thread.MIN_PRIORITY );
    BGlobal.J3DENGINESTARTED = true;


    this.bot = bot;

    canvas = new B3DBotCanvas(res, "3dcanvas");

    setupModes();

    layoutWorkshopChildren();
    configWorkshopChildren();

    res.cleanse();
    res = null;

    BConnectPointShape.setAnimator( new BWorkshopCanvasAnimator(canvas, true)  );
    BSerializableTree.clearCache();
    BDebug.printFreeMem();
  }

  protected void finalize()
    { System.out.println("BWorkshopScreen.finalize()"); }




  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutWorkshopChildren()
  {
    // Add canvas to workshop screen.
    // Since it's heavyweight, it'll always be on top of
    // lightweight swing components...
    
    addOnTop(canvas); 
    

    titleIcon.setIcon(res.getIcon("titleIcon.icon"));
    titleLabel.setIcon(res.getIcon("titleLabel.icon"));

    statusBar.addVGlued(costTextLabel);
    statusBar.addSmallMargin();
    statusBar.addVGlued(costLabel);

    statusBar.addVGlued(weightTextLabel);
    statusBar.addSmallMargin();
    statusBar.addVGlued(weightLabel);

    statusBar.addVGlued(healthTextLabel);
    statusBar.addSmallMargin();
    statusBar.addVGlued(healthLabel);

    systemButtons.add(saveas, 0);
    systemButtons.add(help, 0);
    systemButtons.add(systemButtons.getMargin(),1);

    systemButtons.add(manual, 0);
    systemButtons.add(systemButtons.getMargin(),1);


  }

  protected void configWorkshopChildren()
  {
    setStatus("Entering workshop...");

    // TODO: Lazy initialization!
    designPanel   = new BBodyPanel(this, canvas, bot);
    movesPanel    = new BMovesPanel(this, canvas, bot);
    generalPanel  = new BGeneralPanel(this, canvas, bot);
    aiPanel       = new BAIPanel(this, bot);
    testPanel     = new BTestPanel(this, bot);

    addSystemPanel(designPanel, designButton);
    addSystemPanel(movesPanel , movesButton);
    addSystemPanel(aiPanel, aiButton);
    addSystemPanel(generalPanel, generalButton);
    addSystemPanel(testPanel, testButton);

    // Get initial bot info. values.
    updateBotInfoUI();

    // Listen to bot changes...
    bot.addPropertyChangeListener(this);

    setSystemPanel(designPanel);
    designButton.setSelected(true);

    canvas.addBot(bot);

    saveas.addActionListener(saveAsAction);
    help.setAction(helpAction);
    manual.setAction(manualAction);

    // Setup canvas tooltip system...
    canvas.setRelayListener(
      new BMouseRelayListener(canvas, this));
    
    setToolTipText("");

    // Set up save action...
    InputMap   inputs     = getInputMap(WHEN_IN_FOCUSED_WINDOW); // WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    ActionMap  actions    = getActionMap();
    Object     saveName   = saveAction.getName();
    KeyStroke  saveKey    = saveAction.getAcceleratorKey();

    inputs.put(saveKey, saveName);
    actions.put(saveName, saveAction);

  }


  protected void setupModes()
  {
    modeSelect          = new BModeViewer(canvas, new BModeSelect(canvas));
    modeViewRotate      = new BModeViewer(canvas, new BModeViewRot(canvas));
    modeViewPan         = new BModeViewer(canvas, new BModeViewPan(canvas));
    modeViewZoom        = new BModeViewer(canvas, new BModeViewZoom(canvas));
    modeRotateComponent = new BModeViewer(canvas, new BModeConnectRot(canvas));
  }


  
  // UI Actions...
  // -------------------------------------------------------------
  
  protected class SaveAction extends BAction
  {
    public SaveAction(BUIResources res, String key)
      { super(res, key); }

    public void actionPerformed(ActionEvent e)
      { saveBot(bot.getName()); }
  }

  protected class ManualAction extends BAction
  {
     public ManualAction(BUIResources res, String key)
      { super(res, key); }

    public void actionPerformed(ActionEvent e)
    {
          try
          {
            BDebug.print (BGlobal.SYSTEMPATH + "reader/reader/AcroRd32.exe " + BGlobal.HELPPATH + "help.pdf");
            Runtime.getRuntime().exec(BGlobal.SYSTEMPATH + "reader/reader/AcroRd32.exe " + BGlobal.HELPPATH + "help.pdf");

          }
          catch (Exception ex)
            { BDebug.print(ex.getMessage() );  }
    }


  }

  protected class HelpAction extends BAction
  {
    public HelpAction(BUIResources res, String key)
      { super(res, key); }

    public void actionPerformed(ActionEvent e)
      {
        try
        {

          // Write loading parameters to text file for Flash Loader to read
          FileWriter fileOut = new FileWriter( BGlobal.HELPPATH + "load.txt" ,false);
          String currentPanel = (BWorkshopScreen.this.getSystemPanel()).getName();
          String helpset = "&file="+currentPanel.toLowerCase() + "&";
          fileOut.write(helpset+'\n');
          fileOut.close();

          // Execute Flash Player
          Runtime runner = Runtime.getRuntime();
          Process flashPlayer = runner.exec( BGlobal.HELPPATH + "help.exe" );
          flashPlayer.waitFor();
          runner = null;
          
        } catch (Exception e2) {
          BDebug.print("Help cannot execute");
        }
        
      }
  }


  protected class SaveAsAction extends BThreadedAction
  {

    protected BFileDialog saveDlg =
      BFileDialog.createSaveDialog("save bot",
        "Save your bot as:", BBot.MAX_NAME_LENGTH );

    protected BPromptDialog replaceDlg =
      BPromptDialog.createYesNoDialog("replace bot?", "");


    public SaveAsAction(BUIResources res, String key)
    {
      super(res, key);

      saveDlg.setOKDefault();
      saveDlg.setFilter(new BRoboforgeFilter(BGlobal.BOTSUFFIX));
    }

    public void swingPrepAction(ActionEvent e)
    {
      try
      {
        String path;

        if (BGlobal.CONSTRUCTING_CHASSISBOT)
          path = BGlobal.CHASSISBOTPATH ;
        else if (bot.getType() == BWizardTemplateBot.BOT_TYPE )
          path = BGlobal.WIZARDPATH;
        else
          path = BGlobal.BOTPATH;

        saveDlg.setPath(path);
        BDebug.print ("BWorkshopScreen saveasAction bot.getNAme() " + bot.getName() );
        saveDlg.setSelectedFile(bot.getName());
      }
      catch (Exception ex)
        { BApplication.notifyUser("Could not save bot."); }
    }

    public void threadedAction(ActionEvent e)
    {
      BApplication.showDialog(saveDlg);

      String filename = saveDlg.getSelectedFile();
      String name     = saveDlg.getSelectedName();

      // Check for user cancel...
      if (filename == null)
        return;

      boolean nameChanged =
        !name.equalsIgnoreCase(bot.getName());

      if (filename != null)
      {
        File f = new File(filename);
        if (f.exists() && nameChanged)
        {
          replaceDlg.setMessage(
            "A bot named "+ name+ " already exists. \n\n"+
            "Do you want to replace it with the current bot?");

          BApplication.showDialog(replaceDlg);

          if (!replaceDlg.getResult())
            filename = null;
        }

        if (filename != null)
          saveBot(name);
      }
    }
  }


  public String getToolTipText(MouseEvent e)
  {
    // HACK:
    // Because the canvas is not a swing component and thus can't have
    // a tooltip, we register this screen as having a tooltip instead.
    // Mouse events are relayed from the canvas to here. We delegate
    // tooltip-text requests back to the canvas, if appropriate.

    int x = e.getX() - canvas.getX();
    int y = e.getY() - canvas.getY();

    if (canvas.contains(x, y))
      return canvas.getToolTipText(x, y);
    else
      return getToolTipText();
  }



  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();

    canvas.invalidate();
    canvas.validate();
    canvas.repaint();
  }

  public void onDeactivate()
  {
    animator.stop();
    super.onDeactivate();
  }


  public void onClose()
  {


    animator.stop();
    animator = null;
    
    // Allow BSubScreen to remove client panels...
    super.onClose();

    cleanse();

    BDebug.print("BWorkshopScreen.onClose GC'ing..");
    //MDF enable sound
    BDebug.print("closing workshop - enable sound") ;
    BGlobal.USE_SOUND = true;
    //end MDF
    System.gc();
  }

  private void cleanse()
  {
    canvas.requestClose();
    canvas.removeBot();

    saveas.removeActionListener(saveAsAction);
    saveas = null;
    saveAsAction = null;

    remove(canvas);
    canvas = null;

    BGlobal.clearAllHardCaches();

    // Remove bot from memory...

    bot.cleanse();
    bot = null;

    modeSelect = null;
    modeViewRotate = null;
    modeViewPan = null;
    modeViewZoom = null;
    modeRotateComponent = null;

    res = null;

    designButton = null;
    generalButton = null;
    movesButton  = null;
    aiButton     = null;
    testButton   = null;

    designPanel        = null;
    generalPanel    = null;
    movesPanel        = null;
    aiPanel              = null;
    testPanel          = null;

    costTextLabel          = null;
    costLabel              = null;
    weightTextLabel        = null;
    weightLabel            = null;
    healthTextLabel        = null;
    healthLabel            = null;
    saveDialog =  null;

    saveAction       = null;
    saveAsAction     =null;

  }


  public void onSaveBegin()
  {
    movesPanel.onSaveBegin();
    generalPanel.onSaveBegin();
    aiPanel.onSaveBegin();

  }

  public void onSaveEnd()
  {
    movesPanel.onSaveEnd();
    generalPanel.onSaveEnd();
    aiPanel.onSaveEnd();
  }


  protected void onBackPressed()
  {
    //if (botNeedsSaving())
     // saveBot(bot.getName());

    super.onBackPressed();
  }

  protected void saveBot(String name)
  {
    setBusyWorking(true);

    String path;

    if (BGlobal.CONSTRUCTING_CHASSISBOT)
      path = BGlobal.CHASSISBOTPATH ;
    else if (bot.getType() == BWizardTemplateBot.BOT_TYPE )
      path = BGlobal.WIZARDPATH;
    else
      path = BGlobal.BOTPATH;

    String filename =
      path+ BStrings.convertToFilename(name);

    File f = new File(filename);

    try
    {
      onSaveBegin();
      bot.setName(name);
      bot.saveToFile(filename);
      onSaveEnd();

      setStatus("Saved "+ bot.getName()+ ".");
      isBotChanged = false;
    }
    catch (IOException e)
    {
      setStatus("Error: could not save "+ bot.getName()+ ".");
      BDebug.print("Exception: "+ e);

      BApplication.notifyUser("save error", "Could not save bot "+ bot.getName()+ ".");
    }

    setBusyWorking(false);
  }


  protected boolean botNeedsSaving()
  {
   /* if (isBotChanged)
    {
      BApplication.showDialog(saveDialog);
      return saveDialog.getResult();
    }
    else    */
      return false;
  }

  public void autosave()
  {
    if ( BGlobal.AUTOSAVE )
    {
      //BDebug.print("Autosave Executed.");

      setBusy(true);
      //setStatus("Autosaving - please wait...");
      onSaveBegin();

      try
        { bot.saveToFile(BGlobal.BOTPATH + "_autosave_"); }
      catch (Exception e)
        { BDebug.print("BWorkshopScreen.autosave: error "+e.getMessage() ); }

      onSaveEnd();
      setBusy(false);
      //setStatus("Autosave Completed.");
    }

  }



  // Housekeeping methods...
  // -------------------------------------------------------------

  public void propertyChange(PropertyChangeEvent e)
    { updateBotInfoUI(); }

  protected void updateBotInfoUI()
  {
    float price  = bot.getTotalPrice();
    float weight = bot.getTotalWeight();
    float health = bot.getTotalHealth();

    costLabel.setText(BMath.format(Math.round(price))+ " "+ BGlobal.MONEY_UNIT);
    weightLabel.setText(BMath.format(weight)+ " kg");
    healthLabel.setText(BMath.format(health)+ " hp");

    // SJS [17/07/2000] - Give visual feedback when chassis overloaded.
    BChassis chassis = bot.getChassis();

    if ((chassis != null) && (weight > chassis.getLoadCapacity()))
      weightLabel.setForeground(Color.red);
    else
      weightLabel.setForeground(colors.getMidShade());
  }
  

  // Accessor methods...
  // -------------------------------------------------------------

  public BBot getBot()
    { return bot; }

  public B3DBotCanvas getCanvas()
    { return canvas; }

  public BAnimator getAnimator()
    { return animator; }


  protected boolean confirmBackWithUser()
  {

    BApplication.showDialog(saveDialog);
    if ( saveDialog.getResult() )
      saveBot(bot.getName());

    return  saveDialog.getUserSelectedOption();


  }


  // Ghosting methods...
  // -------------------------------------------------------------
  
  public BGhost createGhost()
  { 
    BBot b = this.bot;
    
    requestClose();
    return new BWorkshopGhost(b); 
  }
  
  protected static class BWorkshopGhost implements BGhost
  {
    //protected BBot bot;

    public BWorkshopGhost(BBot bot)
    {
      // bot.saveToFile is now done in BTestPanel.viewBAttle
      // this is so we can cleanse the bot normally onClose.


    /*
      try
        { bot.saveToFile(BGlobal.USERPATH + "bCache"); }
      catch (Exception e)
        { BDebug.print(" BWorkshopGhost.constructor bot load error " ); }  */
    }
    
    public Object resurrect()
    {
      BBot bot = BBot.loadFromFile(BGlobal.USERPATH + "bCache.bot", false);
      bot.init(false);
      BDebug.print("BWorkshopGhost.resurrect()");
      return new BWorkshopScreen(bot); 
    }
  
    public void banish()
      { } //bot = null;
  }


  // animator class

  public class BWorkshopCanvasAnimator  extends Bj3dFrameElapsedAnimator
  {
      B3DBotCanvas canvas;

      public BWorkshopCanvasAnimator (B3DBotCanvas canvas, boolean autoActive)
      {
        super(canvas, autoActive);
        this.canvas = canvas;
      }

      public void start()
      {
        super.start();
        canvas.addBehavior(this.playerLoop);
      }

      public void stop()
      {
        super.stop();
        canvas.removeBehavior(this.playerLoop);
      }

  }


}
