package roboforge.ui.ai;

import roboforge.BBot;
import roboforge.BDebug;
import roboforge.BGlobal;

import roboforge.ai.BDecisionTreeAI;
import roboforge.ai.decisions.*;
import roboforge.ai.values.*;
import roboforge.ai.memory.*;


import roboforge.ui.BToolPanel;
import roboforge.ui.events.*;
import roboforge.ui.components.*;
import roboforge.ui.dnd.*;
import roboforge.util.graphics.BGraphics;

import roboforge.util.graphics.BIconCache;

import javax.swing.ImageIcon;

import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.Dimension;
import java.awt.*;
import javax.swing.border.Border;



public class BAINodeLibrary extends BToolPanel
{

  protected BBot bot;
  private static int NODE_BOX_SIZE = 45;


  private  Color MEMORY_COLOR = BGlobal.colors.getAIMemoryColor();
  private  Color ACTION_COLOR = BGlobal.colors.getAIActionColor();
  private  Color GROUP_COLOR  = BGlobal.colors.getAIGroupColor();
  private  Color DECISION_COLOR  = BGlobal.colors.getAIDecisionColor_Me();

  protected BUIResources res = new BUIResources(new Object[]
  {
    "add_chassis.icon",             BIconCache.getHSB("ai/chassis_move_30.gif"),
    "add_chassis.tooltip",          "Chassis move action",
    "add_chassis.borderColor",      ACTION_COLOR,

    "add_move.icon",                BIconCache.getHSB("ai/move_30.gif"),
    "add_move.tooltip",             "Bot move action",
    "add_move.borderColor",         ACTION_COLOR,

    "add_orient.icon",              BIconCache.getHSB("ai/orient_30.gif"),
    "add_orient.tooltip",           "Orient action",
    "add_orient.borderColor",       ACTION_COLOR,

    "add_group.icon",               BIconCache.getHSB("ai/group_30.gif"),
    "add_group.tooltip",            "Group block.",
    "add_group.borderColor",       GROUP_COLOR,

    "add_auto_zone.icon",           BIconCache.getHSB("ai/zone_group_30.gif"),
    "add_auto_zone.tooltip",        "Auto update zone-group",
    "add_auto_zone.borderColor",    GROUP_COLOR,

    "health.icon",                  BIconCache.getHSB("ai/health_30.gif"),
    "health.tooltip",               "Health decision",
    "health.borderColor",           DECISION_COLOR,

    "energy.icon",                  BIconCache.getHSB("ai/energy_30.gif"),
    "energy.tooltip",               "Energy decision",
    "energy.borderColor",           DECISION_COLOR,

    "distance.icon",                BIconCache.getHSB("ai/distance_30.gif"),
    "distance.tooltip",             "Distance decision",
    "distance.borderColor",         DECISION_COLOR,

    "angle.icon",                   BIconCache.getHSB("ai/angle_30.gif"),
    "angle.tooltip",                "Angle decision",
    "angle.borderColor",            DECISION_COLOR,

    "zone.icon",                   BIconCache.getHSB("ai/zone_30.gif"),
    "zone.tooltip",                "Zone decision",
    "zone.borderColor",             DECISION_COLOR,

    "time.icon",                   BIconCache.getHSB("ai/time_30.gif"),
    "time.tooltip",                "Time decision",
    "time.borderColor",             DECISION_COLOR,

    "collision.icon",              BIconCache.getHSB("ai/collision_30.gif"),
    "collision.tooltip",           "Collision decision",
    "collision.borderColor",       DECISION_COLOR,

    "reach.icon",                  BIconCache.getHSB("ai/reach_30.gif"),
    "reach.tooltip",               "Reach decision",
    "reach.borderColor",           DECISION_COLOR,

    "component.icon",              BIconCache.getHSB("ai/component_30.gif"),
    "component.tooltip",           "Component decision",
    "component.borderColor",        DECISION_COLOR,

    "cost.icon",                   BIconCache.getHSB("ai/cost_30.gif"),
    "cost.tooltip",                "Cost decision",
    "cost.borderColor",             DECISION_COLOR,

    "detected.icon",               BIconCache.getHSB("ai/detected_30.gif"),
    "detected.tooltip",            "Detected decision",
    "detected.borderColor",         DECISION_COLOR,

    "complex.icon",                BIconCache.getHSB("ai/complex_decision_30.gif"),
    "complex.tooltip",             "Complex decision",
    "complex.borderColor",         DECISION_COLOR,

    "moveCompleted.icon",         BIconCache.getHSB("ai/move_complete_30.gif"),
    "moveCompleted.tooltip",      "Move completed decision",
    "moveCompleted.borderColor",   DECISION_COLOR,

    "isBlind.icon",                BIconCache.getHSB("ai/isBlind_30.gif"),
    "isBlind.tooltip",             "Enemy is blind decision",
    "isBlind.borderColor",         DECISION_COLOR,

    "weight.icon",                BIconCache.getHSB("ai/weight_30.gif"),
    "weight.tooltip",             "Weight decision",
    "weight.borderColor",         DECISION_COLOR,

    "memNumber.icon",            BIconCache.getHSB("ai/mem_item_number_30.gif"),
    "memNumber.tooltip",         "Remember number action",
    "memNumber.borderColor",     MEMORY_COLOR,

    "memString.icon",            BIconCache.getHSB("ai/mem_item_string_30.gif"),
    "memString.tooltip",         "Remember text action",
    "memString.borderColor",     MEMORY_COLOR,

    "memBoolean.icon",            BIconCache.getHSB("ai/mem_item_yn_30.gif"),
    "memBoolean.tooltip",         "Remember y or n",
    "memBoolean.borderColor",     MEMORY_COLOR,

  });

  // actions
  protected BDecisionNodeDragDropButton addChassis;
  protected BDecisionNodeDragDropButton addMove;
  protected BDecisionNodeDragDropButton addOrient;

  // groups
  protected BDecisionNodeDragDropButton addGroup;
  protected BDecisionNodeDragDropButton addAutoZone;

  // decisions
  protected BDecisionNodeDragDropButton health;
  protected BDecisionNodeDragDropButton energy;
  protected BDecisionNodeDragDropButton distance;
  protected BDecisionNodeDragDropButton angle;

  protected BDecisionNodeDragDropButton zone;
  protected BDecisionNodeDragDropButton time;
  protected BDecisionNodeDragDropButton collision;
  protected BDecisionNodeDragDropButton reach;

  protected BDecisionNodeDragDropButton component;
  protected BDecisionNodeDragDropButton cost;
  protected BDecisionNodeDragDropButton detected;
  protected BDecisionNodeDragDropButton complex;

  protected BDecisionNodeDragDropButton moveCompleted;
  protected BDecisionNodeDragDropButton weight;
  protected BDecisionNodeDragDropButton isBlind;

// memoryItems
  protected BDecisionNodeDragDropButton memNumber;
  protected BDecisionNodeDragDropButton memString;
  protected BDecisionNodeDragDropButton memBoolean;


  public BAINodeLibrary( BBot bot )
  {
    super("nodes");
    this.bot = bot;
    layoutNodeLibraryChildren();
    configNodeLibraryChildren();

  }

  protected void initButtons()
  {
    addChassis  = new BDecisionNodeDragDropButton(res, "add_chassis",new BChassisActionNode(bot));
    addMove     = new BDecisionNodeDragDropButton(res, "add_move", new BMoveActionNode(bot));
    addOrient   = new BDecisionNodeDragDropButton(res, "add_orient", new BOrientActionNode(bot));

    addGroup = new BDecisionNodeDragDropButton(res, "add_group", new BGroupNode());
    addAutoZone = new BDecisionNodeDragDropButton(res, "add_auto_zone", new BZoneGroupNode(bot.getZones(), (BDecisionTreeAI)bot.getAI()) );

    health = new BDecisionNodeDragDropButton(res, "health",
                                            new BSimpleDecisionNode("me.health", "=", new BNumber(100), "ai/health_18.gif", "Health decision")
                                            );


    energy = new BDecisionNodeDragDropButton(res, "energy",
                                            new BSimpleDecisionNode("me.energy", "=", new BNumber(100), "ai/energy_18.gif", "Energy decision")
                                            );

    distance = new BDecisionNodeDragDropButton(res, "distance",
                                            new BSimpleDecisionNode("me.distanceToEnemy", "=", new BNumber(10), "ai/distance_18.gif", "Distance decision")
                                            );

    angle  = new BDecisionNodeDragDropButton(res, "angle",
                                            new BSimpleAngleNode("me.angleToEnemy", "<", new BNumber(10), new BNumber(0), "ai/angle_18.gif", "Angle decision")
                                            );


    zone  = new BDecisionNodeDragDropButton(res, "zone",
                                            new BSimpleDecisionNode(".isActive", ".isActive", new BString(""), "ai/zone_18.gif", "Zone decision")
                                            );


    time = new BDecisionNodeDragDropButton(res, "time",
                                            new BSimpleDecisionNode("timeElapsed", ">", new BNumber(10), "ai/time_18.gif", "Time decision")
                                            );

    collision = new BDecisionNodeDragDropButton(res, "collision",
                                            new BSimpleDecisionNode("me.timeSinceCollision", "<", new BNumber(1), "ai/collision_18.gif", "Collision decision")
                                            );

    reach = new BDecisionNodeDragDropButton(res, "reach",
                                             new BSimpleDecisionNode("enemy.reach", ">", new BNumber(1), "ai/reach_18.gif", "Reach decision")
                                            );


    component = new BDecisionNodeDragDropButton(res, "component",
                                             new BSimpleDecisionNode("me.components", "<", new BNumber(10), "ai/component_18.gif", "Component health decision")
                                            );

    cost = new BDecisionNodeDragDropButton(res, "cost",
                                             new BSimpleDecisionNode("enemy.cost", "=", new BNumber(1000), "ai/cost_18.gif", "Cost decision")
                                            );


    detected = new BDecisionNodeDragDropButton(res, "detected",
                                             new BSimpleDecisionNode("enemy.isDetected", "=", new BBoolean(true), "ai/detected_18.gif", "Detected decision")
                                            );

    complex = new BDecisionNodeDragDropButton(res, "complex", new BDecisionNode());


    moveCompleted = new BDecisionNodeDragDropButton(res, "moveCompleted",
                                             new BSimpleDecisionNode("me.moves.x.moveCompleted", "move1", new BBoolean(true), "ai/move_complete_18.gif", "Move decision")
                                            );

    weight = new BDecisionNodeDragDropButton(res, "weight",
                                             new BSimpleDecisionNode("enemy.weight", "=", new BNumber(1000), "ai/weight_18.gif", "Weight decision")
                                            );

    isBlind = new BDecisionNodeDragDropButton(res, "isBlind",
                                             new BSimpleDecisionNode("enemy.isBlind", "", new BBoolean(true), "ai/isBlind_18.gif", "Blind decision")
                                            );

    memNumber     = new BDecisionNodeDragDropButton(res, "memNumber", new BAssignmentNode(new  BMemoryItem("", BType.NUMBER, new BNumber(0) ) ) );
    memString     = new BDecisionNodeDragDropButton(res, "memString", new BAssignmentNode(new  BMemoryItem("", BType.STRING, new BString(""))   ) );
    memBoolean    = new BDecisionNodeDragDropButton(res, "memBoolean", new BAssignmentNode(new  BMemoryItem("", BType.BOOLEAN, new BBoolean() )) );



  }

  protected void layoutNodeLibraryChildren()
  {
    initButtons();

    addGlue();
    add(new BAIRow(addMove, addChassis, addOrient, addGroup) );

    addMargin();
    add(new BAIRow(memNumber,  memString, memBoolean, addAutoZone) );

    addMargin();
    add(new BAIRow(health, energy, distance, angle) );

    addMargin();
    add(new BAIRow(zone, time, collision, reach) );

    addMargin();
    add(new BAIRow(component, cost, detected, weight) );

    addMargin();
    add(new BAIRow(null, isBlind, complex, null) );
    // moveComplete has been removed

    addGlue();
  }

  protected void configNodeLibraryChildren()
  {
    //titleIcon.setIcon(data.getIcon());
    //health.setIcon(BIconCache.getHSB("ai/health_30.gif"));
    //health.setHorizontalAlignment(BLabel.CENTER);


  }



  // internal classes
  //------------------------------------

  private static class BAIRow extends BBoxPanel
  {
    protected static BUIResources rowRes = new BUIResources(new Object[]
    {
        "blankBox.width",               new Integer(NODE_BOX_SIZE),
        "blankBox.height",              new Integer(NODE_BOX_SIZE)

    });

    public BAIRow(Component c1, Component c2, Component c3, Component c4)
    {
      super(BBoxPanel.X_AXIS);

      if (c1 == null)
        {}//add( new BButton(rowRes, "blankBox"));
      else
        add(c1);

      addMargin();

      if (c2 == null)
        add( new BButton(rowRes, "blankBox"));
      else
        add(c2);

      addMargin();

      if (c3 == null)
        add( new BButton(rowRes, "blankBox"));
      else
        add(c3);

      addMargin();

      if (c4 ==null)
        {}//add( new BButton(rowRes, "blankBox"));
      else
        add(c4);

    }


  }




  public static class BDecisionNodeDragDropButton extends BButton
                              implements DragGestureListener, DragSourceListener, Border
{
  DragSource dragSource = DragSource.getDefaultDragSource();
  protected BDecisionTreeNode source;
  protected Color borderColor;

  protected Cursor linkCursor = BGraphics.getCursor("link_cursor.gif", 0, 0);

  //ObjectSelection object;

  public BDecisionNodeDragDropButton(BUIResources res, String key, BDecisionTreeNode data)
  {
    super(res, key);

    dragSource.createDefaultDragGestureRecognizer
    (
         this, // component where drag originates
         DnDConstants.ACTION_COPY_OR_MOVE, // actions
         this
    );

    this.source = data;
    //object = new ObjectSelection(data);
    this.setCursor(BGraphics.getCursor("drag_cursor.gif", 0, 0));
    //this.setRolloverEnabled(false);


  }


  // dnd methods
  //--------------------------------------------------------------

    public void dragGestureRecognized(DragGestureEvent e)
    {
         // drag anything ...
         if (source == null)
         {
            BDebug.print ("ERROR BDragDropButton " + this + " with null data");
            return;
         }

         e.startDrag(linkCursor, // cursor
            new ObjectSelection( source.clone() ), // transferable
            this); // drag source listener

         this.getModel().setPressed(false);

    }

    public void dragDropEnd(DragSourceDropEvent e) {}
    public void dragEnter(DragSourceDragEvent e) {
      DragSourceContext context =
      e.getDragSourceContext();

      context.setCursor(null);
      context.setCursor(linkCursor);
    }
    public void dragExit(DragSourceEvent e) {}
    public void dragOver(DragSourceDragEvent e) {
      DragSourceContext context =
      e.getDragSourceContext();

      context.setCursor(null);
      context.setCursor(linkCursor);
    }
    public void dropActionChanged(DragSourceDragEvent e) {}


    // dnd variables...
    // -------------------------------------------------------------

    private static Insets allInsets =  new Insets(0,0, 0, 0);

    public Insets getBorderInsets(Component c)
            {return allInsets;}

    public boolean isBorderOpaque()
            {return false; }

    private static RenderingHints hints = new RenderingHints ( RenderingHints.KEY_ANTIALIASING,
                                                                  RenderingHints.VALUE_ANTIALIAS_ON );

    public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)
    {
            int ARC = 10;
            Graphics2D g2 = (Graphics2D) g;
            g2.setRenderingHints(hints);
            g.setColor (borderColor);
            g.drawRoundRect(x,y, width-1, height-1,ARC,ARC);
    }



  // Initialization helper...
  // -------------------------------------------------------------

    public void initializeUI(BUIResources res, String key)
    {
      super.initializeUI(res, key);

      Dimension size = new Dimension(NODE_BOX_SIZE, NODE_BOX_SIZE);

      setMinimumSize(size);
      setMaximumSize(size);
      setPreferredSize(size);

      setShowingText(false);
      //setHorizontalAlignment(BLabel.CENTER);

      borderColor =  (Color) res.get(key+ ".borderColor");
      setBackground(borderColor.darker().darker().darker().darker());

      if (borderColor != null)
        super.setBorder(this);

    }



  }



}