package roboforge.ui.ai;

import roboforge.BGlobal;
import roboforge.ai.orient.*;
import roboforge.ui.components.*;
import roboforge.ui.modes.*;
import roboforge.math.BMath;
import roboforge.BDebug;

import roboforge.ui.BBotSymbol;

import java.awt.*;
import java.awt.event.*;

import javax.swing.Icon;
import javax.vecmath.*;
import java.awt.Point;


public class BAnglePicker extends BGrid2D
{

  // Class variables...
  // -------------------------------------------------------------
  protected static final double MIN_ORIENT = -30;
  protected static final double MAX_ORIENT =  30;

  protected static RenderingHints hints = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );


  // Member variables...
  // -------------------------------------------------------------

  protected BAnglePanel      anglePanel;

  protected BBotSymbol heroSymbol = new BBotSymbol(true);

  protected Color            lineColor = BGlobal.colors.getDarkShade();

  protected Point2d enemy   = new Point2d(0,-1);
  protected Point2d hero    = new Point2d(0,0);
  protected float   facing  = 0;

  protected float   angle1   = 0;
  protected float   angle2  = 0;

  protected Point2d o       = new Point2d(0,0);
  protected Point2d p       = new Point2d(0,0);
  protected Vector2d delta  = new Vector2d(0,0);
  protected Vector2d up     = new Vector2d(0,-1);

  protected String angleOperator = "";
  protected boolean rangeMode = false;

  // Constructors...
  // -------------------------------------------------------------

  public BAnglePicker(BUIResources res, String key)
  {
    super(res, key);

    anglePanel = new BAnglePanel();
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setPickerFromAngle(float angle1, float angle2)
  {
    anglePanel.setPanelFromAngle(angle1, angle2);
    repaint();
  }

  public void setAngleOperator(String angleOperator)
  {
    this.angleOperator = angleOperator;
    if(angleOperator.equals("range"))
      rangeMode = true;
    else
    {
      rangeMode = false;
      angle2 = 0.0f;
    }
    repaint();
  }

  public void setViewLocation(Point2d p)
  {
    p.x = constrain(p.x);
    p.y = constrain(p.y);

    super.setViewLocation(p);
  }

  // gets angle 1 - the one used for all non range based angles
  public int getAngle1()
  {
    return (int)Math.round(Math.toDegrees(angle1));
  }

  // gets angle2 - used when the range option is selected
  public int getAngle2()
  {
    return (int)Math.round(Math.toDegrees(angle2));
  }

  protected double constrain(double n)
    { return Math.min(Math.max(n, MIN_ORIENT), MAX_ORIENT); }


  // Paint method.
  // -------------------------------------------------------------

  public void paint(Graphics g)
  {
    super.paint(g);

    Graphics2D g2D =
      (Graphics2D) g;

    if (anglePanel != null)
      anglePanel.paint(g2D);
  }


  // Angle mode...
  // -------------------------------------------------------------

  protected class BAnglePanel implements MouseMotionListener, MouseListener
  {

    protected Point2d mouse = new Point2d();

    protected Point lineEnd1 = new Point();
    protected Point lineEnd2 = new Point();

    protected boolean handle1 = false;
    protected boolean handle2 = false;

    public BAnglePanel ()
    {
      addMouseMotionListener(BAnglePanel.this);
      addMouseListener(BAnglePanel.this);
    }

    public void setPanelFromAngle(float angle1, float angle2)
    {
      setHeroLocation(0, 0);
      setAngle1(angle1);
      setAngle2(angle2);
      setToolTipText("Left mouse button selects angle.");
    }

      public void paint(Graphics2D g)
    {
      g.setRenderingHints(hints);

      metersToPels(enemy, o);
      metersToPels(hero, p);

      int ox = (int) o.x;
      int oy = (int) o.y;
      int px = (int) p.x;
      int py = (int) p.y;

      int ex = ox + (int) metersToXPels(2);

      g.setColor(lineColor);
      g.drawString("0", px - 2, py - 86);
      g.drawString("90", px + 86, py + 4);
      g.drawString("180", px - 9, py + 93);
      g.drawString("270", px - 102, py + 4);

      g.setColor(BGlobal.colors.getMidShade());

      lineEnd1.setLocation((double)(px + (int)(80 * Math.sin(angle1))), (double)(py - (int)(80 * Math.cos(angle1))));
      lineEnd2.setLocation((double)(px + (int)(80 * Math.sin(angle2))), (double)(py - (int)(80 * Math.cos(angle2))));

      if(!rangeMode)
      {
        if (angleOperator.equals("<"))
        {
          g.setColor(BGlobal.colors.getDarkerShade());
          g.fillArc(px - 50, py - 50, 100, 100, 90, (int)Math.toDegrees(-angle1));
          g.setColor(BGlobal.colors.getMidShade());
          g.drawArc(px - 50, py - 50, 100, 100, 90, (int)Math.toDegrees(-angle1));
        }
        else if (angleOperator.equals(">"))
        {
          g.setColor(BGlobal.colors.getDarkerShade());
          g.fillArc(px - 50, py - 50, 100, 100, 90, 360 - (int)Math.toDegrees(angle1));
          g.setColor(BGlobal.colors.getMidShade());
          g.drawArc(px - 50, py - 50, 100, 100, 90, 360 - (int)Math.toDegrees(angle1));
        }
      }
      else
      {
        if (angle2 <= angle1)
        {
          g.setColor(BGlobal.colors.getDarkerShade());
          g.fillArc(px - 50, py - 50, 100, 100, 90 - 360 - (int)Math.toDegrees(angle1), 360 - (360 - (int)Math.toDegrees(angle1) + (int)Math.toDegrees(angle2)));
          g.setColor(BGlobal.colors.getMidShade());
          g.drawArc(px - 50, py - 50, 100, 100, 90 - 360 - (int)Math.toDegrees(angle1), 360 - (360 - (int)Math.toDegrees(angle1) + (int)Math.toDegrees(angle2)));
        }
        else if (angle2 > angle1)
        {
          g.setColor(BGlobal.colors.getDarkerShade());
          g.fillArc(px - 50, py - 50, 100, 100, 90 - 360 - (int)Math.toDegrees(angle2), -(360 - (int)Math.toDegrees(angle2) + (int)Math.toDegrees(angle1)));
          g.setColor(BGlobal.colors.getMidShade());
          g.drawArc(px - 50, py - 50, 100, 100, 90 - 360 - (int)Math.toDegrees(angle2), -(360 - (int)Math.toDegrees(angle2) + (int)Math.toDegrees(angle1)));
        }
      }

      paintBot(g, heroSymbol, ox, py, (float)Math.PI);

      g.drawLine(px + (int)(15 * Math.sin(angle1)), py - (int)(15 * Math.cos(angle1)), (int)lineEnd1.getX(), (int)lineEnd1.getY());
      g.drawLine(px + (int)(15 * Math.sin(angle2)), py - (int)(15 * Math.cos(angle2)), (int)lineEnd2.getX(), (int)lineEnd2.getY());

      g.setColor(lineColor);
      g.drawOval(px - 80, py - 80, 160, 160);

      g.setColor(BGlobal.colors.getMidShade());
      g.fillOval((int)lineEnd1.getX() - 5, (int)lineEnd1.getY() - 5, 10, 10);

      if(rangeMode)
        g.fillOval((int)lineEnd2.getX() - 5, (int)lineEnd2.getY() - 5, 10, 10);

      if(handle1)
        g.drawOval((int)lineEnd1.getX() - 8, (int)lineEnd1.getY() - 8, 15, 15);
      else if (handle2 && rangeMode)
        g.drawOval((int)lineEnd2.getX() - 8, (int)lineEnd2.getY() - 8, 15, 15);


      g.setColor(lineColor);
      Dimension d = getSize();
      paintStats(g, 10, d.height-10);
    }


    protected void paintBot(Graphics2D g, BBotSymbol bot, int x, int y, float theta)
    {
      double sx = pelsPerXMeter / 16.5;
      double sy = pelsPerYMeter / 16.5;

      g.translate(x, y);
      g.scale(sx, sy);
      g.rotate(theta + Math.PI);

      bot.paint(g, 0.0f, "", "", "", false);

      g.rotate(-theta - Math.PI);
      g.scale(1/sx, 1/sy);
      g.translate(-x, -y);
    }

    protected void paintStats(Graphics2D g, int x, int y)
    {
      if(rangeMode)
        g.drawString("angle range: " + Math.round(Math.toDegrees(angle2))+ ".0 - " + Math.round(Math.toDegrees(angle1)) + ".0", x, y);
      else
        g.drawString("angle: "+ Math.round(Math.toDegrees(angle1))+ ".0", x, y);
    }

    // set the angles
    protected void setAngle1(float a)
    {
      angle1 = a;

      if (angle1 == 360.0f)
        angle1 = 0.0f;

      angle1 = (float)Math.toRadians(angle1);
    }
    protected void setAngle2(float a)
    {
      angle2 = a;

      if (angle2 == 360.0f)
        angle2 = 0.0f;

      angle2 = (float)Math.toRadians(angle2);
    }



    protected void setHeroLocation(double x, double y)
    {
      hero.x = constrain(x);
      hero.y = constrain(y);
    }

    public void mousePressed(MouseEvent e)
    {
      int x = e.getX();
      int y = e.getY();

      double distance1 = Math.sqrt( Math.pow(x - lineEnd1.getX(), 2) + Math.pow(y - lineEnd1.getY(),2));
      double distance2 = Math.sqrt( Math.pow(x - lineEnd2.getX(), 2) + Math.pow(y - lineEnd2.getY(),2));

      if(distance1 <= 5.0)
      {
        handle1 = true;
        handle2 = false;
        setAngleFromCursor(x, y, e, 1);
      }
      else if(distance2 <= 5.0 && rangeMode)
      {
        handle2 = true;
        handle1 = false;
        setAngleFromCursor(x, y, e, 2);
      }
      else
      {
        handle1 = false;
        handle2 = false;
        BAnglePicker.this.repaint();
      }
    }

    public void mouseClicked(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseMoved(MouseEvent e) {}

    public void mouseDragged(MouseEvent e)
    {
      int x = e.getX();
      int y = e.getY();

      if(handle1)
        setAngleFromCursor(x, y, e, 1);
      else if(handle2)
        setAngleFromCursor(x, y, e, 2);
    }

    public void mouseReleased(MouseEvent e) {}

    // sets the current angle from two integer co-ordinates
    protected void setAngleFromCursor(int x, int y, MouseEvent e, int handle)
    {
      mouse.set(x, y);
      pelsToMeters(mouse, mouse);
      float xPos = (float)x - (float)p.x;
      float yPos = (float)y - (float)p.y;
      float newAngle;
      float oldAngle;

      if(handle == 1)
        oldAngle = (float)Math.round(Math.toDegrees(angle1));
      else
        oldAngle = (float)Math.round(Math.toDegrees(angle2));

      if (yPos < 0)
        newAngle = (float)Math.toDegrees(Math.atan(xPos / -yPos));
      else
        newAngle = (float)Math.toDegrees(Math.atan(xPos / -yPos) + Math.PI);

      if (newAngle < 0)
        newAngle = (newAngle % 360) + 360;

      // if the right button is pressed then only increment the angle by one
      if(e.isMetaDown())
      {
        if (newAngle > oldAngle)
          newAngle = oldAngle + 1.0f;
        else if (newAngle < oldAngle)
          newAngle = oldAngle - 1.0f;
        else
          newAngle = oldAngle;
      }

      if(handle == 1)
        setAngle1(newAngle);
      else
        setAngle2(newAngle);

      BAnglePicker.this.repaint();
    }
  }
}
