package roboforge.ui.ai;

import roboforge.BBot;
import roboforge.BDebug;

import roboforge.ai.BDecisionTreeAI;
import roboforge.ai.decisions.*;
import roboforge.ai.expressions.*;
import roboforge.ai.expressions.parser.*;
import roboforge.ai.memory.*;
import roboforge.ai.values.*;

import roboforge.ui.components.*;
import roboforge.util.parser.*;
import roboforge.util.graphics.*;
import roboforge.ui.components.text.*;

import java.awt.Dimension;
import java.awt.event.*;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.DefaultComboBoxModel;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.ButtonGroup;

import java.util.*;


public class BAssignmentNodePanel extends BAINodePanel
{

  // Class variables...
  // -------------------------------------------------------------

  protected static BAINodePanel instance = null;
    // A single instance of this panel type.
    // Returned by getInstance().


  // Member variables...
  // -------------------------------------------------------------

  protected BAssignmentNode node;

  // UI Resources...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {

    "addMem.icon",                BIconCache.getHSB("add_icon_small.gif"),
    "addMem.tooltip",             "Add a new memory item.",

    "delMem.icon",                BIconCache.getHSB("del_icon_small.gif"),
    "delMem.tooltip",             "Remove this memory item.",

    "advancedButton.text",        "Advanced",
    "advancedButton.height",      new Integer (30),
    "advancedButton.width",       new Integer (70),

    "numberSlider.max",           new Integer(180),
    "numberSlider.min",           new Integer(0),
    "numberSlider.majorSpacing",  new Integer(45),
    "numberSlider.minorSpacing",  new Integer(5),

    "selectionText.width",        new Integer(30),

    "textValue.width",            new Integer(200),

    "yRadio.text",                "yes",
    "yRadio.width",               new Integer(100),
    "nRadio.text",                "no"

  });


  // Visual elements...
  // -------------------------------------------------------------
  protected BComboBox itemsCombo = new BComboBox();
  protected BButton addMem     = new BButton(res, "addMem");
  protected BButton delMem     = new BButton(res, "delMem");
  protected BNamedEditor nameEditor = new BNamedEditor();

  protected BToggleButton advanceButton = new BToggleButton(res, "advancedButton");

  protected BSlider numberSlider = new BSlider(     res,     "numberSlider"    );
  protected BTextField textValue = new BTextField(  res,  "textValue"       );
  protected BLabel selectionText = new BLabel(      res,      "selectionText"   );

  protected BBoxPanel ynPanel = new BBoxPanel( BBoxPanel.X_AXIS );
  protected BRadioButton yRadio = new BRadioButton(res, "yRadio");
  protected BRadioButton nRadio = new BRadioButton(res, "nRadio");



  protected BExpressionBuilder builder =
    new BExpressionBuilder(res, "builder");


  // Constructor.
  // -------------------------------------------------------------

  public BAssignmentNodePanel()
  {
    super("update memory");

    layoutPanelChildren();
    configPanelChildren();
  }

  protected void layoutPanelChildren()
  {
    addHGlued(new BLabel("Memory item: "));
    addTinyMargin();

    BBoxPanel comboLine = new BBoxPanel(BBoxPanel.X_AXIS);
    comboLine.add(itemsCombo);
    comboLine.addSmallMargin();
    comboLine.add(addMem);
    comboLine.addSmallMargin();
    comboLine.add(delMem);
    add(comboLine);

    addSmallMargin();
    addHGlued(new BLabel("Value: "));

    ynPanel.add(yRadio);
    ynPanel.addGlue();
    ynPanel.add(nRadio);



    BBoxPanel bp = new BBoxPanel(BBoxPanel.X_AXIS);
      bp.addLinear(numberSlider, textValue, ynPanel, getVGlue() );
      bp.addLinear( selectionText, getVGlue()   );
      bp.addGlue();
      bp.addLinear(advanceButton, getVGlue() );
    add(bp);


    //addLinear(itemsCombo, getHStrut(SMALL_MARGIN), icon);
    addMargin();

    add(builder);
  }

  protected void configPanelChildren()
  {
    itemsCombo.addActionListener(updateListener);
   // itemsCombo.addActionListener(new ItemChangeListener());
    itemsCombo.setEditor(nameEditor);
    itemsCombo.setEditable(true);
    nameEditor.addActionListener(new BMemRenameListener() );
    nameEditor.setGuard(new BMemoryGuard() );

    addMem.addActionListener(new AddMemoryListener() );
    delMem.addActionListener(new DelMemoryListener() );

    advanceButton.addActionListener( new AdvancedListener() );

    numberSlider.addChangeListener( new ChangeListener()
      {
        public void stateChanged(ChangeEvent e)
        {
          selectionText.setText( numberSlider.getValue() + "" );
        }
      }
    );

    ButtonGroup bg = new ButtonGroup();
    bg.add(yRadio);
    bg.add(nRadio);



  }


  protected class AddMemoryListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      BMemoryItem item = node.getMemoryItem();
      BMemoryItem m = new BMemoryItem( item );
      String name = "item"+ getAI().getMemoryBank().getSize() ;

      if ( m.getType() == BType.STRING )
      {
        name = "Text_" + name;
        m.setInitialValue(new BString("") );
      }
      else if ( m.getType() == BType.NUMBER )
      {
        name = "Number_" + name;
        m.setInitialValue(new BNumber(0) );
      }
      else
      {
        name = "Logical_" + name;
        m.setInitialValue(new BBoolean(false) );
      }

      m.setName( name );
      getAI().getMemoryBank().add(m);

      // enable the combo box
      itemsCombo.setEnabled(true);
      setAI(ai);                     
      itemsCombo.setSelectedItem(m);
    }
  }

  protected class DelMemoryListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      Object m = itemsCombo.getSelectedItem();
      if (m != null)
      {
        getAI().getMemoryBank().remove(m);

        if (getAI().getMemoryBank().getSize() == 0)
          itemsCombo.setEnabled(false);

        setAI(ai);

      }
    }
  }


  protected class BMemRenameListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      BMemoryItem m = (BMemoryItem)itemsCombo.getSelectedItem();
      if (m!= null)
        m.setName(nameEditor.getText());
    }
  }


  // Activation methods...
  // -------------------------------------------------------------

  public void onClose()
  {
    instance = null;
    super.onClose();
  }

  public static void clearInstance()
    { instance = null; }


  
  // Accessor methods...
  // -------------------------------------------------------------

  protected void toggleAdvancedOn(boolean on)
  {
    if (on)
    {
      builder.setVisible(true);
      textValue.setVisible(false);
      numberSlider.setVisible(false);
      selectionText.setVisible(false);
      ynPanel.setVisible(false);
      advanceButton.setText("Simple");
      advanceButton.setToolTipText("Move the chassis a set distance");

    }
    else
    {
      builder.setVisible(false);

      if (node.getMemoryItem().getType() == BType.NUMBER)
      {
        numberSlider.setVisible(true);
        textValue.setVisible(false);
        ynPanel.setVisible(false);
        selectionText.setVisible(true);
      }
      else if (node.getMemoryItem().getType() == BType.STRING )
      {
        numberSlider.setVisible(false);
        textValue.setVisible(true);
        ynPanel.setVisible(false);
        selectionText.setVisible(false);
      }
      else  if (node.getMemoryItem().getType() == BType.BOOLEAN )
      {
        numberSlider.setVisible(false);
        textValue.setVisible(false);
        selectionText.setVisible(false);
        ynPanel.setVisible(true);

      }

      advanceButton.setText("Advanced");
      advanceButton.setToolTipText("Move the chassis based on a mathematical equation");


    }
  }


  public BDecisionTreeNode getNode()
    { return node; }

  public static BAINodePanel getInstance()
  {
    if (instance == null)
      instance = new BAssignmentNodePanel();

    return instance;
  }

  public void setAI(BDecisionTreeAI ai)
  {
    super.setAI(ai);

    if (node != null)
    {
      BDebug.print ("BAssignmantNodePanel setAI " + ai);

      // fill the combo box
      BMemoryBank mb = ai.getMemoryBank();
      Vector v = new Vector();
      for (int i =  0; i < mb.size(); i++ )
      {
        if (mb.getItem(i).getType() == node.getMemoryItem().getType() )
          v.add(mb.getItem(i));
      }

      DefaultComboBoxModel cModel = new DefaultComboBoxModel(v);
      itemsCombo.setModel(cModel);


      if (v.size() == 0)
        itemsCombo.setEnabled(false);
      else
        itemsCombo.setEnabled(true);



      // set the Builder
      builder.setAI(ai);

    }
  }


  // UI methods...
  // -------------------------------------------------------------

  public void setPanelFromNode(BDecisionTreeNode n)
  {
    BDebug.print ("BAssignmentNodePanel setPanelFromNode node " + n);
    updateListener.setEnabled(false);

    // must happen before setAI()
    this.node = (BAssignmentNode) n;

    // HACK: Gets World states to update...
    if (ai != null)
      setAI(ai);


    BMemoryItem item = node.getMemoryItem();
    if (getAI().getMemoryBank().indexOf(item) != -1)
      itemsCombo.setSelectedItem(item);

    advanceButton.setSelected(!node.getUsesSimpleAssingment());
    toggleAdvancedOn( !node.getUsesSimpleAssingment() );
    builder.setExpression(node.getExpression());

    updateListener.setEnabled(true);
    updateSimpleUI();

  }

  protected void updateSimpleUI()
  {
    BValue simpleV = node.getSimpleValue();
    if (simpleV != null)
    {
      if (simpleV.getType() == BType.NUMBER )
        numberSlider.setValue( (int)((BNumber)simpleV).getValue() );
      else if (simpleV.getType() == BType.STRING )
        textValue.setText(((BString)simpleV).getValue() );
      else if (simpleV.getType() == BType.BOOLEAN )
      {
        boolean b = ((BBoolean)simpleV).getValue();
        if (b)
          yRadio.setSelected(true);
        else
          nRadio.setSelected(true);
      }

    }
    else
    {
      textValue.setText("");
      numberSlider.setValue(10);
      yRadio.setSelected(true);
    }
  }


  protected BValue getSimpleValueFromUI()
  {
    if (node.getMemoryItem().getType() == BValue.NUMBER)
      return new BNumber(numberSlider.getValue() );
    else if (node.getMemoryItem().getType() == BValue.STRING)
      return new BString(textValue.getText() );
    else  if (node.getMemoryItem().getType() == BValue.BOOLEAN)
      return new BBoolean(yRadio.isSelected() );

    BDebug.print ("BAssignmentNodePanel.getSimpleValueFromUI Illegal Type ");
    return null;
  }

  protected void setNodeFromPanel()
  {
    if (node != null)
    {
        BMemoryItem m =  (BMemoryItem) itemsCombo.getSelectedItem();

        if (m != null)
        {
          m.validateAndFixInitialValue();

          node.setMemoryItem(m);
          node.setUsesSimpleAssignment(!advanceButton.isSelected());
          node.setSimpleValue(getSimpleValueFromUI() );
          node.setExpression(builder.getExpression());
       }
    }
  }


  public void onDeactivate()
  {

    super.onDeactivate();

    // node == null must come after super.onDeactive() 
    node = null;

  }

  class AdvancedListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      toggleAdvancedOn(advanceButton.isSelected() );

    }
  }

  public static class BMemoryGuard extends BAbstractGuard
  {
    char underS = '_';

    public BMemoryGuard()
      { super(); }

    protected boolean isValidChar(char c, int i)
    {
      if (Character.isLetterOrDigit(c))
        return true;

      if (c == underS)
        return true;

      return false;

    }
  
  
  }


}