package roboforge.ui.ai;

import roboforge.BBot;
import roboforge.BChassis;
import roboforge.BMove;

import roboforge.ai.BDecisionTreeAI;
import roboforge.ai.actions.*;
import roboforge.ai.decisions.*;
import roboforge.ai.expressions.*;
import roboforge.ai.values.*;
import roboforge.ui.components.*;

import java.util.Vector;

import javax.swing.*;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;






public class BChassisActionNodePanel extends BAINodePanel
{

  // Class variables...
  // -------------------------------------------------------------

  protected static BAINodePanel instance = null;
    // A single instance of this panel type.
    // Returned by getInstance().


  // Member variables...
  // -------------------------------------------------------------

  protected BChassisActionNode node;
  protected BChassis chassis;


  // UI Resources...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources(new Object[]
  {
    "distance.tooltip",             "Enter a positive distance/angle here.",
    "scroller.height",              new Integer(80),

    "distanceSlider.max",           new Integer(180),
    "distanceSlider.min",           new Integer(0),
    "distanceSlider.majorSpacing",  new Integer(45),
    "distanceSlider.minorSpacing",  new Integer(5),


    "distanceSliderText.width",     new Integer(30),
    "advancedButton.text",          "Advanced",
    "advancedButton.height",        new Integer (30),
    "advancedButton.width",         new Integer (70),




  });


  // Visual elements...
  // -------------------------------------------------------------

  protected BComboBox      movesCombo           = new BComboBox();
  protected BSlider        distanceSlider       = new BSlider(res, "distanceSlider");
  protected BLabel         distanceSliderText   = new BLabel(res, "distanceSliderText");
  protected BToggleButton advanceButton = new BToggleButton(res, "advancedButton");

  protected BExpressionBuilder distanceBuilder = new BExpressionBuilder();
  //protected BExpressionPad distance             = new BExpressionPad(res, "distance");
  //protected BScrollPane    complexScroller      = new BScrollPane(res, "scroller", distance);


  // Constructor.
  // -------------------------------------------------------------

  public BChassisActionNodePanel()
  {
    super("move chassis");
    layoutPanelChildren();
    configPanelChildren();
  }

  protected void layoutPanelChildren()
  {

    addHGlued(new BLabel("Chassis move to execute: "));
    addTinyMargin();
    add(movesCombo);

    addSmallMargin();
    addHGlued(new BLabel("Distance: "));

    BBoxPanel bp = new BBoxPanel(BBoxPanel.X_AXIS);
      bp.addLinear(distanceSlider, getVGlue() );
      bp.addLinear(distanceSliderText, getVGlue() );
      bp.addGlue();
      bp.addLinear(advanceButton, getVGlue() );
    add(bp);

    //addLinear(distanceSlider, distanceSliderText, advanceButton) ;
    add(distanceBuilder);

  }

  protected void configPanelChildren()
  {
    ChangeListener updateText = new ChangeListener()
      {
        public void stateChanged(ChangeEvent e)
        {
          updateDistanceSliderText();
        }
      };

    distanceSlider.addChangeListener( updateText );
    movesCombo.addActionListener(new ActionListener()
    {
        public void actionPerformed(ActionEvent e)
        {
          updateDistanceSliderText();
        }
    });

    advanceButton.addActionListener(new ActionListener()
    {
        public void actionPerformed(ActionEvent e)
        {
          toggleAdvancedOn(advanceButton.isSelected() );
        }
    });


    distanceSlider.setSnapToTicks(false);


  }

  protected void toggleAdvancedOn(boolean on)
  {
    if (on)
    {
      distanceBuilder.setVisible(true);
      distanceSlider.setVisible(false);
      distanceSliderText.setVisible(false);
      advanceButton.setText("Simple");
      advanceButton.setToolTipText("Move the chassis a set distance");

    }
    else
    {
      distanceBuilder.setVisible(false);
      distanceSlider.setVisible(true);
      distanceSliderText.setVisible(true);
      advanceButton.setText("Advanced");
      advanceButton.setToolTipText("Move the chassis based on a mathematical equation");


    }

  }


  protected void updateDistanceSliderText()
  {
    BMove move = (BMove) movesCombo.getSelectedItem();
    distanceSliderText.setText(distanceSlider.getValue() +  chassis.getChassisMoveList().getChassisMoveUnits(move, true) );
  }

  // Activation methods...
  // -------------------------------------------------------------

  public void onClose()
  {
    instance = null;
    super.onClose();
  }


  public static void clearInstance()
    { instance = null; }
  
  // Accessor methods...
  // -------------------------------------------------------------

  public BDecisionTreeNode getNode()
    { return node; }

  public static BAINodePanel getInstance()
  {
    if (instance == null)
      instance = new BChassisActionNodePanel();

    return instance;
  }

  public void setAI(BDecisionTreeAI ai)
  {
    super.setAI(ai);

    distanceBuilder.setAI(ai);

    BBot bot  = ai.getBot();
    chassis = bot.getChassis();

    Vector moves =
      chassis.getValidChassisMoves();

    movesCombo.setModel(
      new DefaultComboBoxModel(moves));

    movesCombo.addActionListener(updateListener);

  }


  // UI methods...
  // -------------------------------------------------------------

  public void setPanelFromNode(BDecisionTreeNode n)
  {
    updateListener.setEnabled(false);

    this.node = (BChassisActionNode) n;

    BChassisAction action =
      node.getChassisAction();

    if (action != null)
      movesCombo.setSelectedItem(
        action.getChassisMove());

    distanceBuilder.setExpression(
      node.getDistance());

    // default simple value
    distanceSlider.setValue((int)node.getSimpleDistance() );
    advanceButton.setSelected(!node.getUsesSimpleDistance() ) ;
    toggleAdvancedOn(advanceButton.isSelected());

    updateListener.setEnabled(true);
  }

  protected void setNodeFromPanel()
  {
    if (node != null)
    {
      BMove move =
        (BMove) movesCombo.getSelectedItem();

      BChassisAction action =
        new BChassisAction(move, chassis, 0.0f);

      node.setChassisAction(action);
      node.setDistance(distanceBuilder.getExpression());
      node.setDistance(distanceSlider.getValue());
      node.setUsesSimpleDistance(!advanceButton.isSelected() );
    }
  }

}
