package roboforge.ui.ai;

import roboforge.BDebug;
import roboforge.ai.decisions.*;
import roboforge.ui.components.BDragDropTree;

import javax.swing.border.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import java.awt.*;
import roboforge.util.graphics.*;
import javax.swing.Icon;
import java.util.*;

public class BDecisionTree extends BDragDropTree
{
  protected BDecisionTreeModel model;
  protected BDecisionTreeRenderer renderer;
  protected BAITreeUI ui;
  protected VeotoListener vetoListener;

  public BDecisionTree()
    { super(); }

  public BDecisionTree(BRootNode root)
  {
    super();
    setRoot(root);

    ui = new BAITreeUI();
    setUI ( ui);
    renderer = new BDecisionTreeRenderer();
    setCellRenderer(renderer);
    setRowHeight (BDecisionTreeRenderer.CELLHEIGHT);

    vetoListener = new VeotoListener();
    this.addTreeWillExpandListener(vetoListener);
    this.setOpaque(true);
    //this.setShowsRootHandles(false);

    this.setCursor(BGraphics.getCursor("drag_cursor.gif", 0, 0));
    //renderer.setTextVisible(false);

  }

  public boolean getTextVisible()
    { return renderer.getTextVisible(); }

  public void setTextVisible(boolean b)
  {
    renderer.setTextVisible(b);
    //model.nodeChanged((TreeNode)model.getRoot());
    //renderer.repaint();

    // HACK TO GET LABELS TO CHANGE CORRECTLY
     setRoot((BRootNode)model.getRoot() );

  }

  public void setRoot(BRootNode root)
  {
    if (root == null)
      model = new BDecisionTreeModel();
    else
      model = new BDecisionTreeModel(root, this);


    //expandChildren(root);
    setModel(model);
    expandChildren(root );

  }

  public void updateExpansion()
  {
    int[] selection = getSelectionRows();

    setRoot((BRootNode) model.getRoot());
    setSelectionRows(selection);

  }

  public BDecisionTreeModel getDecisionModel()
    { return model; }



  public BDecisionTreeModel getTreeModel()
    { return model; }


  // Drag 'n drop methods...
  // -------------------------------------------------------------

  protected boolean acceptDrop
  (
    DefaultMutableTreeNode dropped,
    DefaultMutableTreeNode target
  )
    // Determines what objects can be dropped on this tree.
  {
    if (target == null)
    {
      BDebug.print ("BDecisionTree acceptDrop TARGET FALSE");
      return false;
    }

    if (dropped instanceof BDecisionTreeNode)
      // Only accept decision tree-nodes.
    {
      return true;
      /*
      BDecisionTreeNode t =
        (BDecisionTreeNode) target;

      BDecisionTreeNode p =
        (BDecisionTreeNode) target.getParent();

      if (p != null)
        return p.canAddChildren() || t.canAddChildren() || isLastBranch(t);
      else
        return t.canAddChildren(); */
    }
    else
      return false;
  }


  protected boolean acceptDrag(DefaultMutableTreeNode dragged)
    // Determines what objects can be dragged off this tree.
  {
    if (dragged == null)
      return false;
    else
      return ((BDecisionTreeNode) dragged).isRemoveable();
  }


  protected boolean isLastBranch(DefaultMutableTreeNode target)
  {
    BRootNode root = (BRootNode)model.getRoot();

    if (root.getChildCount() == 0)
      return true;

    TreeNode  lastNode = root.getChildAt(root.getChildCount() - 1);
    return target.isNodeAncestor(lastNode);

  }


  protected void insertDropData(
    DefaultMutableTreeNode dropped,
    DefaultMutableTreeNode target)
  {
    DefaultTreeModel model =
      (DefaultTreeModel) getModel();

    BDecisionTreeNode t =
      (BDecisionTreeNode) target;

    BDecisionTreeNode parent =
      (BDecisionTreeNode) target.getParent();

    TreePath targetPath =
      new TreePath(model.getPathToRoot(target));

    boolean isTargetExpanded  = isExpanded(targetPath);
    boolean isParentAddable   = (parent != null) && parent.canAddChildren();
    boolean isTargetAddable   = t.canAddChildren();
    boolean isTargetChildless = (t.getChildCount() == 0);
    boolean isLastBranch      = isLastBranch(target);


    if (isTargetExpanded && isTargetAddable)
      insertNodeInto(dropped, target, 0);

    else if (isTargetAddable && isTargetChildless)
      insertNodeInto(dropped, target, 0);

    else if (isParentAddable)
      insertNodeInto(dropped, parent, parent.getIndex(target) + 1);

    else
    {
      // the root node can always add children
      while (! ( parent.canAddChildren()) )
      {
        target = parent;
        parent = (BDecisionTreeNode) parent.getParent();
      }

      insertNodeInto(dropped, parent ,parent.getIndex(target)+ 1 );


    }

    /* if (isLastBranch)
      insertNodeInto(dropped, (MutableTreeNode)model.getRoot() ,((DefaultMutableTreeNode)model.getRoot()).getChildCount() );
    else if (isTargetAddable)
      insertNodeInto(dropped, target, 0); */

    selectNode(dropped);
   // expandNode(dropped);
  }

  // class

  class VeotoListener implements TreeWillExpandListener
  {
    public void treeWillCollapse(TreeExpansionEvent event)
      throws ExpandVetoException
    {
      throw new ExpandVetoException(event);

    }

    public void treeWillExpand(TreeExpansionEvent event)
     throws ExpandVetoException
    {

        BDecisionTreeNode n = (BDecisionTreeNode)  event.getPath().getLastPathComponent() ;
        if (n.isClosed() )
          throw new ExpandVetoException(event);


      return;
    }

  }

  protected void returnDraggedData()
  {
  //  DefaultTreeModel model =
  //    (DefaultTreeModel) getModel();

    insertNodeInto(draggedNode, draggedNodeParent, draggedNodeIndex);

    //selectNode(draggedNode);
    //expandNode(draggedNode);
  }



  public void insertNodeInto(DefaultMutableTreeNode child,MutableTreeNode parent,int n)
  {
    model.insertNodeInto(child, parent, n);
    expandNode(child);
    this.expandChildren((DefaultMutableTreeNode)child.getParent());


  }

    protected void extractDragData()
  {

    super.extractDragData();
    this.expandChildren(draggedNodeParent);

  }

  public void expandChildren(DefaultMutableTreeNode node)
  // override of the BTree Expand To do stuff
  {

    Enumeration e = node.depthFirstEnumeration();
    TreePath rootPath = new TreePath( node );

    while (e.hasMoreElements() )
    {
      DefaultMutableTreeNode currentN = (DefaultMutableTreeNode) e.nextElement();

      if (currentN.isLeaf() )
      {
        TreeNode[] path = currentN.getPath();
        TreeNode[] newPath = null;

        // cut the path if there are any groups in the way that are closed
        for (int i = 0; i < path.length && (newPath == null); i++ )
        {

            BDecisionTreeNode gn = (BDecisionTreeNode) path[i];
            if (gn.isClosed() )
            {
              newPath = new TreeNode[i + 1];
              System.arraycopy(path, 0, newPath, 0, i + 1);

              // collapse innerPaths
              //TreeNode[] collapsePath = new TreeNode[path.length - (i + 1)];
              //System.arraycopy(path, i + 1 , collapsePath, 0, path.length - (i + 1) );
              //collapsePath(new TreePath(collapsePath ));

            }



        }

        if (newPath != null)
          path = newPath;

        makeVisible( new TreePath(path ));

      }

   

    }

  }



   private static RenderingHints hints = new RenderingHints ( RenderingHints.KEY_ANTIALIASING,
                                                                  RenderingHints.VALUE_ANTIALIAS_ON );
   public void paintComponent(Graphics g)
   {
      Graphics2D g2 = (Graphics2D) g;
      g2.setRenderingHints(hints);
      super.paintComponent(g);
   }






}
