package roboforge.ui.ai;

import roboforge.BBot;
import roboforge.BMove; 
import roboforge.BMoveList;

import roboforge.ai.BDecisionTreeAI;
import roboforge.ai.decisions.*;
import roboforge.ui.components.*;

import roboforge.BDebug;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import javax.swing.DefaultComboBoxModel;


public class BMoveActionNodePanel extends BAINodePanel
{

  // Class variables...
  // -------------------------------------------------------------

  protected static BAINodePanel instance = null;
    // A single instance of this panel type.
    // Returned by getInstance().


  // Member variables...
  // -------------------------------------------------------------

  protected BMoveActionNode node;

  // Visual elements...
  // -------------------------------------------------------------

  protected BComboBox movesCombo = new BComboBox();
  protected BCheckBox override   = new BCheckBox("Override conflicting moves");
  protected BMultiLabel overrideDesc = new BMultiLabel();
  protected BMoveList mList;


  // Constructor.
  // -------------------------------------------------------------

  public BMoveActionNodePanel()
  {
    super("move");
    layoutPanelChildren();
    configPanelChildren();
  }

  protected void layoutPanelChildren()
  {
    addHGlued(new BLabel("Move to execute: "));
    addTinyMargin();
    add(movesCombo);

    addSmallMargin();
    addHGlued(override);
    addHGlued(overrideDesc);

  }

  protected void configPanelChildren()
  {
  //  overrideDesc.setText ("By clicking the override check box you will ensure that your bot will do this move even if other moves are being executed.");
    movesCombo.addActionListener(updateListener);
  }


  // Activation methods...
  // -------------------------------------------------------------

  public void onClose()
  {
    //BDebug.print ("BMoveActionNodePanel.onClose Called ");
    instance = null;
    super.onClose();
  }


  public static void clearInstance()
    { instance = null; }
  /*

  public void onActivate()
    { BDebug.print ("BMoveActionNode onActivate called "); }

  public void onDeactivate()
  {
    super.onDeactivate();
    BDebug.print ("BMoveActionNode onDeactivate called ");
  }     */


  // Accessor methods...
  // -------------------------------------------------------------

  public BDecisionTreeNode getNode()
    { return node; }

  public static BAINodePanel getInstance()
  {
    if (instance == null)
      instance = new BMoveActionNodePanel();

    return instance;
  }

  public void setAI(BDecisionTreeAI ai)
  {
   // BDebug.print ("BMoveActionNode.setAI Called " );
    super.setAI(ai);

    BBot bot  = ai.getBot();
    mList  =  bot.getMoveList();

    bot.getMoveList().addListDataListener( new MovesChangeListener() );

    refreshCombo();

  }


  // UI methods...
  // -------------------------------------------------------------

  public void setPanelFromNode(BDecisionTreeNode n)
  {
    updateListener.setEnabled(false);

    this.node = (BMoveActionNode) n;

    movesCombo.setSelectedItem(node.getMoveAction());
    override.setSelected(node.isOverriding());

    updateListener.setEnabled(true);
  }

  protected void setNodeFromPanel()
  {
    //BDebug.print ("BMoveActionPanel this.isActive() " + this.isActive() );
    if (this.isActive() )
      if (node != null)
        {

          //node.setMoveAction((BMove) mList.getAt(movesCombo.getSelectedIndex())  );
          node.setMoveAction((BMove) movesCombo.getSelectedItem());
          node.setOverriding(override.isSelected());
        }
  }


  public void refreshCombo()
  {
    //BDebug.print ("BMoveActionNodePanel refreshCombo called");
    movesCombo.setModel(new DefaultComboBoxModel(mList.getMoves()) );
  }

  // Moves Change Listener
  class MovesChangeListener implements ListDataListener
  {

      public void intervalAdded(ListDataEvent e)
        { refreshCombo(); }

      public void intervalRemoved(ListDataEvent e)
      {
        BMoveActionNode oldNode = node;
        node = null;
        refreshCombo();
        node = oldNode;
      }

      public void contentsChanged(ListDataEvent e)
        { }
  }


}
