package roboforge.ui.ai;

import roboforge.BGlobal;
import roboforge.ai.orient.*;
import roboforge.ui.components.*;
import roboforge.ui.modes.*;
import roboforge.math.BMath;

import roboforge.ui.BBotSymbol;

import java.awt.*;
import java.awt.event.*;

import javax.swing.Icon;
import javax.vecmath.*;


public class BOrientPicker extends BMouseableGrid2D
{

  // Class variables...
  // -------------------------------------------------------------
  protected static final double MIN_ORIENT = -30;
  protected static final double MAX_ORIENT =  30;

  protected static RenderingHints hints = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );


  // Member variables...
  // -------------------------------------------------------------

  protected BModeMulti             viewMode;
  protected BModeOrient            orientMode;
  protected BModeOrientFull        fullMode;
  protected BModeOrientDist        distMode;
  protected BModeOrientFace        faceMode;
  protected BModeOrientDistFacing  distFaceMode;
  protected BModeOrientLocation    locationMode;


//  protected Icon             icon;
//  protected Icon             simple;

  protected BBotSymbol heroSymbol = new BBotSymbol(true);
  protected BBotSymbol heroSymbol2 = new BBotSymbol(false);
  protected BBotSymbol enemySymbol = new BBotSymbol(false);
  protected BBotSymbol enemySymbol2 = new BBotSymbol(true);

  protected Color            lineColor = BGlobal.colors.getDarkShade();

  protected Point2d enemy   = new Point2d(0,0);
  protected Point2d hero    = new Point2d(0,0);
  protected float   facing  = 0;
  protected float   angle   = 0;
  protected Point2d o       = new Point2d(0,0);
  protected Point2d p       = new Point2d(0,0);
  protected Vector2d delta  = new Vector2d(0,0);
  protected Vector2d up     = new Vector2d(0,-1);



  // Constructors...
  // -------------------------------------------------------------

  public BOrientPicker(BUIResources res, String key)
  {
    super(res, key);

    //icon   = (Icon) res.getIcon(key+ ".icon");
    //simple = new BotIcon(26);

    viewMode = viewModeWrapper(new BModeMouse());
    setMode(viewMode);

    fullMode     = new BModeOrientFull();
    distMode     = new BModeOrientDist();
    faceMode     = new BModeOrientFace();
    distFaceMode = new BModeOrientDistFacing();
    locationMode = new BModeOrientLocation();
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setOrientFromPicker(BOrient o)
    { orientMode.setOrientFromMode(o); }

  public void setPickerFromOrient(BOrient o)
  {
    if (o instanceof BOrientFull)
      orientMode = fullMode;
    else if (o instanceof BOrientDistanceOnly)
      orientMode = distMode;
    else if (o instanceof BOrientFacingOnly)
      orientMode = faceMode;
    else if (o instanceof BOrientDistanceFacing)
      orientMode = distFaceMode;
    else if (o instanceof BOrientLocationOnly)
      orientMode = locationMode;

    orientMode.setModeFromOrient(o);
    viewMode.setDefaultMode(orientMode);

    repaint();
  }


  public void setViewLocation(Point2d p)
  {
    p.x = constrain(p.x);
    p.y = constrain(p.y);

    super.setViewLocation(p);
  }

  protected double constrain(double n)
    { return Math.min(Math.max(n, MIN_ORIENT), MAX_ORIENT); }


  // Paint method.
  // -------------------------------------------------------------

  public void paint(Graphics g)
  {
    super.paint(g);

    Graphics2D g2D =
      (Graphics2D) g;

    if (orientMode != null)
      orientMode.paint(g2D);
  }


  // Orient picking modes...
  // -------------------------------------------------------------

  protected abstract class BModeOrient extends BModeGrid
  {

    protected Point2d mouse = new Point2d();

    // Accessor methods...
    // -------------------------------------------------------------

    public abstract void setModeFromOrient(BOrient o);
    public abstract void setOrientFromMode(BOrient o);

    public void paint(Graphics2D g)
    {
      g.setRenderingHints(hints);
      
      metersToPels(enemy, o);
      metersToPels(hero, p);

      int ox = (int) o.x;
      int oy = (int) o.y;
      int px = (int) p.x;
      int py = (int) p.y;

      paintManipulator(g, ox, oy, px, py);

      int ex = ox + (int) metersToXPels(2);
      g.setColor(lineColor);
      //g.drawString("enemy", ex, oy);

      // Draw bots...
      paintBot(g, getEnemy(), ox, oy, 0);
      paintBot(g, getHero(),  px, py, angle);

      Dimension d = getSize();
      paintStats(g, 10, d.height-10);
    }

    protected abstract void paintManipulator
      (Graphics2D g, int ox, int oy, int px, int py);

//    protected void paintBot(Graphics2D g, Icon bot, int x, int y, float theta)
    protected void paintBot(Graphics2D g, BBotSymbol bot, int x, int y, float theta)
    {
      double sx = pelsPerXMeter / 16.5;
      double sy = pelsPerYMeter / 16.5;

      g.translate(x, y);
      g.scale(sx, sy);
//      g.rotate(theta);
      g.rotate(theta + Math.PI);

      if(bot == enemySymbol)
        bot.paint(g, 0.0f, "", "Enemy", "", true);
      else
        bot.paint(g, 0.0f, "", "", "", false);
      /*
      bot.paintIcon(
        BOrientPicker.this, g,
        -(bot.getIconWidth()  >> 1),
        -(bot.getIconHeight() >> 1));
*/
//      g.rotate(-theta);
      g.rotate(-theta - Math.PI);
      g.scale(1/sx, 1/sy);
      g.translate(-x, -y);
    }

    protected void paintStats(Graphics2D g, int x, int y)
    {
      g.drawString(
        "distance: "+
        BMath.format(BMath.round(hero.distance(enemy), 1))+
        "m", x, y);
    }

    protected BBotSymbol getEnemy()
      {
      //return BOrientPicker.this.icon;
        return BOrientPicker.this.enemySymbol;
      }

    protected BBotSymbol getHero()
      {
      //return BOrientPicker.this.icon;
        return BOrientPicker.this.heroSymbol;
      }

    protected void setFacing(float f)
    {
      facing = f;

      if (facing < 0)
        facing = (facing % 360) + 360;
      else if (facing > 360)
        facing = facing % 360;

      calculateAngle();
    }


    protected void setHeroLocation(double x, double y)
    {
      hero.x = constrain(x);
      hero.y = constrain(y);
      calculateAngle();
    }

    protected void calculateAngle()
    {
      if( !( (hero.x == enemy.x) && (hero.y == enemy.y) ) )
      {
        delta.sub(hero, enemy);
        angle = (float) delta.angle(up);

        if (hero.x < enemy.x)
          angle = -angle;

        angle -= Math.toRadians(facing);
      }
      else
        angle = 0.0f;
    }

    public void mousePressed(MouseEvent e)
    {
      super.mousePressed(e);
      mouse.set(x, y);
      pelsToMeters(mouse, mouse);
    }

    public void mouseDragged(MouseEvent e)
    {
      super.mouseDragged(e);
      mouse.set(x, y);
      pelsToMeters(mouse, mouse);
    }

    public void mouseReleased(MouseEvent e)
    {
      super.mouseReleased(e);
      mouse.set(x, y);
      pelsToMeters(mouse, mouse);
    }
  }


  // Full orient [location + facing]
  // -------------------------------------------------------------

  protected class BModeOrientFull extends BModeOrient
  {
    protected boolean isFacing;

    public void setModeFromOrient(BOrient o)
    {
      BOrientFull orient = (BOrientFull) o;

      setHeroLocation(-orient.getX(), orient.getZ());
      setFacing(orient.getFacing());

      setToolTipText("Left mouse button alters location, right alters facing.");
    }

    public void setOrientFromMode(BOrient o)
    {
      BOrientFull orient = (BOrientFull) o;

      orient.setXZ((float) -hero.x, (float) hero.y);
      orient.setFacing(facing);
    }

    public void mouseDragged(MouseEvent e)
    {
      super.mouseDragged(e);

      isFacing = e.isMetaDown();

      if (isFacing)
        setFacing(facing + dy);
      else
        setHeroLocation(mouse.x, mouse.y);

      BOrientPicker.this.repaint();
    }

    protected BBotSymbol getEnemy()
      { return BOrientPicker.this.enemySymbol2; }

    protected BBotSymbol getHero()
      { return BOrientPicker.this.heroSymbol; }


    protected void paintManipulator(Graphics2D g, int ox, int oy, int px, int py)
    {
      // Draw rubber-banding...
      g.setColor(lineColor);
      g.drawLine(ox, oy, px, py);

      // Draw radius...
      /*
      double dist = hero.distance(enemy);
      int xr = (int) metersToXPels(dist);
      int yr = (int) metersToYPels(dist);

      g.drawOval(ox - xr, oy - yr, xr << 1, yr << 1);
      */
    }

    protected void paintStats(Graphics2D g, int x, int y)
    {
      if (isFacing)
        g.drawString("facing: "+ BMath.format(facing)+ ".0 degrees", x, y);
      else
        g.drawString("location:  x "+
          BMath.format(BMath.round(-hero.x, 1))+ "m,  y "+
          BMath.format(BMath.round( hero.y, 1))+ "m", x, y);
    }
  }


  // Distance-only orient
  // -------------------------------------------------------------

  protected class BModeOrientDist extends BModeOrient
  {
    public void setModeFromOrient(BOrient o)
    {
      BOrientDistanceOnly orient = (BOrientDistanceOnly) o;

      setHeroLocation(0, orient.getDistance());
      setFacing(0);
      setToolTipText("Left mouse button alters distance.");
    }

    public void setOrientFromMode(BOrient o)
    {
      BOrientDistanceOnly orient = (BOrientDistanceOnly) o;
      orient.setDistance((float) hero.distance(enemy));
    }

    public void mouseDragged(MouseEvent e)
    {
      super.mouseDragged(e);
      setHeroLocation(mouse.x, mouse.y);
      BOrientPicker.this.repaint();
    }
/*
    protected Icon getEnemy()
      { return simple; }

    protected Icon getHero()
      { return simple; }
*/
    protected BBotSymbol getEnemy()
      { return BOrientPicker.this.enemySymbol; }

    protected BBotSymbol getHero()
      { return BOrientPicker.this.heroSymbol2; }


    protected void paintManipulator(Graphics2D g, int ox, int oy, int px, int py)
    {
      // Draw radius...
      double dist = hero.distance(enemy);
      int xr = (int) metersToXPels(dist);
      int yr = (int) metersToYPels(dist);

      g.setColor(lineColor);
      g.drawOval(ox - xr, oy - yr, xr << 1, yr << 1);
    }
  }


  // Facing-only orient
  // -------------------------------------------------------------

  protected class BModeOrientFace extends BModeOrient
  {
    public void setModeFromOrient(BOrient o)
    {
      BOrientFacingOnly orient = (BOrientFacingOnly) o;

      setHeroLocation(0, 6);
      setFacing(orient.getFacing());
      setToolTipText("Left mouse button alters facing.");
    }

    public void setOrientFromMode(BOrient o)
    {
      BOrientFacingOnly orient = (BOrientFacingOnly) o;
      orient.setFacing(facing);
    }

    public void mouseDragged(MouseEvent e)
    {
      super.mouseDragged(e);
      setFacing(facing + dy);

      BOrientPicker.this.repaint();
    }

    protected void paintManipulator(Graphics2D g, int ox, int oy, int px, int py)
      { }

//    protected Icon getEnemy()
//      { return simple; }
    protected BBotSymbol getEnemy()
      { return BOrientPicker.this.enemySymbol; }

    protected void paintStats(Graphics2D g, int x, int y)
      { g.drawString("facing: "+ BMath.format(facing)+ ".0 degrees", x, y); }
  }


  // Distance + facing orient
  // -------------------------------------------------------------

  protected class BModeOrientDistFacing extends BModeOrient
  {
    protected boolean isFacing;

    public void setModeFromOrient(BOrient o)
    {
      BOrientDistanceFacing orient = (BOrientDistanceFacing) o;

      setHeroLocation(0, orient.getDistance());
      setFacing(orient.getFacing());

      setToolTipText("Left mouse button alters distance, right alters facing.");
    }

    public void setOrientFromMode(BOrient o)
    {
      BOrientDistanceFacing orient = (BOrientDistanceFacing) o;

      orient.setDistance((float) hero.distance(enemy));
      orient.setFacing(facing);
    }

    public void mouseDragged(MouseEvent e)
    {
      super.mouseDragged(e);

      isFacing = e.isMetaDown();

      if (isFacing)
        setFacing(facing + dy);
      else
        setHeroLocation(mouse.x, mouse.y);

      BOrientPicker.this.repaint();
    }

    protected void paintManipulator(Graphics2D g, int ox, int oy, int px, int py)
    {
      // Draw rubber-banding...
      g.setColor(lineColor);

      // Draw radius...
      double dist = hero.distance(enemy);
      int xr = (int) metersToXPels(dist);
      int yr = (int) metersToYPels(dist);

      g.drawOval(ox - xr, oy - yr, xr << 1, yr << 1);
    }

    protected void paintStats(Graphics2D g, int x, int y)
    {
      if (isFacing)
        g.drawString("facing: "+ BMath.format(facing)+ ".0 degrees", x, y);
      else
        super.paintStats(g, x, y);
    }

//    protected Icon getEnemy()
//      { return simple; }
    protected BBotSymbol getEnemy()
      { return BOrientPicker.this.enemySymbol; }


  }


  // Location-only orient
  // -------------------------------------------------------------

  protected class BModeOrientLocation extends BModeOrient
  {
    public void setModeFromOrient(BOrient o)
    {
      BOrientLocationOnly orient = (BOrientLocationOnly) o;

      setHeroLocation(-orient.getX(), orient.getZ());
      setFacing(0);

      setToolTipText("Left mouse button alters location.");
    }

    public void setOrientFromMode(BOrient o)
    {
      BOrientLocationOnly orient = (BOrientLocationOnly) o;
      orient.setXZ((float) -hero.x, (float) hero.y);
    }

    public void mouseDragged(MouseEvent e)
    {
      super.mouseDragged(e);
      setHeroLocation(mouse.x, mouse.y);
      BOrientPicker.this.repaint();
    }

    protected BBotSymbol getEnemy()
      { return BOrientPicker.this.enemySymbol2; }

    protected BBotSymbol getHero()
      { return BOrientPicker.this.heroSymbol2; }

    protected void paintManipulator(Graphics2D g, int ox, int oy, int px, int py)
    {
      // Draw rubber-banding...
      g.setColor(lineColor);
      g.drawLine(ox, oy, px, py);
    }

    protected void paintStats(Graphics2D g, int x, int y)
    {
      g.drawString("location:  x "+
        BMath.format(BMath.round(-hero.x, 1))+ "m,  y "+
        BMath.format(BMath.round( hero.y, 1))+ "m", x, y);
    }
  }
}
