package roboforge.ui.ai;

import roboforge.BBot;

import roboforge.BDebug;

import roboforge.ai.BDecisionTreeAI;
import roboforge.ai.actions.*;
import roboforge.ai.decisions.*;
import roboforge.ai.expressions.*;
import roboforge.ai.values.*;
import roboforge.ui.components.*;

import roboforge.util.graphics.*;
import java.awt.Dimension;

import roboforge.BGlobal;

import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;

import java.util.Hashtable;

import javax.swing.*;

/*
  A panel that displays world item decisions that can be expressed as an angle
  */

public class BSimpleAnglePanel extends BSimpleDecisionPanel implements ActionListener, MouseListener, MouseMotionListener
{
  protected BUIResources res = new BUIResources( new Object[]
  {
    "titleIcon.foreground",         colors.getMidShade(),
    "titleIcon.size",               new Dimension(40, 40),

    "titleLabel.text",              "",
    "titleLabel.width",             new Integer(200),
    "titleLabel.fontsize",          new Integer(14),
    "titleLabel.height",            new Integer(20),
    "titleLabel.fontstyle",         new Integer(Font.BOLD),
    "titleLabel.v_align",           new Integer(SwingConstants.BOTTOM),

    "smallLine.width",              new Integer(10),

    "infoLabel.text",               "",
    "infoLabel.width",              new Integer(200),

    "optionCombo.tooltip",          "Options",

    "greaterButton.icon",           BIconCache.getHSB("op_icon_gtr.gif", BGlobal.colors.getDarkShade()),
    "greaterButton.tooltip",        "Greater than",

    "equalsButton.icon",            BIconCache.getHSB("op_icon_eq.gif", BGlobal.colors.getDarkShade()),
    "equalsButton.tooltip",         "Is equal to",

    "lessButton.icon",              BIconCache.getHSB("op_icon_less.gif", BGlobal.colors.getDarkShade()),
    "lessButton.tooltip",           "Less than",

    "rangeButton.icon",             BIconCache.getHSB("op_icon_gl.gif", BGlobal.colors.getDarkShade()),
    "rangeButton.tooltip",          "Range",

    "anglePicker.size",             new Dimension(230, 230),
    "anglePicker.background",       colors.getVeryDarkShade(),
    "anglePicker.meters",           new Double(20),
    "anglePicker.metersPerMajor",   new Double(5),
    "anglePicker.minorsPerMajor",   new Integer(5),
    "anglePicker.icon",             BIconCache.getHSB("bot_icon.gif"),

  });


  // hashtable containing world item refs mathched with actual words
  protected Hashtable optionTable = new Hashtable(6, 0.75f);

  //-------------- variable between different ref types
  private String title;
  private HSBImageIcon image;
  private String [] options;

  private double min;
  private double max;

  private String units;
  //---------------------------------------------------

  private static BSimpleAnglePanel instance = null;

  protected BBorderedIcon typeImage;

  protected BComboBox optionCombo = new BComboBox(res, "optionCombo");
  protected BLabel titleLabel = new BLabel(res, "titleLabel");
  protected BButton titleIcon = new BButton(res, "titleIcon")
  {
    protected void processMouseEvent(MouseEvent e)
    {}
  };

  protected BLabel infoLabel = new BLabel(res, "infoLabel");

  protected BLine titleLine = new BLine();
  protected BLine smallLine = new BLine(res, "smallLine");

  protected BRadioButton greaterButton = new BRadioButton(res, "greaterButton");
  protected BRadioButton equalsButton = new BRadioButton(res, "equalsButton");
  protected BRadioButton lessButton = new BRadioButton(res, "lessButton");
  protected BRadioButton rangeButton = new BRadioButton(res, "rangeButton");

  protected ButtonGroup operatorGroup = new ButtonGroup();

  protected BAnglePicker anglePicker = new BAnglePicker(res, "anglePicker");

  protected boolean initialized = false;

  private String optionString = "";
  private String operatorString = "";
  private String valueString = "";
  private String valueString2 = "";

  // constructor
  public BSimpleAnglePanel()
  {
    super("decision");
//    super(title);

    layoutPanelChildren();
    configPanelChildren();
  }

  protected void layoutPanelChildren()
  {
    BBoxPanel mainBox = createHBoxPanel();
    BBoxPanel leftBox = createVBoxPanel();
    BBoxPanel iconBox = createHBoxPanel();
    BBoxPanel lineBox = createVBoxPanel();
    BBoxPanel rightBox = createVBoxPanel();
    BBoxPanel operatorBox = createHBoxPanel();

    titleIcon.setHorizontalAlignment(BLabel.CENTER);

    leftBox.add(iconBox);
    leftBox.addGlue();

    iconBox.add(titleIcon);
    iconBox.add(lineBox);

    lineBox.addVStrut(1);
    lineBox.addSmallMargin();
    lineBox.add(smallLine);

    rightBox.addHGlued(titleLabel);
    rightBox.addTinyMargin();
    rightBox.addHGlued(titleLine);
    rightBox.addBigMargin();
    rightBox.add(optionCombo);
    rightBox.addSmallMargin();
    rightBox.add(operatorBox);
    rightBox.addSmallMargin();
    rightBox.add(anglePicker);
    rightBox.addSmallMargin();
    rightBox.addHGlued(infoLabel);
    rightBox.addGlue();

    operatorBox.add(lessButton);
    operatorBox.add(equalsButton);
    operatorBox.add(greaterButton);
    operatorBox.add(rangeButton);
    operatorBox.addGlue();

    mainBox.add(leftBox);
    mainBox.add(rightBox);
    add(mainBox);
 }


  protected void configPanelChildren()
  {
    operatorGroup.add(greaterButton);
    operatorGroup.add(equalsButton);
    operatorGroup.add(lessButton);
    operatorGroup.add(rangeButton);

    greaterButton.setSelectedIcon(BGraphics.getHSBIcon("op_icon_gtr.gif", BGlobal.colors.getMidShade()));
    equalsButton.setSelectedIcon(BGraphics.getHSBIcon("op_icon_eq.gif", BGlobal.colors.getMidShade()));
    lessButton.setSelectedIcon(BGraphics.getHSBIcon("op_icon_less.gif", BGlobal.colors.getMidShade()));
    rangeButton.setSelectedIcon(BGraphics.getHSBIcon("op_icon_gl.gif", BGlobal.colors.getMidShade()));

    greaterButton.addActionListener(this);
    equalsButton.addActionListener(this);
    lessButton.addActionListener(this);
    rangeButton.addActionListener(this);

    optionCombo.addActionListener(this);

    anglePicker.addMouseListener(this);
    anglePicker.addMouseMotionListener(this);
  }

  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  public void mouseMoved(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}
  public void mouseClicked(MouseEvent e) {}

  public void mousePressed(MouseEvent e)
  { getAngleFromPicker(); }

  public void mouseDragged(MouseEvent e)
  { getAngleFromPicker(); }

  protected void getAngleFromPicker()
  {
    if(this.isActive()) {
      valueString = "" + anglePicker.getAngle1() + ".0";
      valueString2 = "" + anglePicker.getAngle2() + ".0";
      if(initialized)
      {
        updateInfo();
        //setNodeFromPanel();
      }
    }
  }

  public void actionPerformed(ActionEvent e)
  {
    if (this.isActive()) {
      if (e.getSource() == lessButton) {
        operatorString = "less than";
        anglePicker.setAngleOperator("<");
      }
      if (e.getSource() == greaterButton) {
        operatorString = "greater than";
        anglePicker.setAngleOperator(">");
      }
      if (e.getSource() == equalsButton) {
        operatorString = "equal to";
        anglePicker.setAngleOperator("=");
      }
      if (e.getSource() == rangeButton) {
        operatorString = "range";
        anglePicker.setAngleOperator("range");
      }
      if(e.getSource() == optionCombo)
        optionString = (String)optionCombo.getSelectedItem();

      if(initialized) {
        updateInfo();
        //setNodeFromPanel();
      }
    }
  }

  // save the data from the panel to the node
  protected void setNodeFromPanel()
  {
    if(node != null)
    {

      BSimpleAngleNode angleNode = (BSimpleAngleNode) node;

      angleNode.setWorldItemRef((String)optionTable.get(optionString));

      String op;

      if(operatorString.equals("less than"))
        op = "<";
      else if(operatorString.equals("equal to"))
        op = "=";
      else if(operatorString.equals("greater than"))
        op = ">";
      else
        op = "range";

      /// code to handle the range case needed here /////////////////////////////

      angleNode.setValueRef(new BNumber((double)anglePicker.getAngle1()));
      angleNode.setValueRef2(new BNumber((double)anglePicker.getAngle2()));

      angleNode.setOperatorRef(op);

      String exp;

      if(op.equals("range"))
      {
        if(anglePicker.getAngle2() > anglePicker.getAngle1())
          exp = "([" + optionTable.get(optionString) + "] > " + anglePicker.getAngle2() + ") or ([" + optionTable.get(optionString) + "] < " + anglePicker.getAngle1() + ")";
        else
          exp = "([" + optionTable.get(optionString) + "] > " + anglePicker.getAngle2() + ") and ([" + optionTable.get(optionString) + "] < " + anglePicker.getAngle1() + ")";
        angleNode.setName("Angle in range " + valueString2 + " - " + valueString + "?");
      }
      else
      {
        exp = "[" + optionTable.get(optionString) + "] " + op + " " + anglePicker.getAngle1() ;
        angleNode.setName("Angle " + op + " " + valueString + "?");
      }

      updateNodeExpression(exp);

      // set nice name
      //angleNode.setName(title);

    }
  }

  // set up the panel data from the node
  protected void setPanelFromNode(BDecisionTreeNode n)
  {
    if(initialized) {
      this.node = (BSimpleAngleNode)n;

      optionCombo.setSelectedItem(optionTable.get(node.getWorldItemRef()));
      optionString = (String)optionCombo.getSelectedItem();

      if(node.getOperatorRef().equals("<")) {
       lessButton.setSelected(true);
       operatorString = "less than";
       anglePicker.setAngleOperator("<");
      }
      else if (node.getOperatorRef().equals("=")) {
        equalsButton.setSelected(true);
        operatorString = "equal to";
        anglePicker.setAngleOperator("=");
      }
      else if (node.getOperatorRef().equals(">")) {
        greaterButton.setSelected(true);
        operatorString = "greater than";
        anglePicker.setAngleOperator(">");
      }
      else {
        rangeButton.setSelected(true);
        operatorString = "range";
        anglePicker.setAngleOperator("range");
      }

      BSimpleAngleNode angleNode = (BSimpleAngleNode)node;
      anglePicker.setPickerFromAngle((float)((BNumber)angleNode.getValueRef()).getValue(), (float)((BNumber)angleNode.getValueRef2()).getValue());

      valueString = "" + anglePicker.getAngle1() + ".0";
      valueString2 = "" + anglePicker.getAngle2() + ".0";

      updateInfo();
    }
  }

    // setup the AI
  public void setAI(BDecisionTreeAI ai)
  {
    BDebug.print("BSimpleAnglePanel - in setAI method");
    super.setAI(ai);
  }

  public static BAINodePanel getInstance()
  {
    if (instance == null)
      instance = new BSimpleAnglePanel();

    return instance;
  }

  // updates the useful message at the base of the panel
  private void updateInfo()
  {
    if(operatorString.equals("range"))
      infoLabel.setText("Is " + optionString.toLowerCase() + " in the range " + valueString2 + units + " - " + valueString + units + " ?");
    else
      infoLabel.setText("Is " + optionString.toLowerCase() + " " + operatorString + " " + valueString + units + " ?");

  }

  // sets the panel data from a simple number data object
  protected void setData(BSimpleDecisionData unformatedData)
  {
    BSimpleNumberData data = (BSimpleNumberData) unformatedData;

    initialized = false;
    image = data.getIcon();
    title = data.getTitle();
    options = data.getOptions();

    min = data.getMin();
    max = data.getMax();

    units = data.getUnits();

    titleLabel.setText(title + " (" + units + ")");
    titleIcon.setIcon(data.getIcon());

    // clear the comboBox and hashtable
    optionCombo.removeAllItems();
    optionTable.clear();

    // fill the comboBox and hashtable
    for (int i = 0; i < options.length; i += 2) {
      optionTable.put(options[i], options[i+1]);
      optionTable.put(options[i+1], options[i]);
      optionCombo.addItem(options[i + 1]);
    }
    initialized = true;
  }

  public void onClose()
  {
    instance = null;
  }

  public static void clearInstance()
  { instance = null; }

}
