package roboforge.ui.ai;

import roboforge.BBot;

import roboforge.BDebug;

import roboforge.ai.BDecisionTreeAI;
import roboforge.ai.actions.*;
import roboforge.ai.decisions.*;
import roboforge.ai.expressions.*;
import roboforge.ai.values.*;
import roboforge.ui.components.*;

import roboforge.util.graphics.*;
import java.awt.Dimension;

import roboforge.BGlobal;

import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;

import java.util.Hashtable;

import javax.swing.*;

/*
  A panel that displays world item decisions that can be expressed as a simple boolean
  */

public class BSimpleBooleanPanel extends BSimpleDecisionPanel implements ActionListener
{
  protected BUIResources res = new BUIResources( new Object[]
  {
    "titleIcon.foreground",         colors.getMidShade(),
    "titleIcon.size",               new Dimension(40, 40),

    "titleLabel.text",              "",
    "titleLabel.width",             new Integer(200),
    "titleLabel.fontsize",          new Integer(14),
    "titleLabel.height",            new Integer(20),
    "titleLabel.fontstyle",         new Integer(Font.BOLD),
    "titleLabel.v_align",           new Integer(SwingConstants.BOTTOM),

    "smallLine.width",              new Integer(10),

    "infoLabel.text",               "",
    "infoLabel.width",              new Integer(200),

    "optionCombo.tooltip",          "Options",
  });



  // hashtable containing world item refs mathched with actual words
  protected Hashtable optionTable = new Hashtable(6, 0.75f);

  //-------------- variable between different ref types
  private String title;
  private HSBImageIcon image;
  private String [] options;
  //---------------------------------------------------

  private static BSimpleBooleanPanel instance = null;

  protected BBorderedIcon typeImage;

  protected BComboBox optionCombo = new BComboBox(res, "optionCombo");
  protected BLabel titleLabel = new BLabel(res, "titleLabel");
  protected BButton titleIcon = new BButton(res, "titleIcon")
  {
    protected void processMouseEvent(MouseEvent e)
    {}
  };
  
  protected BLine titleLine = new BLine();
  protected BLine smallLine = new BLine(res, "smallLine");

  protected boolean initialized = false;

  protected String optionString = "";

  // constructor
  public BSimpleBooleanPanel()
  {
    super("decision");
    //super(title);

    layoutPanelChildren();
    configPanelChildren();
  }

  protected void layoutPanelChildren()
  {
    BBoxPanel mainBox = createHBoxPanel();
    BBoxPanel leftBox = createVBoxPanel();
    BBoxPanel iconBox = createHBoxPanel();
    BBoxPanel lineBox = createVBoxPanel();
    BBoxPanel rightBox = createVBoxPanel();
    BBoxPanel operatorBox = createHBoxPanel();

    titleIcon.setHorizontalAlignment(BLabel.CENTER);

    leftBox.add(iconBox);
    leftBox.addGlue();

    iconBox.add(titleIcon);
    iconBox.add(lineBox);

    lineBox.addVStrut(1);
    lineBox.addSmallMargin();
    lineBox.add(smallLine);

    rightBox.addHGlued(titleLabel);
    rightBox.addTinyMargin();
    rightBox.addHGlued(titleLine);
    rightBox.addBigMargin();
    rightBox.add(optionCombo);
    rightBox.addGlue();

    mainBox.add(leftBox);
    mainBox.add(rightBox);
    add(mainBox);
 }


  protected void configPanelChildren()
  {
    optionCombo.addActionListener(this);
  }

  public void actionPerformed(ActionEvent e)
  {
    if(this.isActive()) {
      if(e.getSource() == optionCombo)
        optionString = (String)optionCombo.getSelectedItem();
      if(initialized)
        setNodeFromPanel();
    }
  }

  // save the data from the panel to the node
  protected void setNodeFromPanel()
  {
    if(node != null)
    {
      node.setWorldItemRef((String)optionTable.get(optionString));

      String exp = "[" + optionTable.get(optionString) + "]";
      updateNodeExpression(exp);

      // set nice name
      //node.setName(title);
      node.setName(optionString);
    }

  }

  // set up the panel data from the node
  protected void setPanelFromNode(BDecisionTreeNode n)
  {
    if(initialized) {
      this.node = (BSimpleDecisionNode)n;
      optionCombo.setSelectedItem(optionTable.get(node.getWorldItemRef()));
      optionString = (String)optionCombo.getSelectedItem();
    }
  }

  // setup the AI
  public void setAI(BDecisionTreeAI ai)
  {
    super.setAI(ai);
  }

  public static BAINodePanel getInstance()
  {
    if (instance == null)
      instance = new BSimpleBooleanPanel();

    return instance;
  }

  // sets the panel data from a simple number data object
  protected void setData(BSimpleDecisionData data)
  {
    initialized = false;
    image = data.getIcon();
    title = data.getTitle();
    options = data.getOptions();

    titleLabel.setText(title);
    titleIcon.setIcon(data.getIcon());

    // clear the comboBox and hashtable
    optionCombo.removeAllItems();
    optionTable.clear();

    // fill the comboBox and hashtable
    for (int i = 0; i < options.length; i += 2) {
      optionTable.put(options[i], options[i+1]);
      optionTable.put(options[i+1], options[i]);
      optionCombo.addItem(options[i + 1]);
    }
    initialized = true;
  }

  public void onClose()
  {
    instance = null;
  }

  public static void clearInstance()
  { instance = null; }
}