package roboforge.ui.components;

import roboforge.BGlobal;

import java.awt.*;
import javax.swing.*;



public class BBlockBar extends BProgressBar
{
  
  // Class variables...
  // -------------------------------------------------------------

  protected static final Color EMPTY
    = BGlobal.colors.getDarkerShade();
  
  protected static final Color FULL
    = BGlobal.colors.getMidShade();

  
  
  // Constructors...
  // -------------------------------------------------------------

  public BBlockBar(BUIResources res, String key)
    { super(res, key); }

  public BBlockBar(BUIResources res, String key, int min, int max)
    { super(res, key, min, max); }

  public BBlockBar()
    { super(); }


  
  // Paint method...
  // -------------------------------------------------------------

  protected int getCellCount(int w, int h)
  { 
    int g = getGapWidth();
    return (w - g) / (h + g); 
  }

  protected double getCellWidth(int cells, int w, int h)
  { 
    int g     = getGapWidth();
    double n  = cells;
    
    return (w - (n-1)*g) / n; 
  }
  
  protected int getGapWidth()
    { return 2; }
  
  
  public void paint(Graphics g)
  {
    int w     = getWidth();
    int h     = getHeight();
    
    Insets insets = getInsets();
    
    int ew    = w - (insets.left + insets.right);
    int eh    = h - (insets.top + insets.bottom);
    
    int n     = getCellCount(ew, eh);
    int gw    = getGapWidth();
    double cw = getCellWidth(n, ew, eh);
    
    double dx = cw + gw;
    double dl = insets.left;
    double dr = dl + cw;

    int l  = (int) Math.round(dl);
    int r  = (int) Math.round(dr);
    int t  = insets.top;

    
    // X-position of percentage progress...
    int p = l + (int) Math.round(ew * getPercentComplete());

   
    // Draw backdrop...
    g.setColor(getBackground());
    g.fillRect(0, 0, w, h);  // l, t, ew, eh);
    
    // Ensure all painting stays inside component...
    g.setClip(l, t, ew, eh);
    
    // Draw each cell...
    for (int i=0; i<n; i++)
    {
      if (r <= p)
      {
        // Full.
        g.setColor(FULL);
        g.fillRect(l, t, (r - l), eh);
      }
      else if (l >= p)
      {
        // Empty.
        g.setColor(EMPTY);
        g.fillRect(l, t, (r - l), eh);
      }
      else
      {
        // Partially full.
        g.setColor(FULL);
        g.fillRect(l, t, (p - l), eh);
        
        g.setColor(EMPTY);
        g.fillRect(p, t, (r - p), eh);
      }
      
      dl += dx;
      dr += dx;
      
      l = (int) Math.round(dl);
      r = (int) Math.round(dr);
    }
    
  }
  
}
