package roboforge.ui.components;

import roboforge.BDebug;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;



public class BBoxPanel extends BPanel
{

  // Class variables...
  // -------------------------------------------------------------

  public static final int X_AXIS           = BoxLayout.X_AXIS;
  public static final int Y_AXIS           = BoxLayout.Y_AXIS;

  protected static final int TINY_MARGIN   = 5;
  protected static final int SMALL_MARGIN  = 10;
  protected static final int MARGIN        = 15;
  protected static final int BIG_MARGIN    = 30;


  // Member variables...
  // -------------------------------------------------------------

  protected boolean BORDER_DEBUG = false;
  protected BoxLayout layout = null;
  protected int axis;


  // Constructors.
  // -------------------------------------------------------------

  public BBoxPanel(BUIResources res, String key, int axis)
  {
    super(res, key);
    this.axis = axis;

    this.layout = new BoxLayout(this, axis);
    setLayout(layout);
  }

  public BBoxPanel(int axis)
    { this(null, null, axis); }


  // Action methods.
  // -------------------------------------------------------------

  public void setEmptyBorder(int size)
  {
    Border b = getBorder();
    Border e = new EmptyBorder(size, size, size, size);

    if (b == null)
      setBorder(e);
    else
      setBorder(BorderFactory.createCompoundBorder(b, e));
  }

  public BBoxPanel createHBoxPanel()
  {
    BBoxPanel result = new BBoxPanel(BoxLayout.X_AXIS);
    result.setBackground(getBackground());
    return result;
  }

  public BBoxPanel createVBoxPanel()
  {
    BBoxPanel result = new BBoxPanel(BoxLayout.Y_AXIS);
    result.setBackground(getBackground());
    return result;
  }

  public Component getStrut(int size)
  {
    if (axis == BoxLayout.X_AXIS)
      return getHStrut(size);
    else
      return getVStrut(size);
  }

  public Component getOrthogonalStrut(int size)
  {
    if (axis == BoxLayout.X_AXIS)
      return getVStrut(size);
    else
      return getHStrut(size);
  }

  public static Component getVStrut(int size)
    { return Box.createRigidArea(new Dimension(0, size)); }

  public static Component getHStrut(int size)
    { return Box.createRigidArea(new Dimension(size, 0)); }


  public Component getMargin()
    { return getStrut(MARGIN); }

  public Component getTinyMargin()
    { return getStrut(TINY_MARGIN); }

  public Component getSmallMargin()
    { return getStrut(SMALL_MARGIN); }

  public Component getBigMargin()
    { return getStrut(BIG_MARGIN); }


  public void addStrut(int size)
    { add(getStrut(size)); }

  public void addHStrut(int size)
    { add(getHStrut(size)); }

  public void addVStrut(int size)
    { add(getVStrut(size)); }

  public void addMargin()
    { add(getMargin()); }

  public void addTinyMargin()
    { add(getTinyMargin()); }

  public void addSmallMargin()
    { add(getSmallMargin()); }

  public void addBigMargin()
    { add(getBigMargin()); }


  public BBoxPanel createBoxPanel()
  {
    BBoxPanel box;

    if (axis == BoxLayout.X_AXIS)
    {
      box = createVBoxPanel();
      // box.setAlignmentY(Component.TOP_ALIGNMENT);
    }
    else
    {
      box = createHBoxPanel();
      // box.setAlignmentX(Component.LEFT_ALIGNMENT);
    }
    return box;
  }

  public void addLinear(Component c1, Component c2)
  {
    BBoxPanel box = createBoxPanel();

    box.add(c1); box.add(c2);
    add(box);
  }

  public void addLinear(Component c1, Component c2, Component c3)
  {
    BBoxPanel box = createBoxPanel();
    box.add(c1); box.add(c2); box.add(c3);
    add(box);
  }

  public void addLinear(Component c1, Component c2, Component c3, Component c4)
  {
    BBoxPanel box = createBoxPanel();
    box.add(c1); box.add(c2); box.add(c3); box.add(c4);
    add(box);
  }

  public void addGlue()
  {
    if (axis == BoxLayout.X_AXIS)
      addHGlue();
    else
      addVGlue();
  }

  public void addHGlue()
    { add(Box.createHorizontalGlue()); }

  public void addVGlue()
    { add(Box.createVerticalGlue()); }

  public Component getHGlue()
    { return Box.createHorizontalGlue(); }

  public Component getVGlue()
    { return Box.createVerticalGlue(); }


  public void addRigidArea(Dimension d)
    { add(Box.createRigidArea(d)); }


  public void setGlue(JComponent c)
  {
    if (axis == BoxLayout.X_AXIS)
      setHGlue(c);
    else
      setVGlue(c);
  }

  public void setOrthogonalGlue(JComponent c)
  {
    if (axis == BoxLayout.Y_AXIS)
      setHGlue(c);
    else
      setVGlue(c);
  }

  public static void setHVGlue(JComponent c)
  {
    setHGlue(c);
    setVGlue(c);
  }

  public static void setHGlue(JComponent c)
  {
    Dimension d = c.getPreferredSize();
    d.width = Short.MAX_VALUE;
    c.setPreferredSize(d);

    d = c.getMaximumSize();
    d.width = Short.MAX_VALUE;
    c.setMaximumSize(d);
  }

  public static void setVGlue(JComponent c)
  {
    Dimension d = c.getPreferredSize();
    d.height = Short.MAX_VALUE;
    c.setPreferredSize(d);

    d = c.getMaximumSize();
    d.height = Short.MAX_VALUE;
    c.setMaximumSize(d);
  }

  public void addGlued(JComponent c)
    { setOrthogonalGlue(c); add(c); }

  public void addHGlued(JComponent c)
    { setHGlue(c); add(c); }

  public void addVGlued(JComponent c)
    { setVGlue(c); add(c); }

  public void addHVGlued(JComponent c)
    { setHGlue(c); setVGlue(c); add(c); }


  
  public void addBefore(Component c, Component e)
  {
    int i = indexOf(e);
    
    if (i < 0)
      add(c);
    else
      add(c, i);
  }
  
  public void addAfter(Component c, Component e)
  {
    int i = indexOf(e);
    
    if (i < 0)
      add(c, -1);
    else
      add(c, i+1);
  }

  
  public int indexOf(Component c)
  {
    Component[] cs = getComponents();
    int n = cs.length;
    
    for (int i=0; i<n; i++)
      if (cs[i].equals(c))
        return i;
    
    return -1;
  }
  
  /*
  public void add(JComponent c)
  {
    if (BORDER_DEBUG)
    {
      c.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createLineBorder(Color.yellow),
        c.getBorder()));
    }

    super.add(c);
  }
  */


}

