package roboforge.ui.components;

import roboforge.BDebug;
import roboforge.ui.dnd.*;
import roboforge.util.graphics.BGraphics;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;

import java.io.*;
import java.io.IOException;

import javax.swing.JList;
import javax.swing.DefaultListModel;


public class BDragDropList extends BList
  implements
    // BDragDroppable,
    DragSourceListener,
    DragGestureListener
{


  // Internal variables...
  // -------------------------------------------------------------

  protected DragSource dragSource  = null;
  protected DropTarget dropTarget  = null;

  protected Object     dragData    = null;
  protected int        dragIndex   = -1;
  protected int        dropIndex   = -1;

  protected static final DataFlavor OBJECT =
    ObjectSelection.createObjectFlavor();

  protected Cursor linkCursor = BGraphics.getCursor("link_cursor.gif", 0, 0);
  // Constructor...
  // -------------------------------------------------------------

  public BDragDropList()
    { this(null, null); }

  public BDragDropList(BUIResources res, String key)
  {
    super(res, key);

    dragSource = new DragSource();
    dropTarget = new DropTarget(this, new DropListener());
    dragSource.createDefaultDragGestureRecognizer
      (this, DnDConstants.ACTION_MOVE, this);
  }

  // Drag 'n Drop methods...
  // -------------------------------------------------------------


  protected boolean acceptDrop(Object o, int i)
    // Determines what objects can be dropped on this list.
  {
    return true;
  }

  protected boolean acceptDrag(Object o, int i)
    // Determines what objects can be dragged off this list.
  {
    BMutableListModel model =
      (BMutableListModel) getModel();
      
    return model.isRemoveableAt(i);
  }


  public void dragGestureRecognized(DragGestureEvent e)
    // a drag gesture has been initiated
  {
    dragData = getSelectedValue();

    boolean dragAccepted =
      acceptDrag(dragData, getSelectedIndex());

    if ((dragData != null) && dragAccepted)
    {
      // System.out.println("Drag data: "+ dragData);

      // Reset indices...
      dragIndex = getSelectedIndex();
      dropIndex = -1;

      ObjectSelection o = new ObjectSelection(dragData);
      dragSource.startDrag (e, linkCursor, o, this);
    }
    else
      { /* System.out.println( "nothing was selected"); */ }
  }

  public void dragDropEnd (DragSourceDropEvent e)
    // goes to DragSourceListener, informing it that dragging has ended.
  {
    // System.out.println("DragDropEnd: success = "+ e.getDropSuccess());

    if (e.getDropSuccess())
      extractDragData();

    // Reset indices...
    dragIndex = -1;
    dropIndex = -1;
  }

  public void dragEnter (DragSourceDragEvent e)
    // goes to DragSourceListener, informing it that dragging has
    // entered the DropSite.
  {
    // System.out.println( "dragEnter");
  }

  public void dragExit (DragSourceEvent e)
    // this message goes to DragSourceListener, informing it that the
    // dragging has exited the DropSite.
  {
    // System.out.println( "dragExit");
  }

  public void dragOver (DragSourceDragEvent e)
    // this message goes to DragSourceListener, informing it that dragging
    // is currently occuring over the DropSite
  {
    // System.out.println( "dragExit");


  }

  public void dropActionChanged (DragSourceDragEvent e)
    // is invoked when the user changes the dropAction
  {
    // System.out.println( "dropActionChanged");
  }


  // -------------------------------------------------------------
  // Drop Target Listener Class...
  // -------------------------------------------------------------

  protected class DropListener implements DropTargetListener
  {

    public void dragEnter (DropTargetDragEvent e)
      // is invoked when you are dragging over the DropSite
    {
      // debug messages for diagnostics
      // System.out.println( "dragEnter");
      e.acceptDrag(DnDConstants.ACTION_MOVE);
    }

    public void dragExit (DropTargetEvent e)
      // is invoked when you are exit the DropSite without dropping
    {
      // System.out.println( "dragExit");
    }

    public void dragOver (DropTargetDragEvent e)
      // is invoked when a drag operation is going on
    {
      // System.out.println( "dragOver");
    }

    public void drop (DropTargetDropEvent e)
      // a drop has occurred
    {
      // System.out.println("DROP DETECTED");

      try
      {
        Transferable t = e.getTransferable();
        if (t.isDataFlavorSupported(OBJECT))
        {
          Object o = t.getTransferData(OBJECT);
          int i = locationToIndex(e.getLocation());

          if (acceptDrop(o, i))
          {
            e.acceptDrop(DnDConstants.ACTION_MOVE);
            insertDropData(o, i);
            e.getDropTargetContext().dropComplete(true);
          }
          else
            e.rejectDrop();
        }
        else
          e.rejectDrop();
      }
      catch (IOException ex)
      {
        System.err.println("Exception: " + ex.getMessage());
        e.rejectDrop();
      }
      catch (UnsupportedFlavorException ex)
      {
        System.err.println("Exception: " + ex.getMessage());
        e.rejectDrop();
      }
    }

    public void dropActionChanged ( DropTargetDragEvent e )
      // is invoked if the use modifies the current drop gesture
    { }

  }


  // Add/Remove methods...
  // -------------------------------------------------------------

  public void insertDropData(Object o, int i)
  {
    BMutableListModel model = (BMutableListModel) getModel();

    dropIndex = i;

    // BDebug.print("insertDropData("+p.x+","+p.y+"):  PRE: drag= "+dragIndex+". drop= "+dropIndex);

    // Past end of list...
    if (dropIndex < 0)
      dropIndex = model.getSize();

    // handle local drag n' drop...
    else if (dragIndex >= 0)
    {
      if (dragIndex < dropIndex)
        dropIndex++;

      else if (dragIndex > dropIndex)
        dragIndex++;
    }

    // BDebug.print("insertDropData("+p.x+","+p.y+"): POST: drag= "+dragIndex+". drop= "+dropIndex);

    model.addAt(o, dropIndex);

    setSelectedIndex(dropIndex);
  }

  public void extractDragData()
  {
    BMutableListModel model = (BMutableListModel) getModel();

    // BDebug.print("removeDragData():  PRE: drag= "+dragIndex+". drop= "+dropIndex);
    model.removeAt(dragIndex);
  }

}
