package roboforge.ui.components;

import roboforge.ui.BApplication;
import roboforge.ui.events.*;

import java.awt.event.*;
import java.util.Vector;

import javax.swing.JList;
import javax.swing.ListModel;
import javax.swing.KeyStroke;

public class BList extends JList
{

  // Internal variables...
  // -------------------------------------------------------------

  protected static final KeyStroke DELETE =
    KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0);

  protected boolean rowsDeleteable = false;
  protected boolean confirmDelete  = true;
  protected String  itemName       = "item";


  // Constructors...
  // -------------------------------------------------------------

  public BList(BUIResources res, String key)
  {
    super();
    initializeUI(res, key);
  }

  public BList()
    { super(); }

  public BList(ListModel model)
    { super(model); }

  public BList(Vector data)
    { super(data); }

  public BList(Object[] data)
    { super(data); }


  // Visual Initialization methods...
  // -------------------------------------------------------------

  public void initializeUI(BUIResources res, String key)
  {
    if (res != null)
      res.initComponent(this, key);
  }

  // Row Deletion methods...
  // -------------------------------------------------------------

  public void setRowsDeleteable(boolean d)
  {
    if (d)
    {
      if (!rowsDeleteable)
        registerKeyboardAction
          (new DeleteListener(), DELETE, WHEN_FOCUSED);
    }
    else
    {
      if (rowsDeleteable)
        unregisterKeyboardAction(DELETE);
    }

    this.rowsDeleteable = d;
  }

  public boolean areRowsDeleteable()
    { return this.rowsDeleteable; }


  public void setConfirmDelete(boolean c)
    { this.confirmDelete = c; }

  public boolean getConfirmDelete()
    { return this.confirmDelete; }

  public void setItemName(String name)
    { itemName = name; }

  protected BPromptDialog getDeleteConfirmDialog(Object o)
  {
    return
      BPromptDialog.createYesNoDialog
      (
        "confirm remove...",
        "You are about to remove the following "+ itemName+ ":\n"+
        "   "+ o.toString()+ "\n\n"+
        "Are you sure you wish to do this?"
      );
  }


  protected class DeleteListener extends BAsyncActionAdapter
  {
    public void asyncActionApproved(ActionEvent e)
    {
      int selected = getSelectedIndex();

      if (selected != -1)
      {
        BMutableListModel model =
          (BMutableListModel) getModel();

        if (model.isRemoveableAt(selected))
        {
          if (confirmDelete)
          {
            // Get user confirmation...
            Object o = getSelectedValue();

            BPromptDialog dlg = getDeleteConfirmDialog(o);
            BApplication.showDialog(dlg);

            if (dlg.getResult())
              model.removeAt(selected);

            // Get the focus back.
            requestFocus();
          }
          else
            // Delete without confirmation.
            model.removeAt(selected);
        }
      }
    }
  }

}
