package roboforge.ui.components;

import roboforge.BDebug;
import roboforge.ui.modes.*;
import roboforge.util.graphics.*;

import java.awt.*;
import java.awt.event.*;

import javax.vecmath.*;



public class BMouseableGrid2D extends BGrid2D
{

  // Member variables...
  // -------------------------------------------------------------

  protected BModeMouse mode;


  // Constructors...
  // -------------------------------------------------------------

  public BMouseableGrid2D(BUIResources res, String key)
  {
    super(res, key);
    setupHandler();
  }


  protected void setupHandler()
  {
    MouseHandler handler =
      new MouseHandler();

    addMouseListener(handler);
    addMouseMotionListener(handler);
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setMode(BModeMouse m)
  {
    if (mode != null)
      mode.requestDeactivate();

    mode = m;
    mode.requestActivate();
  }

  public BModeMouse getMode()
    { return mode; }


  public BModeMulti viewModeWrapper(BModeMouse defaultMode)
  {
    BModeKeyedMulti view =
      new BModeKeyedMulti(defaultMode);

    view.setCtrlMode(new BModeGridPan());
    view.setShiftMode(new BModeGridZoom());

    return view;
  }


  // Mode classes...
  // -------------------------------------------------------------

  protected class BModeGrid extends BModeMouse
  {
    public void onActivate()
    {
      super.onActivate();
      BMouseableGrid2D.this.setCursor(cursor);
    }
  }

  protected class BModeGridPan extends BModeGrid
  {
    protected Point2d p = new Point2d();

    public BModeGridPan()
    {
      super();
      cursor = BGraphics.getCursor("pan_cursor.gif", 15, 15);
    }

    public void mouseDragged(MouseEvent e)
    {
      super.mouseDragged(e);

      double dmx = dx / pelsPerXMeter;
      double dmy = dy / pelsPerYMeter;

      p.set(xm - dmx, ym - dmy);
      setViewLocation(p);
    }
  }

  protected class BModeGridZoom extends BModeGrid
  {
    public BModeGridZoom()
    {
      super();
      cursor = BGraphics.getCursor("zoom_cursor.gif", 15, 15);
    }

    public void mouseDragged(MouseEvent e)
    {
      super.mouseDragged(e);
      setViewScale(scale + (dx - dy) / 100.0);
    }
  }



  // Mouse handler...
  // -------------------------------------------------------------

  protected class MouseHandler extends MouseAdapter
    implements MouseMotionListener
  {
    public void mousePressed(MouseEvent e)
      { if (mode != null) mode.mousePressed(e); }

    public void mouseDragged(MouseEvent e)
      { if (mode != null) mode.mouseDragged(e); }

    public void mouseReleased(MouseEvent e)
      { if (mode != null) mode.mouseReleased(e); }

    public void mouseMoved(MouseEvent e)
      { }
  }

}
