package roboforge.ui.components.border;

import roboforge.BGlobal;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;



public class BLineBorder implements Border
{

  // Class variables...
  // -------------------------------------------------------------

  public static final int SIDES_ALL               = 0;
  public static final int SIDES_TOP_BOTTOM        = 1;
  public static final int SIDES_LEFT_RIGHT        = 2;
  public static final int SIDES_LEFT              = 3;
  public static final int SIDES_RIGHT             = 4;
  public static final int SIDES_TOP               = 5;
  public static final int SIDES_BOTTOM            = 6;
  public static final int SIDES_TOP_BOTTOM_RIGHT  = 7;


  protected static final Insets[] insets =
    // This array uses the above type IDs as indices.
  {
    new Insets(1, 1, 1, 1),
    new Insets(1, 0, 1, 0),
    new Insets(0, 1, 0, 1),
    new Insets(0, 1, 0, 0),
    new Insets(0, 0, 0, 1),
    new Insets(1, 0, 0, 0),
    new Insets(0, 0, 1, 0),
    new Insets(1, 0, 1, 1)
  };



  // Member variables...
  // -------------------------------------------------------------

  protected int type;

  protected Color enabledColor   = null;
  protected Color focusedColor   = null;
  protected Color disabledColor  = null;


  // Constructors...
  // -------------------------------------------------------------

  public BLineBorder()
    { this(BGlobal.colors.getDarkerShade()); }
  
  public BLineBorder(Color c)
    { this(SIDES_ALL, c, c, c); }

  public BLineBorder(int type, Color c)
    { this(type, c, c, c); }

  public BLineBorder
  (
    int type,
    Color enabledColor,
    Color focusedColor,
    Color disabledColor
  )
  {
    if ((type < SIDES_ALL) || (type > SIDES_TOP_BOTTOM_RIGHT))
      throw new IllegalArgumentException("Unrecognized border-type ID: "+ type);

    this.type          = type;
    this.enabledColor  = enabledColor;
    this.focusedColor  = focusedColor;
    this.disabledColor = disabledColor;
  }



  // Accessor methods...
  // -------------------------------------------------------------

  public Insets getBorderInsets(Component c)
    { return insets[type]; }

  public boolean isBorderOpaque()
    { return false; }


  // Implementation methods...
  // -------------------------------------------------------------

  public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)
  {
    if (c.isEnabled())
    {
      /*
      Component parent = c.getParent();
      
      boolean hasFocus =
        c.hasFocus() || 
        ((parent != null) && parent.hasFocus());
      */
        
      if (c.hasFocus())
        g.setColor(focusedColor);
      else
        g.setColor(enabledColor);
    }
    else
      g.setColor(disabledColor);

    int r = x+w-1;
    int b = y+h-1;

    switch (type)
    {
      case SIDES_ALL:
        g.drawRect(x, y, w-1, h-1);
        break;

      case SIDES_TOP_BOTTOM:
        g.drawLine(x, y, r, y);
        g.drawLine(x, b, r, b);
        break;

      case SIDES_LEFT_RIGHT:
        g.drawLine(x, y, x, b);
        g.drawLine(r, y, r, b);
        break;

      case SIDES_LEFT:
        g.drawLine(x, y, x, b);
        break;

      case SIDES_RIGHT:
        g.drawLine(r, y, r, b);
        break;

      case SIDES_TOP:
        g.drawLine(x, y, r, y);
        break;

      case SIDES_BOTTOM:
        g.drawLine(x, b, r, b);
        break;
        
      case SIDES_TOP_BOTTOM_RIGHT:
        g.drawLine(x, y, r, y);
        g.drawLine(x, b, r, b);
        g.drawLine(r, y, r, b);
        break;
    }

  }
}
