package roboforge.ui.components.verify;

import roboforge.ui.components.*;
import java.awt.Toolkit;
import javax.swing.*;


public class BNumberVerifier extends InputVerifier
{

  // Class variables...
  // -------------------------------------------------------------

  public static final int INTEGER = 0;
  public static final int FLOAT   = 1;
  public static final int DOUBLE  = 2;


  // Predefined verifiers...
  // -------------------------------------------------------------

  public static BNumberVerifier POSITIVE_INTEGER =
    new BNumberVerifier(INTEGER, 0.0, Integer.MAX_VALUE);

  public static BNumberVerifier POSITIVE_FLOAT =
    new BNumberVerifier(FLOAT, 0.0, Float.MAX_VALUE);

  public static BNumberVerifier POSITIVE_DOUBLE =
    new BNumberVerifier(DOUBLE, 0.0, Double.MAX_VALUE);



  // Member variables...
  // -------------------------------------------------------------

  protected int type;
  protected double min;
  protected double max;


  // Constructors...
  // -------------------------------------------------------------

  public BNumberVerifier(int type, double min, double max)
  {
    this.type = type;
    this.min  = min;
    this.max  = max;
  }


  public boolean verify(JComponent input)
  {
    BTextField field = (BTextField) input;
    String     text  = field.getText();

    double value;

    try
    {
      switch (type)
      {
        case INTEGER:
          value = Integer.parseInt(text);
          break;

        case FLOAT:
          value = Float.parseFloat(text);
          break;

        case DOUBLE:
          value = Double.parseDouble(text);
          break;

        default:
          onFail();
          return false;
      }
    }
    catch (NumberFormatException e)
    {
      onFail();
      return false;
    }

    if ((value >= min) && (value <= max))
      return true;
    else
    {
      onFail();
      return false;
    }
  }

  protected void onFail()
    { Toolkit.getDefaultToolkit().beep(); }

}