package roboforge.ui.fusion;

import roboforge.sound.*;
import roboforge.ui.components.HSBImageIcon;
import roboforge.util.graphics.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;



public class FusionToggleButtonUI extends BasicToggleButtonUI
{

  private final static FusionToggleButtonUI ui
    = new FusionToggleButtonUI();

  private static Color   pressedColor  = Color.white;
  private static Color   disabledColor = Color.white;
  private static Color   rolloverColor = Color.white;
  private static Color   iconColor     = Color.white;
  
  private static float   roundness     = 0.0f;
  private static Insets  insets        = new Insets(0,0,0,0);

  protected static RenderingHints hints = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );

  protected static BSound clickSound;
    
  protected static ButtonSoundListener sounds =
    new ButtonSoundListener();

  
  
  
  public FusionToggleButtonUI()
  {
    super();
  }
  
  public static ComponentUI createUI(JComponent b)
    { return ui; }

  protected void installDefaults(AbstractButton b)
  {
    super.installDefaults(b);
    String pp = "ToggleButton.";

    pressedColor   = UIManager.getColor(pp+ "selectedColor");
    disabledColor  = UIManager.getColor(pp+ "disabledColor");
    rolloverColor  = disabledColor;
    iconColor      = b.getForeground();
    
    roundness      = ((Float) UIManager.get(pp + "roundness")).floatValue();
    insets         = UIManager.getInsets(pp + "insets");
    clickSound     = (BSound) UIManager.get(pp+ "clickSound");

    b.setBorder(UIManager.getBorder(pp + "border"));
  }

  
  protected void installListeners(AbstractButton b)
  {
    super.installListeners(b);
    b.addMouseListener(sounds);
  }

  protected void uninstallListeners(AbstractButton b)
  {
    super.uninstallListeners(b);
    b.removeMouseListener(sounds);
  }

  
  
  public void paint(Graphics g, JComponent c)
  {

    Graphics2D g2D = (Graphics2D) g;

    g2D.addRenderingHints(hints);

    AbstractButton b  = (AbstractButton) c;
    ButtonModel model = b.getModel();
    
    if (b.getIcon() instanceof HSBImageIcon)
    {
      HSBImageIcon icon = (HSBImageIcon) b.getIcon();
      icon.setDisabledColor(disabledColor);
    }
    
    if (model.isArmed() && !model.isPressed())
      paintButtonRollover(g, b);

    super.paint(g, c);
  }


  protected void paintButtonRollover(Graphics g, AbstractButton b)
  {
    Dimension size = b.getSize();
    g.setColor(rolloverColor);

    int x = insets.left;
    int y = insets.top;
    int w = size.width  - x - insets.right;
    int h = size.height - y - insets.bottom;

    int arcsize = Math.round(roundness * Math.min(w, h));

    g.fillRoundRect(x, y, w, h, arcsize, arcsize);
  }

  protected void paintButtonPressed(Graphics g, AbstractButton b)
  {
    Dimension size = b.getSize();
    g.setColor(pressedColor);

    int x = insets.left;
    int y = insets.top;
    int w = size.width  - x - insets.right;
    int h = size.height - y - insets.bottom;

    int arcsize = Math.round(roundness * Math.min(w, h));

    g.fillRoundRect(x, y, w, h, arcsize, arcsize);
  }

  protected void paintText(Graphics g, AbstractButton b, Rectangle textRect, String text)
  {
    ButtonModel model = b.getModel();
    FontMetrics fm = g.getFontMetrics();

    if( model.isEnabled() )
      g.setColor(iconColor);
    else
      g.setColor(disabledColor);

    BasicGraphicsUtils.drawString(
      g, text, model.getMnemonic(), textRect.x, textRect.y + fm.getAscent());
  }

  
  protected static class ButtonSoundListener extends MouseAdapter
  {
    public void mousePressed(MouseEvent e)
    { 
      BSound click = FusionButtonUI.clickSound;
      if (click != null) click.play();        
    }
  }


}
