package roboforge.ui.fusion;

import roboforge.BDebug;
import roboforge.sound.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.tree.*;



public class FusionTreeUI extends BasicTreeUI
{

  
  protected BSound selectSound;
  
  
  public FusionTreeUI()
    { super(); }

  public static ComponentUI createUI(JComponent b)
    { return new FusionTreeUI(); }


  protected void installDefaults()
  {
    super.installDefaults();
    selectSound = (BSound) UIManager.get("Tree.selectSound");
  }
  
  /**
   * Paints the horizontal part of the leg. The reciever should
   * NOT modify <code>clipBounds</code>, or <code>insets</code>.<p>
   * NOTE: <code>parentRow</code> can be -1 if the root is not visible.
   */

  protected void paintHorizontalPartOfLeg
  (
    Graphics g, Rectangle clipBounds,
		Insets insets, Rectangle bounds,
		TreePath path, int row,
		boolean isExpanded, boolean hasBeenExpanded,
    boolean isLeaf
  )
  {
  	int clipLeft = clipBounds.x;
	  int clipRight = clipBounds.x + (clipBounds.width - 1);
  	int clipTop = clipBounds.y;
	  int clipBottom = clipBounds.y + (clipBounds.height - 1);
  	int lineY = bounds.y + bounds.height / 2;
	  // Offset leftX from parents indent.
  	int leftX = bounds.x - totalChildIndent + 8;
	  int nodeX = bounds.x - getHorizontalLegBuffer();

  	if (lineY > clipTop && lineY < clipBottom && nodeX > clipLeft && leftX < clipRight )
    {
	    leftX = Math.max(leftX, clipLeft);
	    nodeX = Math.min(nodeX, clipRight);

	    g.setColor(getLegColor(path));
	    paintHorizontalLine(g, tree, lineY, leftX + 1, nodeX);
  	}
  }


  protected Color getLegColor(TreePath path)
  {
    return getHashColor();

    // TreeNode node = (TreeNode) path.getLastPathComponent();
    // if (node instanceof Decision
  }

  /**
   * Paints the vertical part of the leg. The reciever should
   * NOT modify <code>clipBounds</code>, <code>insets</code>.<p>
   */

  protected void paintVerticalPartOfLeg
  (
    Graphics g, Rectangle clipBounds,
    Insets insets, TreePath path
  )
  {
    int lineX = ((path.getPathCount() - 1 + depthOffset) * totalChildIndent) + 8 + insets.left;
  	int clipLeft = clipBounds.x;
	  int clipRight = clipBounds.x + (clipBounds.width - 1);

  	if (lineX > clipLeft && lineX < clipRight)
    {
	    int clipTop = clipBounds.y;
	    int clipBottom = clipBounds.y + clipBounds.height;
	    Rectangle parentBounds = getPathBounds(tree, path);
	    Rectangle lastChildBounds = getPathBounds(tree, getLastChildPath(path));

	    if(lastChildBounds == null)
    		// This shouldn't happen, but if the model is modified
    		// in another thread it is possible for this to happen.
		    // Swing isn't multithreaded, but I'll add this check in
    		// anyway.
		    return;

	    int top;

	    if (parentBounds == null)
        {	top = Math.max(insets.top + getVerticalLegBuffer(), clipTop); }
	    else
    		top = Math.max(parentBounds.y + parentBounds.height +
			          getVerticalLegBuffer(), clipTop);

	    if (path.getPathCount() == 1 && !isRootVisible())
      {
    		TreeModel model = getModel();

    		if (model != null)
        {
  		    Object root = model.getRoot();

  		    if(model.getChildCount(root) > 0)
          {
      			parentBounds = getPathBounds(tree, path.
			  	  pathByAddingChild(model.getChild(root, 0)));

      			if(parentBounds != null)
			        top = Math.max(insets.top + getVerticalLegBuffer(), parentBounds.y + parentBounds.height / 2);
  		    }
    		}
	    }

	    int bottom = Math.min(lastChildBounds.y + (lastChildBounds.height / 2), clipBottom);

	    g.setColor(getLegColor( path) );
	    paintVerticalLine(g, tree, lineX, top, bottom);
  	}
  }

  
  protected void selectPathForEvent(TreePath path, MouseEvent e)
  {
    super.selectPathForEvent(path, e);
    
    if (selectSound != null)
      selectSound.play();
  }

  
  /*
  protected MouseListener createMouseListener()
    { return new FusionMouseHandler(); }
  
  public class FusionMouseHandler extends MouseHandler
  {
    public void mousePressed(MouseEvent e)
    {
      super.mousePressed(e);
      
      if (selectSound != null)
        selectSound.play();
    }
  }
  */
  
}