package roboforge.ui.layers;

import java.awt.*;
import javax.swing.*;



public class BMultiLayerStack extends BAbstractLayerStack
{
  
  // Class variables...
  // -------------------------------------------------------------
  
  public static final LayerVisitor ACTIVATOR   = new LayerActivator();
  public static final LayerVisitor DEACTIVATOR = new LayerDeactivator();
  public static final LayerVisitor CLOSER      = new LayerCloser();

  
  
  // Constructor...
  // -------------------------------------------------------------
  
  public BMultiLayerStack(JComponent holder)
  {
    super(holder);
  }
  
  
  
  // Interface methods...
  // -------------------------------------------------------------
  
  public void push(BLayer l)
  {
    stack.push(l);
    show(l);
  }
  
  public BLayer peek()
    { return (BLayer) stack.peek(); }

  public BLayer pop()
  { return pop(false); }
    
  public BLayer pop(boolean close)
  {
    BLayer l = (BLayer) stack.pop();
    hide(l, close);

    return l;
  }
      
  public boolean remove(BLayer l)
    { return remove(l, false); }
  
  public boolean remove(BLayer l, boolean close)
  {
    boolean removed =
      stack.remove(l);
    
    if (removed) hide(l, close);
    return removed;
  }
  
  
  public void moveToTop(BLayer l)
  {
    if ((l == null) || (l == peek()))
      return;
    
    boolean removed =
      remove(l);
    
    if (removed)
      push(l);
  }
  
  public boolean isOpen(BLayer l)
    { return stack.contains(l); }
  
  public boolean contains(BLayer l)
    { return stack.contains(l); }
    
  
  public int size()
    { return stack.size(); }
  
  
  
  // Activation support methods...
  // -------------------------------------------------------------

  protected void activateLayers()
    { visitLayers(ACTIVATOR); }
  
  protected void deactivateLayers()
    { visitLayers(DEACTIVATOR); }
  
  protected void closeLayers()
  {
    visitLayers(CLOSER);
    stack.clear();
  }
  
  protected void visitLayers(LayerVisitor visitor)
  {
    BLayer l;

    int n = stack.size();

    for (int i=0; i<n; i++)
    {
      l = (BLayer) stack.get(i);
      visitor.visit(l);
    }
  }
  
  
  protected static abstract class LayerVisitor
    { public abstract void visit(BLayer l); }

  protected static class LayerActivator extends LayerVisitor
    { public void visit(BLayer l) { l.requestActivate(); } }
  
  protected static class LayerDeactivator extends LayerVisitor
    { public void visit(BLayer l) { l.requestDeactivate(); } }

  protected static class LayerCloser extends LayerVisitor
    { public void visit(BLayer l) { l.requestClose(); } }
   
}
