package roboforge.ui.modes;

import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.lang.Math.*;

import roboforge.*;
import roboforge.ui.*;
import roboforge.util.timing.*;

public class BModeArenaBotCam extends BModeArena
{
 //private TransformGroup tg;
  private double rotXZAngle = BPhysics.QUARTER_PI;
  private double updownRot = BPhysics.QUARTER_PI;
  private BArenaCanvas canvas;

  private static double y_factor = .005;
  private static double x_factor = .006;

  private static double TOP_CAP = BPhysics.HALF_PI - (BPhysics.QUARTER_PI /4.0);
  private static double BOTTOM_CAP = 0; // -BPhysics.QUARTER_PI;

  private static double FLOOR = BGlobal.HUMAN_HEIGHT ;

    // working variables
  private Transform3D  workingTran = new Transform3D();
  private Transform3D  mulTran = new Transform3D();
  private Vector3d oldTran  = new Vector3d();
  private Vector3d zeroTranslation  = new Vector3d(0,0,0);
  private BBot selectedBot;

  private static float zoom_factor =  0.7f;
  private float zoom = 12.0f * BGlobal.WORLD_TO_CANVAS_SCALE;
  private float ZOOM_MAX = 20.0f * BGlobal.WORLD_TO_CANVAS_SCALE;
  private float ZOOM_MIN = 3.0f * BGlobal.WORLD_TO_CANVAS_SCALE;

  private static final double fov      = BPhysics.degToRad(75); // 0.5;  // Human Field Of View
  private double oldFov;

  private double arenaSize;
  private Vector3d translation = new Vector3d();

  private static double inset = 0.03;

  public BModeArenaBotCam(TransformGroup tg, BArenaCanvas canvas, double arenaSize)
  {
    super();

    this.canvas = canvas;
    //this.tg = tg;
    this.arenaSize = arenaSize;
    this.arenaSize = this.arenaSize - (this.arenaSize * inset );
  }

  public void onActivate()
  {
    super.onActivate();

    oldFov = canvas.getFov();
    canvas.setFov(fov);
    canvas.setConsoleVisible(true);
  }

  public void onDeactivate()
  {
    super.onDeactivate();
    canvas.setFov(oldFov);
    canvas.setConsoleVisible(true);
  }


  public void mouseDragged(MouseEvent e)
  {
    super.mouseDragged(e);

    if (isMouseActive)
    {
      if (e.isShiftDown() )
        incrementZoom(dy);
      else
        moveCamera(dx, -dy);
    }
  }



  protected static final int KEY_DX = 5;
  protected static final int KEY_DY = 5;


  public void keyUpPressed(KeyEvent e)
  {
    if (e.isShiftDown())
      incrementZoom(-KEY_DY);
    else
      moveCamera(0, -KEY_DY);
  }

  public void keyDownPressed(KeyEvent e)
  {
    if (e.isShiftDown())
      incrementZoom(+KEY_DY);
    else
      moveCamera(0, +KEY_DY);
  }

  public void keyLeftPressed(KeyEvent e)
    { moveCamera(+KEY_DX, 0); }

  public void keyRightPressed(KeyEvent e)
    { moveCamera(-KEY_DX, 0); }




  public void setPosition(double  rotXZAngle )
  {
    setPosition(rotXZAngle,  updownRot );
  }

  public void setPosition(double rotXZAngle, double updownRot)
  {
    this.rotXZAngle = rotXZAngle;
    this.updownRot = updownRot;
    updateCamera(selectedBot);

  }

  public void setElevation(double updownRot)
  {
    this.updownRot = updownRot;
    updateCamera(selectedBot);
  }


  public void setZoom(float zoom)
  {
    // Sets the zoom factor [0..1].
    if (zoom < 0f || zoom > 1f)
      throw new IllegalArgumentException
        ("Illegal zoom factor (should be in [0..1]): "+ zoom);

    zoom = ZOOM_MIN + zoom * (ZOOM_MAX - ZOOM_MIN);
    updateCamera(selectedBot);
  }

  public void moveCamera(double dx, double dy)
  {
    rotXZAngle += -dx * x_factor;

    updownRot  = updownRot - dy * y_factor ;
    updownRot = java.lang.Math.min(updownRot, TOP_CAP );
    updownRot = java.lang.Math.max(updownRot, getMinimumAllowableElevation()); // BOTTOM_CAP );

    updateCamera(selectedBot) ;
  }

  protected void incrementZoom(int delta)
  {
    zoom += delta * zoom_factor;

    if (zoom > ZOOM_MAX)
      zoom = ZOOM_MAX;
    else if (zoom < ZOOM_MIN)
      zoom = ZOOM_MIN;

    updateCamera(selectedBot);
  }


  public void animationStep(double t, double dt)
  // do a slow automatic arena animation
  {
  	double delta = java.lang.Math.min(dt * 100.0, 2.0);
    moveCamera(delta, 0);
  }


  /*public void animate()
  {
    moveCamera(1, 0);
  }*/


  public void setBot(BBot theBot)
    { selectedBot = theBot; }

  public void updateCamera()
    { updateCamera (selectedBot) ;}


  private static Transform3D componentTrans   = new Transform3D();
  private static Transform3D cTrans           = new Transform3D();
  private static Vector3f    cVector          = new Vector3f();
  private static Vector3f    bVector          = new Vector3f();
  private static Point3d     lookAt_Point     = new Point3d();
  private static Vector3d    lookAt_Vec       = new Vector3d();


  private synchronized void updateCamera(BBot theBot)
  {
    if (theBot != null)
      if (theBot.getLiveNode(true) !=  null)
      {

        // Get bot floor position.
        theBot.getLiveNode(false).getLocalToVworld(componentTrans);

        // Get camera elevation.
        double elevation = updownRot;

        // Ensure elevation angle is clipped...
        if (elevation < 0f)
          elevation = getMinimumAllowableElevation();

        /*
        {
          // Get bot position.
          componentTrans.get(bVector);

          // Get bot height, in world units.
          float height = bVector.y * BGlobal.CANVAS_TO_WORLD_SCALE;

          // Calculate elevation angle.
          elevation = -(float) Math.asin(height / zoom);
        }
        */


        // work out camera position
        cVector.x = 0;
        cVector.y = 0;
        cVector.z = -zoom;

        // set Sphere Position
        cTrans.rotX(elevation);
        cTrans.transform(cVector);

        cTrans.rotY(rotXZAngle);
        cTrans.transform(cVector);

        cTrans.set(cVector );
        componentTrans.mul(cTrans);

        // this is the resultant position, save to set translation at the end.
        componentTrans.get(cVector);



        // cap position to the edge of the ring
        double distFromCentre = cVector.length();

        if (distFromCentre * BGlobal.CANVAS_TO_WORLD_SCALE > arenaSize)
        {
          cVector.normalize();
          cVector.scale((float)arenaSize * BGlobal.WORLD_TO_CANVAS_SCALE);
        }
        //componentTrans.setTranslation (cVector);


        // cap cVector at y = 5
        if (cVector.y < FLOOR)
          cVector.y  = (float) FLOOR;


        // getBot position
        theBot.getLiveNode(true).getLocalToVworld(cTrans);
        cTrans.get(lookAt_Vec);
        lookAt_Point.set(lookAt_Vec);

        canvas.setCameraPos(cVector.x, cVector.y, cVector.z, true, lookAt_Point);



      }
  }


  protected float getMinimumAllowableElevation()
  {
    // Get bot floor position.
    selectedBot.getLiveNode(false).getLocalToVworld(componentTrans);

    // Get bot position.
    componentTrans.get(bVector);

    // Get bot height, in world units.
    float height = bVector.y * BGlobal.CANVAS_TO_WORLD_SCALE;

    // Return minimum elevation angle.
    return -(float) Math.asin(height / zoom);
  }


}


