package roboforge.ui.modes;

import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.lang.Math.*;

import roboforge.*;
import roboforge.ui.*;
import roboforge.util.timing.*;

public final class BModeArenaSphereCam extends BModeFOVZoom
{

  private TransformGroup tg;
  private float arenaSize;
  private float minY, maxY;


  private double azimuth = 0;
  private double zenith = 0;
  public static final double ZENITH_MAX = BPhysics.HALF_PI  - ( BPhysics.QUARTER_PI / 4.0); //- ( BPhysics.QUARTER_PI / 2.0) ;
  public static final double ZENITH_MIN = 0;


  // working variables
  private final Transform3D  workingTran = new Transform3D();
  private final Transform3D tranXZ = new Transform3D();

  private final double  x_factor = 0.006;
  private final double  y_factor = 0.006;

  private static float inset = 0.03f;



  public BModeArenaSphereCam(BArenaCanvas canvas, TransformGroup tg, float arenaSize, double minY)
  {
    super(canvas);
    this.tg = tg;
    this.arenaSize = arenaSize / 2f   ;
    this.arenaSize = this.arenaSize  - (this.arenaSize * inset );
    this.arenaSize  = this.arenaSize * BGlobal.WORLD_TO_CANVAS_SCALE;

    this.minY = (float)minY;

  }

  public void onActivate()
  {
    super.onActivate();
    setPosition();
  }



 private static Transform3D rotXTran = new Transform3D();
 private static Vector3f v = new  Vector3f();

 private synchronized void setPosition()
 {
      // BDebug.print ("BSphereCam.setPosition() z: "+ zenith+ ", a: "+ azimuth);

     // BDebug.seedTimer();
      v.set (0,minY,arenaSize );
    // x axis transform
      rotXTran.rotX(-zenith);
      rotXTran.transform(v);

      // y axis tranform
      tranXZ.rotY(azimuth);
      tranXZ.transform(v);

      canvas.setCameraPos(v.x,v.y,v.z, true, null);

 }


 public void animationStep(double t, double dt)
  // do a slow automatic arena animation
  {
  	double delta = Math.min(dt * 50, 2);
    moveCamera(delta, 0);
  }

  public void setAzimuth(double a)
  {
  	updateAzimuth(a);
    setPosition();
  }

 protected void updateAzimuth(double a)
 {
 	// Constrain azimuth to [0..2pi].
   if (a < 0f)
     azimuth = (a % BPhysics.TWO_PI) + BPhysics.TWO_PI;
   else if (a > BPhysics.TWO_PI)
     azimuth = a % BPhysics.TWO_PI;
   else
   	azimuth = a;
 }


  public void setZenith(double z)
  {
  	updateZenith(z);
    setPosition();
  }

 protected void updateZenith(double z)
 {
		// Constrain zenith.
    if (z > ZENITH_MAX)
    	zenith = ZENITH_MAX;
    else if (z < ZENITH_MIN)
    	zenith = ZENITH_MIN;
    else
    	zenith = z;
 }


 public void setParameters(double z, double a)
 {
   // Update zenith.
   updateZenith(z);

   // Update azimuth.
   updateAzimuth(a);

   // Update 3D view...
   setPosition();
 }


 public void moveCamera(double dx, double dy )
 {
    // Update zenith.
    updateZenith(zenith + dy * y_factor);

    // Update azimuth.
    updateAzimuth(azimuth + -dx * x_factor);

    // Update 3D view...
    setPosition();
 }


 public void mouseDragged(MouseEvent e)
  {
    super.mouseDragged(e);

    if (isMouseActive && !e.isShiftDown())
      moveCamera(dx, dy);
  }


  protected static final int KEY_DX = 5;
  protected static final int KEY_DY = 5;

  public void keyUpPressed(KeyEvent e)
  {
    super.keyUpPressed(e);
    if (!e.isShiftDown())
      moveCamera(0, +KEY_DY);
  }

  public void keyDownPressed(KeyEvent e)
  {
    super.keyDownPressed(e);
    if (!e.isShiftDown())
      moveCamera(0, -KEY_DY);
  }

  public void keyLeftPressed(KeyEvent e)
    { moveCamera(+KEY_DX, 0); }

  public void keyRightPressed(KeyEvent e)
    { moveCamera(-KEY_DX, 0); }






}




