package roboforge.ui.modes;

import roboforge.BDebug;
import roboforge.ui.*;
import roboforge.ui.events.*;

import java.util.Vector;
import java.util.Hashtable;

public class BModeManager
  implements BActivateable
{

  // Internal variables...
  // -------------------------------------------------------------

  // Support class for handling activation listeners...
  protected BActivationSupport activationSupport
    = new BActivationSupport(this);


  // The current mode...
  protected BMode currentMode = null;

  protected Vector    modes;
  protected Hashtable names;

  protected BModeButtonGroup group;


  // Constructor...
  // -------------------------------------------------------------

  public BModeManager()
  {
    modes = new Vector();
    names = new Hashtable();
  }

  // Accessor methods...
  // -------------------------------------------------------------

  public void addMode(BMode m)
  {
    modes.add(m);
    
    if (m.name != null)
      names.put(m.name, m);
  }

  public void removeMode(BMode m)
  {
    modes.remove(m);
    if (m.name != null)
      names.remove(m.name);
  }

  public BMode getMode(String name)
    { return (BMode) names.get(name); }

  public BMode getMode(int i)
    { return (BMode) modes.get(i); }

  public int getModeCount()
    { return modes.size(); }


  /*
  public void setSelectedMode(String name)
  {
    BMode m = getMode(name);
    if (m != null) setSelectedMode(m);
  }
  */

  public void setActiveButtonGroup(BModeButtonGroup g)
  {
    if (group != g)
    {
      if (group != null)
        group.deselect();
        
      this.group = g;
    }
  }

  public void setSelectedMode(BMode m)
  {
    if (currentMode != m)
    {
      if (currentMode != null)
        currentMode.requestDeactivate();

      currentMode = m;

      if (isActive() && (currentMode != null))
        currentMode.requestActivate();
    }
  }

  public void setSelectedMode(String name)
  {
    BMode m = (BMode) names.get(name);
    setSelectedMode(m);
  }


  public void setSelectedModeIndex(int i)
    { setSelectedMode(getMode(i)); }


  public BMode getSelectedMode()
    { return currentMode; }

  public int getSelectedModeIndex()
    { return modes.indexOf(currentMode); }



  // Activation methods...
  // -------------------------------------------------------------

  public boolean isActive()
    { return activationSupport.isActive(); }

  public boolean isClosed()
    { return activationSupport.isClosed(); }


  public void requestActivate()
    { activationSupport.requestActivate(); }

  public void requestDeactivate()
    { activationSupport.requestDeactivate(); }

  public void requestClose()
    { activationSupport.requestClose(); }


  public void onActivate()
  {
    if (currentMode != null)
      currentMode.requestActivate();

    activationSupport.fireActivate();
  }

  public void onDeactivate()
  {
    if (currentMode != null)
      currentMode.requestDeactivate();

    activationSupport.fireDeactivate();
  }

  public void onClose()
  {
    BDebug.print ("BModeManage.onClose called " );
    BMode mode;
    int n = modes.size();

    for (int i=0; i<n; i++)
    {
      mode = (BMode) modes.get(i);
      mode.requestClose();
    }

    modes.clear();
    names.clear();
    activationSupport.fireClose();

    currentMode = null;
    modes = null;
    names = null;
    activationSupport = null;


  }

  
  public void addActivationListener(BActivationListener l)
    { activationSupport.addActivationListener(l); }

  public void removeActivationListener(BActivationListener l)
    { activationSupport.addActivationListener(l); }

}