package roboforge.ui.net;

import roboforge.BDebug;
import roboforge.BGlobal;

import roboforge.math.BMath;
import roboforge.net.*;
import roboforge.net.client.*;
import roboforge.net.data.*;

import roboforge.ui.*;
import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.events.*;

import roboforge.util.graphics.*;
import roboforge.util.io.*;
import roboforge.util.data.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import roboforge.util.processors.*;


public class BAccountPanel extends BSystemPanel
  implements BDatabase, BDataStore, ActionListener
{


  // Class variables...
  // -------------------------------------------------------------

  protected static final BAccountData EMPTY_DATA = new BAccountData(0);

  
  // Internal variables...
  // -------------------------------------------------------------

  protected BTournamentScreen   tournaments;
  protected BClientStack        server;
  protected boolean             isRefreshRequired;


  protected BAccountData accountData;
    // Last known account information data...

  

  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "left.width",                 new Integer(350),
    "right.width",                new Integer(320),
    "buttons.height",             new Integer(30),
    
    "loginTitle.icon",            BIconCache.getHSB("account_login_info.gif"),
    "loginTitle.foreground",      colors.getMidShade(),
    "loginTitle.height",          new Integer(30),

    "generalTitle.icon",          BIconCache.getHSB("account_general_info.gif"),
    "generalTitle.foreground",    colors.getMidShade(),
    "generalTitle.height",        new Integer(30),

    "gameTitle.icon",             BIconCache.getHSB("account_game_info.gif"),
    "gameTitle.foreground",       colors.getMidShade(),
    "gameTitle.height",           new Integer(30),

    "balanceTitle.icon",          BIconCache.getHSB("account_balance.gif"),
    "balanceTitle.foreground",    colors.getMidShade(),
    "balanceTitle.height",        new Integer(30),
    
    "change.size",                new Dimension(70, 20),
    //"change.font",                new Font("DIALOG", Font.PLAIN, 9),

    "statsButton.text",           "player stats...",
    "statsButton.size",           new Dimension(100,20),

    "balanceButton.text",         "details...",
    "balanceButton.size",          new Dimension(110,20)

  });



  // Visual elements...
  // -------------------------------------------------------------

  protected BLabel          loginTitle    = new BLabel(res, "loginTitle");
  protected BTextField      login         = new BTextField();
  protected BPasswordField  password      = new BPasswordField();
  protected BPasswordField  confirm       = new BPasswordField();
  protected BButton         change        = new BButton(res, "change");
  
  protected BLabel          generalTitle  = new BLabel(res, "generalTitle");
  protected BTextField      email         = new BTextField();
  protected BTextField      name          = new BTextField();
  protected BTextField      address1      = new BTextField();
  protected BTextField      address2      = new BTextField();
  protected BTextField      city          = new BTextField();
  protected BTextField      state         = new BTextField();
  protected BComboBox       country       = new BComboBox();
  protected BTextField      zipcode       = new BTextField();
  
  protected BLabel          gameTitle     = new BLabel(res, "gameTitle");
  protected BTextField      designer      = new BTextField();
  protected BTextArea       blurb         = new BTextArea();

  protected BLabel          balanceTitle     = new BLabel(res, "balanceTitle");

  protected BButton         submit        = new BButton();
  protected BButton         reset         = new BButton();
  protected BButton         statsButton   = new BButton(res, "statsButton");

  protected BButton         balanceButton = new BButton(res, "balanceButton");
  protected BTextField      entryField    = new BTextField();
  protected BTextField      winningsField = new BTextField();

  protected Border labelBorder =
      new EmptyBorder(0,0,0,10);

  // Actions...
  // -------------------------------------------------------------

  protected RefreshAction   refreshAction   = new RefreshAction();
  protected ChangeAction    changeAction    = new ChangeAction();
  protected SubmitAction    submitAction    = new SubmitAction();
  protected ResetAction     resetAction     = new ResetAction();

  protected BPlayerRankingDialog rankingDialog;


  // Constructor.
  // -------------------------------------------------------------

  public BAccountPanel(BTournamentScreen parent)
  {
    super(parent, "account");
    

    this.server             = parent.getServer();
    this.tournaments        = parent;
    this.isRefreshRequired  = true;

    layoutAccountChildren();
    configAccountChildren();
  }


  // Initialization helper methods...
  // -------------------------------------------------------------

  protected void layoutAccountChildren()
  {
    BBoxPanel left    = new BBoxPanel(res, "left",  BBoxPanel.Y_AXIS);
    BBoxPanel right   = new BBoxPanel(res, "right", BBoxPanel.Y_AXIS);
    BBoxPanel buttons = new BBoxPanel(res, "buttons", BBoxPanel.X_AXIS);

    setBorder(new EmptyBorder(30, 45, 10, 45));
    addVGlued(left);
    addBigMargin();
    addVGlued(right);
    
    setHGlue(loginTitle);
    setHGlue(gameTitle);
    setHGlue(balanceTitle);

    left.addLinear(loginTitle, change);
    left.add(new LoginInfoPanel());
    left.addHGlued(generalTitle);
    left.add(new GeneralInfoPanel());
    left.addMargin();
    left.addLinear(balanceTitle, balanceButton);
    left.add(new BalanceInfoPanel() );

    left.addGlue();

    right.addLinear(gameTitle, statsButton);
    right.addMargin();
    right.add(new GameInfoPanel());
    right.addMargin();
    right.addTinyMargin();
    right.add(buttons);
    
    buttons.addHGlue();
    buttons.add(submit);
    buttons.addSmallMargin();
    buttons.add(reset);
    
  }
  
  protected void configAccountChildren()
  {
    login.setEnabled(false);
    password.setEnabled(false);
    confirm.setEnabled(false);
    
    change.setAction(changeAction);
    submit.setAction(submitAction);
    reset.setAction(resetAction);
    
    DefaultComboBoxModel countries =
      new DefaultComboBoxModel(BGlobal.COUNTRIES);
    
    countries.insertElementAt("", 0);
    country.setModel(countries);

    statsButton.addActionListener(this);
    balanceButton.addActionListener(new OpenBalanceActionListener() );
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource() == statsButton)
    {
      BProcessor p = new BProcessor()
      {
        public void process()
        { showPlayerStats(); }
      };
      p.start();
    }
  }

  // display the player statistics dialog on the screen
  protected void showPlayerStats()
  {
    if(accountData != null)
    {
      long accountID = accountData.getID();
      String creatorName = accountData.designerName;

      rankingDialog = new BPlayerRankingDialog(accountID, creatorName, server);
      BApplication.showDialog(rankingDialog);
    }
  }
  
  // Panel Layout classes...
  // -------------------------------------------------------------
  
  protected class BGridBagPanel extends BPanel
  {
    protected GridBagLayout       gridbag;
    protected GridBagConstraints  constraints;
    
    public BGridBagPanel()
    {
      super();
      gridbag = new GridBagLayout();
      constraints = new GridBagConstraints();
      setLayout(gridbag);
    }
  }
  

  protected class LoginInfoPanel extends BGridBagPanel
  {
    protected Border labelBorder =
      new EmptyBorder(0,0,0,10);
    
    public LoginInfoPanel()
    { 
      super(); 
      
      constraints.fill   = GridBagConstraints.HORIZONTAL;
      constraints.insets = new Insets(0,0,10,0);
      
      BLabel label = new BLabel("Login ID:");
      label.setHorizontalAlignment(BLabel.RIGHT);
      label.setBorder(labelBorder);
      
      constraints.weightx = 0.3;
      constraints.gridx = 0;
      gridbag.setConstraints(label, constraints);
      add(label);

      constraints.weightx   = 0.7;
      constraints.gridx     = 1;
      constraints.gridwidth = 3;

      gridbag.setConstraints(login, constraints);
      add(login);
      
      constraints.gridy     = 1;

      label = new BLabel("Password:");
      label.setHorizontalAlignment(BLabel.RIGHT);
      label.setBorder(labelBorder);
      
      constraints.weightx   = 0.3;
      constraints.gridx     = 0;
      constraints.gridwidth = 1;
      
      gridbag.setConstraints(label, constraints);
      add(label);

      constraints.weightx   = 0.3;
      constraints.gridx     = 1;

      gridbag.setConstraints(password, constraints);
      add(password);
      
      label = new BLabel("Confirm:");
      label.setHorizontalAlignment(BLabel.RIGHT);
      label.setBorder(labelBorder);

      constraints.weightx   = 0.1;
      constraints.gridx     = 2;

      gridbag.setConstraints(label, constraints);
      add(label);

      constraints.weightx   = 0.3;
      constraints.gridx     = 3;

      gridbag.setConstraints(confirm, constraints);
      add(confirm);

    }
  }
  
  
  protected class GeneralInfoPanel extends BGridBagPanel
  {
    
    public GeneralInfoPanel()
    {
      super();
      
      constraints.fill   = GridBagConstraints.HORIZONTAL;
      constraints.insets = new Insets(0,0,10,0);

      addField(0, "E-mail address:", email);
      addField(1, "Full name:", name);
      //addField(2, "Address:", address1);
      //addField(3, "", address2);
      //addField(4, "City:", city);
      //addField(5, "State / Province:", state);
      //addField(6, "Country:", country);
      //addField(7, "Zip / postal code:", zipcode);
    }

      
    protected Border labelBorder =
      new EmptyBorder(0,0,0,10);
    
    public void addField(int row, String name, Component c)
    {
      constraints.gridy = row;

      BLabel label = new BLabel(name);
      label.setHorizontalAlignment(BLabel.RIGHT);
      label.setBorder(labelBorder);
      
      constraints.weightx = 0.3;
      constraints.gridx = 0;
      gridbag.setConstraints(label, constraints);
      add(label);
      
      constraints.weightx = 0.7;
      constraints.gridx = 1;
      gridbag.setConstraints(c, constraints);
      add(c);
    }
  }
  /*
  public void loginShow()
  {
    setPanelButtonsEnabled(false);
    setSystemPanel(loginPanel);
  }


  public void loginHide()
  {
    setPanelButtonsEnabled(true);
    setSystemPanel(enterPanel);
    enterButton.setSelected(true);
  }*/


  protected class BalanceInfoPanel extends BBoxPanel
  {
     protected BUIResources res = new BUIResources( new Object[]
    {
      "entry.width",     new Integer(80),
      "entry.text",      "Entry balance:",
      "entry.h_align",   BLabel.ALIGN_RIGHT,
      "entry.border",    labelBorder,

      "winning.width",     new Integer(80),
      "winning.text",      "winnings:",
      "winning.h_align",   BLabel.ALIGN_RIGHT,
      "winning.border",    labelBorder,


    });

     public BalanceInfoPanel()
    {
      super(BBoxPanel.Y_AXIS);
      setBorder(new EmptyBorder(5,0,50,0));

      addSmallMargin();
      addLinear(new BLabel(this.res, "entry"), entryField);
      addSmallMargin();
      addLinear(new BLabel(this.res, "winning"), winningsField);

      entryField.setEnabled(false);
      winningsField.setEnabled(false);

    }
  }

  protected class GameInfoPanel extends BBoxPanel
  {


    protected BUIResources res = new BUIResources( new Object[]
    {
      "nick.width",     new Integer(80),
      "nick.text",      "Nick name:",
      "nick.h_align",   BLabel.ALIGN_RIGHT,
      "nick.border",    labelBorder,

      "desc.width",     new Integer(80),
      "desc.text",      "Description:",
      "desc.h_align",   BLabel.ALIGN_RIGHT,
      "desc.v_align",   BLabel.ALIGN_TOP,
      "desc.border",    labelBorder,
    });

    protected BScrollPane scroller = new BScrollPane(blurb);

    
    public GameInfoPanel()
    {
      super(BBoxPanel.Y_AXIS);
      //setBorder(new EmptyBorder(5,0,50,0));

      addLinear(new BLabel(this.res, "nick"), designer);
      addSmallMargin();

      BLabel desc = new BLabel(this.res, "desc");
      
      setVGlue(desc);
      setVGlue(scroller);

      addLinear(desc, scroller);

      blurb.setBorder(new EmptyBorder(5,5,5,5));
      blurb.setLineWrap(true);
      blurb.setWrapStyleWord(true);
    }
  }

    

  
  

  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();

    email.requestFocus();

    if (isRefreshRequired)
    {
      refreshAction.perform();

      // YLY 13 Jun
      //Always Refresh
      //isRefreshRequired = false;
    }
  }


  
  
  // Data methods...
  // -------------------------------------------------------------
  
  public BDataStore get(int type)
  {
    if (type == BDataTypes.ACCOUNT)
      return this;
    
    return null;
  }
  
  public void put(BDataAtom datum)
  { 
    this.accountData = (BAccountData) datum;
    setPanelFromData(accountData); 
  }

  protected void setPanelFromData(BAccountData data)
  {
    if (data == null) 
      return;
    
    login.setText(data.loginID);
    password.setText(data.password);
    confirm.setText(data.password);
    
    name.setText(data.name);
    email.setText(data.email);
    address1.setText(data.address1);
    address2.setText(data.address2);
    city.setText(data.city);
    state.setText(data.state);
    
    country.setSelectedItem(data.country);
    zipcode.setText(data.postCode);
    
    designer.setText(data.designerName);
    blurb.setText(data.blurb);

    winningsField.setText("$" + BMath.format(data.winningsBalance, 2));
    entryField.setText("$" + BMath.format(data.currentBalance, 2));


  }

  
  protected BAccountData getDataFromPanel()
  {
    BAccountData data   = new BAccountData(accountData.getID());
    
    // TODO: Imput verification, error handling!
    
    data.loginID        = login.getText();
    
    String p            = new String(password.getPassword());
    String c            = new String(confirm.getPassword());
    
    if (!p.equals(c))
      return null;
    
    if (!p.equals(server.getPassword()))
      data.password = p;
    
    data.name           = name.getText();
    data.email          = email.getText();
    data.address1       = address1.getText();
    data.address2       = address2.getText();
    data.city           = city.getText();
    data.state          = state.getText();
    data.postCode       = zipcode.getText();
    data.country        = (String) country.getSelectedItem();
    
    data.designerName   = designer.getText();
    data.blurb          = blurb.getText();

    if(data.designerName.length() > 15)
    {
      BProcessor pr = new BProcessor()
      {
        public void process()
        { showNameError(); }
      };
      pr.start();
      return null;
    }

    return data;
  }


  public void showNameError()
  {
    BPromptDialog nameError;
    nameError = BPromptDialog.createOKDialog("name error", "The nickname you entered is too long.  Your nickname can be no longer than 15 characters.");
    BApplication.showDialog(nameError);
  }

  // UI Listeners...
  // -------------------------------------------------------------
  protected class ChangeAction extends BAction
  {
    public ChangeAction()
      { super("change..."); }
    
    public void actionPerformed(ActionEvent e)
    {
      password.setEnabled(true);
      confirm.setEnabled(true);
      password.setText("");
      confirm.setText("");
      password.requestFocus();
      
      setEnabled(false);
    }
    
    public void reset()
    {
      password.setEnabled(false);
      confirm.setEnabled(false);
      setEnabled(true);
    }
  }

  
  protected class SubmitAction extends BThreadedAction
  {
    protected BAccountData data;
    protected BMessage response;

    public SubmitAction()
      { super("Submit"); }

    public void swingPrepAction(ActionEvent e)
    {
      parent.setStatus("Submitting new account information...");
      parent.setBusyWorking(true);
    }

    public void threadedAction(ActionEvent e)
    {
      try
      {
        response = null;
        data = getDataFromPanel();
        
        if (data == null)
          return;

        BTransmission t = new BTransmission(
          new BRequest(server, "CAI", data));

        response = server.request(t);
      }
      catch (BNetException ex)
      { 
        parent.setStatus("ERROR: "+ ex.getMessage());
        response = null;
      }
    }

    public void swingCleanupAction(ActionEvent e)
    {
      if (response != null)
      {
        if (response.isAcknowledgement())
        {
          parent.setStatus("Account information changed.");
          accountData = data;

          // Update password, if necessary.
          if (data.password != null)
            if (!data.password.equals(""))
              server.setPassword(data.password);
        }
        else
          parent.setStatus("ERROR: "+ response.getData());
      }

      parent.setBusyWorking(false);
    }
  }
  

  protected class ResetAction extends BAction
  {
    public ResetAction()
      { super("Reset"); }
    
    public void actionPerformed(ActionEvent e)
    { 
      setPanelFromData(accountData != null 
        ? accountData : EMPTY_DATA);

      changeAction.reset();
    }
  }
  
  protected class OpenBalanceActionListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      ((BTournamentScreen)parent).showBalancePanel();
    }
  }
  
  protected class RefreshAction extends BThreadedAction
  {
    protected BMessage response;

    public RefreshAction()
      { super("Refresh"); }
      
    public RefreshAction(BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
    {
      parent.setStatus("Refreshing account information...");
      parent.setBusyWorking(true);
    }

    public void threadedAction(ActionEvent e)
    {
      try
      {
        BTransmission t = new BTransmission(
          new BRequest(server, "GAI"));

        response = server.request(t);
      }
      catch (BNetException ex)
      { 
        parent.setStatus("ERROR: "+ ex.getMessage());
        response = null;
      }
    }

    public void swingCleanupAction(ActionEvent e)
    {
      if (response != null)
      {
        if (response.isAcknowledgement())
        {
          BData data = response.getData();

          if (data != null)
            data.store(BAccountPanel.this);
          if(parent != null)
            parent.setStatus("Account information refreshed.");
        }
        else
        {
          if(parent != null)
            parent.setStatus("ERROR: "+ response.getData());
        }
      }

      if(parent != null)
        parent.setBusyWorking(false);
    }
  }
}
