package roboforge.ui.net;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;
import java.io.*;

import javax.media.j3d.Behavior;
import javax.media.j3d.WakeupOnElapsedFrames;
import javax.vecmath.*;
import javax.media.j3d.*;
import java.util.*;
import javax.swing.event.*;

import java.security.*;
import javax.crypto.Cipher;
import iaik.utils.CryptoUtils;
import iaik.security.provider.IAIK;

import roboforge.*;
import roboforge.net.*;
import roboforge.net.data.*;
import roboforge.util.io.*;
import roboforge.net.client.*;

import roboforge.sound.*;
import roboforge.util.data.*;
import roboforge.util.processors.*;
import roboforge.ui.BSystemLogger;
import roboforge.ui.*;
import roboforge.ui.components.*;


public class BChallengeFight implements ActionListener
{
  private boolean invokedStandalone = false;
  private javax.swing.Timer t;
  private BArenaCanvas arena3dCanvas;
  private String filePath;
  private BArena arena;
  private BSimulation simulation;
  private BSimulationBehaviour simRunner;
  private BBot bot1;
  private BBot bot2;

  private BRoomControl roomControl;

  private String tournamentName;
  private long fightID;
  private long challengerTEID;
  private long challengedTEID;

  private boolean battleRunning = false;
  private boolean simToSendUp = false;

  //private String name;
  //private long fightClientID;
  //private String password;
  private PrivateKey privateKey;

  private BClientStack server;

  // Keep those multiple threads in order
  boolean busy = false;
  boolean userStopSignaled = false;

  // constructor
  public BChallengeFight(BRoomControl roomControl, String tournamentName, long fightID, long challengerTEID,
                         long challengedTEID,
                         BBot bot1, BBot bot2, String arenaFileName,
                         float fightDuration, BArenaCanvas arena3dCanvas,
                         String userName, String password)
  {
    try
    {
      this.roomControl = roomControl;
      this.tournamentName = tournamentName;
      this.fightID = fightID;
      this.challengerTEID = challengerTEID;
      this.challengedTEID = challengedTEID;

      this.bot1 = bot1;
      this.bot2 = bot2;
      bot1.removeThumbnails();
      bot2.removeThumbnails();

      this.arena3dCanvas = arena3dCanvas;

      server = new BClientStack();
      BServerTarget serverTarget = new BServerTarget(BGlobal.URLSFARMFEEDER);
      server.setServerTarget(serverTarget);

      server.login(userName, password);


      jbInit();

      BDebug.print("BChallengeFight - fight about to run");

      battleRunning = true;
      runFight(this.bot1, this.bot2, arenaFileName, fightDuration);
    }
    catch(Exception e)
    {
      BDebug.print("Error public BChallengeFight(): " + e.getMessage());
     // e.printStackTrace();

      BProcessor p = new BProcessor()
      {
        public void process()
        {
          showFarmServerError();
        }
      };
      p.start();
    }
  }

  public void showFarmServerError()
  {
    BPromptDialog farmError;
    farmError = BPromptDialog.createOKDialog("battle error", "There was an error logging in to the battle server.  Please try again later.");
    BApplication.showDialog(farmError);
    roomControl.stopTheFight(false);
  }

  private void jbInit() throws Exception
  {

    // set up and start the timer
    t = new javax.swing.Timer(10000,this);
    t.start();

    arena3dCanvas.setMode(BGlobal.CANVAS_MODE_ARENA_AUTOCAM);
    arena3dCanvas.setAutoCam(bot1, bot2);
    arena3dCanvas.updateCamera();

    // Set the simulation running behaviour
    simRunner = new BSimulationBehaviour();
    simRunner.addChangeListener( new SimChangeListner() );

    arena3dCanvas.addBehavior(simRunner);
  }


  class SimChangeListner implements ChangeListener
  {

       public void stateChanged(ChangeEvent e)
       {
          // is running, auto cam, else send the fight stuff up to the server
          if ( simRunner.isRunningSimulation() )
            arena3dCanvas.updateCamera();
          else
          {

            simToSendUp = !userStopSignaled;
            battleRunning = false;
            t.stop();

            if (simToSendUp )
              sendBattle();

            cleanse();

            //arena3dCanvas.clear();
           }

       }


  }



  public void cleanse ()
  {
        BDebug.print("BChallengeFight.cleanse called ");

        t.stop();
        t = null;

        arena3dCanvas.clear();

        // Clean Up
        bot1.cleanse();
        bot2.cleanse();

        arena.cleanse();

        simulation.getBattleRecorder().cleanse();
        roomControl.cleanAfterFight();
        arena3dCanvas.clearBehaviorTable();

        //arena3dCanvas.removeBehavior(simRunner);

        simulation.cleanse();
        simulation = null;
        bot1 = null;
        bot2 = null;
        arena = null;
        simulation = null;

        arena3dCanvas = null;
        simRunner = null;
        roomControl = null;

        server = null;
        BGlobal.clearAllHardCaches();
        System.gc();

  }

 /* public void finalize()
  {
    //super.finalize();
    BDebug.print ("XXXXXXXXXXXXXXXXXXXXXX BChallengeFight.finalize called ");

  }  */



  public void sendBattle ()
  {

    BMessage response = null;
    BRequest r = null;

    try
    {

        simToSendUp = false;

        // There is a completed simulation to send up
        roomControl.disableCancelButton();


        BIdentData identData = new BIdentData(fightID);
         // FRS - Set Fight Over
        r = new BRequest(server, "SFO");

        // Prepare the request Data...
        r.setData(identData);

        // Get the response back from the server...
        BTransmission t = new BTransmission(r);
        response = server.request(t);

        if (response.isAcknowledgement())
        {
          BDebug.print("ACK - BChallengeFight - SFO");
        }
        else
        {
          BDebug.print("NAK - BChallengeFight - SFO ::" + response);
        }


        BDebug.print("BChallengeFight - fight ended - sending sim");

        // Create the SFightResult
        BBattleRecorder battleRecording  = simulation.getBattleRecorder();
        BStats bStats1 = battleRecording.getStats(bot1);
        BStats bStats2 = battleRecording.getStats(bot2);
        battleRecording.setTournamentName(tournamentName);
        battleRecording.setRoundName("Round: 1");
        battleRecording.convertToSaveFormat();

        battleRecording.saveToFile( BGlobal.BATTLERECORDINGCACHEPATH + fightID, null, bot2);
        // nullify stats for sending

        bStats2 = new BStats();
        BDebug.print("BChallengeFight - battle recording done");

        long winningBot;

        if (battleRecording.getVictoriousBot() == null)
        {
          // It was a draw
          winningBot = 0;
        }
        else if (battleRecording.getVictoriousBot() == bot1)
        {
          // Bot 1 Won
          winningBot = challengerTEID;
        }
        else
        {
          // BOt 2 Won
          winningBot = challengedTEID;
        }

        BDebug.print("BChallengeFight - winning bot set");

        BDebug.seedTimer();

        BFightResultData fightResultData = new BFightResultData(fightID);
        fightResultData.battleRecordingBytes = SServerAccess.gzipDeflateArray(SServerAccess.objectToByteArray(battleRecording));

        BDebug.printTime("BChallengeFight zip 1 ");
        BDebug.seedTimer();

        fightResultData.bot1StatsBytes = SServerAccess.gzipDeflateArray(bStats1.toByteArray());
        fightResultData.bot2StatsBytes = SServerAccess.gzipDeflateArray(bStats2.toByteArray());

        BDebug.printTime("BChallengeFight zip stats ");
        BDebug.seedTimer();

        fightResultData.winner = winningBot;
        fightResultData.tournamentEntryID1 = challengerTEID;
        fightResultData.tournamentEntryID2 = challengedTEID;
        fightResultData.winDescription = battleRecording.getWinString();

        BDebug.printTime("BChallengeFight zip end ");

        BDebug.print("BChallengeFight - fight result data set");

        BDebug.print("BChallengeFight - cleaned up");

        // FRS - Fight ReSult
        r = new BRequest(server, "FRS");

        // Prepare the request Data...
        r.setData(fightResultData);

        // Get the response back from the server...
        t = new BTransmission(r);
        response = server.request(t);

        if (response.isAcknowledgement())
        {
          BDebug.print("ACK - BChallengeFight - FRS");
          //doStuff((BData)response.getData());
          //return;
        }
        else
        {
          BDebug.print("NAK - BChallengeFight - FRS");
          //return;
        }

    }
    catch (Exception e)
    {
      //Don't printStackTrace because it causes fatal
      //e.printStackTrace();

      BDebug.print ("BChallengeFight sendBattle ERROR" + e.getMessage() );
      //throw new Exception("Error: BChallengeFight/doStuff: " + e.getMessage());
    }
  }

  protected void stripOutArenaDetails()
  {
    // Remove anena ornaments and cache them
    Vector arenaObjects = arena.getArenaObjects(false);
    BArenaObject a = null;

    for (int i=0; i<arenaObjects.size(); i++)
    {
      a = (BArenaObject) arenaObjects.get(i);
      arena.removeArenaObject(a);
    }
  }

  protected void addBotToArena(BBot b)
  {
    // Clean-up the bot.
    b.cleanseLiveNodes();
    b.clearCurrentJointTransforms();

    // Place the Bot into the first spare slot in the arena.
    Vector v = arena.getPlaceHolders(BPlaceHolder.TYPE_BOT);
    arena.addArenaObject((BPlaceHolder) v.firstElement(), b);
  }

  // stops the fight
  public void stopFight()
  {
    userStopSignaled = true;
    simRunner.stopNextLoop();
    //t.stop();
  }

  // initialize both bots then run the fight
  private void runFight (BBot bot1, BBot bot2, String arenaFileName, float fightDuration) throws Exception
  {
    try
    {
      BGlobal.SHOW_TEXTURES = false;
      BGlobal.USE_TRANSPARENCY = false;

      bot1.init(false);
      bot2.init(false);
 
      BGlobal.SHOW_TEXTURES = true;
      BGlobal.USE_TRANSPARENCY = true;

      //BDebug.print("BChallengeFight - bots initialized");

      // Create the Arena
      filePath = BGlobal.ARENAPATH + arenaFileName + BGlobal.ARENASUFFIX;

      arena = BArena.loadFromFile(filePath, false);

      stripOutArenaDetails();


      addBotToArena(bot2);
      arena3dCanvas.loadArenaObject(bot2);

      addBotToArena(bot1);
      arena3dCanvas.loadArenaObject(bot1);

      bot1.setZonesVisible(false);
      bot2.setZonesVisible(false);

      simulation = new BSimulation(bot2, bot1, arena, fightDuration, arena3dCanvas, true, true);

      BDebug.print("BChallengeFight - starting simRunner;");

      simRunner.start(simulation);

    }
    catch (Exception e)
    {
      throw new Exception("Error: BChallengeFight/runFight: " +e.getMessage());
    }

  }

  public void actionPerformed (ActionEvent ae)
  {

    try
    {

      //BDebug.print("BUSY : " + busy);
      //BDebug.print("BattleRunning : " + battleRunning);
      //BDebug.print("!busy && !battleRunning : " + (!busy && !battleRunning));
      /*if (!busy && !battleRunning)
      {
        // If it's not busy processing then do something
        busy = true;


        BProcessor p = new BProcessor()
        {
          public void process()
          {
            try {
                doStuff(null);
            }
            catch (Exception e) { BDebug.print("BChallengeFight - exception " + e); }
          }
        };
        p.start();

        // Finished process now available for another task
        busy = false;

      }
      else*/ if (battleRunning  && !busy)
      {
        try
        {
          BMessage response = null;
          BRequest r = null;

          // Send a battle progress report to the server
          float complete = (simulation.getElapsedTime()/(simulation.getElapsedTime() + simulation.getRemainingTime())*100);

          BDebug.print("******* PC: " + complete + " FightID: " + fightID);

          BProgressData progressData = new BProgressData(fightID);
          progressData.progress = (int)complete;

          // PRO - PROgress of Fight
          r = new BRequest(server, "PRO");

          // Prepare the request Data...
          r.setData(progressData);

          // Get the response back from the server...
          BTransmission t = new BTransmission(r);
          response = server.request(t);

          if (response.isAcknowledgement())
          {
            BDebug.print("ACK");
            //return;
          }
          else
          {
            BDebug.print("NAK");
          //return;
          }
        }
        catch(Exception ex)
        {
          BDebug.print ("BChallengeFight.actionPerformed ERROR " + ex.getMessage() );
          
        }

      }
    }

    catch (Exception e)
    {
      e.printStackTrace();

      //System.out.println("Error actionPerformed: " + e.getMessage());
      //System.err.println("Error actionPerformed: " + e.getMessage());
    }
  }
}
