package roboforge.ui.net;

import roboforge.BDebug;
import roboforge.BGlobal;

import roboforge.net.*;
import roboforge.net.client.*;
import roboforge.net.data.*;
import roboforge.net.services.*;

import roboforge.ui.*;
import roboforge.ui.action.*;
import roboforge.ui.components.*;
//import roboforge.ui.events.*;
import roboforge.ui.layers.*;

import roboforge.util.graphics.*;
import roboforge.util.io.*;
import roboforge.util.data.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

import javax.swing.*;
//import javax.swing.border.*;
import javax.swing.event.*;
//import javax.swing.tree.*;
//import javax.swing.text.*;

import roboforge.BArenaCanvas;
import roboforge.util.text.*;

import roboforge.util.processors.*;

import roboforge.BBot;
import roboforge.sound.*;
////////////////////////////////////////////////////////////////////////////////
//
// Class BChallengePanel - contains classes and methods for the challenge
// room operation on the client
//
//
//

public class BChallengePanel extends BSystemPanel
  
{

  // Internal variables...
  // -------------------------------------------------------------

  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
protected BTournamentScreen   tournaments;
  protected BClientStack        server;
  protected BClientStack        roomServer;
  protected boolean             isRefreshRequired;
  protected int battleAccuracySave; // stores the battle accuracy

  protected boolean loginError = false;  // true if an error occurs loggin in to the challenge server
  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {

  });


  BRoomPanel room;    // the challenge room
  BLobbyPanel lobby;  // lobby containing all the rooms

  // Constructor.
  // -------------------------------------------------------------
  public BChallengePanel(BTournamentScreen parent)
  {
    super(parent, "challenge room");

    this.server             = parent.getServer();
    this.tournaments        = parent;
    this.isRefreshRequired  = true;

    try {
      roomServer = new BClientStack();
    }
    catch(Exception e) {}
    
    //room = new BRoomPanel("Challenge One", roomServer);
    lobby = new BLobbyPanel(roomServer, this);

    battleAccuracySave = BGlobal.getBattleAccuracy();

    layoutChallengeChildren();
    configChallengeChildren();
  }


  // Initialization helper methods...
  // -------------------------------------------------------------
  protected void layoutChallengeChildren()
  {
    add(lobby);
  }

  protected void configChallengeChildren()
  {
    lobby.setVisible(false);
  }

  public void setPanelButtonsEnabled(boolean enabled)
  {
    tournaments.setPanelButtonsEnabled(enabled);
    //tournaments.closeScreen();
  }

  // get the main server
  public BClientStack getServer()
  { return this.server; }

  public void setupServer()
  {
    try {
      BServerTarget serverTarget = new BServerTarget(BGlobal.URLSCHALLENGE_ROOM);
      BDebug.print("BChallengePanel - server = " + BGlobal.URLSCHALLENGE_ROOM);
      roomServer.setServerTarget(serverTarget);
      roomServer.login(server.getUserName(), server.getPassword());

    }
    catch(Exception e)
    {
      loginError = true;
      BDebug.print("BChallengePanel - setupServer " + e.getMessage());

      BProcessor p = new BProcessor()
      {
        public void process()
        {
          showChallengeServerError();
        }
      };
      p.start();
    }
  }

  public void showChallengeServerError()
  {
    BPromptDialog challengeError;
    challengeError = BPromptDialog.createOKDialog("challenge room error", "There was an error logging in to the challenge room server.  Please try again later.");
    BApplication.showDialog(challengeError);
  }

  // enter a challenge room
  public void enterNewRoom(BChallengeRoomData data)
  {
    room = new BRoomPanel(data, roomServer, this);
    room.onActivate();
    lobby.setVisible(false);
    remove(lobby);
    add(room);
    room.setVisible(true);
    room.onVisible();
  }

  // return to the lobby from a challenge room
  public void exitRoom()
  {
    BDebug.print("BChallengePanel - about to set room invisible");
    room.setVisible(false);
    BDebug.print("BChallengePanel - about to remove room");
    remove(room);
    BDebug.print("BChallengePanel - about to add lobby");
    add(lobby);
    BDebug.print("BChallengePanel - about to set lobby visible");
    lobby.setVisible(true);
    BDebug.print("BChallengePanel - about to set panel buttons enabled");
    setPanelButtonsEnabled(true);
    BDebug.print("BChallengePanel - exit room method done");
  }

  public boolean isBusy()
  {
    if(room != null)
    {
      return room.isRunningBattle();
    }
    else
      return false;
  }

  public void onActivate()
  {
    super.onActivate();

    setupServer();

    if(!loginError)
    {
      lobby.onActivate();

      if(room != null)
      {
        if(room.isRoomActive())
        {
          room.setAFK(false);
        }
        else
          lobby.setVisible(true);
      }
      else
        lobby.setVisible(true);
    }
  }

  public void onDeactivate()
  {
    super.onDeactivate();
    if(room != null)
    {
      if(room.isRoomActive())
      {
        room.setAFK(true);
      }
    }
  }


  public void onClose()
  {
    super.onClose();

    if(room != null)
    {
      room.onClose();
      room.cleanUp();
    }
    if(lobby != null)
      lobby.cleanUp();
    BGlobal.setBattleAccuracy(battleAccuracySave);
  }
}

////////////////////////////////////////////////////////////////////////////////
//
// Class BLobbyPanel
//
// The front end for the challenge room lobby
//
class BLobbyPanel extends BBoxLayer implements ActionListener, ListSelectionListener
{

  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
// UI Resources ...
  // -------------------------------------------------------------
  BLobbyControl control;
  BClientStack server;
  BChallengePanel parent;

  BRoomCreateDialog roomDialog;  // dialog for creating a new challenge room
  BPasswordDialog passDialog;    // dialog for password entry

  BChallengeRoomData roomData;   // data for the selected challenge room

  String newRoomPassword = "";

  int maxRooms;    // the maximum number of rooms allowed by the server
  int maxPlayers;  // the maximum number of players in a room
  float maxCost;   // the maximum cost of a bot in a challenge
  float maxTime;   // the maximum fight time of a challenge fight

  protected BUIResources res = new BUIResources( new Object[]
  {
    "roomScroll.tooltip",           "Challenge rooms.",
    "roomScroll.width",             new Integer(420),

    "descScroll.tooltip",           "Room description.",
    "descScroll.height",            new Integer(120),

    "createButton.text",            "Create",
    "createButton.tooltip",         "Create a challenge room.",

    "enterButton.text",             "Enter",
    "enterButton.tooltip",          "Enter challenge room.",

    "titleLabel.icon",              BIconCache.getHSB("challenge_rooms.gif"),
    "titleLabel.foreground",        BGlobal.colors.getMidShade(),
    "titleLabel.height",            new Integer(30),
    "titleLabel.width",             new Integer(200),

    "refreshButton.name",           "Refresh",
    "refreshButton.icon",           BIconCache.getHSB("net_refresh_icon.gif"),
    "refreshButton.tooltip",        "Refresh challenge rooms.",
    "refreshButton.height",         new Integer(30),
    "refreshButton.width",          new Integer(30),

    "roomTitle.text",               "available rooms",
    "roomTitle.width",              new Integer(420),
    "descTitle.text",               "room description",
  });

  // Visual elements...
  // -------------------------------------------------------------

  BList roomList = new BList();
  Vector roomVec = new Vector();

  BMultiLabel descArea = new BMultiLabel();

  BScrollPane roomScroll = new BScrollPane(res, "roomScroll", roomList);
  BScrollPane descScroll = new BScrollPane(res, "descScroll", descArea);

  BButton createButton = new BButton(res, "createButton");
  BButton enterButton = new BButton(res, "enterButton");

  BButton refreshButton = new BButton(res, "refreshButton");

  BLabel titleLabel = new BLabel(res, "titleLabel");

  BTitlebarLabel roomTitle = new BTitlebarLabel(res, "roomTitle");
  BTitlebarLabel descTitle = new BTitlebarLabel(res, "descTitle");

  BMultiLabel roomInfo = new BMultiLabel();

 // BLabel titleLabel = new BLabel(BIconCache.getHSB("challenge_rooms.gif"));

  // does the list of rooms need updating?
  protected boolean roomUpdateRequired = true;

  // constructor
  public BLobbyPanel(BClientStack server, BChallengePanel parent)
  {
    super(BBoxPanel.X_AXIS);

    this.server = server;
    this.parent = parent;
    control = new BLobbyControl(this, server);

    RoomListRenderer renderer = new RoomListRenderer();
    roomList.setCellRenderer(renderer);

    layoutLobbyChildren();
    configLobbyChildren();
  }

  public void onActivate()
  {
    BDebug.print("BChallengePanel (BLobbyPanel) - activated");
    if(roomUpdateRequired)
    {
      control.getConstants();
      control.getRoomList();
      roomUpdateRequired = false;
    }
  }

  // listener methods
  public void valueChanged(ListSelectionEvent e)
  {
    if(e.getSource() == roomList)
      roomSelected();
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource() == enterButton)
      enterSelectedRoom();

    if(e.getSource() == createButton)
      createNewRoom();

    if(e.getSource() == refreshButton)
    {
      enterButton.setEnabled(false);
      control.getRoomList();
    }
  }

  public void layoutLobbyChildren()
  {
    BBoxPanel left    = new BBoxPanel(BBoxPanel.Y_AXIS);
    BBoxPanel right   = new BBoxPanel(BBoxPanel.Y_AXIS);
    //BBoxPanel buttons = new BBoxPanel(BBoxPanel.X_AXIS);

    left.addMargin();
    left.addLinear(titleLabel, getHGlue());
    left.addSmallMargin();
    left.add(roomTitle);
    left.addTinyMargin();
    left.add(roomScroll);
    left.addSmallMargin();
    left.addLinear(getHGlue(), createButton, getTinyMargin(), enterButton);
    left.addMargin();

    right.addMargin();
    right.addMargin();
    right.addMargin();
    right.addSmallMargin();
    right.addHGlued(descTitle);
    right.addTinyMargin();
    right.addHGlued(descScroll);
    right.addMargin();
    right.addGlued(roomInfo);
    right.addGlue();

    addMargin();
    addLinear(getVStrut(55), refreshButton, getVGlue());
    addMargin();
    add(left);
    addMargin();
    add(right);
    addHStrut(50);
  }

  public void configLobbyChildren()
  {
    descArea.setBorder(null);
    roomList.addListSelectionListener(this);
    enterButton.addActionListener(this);
    createButton.addActionListener(this);
    refreshButton.addActionListener(this);
    enterButton.setEnabled(false);
  }

  // a room has been chosen from the list
  public void roomSelected()
  {
    roomData = (BChallengeRoomData)roomList.getSelectedValue();
    setDescription(roomData);
    if(roomData.currentNoOfPeople < roomData.maxNumberOfParticipants)
      enterButton.setEnabled(true);
  }

  // sets the text in the description field and set the room info
  public void setDescription(BChallengeRoomData data)
  {
    descArea.setText(data.roomDescription);

    roomInfo.setText("Challenge room information:");
    roomInfo.append("\n\nMaximum occupants:\t" + data.maxNumberOfParticipants);
    roomInfo.append("\nCurrent occupants:\t" + data.currentNoOfPeople);
    roomInfo.append("\n\nArena:\t\t" + data.arena);
    roomInfo.append("\nBot cost limit:\t\t" + Math.round(data.maxEntryCost) + "cred");
    roomInfo.append("\nFight time:\t\t" + Math.round(data.maxFightTime) + "s");

    String simAccuracy = "";

    if(data.simulationSpeed == BChallengeRoomData.SIMULATION_ACCURACY_LOW)
      simAccuracy = "Low";
    if(data.simulationSpeed == BChallengeRoomData.SIMULATION_ACCURACY_MEDIUM)
      simAccuracy = "Medium";
    if(data.simulationSpeed == BChallengeRoomData.SIMULATION_ACCURACY_HIGH)
      simAccuracy = "High";
    roomInfo.append("\nSim accuracy:\t\t" + simAccuracy);

    if(data.isPassworded)
      roomInfo.append("\n\nPassword required:\tYes");
    else
      roomInfo.append("\n\nPassword required:\tNo");
  }

  // enter the challenge room selected in the list
  public void enterSelectedRoom()
  {
    if(roomData.isPassworded)
      startPassword();
    else if(control.loginToRoom(roomData, server.getUserName(), server.getPassword(), null))
      parent.enterNewRoom(roomData);
  }

  // start a process to pop up a password prompt box
  public void startPassword()
  {
    BProcessor p = new BProcessor()
    {
      public void process()
      { showPasswordDialog(); }
    };
    p.start();
  }

  // show the password dialog
  public void showPasswordDialog()
  {
    passDialog = new BPasswordDialog();
    passDialog.requestFocus();
    BApplication.showDialog(passDialog);
  }

  // sets up the list of available rooms
  public void setRoomData(Vector rooms)
  {
    enterButton.setEnabled(false);
    descArea.setText("");
    BDebug.print("BChallengePanel - got to setting the list data");
    roomVec = (Vector)rooms.clone();
    roomList.setListData(rooms);

    roomList.clearSelection();
 }

  // start a process to create a new challenge room
  public void createNewRoom()
  {
    BProcessor p = new BProcessor()
    {
      public void process()
      { showRoomDialog(); }
    };

    p.start();
  }

  // show the room creation dialog
  public void showRoomDialog()
  {
    roomDialog = new BRoomCreateDialog();
    roomDialog.requestFocus();
    BApplication.showDialog(roomDialog);

  }

  // ask the lobby control to create a new room
  public void createTheRoom(String name, String description, String password, float cost, float time, String arena, int accuracy)
  {
    roomData = new BChallengeRoomData(-1);

    roomData.maxNumberOfParticipants = maxPlayers;
    roomData.maxEntryCost = cost;
    roomData.maxFightTime = time;
    roomData.password = password;
    roomData.arena = arena;
    roomData.simulationSpeed = accuracy;
    roomData.roomName = name;
    roomData.roomDescription = description;
    control.createTheRoom(roomData);

    newRoomPassword = password;
  }


  // set the constants that limit room setup
  public void setConstants(int maxRooms, int maxPlayers, float maxCost, float maxTime)
  {
    this.maxRooms = maxRooms;
    this.maxPlayers = maxPlayers;
    this.maxCost = maxCost;
    this.maxTime = maxTime;
  }


  //////////////////////////////////////////////////////////////////////////////
  //
  //  inner class used for creating a new challenge room
  //
  protected class BRoomCreateDialog extends BSystemDialog
  {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;

	protected RoomCreateClient client;

    protected BUIResources res = new BUIResources( new Object[]
    {
      "nameLabel.text",         "Challenge room name:",

      "descLabel.text",         "Challenge room description",

      "passwordCheck.text",     "Password",

      "costLabel.text",         "Bot cost limit(cred):",

      "timeLabel.text",         "Fight time(s):",

      "arenaLabel.text",        "Arena:",

      "accuracyLabel.text",     "Sim accuracy:",

      "okButton.text",          "Ok",

      "cancelButton.text",      "Cancel",
    });

    protected BLabel nameLabel = new BLabel(res, "nameLabel");
    protected BLabel descLabel = new BLabel(res, "descLabel");

    protected BLabel costLabel = new BLabel(res, "costLabel");
    protected BLabel timeLabel = new BLabel(res, "timeLabel");
    protected BLabel arenaLabel = new BLabel(res, "arenaLabel");
    protected BLabel accuracyLabel = new BLabel(res, "accuracyLabel");

    protected BCheckBox passwordCheck = new BCheckBox(res, "passwordCheck");
    protected BTextField passwordField = new BTextField();
    protected BTextField nameField = new BTextField();
    protected BTextArea descArea = new BTextArea();
    protected BScrollPane descScroll = new BScrollPane(descArea);

    protected BComboBox costCombo = new BComboBox();
    protected BComboBox timeCombo = new BComboBox();
    protected BFileComboBox arenaCombo = new BFileComboBox();
    protected BComboBox accuracyCombo = new BComboBox();

    protected BButton okButton = new BButton(res, "okButton");
    protected BButton cancelButton = new BButton(res, "cancelButton");

    public BRoomCreateDialog()
    {
      super("create a new challenge room", true);
      setSize(new Dimension(300, 420));
      setLocation(250, 80);

      addButton(cancelButton);
      addButton(okButton);

      client = new RoomCreateClient();
      setClient(client);

      modal = true;
      blocking = false;
    }

    protected class RoomCreateClient extends SystemDialogClient implements ActionListener
    {
      /**
		 * 
		 */
		private static final long serialVersionUID = -1L;
	public RoomCreateClient()
      {
        super(BoxLayout.Y_AXIS);
        layoutDialogChildren();
        configDialogChildren();
      }

      public void layoutDialogChildren()
      {
        BBoxPanel tools = new BBoxPanel(BBoxPanel.Y_AXIS);
        BBoxPanel left = new BBoxPanel(BBoxPanel.Y_AXIS);
        BBoxPanel right = new BBoxPanel(BBoxPanel.Y_AXIS);

        tools.addHGlued(nameLabel);
        tools.addHGlued(nameField);
        tools.addSmallMargin();
        tools.addHGlued(descLabel);
        tools.addHGlued(descScroll);
        tools.addSmallMargin();
        tools.addLinear(passwordCheck, tools.getSmallMargin(), passwordField);
        tools.addSmallMargin();

        left.addHGlued(costLabel);
        left.addSmallMargin();
        left.addHGlued(timeLabel);
        left.addSmallMargin();
        left.addHGlued(arenaLabel);
        left.addSmallMargin();
        left.addHGlued(accuracyLabel);

        right.add(costCombo);
        right.addSmallMargin();
        right.add(timeCombo);
        right.addSmallMargin();
        right.add(arenaCombo);
        right.addSmallMargin();
        right.add(accuracyCombo);

        tools.addLinear(left, tools.getMargin(), right);
        tools.addMargin();

        add(tools);
      }

      public void configDialogChildren()
      {

        descArea.setLineWrap(true);
        descArea.setWrapStyleWord(true);
        descArea.setBorder(null);
        // fill the cost box
        for (int cost = 10000; cost <= 110000; cost += 5000)
          costCombo.addItem(new Integer(cost));

        costCombo.setSelectedIndex(5);
        // fill the time box
        for (int time = 60; time <= maxTime; time += 60)
          timeCombo.addItem(new Integer(time));

        accuracyCombo.addItem("Low");
        accuracyCombo.addItem("Medium");
        accuracyCombo.addItem("High");

        accuracyCombo.setSelectedIndex(2);

        passwordField.setEnabled(false);

        try
        {
          arenaCombo.setFilter(new BRoboforgeFilter(BGlobal.ARENASUFFIX));
          arenaCombo.setSourcePath(BGlobal.ARENAPATH);

          if (!arenaCombo.isEmpty())
            arenaCombo.setSelectedIndex(0);
        }
        catch (IOException e) {}

        passwordCheck.addActionListener(this);
        okButton.addActionListener(this);
        cancelButton.addActionListener(this);
      }

      public void actionPerformed(ActionEvent e)
      {
        // the password checkbox has been clicked
        if(e.getSource() == passwordCheck)
        {
          if(passwordCheck.isSelected())
          {
            passwordField.setEnabled(true);
            passwordField.requestFocus();
          }
          else
            passwordField.setEnabled(false);
        }

        // ok button pressed
        if(e.getSource() == okButton)
        {
          if(verifyData())
          {
            BApplication.hideDialog(BRoomCreateDialog.this);
            dispose();
          }
        }

        // cancel button pressed
        if(e.getSource() == cancelButton)
        {
          BApplication.hideDialog(BRoomCreateDialog.this);
          dispose();
        }
      }

      // check for problems in the creation data and report
      String errorMessage = "";
      public boolean verifyData()
      {
        errorMessage = "";
        // the string that will display the problems if there are any
        boolean problem = false;

        String roomName = nameField.getText();
        String roomDesc = descArea.getText();
        boolean passworded = passwordCheck.isSelected();
        String password = passwordField.getText();
        float cost = ((Integer)costCombo.getSelectedItem()).floatValue();
        float time = ((Integer)timeCombo.getSelectedItem()).floatValue();
        String arena = (String)arenaCombo.getSelectedItem();
        int accuracy = accuracyCombo.getSelectedIndex();

        // check for problems
        if(roomName.length() == 0)
        {
          problem = true;
          errorMessage = "Please enter a name for your challenge room";
        }
        if(passworded && ( passwordField.getText().length() == 0) )
        {
          if(problem)
            errorMessage = errorMessage + " and specify a password";
          else
          {
            problem = true;
            errorMessage = "Please enter a password for you challenge room";
          }
        }
        errorMessage = errorMessage +  ".";

        // if there are no problems then create the room else display the problems
        if(!problem)
        {
          if(!passworded)
            password = null;
          createTheRoom(BDictionaryTree.getSharedInstance().filter(roomName), BDictionaryTree.getSharedInstance().filter(roomDesc), password, cost, time, arena, accuracy);
          return true;
        }
        else
        {
          BProcessor p = new BProcessor()
          {
            public void process()
            {
              BApplication.hideDialog(BRoomCreateDialog.this);
              roomCreateError(errorMessage);
            }
          };
          p.start();
          return false;
        }
      }

      // show the dialog with the problem description
      public void roomCreateError(String errorMessage)
      {
        BPromptDialog createFail;
        createFail = BPromptDialog.createOKDialog("challenge room creation error", errorMessage);
        BApplication.showDialog(createFail);
        BApplication.showDialog(BRoomCreateDialog.this);
      }
    }
  }

    // hide the dialog to prevent it from stopping incomming challenges
  public void hidePassDialog()
  {
    if(passDialog != null)
    {
      BApplication.hideDialog(passDialog);
      passDialog.dispose();
    }
  }

  //////////////////////////////////////////////////////////////////////////////
  //
  //  inner class used for password entry
  //
  protected class BPasswordDialog extends BSystemDialog
  {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;

	protected PasswordClient client;

    protected BUIResources res = new BUIResources( new Object[]
    {
      "passLabel.text",         "Please enter the password for this room:",

      "okButton.text",          "Ok",

      "cancelButton.text",      "Cancel",
    });

    protected BLabel passLabel = new BLabel(res, "passLabel");
    protected BPasswordField passwordField = new BPasswordField();

    protected BButton okButton = new BButton(res, "okButton");
    protected BButton cancelButton = new BButton(res, "cancelButton");

    public BPasswordDialog()
    {
      super("password required", true);
      setSize(new Dimension(250, 185));
      setLocation(275, 200);

      addButton(cancelButton);
      addButton(okButton);

      client = new PasswordClient();
      setClient(client);

      modal = true;
      blocking = false;
    }

    protected class PasswordClient extends SystemDialogClient implements ActionListener
    {
      /**
		 * 
		 */
		private static final long serialVersionUID = -1L;

	public PasswordClient()
      {
        super(BoxLayout.Y_AXIS);
        layoutDialogChildren();
        configDialogChildren();
      }

      public void layoutDialogChildren()
      {
        BBoxPanel panel = new BBoxPanel(BoxLayout.Y_AXIS);

        panel.addHGlued(passLabel);
        panel.addSmallMargin();
        panel.add(passwordField);
        panel.addMargin();

        add(panel);
      }

      public void configDialogChildren()
      {
        okButton.addActionListener(this);
        cancelButton.addActionListener(this);
        passwordField.addActionListener(this);
        passwordField.setNextFocusableComponent(okButton);
      }

      public void actionPerformed(ActionEvent e)
      {
        if(e.getSource() == okButton || e.getSource() == passwordField)
        {
          if(verifyPassword())
          {
            roomData.password = new String(passwordField.getPassword());
            BApplication.hideDialog(BPasswordDialog.this);
            dispose();
            parent.enterNewRoom(roomData);
          }
          else
          {
            passwordField.setText("");
            transferFocus();
          }
        }

        if(e.getSource() == cancelButton)
        {
          BApplication.hideDialog(BPasswordDialog.this);
          dispose();
        }
      }

      // check to see if the password is valid
      public boolean verifyPassword()
      {
        boolean problem = false;

        problem = !( control.loginToRoom(roomData, server.getUserName(), server.getPassword(), new String(passwordField.getPassword()) ) );

        // if there are no problems then enter the room
        if(!problem)
        {
          return true;
        }
        else
        {
        /*
          BProcessor p = new BProcessor()
          {
            public void process()
            {
              BApplication.hideDialog(BPasswordDialog.this);
              passwordError();
            }
          };
          p.start();
*/
          return false;
        }
      }

      // show the dialog with the problem description
      public void passwordError()
      {
        BPromptDialog passFail;
        passFail = BPromptDialog.createOKDialog("invalid password", "The password you have entered is incorrect - please try again.");
        BApplication.showDialog(passFail);
        BApplication.showDialog(BPasswordDialog.this);
      }
    }
  }

  //////////////////////////////////////////////////////////////////////////////
  //
  //  inner class for rendering the list
  //
  class RoomListRenderer extends JLabel implements ListCellRenderer
  {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
	Icon open = BIconCache.getHSB("room_open_icon.gif");
    Icon pass = BIconCache.getHSB("room_pass_icon.gif");
    Icon full = BIconCache.getHSB("room_full_icon.gif");

    public RoomListRenderer()
    {
      setOpaque(true);
    }

    public Component getListCellRendererComponent(
     JList list,
     Object value,            // value to display
     int index,               // cell index
     boolean isSelected,      // is the cell selected
     boolean cellHasFocus)    // the list and the cell have the focus
    {
      BChallengeRoomData d = (BChallengeRoomData)value;

      String s = d.roomName;
      setText(s);
      if(d.isPassworded)
        setIcon(pass);
      else
        setIcon(open);

      if(d.maxNumberOfParticipants == d.currentNoOfPeople)
        setIcon(full);  

      if (isSelected)
      {
        setBackground(list.getSelectionBackground());
        setForeground(list.getSelectionForeground());
      }
      else
      {
        setBackground(list.getBackground());
        setForeground(list.getForeground());
      }

      setEnabled(list.isEnabled());
      setFont(list.getFont());
      return this;
    }
  }
  public void cleanUp()
  {
    roomDialog = null;
    passDialog = null;
    roomData = null;
    newRoomPassword = null;
    roomList = null;
    roomVec = null;
    descArea = null;
    roomScroll = null;
    descScroll = null;
    createButton = null;
    enterButton = null;
    refreshButton = null;
    titleLabel = null;
    roomTitle = null;
    descTitle = null;
    roomInfo = null;

    control.cleanUp();
  }
}

////////////////////////////////////////////////////////////////////////////////
//
// Class BLobbyControl
//
// Communicates with the server
//
class BLobbyControl //implements Runnable
{

  protected BLobbyPanel lobby;
  protected BClientStack server;
  protected String loginError = "";

  public BLobbyControl(BLobbyPanel lobby, BClientStack server)
  {
    this.server = server;
    this.lobby = lobby;
  }

  protected void getRoomListFromServer()
  {
     lobby.parent.setStatus("Refreshing room listings...");
     lobby.parent.setBusyWorking(true);

      try
      {
        BTransmission t = new BTransmission(
          new BRequest(server, "GCR"));

        // Get the response back from the server...
        BMessage response = server.request(t);
        BDataSet dataSet = (BDataSet)response.getData();

        if (response.isAcknowledgement())
          BDebug.print("BChallengePanel - geRoomList ACK");
        else
          BDebug.print("BChallengePanel - geRoomList NAK");

        Vector rooms = new Vector();
        if(dataSet != null)
        {
          if(dataSet.data != null)
          {
            for(int i = 0; i < dataSet.data.length; i++)
            {
              if(dataSet.data[i] != null)
                rooms.add(dataSet.data[i]);
            }
          }
        }
        lobby.setRoomData( rooms );
      }
      catch (Exception e)
      {
        BDebug.print("BChallengePanel (BLobbyControl) - getRoomList " + e.getMessage() + " " + e);
      }


      lobby.parent.setStatus("Rooms refreshed");
      lobby.parent.setBusyWorking(false);
  }
  // gets the next message from the server
  public void getRoomList()
  {

  BProcessor p = new BProcessor()
      {
        public void process()
        {
          getRoomListFromServer();
        }
      };

  p.start();


  }

  // gets a set of constant limit values from the server (max rooms, max participants, max
  // entry cost, max fight time)
  public void getConstants()
  {
      try
      {
        BTransmission t = new BTransmission(
          new BRequest(server, "GCD"));

        // Get the response back from the server...
        BMessage response = server.request(t);

        BChallengeConstantData data = (BChallengeConstantData)response.getData();

        if (response.isAcknowledgement())
          BDebug.print("getConstants - ACK");
        else
          BDebug.print("getConstants - NAK");

        // set the constants in the lobby
        int maxRooms = data.maxNumberOfChallengeRooms;
        int maxPlayers = data.maxNumberOfParticipants;
        float maxCost = data.maxEntryCost;
        float maxTime = data.maxFightTime;
        lobby.setConstants(maxRooms, maxPlayers, maxCost, maxTime);

      }
      catch (Exception e)
      {
        BDebug.print("BChallengePanel (BLobbyControl - getConstants) " + e.getMessage());
      }
  }

  // asks the server to create a new challenge room, given BChallengeRoomData
  public void createTheRoom(BChallengeRoomData data)
  {
      try
      {
        BRequest r = new BRequest(server, "CCR");
        r.setData(data);

        // Get the response back from the server...
        BTransmission t = new BTransmission(r);
        BMessage response = server.request(t);


        if (response.isAcknowledgement())
        {
          BDebug.print("createTheRoom - ACK");
          getRoomList();

          BIdentData roomID = (BIdentData)response.getData();

          data.setID(roomID.getID());

          if( loginToRoom(data, server.getUserName(), server.getPassword(), data.password) )
            lobby.parent.enterNewRoom(data);

        }

        else
        {
          BDebug.print("createTheRoom - NAK");

          loginError = response.getData().toString();

          BDebug.print(loginError);
          BProcessor p = new BProcessor()
          {
            public void process()
            { showRoomError(false); }
          };
          p.start();
        }


      }
      catch (Exception e)
      {
        BDebug.print("BChallengePanel (BLobbyControl - createTheRoom) " + e.getMessage());
      }
  }

  protected boolean loginToRoom(BChallengeRoomData data, String userName, String passWord, String roomPassword)
  {
    try
    {
      // server.login(userName, passWord);
      // Automatically Enter the challenge room
      BMessage response = null;
      BRequest r = new BRequest(server, "ECR");

      BDebug.print( "BChallengePanel - accountID = " + server.getAccountID() );
      BDebug.print( "BChallengePanel - room password = " + roomPassword );

      BCRAuthenticationData d = new BCRAuthenticationData(server.getAccountID().longValue());
      d.challengeRoomID = data.getID();
      d.password = roomPassword;

      r.setData(d);

      // Get the response back from the server...
      BTransmission t = new BTransmission(r);
      response = server.request(t);

      if(response.getData() instanceof BErrorData)
      {
        loginError = response.getData().toString();

        BProcessor p = new BProcessor()
        {
          public void process()
          { showRoomError(true); }
        };
        p.start();
        return false;
      }

      if (response.isAcknowledgement())
      {
        BDebug.print("loginToRoom - ACK");
        return true;
      }
      else
      {
        BDebug.print("loginToRoom - NAK" + response);
        return false;
      }

    }
    catch (Exception e11)
    {
      BDebug.print("BChallengePanel (BLobbyControl - loginToRoom) " + e11.getMessage());
    }
    return false;
  }

  // show the room error message
  public void showRoomError(boolean login)
  {
    lobby.hidePassDialog();
    BPromptDialog reject;
    String topMessage = "";

    if(login)
      topMessage = "login error";
    else
      topMessage = "create error";

    reject = BPromptDialog.createOKDialog(topMessage, loginError);
    BApplication.showDialog(reject);

    // update the room list
    getRoomList();
  }

  public void cleanUp()
  {

  }
}

////////////////////////////////////////////////////////////////////////////////
//
// Class BRoomPanel
//
// The front end for the challenge room
//
class BRoomPanel extends BBoxLayer implements ActionListener, ListSelectionListener
{


  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
BClientStack server;
  // UI Resources ...
  // -------------------------------------------------------------
  BRoomControl control;
  BChallengePanel parent;
  BBotSelectDialog botDialog;
  BPlayerRankingDialog rankingDialog;

  BArenaCanvas arenaCanvas = new BArenaCanvas(null);

  boolean whisperMode = false;

  protected BUIResources res = new BUIResources( new Object[]
  {
    "messageScroll.tooltip",        "Messages.",

    "messageField.tooltip",         "Type your messages here.",
    "messageField.height",          new Integer(30),

    "playersScroll.tooltip",        "Players.",
    "playersScroll.width",          new Integer(110),

    "battleScroll.tooltip",         "Recently completed battles.",
    "battleScroll.width",           new Integer(180),
    "battleScroll.height",          new Integer(160),

    "viewScroll.width",             new Integer(110),
    "viewScroll.height",            new Integer(82),

    "infoScroll.tooltip",           "Information.",
    "infoScroll.width",             new Integer(180),

    "sendButton.text",              "Send",
    "sendButton.height",            new Integer(30),

    "messageLabel.text",            "messages",

    "battleViewLabel.text",         "battle view",
    "battleViewLabel.width",        new Integer(110),

    "playersLabel.text",            "players",
    "playersLabel.width",           new Integer(110),

    "battleLabel.text",             "battles",
    "battleLabel.width",            new Integer(180),

    "infoLabel.text",               "information",
    "infoLabel.width",              new Integer(135),

    "titleLabel.text",              "",
    "titleLabel.width",             new Integer(200),
    "titleLabel.fontsize",          new Integer(14),
    "titleLabel.height",            new Integer(20),
    "titleLabel.fontstyle",         new Integer(Font.BOLD),
    "titleLabel.v_align",           new Integer(SwingConstants.BOTTOM),

    "roomInfoLabel.height",         new Integer(40),

    "blockButton.icon",             BIconCache.getHSB("room_blocked_icon.gif"),
    "blockButton.width",            new Integer(30),
    "blockButton.height",           new Integer(30),
    "blockButton.tooltip",          "Block selected player's messages.",

    "whispButton.icon",             BIconCache.getHSB("room_whisper_icon.gif"),
    "whispButton.width",            new Integer(30),
    "whispButton.height",           new Integer(30),
    "whispButton.tooltip",          "Let selected player recieve whispered messages.",

    "challButton.icon",             BIconCache.getHSB("room_challenge_icon.gif"),
    "challButton.width",            new Integer(30),
    "challButton.height",           new Integer(30),
    "challButton.tooltip",          "Challenge selected player to a fight.",

    "whispModeButton.icon",         BIconCache.getHSB("room_broadcast_icon.gif"),
    "whispModeButton.width",        new Integer(30),
    "whispModeButton.height",       new Integer(30),
    "whispModeButton.tooltip",      "Toggle whisper mode.",

    "downloadButton.icon",          BIconCache.getHSB("play_icon.gif"),
    "downloadButton.width",         new Integer(30),
    "downloadButton.height",        new Integer(30),
    "downloadButton.tooltip",       "Download and play selected battle.",

    "playButton.icon",              BIconCache.getHSB("play_icon.gif"),
    "playButton.width",             new Integer(30),
    "playButton.height",            new Integer(30),
    "playButton.tooltip",           "Play selected battle.",

    "exitButton.text",              "Exit room",
    "exitButton.height",            new Integer(30),

    "cancelButton.text",            "Cancel battle",
    "cancelButton.width",           new Integer(140),
    "cancelButton.height",          new Integer(30),

    "statsButton.text",             "Stats",
    "statsButton.height",           new Integer(19),
    "statsButton.width",            new Integer(45),

    "statsFiller.text",             "",
    "statsFiller.width",            new Integer(45),

    "refreshButton.name",           "Refresh",
    "refreshButton.icon",           BIconCache.getHSB("net_refresh_icon.gif"),
    "refreshButton.tooltip",        "Refresh challenge rooms.",
    "refreshButton.height",         new Integer(30),
    "refreshButton.width",          new Integer(30),

  });

  
  class BCopyableMultiLabel extends BTextArea
  {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
	protected int maxCharacters = 3000;

    public BCopyableMultiLabel ()
    {
      super();
      this.setEditable(false);
      this.setWrapStyleWord(true);
      this.setLineWrap(true);
    }

    public void append(String s)
    {
      if (s== null)
        return;
      else if (s.trim().length() == 0)
        return;

      try
      {
        if (getDocument().getLength() > maxCharacters)
        {

          getDocument().remove(
                                  0,getLineEndOffset(0)
                              );
        }
      }
      catch (Exception e)
      {
        BDebug.print (" BChallengePanel.BCopyLabel error " + e.getMessage() );

      };
      
      super.append(s);

      scrollRectToVisible(new Rectangle(0, messageArea.getHeight(), 1, messageArea.getHeight() + 15) );


    }


  }
  
  // Visual elements...
  // -------------------------------------------------------------
  BCopyableMultiLabel messageArea = new BCopyableMultiLabel();
  BScrollPane messageScroll = new BScrollPane(res, "messageScroll", messageArea);

  BTextField messageField = new BTextField(res, "messageField");
  BButton sendButton = new BButton(res, "sendButton");

  BList playersList = new BList();

  BList battleList = new BList();

  BButton refreshButton = new BButton(res, "refreshButton");
  javax.swing.Timer refreshTimer;
  boolean refreshAllowed = true;

  BScrollPane battleScroll = new BScrollPane(res, "battleScroll", battleList);

  BScrollPane viewScroll = new BScrollPane(res, "viewScroll", arenaCanvas);

  BMultiLabel infoArea = new BMultiLabel();
  BScrollPane infoScroll = new BScrollPane(res, "infoScroll", infoArea);

  BScrollPane playersScroll = new BScrollPane(res, "playersScroll", playersList);

  BTitlebarLabel messageLabel = new BTitlebarLabel(res, "messageLabel");
  BTitlebarLabel playersLabel = new BTitlebarLabel(res, "playersLabel");

  BTitlebarLabel battleLabel = new BTitlebarLabel(res, "battleLabel");
  BTitlebarLabel infoLabel = new BTitlebarLabel(res, "infoLabel");
  BTitlebarLabel statsFiller = new BTitlebarLabel(res, "statsFiller");
  BTitlebarLabel battleViewLabel = new BTitlebarLabel(res, "battleViewLabel");

  BLabel titleLabel = new BLabel(res, "titleLabel");

  BMultiLabel roomInfoLabel = new BMultiLabel(res, "roomInfoLabel");

  BButton blockButton = new BButton(res, "blockButton");
  BButton whispButton = new BButton(res, "whispButton");
  BButton challButton = new BButton(res, "challButton");

  BButton statsButton = new BButton(res, "statsButton");

  BActionButton playButton = new BActionButton();

  DownloadAction downloadAction = new DownloadAction(res, "downloadButton");
  PlayAction playAction = new PlayAction(res, "playButton");

  BButton cancelButton = new BButton(res, "cancelButton");

  BToggleButton whispModeButton = new BToggleButton(res, "whispModeButton");

  BButton exitButton = new BButton(res, "exitButton");

  String oldMessage = "";   // prevent the same message being sent more than once

  protected boolean roomActive = false;
  // does the player need to log in to this room?
  protected boolean loginRequired = true;

  protected BChallengeRoomData data; // the data for this room

  protected boolean challenging = false;

  Vector blockedList = new Vector();  // lists for storing the players who are blocked
  Vector whisperList = new Vector();  // and whispered to

  // constructor
  public BRoomPanel(BChallengeRoomData data, BClientStack server, BChallengePanel parent)
  {
    super(BBoxPanel.X_AXIS);

    this.server = server;
    this.data = data;
    this.parent = parent;

    arenaCanvas.setSize(108, 80);

    control = new BRoomControl(this, server, data);

    PlayerListRenderer renderer = new PlayerListRenderer();
    playersList.setCellRenderer(renderer);

    BattleListRenderer renderer2 = new BattleListRenderer();
    battleList.setCellRenderer(renderer2);

    layoutRoomChildren();
    configRoomChildren();
  }

  public boolean isRunningBattle()
  {
    return cancelButton.isEnabled();
  }

  public void onActivate()
  {
    super.onActivate();

    titleLabel.setText("challenge room - " + data.roomName);
    titleLabel.setForeground(BGlobal.colors.getMidShade());

    roomInfoLabel.setText("");
    roomInfoLabel.append("Bot cost limit:\t" + Math.round(data.maxEntryCost) + "cred");
    roomInfoLabel.append("\tArena:\t" + data.arena);

    roomInfoLabel.append("\nFight time:\t" + Math.round(data.maxFightTime) + "s");

    String simAccuracy = "";
    if(data.simulationSpeed == BChallengeRoomData.SIMULATION_ACCURACY_LOW)
    {
      simAccuracy = "Low";
      BGlobal.setBattleAccuracy(BGlobal.BATTLE_ACCURACY_LOW);
    }
    if(data.simulationSpeed == BChallengeRoomData.SIMULATION_ACCURACY_MEDIUM)
    {
      simAccuracy = "Medium";
      BGlobal.setBattleAccuracy(BGlobal.BATTLE_ACCURACY_MEDIUM);
    }
    if(data.simulationSpeed == BChallengeRoomData.SIMULATION_ACCURACY_HIGH)
    {
      simAccuracy = "High";
      BGlobal.setBattleAccuracy(BGlobal.BATTLE_ACCURACY_HIGH);
    }
    roomInfoLabel.append("\tSim accuracy:\t" + simAccuracy);

    control.getPlayers();
    control.getBattles();
    control.startListener();

    roomActive = true;
  }

  public void onVisible()
  {
    messageField.requestFocus();
  }

  public void layoutRoomChildren()
  {
    BBoxPanel left    = new BBoxPanel(BBoxPanel.Y_AXIS);
    BBoxPanel right   = new BBoxPanel(BBoxPanel.Y_AXIS);
    //BBoxPanel buttons = new BBoxPanel(BBoxPanel.X_AXIS);

    BBoxPanel msgTools = new BBoxPanel(BBoxPanel.X_AXIS);
    BBoxPanel playerButtons = new BBoxPanel(BBoxPanel.X_AXIS);
    BBoxPanel top = new BBoxPanel(BBoxPanel.X_AXIS);
    BBoxPanel farRight = new BBoxPanel(BBoxPanel.Y_AXIS);
    BBoxPanel battleButtons = new BBoxPanel(BBoxPanel.X_AXIS);

    BBoxPanel info = new BBoxPanel(BBoxPanel.X_AXIS);

    playerButtons.add(blockButton);
    playerButtons.addSmallMargin();
    playerButtons.add(whispButton);
    playerButtons.addSmallMargin();
    playerButtons.add(challButton);
    playerButtons.addHGlue();

    top.addHGlued(roomInfoLabel);

    msgTools.add(whispModeButton);
    msgTools.addSmallMargin();
    msgTools.add(messageField);
    msgTools.addSmallMargin();
    msgTools.add(sendButton);

    battleButtons.add(cancelButton);
    battleButtons.addSmallMargin();
    battleButtons.add(playButton);

    info.add(infoLabel);
    info.add(statsButton);
    info.add(statsFiller);

    farRight.addMargin();
    farRight.addStrut(30);
    farRight.addLinear(farRight.getHGlue(), exitButton);
    farRight.addSmallMargin();
    farRight.add(battleLabel);
    farRight.addTinyMargin();
    farRight.add(battleScroll);
    farRight.addSmallMargin();
    farRight.add(info);
    farRight.addTinyMargin();
    farRight.add(infoScroll);
    farRight.addTinyMargin();
    farRight.add(battleButtons);
    farRight.addMargin();
    farRight.addStrut(1);

    left.addMargin();
    left.addHGlued(titleLabel);
    left.addSmallMargin();
    left.addHGlued(top);
    left.addHGlued(messageLabel);
    left.addTinyMargin();
    left.add(messageScroll);
    left.addTinyMargin();
    left.add(msgTools);
    left.addMargin();
    left.addStrut(1);

    right.addMargin();
    right.addStrut(70);

    right.add(battleViewLabel);
    right.addTinyMargin();
    right.add(viewScroll);
    right.addSmallMargin();

    right.addLinear(playersLabel, getHGlue());
    right.addTinyMargin();
    right.addLinear(playersScroll, getHGlue());
    right.addTinyMargin();

    right.add(playerButtons);
    right.addMargin();
    right.addStrut(1);

  //  addHStrut(50);
    addMargin();
    addLinear(getVStrut(85), refreshButton, getVGlue());
    addMargin();
    add(left);
    addMargin();
    add(right);
    addMargin();
    add(farRight);
    addHStrut(50);
  }

  public void configRoomChildren()
  {
    whispModeButton.setSelectedIcon(BIconCache.getHSB("room_whisper_icon.gif"));
    whispModeButton.addActionListener(this);
    messageArea.setBorder(null);
    sendButton.addActionListener(this);
    exitButton.addActionListener(this);
    messageField.addActionListener(this);
    messageField.setNextFocusableComponent(messageField);
    whispButton.addActionListener(this);
    playersList.addListSelectionListener(this);
    battleList.addListSelectionListener(this);
    challButton.addActionListener(this);
    cancelButton.addActionListener(this);
    playButton.addActionListener(this);
    blockButton.addActionListener(this);

    playButton.setEnabled(false);
    arenaCanvas.setEnabled(false);
    cancelButton.setEnabled(false);
    challButton.setEnabled(false);
    blockButton.setEnabled(false);
    whispButton.setEnabled(false);

    playAction.setEnabled(false);
    downloadAction.setEnabled(false);

    playButton.setAction(downloadAction);

    arenaCanvas.setVisible(false);  // hide the 3d canvas until a battle is playing
    statsButton.setVisible(false);
    statsButton.addActionListener(this);
    refreshButton.addActionListener(this);

    refreshTimer = new javax.swing.Timer(10000, this);
    refreshTimer.start();

  }

  // returns true if the room has been created and activated
  public boolean isRoomActive()
  {
    return roomActive;
  }

  protected void refreshRoom()
  {
        parent.tournaments.setStatus("Refreshing room details...");
        parent.tournaments.setWorking(true);
        control.getPlayers();
        control.getBattles();
        infoArea.setText("");
        refreshAllowed = false;
        parent.tournaments.setStatus("Refreshing room details complete.");
        parent.tournaments.setWorking(false);
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource() == refreshTimer)
    {
      refreshAllowed = true;
    }
    if(e.getSource() == refreshButton)
    {
      if(refreshAllowed)
      {

        BProcessor p = new BProcessor()
        {
          public void process()
          {
            refreshRoom();
          }
        };

        p.start();
      }


    }

    if(e.getSource() == sendButton)
    {
       sendMessage();
    }
    else if (e.getSource() == messageField)
    {
       sendMessage();
    }

    if(e.getSource() == exitButton)
      exitRoom(true);

    if(e.getSource() == whispModeButton)
    {
      if(whispModeButton.isSelected())
        whisperMode = true;
      else
        whisperMode = false;
    }

    if(e.getSource() == whispButton)
      setPlayerWhisper();

    if(e.getSource() == challButton)
      issueChallenge();

    if(e.getSource() == blockButton)
      setPlayerBlocked();

    if(e.getSource() == cancelButton)
      control.stopTheFight(true);

    if(e.getSource() == statsButton)
    {
      BProcessor p = new BProcessor()
      {
        public void process()
        { showPlayerStats(); }
      };
      p.start();
    }
  }

  /*
   // pop up the player ranking dialog or the selected player
  public void showPlayerStats()
  {
    BPlayerListData listData = (BPlayerListData)playersList.getSelectedValue();

    if(listData != null)
    {
      try
      {
        long accountID = listData.getMemberData().getID();
        String creatorName = listData.getMemberData().designerName;

        rankingDialog = new BPlayerRankingDialog(accountID, creatorName, parent.getServer());
        BApplication.showDialog(rankingDialog);
      }
      catch(Exception e)
      {
        BDebug.print ("BChallengePanel.BRoomControl.showPlayerStats error: " + e.getMessage() );

      }
    }
  }*/

  // pop up the player ranking dialog or the selected player
  public void showPlayerStats()
  {
    BPlayerListData listData = (BPlayerListData)playersList.getSelectedValue();

    if(listData != null)
    {
      try
      {
        long accountID = listData.getMemberData().getID();
        String creatorName = listData.getMemberData().designerName;

        rankingDialog = new BPlayerRankingDialog(accountID, creatorName, parent.getServer());
        BApplication.showDialog(rankingDialog);
      }
      catch(Exception e)
      {
        BDebug.print ("BChallengePanel.BRoomControl.showPlayerStats error: " + e.getMessage() );

      }
    }
  }

  // hide the dialog to prevent it from stopping incomming challenges
  public void hideStatsDialog()
  {
    if(rankingDialog != null)
    {
      BApplication.hideDialog(rankingDialog);
      rankingDialog.dispose();
    }
  }

  public void hideBotDialog()
  {
    if(botDialog != null)
    {
      BApplication.hideDialog(botDialog);
      botDialog.dispose();
    }
  }

  // list selection listener method
  public void valueChanged(ListSelectionEvent e)
  {
    if(e.getSource() == playersList)
    {
      if(playersList.getSelectedIndices().length != 1)
        challButton.setEnabled(false);
      else if ( ((BPlayerListData)playersList.getSelectedValue()).getMemberData().afk )
        challButton.setEnabled(false);
      else if ( ((BPlayerListData)playersList.getSelectedValue()).getMemberData().getID() == server.getAccountID().longValue() )
      {
        challButton.setEnabled(false);
        whispButton.setEnabled(false);
        blockButton.setEnabled(false);
      }
      else
      {
        challButton.setEnabled(true);
        whispButton.setEnabled(true);
        blockButton.setEnabled(true);
      }
      playAction.setEnabled(false);
      downloadAction.setEnabled(false);
      showPlayerInfo();
      battleList.removeListSelectionListener(this);
      battleList.clearSelection();
      battleList.addListSelectionListener(this);
    }
    else if(e.getSource() == battleList)
    {
      challButton.setEnabled(false);
      whispButton.setEnabled(false);
      blockButton.setEnabled(false);
      BCRTournamentData tData = (BCRTournamentData)battleList.getSelectedValue();
      if(tData != null)
      {
        if(tData.tournamentStatus.equals("FINISHED"))
        {
          control.fightID = tData.fightID;
          String fileName = getBattleFile();
          File f = new File(fileName);
          boolean fightExists = f.exists();
          if(fightExists)
          playButton.setAction(playAction);
          else
            playButton.setAction(downloadAction);

          if(!isRunningBattle())
          {
            playAction.setEnabled(true);
            downloadAction.setEnabled(true);
          }
          else
          {
            playAction.setEnabled(false);
            downloadAction.setEnabled(false);
          }
        }

        else
        {
          playAction.setEnabled(false);
          downloadAction.setEnabled(false);
        }
      }
      else
      {
        showBattleInfo(tData);
        playAction.setEnabled(false);
        downloadAction.setEnabled(false);
      }
      showBattleInfo(tData);
      playersList.removeListSelectionListener(this);
      playersList.clearSelection();
      playersList.addListSelectionListener(this);
    }
  }

  public void cleanUp()
  {
    refreshTimer.stop();
    arenaCanvas = null;
    botDialog = null;
    messageArea = null;
    messageScroll = null;
    messageField = null;
    sendButton = null;
    playersList = null;
    battleList = null;
    battleScroll = null;
    viewScroll = null;
    infoArea = null;
    infoScroll = null;
    playersScroll = null;
    messageLabel = null;
    playersLabel = null;
    battleLabel = null;
    infoLabel = null;
    battleViewLabel = null;
    titleLabel = null;
    roomInfoLabel = null;
    blockButton = null;
    whispButton = null;
    challButton = null;
    playButton = null;
    downloadAction = null;
    playAction = null;
    cancelButton = null;
    whispModeButton = null;
    exitButton = null;
    oldMessage = null;
    data = null;

    control.cleanUp();
  }

  // closing down
  public void onClose()
  {

    super.onClose();
    if( !downloadAction.isStopped() )
      downloadAction.stop();

    // busy wait until the download thread has stopped
    while ( !downloadAction.isStopped() )
    {
      try {
        wait((long)20);
      }catch (InterruptedException e) {}
    }
    exitRoom(false);
    roomActive = false;
  }

  // ask the controller to inform the server of the away from keyboard mode
  public void setAFK(boolean away)
  {
    control.setAFK(away);
  }

  // show the player description in the information area
  public void showPlayerInfo()
  {
    infoLabel.setText("player information");
    statsFiller.setVisible(false);
    statsButton.setVisible(true);
    BPlayerListData d = (BPlayerListData)playersList.getSelectedValue();
    if(d != null)
    {
      BCRMemberData member = d.getMemberData();
      infoArea.setText(BDictionaryTree.getSharedInstance().filter(member.designerBlurb));
    }
  }

  // show the battle description in the information area
  public void showBattleInfo(BCRTournamentData tData)
  {
    infoLabel.setText("battle information");
    statsButton.setVisible(false);
    statsFiller.setVisible(true);
    if(tData != null)
    {
      infoArea.setText("");
      infoArea.append("Creator: " + BDictionaryTree.getSharedInstance().filter(tData.challengerDesignerName) + "\n");
      infoArea.append("Bot:       " + BDictionaryTree.getSharedInstance().filter(tData.challengerBotName) + "\n\n");
      if(tData.challengedBotName.toLowerCase().equals("waiting for update"))
        infoArea.append("Creator: Waiting for Update\n");
      else
        infoArea.append("Creator: " + BDictionaryTree.getSharedInstance().filter(tData.challengedDesignerName) + "\n");
      infoArea.append("Bot:       " + BDictionaryTree.getSharedInstance().filter(tData.challengedBotName));
    }
  }

  // issue a challenge to another player in the room
  public void issueChallenge()
  {
    challenging = true;
    startBotSelect();
  }

  public void startBotSelect()
  {
    BProcessor p = new BProcessor()
    {
      public void process()
      { showBotSelectDialog(true, "", "", -1); }
    };

    p.start();
  }

  // show the dialog for bot selection - issuing is true if the challenge is being issued
  // and false if it is being received
  public void showBotSelectDialog(boolean issuing, String challengerName, String challengerBotName, long tournamentID)
  {
    if(issuing)
      botDialog = new BBotSelectDialog("Please select a bot", "Cancel", "Ok", issuing, -1);
    else
      botDialog = new BBotSelectDialog("You have been challenged by " + challengerName + " using " + challengerBotName + ". Please select a bot.", "Decline", "Accept", issuing, tournamentID);

    botDialog.requestFocus();
    BApplication.showDialog(botDialog);

  }


  // accessor methods for the blocked and whisper lists
  public Vector getWhisperList()
  {
    return whisperList;
  }

  public Vector getBlockedList()
  {
    return blockedList;
  }


  // sets the selected players to whisper or not
  public void setPlayerWhisper()
  {
    if(playersList.getSelectedIndex() >= 0)
    {
      Object [] selected = playersList.getSelectedValues();
      for(int i = 0; i < selected.length; i++)
      {
        if( ((BPlayerListData)selected[i]).isWhisper() )
        {
          ((BPlayerListData)selected[i]).setWhisper(false);
          removeWhisper( ((BPlayerListData)selected[i]).getMemberData().getID() );
        }
        else
        {
          ((BPlayerListData)selected[i]).setWhisper(true);
          whisperList.add( new Long(((BPlayerListData)selected[i]).getMemberData().getID()) );
        }
      }
      playersList.repaint();
    }
  }

  // sets the selected players to whisper or not
  public void setPlayerBlocked()
  {
    if(playersList.getSelectedIndex() >= 0)
    {
      Object [] selected = playersList.getSelectedValues();
      for(int i = 0; i < selected.length; i++)
      {
        if( ((BPlayerListData)selected[i]).isBlocked() )
        {
          ((BPlayerListData)selected[i]).setBlocked(false);
          removeBlocked( ((BPlayerListData)selected[i]).getMemberData().getID() );
        }
        else
        {
          ((BPlayerListData)selected[i]).setBlocked(true);
          blockedList.add( new Long(((BPlayerListData)selected[i]).getMemberData().getID()) );
        }
      }
      playersList.repaint();
    }
  }

  public void removeBlocked(long id)
  {
    if(blockedList.size() > 0)
    {
      for(int j = blockedList.size()-1; j >= 0; j--)
      {
        Long blockedId = (Long)blockedList.elementAt(j);

        if(blockedId.longValue() == id)
          blockedList.removeElementAt(j);
      }
    }
  }

  public void removeWhisper(long id)
  {
    if(whisperList.size() > 0)
    {
      for(int j = whisperList.size()-1; j >= 0; j--)
      {
        Long whisperId = (Long)whisperList.elementAt(j);

        if(whisperId.longValue() == id)
          whisperList.removeElementAt(j);
      }
    }
  }

  // fills the player list with the players currently in a room
  public void setPlayerList(Vector players)
  {
    playersList.setListData(players);
    playersList.clearSelection();
  }

  // fills the battle list with the battles fought and fighting
  public void setBattleList(Vector battles)
  {
    battleList.setListData(battles);
    battleList.clearSelection();
  }

  // sends a message throught the control to the server
  public void sendMessage()
  {
    BDebug.print("BChallengePanel - sending message");
    if ( messageField.getText().equals(oldMessage) )
    {
      messageArea.append("( ---Identical message detected, message NOT sent--- )\n");
    }

    if(messageField.getText().length() > 0 && !(messageField.getText().equals(oldMessage)) )
    {
      oldMessage = messageField.getText();
      // if the system is in whisperMode then send only to whispered players (also to yourself)
      if(!whisperMode)
      {
        BProcessor p = new BProcessor()
        {
          public void process()
          {  control.sendMessage(oldMessage, false, -1); }
        };
        p.start();
      }
      else
      {
        BProcessor p = new BProcessor()
        {
          public void process()
          {  sendWhisper(); }
        };
        p.start();
      }
    }
    messageField.setText("");
  }

  // sends a whisper message
  public void sendWhisper()
  {
    ListModel model = playersList.getModel();
    for(int i = 0; i < model.getSize(); i++)
    {
      if( ((BPlayerListData)model.getElementAt(i)).isWhisper()  || ( ((BPlayerListData)model.getElementAt(i)).getMemberData().getID() == server.getAccountID().longValue() ) )
      {
         control.sendMessage(oldMessage, true, ((BPlayerListData)model.getElementAt(i)).getMemberData().getID());
      }
    }
  }

  // tell the server and exit the room
  public void exitRoom(boolean battleCheck)
  {
    // if the cancel button is enabled it means a fight is being run
    if(isRunningBattle() && battleCheck)
    {
      BProcessor p = new BProcessor()
      {
        public void process()
        { showConfirmExit(); }
      };
      p.start();
    }
    else
    {
      if(isRunningBattle())
        control.stopTheFight(true);
//      if(monitor != null)
//        monitor.setStopped(true);
      roomActive = false;
      control.exitRoom();
      parent.exitRoom();
    }
  }

  // confirm exit of room
  public void showConfirmExit()
  {
    BPromptDialog exitCheck =
    BPromptDialog.createYesNoDialog("Exit room?",
         "You are running a battle - exiting this challenge room will cancel the battle. Do you want to exit?");

    BApplication.showDialog(exitCheck);
    boolean exitRoom = exitCheck.getResult();

    if(exitRoom)
    {
      control.stopTheFight(true);
//      if(monitor != null)
//        monitor.setStopped(true);
      roomActive = false;
      control.exitRoom();
      parent.exitRoom();
    }
  }
  // adds a message to the message area
  public void addMessage(String message)
  {
    if(message != null)
      if (!message.trim().equals(""))
        messageArea.append(message);
  }

  //////////////////////////////////////////////////////////////////////////////
  //
  //  inner class used for bot selection
  //
  protected class BBotSelectDialog extends BSystemDialog
  {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
	protected BotSelectClient client;
    protected boolean issuing = true;
    protected long tournamentID;

    protected BUIResources res = new BUIResources( new Object[]
    {
      "botLabel.text",         "Please select a bot:",

      "okButton.text",         "Ok",

      "cancelButton.text",     "Cancel",
    });

    protected BMultiLabel botLabel = new BMultiLabel(res, "botLabel");
    protected BFileComboBox botCombo = new BFileComboBox();

    protected BButton okButton = new BButton(res, "okButton");
    protected BButton cancelButton = new BButton(res, "cancelButton");

    public BBotSelectDialog(String message, String cancelMessage, String okMessage, boolean issuing, long tournamentID)
    {
      super("select a bot", false);
      setSize(new Dimension(250, 215));
      setLocation(275, 200);

      client = new BotSelectClient();
      setClient(client);

      botLabel.setText(BDictionaryTree.getSharedInstance().filter(message));
      cancelButton.setText(cancelMessage);
      okButton.setText(okMessage);

      addButton(cancelButton);
      addButton(okButton);

      this.issuing = issuing;
      this.tournamentID = tournamentID;

      modal = true;
      blocking = false;
    }

    protected class BotSelectClient extends SystemDialogClient implements ActionListener
    {
      /**
		 * 
		 */
		private static final long serialVersionUID = -1L;

	public BotSelectClient()
      {
        super(BoxLayout.Y_AXIS);
        layoutDialogChildren();
        configDialogChildren();
      }

      public void layoutDialogChildren()
      {
        BBoxPanel panel = new BBoxPanel(BoxLayout.Y_AXIS);

        panel.addGlued(botLabel);
        panel.addSmallMargin();
        panel.add(botCombo);
        add(panel);
        addMargin();
        addMargin();
      }

      public void configDialogChildren()
      {
        okButton.addActionListener(this);
        cancelButton.addActionListener(this);
        try
        {
          botCombo.setFilter(new BRoboforgeFilter(BGlobal.BOTSUFFIX));
          botCombo.setSourcePath(BGlobal.BOTPATH);

          if (!botCombo.isEmpty())
            botCombo.setSelectedIndex(0);
        }
        catch (IOException e)
        {}
      }

      public void actionPerformed(ActionEvent e)
      {
        if(e.getSource() == okButton)
        {
          parent.tournaments.setStatus("Verifying bot...");
          parent.tournaments.setBusyWorking(true);
          BBot bot = BBot.loadFromFile(botCombo.getSelectedFileWithPath(), false);
          BPlayerListData playerData = (BPlayerListData)playersList.getSelectedValue();

          // if the dialog is in issuing mode then issue the challenge
          if(issuing)
          {
            BDebug.print("BChallengePanel - about to test issue bot");
            if(verifyBot(bot))
            {
              BDebug.print("BChallengePanel - about to issue the challenge");
              if(playerData != null)
              {
                control.issueChallenge(server.getAccountID().longValue(), playerData.getMemberData().getID(), data.getID(), data.password, bot);
                parent.tournaments.setStatus("Bot verified...");
              }
              else
                parent.tournaments.setStatus("Player no longer in room...");
              parent.tournaments.setBusyWorking(false);
              BApplication.hideDialog(BBotSelectDialog.this);
              challenging = false;
              dispose();
            }
          }
          else
          {
            if(verifyBot(bot))
            {
              // if the verification was ok then acknowledge
              control.challengedBot = bot;
              control.sendAcknowledge(null, true, true, bot.getName(), tournamentID);
              // set the challeged bot to be the bot just selected
              parent.tournaments.setStatus("Bot verified...");
              parent.tournaments.setBusyWorking(false);
              BApplication.hideDialog(BBotSelectDialog.this);
              challenging = false;
              dispose();
            }
          }
        }
        if(e.getSource() == cancelButton)
        {
          if(!issuing)
            control.sendAcknowledge(null, false, false, null, tournamentID);

          BApplication.hideDialog(BBotSelectDialog.this);
          challenging = false;
          dispose();
        }
      }
    }

    Vector veriFyBotVIssues = new Vector();

    // call the control to verify the bot meets requirements
    public boolean verifyBot(BBot bot)
    {
      veriFyBotVIssues = new Vector();
      // if the bot is not valid - notify the player and reprompt for a bot
      if(!control.validateBot(bot, veriFyBotVIssues))
      {
        BDebug.print ("veriFyBotVIssues errors " + veriFyBotVIssues);
        BProcessor p = new BProcessor()
        {
          public void process()
          {
            BApplication.hideDialog(BBotSelectDialog.this);
            BDebug.print ("veriFyBotVIssues errors 2 " + veriFyBotVIssues);
            botValidError(veriFyBotVIssues);


          }
        };
        p.start();
        return false;
      }
      return true;
    }
    // show the dialog with the problem description
    public void botValidError(Vector errors)
    {
      BDebug.print ("botValidError errors " + errors);
      String sText = "";

      for (int i = 0 ;  i < errors.size() ; i++)
          sText = sText + errors.get(i) + "\n";


      sText =  "Your bot was rejected for the following reasons\n\n" +  sText;


      BPromptDialog botFail;
      botFail = BPromptDialog.createOKDialog("invalid bot", sText);
      BApplication.showDialog(botFail);
      BApplication.showDialog(BBotSelectDialog.this);
      parent.tournaments.setStatus("Verification failed...");
      parent.tournaments.setBusyWorking(false);
    }
  }

  //////////////////////////////////////////////////////////////////////////////
  //
  //  inner class for rendering the player list
  //
  class PlayerListRenderer extends JLabel implements ListCellRenderer
  {
    Icon norm  = BIconCache.getHSB("room_broadcast_icon.gif");
    Icon block = BIconCache.getHSB("room_blocked_icon.gif");
    Icon whisp = BIconCache.getHSB("room_whisper_icon.gif");

    public PlayerListRenderer()
    {
      setOpaque(true);
    }

    public Component getListCellRendererComponent(
    JList list,
    Object value,            // value to display
    int index,               // cell index
    boolean isSelected,      // is the cell selected
    boolean cellHasFocus)    // the list and the cell have the focus
    {
      BPlayerListData d = (BPlayerListData)value;

      Color awayB;            // colours for the away shade of grey
      Color awayF;

      String s = BDictionaryTree.getSharedInstance().filter(d.getMemberData().designerName);
      boolean away = d.getMemberData().afk;

      if(d.isWhisper())
        setIcon(whisp);
      else
        setIcon(norm);
      if(d.isBlocked())
        setIcon(block);

      if (isSelected)
      {
        setBackground(list.getSelectionBackground());
        setForeground(list.getSelectionForeground());
        awayB = list.getSelectionBackground();
        awayF = list.getSelectionForeground();
      }
      else
      {
        setBackground(list.getBackground());
        setForeground(list.getForeground());
        awayB = list.getBackground();
        awayF = list.getForeground();
      }

      if(d.getMemberData().proStatus != null)
      {
        if(d.getMemberData().proStatus.equals("M"))
        {
          setForeground(BGlobal.MASTER_COLOR);
          awayF = BGlobal.MASTER_COLOR;
        }
        else if(d.getMemberData().proStatus.equals("GM"))
        {
          setForeground(BGlobal.GRAND_MASTER_COLOR);
          awayF = BGlobal.GRAND_MASTER_COLOR;
        }
      }

      if(away)
      {
        if(isSelected)
          setBackground(awayB.darker().darker());
        setForeground(awayF.darker().darker());
        s = s + " (away)";
      }
      setText(s);

      setEnabled(list.isEnabled());
      setFont(list.getFont());
      return this;
    }
  }

  //////////////////////////////////////////////////////////////////////////////
  //
  //  inner class for rendering the battle list
  //
  class BattleListRenderer extends JLabel implements ListCellRenderer
  {

    Icon complete  = BIconCache.getHSB("room_complete_icon.gif");
    Icon waiting   = BIconCache.getHSB("room_waiting_icon.gif");
    Icon progress  = BIconCache.getHSB("room_progress_icon.gif");

    public BattleListRenderer()
    {
      setOpaque(true);
    }

    public Component getListCellRendererComponent(
     JList list,
     Object value,            // value to display
     int index,               // cell index
     boolean isSelected,      // is the cell selected
     boolean cellHasFocus)    // the list and the cell have the focus
    {
      BCRTournamentData d = (BCRTournamentData)value;

      String s = BDictionaryTree.getSharedInstance().filter(d.tournamentName);

      if(d.tournamentStatus.equals("FINISHED"))
      {
        setText(s);
        setIcon(complete);
      }
      if(d.tournamentStatus.equals("WAITING"))
      {
        setText(s);
        setIcon(waiting);
      }
      if(d.tournamentStatus.equals("FIGHTING"))
      {
        int percentC = (int)d.percentageComplete;

        if (percentC == 100)
          setText("(uploading) " + s);
        else
          setText("(" + (int)d.percentageComplete + "%) " + s);

        setIcon(progress);
      }
      //if(d.isPassworded)
      //  setIcon(pass);
      //else
      if (isSelected)
      {
        setBackground(list.getSelectionBackground());
        setForeground(list.getSelectionForeground());
      }
      else
      {
        setBackground(list.getBackground());
        setForeground(list.getForeground());
      }

      setEnabled(list.isEnabled());
      setFont(list.getFont());
      return this;
    }
  }

  //////////////////////////////////////////////////////////////////////////////
  //
  // inner class for downloading the fights
  //
  public class DownloadAction extends BThreadedAction
  {
    protected BMessage response;
    protected BTransmission transmission;

    // was the download stopped by the user ?
    protected boolean manualStop = false;

    public DownloadAction(BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
    {
      parent.tournaments.setStatus("Downloading battle...");
      parent.tournaments.setWorking(true);
    }

    public void threadedAction(ActionEvent e)
    {
      try{
      try
      {
        response = null;

        BRequest r = new BRequest(parent.server, "GBA");
        r.setData(new BIdentData(control.fightID));

        transmission = new BTransmission(r);
        parent.tournaments.setMonitored(transmission);

        response = parent.server.request(transmission);
      }
      catch (BNetException ex)
      {
        if(!manualStop)
          parent.tournaments.setStatus("ERROR: " + ex.getMessage());
        else
          parent.tournaments.setStatus("Download terminated.");
      }
      finally
        { parent.tournaments.setMonitored(null); }

      if (response != null)
      {
        if (response.isAcknowledgement())
          saveBattle((BFightRecordData) response.getData());
        else
        {
          if(!manualStop)
            parent.tournaments.setStatus("ERROR: " + response);
          else
            parent.tournaments.setStatus("Download terminated.");
        }
      }
      }
      catch(Exception ex)
      {
        BDebug.print("BChallengePanel " + ex.getMessage());
      }
    }

    public void swingCleanupAction(ActionEvent e)
    {
      if(parent != null)
      {
        parent.tournaments.setBusyWorking(false);
        //setActionStatus();
      }
    }

    protected void saveBattle(BFightRecordData data)
    {
      try
      {
        //cancelAction.setEnabled(false);
        parent.tournaments.setStatus("Saving battle to local cache...");

        parent.tournaments.setBusyWorking(true);

        String filename =
          BGlobal.BATTLERECORDINGCACHEPATH+ data.getID();

        data.save(filename);
        System.gc();


        parent.tournaments.setStatus("Battle download completed.");

        parent.tournaments.setBusyWorking(false);


        // start the battle player
        playButton.setAction(playAction);
        playButton.doClick();

      }
      catch (Exception e)
      {
         if(!manualStop)
          parent.tournaments.setStatus("ERROR: "+ e);
        else
          parent.tournaments.setStatus("Download terminated.");
      }
    }


    public boolean isStopped()
    {
      if(transmission != null)
        return transmission.isStopped();
      else
        return true;
    }

    // Terminate the download
    public void stop()
    {
      manualStop = true;          // the download has been interrupted by the user
      transmission.stop();   // stop the download thread
    }
  }

  //////////////////////////////////////////////////////////////////////////////
  //
  // inner class for playing a selected battle
  //
  protected class PlayAction extends BAction
  {
    public PlayAction(BUIResources res, String key)
    { super(res, key); }

    public void actionPerformed(ActionEvent e)
    {
      String fileName = getBattleFile();
      if (fileName != null)
        BBattleScreen.load(fileName);
    }
  }

  public void playFight()
  {
    String fileName = getBattleFile();
    if (fileName != null)
    BBattleScreen.load(fileName);
  }

  // returns the battle file associated with the fight
  protected String getBattleFile()
   { return BGlobal.BATTLERECORDINGCACHEPATH + control.fightID + ".rec"; }

}

//////////////////////////////////////////////////////////////////////////////
//
// class for storing player data - whisper details etc.
//
class BPlayerListData
{
  protected BCRMemberData memberData; // the mebmer data from the server
  protected boolean whisper = false;  // true to recieve whispered messages
  protected boolean blocked = false;  // true to be blocked

  public BPlayerListData(BCRMemberData memberData)
  {
    this.memberData = memberData;
  }

  // accessor methods
  public BCRMemberData getMemberData()
  { return memberData; }

  public boolean isWhisper()
  { return whisper; }

  public boolean isBlocked()
  { return blocked; }

  public void setWhisper(boolean whisper)
  { this.whisper = whisper; }

  public void setBlocked(boolean blocked)
  { this.blocked = blocked; }
}

////////////////////////////////////////////////////////////////////////////////
//
// Class BRoomControl
//
// Communicates with the server
//
class BRoomControl implements Runnable
{

  protected BRoomPanel room;  // the room user interface
  protected BClientStack server; // the server

  //protected BMonitorService monitor;  // going to be used to kill the thread on close

  protected BChallengeRoomData data; // information about the current room

  protected Thread listener; // listens for messages from the server

  protected boolean roomActive = true;

  // vector storing the players in the room
  protected Vector playerListVec = new Vector();

  // vector storing the battles
  protected Vector battleListVec = new Vector();

  // variables needed for the battle simulation
  protected String challengerName = "";
  protected String challengerBotName = "";
  long tournamentID;
  long fightID;

  protected BBot challengerBot = null;
  protected BBot challengedBot = null;

  // the thing that runs the fights on the client
  protected BChallengeFight fightRunner = null;
  // new room welcome chime
  protected transient BSound welcomeChime;
  
  public BRoomControl(BRoomPanel room, BClientStack server, BChallengeRoomData data)
  {
    this.room = room;
    this.server = server;
    this.data = data;
    listener = new Thread(this);
    listener.setPriority(Thread.MIN_PRIORITY);
    //MDF 23-10-06
    welcomeChime = BSound.getSound(BGlobal.SOUNDPATH + "/interface/8-5.wav", 0, BSound.SOUND_TYPE_FOREGROUND);
    
  }

  public void run()
  {
    try
    {
      while (roomActive)
      {
        BDebug.print("BChallengePanel - waiting for next message");
        room.addMessage(getNextMessage());
      }
    }
    catch (Exception e)
    {
      BDebug.print("BChallengePanel.BRoomControl.run() " + e.getMessage());
    }
  }

  // start the message listener 
  public void startListener()
  {
    listener.start();
  }

  // gets the next message or set of messages from the server
  private String getNextMessage()
  {
    String nextMessage = null;
      try
      {
        BMessage response = null;
        BRequest r = new BRequest(server, "GMES");

       // monitor = new BMonitorService();

        // send authentication data to the server along with the message request
        BCRAuthenticationData d = new BCRAuthenticationData(server.getAccountID().longValue());
        d.challengeRoomID = data.getID();
        d.password = data.password;

        r.setData(d);

        // Get the response back from the server...
      //  response = server.request(r, monitor, monitor);
        BTransmission t = new BTransmission(r);
        response = server.request(t);

        //response = server.request(r,monitor);
      //  BDebug.print("BChallengePanel pos1");

        BDataSet dataSet = null;

        if(response == null)
        {
//        BDebug.print("BChallengePanel get next message response:" +response.getData());
          BDebug.print("BChallengePanel get next message error: response is null" );
          room.parent.setStatus("BChallengePanel get next message error: response is null" );
          //forceExitRoom();
        }

        if(response.getData() == null)
        {
          room.parent.setStatus("BChallengePanel getNextMessage error: null data" );
          BDebug.print("BChallengePanel get next message error: null data");
         // forceExitRoom();
        }
        if(response.getData() instanceof BErrorData)
        {
          BDebug.print("BChallengePanel get next message error: ERRORDATA");
          BDebug.print("BChallengePanel get next message error:" +response.getData());
          room.parent.setStatus("BChallengePanel get next message error:" +response.getData() );
          //forceExitRoom();
        }


        if(response.getData() instanceof BDataSet)
          dataSet= (BDataSet)response.getData();

        BDebug.print("BChallengePanel - message length = " + dataSet.data.length);

        for (int i = 0; i < dataSet.data.length; i++)
        {
          if(dataSet.data[i] == null)
          {
            //BDebug.print("BChallengePanel ***********null data***********");
            room.parent.setStatus("BChallengePanel ***********null data***********");
            //forceExitRoom();
          }
          BDebug.print("BChallengePanel - *** messageData *** = " + dataSet.data[i]);

          ///////// normal text message //////////
          if (((BAbstractAtom)dataSet.data[i]).getType() == BDataTypes.MESSAGE)
          {

            boolean senderBlocked = false;
            if(playerListVec.size() > 0)
            {
              for(int j = 0; j < playerListVec.size(); j++)
              {
                BPlayerListData listData = (BPlayerListData)playerListVec.elementAt(j);
                BDebug.print("BChallengePanel list ID = " + listData.getMemberData().getID() + ", sender ID = " + ((BMessageData)dataSet.data[i]).senderAccountID);

                if(listData.getMemberData().getID() == ((BMessageData)dataSet.data[i]).senderAccountID)
                  if(listData.isBlocked())
                    senderBlocked = true;
              }
            }

            if(!senderBlocked)
            {
              String m = BDictionaryTree.getSharedInstance().filter(((BMessageData)dataSet.data[i]).message);

              if  (nextMessage == null)
              {
                nextMessage = BDictionaryTree.getSharedInstance().filter(((BMessageData)dataSet.data[i]).designerName) + ": " + m ;
              }
              else
                nextMessage += "\r\n" + BDictionaryTree.getSharedInstance().filter(((BMessageData)dataSet.data[i]).designerName) + ": " + m ;
            }
          }

          ///////// enter room message ///////////
          else if (((BAbstractAtom)dataSet.data[i]).getType() == BDataTypes.CR_ENTER_ROOM)
          {
        	  welcomeChime.reset();
        	  welcomeChime.play();
            BEnterRoomData enterData = (BEnterRoomData)dataSet.data[i];

            if (nextMessage == null)
              nextMessage = BDictionaryTree.getSharedInstance().filter(enterData.designerName) + " has entered the room.";
            else
              nextMessage += "\r\n" + BDictionaryTree.getSharedInstance().filter(enterData.designerName) + " has entered the room.";

            BCRMemberData newPlayer = new BCRMemberData(enterData.getID());
                          newPlayer.designerName = enterData.designerName;
                          newPlayer.designerBlurb = enterData.designerBlurb;
                          newPlayer.proStatus = enterData.proStatus;

            boolean exists = false;
            if(playerListVec.size() > 0)
            {
              for(int j = 0; j < playerListVec.size(); j++)
              {
                BPlayerListData listData = (BPlayerListData)playerListVec.elementAt(j);

                if(listData.getMemberData().getID() == newPlayer.getID())
                  exists = true;
              }
            }

            BPlayerListData newData = new BPlayerListData( newPlayer );

            if( isBlocked(newPlayer.getID()) )
              newData.setBlocked(true);
            if( isWhisper(newPlayer.getID()) )
              newData.setWhisper(true);

            if(!exists)
              playerListVec.add( newData );

            room.setPlayerList(playerListVec);
          }

          ///////// exit message //////////
          else if (((BAbstractAtom)dataSet.data[i]).getType() == BDataTypes.CR_EXIT_ROOM)
          {
            BDebug.print("BChallengePanel - player has exited the room");
            BExitRoomData exitData = (BExitRoomData)dataSet.data[i];
            long id = exitData.getID();
            for(int j = 0; j < playerListVec.size(); j++)
            {
              BPlayerListData listData = (BPlayerListData)playerListVec.elementAt(j);
              if(listData.getMemberData().getID() == id)
              {
                if (nextMessage == null)
                  nextMessage = BDictionaryTree.getSharedInstance().filter(listData.getMemberData().designerName) + " has left the room.";
                else
                  nextMessage += "\r\n" + BDictionaryTree.getSharedInstance().filter(listData.getMemberData().designerName) + " has left the room.";
                BDebug.print("BChallengePanel - player has been removed from the list");
                playerListVec.remove(listData);
                room.setPlayerList(playerListVec);
                break;
              }
            }
          }

          ///////// test alive message - ignore this /////////
          else if (((BAbstractAtom)dataSet.data[i]).getType() == BDataTypes.CR_TEST_ALIVE)
          {
            BDebug.print("BChallengePanel - test alive message");
            //nextMessage = "";
          }

          ///////// challenge message ////////////////////////
          else if (((BAbstractAtom)dataSet.data[i]).getType() == BDataTypes.CR_ACCEPT_CHALLENGE)
          {
            BDebug.print("BChallengePanel - Challenge received ");
            BAcceptChallengeData challData = (BAcceptChallengeData)dataSet.data[i];
            BDebug.print("BChallengePanel - challengeData set ");
            BBot challBot = challData.getBot();
            BDebug.print("BChallengePanel - bot retrieved ");
            Vector validationIssues = new Vector();

            boolean senderBlocked = false;
            BDebug.print("BChallengePanel - about to serarch for blocked player ");
            if(playerListVec.size() > 0)
            {
              for(int j = 0; j < playerListVec.size(); j++)
              {
                BPlayerListData listData = (BPlayerListData)playerListVec.elementAt(j);
                BDebug.print("BChallengePanel list ID = " + listData.getMemberData().getID() + ", sender ID = " + challData.getID());

                if(listData.getMemberData().getID() == challData.getID())
                  if(listData.isBlocked())
                    senderBlocked = true;
              }
            }
            BDebug.print("BChallengePanel - blocked search complete ");
            if(senderBlocked || room.challenging == true)
              sendAcknowledge(validationIssues, false, false, challBot.getName(), challData.tournamentID);

            else if(!validateBot(challBot, validationIssues))
              sendAcknowledge(validationIssues, false, false, challBot.getName(), challData.tournamentID);
            else if (room.challenging == false) {
              challengerBotName = challBot.getName();
              challengerName = challData.challengerDesignerName;
              tournamentID = challData.tournamentID;
              challengerBot = challBot;
              room.hideStatsDialog();
              startBotSelect();
            }
            else if(room.challenging == true)
              sendAcknowledge(validationIssues, false, false, challBot.getName(), challData.tournamentID);
          }

          ///////// tournament update message - refresh the status of the fight list /////////
          else if (((BAbstractAtom)dataSet.data[i]).getType() == BDataTypes.CR_TOURNAMENT_UPDATE)
          {
            BDebug.print("BChallengePanel battles update received");
            BTournamentUpdateData updateData = (BTournamentUpdateData)dataSet.data[i];
            updateBattleList(updateData);
          }

          ///////// afk status changed message /////////
          else if (((BAbstractAtom)dataSet.data[i]).getType() == BDataTypes.CR_AFK)
          {
            BDebug.print("BChallengePanel afk status change received");
            BAFKData afkData = (BAFKData)dataSet.data[i];
            long id = afkData.getID();
            for(int j = 0; j < playerListVec.size(); j++)
            {
              BPlayerListData listData = (BPlayerListData)playerListVec.elementAt(j);
              if(listData.getMemberData().getID() == id)
               listData.getMemberData().afk = afkData.afk;
            }
            room.setPlayerList(playerListVec);
          }
        }
      }
      catch (Exception e)
      {
        BDebug.print("BChallengePanel (BRoomControl - getNextMessage) " + e.getMessage());
       // tournaments.setStatus("BChallengePanel (BRoomControl - getNextMessage) " + e.getMessage());
        //forceExitRoom();
      }

      //XXXXX
      if (nextMessage != null)
        if ( !nextMessage.equals("") )
           nextMessage = nextMessage + "\n";

      return nextMessage ;
  }


  // by looking at the list of blocked players in the room's blocked list,
  // this returns true or false if an id is blocked or not
  public boolean isBlocked(long id)
  {
    Vector blockedList = room.getBlockedList();
    if(blockedList.size() > 0)
    {
      for(int j = 0; j < blockedList.size(); j++)
      {
        Long blockedId = (Long)blockedList.elementAt(j);

        if(blockedId.longValue() == id)
          return true;
      }
    }
    return false;
  }
  // by looking at the list of whisper players in the room's whisper list,
  // this returns true or false if an id is in your whisper group
  public boolean isWhisper(long id)
  {
    Vector whisperList = room.getWhisperList();
    if(whisperList.size() > 0)
    {
      for(int j = 0; j < whisperList.size(); j++)
      {
        Long whisperId = (Long)whisperList.elementAt(j);

        if(whisperId.longValue() == id)
          return true;
      }
    }
    return false;
  }


  // force the player from the room and notify them
  public void forceExitRoom()
  {
    if(roomActive)
    {
      BProcessor p = new BProcessor()
      {
        public void process()
        { showForcedExit(); }
      };
      p.start();
      room.exitRoom(false);
    }
  }

  // show the logged out message if the attempt to set the afk to false fails
  public void showForcedExit()
  {
    room.hideStatsDialog();
    room.hideBotDialog();
    BPromptDialog forced;
    forced = BPromptDialog.createOKDialog("logged out", "You were logged out of the challenge room because of a data error.");
    BApplication.showDialog(forced);
  }

  // updates the list of battles
  public void updateBattleList(BTournamentUpdateData updateData)
  {
    BDebug.print("BChallengePanel battles update - updating the list");
    // find the fight in the existing list if it exists - update the entry
    if(updateData.status == updateData.UPDATED)
    {
      BDebug.print("BChallengePanel battles update UPDATED");

      boolean found = false;

      for(int i = 0; i < battleListVec.size(); i++)
      {
        BCRTournamentData tData = (BCRTournamentData)battleListVec.elementAt(i);
        if(tData.getID() == updateData.getID())
        {
          found = true;
          tData.tournamentStatus = updateData.tournamentStatus;
          tData.challengerBotName = updateData.challengerBotName;
          tData.challengedBotName = updateData.challengedBotName;
          tData.fightID = updateData.fightID;
          if(tData.tournamentStatus.equals("FIGHTING"))
            tData.percentageComplete = updateData.fightProgress;
        }
      }
      //if the battle was not found - add it to the list
      if(!found)
      {
        BCRTournamentData addFight =
        new BCRTournamentData(updateData.getID(), updateData.tournamentName,
                            updateData.tournamentStatus, updateData.fightProgress,
                            updateData.challengerID.longValue(), updateData.challengedID.longValue(),
                            updateData.challengerDesignerName, updateData.challengedDesignerName,
                            updateData.fightID );
        addFight.challengerBotName = updateData.challengerBotName;
        addFight.challengedBotName = updateData.challengedBotName;
        battleListVec.add(addFight);
      }
    }
    else if(updateData.status == updateData.ADDED)
    {
      BDebug.print("BChallengePanel battles update ADDED");
      BCRTournamentData addFight =
      new BCRTournamentData(updateData.getID(), updateData.tournamentName,
                            updateData.tournamentStatus, updateData.fightProgress,
                            updateData.challengerID.longValue(), updateData.challengedID.longValue(),
                            updateData.challengerDesignerName, updateData.challengedDesignerName,
                            updateData.fightID );
      addFight.challengerBotName = updateData.challengerBotName;
      addFight.challengedBotName = updateData.challengedBotName;
      battleListVec.add(addFight);
    }
    else if(updateData.status == updateData.DELETED)
    {
      BDebug.print("BChallengePanel battles update DELETED");
      for(int i = 0; i < battleListVec.size(); i++)
      {
        BCRTournamentData tData = (BCRTournamentData)battleListVec.elementAt(i);
        if(tData.fightID == updateData.fightID)
        {
          battleListVec.remove(tData);
          break;
        }
      }
    }
    room.setBattleList(battleListVec);
  }

  // pop up the dialog for bot selection
  public void startBotSelect()
  {
    BProcessor p = new BProcessor()
    {
      public void process()
      { room.showBotSelectDialog(false, challengerName, challengerBotName, tournamentID); }
    };

    p.start();
  }

  // make sure a bot is within certain boundrys
  public boolean validateBot(BBot bot, Vector validationIssues)
  {
    // Init the bot

    // The Old Way
    Vector validationIssuesL = bot.validate(true);

    // Check to see if the bot is the correct cost for the tournament
    if (bot.getTotalPrice() > data.maxEntryCost)
    {
      // The bot is over priced for the tournament
      validationIssuesL.add("Bot is Overpriced");
    }

    if ( !validationIssuesL.isEmpty() )
      validationIssues.addAll(validationIssuesL);

    //BDebug.print ("BChallengePanel validateBot validationIssues " +  validationIssues);
    return (validationIssuesL.isEmpty());

  }

  // gets the list of current inhabitants from the server
  public void getPlayers()
  {
      try
      {
        BMessage response = null;
        BRequest r = new BRequest(server, "GMEM");

        // send authentication data to the server along with the message request
        BCRAuthenticationData d = new BCRAuthenticationData(server.getAccountID().longValue());
        d.challengeRoomID = data.getID();
        d.password = data.password;

        r.setData(d);

        // Get the response back from the server...
        BTransmission t = new BTransmission(r);
        response = server.request(t);
        //response = server.request(r,monitor);

        BDataSet dataSet = (BDataSet)response.getData();

        playerListVec = new Vector();
        for (int i = 0; i < dataSet.data.length;i++)
        {
          if (((BAbstractAtom)dataSet.data[i]).getType() == BDataTypes.CR_MEMBER)
          {
            BPlayerListData newData = new BPlayerListData( ((BCRMemberData)dataSet.data[i]) );

            if( isBlocked(((BCRMemberData)dataSet.data[i]).getID()) )
              newData.setBlocked(true);
            else
              newData.setBlocked(false);
            if( isWhisper(((BCRMemberData)dataSet.data[i]).getID()) )
              newData.setWhisper(true);
            else
              newData.setWhisper(false);

            playerListVec.add( newData );
          }
        }
        room.setPlayerList(playerListVec);
      }
      catch (Exception e)
      {
        BDebug.print("BChallengePanel (BRoomControl) - get players " + e.getMessage());
      }
  }

  // gets the list of battles (both in progress and recently completed) from the server
  public void getBattles()
  {
      try
      {
        BMessage response = null;
        BRequest r = new BRequest(server, "GTRN");

        // send authentication data to the server along with the message request
        BCRAuthenticationData d = new BCRAuthenticationData(server.getAccountID().longValue());
        d.challengeRoomID = data.getID();
        d.password = data.password;

        r.setData(d);

        // Get the response back from the server...
        BTransmission t = new BTransmission(r);
        response = server.request(t);

        BDebug.print("BChallengePanel - getBattle response = " + response.getData());

        BDataSet dataSet = (BDataSet)response.getData();

        battleListVec = new Vector();
        if(dataSet.data != null)
        {
          for (int i = 0; i < dataSet.data.length;i++)
          {
            if (((BAbstractAtom)dataSet.data[i]).getType() == BDataTypes.CR_TOURNAMENT)
            {
              battleListVec.add( (BCRTournamentData)dataSet.data[i] );
            }
          }
        }
        room.setBattleList(battleListVec);
      }
      catch (Exception e)
      {
        BDebug.print("BChallengePanel (BRoomControl) - getBattles - " + e.getMessage());
      }
  }

  // sends a message to the other players
  public synchronized void sendMessage(String message, boolean whisper, long accountID)
  {
    try
    {
      BMessage response = null;
      BRequest r = new BRequest(server, "SMES");

      BMessageData messageData = new BMessageData(-1);
      if(whisper)
        messageData.message = "(whispered) " + message;
      else
        messageData.message = message;

      BCRWrapperData wrapperData;

      if(whisper)
      {
        wrapperData = new BCRWrapperData(accountID);
        wrapperData.destination = BCRWrapperData.TO_ACCOUNT;
      }
      else
      {
        wrapperData = new BCRWrapperData(data.getID());
        wrapperData.destination = BCRWrapperData.SPECIFIED_CHALLENGE_ROOM;
      }

      wrapperData.challengeData = messageData;
      wrapperData.accountID = server.getAccountID();
      wrapperData.password = data.password;
      wrapperData.challengeRoomID = new Long(data.getID());

      r.setData(wrapperData);

      // Get the response back from the server...
      BTransmission t = new BTransmission(r);
      response = server.request(t);

      if (response.isAcknowledgement())
        BDebug.print("BChallengePanel - sendMessage ACK");
      else
      {
        BDebug.print("BChallengePanel - sendMessage NAK");
        BDebug.print("BChallengePanel - message - send error = " + response.getData());
      }
    }
    catch (Exception e1)
    {
      BDebug.print("BChallengePanel.BRoomControl.sendMessage() " + e1.getMessage());
    }
  }

  // sends an acknowledement of a fight challenge to the server
  BStartFightData startData;
  public void sendAcknowledge(Vector validationIssues, boolean valid, boolean accepted, String botName, long tournamentID)
  {
    try
    {
      BMessage response = null;
      BRequest r = new BRequest(server, "ACHA");

      BDebug.print("BChallengePanel (sendAck) accountID = " + server.getAccountID().longValue());
      

      BAcknowledgeChallengeData ackData = new BAcknowledgeChallengeData(tournamentID , data.getID(),
                                                                        botName, accepted, valid,
                                                                        validationIssues, data.password, server.getAccountID().longValue());

      r.setData(ackData);

      // Get the response back from the server...
      BTransmission t = new BTransmission(r);
      t.setSendService(BServices.zip());

      response = server.request(t);

      if (response.isAcknowledgement())
        BDebug.print("BChallengePanel - sendAck ACK");
      else
      {
        BDebug.print("BChallengePanel - sendAck NAK");
        BDebug.print("BChallengePanel - send ACK - send error = " + response.getData());
      }

      if( ((BAbstractAtom)response.getData()).getType() == BDataTypes.CR_START_FIGHT )
      {
        // the ack was acked and all is well - set the bots up ready to fight
        startData = (BStartFightData)response.getData();
        BProcessor p = new BProcessor()
        {
          public void process()
          {  startTheFight(startData);  }
        };
        p.start();
      }
    }
    catch (Exception e1)
    {
      BDebug.print("BChallengePanel (BRoomControl) - sendAcknowledge - " + e1.getMessage());
    }
  }

  // tell the server if user is away or not
  public void setAFK(boolean away)
  {
      try
      {

        BMessage response = null;
        BRequest r = new BRequest(server, "AFK");

        // send authentication data to the server (includes the AFK boolean)
        BCRAuthenticationData d = new BCRAuthenticationData(server.getAccountID().longValue());
        d.challengeRoomID = data.getID();
        d.password = data.password;
        d.afk = away; // away or not

        r.setData(d);

        // Get the response back from the server...
        BTransmission t = new BTransmission(r);
        response = server.request(t);

        if (response.isAcknowledgement())
        {
          BDebug.print("ACK - afk set: " + away);

          // if the user is leaving the room then stop the message polling thread
          if(away)
          {
            BDebug.print("BChallengeRoom - stopping listener");
            roomActive = false;
          }
          else
          {
            getPlayers();
            getBattles();

            roomActive = true;
            BDebug.print("BChallengeRoom - starting listener");
            listener = new Thread(this);
            listener.setPriority(Thread.MIN_PRIORITY);
            listener.start();
          }
        }
        else
        {
          BDebug.print("NAK");
          BDebug.print("BChallengePanel - setAFK error = " + response.getData());
          if(away == false)
          {
            BProcessor p = new BProcessor()
            {
            public void process()
              { showLoggedOutMessage(); }
            };
            p.start();
            room.parent.exitRoom();
          }
        }
      }
      catch (Exception e)
      {
        BDebug.print("BChallengePanel (BRoomControl) - setAFK  " + e.getMessage());
      }
  }

  // show the logged out message if the attempt to set the afk to false fails
  public void showLoggedOutMessage()
  {
    room.hideStatsDialog();
    room.hideBotDialog();
    BPromptDialog logged;
    logged = BPromptDialog.createOKDialog("logged out", "You were away for too long and have been logged out of the challenge room.");
    BApplication.showDialog(logged);
  }

  // using the start fight data returned from the server, begin a client side fight
  public void startTheFight(BStartFightData startData)
  {
    room.parent.setPanelButtonsEnabled(false);
    challengerBot.setCreatorName(startData.challengerName);
    challengerBot.setBotSerialID(startData.challengerTEID + "");

    challengedBot.setCreatorName(startData.challengedName);
    challengedBot.setBotSerialID(startData.challengedTEID + "");

    String arenaFileName = BStrings.convertToFilename(data.arena);

    String tournamentName = startData.challengerName + " vs " + startData.challengedName;

    room.arenaCanvas.setVisible(true);
    fightRunner = new BChallengeFight(this, tournamentName, startData.getID(), startData.challengerTEID,
                                      startData.challengedTEID, challengerBot,
                                      challengedBot, arenaFileName, data.maxFightTime,
                                      room.arenaCanvas, server.getUserName(), server.getPassword());

    room.cancelButton.setEnabled(true);
  }

  // called when the client running the fight presses cancel
  public void stopTheFight(boolean stopRunner)
  {
    room.cancelButton.setEnabled(false);

    if(stopRunner)
      fightRunner.stopFight();

    cancelBattle();
    cleanAfterFight();
  }

  public void cleanAfterFight()
  {
    room.parent.setPanelButtonsEnabled(true);
    room.arenaCanvas.setVisible(false);
  }

  public void disableCancelButton()
  {
    room.cancelButton.setEnabled(false);
  }

  // cancels a battle that is in progress
  public void cancelBattle()
  {
      try
      {
        BMessage response = null;
        BRequest r = new BRequest(server, "CCHAL");

        // send authentication data to the server (containing tournID)
        BCRAuthenticationData d = new BCRAuthenticationData(server.getAccountID().longValue());
        d.challengeRoomID = data.getID();
        d.tournamentID = tournamentID;
        d.password = data.password;

        r.setData(d);

        // Get the response back from the server...
        BTransmission t = new BTransmission(r);
        response = server.request(t);

        if (response.isAcknowledgement())
          BDebug.print("BChallengePanel - cancelBattle ACK");
        else
        {
          BDebug.print("NAK");
          BDebug.print("BChallengePanel - cancelBattle error = " + response.getData());
        }
      }
      catch (Exception e)
      {
        BDebug.print("BChallengePanel (BRoomControl) - cancelBattle - " + e.getMessage());
      }
  }

  // issues a challenge from this player to the selected player
  String challengeReject = "";
  public void issueChallenge(long challengerID, long challengedID, long roomID, String password, BBot bot)
  {
    BDebug.print("BChallengePanel - issuing challenge ");
    try
    {
      BMessage response = null;
      BRequest r = new BRequest(server, "CHAL");

      BCreateChallengeData createData = new BCreateChallengeData(challengerID,
                                                                 bot, password, roomID,
                                                                 challengedID);

      r.setData(createData);

      BTransmission t = new BTransmission(r);
      t.setSendService(BServices.zip());

      // Get the response back from the server...
      response = server.request(t);

      if (response.isAcknowledgement())
        BDebug.print("ACK - challenge issued");
      else
      {
        BDebug.print("NAK");
        BDebug.print("BChallengePanel - issue challenge - send error = " + response.getData());
        challengeReject = response.getData().toString();
        BProcessor p = new BProcessor()
        {
          public void process()
          { showChallengeReject(challengeReject); }
        };
        p.start();

      }
    }
    catch (Exception e1)
    {
      BDebug.print("BChallengePanel (BRoomControl.issueChallenge()) " + e1.getMessage());
    }
  }

  // show the challenge rejected message
  public void showChallengeReject(String rejectMessage)
  {
    room.hideStatsDialog();
    BPromptDialog reject;
    reject = BPromptDialog.createOKDialog("challenge rejected", rejectMessage);
    BApplication.showDialog(reject);
  }

  // log out of a challenge room
  public void exitRoom()
  {
    try
    {
      roomActive = false;
      //mdf
      welcomeChime.reset();
      welcomeChime=null;
    //  if(monitor != null)
    //    monitor.setStopped(true);
      BMessage response = null;
      BRequest r = new BRequest(server, "EXCR");

      BCRAuthenticationData d = new BCRAuthenticationData(server.getAccountID().longValue());
      d.challengeRoomID = data.getID();
      d.password = data.password;

      r.setData(d);

      // Get the response back from the server...
      BTransmission t = new BTransmission(r);
      response = server.request(t);

      if (response.isAcknowledgement())
        BDebug.print("BChallengePanel - exitRoom ACK");
      else
        BDebug.print("BChallengePanel - exitRoom NAK");

    }
    catch (Exception e1)
    {
      BDebug.print("BChallengePanel (exitRoom) " + e1.getMessage());
    }
  }

  public void cleanUp()
  {
    battleListVec = null;
    playerListVec = null;
    data = null;
    challengerName = null;
    challengerBotName = null;
    challengerBot = null;
    challengedBot = null;
    fightRunner = null;
  }


}
