package roboforge.ui.net;

import roboforge.net.client.*;
import roboforge.net.data.*;
import roboforge.util.data.*;
import roboforge.BGlobal;

import roboforge.ui.components.*;
//import roboforge.ui.components.tree.*;

import java.util.*;
import java.io.*;
import javax.swing.tree.*;



public class BHistoryTree extends BTree implements BDatabase
{

  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;

protected BDataNode root = 
    new BDataNode("Completed Tournaments");
  
  protected BClientStack server;

  protected DefaultTreeModel  model;

  protected TournamentStore   tournamentStore;
  protected EntryStore        entryStore;
  protected FightStore        fightStore;


  public BHistoryTree()
    { this(null, null); }

  public BHistoryTree(BUIResources res, String key)
  {
    super(res, key);

    model = new DefaultTreeModel(root);
    setModel(model);

    tournamentStore = new TournamentStore();
    entryStore      = new EntryStore();
    fightStore      = new FightStore();

   // BDelegatingCellRenderer renderer =
   //   new BDelegatingCellRenderer();
    BHistoryTreeCellRenderer renderer = new BHistoryTreeCellRenderer();

    setCellRenderer(renderer);
  }

  public void setServer(BClientStack server)
    { this.server = server; }

  public void store(BData d)
  {
    tournamentStore.clear();
    entryStore.clear();

    d.store(this);
    model.reload();
  }


  public BDataStore get(int type)
  {
    switch (type)
    {
      case BDataTypes.TOURNAMENT:
        return tournamentStore;

      case BDataTypes.TOURNAMENT_ENTRY:
        return entryStore;

      case BDataTypes.FIGHT:
        return fightStore;

      default:
        return null;
    }
  }



  protected class TournamentStore implements BDataStore
  {
    protected HashMap tournaments
      = new HashMap();

    public void clear()
    {
      tournaments.clear();
      root.removeAllChildren();

      root.add(new BDataNode("All Time Greats"));
    }

    public void put(BDataAtom datum)
    {
      BTournamentData d =
        (BTournamentData) datum;

      BDataNode node =
        new BDataNode(d);

      node.add(new BDataNode("My Results"));
      node.add(new BDataNode("Finals"));
      node.add(new BDataNode("Featured Fights"));

      tournaments.put(d.getIDLong(), node);
      root.insert(node, model.getChildCount(root) - 1);
      //root.insert(node, 0);
    }

    public BDataNode get(Long id)
      { return (BDataNode) tournaments.get(id); }

  }


  protected class EntryStore implements BDataStore
  {
    protected HashMap entries
      = new HashMap();
    
    public void clear()
      { entries.clear(); }

    public void put(BDataAtom datum)
    {
      BTournamentEntryData d = 
        (BTournamentEntryData) datum;
      
      BDataNode node = 
        new BDataNode(d);
      
      BDataNode tourn =
        tournamentStore.get(d.tournamentID);          

      if (tourn != null)
      {
        if (d.accountID.equals(server.getAccountID()))
          get(tourn, 0).add(node);
        else
          get(tourn, 1).add(node);

        entries.put(d.getIDLong(), node);
      }
    }

    public BDataNode get(Long id)
      { return (BDataNode) entries.get(id); }

    protected BDataNode get(BDataNode n, int i)
      { return (BDataNode) n.getChildAt(i); }
  }


  protected class FightStore implements BDataStore
  {
    public void put(BDataAtom datum)
    {
      BFightData d =
        (BFightData) datum;

      boolean r1 =
        register(d, d.entry01ID);

      boolean r2 =
        register(d, d.entry02ID);


      if(  tournamentStore.get(d.tournamentID) != null)
      {
        BTournamentData tourn = (BTournamentData) tournamentStore.get(d.tournamentID).getData();
        int finalRound = tourn.currentRound;

        if (!(r1 || r2)  && tourn.statusCode == BTournamentData.COMPLETE)
          registerOther(d, finalRound);
        else if ( (d.round >= finalRound -2) && tourn.statusCode == BTournamentData.COMPLETE) // put the finals and semi finals in.
          registerOther(d, finalRound);
      }
      registerFeaturedFights(d);

    }

    protected boolean register(BFightData d, Long ID)
    {
      BDataNode e =
        entryStore.get(ID);

      BDataNode tourn =
        tournamentStore.get(d.tournamentID);

      BFightDataNode node =
        new BFightDataNode(d);

      File f = new File(getBattleFile(d));
      boolean fightIsDownloaded = f.exists();
      node.setDownloaded(fightIsDownloaded);

      if(testForNominated(d))
        node.setNominated(true);

      if (e != null)
      {
        e.add(node);
       // setFeaturedFights(d, node, tourn);
      }
      return (e != null);
    }

    protected void registerOther(BFightData d, int finalRound)
    {
      BDataNode tourn =
        tournamentStore.get(d.tournamentID);

      BFightDataNode node =
        new BFightDataNode(d);

      File f = new File(getBattleFile(d));
      boolean fightIsDownloaded = f.exists();
      node.setDownloaded(fightIsDownloaded);

      if(testForNominated(d))
        node.setNominated(true);

      if (tourn != null)
      {
        if(d.round >= finalRound - 2)
          get(tourn, 1).add(node);
//        setFeaturedFights(d, node, tourn);
      }
    }

    protected void registerFeaturedFights(BFightData d)
    {
      BDataNode tourn =
        tournamentStore.get(d.tournamentID);

      BFightDataNode node =
        new BFightDataNode(d);

      File f = new File(getBattleFile(d));
      boolean fightIsDownloaded = f.exists();
      node.setDownloaded(fightIsDownloaded);

      if(testForNominated(d))
        node.setNominated(true);

      long features [] = d.featuredFightTypes;
      if(features != null)
      {
        for (int i = 0; i < features.length; i++)
        {
          if(features[i] == BFeaturedFightTypeData.FEATURED && tourn != null)
            get(tourn, 2).add((BFightDataNode)node.clone());
          if(features[i] == BFeaturedFightTypeData.ALL_TIME_GREAT)
            get(root, model.getChildCount(root) - 1).add((BFightDataNode)node.clone());
        }
      }
    }

    protected boolean testForNominated(BFightData d)
    {
      if(d.featuredFightTypes != null)
        return true;
      else
        return false;
    }

    protected BDataNode get(BDataNode n, int i)
      { return (BDataNode)n.getChildAt(i); }
    }

    protected String getBattleFile(BFightData fight)
      { return BGlobal.BATTLERECORDINGCACHEPATH+ fight.getID()+ ".rec"; }


}