package roboforge.util.graphics;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.util.text.*;
import roboforge.ui.components.HSBImageIcon;

import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;
import java.io.File;
import java.util.*;

import javax.swing.*;


public class BGraphics
{

  // Class variables...
  // -------------------------------------------------------------

  protected final static Toolkit toolkit
    = Toolkit.getDefaultToolkit();
    // Toolkit (for loading images.)

  protected final static Component component
    = new Component() {};
    // Dummy component for MediaTracker.

  protected final static MediaTracker tracker
    = new MediaTracker(component);
    // For ensuring images get fully loaded.



  // Image methods...
  // -------------------------------------------------------------

  public static Image getImage(String name)
  {

   try {
    // Check for file existence...
    File file = new File(name);
    if (!file.exists()) return null;

    // Load image...
    Image image = toolkit.createImage(name);

    // Wait for loading to complete...
    synchronized(tracker)
    {
	    tracker.addImage(image, 0);

      try
        { tracker.waitForID(0, 5000); }
      catch (InterruptedException e)
        {	}

	    tracker.removeImage(image, 0);
    }

    return image;
   } catch (Exception ex) {
    BDebug.print("BGraphics.getImage Exception: "+ex);
   }

   return null;

  }


  public static Image getUIImage(String name)
    { return getImage(getUIName(name)); }


  public static java.util.List getImageSet(String root, int digits, String ext)
  {
    // Loads all images with names beginning with 'root' and
    // ending in a numeric ID then extension 'ext'.
    // e.g. ui/images/pic001.jpg, ui/images/pic002.jpg...

    BDebug.print("BGraphics.getImageSet("+ root+ ", "+ ext+ ", "+ digits+ ")");


    String file, num, padded;
    int current     = 0;
    boolean isDone  = false;

    Image img;
    ArrayList imgs = new ArrayList();

    while (!isDone)
    {
      num     = Integer.toString(current++);
      padded  = BStrings.padLeftWithChar(num, '0', digits);
      file    = root + padded + ext;
      img     = getImage(file);
      isDone  = (img == null);

      BDebug.print("file: "+ file+", FOUND="+ (!isDone));

      if (!isDone) imgs.add(img);
    }

    imgs.trimToSize();
    return imgs;
  }


  public static BufferedImage scale(BufferedImage source, int w, int h)
  {
    int type  = source.getType();
    double sw = source.getWidth();
    double sh = source.getHeight();

    BufferedImage result =
      new BufferedImage(w, h, type);

    AffineTransform scale =
      new AffineTransform();

    scale.scale(
      w / sw,
      h / sh);

    AffineTransformOp filter =
      new AffineTransformOp(scale, AffineTransformOp.TYPE_BILINEAR);

    filter.filter(source, result);

    return result;
  }


  public static void copyImage(Image src, Image dest)
  {
    Graphics g = dest.getGraphics();
    g.drawImage(src, 0, 0, null);
  }
  

  // Cursor methods...
  // -------------------------------------------------------------

  public static Cursor getCursor(String name, int x, int y)
  {
    Image i = getUIImage(name);
    return toolkit.createCustomCursor(i, new Point(x, y), "");
  }

  public static Cursor getCursor()
    { return Cursor.getDefaultCursor(); }
    

  // Icon methods...
  // -------------------------------------------------------------

  public static ImageIcon getIcon(String name)
    { return new ImageIcon(getUIName(name)); }

  public static HSBImageIcon getHSBIcon(String name)
    { return new HSBImageIcon(getUIName(name)); }

  public static HSBImageIcon getHSBIcon(String name, Color c)
    { return new HSBImageIcon(getUIName(name), c); }



  // Implementation methods...
  // -------------------------------------------------------------

  protected static String getUIName(String name)
    { return BGlobal.IMAGEPATH+ "ui/"+ name; }


}
