package roboforge.util.processors;

import roboforge.BProgressMonitorable;
import java.util.Observable;



public abstract class BTask extends Observable
  implements Runnable, BProgressMonitorable
{

  // Member variables...
  // -------------------------------------------------------------

  protected Thread  thread;

  protected boolean isRunning;
  protected boolean isComplete;

  protected String  progressMessage;
  protected int     progressPercent;


  // Constructor...
  // -------------------------------------------------------------

  public BTask()
  {
    super();

    isRunning  = false;
    isComplete = false;
  }


  // Implementation methods...
  // -------------------------------------------------------------

  protected void updateObservers()
  {
    setChanged();
    notifyObservers();
  }


  // Public methods...
  // -------------------------------------------------------------

  public synchronized void start()
  {
    if (isRunning)
      return;

    thread = new Thread(this);
    thread.setPriority(Thread.MIN_PRIORITY);

    isRunning  = true;
    isComplete = false;

    thread.start();
    updateObservers();
  }

  public synchronized void startAndWait()
  {
    start();

    try
      { wait(); }
    catch (InterruptedException e)
      { }
  }


  public final void run()
  {
    progressPercent = 0;
    progressMessage = null;

    try
      { perform(); }
    finally
    {
      synchronized (this)
      {
        isRunning  = false;
        isComplete = true;

        thread = null;
        notifyAll();
      }

      updateObservers();
    }
  }

  /*
  public synchronized void stop()
  {
    if (isRunning)
      thread.interrupt();
  }

  public void setPriority(int priority)
  {
    synchronized (thread)
      { thread.setPriority(priority); }
  }
  */


  public synchronized boolean isRunning()
    { return isRunning; }

  public synchronized boolean isComplete()
    { return isComplete; }


  public String getProgressMessage()
    { return progressMessage; }

  public int getProgressPercent()
    { return progressPercent; }



  // Public methods...
  // -------------------------------------------------------------

  public abstract void perform();
    // Perform the task.

}
