package roboforge;

import java.awt.event.*;
import javax.swing.*;
import java.awt.*;
import javax.swing.event.*;
//import com.sun.j3d.utils.behaviors.mouse.MouseZoom;
import java.io.*;

import roboforge.ui.components.*;
//import roboforge.ui.*;

//import javax.swing.border.*;
//import javax.swing.plaf.metal.*;

/*

* Description:
  Creates a dialog box which allows a user to select from
  a list of textures.
  Returns a texture file name.
  Use the static method getTextureName to create the dialog.
  It needs to be passed the parent JFrame object.

* Example:  String element = BTextureDialog.getTextureName(this);
* Modification History ( Name, Date, Notes )
  HSC, 13 April 1999
  YLY, Copied Texture Dialog and converted to Apopearance Dialog.
  YLY, Changed for new analogy of setting the selected Appearance on a bot Canvas
  AWH, UI modifieid to use new classes - sliders fixed up

*/

public class BAppearanceDialog extends JDialog
{

  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;

// resources
  protected BUIResources res = new BUIResources( new Object[]
  {
    "shineSlider.min",                 new Integer(0),
    "shineSlider.max",                 new Integer(128),

    "depthSlider.min",                 new Integer(1),
    "depthSlider.max",                 new Integer(100),

    "transSlider.min",                 new Integer(0),
    "transSlider.max",                 new Integer(100),

    "shineLabel.text",                 "Shine",
    "shineLabel.width",                new Integer(40),

    "depthLabel.text",                 "Depth",
    "depthLabel.width",                new Integer(40),

    "transLabel.text",                 "Trans",
    "transLabel.width",                new Integer(40),

  });

  // SIZE CONSTANTS
  private static int WIDTH = 400;
  private static int HEIGHT = 500;
  private static int TEXTURE_WIDTH = 157;
  private static int BUTTONY = 40;
  private static int CANVAS_HEIGHT = 200;
  private static int COLORBUTTON_SIZE = 30;
  private static int GAP = 10;
  private static int SLIDER_HEIGHT = 30;

  private static int PICKER_HEIGHT =  130;
  //private static int PICKER_WIDTH =  240;

  // CURRENTLY SELECTED STUFF
  private String selectedTexture;
  private Color selectedColor;
  private float selectedShine;
  private float selectedTrans;
  private int selectedColourDepth;

  // SUB SECTIONS
  private BTextureList textureList = new BTextureList();
  private BComponentCanvas appCanvas;
  private BComponent theComponent;
  private JPanel controlPanel= new JPanel();
  private roboforge.ui.components.BSlider depthSlider;

  // BBotCanvas
  private BBotCanvas parent;
  private BComponent oldComponent;

  // Labels
  private BLabel shineLabel = new BLabel(res, "shineLabel");
  private BLabel depthLabel = new BLabel(res, "depthLabel");
  private BLabel transLabel = new BLabel(res, "transLabel");

  private boolean textureChanged = false;

  public BAppearanceDialog(JFrame frame, BComponent newComponent, BBotCanvas parentCanvas )
  {
    super(frame,"AppearancePicker",true);

    parent = parentCanvas;

    // This
    this.setResizable(false);
    this.getContentPane().setLayout(null);
    this.setSize(WIDTH,HEIGHT+36);    // Canvas and component
    appCanvas = new BComponentCanvas(null);
    appCanvas.setSize(WIDTH - TEXTURE_WIDTH, CANVAS_HEIGHT);

    oldComponent = newComponent;
    theComponent =  (BComponent)newComponent.clone();
    if (parent != null)
    {
        theComponent.setAppearance(parent.getSelectedTextureName(), parent.getSelectedColor(), parent.getSelectedShine(), parent.getSelectedColorDepth());

         // Setting the selects
        selectedTexture = parent.getSelectedTextureName();
        BDebug.print("BAppearance Dialog - parent texture = " + selectedTexture);
        selectedShine = parent.getSelectedShine();
        selectedColor = parent.getSelectedColor();
        selectedColourDepth = parent.getSelectedColorDepth();
    }
    else if(newComponent != null)
    {
       selectedShine = newComponent.getShine();
       selectedColor = newComponent.getColor();
       selectedColourDepth = newComponent.getColorDepth();

       BDebug.print("BAppearance Dialog - texture = " + selectedTexture);

    }
    else
    {
       selectedShine = 1;
       selectedColor = Color.white;
       selectedColourDepth = 50;
    }

    if(selectedColor == null)
      selectedColor = Color.white;

    appCanvas.loadComponent(theComponent, false);

    final BColorPicker bColorPicker = new BColorPicker(selectedColor);
    bColorPicker.setSize(WIDTH - TEXTURE_WIDTH ,PICKER_HEIGHT+36);
    bColorPicker.setLocation(0,300);
    bColorPicker.addActionListener( new ActionListener()
    {
      public void actionPerformed(ActionEvent ev)
        {

                theComponent.setColor(bColorPicker.getSelectedColor());
               // depthSlider.setColor(bColorPicker.getSelectedColor());


        } // action performed
    });

    final JComboBox cbDirectory = new JComboBox();
    cbDirectory.setSize(TEXTURE_WIDTH - 7 ,20);
    cbDirectory.setLocation(WIDTH - TEXTURE_WIDTH,0);
    cbDirectory.addActionListener( new ActionListener()
        {
            public void actionPerformed(ActionEvent ev)
            {
                 textureList.loadDir(BGlobal.TEXTUREPATH + "/" + cbDirectory.getSelectedItem());
            } // action performed
        }
    );

    // Load the textures
    File file = new File(BGlobal.TEXTUREPATH);
    File fileInternal = new File(BGlobal.TEXTUREPATH + "/internal");
    String[] titles = file.list(new BFileNameFilter());

    String ext[] = {"null"};

    File[] internalTitles = fileInternal.listFiles(new BFileFilter(ext, "", true));

    for(int i = 0;i< titles.length; i++)
    {
      //if (! titles[i].equalsIgnoreCase("internal"))
        cbDirectory.addItem(titles[i]);
    }

    for(int i = 0;i< internalTitles.length; i++)
    {
      //if (! titles[i].equalsIgnoreCase("internal"))
        cbDirectory.addItem("internal/" + internalTitles[i].getName());
    }

    // texturePane
    JScrollPane scrollPane = new JScrollPane(textureList);
    scrollPane.setSize(TEXTURE_WIDTH - 7 ,HEIGHT - 10);
    scrollPane.setLocation(WIDTH - TEXTURE_WIDTH,20);

    //Control Panel
    controlPanel.setLayout(null);
    controlPanel.setSize(WIDTH - TEXTURE_WIDTH, HEIGHT- CANVAS_HEIGHT - BUTTONY);
    controlPanel.setLocation(0,CANVAS_HEIGHT);

    //Image sliderIcon =  getToolkit().createImage(BGlobal.IMAGEPATH + "sliderKnob.gif" );

    // Shine Slider
    final roboforge.ui.components.BSlider shineSlider =
          new roboforge.ui.components.BSlider(res, "shineSlider");
          shineSlider.setValue(java.lang.Math.min( java.lang.Math.abs( 120 - ((int)selectedShine * 10 ) ), 120));

    shineSlider.addChangeListener(new ChangeListener()
    {
        public void stateChanged(ChangeEvent e)
        {

         try{


               selectedShine =  java.lang.Math.abs(  128 -(float)shineSlider.getValue()  ) / 10 ;

               // selectedShine =  java.lang.Math.abs(  128 -(float)shineSlider.getValue()  )  ;
               selectedColor = bColorPicker.getSelectedColor();
               theComponent.setColor( selectedColor, selectedShine, selectedColourDepth  );

             }catch (Exception ex){BDebug.print("BApperanceDialog.shineSlider.actionPerformed ERROR :" + ex.getMessage() );}


        }

    });
    shineLabel.setBounds(0, (COLORBUTTON_SIZE ) + (GAP* 2) + (SLIDER_HEIGHT )-35, 40, SLIDER_HEIGHT - 10);
    shineSlider.setBounds(40, (COLORBUTTON_SIZE ) + (GAP* 2) + (SLIDER_HEIGHT )-30, WIDTH - TEXTURE_WIDTH - 40, SLIDER_HEIGHT - 10);
    controlPanel.add(shineSlider);
    controlPanel.add(shineLabel);

   // DO NOT LET THE USER CHOOSE 0 for COLOUR DEPTH.
   // 0 is the default value for old components ,

   depthSlider = new roboforge.ui.components.BSlider(res, "depthSlider");
   depthSlider.setValue(selectedColourDepth);

   depthSlider.addChangeListener(new ChangeListener()
   {
      public void stateChanged(ChangeEvent e)
      {
         try{
           selectedColor = bColorPicker.getSelectedColor();
           selectedColourDepth = depthSlider.getValue();
           theComponent.setColor( selectedColor, selectedShine, selectedColourDepth );
         }catch (Exception ex){BDebug.print("BApperanceDialog.depthSlider.actionPerformed ERROR :" + ex.getMessage() );}
       }
    });
    depthLabel.setBounds(0, (COLORBUTTON_SIZE ) + (GAP* 2)- 30, 40, SLIDER_HEIGHT - 10);
    depthSlider.setBounds(40, (COLORBUTTON_SIZE ) + (GAP* 2)- 25, WIDTH - TEXTURE_WIDTH - 40, SLIDER_HEIGHT - 10);
    controlPanel.add(depthSlider);
    controlPanel.add(depthLabel);

    selectedTrans = theComponent.getTransparency() * 100;

    // Transparency Slider
    final roboforge.ui.components.BSlider transSlider =
          new roboforge.ui.components.BSlider(res, "transSlider");

    transSlider.setValue((int)selectedTrans);

    transSlider.addChangeListener(new ChangeListener()
    {
        public void stateChanged(ChangeEvent e)
        {
         float value = (float)transSlider.getValue();

         // cap value at 2.
         // set trans ro 0, to avoid the graphic glitches
         if (value < 2 )
                selectedTrans = 0;
         else
                selectedTrans = value / 100;
         theComponent.setTransparency( selectedTrans );
       }

    });
    transLabel.setBounds(0, (COLORBUTTON_SIZE ) + (GAP* 2) + (SLIDER_HEIGHT * 2)-40, 40, SLIDER_HEIGHT - 10);
    transSlider.setBounds(40, (COLORBUTTON_SIZE ) + (GAP* 2) + (SLIDER_HEIGHT * 2)-35, WIDTH - TEXTURE_WIDTH - 40, SLIDER_HEIGHT - 10);
    controlPanel.add(transSlider);
    controlPanel.add(transLabel);

    textureList.addMouseListener(new MouseAdapter()
    {
      public void mouseClicked(MouseEvent e)
      {
          textureChanged = true;
          selectedTexture = (String)textureList.getModel().getElementAt(textureList.getSelectedIndex());
          selectedTexture = selectedTexture.trim();

          if (selectedTexture.equals("No Texture"))
          {
            //BDebug.print("noTexture");
            theComponent.setTexture(null);
            selectedTexture = null;
          }
          else
            theComponent.setTexture(cbDirectory.getSelectedItem() + "/" +selectedTexture);
      }
   });

   // Apply Button
   JButton applyButton = new JButton("Apply");
   applyButton.setSize(( WIDTH - TEXTURE_WIDTH) /2 ,BUTTONY);
   applyButton.setLocation(0,HEIGHT - BUTTONY - 27 + 36);
   applyButton.addActionListener(new ActionListener()
   {
         public void actionPerformed(ActionEvent ev)
        {
                selectedColor = bColorPicker.getSelectedColor();

                if (parent != null)
                {
                        parent.setSelectedColor(selectedColor);

                        if (selectedTexture == null)
                                parent.setSelectedTextureName(null);
                        else if (selectedTexture.equals("No Texture"))
                                parent.setSelectedTextureName(null);
                        else if (textureChanged)
                                parent.setSelectedTextureName(cbDirectory.getSelectedItem() + "/" +selectedTexture);

                        //parent.setSelectedTextureName(cbDirectory.getSelectedItem() + "/" +selectedTexture);
                        parent.setSelectedShine(selectedShine);
                        parent.setSelectedColorDepth(selectedColourDepth);
                }
                if (oldComponent != null)
                {
                        oldComponent.setColor( selectedColor, selectedShine, selectedColourDepth  );
                        oldComponent.setTransparency( selectedTrans );

                        BDebug.print("BAppearanceDialog - selected texture = " + selectedTexture);

                        //if (selectedTexture == null)
                        //        oldComponent.setTexture(null);
                        if(selectedTexture != null)
                        {
                          if (selectedTexture.equals("No Texture"))
                             oldComponent.setTexture(null);
                          else
                             oldComponent.setTexture(cbDirectory.getSelectedItem() + "/" +selectedTexture);
                        }
                        oldComponent.setNormalAppearance();
                }
             BAppearanceDialog.this.setVisible(false);
             BAppearanceDialog.this.dispose();
        }
    });

   JButton cancelButton = new JButton("Cancel");
   cancelButton.setSize(( WIDTH - TEXTURE_WIDTH) /2 ,BUTTONY);
   cancelButton.setLocation(( WIDTH - TEXTURE_WIDTH) /2,HEIGHT - BUTTONY- 27 + 36);
   cancelButton.addActionListener(new ActionListener()
   {

         public void actionPerformed(ActionEvent ev)
        {
                BAppearanceDialog.this.setVisible(false);
                BAppearanceDialog.this.dispose();

        }


    });

    this.getContentPane().add(bColorPicker);
    this.getContentPane().add(scrollPane);
    this.getContentPane().add(applyButton);
    this.getContentPane().add(appCanvas);
    this.getContentPane().add(cancelButton);
    this.getContentPane().add(controlPanel);
    this.getContentPane().add(cbDirectory);

    this.setVisible(true);
    frame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
  }






  static public void start(JFrame f, BBotCanvas parentCanvas)
  {
    //.parentCanvas.setCursor(new Cursor(Cursor.WAIT_CURSOR));
    //BComponent viewer;
    //if (parentCanvas == null)
    //    viewer = new BComponent("Test", BGlobal.DEFAULT_APP_OBJECT);
    //else if (parentCanvas.getLastSelected() != null)
    //    viewer =  ((BShape3D)parentCanvas.getLastSelected()).getComponent();
    //else
    //    viewer = new BComponent("Test", BGlobal.DEFAULT_APP_OBJECT);

    //BAppearanceDialog dialog = new BAppearanceDialog(f, viewer, parentCanvas);


  }

  static public void main(String[] args)
  {
        BGlobal.init();
        JFrame frameStubb = new JFrame()
        {
          /**
			 * 
			 */
			private static final long serialVersionUID = -1L;

		protected void processWindowEvent(WindowEvent e)
          {
                super.processWindowEvent(e);
                if (e.getID() == WindowEvent.WINDOW_CLOSING)
                {
                        System.exit(0);
                }
          }
        };
        BTextures.setListener(frameStubb);
        start (new JFrame(), null);
  }

  //private String getValue()
  //{
  //  return selectedTexture;
  //}
}