package roboforge;

import java.util.*;
import java.awt.*;
import java.io.*;
//import javax.swing.tree.*;
//import javax.vecmath.*;
//import javax.media.j3d.*;


import roboforge.animation.*;


public class BArena implements Serializable, BAnimation
{
/*

* Description:
  TBA

* Example:
* Modification History ( Name, Date, Notes )

  Created HSC, 21 May.

  SJS 20/02/2001
    - Added Virtual Ref. member [refereePath].
    - Added victory sound member [victorySound].

*/

  static final long serialVersionUID = -1L;

  public static final double WALL_HEIGHT = 114.415; // In Canvas Units

  public static int DEFAULT_ARENA_SIZE = 100;

  static transient BComponent arenaComponent = new BComponent("Arena")
                                     {
                                       static final long serialVersionUID = -6472306058713497478L;

                                       public int getHealthCurrent()
                                          { return 100000000; }

                                       public boolean getDestroyed()
                                          { return false; }

                                     };

  // members
  private int arenaSize;  // diameter
  private BArenaObjects arenaObjects;

  private String name;
  private String fileName;

  private float floorShine;
  private float floorTrans;
  private String floorTexture;
  private Color floorColor;
  private int floorColourDepth;

  // attributes that only have a  set and get functions, so well make them friendsly
  public String floorObjectName;

  public Color AmbientColor;
  public Color directionalColor;
  public float dirX;
  public float dirY;
  public float dirZ;


  private Color directionalColor2;
  private float dirX2;
  private float dirY2;
  private float dirZ2;

  public String backgroundFileName;
  public boolean useFog = false;
  public Color fogColor;
  public float fogDensity;

  public String ambientSoundName = null;
  public String randomSoundName = null;
  public String winSoundName = null;
  public String drawSoundName = null;

  private float cameraX = 0f;
  private float cameraZ = 10f;
  private float cameraY = 10f;

  protected String refereePath;
    // specifies which folder to use for this arena's ref.

  protected String victorySound;
    // specifies which victory sound to use for this arena.


  // transient members
  public transient BSimulation simulation;


  // SJS 22/02/2001 - Added a compound animation to the arena,
  // so that its components can be animated.

  private transient BAnimationSet animations;


  public BArena()
  {
    arenaSize = DEFAULT_ARENA_SIZE;
    arenaObjects = new BArenaObjects();

    arenaComponent.damageMax = 1000000;
  }



  public void setSimulation(BSimulation simulationIn)
  {
    simulation = simulationIn;
  }

  public BSimulation getSimulation ()
  {
    return simulation;
  }

  public void setCameraPos (float x, float y, float z )
  {
      cameraX = x;
      cameraZ = z;
      cameraY = y;
  }

  public String getRandomSoundName()
    { return randomSoundName; }

  public String getAmbientSoundName()
    { return ambientSoundName; }

  public String getDrawSoundName()
    { return drawSoundName; }

  public String getWinSoundName()
    { return winSoundName; }

  public Color getAmbientColor()
    { return AmbientColor; }

  public Color getDirectionalColor()
    { return directionalColor; }

  public float getCameraX()
    { return cameraX; }

  public float getCameraZ()
    { return cameraZ; }

  public float getCameraY()
    { return cameraY; }


  public BArenaObjects getArenaObjects()
    { return arenaObjects; }

  public String getBackgroundFileName()
    { return backgroundFileName; }


  public BArenaObjects getArenaObjects(boolean getPlaceHolders)
  {


    BArenaObjects v = new BArenaObjects();

    // Do the loop, compare and add thing
    for (ListIterator x = arenaObjects.listIterator();x.hasNext();)
    {
      Object obj = x.next();
      if (obj instanceof BPlaceHolder && getPlaceHolders)
      {
        // Add it to the vector
        v.addElement(obj);
      }
      else if (!(obj instanceof BPlaceHolder) && !getPlaceHolders)
      {
        v.addElement(obj);
      }
    }
    return v;

  }

  public void addArenaObject (BArenaObject arenaObject)
  {
    // Adds a placeholder for an arenaObject to the Arena
    try
    {
      arenaObjects.addElement(arenaObject);
    }
    catch (Exception e)
    {
      BDebug.print(e.toString());
    }
  }

  public void addArenaObject (BPlaceHolder placeHolder,BArenaObject arenaObject)
  {
    // If the passed placeholder exists then replace with the arenaObject
    arenaObjects.replacePlaceholder(placeHolder,arenaObject);
  }

  public Vector getPlaceHolders (String placeHolderType)
  {
  /*
  Loop through all the elements in ArenaObjects adding
  all the elements of type BPlaceHolder that are of the required
  placeHolder type to a Vector.
  */

    BArenaObjects v = new BArenaObjects();

    // Do the loop, compare and add thing
    for (ListIterator x = arenaObjects.listIterator();x.hasNext();)
    {
      Object obj = x.next();
      if (obj instanceof BPlaceHolder)
      {
        if ((((BPlaceHolder)obj).getType()).equals(placeHolderType))
        {
          // Add it to the vector
          v.addElement(obj);

        }
      }
    }
    return v;
  }

  /*
  public BArenaStructure3D getArenaStructure3D()
  {
    return structure3D;
  }

  public void setArenaStructure3D(BArenaStructure3D arenaStructure3DIn)
  {
    structure3D = arenaStructure3DIn;
  } */

  public void removeArenaObject (BArenaObject arenaObject)
  {
    arenaObjects.remove(arenaObject);
  }

  public String getFileName ()
  {
    return fileName;
  }

  public void saveToFile (String filename) throws IOException
  {
        if (  ! BGlobal.ARENASUFFIX.equalsIgnoreCase(     filename.substring(filename.length() - 4 )  )  )
                filename = filename +  BGlobal.ARENASUFFIX;




        fileName = filename.substring( BGlobal.ARENAPATH.length() );
        BDebug.print(fileName);
        FileOutputStream fout = new FileOutputStream(filename);
        ObjectOutputStream out = new ObjectOutputStream(fout);
        out.writeObject(this);
        out.close();
        System.out.println(filename +"Save completed");

  }

  public static final String DEFAULT_ARENA = "roboforge_lab.are";

  public static BArena loadFromFile (String filename, boolean init)
  {

    File fileChecker = new File(filename);
    if (  !fileChecker.exists() )
    {
      BDebug.print ("BArena " + filename + " does not exist..... loading lab " );
      filename = BGlobal.ARENAPATH +  DEFAULT_ARENA;
    }

    BArena loadedArena = null;


    try
    {
      FileInputStream fin = new FileInputStream(filename);
      ObjectInputStream in = new ObjectInputStream(fin);
      loadedArena = (BArena)in.readObject();
      in.close();
      fin.close();

      if (init)
        loadedArena.init();
    }
    catch (Exception e)
    {
      e.printStackTrace(System.out);
      System.out.println("Arena Load failed");
    }

    return loadedArena;
  }


  public void init()
    { init(false); }

  public void init(boolean initArenaDetails)
  // Only call init separately if, the init boolean was false when arena was created
  {
    animations = new BAnimationSet();

    // Init all objects
    for (ListIterator x = this.getArenaObjects().listIterator();x.hasNext();)
    {

      BArenaObject o = (BArenaObject) x.next();
      o.init();

      if (initArenaDetails)
        o.initArenaDetails(this);
    }
  }

  public void setName (String nameIn)
  {
    name = nameIn;
  }

  public String getName ()
  {
    return name;
  }

  public void setFloorDetails(float floorShineIn, float floorTransIn, Color floorColorIn, String floorTextureIn)
  {
    floorShine = floorShineIn;
    floorTrans = floorTransIn;
    floorColor = floorColorIn;
    floorTexture = floorTextureIn;
  }

  public String getFloorObjectName()
    { return floorObjectName; }

  public float getfloorShine()
    { return floorShine; }

  public int getFloorColorDepth()
    { return floorColourDepth; }

  public float getfloorTrans()
    { return floorTrans; }

  public Color getfloorColor()
    { return floorColor; }

  public String getfloorTexture()
    { return floorTexture; }

  public int getSize()  // diameter
    { return arenaSize; }

  public float getDirX()
    { return dirX; }

  public float getDirY()
    { return dirY; }

  public float getDirZ()
    { return dirZ; }

  public Color getDirectionalColor2()
  { return directionalColor2; }

  public float getDirX2()
  { return dirX2 ;}

  public float getDirY2()
  { return dirY2; }

  public float getDirZ2()
  { return dirZ2; }

  public void setDirectionalColor2(Color c)
  { directionalColor2 = c; }

  public void setDirX2(float x)
  { dirX2  = x;}

  public void setDirY2(float y )
  { dirY2 = y ;}

  public void setDirZ2(float z)
  { dirZ2 = z; }



  public void setSize(int sizeIn)
  {
    arenaSize = sizeIn;
  }




  public void setRefereePath(String path)
    { refereePath = path; }

  public String getRefereePath()
    { return (refereePath == null) ? "standard" : refereePath; }
    // TODO: Should path default to 'standard' like this?

  public String getRefereeFullPath()
    { return BGlobal.REFPATH+ getRefereePath()+ "/"; }


  public void setVictorySound(String sound)
    { victorySound = sound; }

  public String getVictorySound()
    { return (victorySound == null) ? "standard.wav" : victorySound; }
    // TODO: Should sound default to 'standard.wav' like this?

  public String getVictorySoundFullPath()
    { return BGlobal.SOUNDPATH+ "arena/victory/"+ getVictorySound(); }


 /* DJG Jun 2001
 private static Vector testCollisions_cols = new Vector();

 public Vector testCollisions ( javax.media.j3d.Locale myLocale, boolean checkSelf , BBot botA, BBot botB)
  {

   // Vector testedComponents = new Vector();
   testCollisions_cols.clear();
   botA.testCollisions ( botB, testCollisions_cols ,  myLocale, true);


   return testCollisions_cols;

  }
  */

  public void cleanse()
  {
    arenaObjects.clear();
    arenaObjects = null;
    floorTexture = null;
    floorObjectName = null;

    AmbientColor = null;
    directionalColor = null;

    directionalColor2 = null;
    backgroundFileName = null;

    simulation = null;


  }

 /*  public void finalize()
  {
    BDebug.print("[BArena.finalize()]");
  }*/



  // SJS 22/02/2001 - Animation mgmt. functions...

  public void addAnimation(BAnimation a)
    { animations.add(a); }

  public void removeAnimation(BAnimation a)
    { animations.remove(a); }


  // SJS 22/02/2001 - Arena animation functions.

  public void start()
  {
    // BDebug.print("BArena.start()");
    if (animations != null)
      animations.start();
  }

  public boolean animate()
  {
    // BDebug.print("BArena.animate()");

    if (animations != null)
      animations.animate();

    return true;
  }

  public void stop()
  {
    // BDebug.print("BArena.stop()");
    if (animations != null)
      animations.stop();
  }

}