package roboforge;

//: BBot.java

import roboforge.animation.alpha.*;
import roboforge.attributes.*;
import roboforge.btree.*;
import roboforge.math.*;

//import roboforge.ui.BApplication;
//import roboforge.ui.events.*;
//import roboforge.util.processors.*;
import roboforge.util.text.*;

import roboforge.ai.*;
import roboforge.ai.actions.*;
//import roboforge.ai.orient.*;
import roboforge.ai.worldstate.*;
import roboforge.ai.values.*;

//import roboforge.util.graphics.*;
import roboforge.botanalyst.*;

import java.io.*;
import java.util.*;
//import javax.swing.SwingUtilities;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.beans.PropertyChangeListener;
import javax.vecmath.*;
import javax.media.j3d.*;
//import java.awt.Color;

import roboforge.advertising.*;

public class BBot
  implements
    BActor,
    BArenaObject,
    BNameable,
    BProgressMonitorable,
    Cloneable,
    Serializable
{
/*
 *
 * Description:
 * 'The Big One'
 *
 * Example:
 * Modification History ( Name, Date, Notes )
 * Created HSC, 19 April.
 * Modified DJG, 20 May 1999, Upgrade Components.
 *
 */

  // Serialization ID.
  // -----------------------------------------------------------
  static final long serialVersionUID = -1L;


  // Main Members
  // -------------------------------------------------------------

  private BComponentTree componentTree;
    // Component hierachy within the bot.
    // Describes the bot's physical configuration.

  private BChassis chassis = null;
    // Each bot is based upon a single chassis.
    // A chassis provides the bot with basic locomotive capabilities.

  private BAI ai;
    // The bot's 'artificial intelligence'. AI is responsible
    // for the bot's behaviour over time, dealing with the bot
    // through its BActor interface.

  private BZones myZones;
    // The bot's collection of 'zones'.
    // A zone is a region of space which detects the prescence
    // of foreign objects.

  private String creator;
    // Name of the Bots creator

  private String botName;
    // Name of this bot.

  private long tournamentEntryID;
    // The tournament Entry ID of this Bot

  private String botSerialID;
    // Unique ID for this bot.

  private String botFilePath;
    // ??

  private Hashtable calculatedConnectedPairs = null;
    // Cache used to speed up collisions.

  // advert variables
  private String advertID;
  private BAdvert advert;

  // Transient Working Variables

  protected transient double progressPercent;
  protected transient String progressMessage;


  protected float botTotalPrice;
  protected float botTotalWeight;
  protected int   botTotalComponents;
  protected int   botTotalHealth;

  private transient float totalEnergy;
  private transient float   totalDamage;
  private transient float battleTime;

  private String botDescription;


  // SJS [10/02/2000]: 'propertyManager' is used to keep track of 'listeners'.
  // Objects that are interested in this bot's properties can register themselves.
  // They will then get notified when the bot's properties change.

  private transient SwingPropertyChangeSupport propertyManager;

  // Variables to hold max values used for calculating percentages
  // Need to be calculated in pre process
  // So don't assign values here.
  protected transient float maxEnergy;
  private transient float maxDamage;

  private transient float percentEnergy;
  private transient float percentDamage;

  private transient boolean currentlyDetectingEnemy = false;

  /*  SJS [15/02/2000]
      - Deprecated botOrientation.
      - Replaced by botRotation & botTranslation.

    private Matrix4f botOrientation;
  */

  // SJS [15/02/2000] - These vectors specify the initial workshop viewpoint.
  private transient Vector3d botRotation;
  private transient Vector3d botTranslation;
  private transient Vector3d botOrigin;

  private transient boolean upgraded;

  private BMoveList moveList;
  // the list of Moves for the bot (created by ther user)

  private transient int cpuCyclesPerSecond;
    // Total Decisions processed per second
    // SJS [29/05/2000] - Deprecated. YLY to remove.

  private transient BranchGroup liveNode;

  // Game Engine
  private transient BBotActionList actionList;
  // contains the currently executing actions

  private transient Vector arenaComments;
    // Comments to be passed back for the Battle Player

  private transient BArena arena;
    // Arena object in Battle Engine

  private transient BArenaObjectPosition arenaPosition;
  public  transient BArenaObjectPosition startPosition;

  private transient boolean collided = false;

  public transient boolean isAlive = true;
  public transient float damageRate; // init in calculateBotStats
  public transient Vector energies; // init in calculateBotStats
  private transient BSensor[] sensors;     // init in initArenaDetails
  public transient BComponent[] breadthFirstComponents;  // init in initArenaDetails
  private transient BComponent[] chassisAndLeaves;      // init in initArenaDetails


  protected transient float transparency;
    // Bot's transparency factor [0..1].

  protected transient boolean isVisible;
    // Whether bot is visible or not.
    

  protected transient boolean zonesAttached;
  protected transient boolean rangesDisplayed;

  public static String stateCode = "me" ;
  private transient BBot opponentBot = null;
  private transient BArenaObjectPosition lastDetectedOpponentPos = null;
  private transient BWorldStateNode worldState = null;

  // start vicrtory move transient helpers
  public static  float STARTVIC_MIN_TIME = 5f;
  public static  float STARTVIC_MAX_TIME = 10f;
  private transient float totalStartMoveTime = 0;
  private transient float totalVictoryMoveTime = 0;

  private transient String terminationString = null;
  public static final int MAX_NAME_LENGTH = 20;

  private transient boolean tipped = false;
  private transient boolean checkTip = true;

  private transient float lastExecuteTime;

  // DJG June 2001
  public transient float botReach = 0.0f; // Used for Collision Optimisation
  public transient double currentAbsRadius = 0.0d;
  public transient boolean checkArenaCollisions = false;
  private transient BZone[] zoneList; // Init Arena Details
  public float fastestInboundsComponentSpeed = 0f;
  public float fastestComponentSpeed = 0f;
  public boolean movedInLastTimeslice = false;
  //public transient BSimulation mySim; // For Timer Info - remove when finished




  public BBot()
    { botName = "New Bot"; }

  
  public BBot(BChassis c)
  { this(c, true); }
  
  public BBot(BChassis c, boolean init)
  {
   chassis = c;

    // Extract the information by making the Bot component tree equal the77
    // handle to the Chassis component tree

   componentTree = 
     chassis.getChassisComponentTree();
   
   // set the chassi Comp tree to null
   chassis.setChassisComponentTree(null);

   botName = "New Bot";

   // Initialize the bot (upgrading components)...
   if (init)
    init(true);

   chassis.setArenaPosition( new BArenaObjectPosition());
  }
  

  public BBot(BWizardTemplateBot wizB)
  // creates a bot from a wizards template Bot.
  // PRE:: wizB.convertToBaseBot() has been called.
  {
    // cast wiz to BBot, so we cast access the private members
    BBot botCast = (BBot) wizB;

    moveList = botCast.moveList;
    componentTree = botCast.componentTree;
    chassis = botCast.chassis;
    myZones = botCast.myZones;

    botName = botCast.botName;

    ai = botCast.ai;

   // test wizard bot AI
   // will be moved to WizardBotConvert
   ((BDecisionTreeAI)ai).setBot(this) ;

   // remove thumbnails
   this.removeThumbnails();

   init();
    // recalculateBotInfo();  [SJS 30/07/2000] - init already does this.
  }

   public String getTerminationString()
  {

    if (terminationString == null)
      return "";
    else
      return terminationString;

  }

  // SJS [10/02/2000]: Property management methods...
  // -------------------------------------------------------------

  public void addPropertyChangeListener(PropertyChangeListener listener)
  {
    propertyManager.addPropertyChangeListener(listener);
  }

  public void addPropertyChangeListener(
    String propertyName, PropertyChangeListener listener)
  {
    propertyManager.addPropertyChangeListener(propertyName, listener);
  }

  public void removePropertyChangeListener(PropertyChangeListener listener)
  {
    propertyManager.removePropertyChangeListener(listener);
  }

  public void removePropertyChangeListener(
    String propertyName, PropertyChangeListener listener)
  {
    propertyManager.removePropertyChangeListener(propertyName, listener);
  }

  public void removeAllPropertyChangeListeners()
  {
   // BDebug.print("BBot.removeAllListDataListeners called ");

    if ( propertyManager != null)
      propertyManager = new SwingPropertyChangeSupport(this);

    if (myZones != null)
      myZones.removeAllListDataListeners();

    if (moveList != null)
      moveList.removeAllListDataListeners();
  }

  protected void fireBotInfoChange(
    float oldPrice, float oldWeight, int oldComponents, int oldHealth)
  {
    // Tell listeners that the bot's info has changed.
    propertyManager.firePropertyChange("totalPrice",
      new Float(oldPrice), new Float(botTotalPrice));

    propertyManager.firePropertyChange("totalWeight",
      new Float(oldWeight), new Float(botTotalWeight));

    propertyManager.firePropertyChange("totalComponents",
      new Integer(oldComponents), new Integer(botTotalComponents));

    propertyManager.firePropertyChange("totalHealth",
      new Integer(oldHealth), new Integer(botTotalHealth));
  }


  // SJS [10/02/2000]: Recalculation of general bot info...
  // -------------------------------------------------------------

  protected void recalculateBotInfo()
    { recalculateBotInfo(true); }

  protected void recalculateBotInfo(boolean fireChangeEvent)
  {
    //BDebug.print ("BBot.recalInfo entered");

    float oldPrice     = botTotalPrice;
    float oldWeight    = botTotalWeight;
    int oldComponents  = botTotalComponents;
    int oldHealth      = botTotalHealth;

    float tbotTotalPrice      = 0.0f;
    float tbotTotalWeight     = 0.0f;
    int   tbotTotalComponents = 0;
    int   tbotTotalHealth     = 0;

    BComponent[] components = this.getBreathComponents();
    BComponent c = null;

    // add in the chassis
    tbotTotalPrice  = getChassis().getPrice();
    //BDebug.print ("BBot.recalculateBotInfo chassis price " +   tbotTotalPrice );

    tbotTotalWeight = 0;
    tbotTotalHealth = getChassis().getDamageMax();

    // depthFirst traversal
    for (int i = components.length-1 ; i >=0; i-- )
    {
      c = components[i];

      c.calculateIsTopOfStableLimb();

      if (!c.getChassisLock() )
      {
        tbotTotalPrice  += c.getPrice();
        //BDebug.print ("BBot.recalculateBotInfo added price of " + c + " price " + c.getPrice() + " total " +   tbotTotalPrice );

        tbotTotalHealth += c.getDamageMax();
        tbotTotalComponents++;
      }

      tbotTotalWeight += c.getWeight();
    }

    // BDebug.print ("BBot.recalInfo before fire ");
    botTotalPrice      = tbotTotalPrice;
    botTotalWeight     = tbotTotalWeight;
    botTotalComponents = tbotTotalComponents;
    botTotalHealth     = tbotTotalHealth;

    // Fire off property-change events.
    if (fireChangeEvent)
      fireBotInfoChange(oldPrice, oldWeight, oldComponents, oldHealth);

    // BDebug.print ("BBot.recalInfo after fire ");

  }

  public float getTotalPrice()
    { return botTotalPrice; }

  public float getTotalWeight()
    { return botTotalWeight; }

  public int getTotalComponents()
    { return botTotalComponents; }

  public int getTotalHealth()
    { return botTotalHealth; }


  public BAI getAI()
    { return ai; }

  public void setAI(BAI ai)
    { this.ai = ai; }

  public BChassis getChassis()
  {
    if (chassis == null)
      chassis = (BChassis)((DefaultMutableTreeNode)componentTree.getRoot()).getUserObject();
    return chassis;
  }

  public BMoveList getChassisMoveList()
  {
    if (chassis == null)
      return null;
    else
      return chassis.getChassisMoveList();
  }

  public BMove getChassisMove(int i)
  {
    if (chassis == null)
      return null;
    else
      return chassis.getChassisMove(i);
  }

  public void setComponentTree (BComponentTree componentTreeIn)
  {
    componentTree = componentTreeIn;
  }

  public BComponentTree getComponentTree()
  {
    return componentTree;
  }


  public static BBot loadFromFile(String filename)
   { return loadFromFile(filename, true); }

  //public static BBot loadFromFileNoInit(String filename)
  //{
    // SJS [07/04/2000]: Collapsed loadFromFileNoInit into loadFromFile.
   // return loadFromFile(filename, false);
  //}

  public static BBot loadFromFile(String filename, boolean doInitialize)
  {
    BBot bot = null;
   // BDebug.print("BBot.loadFromFile(): Loading..." + filename);

    try
    {
      filename = filename.toLowerCase();

      FileInputStream fin = new FileInputStream(filename);
      ObjectInputStream in = new ObjectInputStream(fin);

      bot = (BBot) in.readObject();

      in.close();
      fin.close();

      if (doInitialize)
        bot.init(BGlobal.DEVELOPER_MODE);
      else
        bot.upgradeOnly();

    }

    catch (Exception e)
      { BDebug.print("BBot.loadFromFile(): ERROR: " + e); }

    BSerializableTree.clearCache();

    return bot;
  }

  class ZoneListener implements ListDataListener
  {

      public void intervalAdded(ListDataEvent e)
        {}

      public void intervalRemoved(ListDataEvent e)
      {
        prune();
        BDebug.print ( "BBot.ZoneLIstener zone removal detected " );
      }

      public void contentsChanged(ListDataEvent e)
        {}
  }

  public Vector validate(boolean onlyFatal)
  {
     Vector warnings = new Vector(5);

     BComponent[] components =  getBreathComponents();

     boolean containsCpu = false;
     boolean containsEnergy = false;
     boolean containsSensor = false;

     for (int i = 0; i <components.length; i++)
     {
        if (components[i] instanceof BCpu )
          containsCpu = true;
        else if (components[i] instanceof BEnergyGenerator )
          containsEnergy = true;
        else if (components[i] instanceof BSensor )
          containsSensor = true;

        try
        {
          components[i].upgrade();

          if (components[i].isInternal())
            if (!components[i].getChassisLock() )
            {
              BComponentWarning compW = new BComponentWarning( components[i] );
              compW.setDescription("This component is an internal Roboforge component. It cannot be used.");
              warnings.add(compW);
            }

        }
        catch (Exception e)
        {
          warnings.add( new BComponentWarning( components[i] ) );

        }
     }

     String filteredName = BDictionaryTree.getSharedInstance().filter(getName());
     if (!filteredName.equals(getName() ) )
      warnings.add(new BBotWarning("Offensive name detected" ,
                                   "This bot will be rejected from tournaments because its name may be offensive to some people") );


    if (!onlyFatal)
    {
      if (!containsCpu)
        warnings.add(new BBotWarning("No controller detected" ,
                                   "Your bot must contain at least 1 controller. If all your controllers are destroyed in a fight, you lose the battle.") );
      if (!containsEnergy)
        warnings.add(new BBotWarning("No energy generator detected" ,
                                    "The bot requires energy to move") );

      if (!containsSensor)
        warnings.add(new BBotWarning("No sensors detected" ,
                                    "Zones will only become active if they are detected by a sensor. Also certain world items will only update if the enemy is detected by one of your sensors") );

      getMoveList().validate(warnings, onlyFatal);
      ((BDecisionTreeAI)ai).validate(warnings, onlyFatal);
    }

     return warnings;

  }

  private void upgradeOnly()
  // Upgrade components without initializing 3d structure
  {

    if ( (getChassis().getVersion() != BGlobal.APP_VERSION) || BGlobal.DEVELOPER_MODE )
    {
      BDebug.print ("BBot.upgradeOnly upgrading bot");
      propertyManager = new SwingPropertyChangeSupport(this);

      BComponent[] components =  getBreathComponents(true) ;
      for (int i = 0; i <components.length; i++)
        components[i].upgrade();

      recalculateBotInfo(false);
    }

  }


  public void init() // For ArenaObject interface
    { init(false); }

  public void init(boolean forceUpgrade)
     { init(forceUpgrade, null, null, false); }



  public void init(boolean forceUpgrade, Vector upgradeErrors, Vector uniqueItemsOwned, boolean optimise)
  {
      //BDebug.seedTimer();

      if (forceUpgrade)
        BDebug.print("BBOT.INIT UPGRADE FORCED...");
      //forceUpgrade = false;
      //BDebug.print("BBOT.INIT FORCE UPGRADE REMOVED !!!...");

      if (myZones != null)
      {
        myZones.init();
        myZones.addListDataListener(new ZoneListener());
      }
      if (moveList != null)
        moveList.init();

      if (ai == null)
        ai = new BDecisionTreeAI(this);

      // SJS [10/02/2000]: Initialize property manager...
      propertyManager = new SwingPropertyChangeSupport(this);

      BComponent[] components =  getBreathComponents(true) ;

      // Prepare for progress calculation...
      progressPercent = 0.0;
      double progressDelta  = 100.0 / components.length;


      // Need to loop through the BComponentTree of BBot calling the init method for all
      // components and upgrade if necessary.
      this.upgraded = false;

      for (int i = 0; i <components.length; i++)
      {
        BComponent c = components[i];

        progressMessage = "Initializing component: "+ c.toString()+ "...";
        //BDebug.print(progressMessage);

        c.init(optimise);

        // set bot for old bots (WILL BE DEPRECATED )
        c.setBot(this);

        if (!optimise)
          if ((c.getVersion() != (BGlobal.APP_VERSION) || forceUpgrade)
              && c.getComponentFilePath() != null)
            try
            {
              if (uniqueItemsOwned != null && c.isUnique())
              {
                // Test to see if the owner is allowed to own this Unique Item
                if (!uniqueItemsOwned.contains(c.getSerialNo()))
                {
                  // Account doesn't own this unique Item
                  upgradeErrors.addElement("Unique Item not owned " + c.getComponentFilePath());
                }
              }

              progressMessage = "Upgrading component: "+ c.toString()+ "...";
              c.upgrade();
            }
            catch (Exception e)
            {
              BDebug.print ("BBot.upgrading " + c.getComponentFilePath() + " failed.");

              if(upgradeErrors != null)
                upgradeErrors.addElement("BBot.upgrading " + c.getComponentFilePath() + " failed.");
            }

        progressPercent += progressDelta;

        // Allow Swing to repaint progress components...
        // Thread.currentThread().yield();
      }

    if (!optimise)
    {
      if (calculatedConnectedPairs  == null )
        recalculateConnectedWithoutJoints();
    }

   // BDebug.print ("BBot.init before calc ");
   // SJS [08/02/2000]: Calculate bot price, weight, #components, etc.
   recalculateBotInfo(false);

   // Init zones display...
   zonesAttached = false;

   // Init range display...
   rangesDisplayed = false;

   // tip flag
   checkTip = true;

   // Initialize transparency to opaque.
   transparency = 0f;

   // Bot is visible, initially.
   isVisible = true;

   ai.init();

 //  BDebug.printTime ("BBot.init completed ");
  }

  public String getProgressMessage()
    { return progressMessage; }

  public int getProgressPercent()
    { return (int) Math.round(progressPercent); }


  public BSnapshot getSnapFromSimulation()
  {
    BComponent[] breath = this.getBreathComponents();
    BSnapshot snapshot = new BSnapshot("Battleshot");

    for (int i = 0; i < breath.length; i++ )
      if ( breath[i]instanceof BJoint   )
      {
        BJoint theJoint = (BJoint)breath[i];
        if (  theJoint.hasMovedSinceLastSnap()  )
        {
          //if (theJoint instanceof BWheel )
          //  BDebug.print ("Taking snap of " + theJoint );

          BComponentTransform ct = new BComponentTransform(theJoint);
          snapshot.add(ct);
          theJoint.setLastSnapPosition(ct.getJointPosition());
          
        }
      }

    return snapshot;


  }

  public BSnapshot getSnapshot (String nameIn, boolean getChassisJoints, boolean onlySnapChangedJoints, boolean snapLimbLocked, BMove move, int newSnapIndex)
  {
    // if we are construction a chassis bot, all moves must be limbLocked
    if (BGlobal.CONSTRUCTING_CHASSISBOT)
      snapLimbLocked = true;


    // Create the Snapshot
    BSnapshot snapshot = new BSnapshot(nameIn);

    // Need to loop through the BComponentTree of BBot
    // MUST BE BREADTH FIRST
    BComponent[] breath = this.getBreathComponents();

   // DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode)componentTree.getRoot();
   // Enumeration enumerator = rootNode.breadthFirstEnumeration();

    for (int i = 0; i < breath.length; i++ )
    {

      // dont take autoJoints
      if ( breath[i].getComponentType() == BJoint.COMPONENT_TYPE   )
      {
        BJoint theJoint = (BJoint)breath[i];

        if(theJoint.getChassisLock() && getChassisJoints)
        {
          // If it is Chassis Locked and we want chassis joints
          // Create a new BComponentTransfrom object and add it to the snapshot
          if (onlySnapChangedJoints)
          {
                if (  theJoint.hasMovedSinceLastSnap() )
                  snapshot.add(new BComponentTransform(theJoint));

          }
          else
               snapshot.add(new BComponentTransform(theJoint));


        }
        if (!theJoint.getChassisLock())
        {
          // If it's not Chassis locked then add it to the snapshot
          // Create a new BComponentTransfrom object and add it to the snapshot


          if (onlySnapChangedJoints)
          {


               // if (  !theJoint.getJointPosition().equals(theJoint.getDefaultJointPosition()) )
                if ( theJoint.hasMovedSinceLastSnap() )
                  snapshot.add(new BComponentTransform(theJoint));
                //else
                  //  BDebug.print ("skipping the recording of  " + theJoint );


          }
          else if (!snapLimbLocked )
          {

             if (  !theJoint.getJointPosition().equals(theJoint.getDefaultJointPosition()) )
             {
                snapshot.add(new BComponentTransform(theJoint));
             }
             else
             {
                boolean jointPreviouslyMoved = false;
                for (int innerLoop =  1; innerLoop < newSnapIndex && ! jointPreviouslyMoved; innerLoop++)
                {
                  BSnapshot s = (BSnapshot) move.getSnapshots().get(innerLoop);
                  jointPreviouslyMoved = s.contains(theJoint);
                  //BDebug.print ("BBot.getSnapshot snap checking " + theJoint + " with snap " +  innerLoop + " result " +  jointPreviouslyMoved);

                }

                if (jointPreviouslyMoved)
                  snapshot.add(new BComponentTransform(theJoint));
             }

          }

          else
             snapshot.add(new BComponentTransform(theJoint));

        }
      }
    }

    // SJS [14/03/2000]: Return a non-null snapshot,
    // even if no joints were found.
    // TODO: Why return a null pointer anyways?

    return snapshot;
  }

 /* public BSnapshot getSnapshot (String nameIn)
  {

    return getSnapshot(nameIn,false, false, true);

  }   */

  

   public BSnapshot getDefaultSnapshot (String nameIn)
  /*

  * Description:
    Cycles through the BComponentTree of the bot, populating the snapshot
    with BComponentTransforms for every node that contains a BJoint Object.
    It resets all joints to their DefaultPosition (via a clone of course).

  * Example:
  * Modification History ( Name, Date, Notes )
    Created HSC, 28 June.

  */

  {
    //int noOfJoints = 0;

    // Create the Snapshot
    BSnapshot snapshot = new BSnapshot(nameIn);

    // Need to loop through the BComponentTree of BBot

    BComponent[] breath = this.getBreathComponents();

    //DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode)componentTree.getRoot();
    //Enumeration enumerator = rootNode.breadthFirstEnumeration();

    //while (enumerator.hasMoreElements())
    for (int i = 0; i < breath.length; i++ )
    {
      //DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();

      //if (((BComponent)node.getUserObject()) instanceof BJoint && !((BComponent)node.getUserObject()).getChassisLock() )
       if ( (breath[i].getComponentType() == BJoint.COMPONENT_TYPE) && ! breath[i].getChassisLock() )

        // If it's a joint, and not Chassis locked then add it to the snapshot
        {

          BJoint joint = (BJoint) breath[i];

          // Create a new BComponentTransfrom object and add it to the snapshot
          BComponentTransform componentTransform = new BComponentTransform(joint);
          // Set the BComponentTransform to equal the default joint position
          BJointPosition jointPosition = (BJointPosition)joint.getDefaultJointPosition().clone();
          componentTransform.setJointPosition(jointPosition);

          snapshot.add(componentTransform);
          //noOfJoints++;
        }
    }


    return snapshot;
  }

  public void saveToFile(String filename)
    throws IOException
  {
     if (!BGlobal.BOTSUFFIX.equalsIgnoreCase(filename.substring(filename.length()-4)))
        filename = filename +  BGlobal.BOTSUFFIX;
     save(filename);
  }

  protected void save ( String filename)
    throws IOException
  {

    FileOutputStream fout = new FileOutputStream(filename);
    ObjectOutputStream out = new ObjectOutputStream(fout);

    out.writeObject(this);
    out.close();

    BSerializableTree.clearCache();

  }

  public void setName (String botNameIn)
  {
    String old = botName;
    botName = botNameIn;

    propertyManager.firePropertyChange("name", old, botName);
  }

  public String getName()
  {
    return botName;
  }

  // set the arena object advert ID
  public void setAdvertID(String advertID)
  {  this.advertID = advertID;  }

  // get the arena object advert ID
  public String getAdvertID()
  {  return advertID;  }

  // set the arena object advert
  public void setAdvert(BAdvert advert)
  {  this.advert = advert;  }


  public String toString()
  {
    return botName;
  } 


   public void addRangeDisplay()
  {
    BSensor[] sensors = this.getSensors();

    for (int i = 0 ; i < sensors.length ; i++ )
           sensors[i].addRangeDisplay();

    rangesDisplayed = true;
  }

  public void removeRange()
  {
    BSensor[] sensors = this.getSensors();

    if (sensors != null)
    {
      for (int i = 0 ; i < sensors.length ; i++ )
        sensors[i].removeRange();

      rangesDisplayed = false;
    }

  }

  public boolean areRangesDisplayed()
    { return rangesDisplayed; }


  
  public void setBotFilePath (String botFilePathIn)
  {
    botFilePath = botFilePathIn;
  }

  public String getBotFilePath()
  {
    return botFilePath;
  }

  public void setMoveList(BMoveList moveListIn)
  {
    moveList = moveListIn;
  }

  public BMoveList getMoveList()
  {
    if ( moveList == null)
      moveList = new BMoveList();
    return moveList;
  }



  public void setViewRotation(Vector3d r)
  {
    botRotation = r;
  }

  public Vector3d getViewRotation()
  {
    if (botRotation == null)
      botRotation = new Vector3d
      (
        BPhysics.degToRad(-25.0f),
        BPhysics.degToRad(+25.0f),
        0.0
      );

    return botRotation;
  }

  public void setViewTranslation(Vector3d t)
  {
    botTranslation = t;
  }


  public void setViewXTranslation(double x)
  {
    getViewTranslation();
    botTranslation.x = x;
  }

  public void setViewYTranslation(double y)
  {
    getViewTranslation();
    botTranslation.y = y;
  }

  public void setViewZTranslation(double z)
  {
    getViewTranslation();
    botTranslation.z = z;
  }

  public Vector3d getViewTranslation()
  {
    // TODO: Fix distance hack.
    if (botTranslation == null)
      botTranslation =
        new Vector3d(0.0, 0, 300.0);

    return botTranslation;
  }

  public Vector3d getViewOrigin()
  {
    // PRE: Bot has to be added into a canvas.

    // return getChassis().getArenaPosition().getTranslation();

    if (botOrigin == null)
    {
      double y = chassis.calculateHeight();
      botOrigin = new Vector3d(0.0, y, 0.0);
    }

    if (arenaPosition != null)
    {
      botOrigin.x = arenaPosition.x * BGlobal.HEIGHT_SCALE;
      botOrigin.z = arenaPosition.z * BGlobal.HEIGHT_SCALE;
    }

    return botOrigin;
  }

  public void setViewOrigin(Vector3d o)
    { botOrigin = o; }

  public void setViewOriginXZ(double x, double z)
  {
    botOrigin.x = x;
    botOrigin.z = z;
  }


  public void addComponent
  (
    BComponent child,  BConnectPoint childPt,
    BComponent parent, BConnectPoint parentPt
  )
  {
    // BConnectPoint childPt  = child.getConnectPointSelected();
    // BConnectPoint parentPt = parent.getConnectPointSelected();

    if ((childPt != null) && (parentPt != null))
    {
       // Set the base connector to the currently selected connect-point.
       child.setConnectPointBase(childPt);

       // Set the parent connector handle.
       child.setConnectPointParent(parentPt);

       // Add the component to the bot tree.
       componentTree.addComponent(child, parent.getTreeNode());

       /*
       // Reset the connect-points' selection status.
       parent.setConnectPointSelected(null);
       child.setConnectPointSelected(null);
       */

      child.setBot(this);

      // Update general bot information...
      float oldPrice    = botTotalPrice;
      float oldWeight   = botTotalWeight;
      int oldComponents = botTotalComponents;
      int oldHealth     = botTotalHealth;

      botTotalPrice  += child.getPrice();
      botTotalWeight += child.getWeight();
      botTotalHealth += child.getDamageMax();
      botTotalComponents++;

      // Fire off property-change events.
      fireBotInfoChange(oldPrice, oldWeight, oldComponents, oldHealth);

      addComponentToCollisionCache(child);

      // add to lazy worlditem cache
      if (worldState != null)
        addComponentWorldItem(worldState, child);



      // nullify cache
      breadthFirstComponents = null;

      if (child instanceof BSensor)
        sensors = null;



    }
  }

  public void removeComponent(BComponent toRemove)
  {

    DefaultMutableTreeNode startNode =
      (DefaultMutableTreeNode) toRemove.getTreeNode();

    DefaultMutableTreeNode currentNode = null;
    BComponent c = null;

    // Changed to DepthFirst -YLY
    Enumeration e = startNode.depthFirstEnumeration();

    // Reset parent connect point's status...
    BConnectPoint parentPt = toRemove.getConnectPointParent();

    if (parentPt != null)
    {
      toRemove.getParent().setConnectPointActive(parentPt, true);
      //parentPt.setActive(true);
    }
      // Remember bot information (for property change events)...
    float oldPrice    = botTotalPrice;
    float oldWeight   = botTotalWeight;
    int oldHealth     = botTotalHealth;
    int oldComponents = botTotalComponents;

    boolean jointWasRemoved = false;

     //  delete tree first
    this.getComponentTree().deleteComponent(startNode);

    while (e.hasMoreElements())
    {
        currentNode = (DefaultMutableTreeNode) e.nextElement();
        c = (BComponent) currentNode.getUserObject();

        if (currentNode.isLeaf() && BGlobal.CONSTRUCTING_CHASSISBOT )
          getChassis().setChassisAndFeetNotValid();

        removeComponentFromCollisionCache(c);

        // Update general bot information.
        this.botTotalPrice  -= c.getPrice();
        this.botTotalWeight -= c.getWeight();
        this.botTotalHealth -= c.getDamageMax();
        this.botTotalComponents--;

        // remove component from lazy cache
        if (worldState != null)
        {
          char dot = BWorldState.DELIMITER;

          String key =
            BBot.stateCode+ dot+
            BComponent.stateCode+ dot+
            c.getComponentType().toLowerCase()+ dot+
            c.getName();

            worldState.remove(key);
        }



        if (c.getComponentType() == BJoint.COMPONENT_TYPE)
          jointWasRemoved = true;

        // Clean out backward-pointing object handles prior to final removal.
        c.cleanse();
    }

   // BDebug.print("BBot.Child Loop Exited: entered ");



    // nullify cache
    breadthFirstComponents = null;
    sensors = null;
    chassisAndLeaves = null;

   // BDebug.print("BBot.Joints Cleansed entered ");
    fireBotInfoChange(oldPrice, oldWeight, oldComponents, oldHealth);

    prune(jointWasRemoved);
    
    // sneak in a GC
    System.gc();

    //BDebug.printTime("BBot.remove component");
  }

  
  public void prune()
  { prune(false); }

  protected void prune(boolean pruneMoveList)
  // clean and remove unneeded stuff
  {
    //if (worldState != null)
    //  BDebug.printTree( " worldState " , worldState);

    ai.prune();

    if (pruneMoveList)
      if (this.moveList != null)
        this.moveList.cleanseJointsNotInBot(this);

  }

  public void destroyComponent(BComponent bComponentIn)
  {

    // do not destroy Chassis Lock components.
    // this is handled in BChassis
    if (bComponentIn.getChassisLock() )
      return;
                                

    DefaultMutableTreeNode startNode =
      (DefaultMutableTreeNode) bComponentIn.getTreeNode();

    BComponentNode currentNode = null;
    BComponent c = null;

    // Changed to DepthFirst -YLY
    Enumeration e = startNode.depthFirstEnumeration();

    // Loop through sub-components...
      while (e.hasMoreElements())
      {
        currentNode = (BComponentNode) e.nextElement();
        c = currentNode.getComponent();

        if (  !c.getDestroyed() )
        {
          // Update bot damage to reflect this component's destruction.
          if (c != bComponentIn)
            this.addToBotDamage(c.damageMax - c.getDamageCurrent());

          // Flag component as destroyed.
          c.destroy();

       }

     }
  }

  public void setUpgraded(boolean upgradedIn)
  {
    upgraded = upgradedIn;
  }

  public boolean isUpgraded()
  {
    return upgraded;
  }

  // Game Engine Stuff

  private void createBreathFirstAndLeafList()
  {
    DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode)componentTree.getRoot();
    Vector breadthFirstComponentsV = new Vector();
    Vector chassisAndLeavesV = new Vector();

    for (Enumeration children = rootNode.breadthFirstEnumeration(); children.hasMoreElements() ;)
    {
        BComponentNode child = (BComponentNode)children.nextElement();
        breadthFirstComponentsV.add(child.getUserObject());

        if (child.isLeaf() || (child.getUserObject() instanceof BChassis ) )
                chassisAndLeavesV.add(child.getUserObject());
    }

    breadthFirstComponents = new BComponent[breadthFirstComponentsV.size() ];
    for (int i = 0; i < breadthFirstComponentsV.size(); i++)
      breadthFirstComponents[i] = (BComponent)breadthFirstComponentsV.get(i);

    chassisAndLeaves = new BComponent[chassisAndLeavesV.size() ];
    for (int i = 0; i < chassisAndLeavesV.size(); i++)
      chassisAndLeaves[i] = (BComponent)chassisAndLeavesV.get(i);

  }


  public void initArenaDetails(BArena arenaIn)
  {
    initArenaDetails( arenaIn, null, null);
  }

  transient static double arenaRadius = 0.0d;

  public void initArenaDetails(BArena arenaIn, BBot opponent, BSimulation sim)
  {
    totalDamage = 0;
    opponentBot = opponent;
    lastDetectedOpponentPos = new BArenaObjectPosition();
    myZones.clearDetected();
    opponentBotVelocity = 0.0f;

    // DJG June 2001
    //mySim = sim;

    if (opponent != null)
      lastDetectedOpponentPos.set(opponent.getArenaPosition() );

    getWorldState();  // create a new world state if one does not already exist
    currentlyDetectingEnemy = false;

    actionList = new BBotActionList();
   // BDebug.print ("BBot.initArenaDetails" + this + " actionList " + actionList );

    lastCollisionTime = -180;
    //lastUpdatedCollisionTime = 0;

    //BDebug.print ("BBot.initArena Details " + this +" "+ arenaIn);
    arena = arenaIn;
    arenaPosition = null;
    startPosition = null;
    isAlive = true;
    getChassis().terminateChassisMove(true);

    if (moveList != null)
      moveList.preProcessMoves(getChassis());


    // fill the fast Vectors with all the components
    getBreathComponents();

    // if a chassis move list exist then pre-proocess it,  ( ChassisBots will not have a move List  )
    if ( getChassis().getChassisMoveList() != null)
    {
      getChassis().getChassisMoveList().preProcessMoves(   getChassis()    );
      BGlobal.CONSTRUCTING_CHASSISBOT = false;
    }
    else
      BGlobal.CONSTRUCTING_CHASSISBOT = true;


    calculateBotStats(true, false, BGlobal.CALC_TIMESLICE, true);

    actionList.addContinualAction(new BChassisContinualAction(getChassis()) );

     for (Enumeration e = getWorldState().getWorldItems() ; e.hasMoreElements() ;)
      ((BWorldItem)e.nextElement() ).initForSim(sim) ;


     // SJS [09/09/2000] - Initialize the bot's AI module.
     ai.init();

     clearTippingFlag();
     setBotAlive(true);
     lastDistance = 0; // used for getOpponentRelativeVelocity

     if(opponentBot != null)
      lastOpponentPos = opponentBot.getArenaPosition();
     else
      lastOpponentPos = new BArenaObjectPosition(0,0,0,0);

      percentEnergy = 0;

      // DJG June 2001
      arenaRadius = (arena.getSize() / 2.0);

  }


  public class BComponentNumberWorldItem extends BComponentWorldItem
  {
    /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
	BNumber theNumber;

    public BComponentNumberWorldItem(String key, String name ,BNumber defaultValue,BNumber lower, BNumber upper ,String units,String description, BComponent myComponent)
    {
      super( key,  name,  defaultValue,  lower,  upper,  units,description,  myComponent);
      theNumber = defaultValue;
    }

    public void setValue (double v)
      { theNumber.setValue(v); }

  }

  private void addComponentWorldItem(BWorldState root, BComponent component)
  {
    char dot = BWorldState.DELIMITER;

    String key = component.getWorldStateCode();

        BComponentNumberWorldItem componentItem =
          new BComponentNumberWorldItem(
            key+ dot+ "health", "Health",
            new BNumber(100.0),
            new BNumber(0.0),
            new BNumber(100.0) ,"%", "Remaining health of this component",
            component)
        {
          /**
			 * 
			 */
			private static final long serialVersionUID = -1L;

		public BValue evaluate()
          {
            setValue(this.getComponent().getHealthPercent());
            return getValue();
          }
        } ;

        root.put(  componentItem  );

  }


  public void addWorldItems(BWorldState root)
  {
    // add Zone Items
    getZones().addWorldItems( worldState);

    // addMoves
    //getMoveList().addWorldItems(BWorldState root);
    getMoveList().addWorldItems(root);

    // add Component Items
    BComponent[] components = getBreathComponents();

    for (int i = 0; i <components.length; i++)
      if (!components[i].getChassisLock() )
      {
        addComponentWorldItem(root, components[i]);

      }
  }


 public Vector getComponentsPastMaxDist()
 {
    Vector v = new Vector(2,2);
    Vector3f chassisVector = getChassis().getCachedVWorld();

    BComponent components[] = this.getBreathComponents();
    for (int i = components.length - 1; i >= 0 ; i--)
    {
      Vector3f componentVector = components[i].getCachedVWorld();
      isBotTippedRelativeV.sub(  chassisVector, componentVector  );
      isBotTippedRelativeV.scale(BGlobal.CANVAS_TO_WORLD_SCALE);
      if (isBotTippedRelativeV.length() > BPhysics.MAX_BOT_RADIUS )
        v.add(components[i]);


    }

    return v;

 }

 private static Vector3f isBotTippedRelativeV = new Vector3f();
 private transient Vector3f tippingAxis;
 //private static Transform3D chassisTran = new Transform3D();
 //private static Vector3f componentVector = new Vector3f();


 private transient float height = 0.0f;
 private transient float width = 0.0f;
 private transient float depth = 0.0f;

 private Vector3f getTippingSum()
 {
    Vector3f chassisVector = getChassis().getCachedVWorld();
    Vector3f chassisHeight = new Vector3f(chassisVector);
    chassisHeight.scale(BGlobal.CANVAS_TO_WORLD_SCALE);

    BComponent components[] = this.getBreathComponents();
    Vector3f sum = new Vector3f();

    height = 0.0f;
    width = 0.0f;
    depth = 0.0f;
    botReach = 0.0f;

    for (int i = components.length - 1; i >= 0 ; i--)
      if (!components[i].getChassisLock() && !components[i].getDestroyed()         )
      {
        Vector3f componentVector = components[i].getCachedVWorld();

        isBotTippedRelativeV.sub( componentVector, chassisVector );
        isBotTippedRelativeV.scale(BGlobal.CANVAS_TO_WORLD_SCALE);

        if(Math.abs(isBotTippedRelativeV.x) > width)
          width = Math.abs(isBotTippedRelativeV.x);
        if(Math.abs(isBotTippedRelativeV.z) > depth)
          depth = Math.abs(isBotTippedRelativeV.z);
        if( (isBotTippedRelativeV.y + chassisHeight.y) > height)
          height = isBotTippedRelativeV.y + chassisHeight.y;

        botReach = java.lang.Math.max(botReach, (isBotTippedRelativeV.length() + components[i].boundingRadius));


        // this will be radius

        isBotTippedRelativeV.scale(components[i].getWeight());
        sum.add(isBotTippedRelativeV);
        //BDebug.print ( "torque vector " + isBotTippedRelativeV + " :: sum " +  sum );

      }


    sum.y = 0;

    return sum;
 }

 // mehtods for getting the dimensions of this bot
 public float getHeight()
 {
   return height;
 }
 public float getWidth()
 {
   return width;
 }
 public float getDepth()
 {
   return depth;
 }

 // mehtods for getting the dimensions of the opponent of this bot
 public float getOpponentHeight()
 {
   return opponentBot.getHeight();
 }
 public float getOpponentWidth()
 {
   return opponentBot.getWidth();
 }
 public float getOpponentDepth()
 {
   return opponentBot.getDepth();
 }


 public boolean isBotTipped()
 {

    Vector3f sum = getTippingSum();
    //BDebug.print ("BBot.isBotTipped sum " + sum );

    boolean offBalance = sum.length() > getChassis().getMaxBalancedTorque();

    if (offBalance)
    {
      //BDebug.print ("BBot.isBotTipped offBalance DETECTED ");
      //BDebug.print ( this + " BBot.isBotTipped sum" + sum );
      tippingAxis = new Vector3f(sum);
      Transform3D tangentRot = new Transform3D();
      tangentRot.rotY(BPhysics.HALF_PI);
      tangentRot.transform(tippingAxis);
      //BDebug.print (this + "BBot.isBotTipped tippingAxis" + tippingAxis );



    }


    //BDebug.print ("final length " + sum.length() );
    return offBalance;

 }

 public void clearTippingFlag()
  {
   // BDebug.print ("BBot.clearTippingFlag ");
    tipped = false;
  }

  private void checkBalance()
  {
    if (!tipped)
      if (isBotTipped() )
      {
        tipped = true;
        BBotFallingAction bfAction = new BBotFallingAction (this,tippingAxis );
        actionList.addAction(bfAction);
       // BDebug.print ("BBot.checkBalance setting tipping flag for" + this.hashCode() );

      }
  }


  public boolean isTipped()
  {
    

    return tipped;
  }

  // BBot simulation execution method...
  // -----------------------------------------------------------

  public float getLastExecuteTime()
    { return lastExecuteTime; }

  /* DJG June 2001
  protected void worldStateUpdate(javax.media.j3d.Locale myLocale)
  {
    //BDebug.print ( " BBot.testCollision " );
    getZones().testCollision(opponentBot, myLocale);
    opponentBot.getZones().testCollision(this, myLocale);

  }
  */

  public void execute(float t)
    { execute(t, true, t, true); }

  // DJG June 2001 - Massive Changes
  //public void execute(float t, boolean worldStateCycle, javax.media.j3d.Locale  myLocale)
  public void execute(float t, boolean worldStateCycle, float statsTimeslice, boolean botStatsCycle)
  // Simulate the bot's activity for 't' seconds.
  {
    if (this.isAlive )
    {
      //BDebug.print (getName() + " BBot.execute execAI :" + executeAI + " t=" + t );
      lastExecuteTime = t;

      if (worldStateCycle)
      {
        // DJG June 2001
        //if (myLocale != null)
         // worldStateUpdate(myLocale);

        checkBalance();
        // check the bots balance and start tip move if the bot is off balance

        // Use AI to determine activity.
        // Bot Actions end up on the actionList.
        //mySim.movesAITimer.start();
        //ai.execute(Math.round(cpuCyclesPerSecond) );
        //BDebug.print (getName() + " BBot.execute - AI Cycles:" + Math.max( Math.round(cpuCyclesPerSecond * BGlobal.CALC_TIMESLICE), 1) + " t=" + BGlobal.CALC_TIMESLICE );
        ai.execute( Math.max( Math.round(cpuCyclesPerSecond * BGlobal.CALC_TIMESLICE), 1) );
        //mySim.movesAITimer.stop();

      }

      // Save Joint and Chassis Positions for possible Rollback
      if ( botStatsCycle )
        savePrevArenaPosition();

      // Execute bot activity for 't' seconds.
      //mySim.movesMVTimer.start();
      actionList.executeActions(t);
      //mySim.movesMVTimer.stop();
      // BDebug.print ("BBot.execute " + this + " actionList " + actionList );

      // Calculate statistics for the bot. Calculated after moves are executed so it can take measurements
      // of what happened in the 3D world (torque/acceleration) to be used in the next timeslice.
      if (botStatsCycle)
      {
      	//mySim.movesCSTimer.start();
      	calculateBotStats(false, false, statsTimeslice, worldStateCycle);
      	//mySim.movesCSTimer.stop();

      }
    }
  }


  // BActor interface Methods...
  // -----------------------------------------------------------

  public boolean canAddAction(BAction a)
    // Can the specified action be executed by the Actor?
  {
       return actionList.canAddAction(a);

  }

  public boolean addAction(BAction a)
    // Request that the Actor executes Action a.
  {
     // BDebug.print ("BBot.addAction " + a );
      actionList.addAction(a) ;
      return true;
  }

  public BWorldState getWorldState()
    // Returns an object that contains information about the
    // Actor's perceived world-state.
  {
    //BDebug.print (this + "getWorldState() Called");
    if (worldState == null)
    {
      worldState = BSimulation.getWorldState(this);
      addWorldItems(worldState);
    }

    return worldState;
  }



  public BArenaObjectPosition getSpecialMovePosition(float timeSlice, Vector channels, String specialMove)
  {
    return new BArenaObjectPosition();
  }

  public void setWireFrame()
  {
    // Wire Frame live Objects
    BComponent c;
    DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode)getComponentTree().getRoot();
    Enumeration enumerator = rootNode.breadthFirstEnumeration();

    while (enumerator.hasMoreElements())
    {
       DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
       c = (BComponent)node.getUserObject();
       c.setWireFrame();


    }



  }


  public void setNormalAppearance()
  {

    // Recurse tree setting normal appearance to all objects
    BComponent c;
    DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode)getComponentTree().getRoot();
    Enumeration enumerator = rootNode.breadthFirstEnumeration();

    while (enumerator.hasMoreElements())
    {
       DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
       c = (BComponent)node.getUserObject();
       c.setNormalAppearance();


    }


  }
  // Return to the normal appearance

  public void setArenaPosition(BArenaObjectPosition newPos)
  {

    if (arenaPosition == null)
       arenaPosition = newPos;
    else
      arenaPosition.set(newPos);

    getChassis().setArenaPosition(arenaPosition);
    getChassis().clearWheelVelocity();

    if ( startPosition == null )
    {
      startPosition = new BArenaObjectPosition();
      startPosition.set(arenaPosition);
    }
  }


  public BArenaObjectPosition getArenaPosition()
  {
    if ( arenaPosition == null )
    {
      arenaPosition = getChassis().getArenaPosition();
    }
    
    if ( startPosition == null )
    {
      startPosition = new BArenaObjectPosition();
      startPosition.set(arenaPosition);

     }

    return arenaPosition;
  }


  public void setLiveNodes(BranchGroup wrapperBranch , TransformGroup newDisplacementGroup, TransformGroup newArenaRotateGroup)
  {

    // set capability to allow shadows
    liveNode.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND );

    liveNode = wrapperBranch;

    BChassis rootC =   getChassis();
    rootC.setLiveNodes(newDisplacementGroup, newArenaRotateGroup);

  }


  public Vector getZoneNames()
  {
    if (myZones != null)
      return myZones.getZoneNames();
    else
      return null;
  }

  public void attachZones()
    { attachZones(false, true); }

  public void attachZones(boolean optimize)
    { attachZones(optimize, true); }

  public void attachZones(boolean optimize, boolean visible)
  {
    //BDebug.print ("BBot.attachZones myZones " + myZones);


    zonesAttached = true;
    // check if null for old versions of bots
    if (myZones == null)
      myZones = new BZones();

    Group botCentre = getChassis().getOrientNode();

    if (botCentre ==  null)
      botCentre = getChassis().getLiveNode();

    //BDebug.print ("BBot.attachZones botCentre " + botCentre);


    myZones.attach(botCentre, optimize, visible);

    // DJG June 2001
    //getZoneList(true);
    Vector vZones = myZones.getZones();
    zoneList = new BZone[vZones.size()];
    for (int i=0; i < vZones.size(); i++)
    {
    	zoneList[i] = (BZone)vZones.get(i);
    }
    //BDebug.print("BBot.attachZones: "+vZones.size()+" Zones.");

  }

  /*
  private BZone[] getZoneList(boolean rebuild)
  {
    if ( myZones != null && (zoneList == null || rebuild) )
    {
      // DJG June 2001
    	Vector vZones = myZones.getZones();
    	zoneList = new BZone[vZones.size()];
    	for (int i=0; i < vZones.size(); i++)
    	{
      	zoneList[i] = (BZone)vZones.get(i);
    	}
    }

    return zoneList;
  }
  */

  public boolean areZonesAttached()
    { return zonesAttached; }


  public void detachZones()
  {


    if (myZones != null)
      if (zonesAttached )
        myZones.detach();

   zonesAttached = false;


  }

  
  public BZones getZones()
  {
    // [SJS 30/07/2000] - Ensure zones are non-null.
    if (myZones == null)
      myZones = new BZones();

    return myZones;
  }
  



  public void cleanseLiveNodes()
  {
    //BDebug.print ("cleanseLiveNode called for " + this);

    // only cleanseLiveNodes, if the Bot has not Been cleansed
   if (liveNode != null)
   {
    //BDebug.print ("BBot.cleanseLiveNodes: " + this);
    detachZones();
    myZones.cleanseLiveNodes();

    if (liveNode.isLive() )
      liveNode.detach();

    liveNode = null;

    if (actionList != null)
    {
      actionList.clearActions();
      actionList = null;
    }

    if (getWorldState() != null)
      for (Enumeration e = getWorldState().getWorldItems() ; e.hasMoreElements() ;)
        ((BWorldItem)e.nextElement() ).initForSim(null) ;

    // BComponent c= null;
    BComponent[] components = this.getBreathComponents();

    for (int i = components.length - 1; i >= 0; i--)
      components[i].cleanseLiveNodes();

    // DJG July 2001
    //getChassis().cleanseLiveNodes();
    if ( moveList != null )
      moveList.cleanseLiveNodes();

    arenaComments = null;  // Comments to be passed back for the Battle Player
    arena = null;  // Arena object in Battle Engine
    chassisAndLeaves = null;
    opponentBot = null;
    lastDetectedOpponentPos = null;
    //floorCollisionCache = null;
    energies = null; // init in calculateBotStats
    sensors = null;     // init in initArenaDetails
    actionList = null;
    arenaPosition = null;
    startPosition = null;

    sensors = null;     // init in initArenaDetails
    lastDetectedOpponentPos = null;
    //fastestV = null;
    //passThroughCorrection = null;
    lastExecuteTime = 0;

    // DJG June 2001
    //mySim = null;
    zoneList = null;
    botReach = 0.0f;
  	currentAbsRadius = 0.0d;
  	checkArenaCollisions = false;
  	fastestInboundsComponentSpeed = 0f;
  	fastestComponentSpeed = 0f;
  	movedInLastTimeslice = false;


   }

  }

  public void cleanse()
  // cleans up all the nodes for disposal
  {
    //BDebug.print (getName() + " cleansed ");

    removeAllPropertyChangeListeners();

    //botZones.cleanse();
    cleanseLiveNodes();
    detachZones();

    if (componentTree != null)
    {
      BComponent[] components = this.getBreathComponents();
      for (int i = components.length - 1; i >= 0; i--)
        components[i].cleanse();
    }
    componentTree = null;

    breadthFirstComponents = null;
    zoneList = null;
    chassisAndLeaves = null;
    chassis = null;
    ai = null;

    if (myZones != null)
      myZones.cleanse();

    myZones = null;

    botFilePath = null;
    calculatedConnectedPairs = null;
    propertyManager = null;
    botRotation = null;
    botTranslation = null;
    botOrigin = null;

    if (moveList != null)
      moveList.cleanse();
    moveList = null;

    liveNode = null;

    if (worldState != null)
      worldState.cleanse();
      
    worldState = null;

    // caches
    getHighestJointsNotDefault_V = null;
    getHighestLimbsInResultantMove_tempV  = null;
    getComponentsConnectedToChassisLock_V = null;

    // DJG June 2001
    //mySim = null;
    zoneList = null;
    botReach = 0.0f;
  	currentAbsRadius = 0.0d;
  	checkArenaCollisions = false;
  	fastestInboundsComponentSpeed = 0f;
  	fastestComponentSpeed = 0f;
  	movedInLastTimeslice = false;


  }

  public void finalize()
  {
    BDebug.print("BBot.finalize() for [" + getName() + "]");
  }




  private void loadComponent(BComponent newComponent, BComponent parentComponent)
 {


 try
 {

        BranchGroup tempLive;

       // if  (newComponent instanceof BChassis)
                tempLive = newComponent.getStructure3D();
       // else
       //         tempLive = newComponent.cloneStructure3D(false, true, true);

        tempLive.setCapability(Group.ALLOW_CHILDREN_WRITE);
        tempLive.setCapability(BranchGroup.ALLOW_DETACH);
        tempLive.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
        tempLive.setCapability(Group.ALLOW_CHILDREN_EXTEND);

        // New Live Node
        // should have no affect if the component is a BChassis
        

        if (parentComponent == null )
        {

                newComponent.setLiveBotNode(tempLive);
                liveNode = new BranchGroup();
                liveNode.addChild(tempLive);
                liveNode.setUserData("Bot Live Node");

                liveNode.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
                liveNode.setCapability(Node.ALLOW_BOUNDS_READ);
                liveNode.setCapability(BranchGroup.ALLOW_DETACH);

        }
        else
        {

            newComponent.attach(parentComponent, tempLive, false, true);



         };

      } catch(Exception e) { BDebug.print("ERROR :BBot.loadComponent:" + e.getMessage() );  };
 }


 public boolean isPartOf(BShape3D theShape)
 {
    boolean isTrue = false;

  //  BDebug.print ("BBot.isPartOf shape.getComponent"  + theShape.getComponent() );

    BComponent theComponent = theShape.getComponent();
    DefaultMutableTreeNode theRoot = (DefaultMutableTreeNode) theComponent.getTreeNode().getRoot();

    isTrue = ( theRoot == getComponentTree().getRoot() );


    return isTrue;


 }

 public void setPickable(boolean p)
 {

  getChassis().setPickable(p);
  //BComponent[] comps = this.getBreathComponents(true);
  //for (int i=0; i < comps.length; i++ )
  //  comps[i].setPickable(p);


 }


  public void setZonesVisible(boolean v)
    { getZones().setVisible(v); }

  public void setZonesAttached(boolean v)
  {
    if (v && !areZonesAttached())
      attachZones();

    else if (!v)
      detachZones();
  }

  public void setSensorsAttached(boolean v)
  {
    if (v && !areRangesDisplayed())
      addRangeDisplay();

    else if (!v)
      removeRange();
  }





 public BranchGroup getLiveNode(boolean centerOfObject)
 {

  if (centerOfObject)
    return  getChassis().getLiveNode();
  else
    return liveNode;

 }




// private static float KW_CONVERT;


 // remember to change back to private
 public final void  calculateBotStats(boolean initArena, boolean optimiseForMovesPanel, float timeSlice, boolean worldStateCycle)
 // Calculates stats and pre-processes Bot for Arena-Combat
 // if initArena flag is set then it also sets up Arena specific variables

 {
  // BDebug.seedTimer();



   if ( worldStateCycle || initArena )
   {
     cpuCyclesPerSecond = 0;
     maxEnergy = 0f;
   }

   botTotalWeight =0;
   checkArenaCollisions = false;
   //BDebug.print ( this + " calculateBotStats maxEnergy " +  maxEnergy);

   Vector autoJointsTempV = null;

   if (initArena)
   {
     autoJointsTempV = new Vector();
     maxDamage = getChassis().getDamageMax();
     percentDamage = 0f;

     totalEnergy = 0f;
     botTotalWeight = 0f;
     damageRate = 0f;
     //init chassis first
     getChassis().initForArena( getChassis(), this );
     //KW_CONVERT = (360f / BGlobal.CALC_TIMESLICE);

   }


   // Check to see if the Chassis is still intact
   if (chassis.getDestroyed() && ( !initArena   ) )
   {
     setArenaComments( getName() + "Bot Terminated - Chassis Death");
     terminationString = "Chassis Destroyed! ";
     this.setBotAlive(false);
   }

   this.fastestInboundsComponentSpeed = 0.0f;
   this.fastestComponentSpeed = 0.0f;
   this.movedInLastTimeslice = false;

   //calcBotReach();

   // backwards for depthfirst
   for (int i =  breadthFirstComponents.length - 1; i >= 0  ; --i)
   {

        BComponent current = breadthFirstComponents[i];

        if ( initArena )
        {

          //  The chassis has already been inited  ( The chassis is always the first component in a breath first trarsal )
          if (i != 0)
            current.initForArena( getChassis(), this );

          if (!current.getChassisLock() )
            maxDamage += current.damageMax;

          if ( current instanceof BAutoRotateJoint )
             autoJointsTempV.add(current) ;

          current.calculateIsTopOfStableLimb();

        }
        // calculate LimbLoad (dynamic Torque)
        if (! current.getDestroyed())
        {
          current.calculateLimbLoad(timeSlice, worldStateCycle);
          botTotalWeight +=  current.weight;
        }

        if (!optimiseForMovesPanel && ( worldStateCycle || initArena ) ) // Do only once every CALC_TIMESLICE
          if (! current.getDestroyed())
          {
              // Calculate Total CPU Capacity (for multiple CPUs)
              if ( current.getComponentType() == BCpu.COMPONENT_TYPE  )
              {
                BCpu c = (BCpu) current;
                cpuCyclesPerSecond += c.getCyclesPerSecond();
              }

              // Regenerate Power from Generators
              if ( current.getComponentType() == BEnergyGenerator.COMPONENT_TYPE)
              {

                BEnergyGenerator eG = (BEnergyGenerator)current;
                maxEnergy += eG.energyCapacity;
               // BDebug.print( this + " stats adding " +  eG.energyCapacity + " .. total .. " + maxEnergy);

                if ( initArena )
                {
                 // maxEnergy += eG.energyCapacity;
                  totalEnergy += eG.energyCapacity;
                 // BDebug.print("BBot.calcalate bots stats adding " +  eG.energyCapacity +" .. totalEnergy .. " + totalEnergy);

                  //eG.energyLevel = eG.energyCapacity;
                }
                else // if ( totalEnergy < maxEnergy )   // cap our of the loop
                {
                  float thisCharge = eG.energyOutputRate / (360f / BGlobal.CALC_TIMESLICE ); // in KW

                	//  BDebug.print ("charging  eG.energyOutputRate " + eG.energyOutputRate);
                	//  BDebug.print ("charging  KW_CONVERT " + KW_CONVERT);

                  totalEnergy += thisCharge;
                	//  BDebug.print("BBot.calcalate bots stats charging " + thisCharge +" .. totalEnergy .. " + totalEnergy);


                }

              }
            }


    } // end loop


    totalEnergy = java.lang.Math.min(totalEnergy,maxEnergy);
    //BDebug.print("BBot.calcalate bots stats final capped total "  + totalEnergy);


    if ( BGlobal.CONSTRUCTING_CHASSISBOT  || optimiseForMovesPanel)
    {
        if (maxEnergy == 0)
          maxEnergy = 1000000f;
        totalEnergy = maxEnergy;
        isAlive = true;
    }
    else
    {

      if (cpuCyclesPerSecond == 0 )
      {
        setBotAlive(false);
        setArenaComments( getName() + " Terminated - Controller Death");
        terminationString = "Controller Destroyed.";
      }



    }

    if (totalEnergy <= 0)
      getChassis().returnToStand();


    // fill in caches
    if ( initArena )
    {
      BAutoRotateJoint[] autoJoints;

      autoJoints = new BAutoRotateJoint[autoJointsTempV.size()];

      for (int i = 0; i < autoJointsTempV.size(); i++)
      {
        autoJoints[i] = (BAutoRotateJoint)autoJointsTempV.get(i);
        autoJoints[i].clearVelocity();

      }

      actionList.addContinualAction(new BAutoJointAction(autoJoints ) );



    }


   // stats
   if ( worldStateCycle )
   {
   	if(!optimiseForMovesPanel && !initArena)
   	{
    	BArenaObjectPosition currentOpponentPos = new BArenaObjectPosition(opponentBot.getArenaPosition());
    	float distance = BPhysics.distanceBetween(lastOpponentPos, currentOpponentPos);
    	lastOpponentPos = currentOpponentPos;

    	if(lastExecuteTime > 0)
       	opponentBotVelocity = Math.min(distance / BGlobal.CALC_TIMESLICE, 10f);
        //opponentBotVelocity = distance / lastExecuteTime;
    	else
       	opponentBotVelocity = 0.0f;
   	}
   	else
    	opponentBotVelocity = 0.0f;
   }

   // DJG June 2001
   if ( arenaPosition != null )
     currentAbsRadius = java.lang.Math.sqrt((double)(( arenaPosition.x * arenaPosition.x ) + (arenaPosition.z * arenaPosition.z)));

   // BDebug.printTime(" CalculateLimbLoad ");


 }



 public void resetVictoryMove()
 {
  resetSingleMove(moveList.getVictoryMove());
  getChassis().returnToStand();
  totalVictoryMoveTime = 0f;
  checkTip = false;
 }

 public void resetStartMove()
 {
    BMove start = moveList.getStartMove();
    if (start != null)
      if (start.isValid())
      {
        resetSingleMove(start);
      }

    totalStartMoveTime = 0;
    checkTip = false;

 }

  public void applySnapshot(BMove move,  int index)
  {
    //BDebug.print ("BBot.applySnap called index " + index);
    // if it is the first snapshot then apply the defaut
    this.clearCurrentJointTransforms();
    for (int i = 1; i <= index; i++)
      move.getSnapshot(i).applyToBot();

  }


  public void resetSingleMove(BMove move)
 // reset a move  that is not part of a simulation
 {

  if (actionList!= null)
      actionList.clearActions();

  if (move != null)
  {    
    // hack for constucting chassis moves
     if (BGlobal.CONSTRUCTING_CHASSISBOT)
     {
       BChassis theChassis = this.getChassis();

       float distance = 0f;
       Vector snapshots = move.getSnapshots();
       if ( snapshots != null )
       {
         for ( int i=0; i<snapshots.size(); i++)
         {
           BSnapshot thisSnapshot = (BSnapshot)snapshots.get(i);
           distance += thisSnapshot.getMetersToMove();
         }
       }

       distance = distance * BPhysics.CONSTRUCTING_CHASSIS_MOVE_EXTENSION;


       theChassis.setConstructionChassisMove(move,java.lang.Math.max(distance,0.1f),  this.getMoveList().getMoves().indexOf(move)   );

       //BDebug.print("BBot.resetSingleMove CONSTRUCTING CHASSISBOT DETECTED");

     }
     else
      getChassis().terminateChassisMove(true);

    //BDebug.print("BBot.resetSingleMove 1");
    // the rest
    calculateBotStats(true,false,BGlobal.CALC_TIMESLICE,true);

    //BDebug.print("BBot.resetSingleMove 2");


    //BDebug.print("BBot.resetSingleMove 3");
    move.resetMove(false,0,0);

  //  BDebug.print("BBot.resetSingleMove end");

  }

 }



 public boolean executeSingleMove(BMove move, float timeSlice)
 {
    boolean moved = false;
    boolean chassisMoveExecuted = false;
    lastExecuteTime = timeSlice;
    BChassis theChassis = getChassis();

    if ( !BGlobal.CONSTRUCTING_CHASSISBOT )
        moved =  move.execute(timeSlice);

    actionList.executeActions(timeSlice);
    chassisMoveExecuted  = theChassis.isActive();

    // BDebug.print ("BBot.executeSingleMove chassisMoveExecuted " + chassisMoveExecuted);
    //BDebug.print ("BBot.executeSingleMove moved " + moved);

    calculateBotStats(false, true, timeSlice, true); // After moves are executed

    // Check and initialise tipping
    if (checkTip)
      checkBalance();

    return moved || chassisMoveExecuted || !actionList.isEmpty();

 }

 public boolean runStartMove(float timeSlice)
 {
    totalStartMoveTime = totalStartMoveTime + timeSlice;

    if (moveList.getStartMove() == null)
    {
      checkTip = true;
      return (totalStartMoveTime <  STARTVIC_MIN_TIME );
    }
    else
    {

      boolean moved = false;

      if (moveList.getStartMove().isValid() )
        moved = executeSingleMove(moveList.getStartMove() , timeSlice);

      boolean notFin = (totalStartMoveTime <  STARTVIC_MIN_TIME ) || ( moved  &&  (totalStartMoveTime <  STARTVIC_MAX_TIME ));

      if ( !notFin )
        checkTip = true;

      return  notFin;
    }
 }

 public boolean runVictoryMove(float timeSlice)
 {

    totalVictoryMoveTime = totalVictoryMoveTime + timeSlice;

    if (moveList.getVictoryMove() == null)
    {
      checkTip = true;
      actionList.executeActions(timeSlice);
      return (totalVictoryMoveTime <  STARTVIC_MIN_TIME );
    }
    else
    {
      boolean moved = executeSingleMove(moveList.getVictoryMove() , timeSlice);
      boolean notFin = (totalVictoryMoveTime <  STARTVIC_MIN_TIME ) || ( moved  &&  (totalVictoryMoveTime <  STARTVIC_MAX_TIME ));

      if ( !notFin )
        checkTip = true;

      return notFin;
    }

 }



 public Vector getCurrentActionNames()
 {
    // Returns a vector of all the current moves the Bot is performing
    return actionList.getActionNames();
 }

 public String getCurrentChassisMoveName()
 {
    // Returns the current Chassis move the bot is performing
    return this.getChassis().getCurrentChassisMoveName();
 }


 public void resetBotPosition()
 {
    // PRECONDITION - Bot must be part of a live Universe.
    // HSC 28-June-1999
    // Resets all bots joints to Default position

    // reset Joints
    BComponent[] components = this.getBreathComponents(true);

    for (int i =0; i < components.length; i++ )
    {
      components[i].inResultantMove = false;

      if ( components[i].getComponentType() == BJoint.COMPONENT_TYPE )
          ((BJoint)components[i]).applyTransform(true);


    }
    if (startPosition != null)
      setArenaPosition(startPosition);

    getChassis().terminateChassisMove(true);

    if (actionList != null)
    {
     actionList.clearActions();

    }

    if (opponentBot != null)
      lastOpponentPos = new BArenaObjectPosition(opponentBot.getArenaPosition());
    lastDistance = 0;

 }


 public void useEnergy(float e)
 {
   // BDebug.print ( "BBot useEnergy" + e + " total "  + totalEnergy );
    totalEnergy = totalEnergy - e;

    if (totalEnergy < 0)
      totalEnergy = 0;
 }

 public float getEnergy()
  { return totalEnergy ;}

 protected void setPercentageEnergyLevel(float e)
   { percentEnergy = e; }



 public float getPercentageEnergyLevel()
 {
    //BDebug.print ( this + "getPercentageEnergyLevel getPercentageEnergyLevel maxEnergy " + maxEnergy);
    //BDebug.print ( this + "getPercentageEnergyLevel getPercentageEnergyLevel totalEnergy" + totalEnergy);

    if ( maxEnergy > 0f )
      return totalEnergy / maxEnergy * 100f;
    else
      return percentEnergy;
 }

  public void addToBotDamage(float newDam)
  {
    if (newDam < 0)
      return;

    totalDamage = totalDamage + newDam;

    if (totalDamage > this.maxDamage)
      totalDamage = maxDamage;



  }


  protected void setPercentageDamageLevel(float p)
  {
    // Called by BBattleRecorder to keep bot's
    // damage percent correct in battle player.

    percentDamage = p;
  }

  public float getPercentageDamageLevel()
 {
    if (maxDamage > 0f)
    {
      return (totalDamage / maxDamage) * 100f;

    }
    else
      return percentDamage;
 }

 public double getCurrentDamage()
  { return (double)totalDamage; }



 public Enumeration getComponents()
 {
    return getChassis().getTreeNode().depthFirstEnumeration();
 }

 public BComponent[] getBreathComponents()
 {
  return getBreathComponents(false);
 }

 public BComponent[] getBreathComponents(boolean forceRebuild)
 {
        if ( (forceRebuild) || (breadthFirstComponents == null) )
          this.createBreathFirstAndLeafList();

        return breadthFirstComponents;

 }

 public BComponent[] getChassisAndLeaves()
 {
    if (chassisAndLeaves == null)
      this.createBreathFirstAndLeafList();

    return chassisAndLeaves;
 }

 public Bounds getBounds()
 {
    return getChassis().getLiveNode().getBounds();
 }

 private void activateZones()
 {
   // BDebug.print ("activateZones called from BBot");
    myZones.activateZones( getSensors() );
    myZones.clearDetected();
 }


  public boolean getContainsSensor()
  // returns true if the bot has at least 1 sensor
  {
    return ( getSensors().length > 0) ;

  }

  public boolean isBlind()
  {
    BSensor[] sensors = getSensors();
    boolean canSee = false;

    for (int i = sensors.length - 1; i >=0; i-- )
      canSee= canSee || !sensors[i].getDestroyed();

    return !canSee;

  }


  public BSensor[] getSensors()
  // PRE  initArenaDetails has been called
  {
    // if the cache is null create
    if (sensors == null)
    {
      TreeSet sensorBuildVector = new TreeSet() ;

       BComponent[] components = this.getBreathComponents();

       for (int i = 0 ; i < components.length ; i++ )
          if (components[i].getComponentType() == BSensor.COMPONENT_TYPE )
           sensorBuildVector.add((BSensor)components[i]);

       sensors = new BSensor[sensorBuildVector.size()];

       int i = 0;
       for (Iterator it = sensorBuildVector.iterator(); it. hasNext()   ;  i++)
       {
        sensors[i] = (BSensor) it.next();
        //BDebug.print( " inserting " + sensors[i]);
       }

    }

    // return the cache

    return sensors ;

  }



 public Vector getArenaComments()
 {
   if (arenaComments == null)
     arenaComments = new Vector();
   return arenaComments;
 }

 public void setArenaComments(String comment)
 {
   if (arenaComments == null)
     arenaComments = new Vector();

   //BDebug.print ("BBot.setArenaComment : " + comment );
   
   arenaComments.addElement(comment);
 }




public BranchGroup getStructure3D(boolean addZones, boolean addSensors)
  {


    DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode)getComponentTree().getRoot();
    Enumeration enumerator = rootNode.breadthFirstEnumeration();

    BComponent parentC = null;
    BComponent c= null;

    while (enumerator.hasMoreElements())
    {
       DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
       c = (BComponent)node.getUserObject();

       DefaultMutableTreeNode parentN = (DefaultMutableTreeNode)node.getParent();
       if ( parentN != null )
        parentC =  (BComponent)parentN.getUserObject();
       else
         parentC = null;

       loadComponent(c, parentC);

    }

    if (addZones)
    {
       // clean any old objects
       myZones.cleanseLiveNodes();

       BranchGroup zoneGroup = new BranchGroup();
       zoneGroup.setUserData(getName() + "BBot zoneGroup (New botLiveNode)" );

       zoneGroup.addChild(liveNode);
       zoneGroup.setCapability(BranchGroup.ALLOW_DETACH);
       liveNode = zoneGroup;

       // ZONEX  TODO
      // zoneGroup.addChild(botZones.getStructure3D());
    }

   // BDebug.printj3D(getName(), liveNode);

    return liveNode;

  }



  public void surrender()
  {
    this.setBotAlive(false);
    terminationString = this + " surrenders." ;

  }


  /* DJG June 2001
  public void recurseCollisionTest(BComponent a, BZone z, javax.media.j3d.Locale myLocale )
  // PRE::  a and z have intersecting bounds
  {
    //if (!a.getDestroyed() && !z.getDetected() )
    if (  !a.getDestroyed()   )
    {

      z.testCollision(a, myLocale);

      // z may have been detected otherwise check children
      //if (!z.getDetected() || BGlobal.getCollisionCode()== BGlobal.COLLISION_FULL )
      //{
          // Check all of the direct children
          DefaultMutableTreeNode treeNode =  a.getTreeNode();
          Enumeration immediateChildren = treeNode.children();
          // recurse chidren;
          // break if z gets detected
          while (immediateChildren.hasMoreElements() )
          {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)immediateChildren.nextElement();
            if (! ((BComponent)node.getUserObject()).getDestroyed() )
              if (  BCollision.boundsIntersect(z.getShape(), ((BComponent)node.getUserObject()).getLiveNode()) )
                recurseCollisionTest( (BComponent)node.getUserObject(), z,   myLocale );
          }



      //}


    }




  }
  */


 private Vector getPath (BComponent a, BComponent b)
 // pre a and b must be from this bot
 {
    Vector pathVector = new Vector();
    //BDebug.print ("BBot.getPath.entered");

    if (a.getTreeNode().isNodeDescendant(b.getTreeNode()) )
    {
      //BDebug.print ("BBot.getPath b.child of A");

      TreeNode current = b.getTreeNode();
      //pathVector.add(current);

      while (current != a.getTreeNode() )
      {
        current = current.getParent();
        if (current != a.getTreeNode() )
          pathVector.add(current);
      }


    }
    else if (b.getTreeNode().isNodeDescendant(a.getTreeNode()) )
     {
      //BDebug.print ("BBot.getPath A.child of B");

      TreeNode current = a.getTreeNode();
      //pathVector.add(current);

      while (current != b.getTreeNode() )
      {
        current = current.getParent();
        if (current != b.getTreeNode() )
          pathVector.add(current);
      }


    }
    else
    {
      //BDebug.print ("BBot.lowestCommonLimbFound  entered"  );

      BComponent[] components = this.getBreathComponents(false);
      boolean lowestCommonLimbFound = false;

      for (int i = components.length - 1; (i >= 0) && !lowestCommonLimbFound    ; i --)
      {
        if (  components[i].getTreeNode().isNodeDescendant(a.getTreeNode() )
              && components[i].getTreeNode().isNodeDescendant(b.getTreeNode() )    )

        {
          //BDebug.print ("BBot.lowestCommonLimbFound  " + components[i]);

          lowestCommonLimbFound = true;

          Vector aPath =  getPath (components[i], a) ;
          pathVector.addAll(aPath);
          Vector bPath =  getPath (components[i], b) ;
          pathVector.addAll(bPath);



        }

      }


    }


    return pathVector;


 }


 // working variable
 //private transient BCollisionRecord notInConnectedWithoutJointsCache_checkPair;

 protected boolean notInConnectedWithoutJointsCache(BComponent a, BComponent b)
 {
  if (a.getBot() == b.getBot())
  {
      //BDebug.print ("BBot.notInConnectedWithoutJointsCache entered " + a + " " + b);

      if (calculatedConnectedPairs == null)
        BDebug.print(" WARNING ASSERTION FAILURE  calculatedConnectedPairs == null ");

      if ( calculatedConnectedPairs.containsKey(  a.toString() + b.toString()  )   )

      {
         // BDebug.print ("BBot.getAreConnectedWithoutJoints found in cache for" +  a + " " + b );
          return false;
      }

  }




  return true;

 }

 private void addComponentToCollisionCache(BComponent c)
 {
    BComponent parentJoint = null;
    BComponentNode parent = c.getTreeNode();

    while (parentJoint == null)
    {
      parent = (BComponentNode)parent.getParent();
      if (  (parent.getComponent().isJoint() )
            || (parent.getComponent() instanceof BChassis)
          )
          parentJoint =  parent.getComponent() ;

    }


    DefaultMutableTreeNode current;
    Enumeration e = parent.depthFirstEnumeration();
    while(e.hasMoreElements())
    {
        current = (DefaultMutableTreeNode)e.nextElement();

        getAreConnectedWithoutJoints(c, (BComponent) current.getUserObject() );


    }





 }


 private void removeComponentFromCollisionCache(BComponent c)
 {


    String key =  c.toString();
    String currentKey;

    Enumeration keys = calculatedConnectedPairs.keys();
    Vector keysToRemove = new Vector();

    while (keys.hasMoreElements())
    {
      currentKey =  (String) keys.nextElement();

      if (key.length() < currentKey.length())
      {
        if (key.equals(currentKey.substring(0, key.length() ) ))
          keysToRemove.add(currentKey);
        else if (key.equals(currentKey.substring(currentKey.length() - key.length()  , currentKey.length() ) )  )
          keysToRemove.add(currentKey);
      }

    }

    for (int i=0; i< keysToRemove.size(); i++)
    {
      calculatedConnectedPairs.remove(keysToRemove.get(i));
      //BDebug.print ( "BBot removeComponentFromCollisionCache " + keysToRemove.get(i) );
    }



 }

 private boolean getAreConnectedWithoutJoints(BComponent a, BComponent b)
 {
  boolean noJoint = true;

  if (    (a.getBot() == b.getBot())
        &&(a != b)
        &&(a.getTreeNode().getParent() != b.getTreeNode() )
        && (b.getTreeNode().getParent() != a.getTreeNode() )
      )
  {


    // if not found in cache the check path
    Vector path = this.getPath(a,b);

    for (int i=0; i <  path.size() && noJoint; i++ )
    {

      BComponent c =  (BComponent) ((DefaultMutableTreeNode)path.get(i)).getUserObject();
      if (c instanceof BJoint)
        noJoint = false;

    }


    // if no joint then place in cache
    if (noJoint)
    {

      //BCollisionRecord thePair = new BCollisionRecord(a, this, b);
      String key1 =  a.toString()+ b.toString();
      String key2 =  b.toString() + a.toString();
      //BDebug.print ("getAreConnectedWithoutJoints adding " + key1);
      //BDebug.print ("getAreConnectedWithoutJoints adding "  +key2);


//      calculatedConnectedPairs.put(key1, key1 );
//      calculatedConnectedPairs.put(key2, key2 );
      calculatedConnectedPairs.put(key1, key1 );
      calculatedConnectedPairs.put(key2, key2 );


    }



  }
  else
    noJoint = false;

  //BDebug.print ("BBot.getAreConnectedWithoutJoints :: " +  noJoint );


  return noJoint;

 }

 public BArena getArena()
 { return arena ; }



 /* DJG June 2001
 private transient BComponent[] floorCollisionCache = null;  // this array is deleted in cleanseliveNodes

 public void testCollisionFloor(javax.media.j3d.Locale myLocale, Vector cols)
 {

    // initialise cahce
   if(floorCollisionCache == null)
   {
      Vector cacheWorking = new Vector();
      BComponent[] components = this.getBreathComponents();

      // starting at 1 so that we miss the chassis
      // get all the components that are leaves of the chassis
      for (int i = 1; i < components.length; i++)
      {
        if (!components[i].getChassisLock() && components[i].getParent().getChassisLock())
          cacheWorking.add(components[i] );

      }
      
      floorCollisionCache = new BComponent[cacheWorking.size()];
      for (int i = 0; i < cacheWorking.size(); i++)
        floorCollisionCache[i] = (BComponent) cacheWorking.get(i);


   }


   for (int i = 0; i < floorCollisionCache.length; i++)
      recurseCollisionTestFloor( floorCollisionCache[i],  cols, myLocale );


   // BDebug.printTime("Floor CollisionTime");
 }
 */

 /* DJG June 2001
 private void testArenaEdgeCollisions(Vector collisionList, javax.media.j3d.Locale myLocale)
 {
  //double arenaRadius = (arena.getSize() / 2.0) * BGlobal.WORLD_TO_CANVAS_SCALE ;

  if (BCollision.boundsFurtherThanDistance(arenaRadius, getChassis().getLiveNode(true), BPhysics.CENTEROFUNIVERSE_D ) )
    recurseCollisionTestWall(getChassis(), collisionList, myLocale);

 }
 */

 /* DJG June 2001
 public void recurseCollisionTestWall(BComponent a,  Vector collisionList, javax.media.j3d.Locale myLocale )

 {

    if (!a.getDestroyed() )                 // dont test destryed component
    {

      if (BCollision.outOfBounds(a, (arena.getSize() / 2.0) * BGlobal.WORLD_TO_CANVAS_SCALE) )
      {
          collisionList.add(new BCollisionRecord(a, this, BArena.arenaComponent));
          //BDebug.print ("BBot.recurseCollisionTestFloor Collision Detected With " + a);

      }

       // Check all of the direct children
      DefaultMutableTreeNode treeNode =  a.getTreeNode();
      Enumeration immediateChildren = treeNode.children();
      // recurse chidren;
      while (immediateChildren.hasMoreElements())
      {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)immediateChildren.nextElement();
            BComponent userComponent =   (BComponent)node.getUserObject();
            if (! userComponent.getDestroyed() )
                if (  BCollision.boundsFurtherThanDistance((arena.getSize() / 2.0)   * BGlobal.WORLD_TO_CANVAS_SCALE     , userComponent.getLiveNode(), BPhysics.CENTEROFUNIVERSE_D           ) )
                  recurseCollisionTestWall( userComponent, collisionList,  myLocale );


      }


    }

 }
 */

 public void recurseCollisionTestFloor(BComponent a, Vector collisionList, javax.media.j3d.Locale myLocale )

 {


    // check Component

    if (!a.getDestroyed() )                 // dont test destroyed components
      if (!a.getChassisLock() )           // dont test chassis lock components
        if (BCollision.intersectFloor(a) )
        {
          collisionList.add(new BCollisionRecord(a, this, BArena.arenaComponent));
          //BDebug.print ("BBot.recurseCollisionTestFloor Collision Detected With " + a);


        }
    // check children
    if (!a.getDestroyed() )
      {
          // Check all of the direct children
          DefaultMutableTreeNode treeNode =  a.getTreeNode();
          Enumeration immediateChildren = treeNode.children();
          // recurse chidren;
          while (immediateChildren.hasMoreElements())
          {
            DefaultMutableTreeNode node = (DefaultMutableTreeNode)immediateChildren.nextElement();
            BComponent userComponent =   (BComponent)node.getUserObject();
            if (! userComponent.getDestroyed() )
              if (!userComponent.getChassisLock() )
                if (  BCollision.boundsIntersectFloor(userComponent.getLiveNode()) )
                {
                  //BDebug.print ("BBot.limb int at " + userComponent );
                  recurseCollisionTestFloor( userComponent, collisionList,  myLocale );

                }
          }



      }







 }


 /*
 private void recurseCollisionTest(BComponent a, BComponent b, Vector collisionList,  javax.media.j3d.Locale myLocale, boolean recurseChildrenOfa)
 // PRE::  a and b have intersecting bounds
 {
   // BDebug.print ("BBot testCollision testing " + a + !a.getDestroyed() +" and " + b + !b.getDestroyed() );

    if ( !a.getDestroyed() && !b.getDestroyed() )
      if (BCollision.intersect(a, b, myLocale, true, true))           // do the intersect
      {
       // BDebug.print ("BBot testCollision coillision detected between " + a + " and " + b );
        BCollisionRecord testerCol =  new BCollisionRecord(a, this, b);


        if (BGlobal.PENETRATION_CORRECTION)
          for (int i = collisionList.size()-1; i >= 0; i-- )
          {
            BCollisionRecord iC = (BCollisionRecord)collisionList.get(i);
            BComponent commonComponent = testerCol.getCommonComponent(iC);

            if ( commonComponent != null )
            {
              BCollisionRecord bestCol = testerCol.getBestCollision(iC, commonComponent, myLocale);

              if ( bestCol == testerCol )
              {
                iC.setNullified(true);
                //collisionList.remove(i);
                //this.setArenaComments (testerCol+ " tested best over " +  iC );
              // this.opponentBot.setArenaComments (testerCol+ " tested best over " +  iC );
              }
              else
              {
                testerCol.setNullified(true);
                //this.setArenaComments (iC+ " tested best over " +  testerCol );
              // this.opponentBot.setArenaComments (iC+ " tested best over " +  testerCol );

              }
            }




          }


        collisionList.add(testerCol );

      }


      // Check all of b's direct children
      DefaultMutableTreeNode treeNode =  b.getTreeNode();
      Enumeration immediateChildren = treeNode.children();
      DefaultMutableTreeNode node;
      BComponent  nodeComponent;

      // recurse chidren;
      while (immediateChildren.hasMoreElements())
      {
        node = (DefaultMutableTreeNode)immediateChildren.nextElement();
        nodeComponent =  (BComponent)node.getUserObject();

        if (! nodeComponent.getDestroyed() )
            if (  BCollision.boundsIntersect(a.myShapesNode, nodeComponent.getLiveNode()) )
                recurseCollisionTest(a, nodeComponent, collisionList,  myLocale, false);

      }

    // BDebug.printTime("ChildrenOfB " + a + " and " + b);
    // BDebug.seedTimer();

      if (recurseChildrenOfa)
       {
          // Check all of a's direct children
          treeNode =  a.getTreeNode();
          immediateChildren = treeNode.children();


          // recurse chidren;
          while (immediateChildren.hasMoreElements())
          {
              node = (DefaultMutableTreeNode)immediateChildren.nextElement();
              nodeComponent =  (BComponent)node.getUserObject();

              if (! nodeComponent.getDestroyed() )
                 if (  BCollision.boundsIntersect(b.getLiveNode(), nodeComponent.getLiveNode()) )
                    recurseCollisionTest(b, nodeComponent, collisionList,  myLocale, true);

          }

       }

      // BDebug.printTime("ChildrenOfA " + a + " and " + b);

 }
 */

 private void addCollisionRecord(BComponent a, BComponent b, Vector collisionList,  javax.media.j3d.Locale myLocale)
 {
      // PRE: A and B bounds intersect

      //if ( a.toString().startsWith("C2 ") && b.toString().startsWith("C179 ") )
      //  BDebug.print("C2 & C179 - Bounds Collided");

      if (BCollision.intersect(a, b, myLocale, false, true))           // do the intersect
      {

        //if ( a.toString().startsWith("C2 ") && b.toString().startsWith("C179 ") )
        //  BDebug.print("- Polys Too!");
        //BDebug.print("POLYS Collided: "+a.toString()+" & "+b.toString());

        BCollisionRecord testerCol =  new BCollisionRecord(a, this, b);
        //MDF SC logic
        //orig
        //if (BGlobal.PENETRATION_CORRECTION)
        //if switch, and *both* components aren't chassis locked,and components arent part of the same bot 
        if (BGlobal.PENETRATION_CORRECTION && !(a.getChassisLock()&& b.getChassisLock()) && (a.ownerBot!=b.ownerBot))
        {
        	for (int i = collisionList.size()-1; i >= 0; i-- )
        	{
          	BCollisionRecord iC = (BCollisionRecord)collisionList.get(i);
          	BComponent commonComponent = testerCol.getCommonComponent(iC);

          	if ( commonComponent != null )
          	{
            	BCollisionRecord bestCol = testerCol.getBestCollision(iC, commonComponent, myLocale);

            	if ( bestCol == testerCol )
            	{
              	iC.setNullified(true);
            	} else {
              	testerCol.setNullified(true);
            	}
          	}
        	}
        }

        collisionList.add(testerCol);

      } //else { BDebug.print("Bounds Collided: "+a.toString()+" & "+b.toString()); }

 }
 // DJG - END

  private void recurseSelfCollisionTest(BComponent a, BComponent b, Vector collisionList,  javax.media.j3d.Locale myLocale, boolean recurseChildrenOfa)
 // PRE::  a and b are from the same Bot
 {
  //BDebug.print ("BBot.recurseSelfCollisionTest checking " + a + " and " + b );

  if ((a==b) && a.getIsTopOfStableLimb() )
  {
    // BDebug.print ("BBot.recurseSelfCollisionTest :Skipping Stable Limb " + a);
  }
  else
  {
   //  BDebug.print ("BBot.recurseSelfCollisionTest checking" + a + " and " + b );

    if ( !a.getDestroyed() && !b.getDestroyed() )
     {
      // check these shapes
          if (  !(a.getTreeNode() ==  b.getTreeNode().getParent() )   )     // if one is not the parent
            if (  !(a.getTreeNode().getParent() ==  b.getTreeNode() )   )   // if one is not the child
             if ( a != b ) // -------------------------------------------->  //if the components are not the same
               if(  ! (a.getChassisLock() && b.getChassisLock()    // if the 2 components are of the same bot and chassis locked
                      )

                  )
                   if (notInConnectedWithoutJointsCache(a,b)  )
                   {
                        if (BCollision.intersect(a, b, myLocale, true, true))           // do the intersect
                        {
                              collisionList.add(new BCollisionRecord(a, this, b));

                        }

                   }

      // BDebug.print ("BBot.recurseSelfCollisionTest checking b's children");

      // Check all of b's direct children
      BComponentNode treeNode =  b.getTreeNode();
      Enumeration immediateChildren = treeNode.children();
      BComponentNode node;
      BComponent  nodeComponent;

      // recurse chidren of b;
      while (immediateChildren.hasMoreElements())
      {
        node = (BComponentNode)immediateChildren.nextElement();
        nodeComponent =  node.getComponent();

        //BDebug.print ("BBot.testSelfCollision b's children "  + nodeComponent );

        if ((a==b) && node.isLeaf() )
        {
          //BDebug.print ("BBot.testSelfCollision skipping leafTest" );
        }
        else if (! nodeComponent.getDestroyed() )
          if (  BCollision.boundsIntersect(a.myShapesNode, nodeComponent.getLiveNode()) )
            recurseSelfCollisionTest(a, nodeComponent, collisionList,  myLocale, false);

      }

    // BDebug.printTime("ChildrenOfB " + a + " and " + b);
    // BDebug.seedTimer();

  //   BDebug.print ("BBot.recurseSelfCollisionTest checking a's children");
       if (recurseChildrenOfa)
       {
          // Check all of a's direct children
          treeNode =  a.getTreeNode();
          immediateChildren = treeNode.children();


          // recurse chidren;
          while (immediateChildren.hasMoreElements())
          {
              node = (BComponentNode)immediateChildren.nextElement();
              nodeComponent =  node.getComponent();

             // BDebug.print ("BBot.testSelfCollision a's children "  + nodeComponent );

              if (! nodeComponent.getDestroyed() )
                 if (  BCollision.boundsIntersect(b.getLiveNode(), nodeComponent.getLiveNode()) )
                    recurseSelfCollisionTest(b, nodeComponent, collisionList,  myLocale, true);

          }

       }


      // BDebug.printTime("ChildrenOfA " + a + " and " + b);


       }

  }//else stable Limb
 }

 private transient Vector getHighestJointsNotDefault_V;

 public Vector getHighestJointsNotDefault()
 {

   // clear vector
    if (getHighestJointsNotDefault_V == null)
      getHighestJointsNotDefault_V = new Vector();
    else
      getHighestJointsNotDefault_V.clear();


    BComponent[] components = getBreathComponents();

    for (int i = 0; i < components.length; i++ )
    {
      BJoint current;
      BComponent compInList;
      if (!components[i].getChassisLock() )
        if (components[i].isJoint() )
        {
          current = (BJoint) components[i];

          if (!current.getIsInDefaultPosition() )
          {

            boolean noJointsHigher = true;
            for (int innerC = 0; (innerC < getHighestJointsNotDefault_V.size() ) && noJointsHigher; innerC++ )
            {

              compInList = (BComponent)getHighestJointsNotDefault_V.get(innerC);
              if (current.getTreeNode().isNodeAncestor(compInList.getTreeNode() )  )
                noJointsHigher = false;
            }

            if (noJointsHigher)
            {
              getHighestJointsNotDefault_V.add(current);
              //BDebug.print("BBot.getHighestJointsNotDefault adding " +current );

            }

          }
        }

    }

    return getHighestJointsNotDefault_V;


 }


 private transient Vector getHighestLimbsInResultantMove_tempV;

 private Vector getHighestLimbsInResultantMove()
 {

  if ( getHighestLimbsInResultantMove_tempV == null)
    getHighestLimbsInResultantMove_tempV = new Vector();
  else
    getHighestLimbsInResultantMove_tempV.clear();


  actionList.getHighestLimbsInResultantMove( getHighestLimbsInResultantMove_tempV );

  return getHighestLimbsInResultantMove_tempV;


 }

 public boolean isExecutingResultantMoves()
 {
   return actionList.containsResultantMoves();
 }


 private Vector getComponentsConnectedToChassisLock_V;

 private Vector getComponentsConnectedToChassisLock()
 {
    if (getComponentsConnectedToChassisLock_V == null)
      getComponentsConnectedToChassisLock_V = new Vector();
    else
      getComponentsConnectedToChassisLock_V.clear();

    BComponent[] components = this.getBreathComponents();
    BComponent current;
    BComponent parent;

    for (int i = 1; i < components.length; i++)
    {
      current = components[i];
      parent = (BComponent) ((DefaultMutableTreeNode)current.getTreeNode().getParent()).getUserObject();


      if (parent.getChassisLock() && !current.getChassisLock() )
        getComponentsConnectedToChassisLock_V.add(current);

    } // for

    return getComponentsConnectedToChassisLock_V;

 }



 public void testSelfCollisions (   Vector collisionList ,  javax.media.j3d.Locale myLocale, boolean onlyCheckLimbsNotInDefaultPosition)
 {


      Vector limbsToCheck;

    //  BDebug.seedTimer();

      if (onlyCheckLimbsNotInDefaultPosition)
        limbsToCheck =  getHighestJointsNotDefault();
      else
        limbsToCheck = getComponentsConnectedToChassisLock();

      for (int i = 0; i < limbsToCheck.size(); i++)
        recurseSelfCollisionTest( getChassis(), (BComponent)limbsToCheck.get(i), collisionList, myLocale, true);



     // BDebug.printTime("BBot.testSelfCollisions: Bot Collision ");


 }

  public void testSelfCollisions (   Vector collisionList ,  javax.media.j3d.Locale myLocale, BComponent limbMoved)
 {

      //BDebug.printTime("BBot.testCollisions: single Limb " + limbMoved);

      //BDebug.seedTimer();

      recurseSelfCollisionTest( getChassis(),  limbMoved , collisionList, myLocale, true);

      //BDebug.printTime("BBot.testSelfCollisions:" );
      //for (int i = 0; i < collisionList.size(); i++ )
      //  BDebug.print(collisionList.get(i));


 }


// protected transient int testCollisionsCounter = 0;
  // DJG June 2001
  //private static transient Vector3f testCollisionV3F_A = new Vector3f();
  //private static transient Point3f testCollisionP3F_A = new Point3f();
  //private static transient Transform3D testCollisionT3D_A = new Transform3D();
  //private static transient Vector3f testCollisionV3F_B = new Vector3f();
  //private static transient Point3f testCollisionP3F_B = new Point3f();
  //private static transient Transform3D testCollisionT3D_B = new Transform3D();


 public void testCollisions (BBot otherBot,  Vector collisionList ,  javax.media.j3d.Locale myLocale, boolean worldStateCycle, boolean botsBoundsCollide, boolean checkAll)
 {
  // testCollisionsCounter ++;

   //BComponent myChassis = getChassis();
   //BComponent otherChassis = otherBot.getChassis();
   Node aNode;
   Node bNode;

   if ( worldStateCycle )
   {
     this.activateZones();
     otherBot.activateZones();
   }

   if (BGlobal.getCollisionCode() != BGlobal.COLLISION_MINIMAL )
   {

      // BDebug.seedTimer();
      // DJG June 2001

      if ( botsBoundsCollide )
      {

        //recurseCollisionTest(myChassis, otherChassis, collisionList, myLocale, true);

        float diffX = 0.0f;
        float diffY = 0.0f;
        float diffZ = 0.0f;
        float colDistance = 0.0f;
        boolean otherBotTested = false;

        for ( int i=0; i <  breadthFirstComponents.length; i++ )
        {

           BComponent a = breadthFirstComponents[i];

           if ( (!a.getDestroyed() || a.getChassisLock()) && (a.collisionCandidate || worldStateCycle || checkAll) )
           {

              //if ( a.toString().startsWith("C2 ") )
              //  BDebug.print("   C2 collisionCandidate="+a.collisionCandidate+"  moved="+a.hasMoved);

              if (a.liveShapes.size() > 1)
                aNode = a.getArbitraryShape_live();
              else
                aNode = a.myShapesNode;

              for ( int j=0; j <  otherBot.breadthFirstComponents.length; j++ )
              {
                BComponent b = otherBot.breadthFirstComponents[j];
                if ( (!b.getDestroyed() || b.getChassisLock()) && (b.collisionCandidate || worldStateCycle || checkAll) )
                {
                  //if ( a.toString().startsWith("C2 ") && b.toString().startsWith("C179 ") )
                	//	BDebug.print("   C179 collisionCandidate="+b.collisionCandidate+"  moved="+b.hasMoved);

                  // 1. Check if A or B moved
                  // 2. Check if A radius intersects B radius
                  // 3. Check Bounds

                  if (b.liveShapes.size() > 1)
                    bNode = b.getArbitraryShape_live();
                  else
                    bNode = b.myShapesNode;

                  if ( a.hasMoved || b.hasMoved || checkAll )
                  {
                    diffX = java.lang.Math.abs(a.currentX - b.currentX);
                    diffY = java.lang.Math.abs(a.currentY - b.currentY);
                    diffZ = java.lang.Math.abs(a.currentZ - b.currentZ);
                    colDistance = a.boundingRadius + b.boundingRadius;

                    //if ( a.toString().startsWith("C2 ") && b.toString().startsWith("C179 ") )
                    //    BDebug.print("- Coords: "+a.toString()+" "+b.toString()+" - x="+diffX+" - y="+diffY+" z="+diffZ+" dist="+colDistance);

                    if ( diffX <= colDistance && diffY <= colDistance && diffZ <= colDistance)
                    {
                      //if ( a.toString().startsWith("C2 ") && b.toString().startsWith("C179 ") )
                      //   BDebug.print("C2 & C179 - In range...");

            				  if (  BCollision.boundsIntersect(aNode, bNode) )
                				 addCollisionRecord(a, b, collisionList,  myLocale);

                    }
                  }

                  // Check Opponent Components for Arena/Zone Collisions
                  if ( ! otherBotTested ) // Only Test Once
                  {
                  	if ( otherBot.checkArenaCollisions )
                    {

                    	// Check collision with Arena Wall
                      if ( b.currentAbsRadius + b.boundingRadius > arenaRadius && b.hasMoved )
                      {
                      	if (BCollision.outOfBounds(b, arenaRadius * BGlobal.WORLD_TO_CANVAS_SCALE) )
                        {
          								collisionList.add(new BCollisionRecord(b, this, BArena.arenaComponent));
      									}
                  		}

                  		// Check collision with Arena Floor
                  		if ( !b.getChassisLock() && b.currentY - b.boundingRadius <= 0 && b.hasMoved )
                  		{
                    		if ( BCollision.intersectFloor(b) )
                    		{
          								collisionList.add(new BCollisionRecord(b, this, BArena.arenaComponent));
      									}
                  		}

                    }

                    if ( worldStateCycle )
                    {
                      BZone z;
                      for ( int k=0; k <  this.zoneList.length; k++ )
              				{
                				z = this.zoneList[k];
                        if (z.getActive())
                          // Note replace with distance test
                          //if ( BCollision.boundsIntersect( bNode,z.getShape()) )
                          z.testCollision(b, bNode, myLocale);

                        if (z.getDetected() )
                          this.getZones().anyZoneDetected = true;
                      }
                    }

                  } // Other Collisions

                }

              } // Next j

              otherBotTested = true;

              if ( this.checkArenaCollisions )
        			{

              	// Check collision with Arena Wall
              	if ( a.currentAbsRadius + a.boundingRadius > arenaRadius && a.hasMoved )
              	{
              		if (BCollision.outOfBounds(a, arenaRadius * BGlobal.WORLD_TO_CANVAS_SCALE) )
              		{
              			collisionList.add(new BCollisionRecord(a, this, BArena.arenaComponent));
              		}
              	}

              	// Check collision with Arena Floor
              	if ( !a.getChassisLock() && a.currentY - a.boundingRadius <= 0 && a.hasMoved )
              	{
              		if ( BCollision.intersectFloor(a) )
           				{
              			collisionList.add(new BCollisionRecord(a, this, BArena.arenaComponent));
              		}
              	}

              }

              // Check Zones
              if ( worldStateCycle )
              {
              	BZone z;
              	for ( int k=0; k <  otherBot.zoneList.length; k++ )
              	{
              		z = otherBot.zoneList[k];
              		if (z.getActive())
              			// Note replace with distance test
              			//if ( BCollision.boundsIntersect( aNode,z.getShape()) )
                    z.testCollision(a, aNode, myLocale);

              			if (z.getDetected() )
              				otherBot.getZones().anyZoneDetected = true;
                }
              }

    				} // Next i
        } // Not Destroyed

      } else { // Bot Bounds Don't Intersect

        // Test Arena/Zone Collisions - Self

          for ( int i=0; i <  breadthFirstComponents.length; i++ )
        	{
          	BComponent a = breadthFirstComponents[i];
          	if ( !a.getDestroyed() )
          	{

            	if ( this.checkArenaCollisions )
        			{
              	// Arena Edge
              	if ( a.currentAbsRadius + a.boundingRadius > arenaRadius && a.hasMoved )
              	{
              		if (BCollision.outOfBounds(a, arenaRadius * BGlobal.WORLD_TO_CANVAS_SCALE) )
              		{
              			collisionList.add(new BCollisionRecord(a, this, BArena.arenaComponent));
              		}
              	}

              	// Arena Floor
              	if ( !a.getChassisLock() && a.currentY - a.boundingRadius <= 0 && a.hasMoved )
              	{
              		if ( BCollision.intersectFloor(a) )
           				{
              			collisionList.add(new BCollisionRecord(a, this, BArena.arenaComponent));
              		}
              	}

              }

              // Zones
              if ( worldStateCycle )
              {
                if (a.liveShapes.size() > 1)
                  aNode = a.getArbitraryShape_live();
                else
                  aNode = a.myShapesNode;

              	BZone z;
              	for ( int k=0; k <  otherBot.zoneList.length; k++ )
              	{
              		z = otherBot.zoneList[k];
              		if (z.getActive())
              			// Note replace with distance test
              			//if ( BCollision.boundsIntersect( aNode,z.getShape()) )
                    z.testCollision(a, aNode, myLocale);

              			if (z.getDetected() )
              				otherBot.getZones().anyZoneDetected = true;
                }
              }

            }
          }


        	// Test Arena/Zone Collisions - Opponent

          for ( int i=0; i <  otherBot.breadthFirstComponents.length; i++ )
        	{
          	BComponent a = otherBot.breadthFirstComponents[i];
          	if ( !a.getDestroyed() )
          	{

              if ( otherBot.checkArenaCollisions )
        			{

              	// Arena Edge
              	if ( a.currentAbsRadius + a.boundingRadius > arenaRadius && a.hasMoved )
              	{
              		if (BCollision.outOfBounds(a, arenaRadius * BGlobal.WORLD_TO_CANVAS_SCALE) )
              		{
              			collisionList.add(new BCollisionRecord(a, this, BArena.arenaComponent));
              		}
              	}

              	//Arena Floor
              	if ( !a.getChassisLock() && a.currentY - a.boundingRadius <= 0 && a.hasMoved )
              	{
              		if ( BCollision.intersectFloor(a) )
           				{
              			collisionList.add(new BCollisionRecord(a, this, BArena.arenaComponent));
              		}
              	}

              }

              // Zones
              if ( worldStateCycle )
              {
                if (a.liveShapes.size() > 1)
                  aNode = a.getArbitraryShape_live();
                else
                  aNode = a.myShapesNode;

              	BZone z;
              	for ( int k=0; k <  this.zoneList.length; k++ )
              	{
              		z = this.zoneList[k];
              		if (z.getActive())
              			// Note replace with distance test
              			//if ( BCollision.boundsIntersect( aNode,z.getShape()) )
                    z.testCollision(a, aNode, myLocale);

              			if (z.getDetected() )
              				this.getZones().anyZoneDetected = true;
                }
              }

            }

        }

      } // End: Bounds dont Intersect

   }

   // Test Self Collisions for Resultant Move
   if ( BGlobal.getCollideSelf() && worldStateCycle )
   //if ( BGlobal.getCollideSelf() && !collisionList.isEmpty() && botsBoundsCollide && worldStateCycle)
   {
      /*
      // check this bot's resultant moves
      Vector tempV = getHighestLimbsInResultantMove();

      BDebug.print("BBot.testCollisions: testing self collisions - "+tempV.size());

      for (int i = 0; i < tempV.size(); i++ )
        recurseSelfCollisionTest(getChassis(), (BComponent)tempV.get(i), collisionList, myLocale, true);


      // check otherBots
      tempV = otherBot.getHighestLimbsInResultantMove();

      for (int i = 0; i < tempV.size(); i++ )
        otherBot.recurseSelfCollisionTest(otherBot.getChassis(), (BComponent)tempV.get(i), collisionList, myLocale, true);
      */
      if ( this.isExecutingResultantMoves() )
      	testSelfCollisions(collisionList, myLocale);
        
      if ( otherBot.isExecutingResultantMoves() )
      	otherBot.testSelfCollisions(collisionList, myLocale);

   }


   aNode = null;
   bNode = null;

 }

 // New Self Collision Code
 public void testSelfCollisions (Vector collisionList ,  javax.media.j3d.Locale myLocale)
 {

   //BDebug.print("BBot.testSelfCollisionsNew: "+this.toString());

   Node aNode;
   Node bNode;

   if ( BGlobal.getCollideSelf() )
   {

        float diffX = 0.0f;
        float diffY = 0.0f;
        float diffZ = 0.0f;
        float colDistance = 0.0f;

        for ( int i=0; i <  breadthFirstComponents.length; i++ )
        {

           BComponent a = breadthFirstComponents[i];

           if ( !a.getDestroyed() )
           {

              if (a.liveShapes.size() == 1)
                aNode = a.getArbitraryShape_live();
              else
                aNode = a.myShapesNode;

              for ( int j=0; j <  breadthFirstComponents.length; j++ )
              {
                BComponent b = breadthFirstComponents[j];

                boolean testFlag = true;

                if ( b.getDestroyed() )
                	testFlag = false;
                else if( a.getChassisLock() && b.getChassisLock() )  // if the 2 components are chassis locked
                  testFlag = false;
                else if ( a == b )  //if the components are the same
                  testFlag = false;
                //else if ( !a.hasMoved && !b.hasMoved )
                //  testFlag = false;
                else if ( a.getTreeNode() ==  b.getTreeNode().getParent() )     // if one is the parent
                  testFlag = false;
				else if ( a.getTreeNode().getParent() ==  b.getTreeNode() )   // if one is the child
                	testFlag = false;
                else if ( !notInConnectedWithoutJointsCache(a,b) ) // In same limb section (this should be redone DJG)
                	testFlag = false;                   //testFlag = true;

                if ( testFlag )
                {
                  // 1. Check if A or B moved
                  // 2. Check if A radius intersects B radius
                  // 3. Check Bounds

                  if (b.liveShapes.size() == 1)
                    bNode = b.getArbitraryShape_live();
                  else
                    bNode = b.myShapesNode;

                  diffX = java.lang.Math.abs(a.currentX - b.currentX);
                  diffY = java.lang.Math.abs(a.currentY - b.currentY);
                  diffZ = java.lang.Math.abs(a.currentZ - b.currentZ);
                  colDistance = a.boundingRadius + b.boundingRadius;

                  if ( diffX <= colDistance && diffY <= colDistance && diffZ <= colDistance)
                  {
            				  if (  BCollision.boundsIntersect(aNode, bNode) )
                				 addCollisionRecord(a, b, collisionList,  myLocale);

                  }

                }

              } // Next j

    				} // Next i

        } // Not Destroyed

   }

   aNode = null;
   bNode = null;

 }

 // calculate the absolute velocity of the opponent
 private transient BArenaObjectPosition lastOpponentPos;
 //private transient float lastDis = 0;
 private transient float opponentBotVelocity = 0;

 public float getOpponentVelocity()
 {
    return opponentBotVelocity;
 }

 // get the direction of the opponent velocity - 0 = straight towards this bot
 // 180 = directly away from this bot
 public double getOpponentVelocityAngle()
 {
   float x = opponentBot.getArenaPosition().getX();
   float y = opponentBot.getArenaPosition().getY();
   float z = opponentBot.getArenaPosition().getZ();
//   float x = getLastDetectedOpponentPosition().getX();
//   float y = getLastDetectedOpponentPosition().getY();
//   float z = getLastDetectedOpponentPosition().getZ();
  /*
   float mex = getArenaPosition().getX();
   float mey = getArenaPosition().getY();
   float mez = getArenaPosition().getZ();
  */
   Point3f currentP = new Point3f(x,y,z);
  // Point3f mecurrentP = new Point3f(mex,mey,mez);

   Point3f oldP = new Point3f(lastOpponentPos.getX(),
                              lastOpponentPos.getY(),
                              lastOpponentPos.getZ());

   Vector3f direction = new Vector3f();
 //  Vector3f mevec = new Vector3f();

   direction.sub(currentP, oldP);
   //mevec.sub(mecurrentP, oldP);

   if(direction.length() == 0)
    return 0;

   direction.normalize();

   if(direction.x == 0)
     direction.x = 0.00001f;

   /*
   // DJG Sep 2001
   // This seemed to have problems in certain circumstances
   if(currentP.x > oldP.x)
   	 facingAngle = Math.atan( (-direction.z / direction.x) ) + (Math.PI / 2);
   else
   	 facingAngle = Math.atan( (-direction.z / direction.x) ) + Math.PI + (Math.PI / 2);
   */

   double facingAngle = Math.atan( (direction.x / direction.z) );
   if ( direction.z < 0 )
   	facingAngle += Math.PI;

   //mevec.normalize();
   // Vector3f reference = new Vector3f(0,0,-1);

   //   float facingAngle = direction.dot(reference);
   //    float velocityAngle = 90 * (mevec.dot(direction));

   BArenaObjectPosition tempPos = new BArenaObjectPosition(x,y,z, (float)facingAngle);

   float velocityAngle = BPhysics.angleFrom(tempPos, getArenaPosition());

   return velocityAngle;
 }


 private transient float lastDistance = 0;

 public double getOpponentRelativeVelocity()
 {

    float distance = BPhysics.distanceBetween(this, opponentBot);
    float deltaDistance = lastDistance - distance;
    if (lastDistance == 0)
      deltaDistance = 0;
    lastDistance = distance;
    //return deltaDistance / lastExecuteTime;
    return deltaDistance / BGlobal.CALC_TIMESLICE;

    /*
    // get relative Vector
    getOpponentRelativeVelocity_myPos.set(getArenaPosition().getX(), 0, getArenaPosition().getZ() );
    //BDebug.print( "getOpponentRelativeVelocity_myPos" + getOpponentRelativeVelocity_myPos);

    getOpponentRelativeVelocity_oppPos.set(opponentBot.getArenaPosition().getX(), 0, opponentBot.getArenaPosition().getZ() );
    //BDebug.print( "getOpponentRelativeVelocity_oppPos" + getOpponentRelativeVelocity_oppPos);


    getOpponentRelativeVelocity_diff.sub( getOpponentRelativeVelocity_oppPos, getOpponentRelativeVelocity_myPos);
    //BDebug.print( "getOpponentRelativeVelocity_diff" + getOpponentRelativeVelocity_diff);

    // get opponent Velocity
    BChassis oppChassis = opponentBot.getChassis();
    oppChassis.getComponentVelocity( getOpponentRelativeVelocity_oppVelocity );
    getOpponentRelativeVelocity_oppVelocity.y = 0;
    //BDebug.print( "getOpponentRelativeVelocity_oppVelocity" + getOpponentRelativeVelocity_oppVelocity);


    getChassis().getComponentVelocity( getOpponentRelativeVelocity_myVelocity );
    getOpponentRelativeVelocity_myVelocity.y = 0;
    //BDebug.print( "getOpponentRelativeVelocity_myVelocity" + getOpponentRelativeVelocity_myVelocity);


    getOpponentRelativeVelocity_myVelocity.add(getOpponentRelativeVelocity_oppVelocity);
    // BDebug.print( "added Velocities " + getOpponentRelativeVelocity_myVelocity);

    float angleBetween =  getOpponentRelativeVelocity_diff.angle(getOpponentRelativeVelocity_myVelocity);
    //BDebug.print( "angleBetween" + angleBetween);

    float oppVelocity = getOpponentRelativeVelocity_myVelocity.length();
    //BDebug.print( "raw Velocity " + oppVelocity);

    //BDebug.print( "cos" + java.lang.Math.cos(angleBetween));

    if (oppVelocity == 0.0)
      return 0.0;
    else
      return oppVelocity * (float)java.lang.Math.cos(angleBetween);   */

 }


 private boolean jointParentInCollisionVector(BJoint j, Vector collisions)
 {
    boolean isParent = false;

    for (int i = collisions.size() - 1; i >=0 && !isParent; i--)
    {

      BCollisionRecord cR = (BCollisionRecord) collisions.get(i);
      if (cR.containsBot(this) )
        if (j == cR.armComponent ||  j == cR.triggerComponent ||  j.isParentOf(cR.armComponent) || j.isParentOf(cR.triggerComponent)  )
          isParent = true;

    }

    return isParent;
 }




 protected BJoint[] getCollisionRelatedMovedJoints(Vector collisions)
 {
  BComponent[] components = this.getBreathComponents();
  Vector movedJoints = new Vector();

  for (int i = components.length - 1; i >=0; i-- )
    if (!components[i].getDestroyed() )
      if (components[i].isJoint() )
      {
        BJoint j = (BJoint) components[i];
        //if (j.hasMoved())
          if (jointParentInCollisionVector(j, collisions) )
          {
            //BDebug.print (getName() + "BBot.getCollisionRelatedMovedJoints addingJoint " + j);
            movedJoints.add(j);
          }
      }

  BJoint[] jointArray = new BJoint[movedJoints.size()];
  for(int i = jointArray.length - 1; i >= 0 ; i-- )
    jointArray[i] = (BJoint) movedJoints.get(i);

  return jointArray;


 }

 public void completelyRollback(Vector collisions)
 {
   // Called from BSimulation if collisions still exist
   BJoint[] movedJoints = getCollisionRelatedMovedJoints(collisions);
   completelyRollback(movedJoints);
 }

 private void completelyRollback(BJoint[] movedJoints )
 {
    BComponent[] components = this.getBreathComponents();

    for (int i = movedJoints.length - 1; i >=0; i--)
      if ( !movedJoints[i].getChassisLock() )   // chassis locked components handled in next loop
        movedJoints[i].rollbackToPrev();

    // Roll back all chassis joints for first full rollback
    for(int i = components.length - 1; i >=0; i--)
    	if ( components[i].getChassisLock() )
          if (components[i].isJoint())
            ((BJoint)components[i]).rollbackToPrev();

    getChassis().rollbackToPrev();
    getChassis().setHeight();
    completelyRolledBack = true;
 }

 public void completelyRollback()
 {
    BComponent[] components = this.getBreathComponents();

    // Roll back all joints regardless
    for(int i = components.length - 1; i >=0; i--)
    	if (components[i].isJoint())
    		((BJoint)components[i]).rollbackToPrev();

    getChassis().rollbackToPrev();
    getChassis().setHeight();
    completelyRolledBack = true;
 }

 private void applyPositions(BJoint[] movedJoints, BJointPosition[] jointPositions)
 {
    for (int i = movedJoints.length - 1; i >= 0 ; i-- )
    {
      movedJoints[i].setJointPosition(jointPositions[i]);
      movedJoints[i].applyTransform();
    }

    getChassis().rollBack(bestChassisPosition);

 }


 private transient BJointPosition[] bestJointPositions;
 private transient BArenaObjectPosition bestChassisPosition;
 //private transient Vector passThroughCorrection;
 public transient boolean completelyRolledBack;

 public void rollBack(javax.media.j3d.Locale myLocale, Vector collisions, boolean botsBoundsCollide)
 {
    // null out recursion  results
    bestChassisPosition = null;
    bestJointPositions = null;
    completelyRolledBack = false;
    boolean botInCollision = false;

    BJoint[] movedJoints = getCollisionRelatedMovedJoints(collisions);
    //BJointPosition[] jointPositions = new BJointPosition[movedJoints.length];

    if ( movedJoints.length <= 0 )
    {
      for (int i = collisions.size() - 1; i >=0 && !botInCollision; i--)
    	{
      	BCollisionRecord cR = (BCollisionRecord) collisions.get(i);
      	if ( cR.containsBot(this) )
          	botInCollision = true;

    	}
    } else {
    	botInCollision = true;
    }

    /*
    BJoint lowestJoint = null;
    for(int i = movedJoints.length-1; i >=0; i--)
    {
      //jointPositions[i] = (BJointPosition) movedJoints[i].getPreviousJointPosition().clone();
      if ( lowestJoint == null )
        lowestJoint = movedJoints[i];
      else if ( lowestJoint.getTreeNode().isNodeDescendant(movedJoints[i].getTreeNode()) )
        lowestJoint = movedJoints[i];
    }

    // Pause all moves with Joints that are parents of lowest collision joint
    if ( lowestJoint != null )
       actionList.pauseMoves(lowestJoint);
    */

    /*
    // check to see if bot has moved at all
    if ( (movedJoints.length == 0) && getChassis().getPrevPosition().equals(getChassis().getArenaPosition()) )
    {
      //BDebug.print ( this + " BBot.rollback returning because bot did not move");
      return;
    }

    // DJG June 2001
    // Given the current accuracy (10cm) and move standoff/pause, not sure whether this is necessary
    // Especially since complete rollbacks often don't clear the collision (world update issue?)
    if ( ROLLBACK_DEPTH > 0 )
      recurseRollback(myLocale, collisions, movedJoints, jointPositions,new BArenaObjectPosition(getChassis().getPrevPosition()) ,1);
    */

    //if ( bestJointPositions == null )
    if ( botInCollision )
    {
      completelyRollback(movedJoints);
      //completelyRollback();
      //BDebug.print ( this + " BBot.rollBack Complete Rollback executed" );

    }
    /*
    else
    {
      //BDebug.print (this + "BBot.rollBack Partial Rollback executed" );
      applyPositions(movedJoints, bestJointPositions);

      // after partial rollback, check if anthing is colliding, if it is, do a complete rollback.
      // This is to stop the situation where rollback causes collisions that previouly did not occur due to pass-through.
      if (BGlobal.ADVANCED_PASSTHROUGH_CORRECTION)
      {
        if (passThroughCorrection == null)
          passThroughCorrection = new Vector();
        else
          passThroughCorrection.clear();

        testCollisions (opponentBot,  passThroughCorrection ,  myLocale, false, botsBoundsCollide, false);
        if (passThroughCorrection.size() != 0)
          completelyRollback(movedJoints);

      }

    }
    */



 }

 public static int ROLLBACK_DEPTH = 4;

 private void recurseRollback(javax.media.j3d.Locale myLocale, Vector collisions,BJoint[] movedJoints,BJointPosition[] jointPositions, BArenaObjectPosition chassisPos, int depth)
 {
  if (depth <= ROLLBACK_DEPTH)
  {
    // save current Position
    BJointPosition[] lastCurrent = new   BJointPosition[jointPositions.length] ;
    BArenaObjectPosition lastChassisPos =  new BArenaObjectPosition();
    lastChassisPos.set( getChassis().getArenaPosition() ) ;

    // apply avg of the 2
    for (int i = jointPositions.length - 1; i >=0; i--)
    {
      lastCurrent[i] = (BJointPosition) movedJoints[i].getJointPosition().clone();
      movedJoints[i].AVGandApply( jointPositions[i]);
     // BDebug.print ("BBot.recurseRollback testing " + movedJoints[i] + " jointPositions[i] : " + movedJoints[i].getJointPosition() );


    }

    getChassis().AVGandApply(chassisPos);

    //check collision and recurse as required

    if ( colliding (myLocale, collisions ) )
    {
     // BDebug.print ("BBot.recurse Collision collided recursing back ");
      recurseRollback( myLocale, collisions,movedJoints,jointPositions, chassisPos, depth + 1) ;
    }
    else
    {
      //BDebug.print ("BBot.recurseCollision no collision, recursing forward ");

      if ( bestJointPositions == null)
        bestJointPositions = new BJointPosition[jointPositions.length ];

      for (int i = bestJointPositions.length - 1; i >=0; i-- )
        bestJointPositions[i] = (BJointPosition) movedJoints[i].getJointPosition().clone();

      bestChassisPosition =   new BArenaObjectPosition();
      bestChassisPosition.set( getChassis().getArenaPosition() );

      recurseRollback( myLocale, collisions,movedJoints,lastCurrent, lastChassisPos, depth + 1) ;


    }


  }

 }

 private boolean colliding (javax.media.j3d.Locale myLocale,  Vector collisions)
 {
  boolean colliding = false;

  for (int i = collisions.size() - 1; i >=0 && !colliding; i--)
  {
    BCollisionRecord cR = (BCollisionRecord) collisions.get(i);
    if (cR.containsBot(this) )
      if (BCollision.intersect(cR.armComponent, cR.triggerComponent, myLocale, false, false) )
        colliding = true;


  }

  return colliding;

 }



 /*
 public void rollBack(javax.media.j3d.Locale myLocale, Vector collisions)
 {

    BComponent c = null;
    BComponent otherComponent = null;
    BComponentNode rootNode = getChassis().getTreeNode();


    for (int i =0; i< collisions.size(); i++ )
    {
     BCollisionRecord colRec = (BCollisionRecord) collisions.get(i);
   // BDebug.print ("BBot.rollback for " + colRec);



     if (colRec.triggerComponent.getBot() == this)
     {
      c = colRec.triggerComponent;
      otherComponent = colRec.armComponent;
     }
     else if (colRec.armComponent.getBot() == this)
     {
      c =  colRec.armComponent;
      otherComponent = colRec.triggerComponent;

     }
     else
      c = null;

    // BDebug.print ("BBot.rollback c=" + c);
   //  BDebug.print ("BBot.rollback otherComponent=" + otherComponent);


     if (c != null)
     {
      BComponentNode innerNode = (BComponentNode)c.getTreeNode();
      BComponent p = null;
      if (innerNode !=  null)
        p = innerNode.getComponent();
      boolean breakRoll = false;

      while ( !breakRoll && (innerNode !=  null) && (innerNode !=  rootNode) && ( innerNode !=  otherComponent.getTreeNode() )  )
      {


        if (p instanceof  BJoint )
        {
                     BJoint j = (BJoint)p;
                     j.rollBack(c,otherComponent, myLocale );
                   //  BDebug.print("BBot.rollback " + this + " collision rolling back joint " + j);

                     if (j.getlastRollbackwasClear() )
                     {
                     //  BDebug.print("BBot.rollback Joint Cleared : returning" );
                       breakRoll = true;
                       j.clearRollbackFlag();


                     }

        }

                innerNode = (BComponentNode)innerNode.getParent();
                p = innerNode.getComponent();
      }

      if (!breakRoll)
        getChassis().rollBack(c,otherComponent, myLocale );

     }
    }

 }   */


// private transient static Vector3f  createResultantMove_dotP = new Vector3f();
 private transient static Vector3f  componentInertia_tempV = new Vector3f();

 public void createResultantMove(javax.media.j3d.Locale myLocale, Vector collisions)
 {

   //BDebug.print("BBOT.resultant MOVES ");
   //BDebug.print( "BBot.createResultantMove: "+ this.toString() );

   //Vector3f netCurrent = new Vector3f(c.collisionComponent.collisionInertia);
   //Vector3f netCollision = new Vector3f(c.collisionInertia);

   BMove resultantMove = new BMove("-- recoil --", false);
   BSnapshot resultantSnap = new BSnapshot("-- recoil --");

   resultantMove.setIsResultantMove(true);

   //BJoint lowestJointResult = null;
   //BBot oppBot = null;
   // rollback the bot

   //DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
   BComponent c = null;
   BComponent otherComponent = null;
   BComponentNode rootNode = getChassis().getTreeNode();

   for (int i =0; i< collisions.size(); i++ )
   {
     BCollisionRecord colRec = (BCollisionRecord) collisions.get(i);
     //BDebug.print("BBOT.resultant colRec ::" + colRec);
     //BDebug.print("BBOT.resultant colRec.getNullified() " + colRec.getNullified());

     if (colRec.triggerComponent.getBot() == this)
     {
      c = colRec.triggerComponent;
      otherComponent = colRec.armComponent;
     }
     else if (colRec.armComponent.getBot() == this)
     {
      c =  colRec.armComponent;
      otherComponent = colRec.triggerComponent;

     }
     else
      c = null;


    //if(!colRec.getNullified() || colRec.isInContact() )
    if(!colRec.getNullified()  )
     if (c != null)
      if ( c.collisionInertia != null && c.currentInertia != null )
      {

       //oppBot = otherComponent.getBot();

        //if ( BGlobal.DEADLOCKDETECTED )
        //{
          //c.collisionInertia.normalize();
          //c.collisionInertia.scale(BPhysics.DEADLOCK_CLEARING_MOMENTUM);
          //setArenaComments ("------DEALOCK RESULTANT MOVE CREATED ------");
          //resultantMove.setName("Deadlock Clearance Move");

        //}

          //BDebug.print("BBOT.resultantMOVES  Collision was detected .. processing parents");

          // get First
          DefaultMutableTreeNode innerNode = (DefaultMutableTreeNode)c.getTreeNode().getParent();
          BComponent p = null;
          if (innerNode !=  null)
                p = (BComponent)innerNode.getUserObject();

        // check all parents, if it is a joint call  getResultantPosition
        // Drop out of the loop if the chassis is reached or the parent that this component collided with is reached

          while ( (innerNode !=  null) &&(innerNode !=  rootNode) && ( innerNode !=  otherComponent.getTreeNode() )  )
          {

                if (p instanceof BJoint)
                {
                     BJoint j = (BJoint)p;
                     p.inResultantMove = true;

                     if (c.collisionInertia.length()  <  BPhysics.MIN_MOMENTUM_TO_MOVE )
                     {
                        //BDebug.print ("BBot.createResultantMove Momentum to small to move Joint " + c.collisionInertia.length() +  " MIN " + BPhysics.MIN_MOMENTUM_TO_MOVE );
                     }
                     else
                     {

                        //BDebug.print ("BBot.createResultantMove movingJoint with " + c.collisionInertia    );

                        //lowestJointResult  = j;
                        BJointPosition newPos = (BJointPosition) j.getJointPosition().clone();
                        componentInertia_tempV.set(c.collisionInertia);
                        float percentAbsorbed = j.getResultantPosition ( componentInertia_tempV, c, newPos, otherComponent );

                        if (!newPos.equals(j.getJointPosition() ) )
                        {
                          // DJG June 2001
                        	// Stop massive ripple effect and incorrect results from getResultantPosition
                          // Will only move 3 joints
                          percentAbsorbed = java.lang.Math.abs(percentAbsorbed);

                          //BDebug.print( "- "+ j + " percentAbsorbed " + percentAbsorbed + " of Inertia "+c.collisionInertia.length() );

                          if ( percentAbsorbed > 0.0f ) // Joint has moved
                        	{
                          	percentAbsorbed = java.lang.Math.max(percentAbsorbed,0.2f); // No less than 20%
                          	percentAbsorbed = java.lang.Math.min(percentAbsorbed,0.999f); // No greater than 100%
                        	}

                          //BDebug.print( "- "+ j + " percentAbsorbed " + percentAbsorbed + " of Inertia "+c.collisionInertia.length() );

                          c.collisionInertia.scale(1.0f - percentAbsorbed);

                          BComponentTransform bcT =  new BComponentTransform(j);
                        //  BDebug.print( " adding EndPosition " + newPos.getAngle() );
                          bcT.setJointPosition ( newPos );
                          resultantSnap.add( bcT );
                        }

                      }

                }
                // get Next
                innerNode = (DefaultMutableTreeNode)innerNode.getParent();
                p = (BComponent)innerNode.getUserObject();

         }  // end of while


      // Process the Chassis
      if (  (c.collisionInertia.length()  >  BPhysics.MIN_MOMENTUM_TO_MOVE )    )
      {
          //BDebug.print(toString()+"BBot.createResultantMove: Creating Resultant Chassis move for "+c.collisionInertia.length() );
          chassis.createResultantMove(c, otherComponent);
      }


      c.collisionInertia = null;

      } // collision inertia



   }


   if (resultantSnap.getComponentTransforms ().size() > 0)
   {
        //BDebug.print ("BBot.createResultantMove addingResultantMove "   );

        //terminateMoveForComponent(lowestJointResult, oppBot);

        Vector snapStub = new Vector();
        snapStub.add( this.getSnapshot ("-- recoil --", true, false, false, resultantMove, 0) );
        resultantSnap.setDelayJoints(false);
        snapStub.add(resultantSnap);

        resultantMove.setSnapshots(snapStub);
        resultantMove.setOverriding(true);

        // DJG June 2001
        //resultantMove.calculateMovedJoints();
        resultantMove.initForSim(getChassis());

        actionList.addResultantMove(resultantMove, false);
        //actionList.addResultantMove(resultantMove, BGlobal.DEADLOCKDETECTED);


        //if (BGlobal.DEADLOCKDETECTED)
          //setArenaComments("PERFORMING DEALOCK CLEARANCE MOVE");
    }

 }



 public void clearCurrentJointTransforms()
 // resets the current Transform to be the same as defaut Tranform
 {

    BComponent[] components = this.getBreathComponents();

    for (int i = 0; i < components.length; i++)
    {

          BComponent c = components[i];


          if (c.getComponentType() == BJoint.COMPONENT_TYPE)
          {
            BJoint j = (BJoint) c;
            j.setJointPosition(    (BJointPosition)j.getDefaultJointPosition().clone() );
            j.applyTransform(false);

          }
    }


 }


 public void setDefaultJointTransformsToCurrent()
 // resets the current Transform to be the same as defaut Tranform
 {
    DefaultMutableTreeNode rootNode = getComponentTree().getRoot();
    Enumeration enumerator = rootNode.breadthFirstEnumeration();

        while (enumerator.hasMoreElements())
        {
          DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
          BComponent c = (BComponent)node.getUserObject();

          if (c.getComponentType() == BJoint.COMPONENT_TYPE)
            ((BJoint)c).setDefaultJointPosition(    (BJointPosition)((BJoint)c).getJointPosition().clone() );

        }


 }



  public void setBotAlive (boolean alive)
  {

    isAlive = alive;

    //if (getChassis().getDestroyed() )
    //  totalDamage =  botTotalHealth;


  }

  public void clearActions()
  {
    actionList.clearActions();
  }

  public boolean getBotAlive ()
  {
    return isAlive;
  }


  public void setBattleTime(float secs)
    { battleTime = secs; };

  public BBot getOpponentBot()
  { return opponentBot;}

  public BArenaObjectPosition getStartArenaPosition()
  { return startPosition; }

  public BArenaObjectPosition getLastDetectedOpponentPosition()
  {
    if (lastDetectedOpponentPos == null)
      return startPosition;
    else
      return  lastDetectedOpponentPos;

  }

  public void setBotSerialID(String botSerialIDIn)
  {
    botSerialID = botSerialIDIn;
  }

  public String getBotSerialID ()
  {
    //BDebug.print ( this + "pre bot SerialID " + botSerialID );

    if (botSerialID == null)
      botSerialID = ((int)( java.lang.Math.random() * 100 ) ) + "";

    //BDebug.print ( this + "post bot SerialID " + botSerialID );

    return botSerialID;
  }



  private static transient Point3d getBoundingRadius_up = new Point3d();
  private static transient Point3d getBoundingRadius_low = new Point3d();
  private static transient Transform3D getBoundingRadius_aVWorld = new Transform3D();


 public float getBoundingRadius()
 //PRE :: Must be part of a live scene graph
 //POST:: returns the radius  of the bounds in Canvas Units
 {
    float boundingRadius = 0.0f;

    BranchGroup centre = getLiveNode(true);
    Bounds bounds = centre.getBounds();

    centre.getLocalToVworld(getBoundingRadius_aVWorld);
    bounds.transform(getBoundingRadius_aVWorld);


    if (bounds instanceof BoundingBox)
    {
       BoundingBox bb = (BoundingBox) bounds;

       bb.getLower(getBoundingRadius_low);
       bb.getUpper(getBoundingRadius_up);

       boundingRadius = (float)  getBoundingRadius_up.distance(getBoundingRadius_low) / 2f;
       //BDebug.print("BBot.getBoundingRadius[box]: radius="+boundingRadius*BGlobal.CANVAS_TO_WORLD_SCALE);
    }
    else if (bounds instanceof BoundingSphere)
    {
      BoundingSphere sphere = (BoundingSphere)bounds;
      float radius =  (float)sphere.getRadius() ;
      sphere.getCenter(getBoundingRadius_up);
      //radius = radius; // +  (float)getBoundingRadius_up.distance(BPhysics.CENTEROFUNIVERSE_D);
      boundingRadius =  radius;
      //BDebug.print("BBot.getBoundingRadius[sphere]: radius="+boundingRadius*BGlobal.CANVAS_TO_WORLD_SCALE);

    }
    else
    {
      BDebug.print("BBot.getLongestDimension(): ERROR - Unhandled bounding type.");
      boundingRadius = 0.0f;
    }

    //BDebug.print("BBot.getBoundingRadius: radius="+boundingRadius*BGlobal.CANVAS_TO_WORLD_SCALE);

    return boundingRadius;

 }

 // DJG June 2001
 public float getBotReach()
 {
   // In Metres
   //if ( botReach <= 0.0f )
   //  calcBotReach();

   return botReach;
 }

 /*
 // DJG June 2001
 public void calcBotReach()
 {
   // Calculate Maximum Reach in Metres

   float chassisX = getChassis().getArenaPosition().x * BGlobal.CANVAS_TO_WORLD_SCALE;
   float chassisY = getChassis().getArenaPosition().y * BGlobal.CANVAS_TO_WORLD_SCALE;
   float chassisZ = getChassis().getArenaPosition().z * BGlobal.CANVAS_TO_WORLD_SCALE;

   botReach = 0.0f;

 	 for ( int i=0; i <  breadthFirstComponents.length; i++ )
   {

   		BComponent a = breadthFirstComponents[i];

      if ( !a.getDestroyed() && a.isLive() )
      {
        botReach = java.lang.Math.max(botReach, (java.lang.Math.abs(a.currentX - chassisX) + a.boundingRadius) );
        botReach = java.lang.Math.max(botReach, (java.lang.Math.abs(a.currentY - chassisY) + a.boundingRadius) );
        botReach = java.lang.Math.max(botReach, (java.lang.Math.abs(a.currentZ - chassisZ) + a.boundingRadius) );
      }
   }

   //BDebug.print("BBot.calcBotReach: botReach="+botReach);

 }
 */

 public BBot clone_shallow()
 {


  BBot o = null;
  try
  {
  o = (BBot)super.clone();
  }
  catch (CloneNotSupportedException e)
  { e.printStackTrace();}


  o.moveList = null;
  // XXX
  //TODO PLACE BACK IN
  o.ai = null;

  // o.aiVector = null;
  // o.decisionStates = null;

  o.botRotation = null;
  o.botTranslation = null;
  o.botOrigin = null;
  o.calculatedConnectedPairs = null;
  //o.myZones = null; // need zones for the stats



  return o;
 }

 public void removeThumbnails()
 {
  moveList.removeThumbnails();
 }




 public void recalculateConnectedWithoutJoints()
 {

  if (calculatedConnectedPairs != null)
    calculatedConnectedPairs.clear();
  else
    calculatedConnectedPairs = new Hashtable();

  BComponent[] components = this.getBreathComponents();

  BComponent a;
  BComponent b;

  for (int i = components.length-1 ; i >=0; i-- )
  {

    a=  components[i];
    for (int innerC = components.length-1 ; innerC >=0; innerC-- )
    {
      b= components[innerC];

      if (a!=b)
        if (! (a.getChassisLock() && b.getChassisLock() )  )
          getAreConnectedWithoutJoints( a,  b);
    }


  }

 }




    public void setCurrentlyDetectingEnemy(boolean b)
    {
      currentlyDetectingEnemy = b;

      if (b)
        lastDetectedOpponentPos.set(getOpponentBot().getArenaPosition() );
    }

    public boolean getCurrentlyDetectingEnemy()
    { return currentlyDetectingEnemy; }


    // returns bot type
    // Wizard bots will overwrite this to return "Wizard"
    public static String BOT_TYPE = "Bot";

    public String getType()
    { return BOT_TYPE; }

    public String getCreatorName()
    {
      if (creator == null)
        return "";
      else
        return creator;
    }

    public void setCreatorName(String creator )
    { this.creator = creator; }


    public long getTournamentEntryID()
    {
      return tournamentEntryID;
    }

    public void setTournamentEntryID(long setTournamentEntryID)
    {
      this.tournamentEntryID = setTournamentEntryID;
    }

    private transient float lastCollisionTime = -180;
    //private transient float lastUpdatedCollisionTime = 0;

    public void setCollided(boolean b, float time)
    {
    //BDebug.print (this + " set Collided " + b);
    collided = b;
    //lastUpdatedCollisionTime = time;
    if (b)
      lastCollisionTime = time;
    }

    public  boolean getCollided()
      { return collided; }

    public  float  getLastCollidedTime()
    {
      return lastCollisionTime;

    }

    public float getTimeSinceLastCollision()
    {
      return battleTime - lastCollisionTime;

    }
    public int getPolyCount()
    {
      BComponent components[] = getBreathComponents();
      int poly = 0;
      for (int i =  components.length - 1; i >= 0 ; i-- )
        poly = poly + components[i].getPolyCount();

      return poly;


    }

    public String getBotDescription()
      { return botDescription; }

    public void setBotDescription(String desc)
      {botDescription = desc;}


    
  // SJS [28/09/2000]: Bot Attributes...
  // -------------------------------------------------------------
    
  public BAttributeSet getAttributes()
  {
    BAttributeSet as = 
      new BAttributeSet();
    
    as.add(0, new BAttribute(
      "Cost", 
      new BAbsoluteValue(BMath.format(botTotalPrice)+ " "+ BGlobal.MONEY_UNIT),
      BAttribute.HIGH));
    
    as.add(0, new BAttribute(
      "Weight", 
      new BAbsoluteValue(BMath.format(botTotalWeight)+ " "+ BGlobal.WEIGHT_UNIT),
      BAttribute.HIGH));

    as.add(0, new BAttribute(
      "Hit points", 
      new BAbsoluteValue(BMath.format(botTotalHealth)+ " "+ BGlobal.HEALTH_UNIT),
      BAttribute.HIGH));

    as.addAll(chassis.getChassisAttributes());
    
    return as;
  }
  

  public void setTerminationString(String s)
    { terminationString = s; }

  public void setOrientation()
  {
    Matrix3f id = new Matrix3f();
    id.setIdentity();

    getChassis().setOrientation(id);
    getChassis().setHeight();
  }

  public BComponent getComponent(String name)
  {
    BComponent[] cArray = this.getBreathComponents();
    for (int i = cArray.length -1 ; i >= 0 ; i-- )
      if (cArray[i].getName().equals(name) )
        return cArray[i];

    return null;

  }


  public void setTransparency(float t)
    { setTransparency(t, false); }

  public void setTransparency(float t, boolean updateDefault)
  {
    this.transparency = t;

    // If bot is totally transparent, don't render it...
    setVisible(t < 1f);

    BComponent[] cs = this.getBreathComponents();
    for (int i = 0; i < cs.length; i++ )
      cs[i].setTransparency(t, updateDefault);
  }

  public void setVisible(boolean v)
  {
    if (!isVisible && v)
      updateVisibility(true);
    else if (isVisible && !v)
      updateVisibility(false);

    this.isVisible = v;
  }

  public float getTransparency()
    { return transparency; }


  protected void updateVisibility(boolean v)
  {
    BComponent[] cs = this.getBreathComponents();

    for (int i = 0; i < cs.length; i++ )
      cs[i].setVisible(v);
  }


  public void restoreTransparency()
  {
    // Ensure bot is visible...
    setVisible(true);

    BComponent[] cArray = this.getBreathComponents();
    for (int i = 0; i < cArray.length; i++ )
    {
      cArray[i].setTransparency(cArray[i].getTransparency(), false);
      cArray[i].displayColor(cArray[i].getColor());
    }

    transparency = 0f;
  }



  public BMove createNewMove(String name)
 {

    BMove m = new BMove(name);

    BSnapshot s = getDefaultSnapshot("Initial Position");

    m.addSnapshot(s);
    m.setLimbLocked(false, this);

    return m;
 }



  // BAlphaAcceptor implementation...
  // -------------------------------------------------------------
  // SJS [23/03/2001]

  public void setAlphaValue(BAlphaValue value)
  {
    BArenaObjectPosition p
      = (BArenaObjectPosition) value;

    setArenaPosition(p);
  }


  /* DJG June 2001
  public transient Vector3f fastestV;

  public float getFastestComponentSpeed()
  {

    if (fastestV == null)
      fastestV = new Vector3f();

    float fastestSpeed = 0;

    BComponent[] components = this.getBreathComponents();

    for(int i = components.length - 1; i >=0; i--)
    {
      if (!components[i].getChassisLock() )
      {
        components[i].getVelocity(fastestV);
        fastestSpeed = Math.max(fastestSpeed, fastestV.length() );
      }
    }

   // BDebug.print (getName() + " getFastestComponentSpeed: " +  fastestSpeed );
    return  fastestSpeed;

  }
  */

  // DJG June 2001
  public void pauseChassisMove()
  {
      getChassis().pause();
  }

  public void pauseMoves(BComponent collidedComponent)
  {
    actionList.pauseMoves(collidedComponent);
  }

  public void stopMoves(BComponent collidedComponent)
  {
    actionList.stopMoves(collidedComponent);
  }

  // DJG June 2001
  // This is required to ensure a complete snapshot of a bots position is saved
  // To ensure "clean" rollbacks.
  private void savePrevArenaPosition()
  {
  	for ( int i=0; i <  breadthFirstComponents.length; i++ )
  	{

  		BComponent a = breadthFirstComponents[i];

  		if ( (!a.getDestroyed() || a.getChassisLock()) && a.isJoint() )
  		{
        ((BJoint)a).setPreviousJointPosition();
  		}

  	}

    getChassis().setPreviousArenaPosition();

  }

  public float getTimer()
  {
    if ( getChassis() != null )
      return getChassis().getTimer();
    else
      return 0f;

  }

  public void resetForSim()
  {

    BComponent[] components = this.getBreathComponents(true);

    for (int i =0; i < components.length; i++ )
    {
      components[i].resetForSim();
    }

    if ( moveList != null )
      moveList.cleanseLiveNodes();

  }

  // Used for dumping bot data for debugging
  public void debugDump()
  {

    BComponent[] components = this.getBreathComponents(true);

    for (int i =0; i < components.length; i++ )
    {
      BComponent bc = components[i];
      Vector3f v3f = bc.getCachedVWorld();
      BDebug.print(bc.toString()+" "+v3f.length());
    }

    if ( actionList != null )
    {
      Vector names = actionList.getActionNames();
      for (int i=0; i < names.size(); i++ )
      {
        BDebug.print("Move: "+names.get(i));
      }
    }

    BDebug.print("Chassis: "+getChassis().getCurrentChassisMoveName());
    BArenaObjectPosition cPos = getChassis().getArenaPosition();
    BDebug.print("Chassis: x="+cPos.x+" y="+cPos.y+" z="+cPos.z);

  }


 }///:~

