package roboforge;

//: BBotPicker.java

import roboforge.ui.events.*;

import java.awt.event.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;

/*

* Description:
  Enables the user to pick a bot when creating a new bot.

* Example:  BBot myBot = BBotPicker.getBot(this);
* Modification History ( Name, Date, Notes )
  DJG, 20 April 1999


*/

public class  BBotPicker extends JDialog
{
  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
static int FRAMESIZE = 2;
  BBot selectedBot;
  TreePath lastFPath;
  BFileSelectionTree lastFTree;

  public BBotPicker(JFrame frame)
  {
    this(frame, BGlobal.BOTPATH, BGlobal.BOTSUFFIX);
  }

  public BBotPicker(JFrame frame, String startPath, String filter)


  {

    super(frame,"Select a Bot",true);

    this.getContentPane().setLayout(null);
    this.setSize(BGlobal.FRAMEWIDTH/3+6,BGlobal.FRAMEHEIGHT - 75);
    this.setLocation(BGlobal.FRAMEWIDTH/3*2-10,25);

    try
    {
      // FILE LOAD: Set Up File Tree and add Listeners
      //UIManager.put("Tree.leafIcon",new IconUIResource(new ImageIcon(BGlobal.IMAGEPATH+"component.gif")));
      //UIManager.put("Tree.openIcon",new IconUIResource(new ImageIcon(BGlobal.IMAGEPATH+"tboxopen.gif")));
      //UIManager.put("Tree.closedIcon",new IconUIResource(new ImageIcon(BGlobal.IMAGEPATH+"tboxclosed.gif")));

      BFileSelectionTree fTree = new BFileSelectionTree(startPath, filter, "Bot Library",true);

      fTree.putClientProperty("JTree.lineStyle","Angled");
      fTree.addTreeSelectionListener(new FTreeSelectionHandler());
      // Set Tree Cell Renderer Parameters
      ComponentUI ftui = fTree.getUI();
      if (ftui instanceof BasicTreeUI)
      {
        fTree.setRowHeight(20);
        ((BasicTreeUI)ftui).setRightChildIndent(21);
        //((BasicTreeUI)ftui).setCollapsedIcon(new ImageIcon(BGlobal.IMAGEPATH+"collapsed.gif"));
        //((BasicTreeUI)ftui).setExpandedIcon(new ImageIcon(BGlobal.IMAGEPATH+"expanded.gif"));
      }
      fTree.setVisible(true);

      if (BGlobal.DEPRECATED)
      {
        fTree.addMouseListener(new MouseAdapter()
        {
          public void mouseClicked(MouseEvent evt)
          {
            if (evt.getClickCount()==2)
              { loadBotAndReturn(); }
          }
        });
      }

      JScrollPane scrollPane = new JScrollPane(fTree);
      scrollPane.setSize(BGlobal.FRAMEWIDTH/3 - FRAMESIZE*2,BGlobal.FRAMEHEIGHT - FRAMESIZE*4 - 130);
      scrollPane.setLocation(FRAMESIZE,FRAMESIZE);
      this.getContentPane().add(scrollPane);
    }
    catch (Throwable t)
      { t.printStackTrace(); }

    JButton button = new JButton("Select Bot");

    if (BGlobal.DEPRECATED)
      button.addActionListener(new SyncLoader());
    else
      button.addActionListener(new AsyncLoader());


    button.setSize(BGlobal.FRAMEWIDTH/3 - FRAMESIZE*2,30);
    button.setLocation(FRAMESIZE,BGlobal.FRAMEHEIGHT - FRAMESIZE*2 - 130);
    this.getContentPane().add(button);

    this.setVisible(true);

  }

  protected class SyncLoader implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
     { loadBotAndReturn(); }
  }

  protected class AsyncLoader extends BAsyncActionAdapter
  {
    public void asyncActionApproved(ActionEvent e)
     { loadBotAndReturn(); }
  }


  protected void loadBotAndReturn()
  {
    BDebug.print("CURRENT THREAD (BotPicker): "+ Thread.currentThread());

    loadFilePath();
    BBotPicker.this.setVisible(false);
    BBotPicker.this.dispose();
  }

  static public BBot getBot(JFrame f)
  {
  BBotPicker botPicker = new BBotPicker(f);
    return botPicker.getValue();

  }


  static public BBot getBot(JFrame frame, String startPath, String filter)
  {
    BBotPicker botPicker = new BBotPicker(frame, startPath, filter);
    return botPicker.getValue();
  }


  private BBot getValue()
  {
    return selectedBot;
  }

  // Load File Path
  private void loadFilePath()
  {
    try
    {
      String fname = lastFTree.getPathName(lastFPath);
      File f = new File(fname);
      if (f.isFile())
      {

        // Instantiate BBot
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
        System.out.println("Loading Bot File "+fname);
        selectedBot = BBot.loadFromFile(fname);
        selectedBot.setBotFilePath(fname);


      }
    }
    catch (Throwable t)
    {
      // Do Nothing
    }
  }

  // LISTENER HANDLERS

  // File Tree
  private class FTreeSelectionHandler implements TreeSelectionListener
  {
    public void valueChanged(TreeSelectionEvent evt)
    {
      try
      {
        lastFPath = evt.getPath();
        lastFTree = (BFileSelectionTree)evt.getSource();
      } catch (Throwable t){
        // Do Nothing
      }
    }
  }

}///:~