package roboforge;

//import java.awt.Color;
import java.util.*;
//import javax.swing.border.*;
//import javax.swing.plaf.metal.*;
//import javax.swing.*;
import java.io.*;

//import roboforge.animation.*;
//import roboforge.particles.*;
//import roboforge.particles.systems.*;
import roboforge.sound.*;
import roboforge.util.text.BStrings;

//import javax.media.j3d.*;


public class BComponentMaterial extends Object implements Serializable
{

  static final long serialVersionUID = -1L;

  // MEMBERS
  public float density = 0.0f;     //grams/cm^3 Industry standard
  public float damageToWeightRatio = 0.0f;  // Better Materials have higher values
  public float baseDamage = 0.0f;           // Base damage value for component materials
  public String name = null;
  public int type = DENSE_METAL; // Dense Metal default

  public String destructSoundFile = null;
  private String materialCode = null;

  private static BComponentMaterial DEFAULTMATERIAL = getMaterial("std_alloys");

  // STATICS

  // Static Vector holding all Materials
  private static Vector materials = new Vector();

  // used for sound processing
  public static int DENSE_METAL = 0;
  public static int HOLLOW_METAL = 1;
  public static int PLASTIC = 2;
  public static int GLASS = 3;
  public static int ENERGY = 4;
  public static int FLESH = 5;
  public static int STONE = 6; // Ceramic etc   *** IMPORTANT, READ BELOW
  // IF YOU CHANGE LAST INDEX MUST BE (CURRENTLY STONE) YOU MUST CHANGE THE  maxIndex (below)
  public static int MAXINDEX =  STONE + 1;
  private static String DEFAULT_COLLISIONSOUND = "collisions/clash.wav";
  private static String DEFAULT_DESTRUCTSOUND = "explosions/standardexplosion.wav";

  public static int DEFAULT_TYPE =  DENSE_METAL;




  private static String[][] collisionSoundFileNames = null;
  // private static String[] destructFileNames = null; // Sound File Names

  private static Hashtable soundCache;



  public BComponentMaterial(String newName, float newdensity, int newType, float dmgRatio, float baseDmg, String newDestructSeqFile)
  {
    density = newdensity;
    name = newName;
    type = newType;
    damageToWeightRatio = dmgRatio;
    baseDamage = baseDmg;
  }

  public String getCode()
  {
    return materialCode;

  }

  public String getName()
  {
    return name;
  }

  public void setCode(String materialCode)
  {
    this.materialCode = materialCode;

  }

  public static void initMaterialSounds()
  {

    // collision Names
    collisionSoundFileNames = new String[MAXINDEX][MAXINDEX];

    collisionSoundFileNames[HOLLOW_METAL][GLASS] = "collisions/chink.wav";
    collisionSoundFileNames[GLASS][GLASS] = "collisions/chink.wav";
    collisionSoundFileNames[HOLLOW_METAL][HOLLOW_METAL] = "collisions/clash.wav";
    collisionSoundFileNames[HOLLOW_METAL][DENSE_METAL] = "collisions/clashsolid.wav";
    collisionSoundFileNames[DENSE_METAL][DENSE_METAL] = "collisions/clashsolid.wav";
    collisionSoundFileNames[DENSE_METAL][STONE] = "collisions/clashsolid.wav";

    collisionSoundFileNames[ENERGY][ENERGY] = "collisions/energy.wav";
    collisionSoundFileNames[ENERGY][DENSE_METAL] = "collisions/energy.wav";
    collisionSoundFileNames[ENERGY][HOLLOW_METAL] = "collisions/energy.wav";
    collisionSoundFileNames[ENERGY][PLASTIC] = "collisions/energy.wav";
    collisionSoundFileNames[ENERGY][FLESH] = "collisions/energy.wav";
    collisionSoundFileNames[ENERGY][STONE] = "collisions/energy.wav";

    collisionSoundFileNames[FLESH][FLESH] = "collisions/organic.wav";
    collisionSoundFileNames[FLESH][STONE] = "collisions/organic.wav";
    collisionSoundFileNames[FLESH][GLASS] = "collisions/organic.wav";
    collisionSoundFileNames[FLESH][PLASTIC] = "collisions/organic.wav";
    collisionSoundFileNames[FLESH][ENERGY] = "collisions/organic.wav";

    collisionSoundFileNames[PLASTIC][PLASTIC] = "collisions/plastic.wav";
    collisionSoundFileNames[PLASTIC][GLASS] = "collisions/plastic.wav";
    collisionSoundFileNames[PLASTIC][STONE] = "collisions/plastic.wav";

  }

  public BSound getDestructSound()
    { return BComponentMaterial.getDestructSound(type); }

    
  public static BSound getDestructSound(int materialType)
  {
    // if (destructFileNames == null)
    //   initMaterialSounds();

    if (soundCache == null)
      soundCache =  new Hashtable();

    String fileName  = null;

    // if ( materialType <=  MAXINDEX -1)
    //  fileName = destructFileNames[materialType];

    // if ((fileName == null) || fileName.equals("") )
      fileName =  DEFAULT_DESTRUCTSOUND;


    BSound theSound;
    String soundName = BGlobal.SOUNDPATH+fileName;


    if (soundCache.containsKey(soundName))
    {
      theSound = (BSound)soundCache.get(soundName);

    }
    else
    {
      theSound =  BSound.getSound(soundName);
      soundCache.put(soundName, theSound);

    }

    return theSound;
  // return BSound.getSound(BGlobal.SOUNDPATH+fileName);

  }



  public static BSound getCollisionSound( int type1, int type2 )
  {
    if (collisionSoundFileNames == null)
      initMaterialSounds();

    if (soundCache == null)
      soundCache = new Hashtable();

    String colSoundName= null;


    if ( (type1 <MAXINDEX ) &&  (type2 <MAXINDEX )  )
    {
      colSoundName =  collisionSoundFileNames[type1][type2];

      // if null try the reverse
      if (colSoundName == null)
        colSoundName =  collisionSoundFileNames[type2][type1];
    }

    if (colSoundName == null)
      colSoundName = DEFAULT_COLLISIONSOUND;

    //BDebug.print("BComponentMaterial.getCollisionSound (" + type1 + ") ( " + type2);

    BSound theSound;
    String soundName = BGlobal.SOUNDPATH+colSoundName;


    if (soundCache.containsKey(soundName))
    {
      theSound = (BSound)soundCache.get(soundName);
      //BDebug.print ("BComponentMaterial found cache");
    }
    else
    {
      theSound =  BSound.getSound(soundName);
      soundCache.put(soundName, theSound);

    }

    return theSound;



  }




  public String toString()
  {
    return name;
  }




  public static Vector getAllMaterials()
  {

      //BDebug.print ("BComponentMaterial getAllMaterials called" );
      materials.clear();

      try {

        String path = BGlobal.MATERIALPATH;
        path = path.substring(0,(int) path.length() - 1);
        File f = new File(path);
        String[] names = f.list();
        // Process Files
        for (int i = 0; i < names.length; i++)
        {
          String name = names[i];
          File d = new File(path, name);
          if ( !d.isDirectory() )
          {
            materials.add(BComponentMaterial.loadFromFile(BGlobal.MATERIALPATH+name));
          }
        }


      }
      catch (Exception e)
      {
			 //	BDebug.print("Materials Load Failed: "+e.toString());
      }

    return materials;
  }


  public static void clearCache()
  {
    // destructFileNames = null;
    materials.clear();
    soundCache = null;

    collisionSoundFileNames = null;
    // destructFileNames = null;




  }



  
  public static BComponentMaterial getMaterial(String theCode)
  {

    // trap old components
    if(  ( theCode == null) || (theCode.equals("") ) )
    {
      //BDebug.print ("BComponentMaterial getMaterial :: MATERIAL NOT FOUND RETURNING DEFAULT" );
      return DEFAULTMATERIAL;

    }



    if (materials == null)
      materials = new Vector();

    // LOOK IN THE CACHE
    BComponentMaterial thisMaterial;
    for ( int i=0; i<materials.size(); i++ )
    {
     thisMaterial = (BComponentMaterial)materials.get(i);
     if ( thisMaterial.getCode().equals(theCode) )
     {
        //BDebug.print ("BComponentMaterial getMaterial:: Material found in cache " + theCode);
        return thisMaterial;


     }
    }

    // Material is Not in cache, load it int Materials

    BComponentMaterial theMat = BComponentMaterial.loadFromFile(BGlobal.MATERIALPATH+ theCode + BGlobal.MATERIALSUFFIX);

    if (theMat == null)
    {
      //BDebug.print ("BComponentMaterial getMaterial :: MATERIAL NOT FOUND RETURNING DEFAULT" );
      return DEFAULTMATERIAL;

    }
    else
    {

      materials.add(theMat);
      //BDebug.print ("Material Loading from file " + theCode);
      return theMat;
    }

    //return null; // Not Found





  }

  public boolean equals(Object o)
  {
    if (o instanceof BComponentMaterial)
      return ((BComponentMaterial)o).getCode().equals(getCode() );

    return false;

  }

  public static BComponentMaterial loadFromFile(String filename)
  {
    BComponentMaterial newIn = null;
    filename = BStrings.convertToFilePath(filename);

    try
    {

      FileInputStream fin = new FileInputStream(filename);
      ObjectInputStream in = new ObjectInputStream(fin);
      newIn = (BComponentMaterial)in.readObject();
      //newIn.init();
      in.close();
      fin.close();

    } catch (Exception e) {BDebug.print(e.getMessage());}

    return newIn;

  }

  public String getFileName()
  {
    // Added 'toLowerCase' to ensure lower case file names (AWH 11 Jan 01)
    return BGlobal.MATERIALPATH + materialCode.toLowerCase() +  BGlobal.MATERIALSUFFIX;
  }

  public void saveToFile()
  {

    // change to save under description name

    String fileName = getFileName();

    try{

      //if (  ! BGlobal.MATERIALSUFFIX.equalsIgnoreCase(     filename.substring(filename.length() - 4 )  )  )
      //          filename = filename +  BGlobal.MATERIALSUFFIX;


      FileOutputStream fout = new FileOutputStream(fileName);
      ObjectOutputStream out = new ObjectOutputStream(fout);
      out.writeObject(this);
      out.close();
      System.out.println(this.getCode() +" Save completed");
    }
    catch (Exception e) {
      BDebug.print("BComponentMaterial.saveToFile " + e.getMessage() );
    }
  }


} ///:~





