package roboforge;

//import com.sun.j3d.utils.geometry.Primitive;
//import com.sun.j3d.utils.geometry.Cone;
//import com.sun.j3d.utils.geometry.Sphere;
//import com.sun.j3d.utils.geometry.ColorCube;

import javax.media.j3d.*;
import javax.vecmath.*;
//import java.awt.event.*;
import java.awt.Color;


// import roboforge.ui.components.BUIColorScheme;

import roboforge.geometry.*;
import roboforge.animation.*;
import roboforge.animation.animation.*;


public class BConnectPointShape extends Shape3D implements BHighlightShape
{
 /*
  * Description:  The Shape of a connect point
  * Modification History ( Name, Date, Notes ):
                         YLY, 31- Mar 99
                         YLY, 6 Apr 99, Changes Colours
                         YLY, 7 MAr 99, Changed SetHilight
                         YLY, 8th mar - changed scale
                         YLY, 21 Apr  - Added ActionListener
                         SJS, 20 Feb  - Re-vamped appearances, etc.
  */


  // Member variables
  // -------------------------------------------------------------

  protected BConnectPoint connectPoint;
  protected BComponent parentComponent;

  private static BAnimator animator;



  // Appearance
  // -------------------------------------------------------------

  // protected static BUIColorScheme colors          = BGlobal.colors;

  protected static Color3f CONNECT_COLOR_ACTIVE   = BGlobal.HIGHLIGHT_COLOR_3F;
  protected static Color3f CONNECT_COLOR_SELECT   = BGlobal.SELECTED_COLOR_3F;
  protected static Color3f CONNECT_COLOR_INACTIVE = BGlobal.LOCKED_COLOR_3F;
  protected static Color3f CONNECT_COLOR_BASE     = BGlobal.HIGHLIGHT_COLOR_3F;

  protected static Color COLOR_ACTIVE   = BGlobal.HIGHLIGHT_COLOR;
  protected static Color COLOR_SELECT   = BGlobal.SELECTED_COLOR;
  protected static Color COLOR_INACTIVE = BGlobal.LOCKED_COLOR;
  protected static Color COLOR_BASE     = BGlobal.HIGHLIGHT_COLOR;
  protected static Color DARKBASE;

  protected static Color3f BLACK                  = BGlobal.BLACK_3F;
  protected static float   SHINE                  = 100.0f;
  protected static ColoringAttributes shading     = null;

  protected static Appearance APPEARANCE_ACTIVE   = getAppearance(CONNECT_COLOR_ACTIVE,   1.0f,  true); // 0.25f
  protected static Appearance APPEARANCE_SELECT   = getAppearance(CONNECT_COLOR_SELECT,   1.0f,  true);
  protected static Appearance APPEARANCE_INACTIVE = getAppearance(CONNECT_COLOR_INACTIVE, 0.8f,  true);
  protected static Appearance APPEARANCE_BASE     = getAppearance(CONNECT_COLOR_BASE,     0.10f, true);

  
  // Geometry
  // -------------------------------------------------------------

  protected static float CONNECT_PT_RADIUS        = 2.5f;
  private static final String MAP_NAME            = "internal/connectpoint.gif";
  protected static Geometry geometry              = null;
  protected static Geometry altGeometry           = null;


  // Appearance & geometry construction functions...
  // -------------------------------------------------------------

  protected static Appearance getAppearance(Color3f c, float brightnessBias, boolean useTexture)
  {
    Appearance a = new Appearance();
    a.setMaterial(getMaterial(c, brightnessBias));
    a.setCapability(Appearance.ALLOW_MATERIAL_READ);
    a.setCapability(Appearance.ALLOW_MATERIAL_WRITE);

    a.setColoringAttributes(getShading());

    if (useTexture)
    {
      a.setTexture(BTextures.getTexture(BGlobal.TEXTUREPATH +MAP_NAME) );

      TextureAttributes ta = new TextureAttributes();
      ta.setTextureMode(TextureAttributes.MODULATE) ;
      a.setTextureAttributes(ta);
    }

    return a;
  }

  protected static ColoringAttributes getShading()
  {
    if (shading == null)
    {
      shading = new ColoringAttributes();
      shading.setShadeModel(ColoringAttributes.SHADE_FLAT);
    }
    return shading;
  }

  protected static Material getMaterial(Color3f c, float brightnessBias)
  {
    //float r = c.x;
    //float g = c.y;
    //float b = c.z;

    //float bias = brightnessBias;

    Material mat =  new Material
    (
      BLACK, // BGlobal.WHITE_3F,
      c, //new Color3f(Color.green), // r*bias, g*bias, b*bias),
      BLACK, // new Color3f(r*bias, r*bias, r*bias),
      BLACK,
      SHINE
    );

    mat.setCapability(Material.ALLOW_COMPONENT_WRITE) ;

    return mat;
  }

  protected static Geometry getDefaultGeometry()
  {
    if (geometry == null)
    {
      BBox box = new BBox(CONNECT_PT_RADIUS,CONNECT_PT_RADIUS,CONNECT_PT_RADIUS);
      geometry = box.getGeometry();
    }
    return geometry;
  }

  protected static Geometry getAltGeometry()
  {
    if (altGeometry == null)
    {
      BBox box = new BBox(CONNECT_PT_RADIUS*1.3d,CONNECT_PT_RADIUS*1.3d,CONNECT_PT_RADIUS*1.3d);
      altGeometry = box.getGeometry();
    }
    return altGeometry;
  }


  // Constructor.
  // -------------------------------------------------------------

  public BConnectPointShape(BConnectPoint c, BComponent newComponent)
  {
    // Created like this so we can use the Sphere construction set,
    // But we need this to be of type Shape 3ED

    super(getDefaultGeometry(),  (Appearance)APPEARANCE_ACTIVE.cloneNodeComponent(true) );

    if ( BGlobal.DISABLE_FLASHING_OBJECTS )
    {
      CONNECT_COLOR_SELECT = BGlobal.ALT_HIGHLIGHT_COLOR_3F;
      COLOR_SELECT   = BGlobal.ALT_HIGHLIGHT_COLOR;
      APPEARANCE_SELECT   = getAppearance(CONNECT_COLOR_SELECT,   1.0f,  true);

      if (!getAltGeometry().isLive()  && !getAltGeometry().isCompiled() )
      {
	      getAltGeometry().setCapability(Geometry.ALLOW_INTERSECT );
        getAltGeometry().setCapability(Geometry.ALLOW_INTERSECT );
        getAltGeometry().setCapability(GeometryArray.ALLOW_FORMAT_READ);
      }
      
    }

    if (!isLive() && !isCompiled() )
    {
      setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
      setCapability(Shape3D.ALLOW_APPEARANCE_READ);

      setCapability(Shape3D.ALLOW_GEOMETRY_READ);
      setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);

    }

    if (!this.getGeometry().isLive() && !this.getGeometry().isCompiled() )
	    this.getGeometry().setCapability(Geometry.ALLOW_INTERSECT );


    if (!getGeometry().isLive()  && !getGeometry().isCompiled() )
    {
      getGeometry().setCapability(Geometry.ALLOW_INTERSECT );
      getGeometry().setCapability(GeometryArray.ALLOW_FORMAT_READ);
    }

    connectPoint = c;
    parentComponent = newComponent;

    if (!connectPoint.active)
      super.setAppearance(APPEARANCE_INACTIVE);

    if (parentComponent.getConnectPointBase() != null)
      if (connectPoint.position.equals(parentComponent.getConnectPointBase().position))
        super.setAppearance(APPEARANCE_BASE);
  }


  private void createDarkColor()
  {
      float baseBrightness = 0.3f;
      float[] hSB = Color.RGBtoHSB(COLOR_ACTIVE.getRed(),COLOR_ACTIVE.getGreen(),COLOR_ACTIVE.getBlue(),null);
      DARKBASE = Color.getHSBColor(hSB[0], hSB[1], baseBrightness);

  }


  class BHighlightAnimation extends BSinColorCycler
  {
    private Color3f newColor3f = new Color3f();
    private boolean active = true;
    long lastTime = 0;

    public void start() {}
    public void stop() {}

    public BHighlightAnimation()
    {
      super(BGlobal.SELECTED_COLOR, DARKBASE, 10, 60, 0, 1f);
    }


    public void setActive(boolean b)
      { active = b;}

    public boolean animate()
    {
     // BDebug.print("BConnectPointShape.animate entered");

     try
     {
        long time = System.currentTimeMillis();
        long deltaTime = time - lastTime;

        if (active)
          if (deltaTime > 70)
            if (BConnectPointShape.this.isLive() )
            {
              lastTime = time;
              super.animate();
              newColor3f.set(thisCyclesColor);
              getAppearance().getMaterial().setEmissiveColor(newColor3f);
              getAppearance().setMaterial( getAppearance().getMaterial());
            }

        return true;
      }
      catch (Exception e)
        { }

      return false;


    }

  }  // :~


 // protected Object  lock  = new Object() {};


  private transient BHighlightAnimation highlightAnimation;

  public void removeAnimation()
  {

  // BDebug.print ("BConnectPointShape.removeAnimation Called ");
  
    if (highlightAnimation != null)
    {

      // if (highlightAnimation.isRendering())
        //synchronized (highlightAnimation)
        //{
         // try
         // {
            highlightAnimation.setActive(false);
           // highlightAnimation.wait();

            if (animator != null)
            {
              animator.remove(highlightAnimation);
            }
         // }
         // catch (InterruptedException ex)
         // { }

       // }


        highlightAnimation = null;

    }



  }


  public static void cleanseStatic()
  // the animator should clear itself, but this is just in case...
  {
    if (animator != null)
    {
     // BDebug.print("BConnectPointShape.cleanseStatic Destroying animator");
      animator.stop();
      animator.clear();
      animator = null;

    }

    shading = null;


  }

  // Accessor methods...
  // -------------------------------------------------------------

  // SJS [22/02/2000]: Deprecated.
  public void setHighlight(boolean turnOn)
  {
    if (turnOn)
    {
      if (connectPoint.active)
      {

        if (DARKBASE == null)
          createDarkColor();

        if ( BGlobal.DISABLE_FLASHING_OBJECTS )
        {
          getAppearance().getMaterial().setEmissiveColor(CONNECT_COLOR_SELECT);
          animator = null;
          highlightAnimation = null;
          setGeometry( getAltGeometry() );
        }
        else
        {

          if (animator == null)
          {
            //BDebug.print("BConnectPointShape creating animator");
            animator = new BSleepingThreadAnimator(70, true);
          }

          if (highlightAnimation == null)
          {
            //BDebug.print("BConnectPointShape adding ani");
            highlightAnimation = new BHighlightAnimation();
            animator.add(highlightAnimation);
          }
        }

        //super.setAppearance(APPEARANCE_SELECT);

      }
    }
    else
    {
      removeAnimation();

      if ( this.isLive() )
      {
        if ( BGlobal.DISABLE_FLASHING_OBJECTS )
          setGeometry( getDefaultGeometry() );

        if (connectPoint.active)
          getAppearance().getMaterial().setEmissiveColor(CONNECT_COLOR_ACTIVE);
        else
          getAppearance().getMaterial().setEmissiveColor(CONNECT_COLOR_INACTIVE);

      }
        //super.setAppearance(APPEARANCE_INACTIVE);


      // A Hack to get the appearance to update on screen the second time the workshop is instantiated
      getAppearance().setMaterial( getAppearance().getMaterial());

    }
}

  // SJS [17/02/2000] - Added selection accessor method.
  public void setSelected(boolean selected)
  {
    // added this check to make the code compatible with old interface
    if  (connectPoint != null)    
      setHighlight(selected);

  }

  public boolean setActiveArea()
  {
        return false;
  }


  public void cleanse()
  {
    removeAnimation();
    connectPoint = null;
    parentComponent = null;

    setAppearance(null);
    setGeometry(null);
  }

  public BConnectPoint getConnectPoint()
    { return connectPoint; }


  public BComponent getParentComponent()
    { return parentComponent; }

  public static void setAnimator(BAnimator a)
    { animator = a; }


}

