package roboforge;

import java.awt.Color;
import java.io.*;
import java.util.*;

import javax.swing.tree.*;
//import javax.swing.event.*;

//import javax.vecmath.*;
//import javax.media.j3d.*;

import roboforge.btree.*;

public class BLimb implements Serializable
{
  static final long serialVersionUID = -1L;

  protected String name;
  protected BComponentTree components;

  public BLimb(BComponent theComponent)
  {
    components = new BComponentTree( theComponent.getTreeNode() );
    name = theComponent.getName() + " limb";
  }

  public BComponent getBaseComponent()
    { return (BComponent) components.getRoot().getUserObject(); }

  public boolean isLive()
  {
    BComponent base = getBaseComponent();

    if (base != null)
      return base.isLive();
    else
      return false;
  }



  public Object clone()
  {
    BComponent newBase = ((BComponentNode)getBaseComponent().getTreeNode().cloneTree()).getComponent();
    return new BLimb(newBase);
  }



  public void cleanse()
  {

    Enumeration enumerator = getBreathComponents();

    while (enumerator.hasMoreElements())
    {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
      BComponent component = (BComponent)node.getUserObject();
      component.cleanse();
     // BDebug.print ("BLimb.cleansing component " + component );


    }

  }


  public float getPrice()
  {
    float total = 0f;

    Enumeration enumerator =getBreathComponents() ;

    while (enumerator.hasMoreElements())
    {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
      BComponent component = (BComponent)node.getUserObject();
      total += component.getPrice();

    }

    return total;
  }


  public float getWeight()
  {
    float weight = 0f;

    Enumeration enumerator =getBreathComponents() ;
    while (enumerator.hasMoreElements())
    {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
      BComponent component = (BComponent)node.getUserObject();
      weight += component.getWeight() ;

    }

    return weight;
  }

  public int getDamageMax()
  {
    int dm = 0;

    Enumeration enumerator =getBreathComponents() ;
    while (enumerator.hasMoreElements())
    {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
      BComponent component = (BComponent)node.getUserObject();
      dm += component.getDamageMax() ;
    }

    return dm;
  }


  public int getComponentCount()
  {
    int count = 0;
    Enumeration e = getBreathComponents();

    while (e.hasMoreElements())
    {
      e.nextElement();
      count++;
    }

    return count;
  }


  public void setNormalAppearance()
  {
    Enumeration enumerator =getBreathComponents() ;
    while (enumerator.hasMoreElements())
    {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
      BComponent component = (BComponent)node.getUserObject();
      component.setNormalAppearance();
    }
  }

  public void displayColor(Color color)
  {
    Enumeration e = getBreathComponents();

    while (e.hasMoreElements())
    {
      DefaultMutableTreeNode node =
        (DefaultMutableTreeNode) e.nextElement();

      BComponent c =
        (BComponent) node.getUserObject();

      if (c.isLive())
        c.displayColor(color);
    }
  }

  public void setTransparency(float t, boolean updateDefault)
  {
    Enumeration e = getBreathComponents();

    while (e.hasMoreElements())
    {
      DefaultMutableTreeNode node =
        (DefaultMutableTreeNode) e.nextElement();

      BComponent c = (BComponent) node.getUserObject();

      c.setTransparency(t, updateDefault);
    }
  }

  public void setTexture(String t, boolean updateDefault)
  {
    Enumeration e = getBreathComponents();

    while (e.hasMoreElements())
    {
      DefaultMutableTreeNode node =
        (DefaultMutableTreeNode) e.nextElement();

      BComponent c = (BComponent) node.getUserObject();

      c.setTexture(t, updateDefault);
    }
  }



  public Enumeration getBreathComponents()
  {
      DefaultMutableTreeNode d = components.getRoot();
      return d.breadthFirstEnumeration();

  }

  public void init()
  {

    Enumeration enumerator =getBreathComponents() ;

    while (enumerator.hasMoreElements())
    {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
      BComponent component = (BComponent)node.getUserObject();
      component.init(false);

    }


  }

   public void saveToFile(String filename)
    throws IOException
  {
     if (!BGlobal.LIMBSUFFIX.equalsIgnoreCase(filename.substring(filename.length()-4)))
        filename = filename +  BGlobal.LIMBSUFFIX;
     save(filename);
  }

  private void save ( String filename )
    throws IOException
  {

    FileOutputStream fout = new FileOutputStream(filename);
    ObjectOutputStream out = new ObjectOutputStream(fout);

    out.writeObject(this);
    out.close();

    BSerializableTree.clearCache();

  }

    public static BLimb loadFromFile(String filename)
  {
    BLimb limb = null;

    try
    {
      filename = filename.toLowerCase();

      FileInputStream fin = new FileInputStream(filename);
      ObjectInputStream in = new ObjectInputStream(fin);

      limb = (BLimb) in.readObject();

      in.close();
      fin.close();

      limb.init();

    }

    catch (Exception e)
      { BDebug.print("BLimb.loadFromFile(): ERROR: " + e); }

    BSerializableTree.clearCache();

    return limb;
  }


  public String getName()
    {return name; }

  public void setName (String s)
    { name = s; }


  public String toString()
  {
    return getName();
  }



} 