package roboforge;

import java.util.*;
import java.io.*;
//import javax.swing.tree.*;
import javax.media.j3d.*;
import javax.vecmath.*;

import roboforge.ai.actions.*;
//import roboforge.ai.orient.*;
import roboforge.ai.worldstate.*;
import roboforge.ai.values.*;

public class BMove
  implements Serializable,  BNameable, BAction
{
  /*

  * Description:
    A Collection
    which holds BSnapshots.

  * Example:
  * Modification History ( Name, Date, Notes ):
    HSC, 4-April-1999
    DJG, 4-June-1999, Orientation Info

  */
  static final long serialVersionUID = -1L;
  private Vector snapshotCollection;
  private String moveName;
  private boolean moveLimbLocked = true;
  private boolean overRiding = false;

  // Game Engine Stuff
  private transient int currentSnapshotIndex;
  public transient BSnapshot currentSnapshot;
  private transient BChassis theChassis;

  private transient BJoint[] movedJoints;
  private transient BJoint[] highestMovedJoints;
  private transient boolean isResultantMove = false;

  // world state calculation variables
  private transient double start_myBotDam = 0;
  private transient double start_oppBotDam = 0;

  private transient double lastC_damageDone = 0;
  private transient double lastC_damageTaken = 0;
  private transient boolean moveComplete = false;
  private transient boolean isActive = false;

  private transient boolean waitForChassisMove = false;

  private transient BMoveList parentList;
  private transient boolean containsChassisMove =false;  // calculated in initForSim -> CalculateMovedJoints()

  // DJG June 2001
  //private transient float simTime = 0;  // secs
  private transient float lastActiveTime = 0;  // secs
  private transient float pauseTime = 0;  // Pause the Move after a collision
  private transient float firstPausedTime = 0;


  public BMove(String name)
  { this(name, true); }

  public BMove(String name, boolean limbLocked)
  {
     snapshotCollection = new  Vector();
     moveName = name;
     resetMove(false,0f,0f);

     // SJS [10/03/2000]: Choose a reasonable default orient distance...
     //orientVector = new Vector3f(0f, 0f, BGlobal.MOVE_ORIENT_Z);

     // SJS [12/04/2000]: Choose a reasonable default orient timeout...
     //orientTimeoutSecs = BGlobal.MOVE_ORIENT_TIMEOUT;

     moveLimbLocked = limbLocked;

     //BDebug.print("BMove constructor: "+name);

  }

  public void init(BMoveList parent)
  {
    //  BDebug.print ("BMove " + this + this.hashCode() + " init ");

       this.parentList = parent;

  }

  public void setParentList (BMoveList parent)
    { this.parentList = parent; }



  public void removeThumbnails()
  {
    BSnapshot current;
    for (int i=0; i<snapshotCollection.size(); i++)
    {
      current = (BSnapshot) snapshotCollection.get(i);
      current.setThumbnail(null);
    }
  }


  public void cleanse()
  {
   //BDebug.print ("BMove: Cleansed.");
   if ( snapshotCollection != null)
   {
    BSnapshot current;
    for (int i=0; i<snapshotCollection.size(); i++)
    {
      current = (BSnapshot) snapshotCollection.get(i);
      current.cleanse();
    }

    snapshotCollection.clear();
    snapshotCollection = null;

   }

   // DJG June 2001
   //simTime = 0;
   lastActiveTime = 0;
   pauseTime = 0;
   firstPausedTime = 0;

   currentSnapshot = null;
   theChassis = null;

   movedJoints = null;
   highestMovedJoints = null;

   parentList = null;

  }

  public void cleanseLiveNodes()
  {
    //BDebug.print ("BMove: "+toString()+" CleanseLiveNodes.");

    // DJG June 2001
  	//simTime = 0;
  	lastActiveTime = 0;
  	pauseTime = 0;
    firstPausedTime = 0;

    currentSnapshot = null;
   	//theChassis = null;

   	//movedJoints = null;
   	//highestMovedJoints = null;

    currentSnapshotIndex = 0;
    waitForChassisMove = false;
    isActive = false;

   	//parentList = null;

  }

  public void cleanseJointsNotInBot(BBot theBot)
  {
    BSnapshot current;
    for (int i=0; i<snapshotCollection.size(); i++)
    {
      current = (BSnapshot) snapshotCollection.get(i);
      current.cleanseJointsNotInBot(theBot);
    }


  }

  public void addWorldItems(BWorldState root)
  {

      char dot = BWorldState.DELIMITER;

      String key = BBot.stateCode+ dot+ BMoveList.stateCode + dot+ getName();

     /* BDebug.print ("BMove.addWorldItems.adding " + key +dot +"isActive" );
      BDebug.print ("BMove.addWorldItems.adding " + key +dot +"hasCompleted" );
      BDebug.print ("BMove.addWorldItems.adding " + key +dot +"damageDone" );
      BDebug.print ("BMove.addWorldItems.adding " + key +dot +"damageTaken" );  */


      // isActive
      BMoveBooleanWorldItem activeItem = new BMoveBooleanWorldItem(key +dot +"isActive", "Is active", new BBoolean(),"'yes' if this move is currently being executed by my Bot", this)
      {
        /**
		 * 
		 */
		private static final long serialVersionUID = -1L;

		public BValue evaluate()
        {
            setValue( this.getMove().getIsActive() );
            return getValue();
        }
      };

      // isCompleted
      BMoveBooleanWorldItem completeItem = new BMoveBooleanWorldItem(key + dot+ "hasCompleted", "Has completed", new BBoolean() ,"'yes' for the one AI cycle after this move has completed executing", this)
      {
        /**
		 * 
		 */
		private static final long serialVersionUID = -1L;

		public BValue evaluate()
        {
            setValue( this.getMove().getIsMoveComplete() );
            return getValue();
        }
      };

      // damage Done
      BMoveNumberItem damageDoneItem = new BMoveNumberItem(key + dot+ "damageDone", "Damage done last execution", new BNumber(0),new BNumber(0), new BNumber(10000.0) ,"hp", "The amount of damage caused to the enemy bot during the last execution period of this move", this)
       {
        /**
		 * 
		 */
		private static final long serialVersionUID = -1L;

		public BValue evaluate()
        {
            setValue( this.getMove().getDamageDone() );
            return getValue();
        }
      };

      // damage Taken
      BMoveNumberItem damageTakenItem = new BMoveNumberItem(key + dot+ "damageTaken", "Damage taken", new BNumber(0),new BNumber(0),new BNumber(10000.0),"hp", "The amount of damage recieved by my Bot during the last execution period of this move", this)
      {
        /**
		 * 
		 */
		private static final long serialVersionUID = -1L;

		public BValue evaluate()
        {
            setValue( this.getMove().getDamageTaken() );
            return getValue();
        }
      };

      root.put( activeItem);
      root.put( completeItem);
      root.put( damageDoneItem);
      root.put( damageTakenItem);



  }


  public String toString()
  {
    return moveName;
  }

  public String getName()
  {
    return moveName;
  }

  /*
  public boolean equals(Object o)
  {
    if (o instanceof BMove)
    {
      BMove m = (BMove) o;
      return moveName.equals(m.moveName);
    }
    else
      return super.equals(o);
  }
  */


  public BSnapshot getFirstSnapshot()
  {

    BSnapshot tempS = null;
    tempS = (BSnapshot)snapshotCollection.get(1);
    return tempS;

  }



  public void initForSim(BChassis newChas)
  {
    //BDebug.print (getName() + " BMove.initForSim called, new chassis is " + newChas);
    waitForChassisMove = false;
    moveComplete = false;

    calculateMovedJoints();
    theChassis = newChas;

  }

  public BChassis getChassis()
  { return theChassis; }

 public void start()
 // called when a move is started
 {

    // DJG June 2001
    if ( standDown(lastActiveTime) )
    {
    	// DJG June 2001
      // Stand down move
      isActive = false;
      //BDebug.print("BMove.start(): bump due to stand down - "+toString());
      //if ( theChassis != null )
      //BDebug.print("               "+theChassis.getTimer()+" - "+lastActiveTime);

    } else {

      if ( theChassis != null )
    	  lastActiveTime = theChassis.getTimer();

    	resetMove(false, 0, 0);

    	if (!isResultantMove )
    	{
      	start_myBotDam = theChassis.getBot().getCurrentDamage();
      	start_oppBotDam = theChassis.getBot().getOpponentBot().getCurrentDamage();
      	theChassis.getBot().getMoveList().setAnyMoveActive(true);
    	}

    	isActive = true;
    }


 }

 public void resetMove(boolean isChassisMove,
                        float myDamPercent,
                        float oppDamPercent
                      )
  {
      resetMove(        isChassisMove,
                        myDamPercent,
                        oppDamPercent,
                        false
                      );

  }

  public void resetMove(boolean isChassisMove,
                        float myDamPercent,
                        float oppDamPercent,
                        boolean onlyResetSnapshotIndex )
  {

     if (!onlyResetSnapshotIndex)
     {
      //orientAchieved = false;
      //timeSpentOrienting = 0;
      waitForChassisMove = false;
     }

     if (BGlobal.DEPRECATED)
      currentSnapshotIndex = 0;
     else
      currentSnapshotIndex = 0;

     getNextSnapshot(isChassisMove);

     pauseTime = 0;
     firstPausedTime = 0;

  }


  public void setName(String nameIn)
  {

    if ( ( nameIn == null) ||  nameIn.equals("") )
      return;

    if (!moveName.equals(nameIn) )
    {

      String oldName = moveName;

      moveName = nameIn;
      if (parentList != null)
        parentList.moveNameChanged(this, oldName);
    }


  }

  public Vector getSnapshots()
  {
    return snapshotCollection;
  }

  public int size()
    { return snapshotCollection.size(); }

  public void setSnapshots(Vector snapshotsIn)
  {
    snapshotCollection = snapshotsIn;
  }

  public void setLimbLocked (boolean in, BBot parentBot)
  {
    // if we are constrructing chassis bots then limb locked must be on
    if (BGlobal.CONSTRUCTING_CHASSISBOT)
      in = true;


    if (in != moveLimbLocked )
    {
      moveLimbLocked = in;

      parentBot.resetBotPosition();
      Vector newSnapCol = new Vector();

      // add defualt
      newSnapCol.add(this.snapshotCollection.get(0) );
      BSnapshot current;

      for (int i = 1; i < snapshotCollection.size(); i++ )
      {
        current = (BSnapshot) snapshotCollection.get(i);
        current.applyToBot();

        BSnapshot newSnap = parentBot.getSnapshot(current.getName(), false, false, moveLimbLocked, this, i);

        // Moves that are not limb locked cannot have chassis moves
        // therefore either the old or new will not have chassis move
        //newSnap.setMetersToMove(current.getMetersToMove());
        //newSnap.setChassisMove(current.getChassisMove());
        newSnapCol.add(newSnap);
      }


        snapshotCollection = newSnapCol;
        parentBot.resetBotPosition();

      }
  }

  public boolean getLimbLocked ()
  {
    return moveLimbLocked;
  }

  public boolean getContainsChassisMove()
  { return containsChassisMove; }

  public void calculateMovedJoints()
  {
   // BDebug.print ("BMove " + this + " calculateMovedJoints " );

    try
    {
      Vector highestMovedJointsV = new Vector();
      Vector movedJointsV = new Vector();
      containsChassisMove = false;

      for ( int i=1; i<snapshotCollection.size(); i++)
      {

          BSnapshot thisSnapshot = (BSnapshot) snapshotCollection.get(i);
         // BDebug.print ("BMOve.calculateMoveJoint thisSnapshot " + thisSnapshot );
          containsChassisMove = containsChassisMove || (thisSnapshot.getChassisMove() != null);

          // for every snapshot
          // skip the first snapshot
          for ( int j=0; j<thisSnapshot.getComponentTransforms().size(); j++)
          {
            BComponentTransform thisTransform = (BComponentTransform)thisSnapshot.getComponentTransforms().get(j);
            BJoint currentJoint = thisTransform.getJoint();

           // BDebug.print ("BMOve.calculateMoveJoint thisTransform " + thisTransform );
           // BDebug.print ("BMOve.calculateMoveJoint currentJoint " + currentJoint );

            boolean notOriginalPos = (!thisTransform.getJointPosition().equals(currentJoint.getDefaultJointPosition()));

            // if the joint is not in movedJoints then add it
            if (!movedJointsV.contains(currentJoint) )
            {
            //  BDebug.print(" snapshot " + i + " adding " + currentJoint );
              movedJointsV.add(currentJoint);
            }

            boolean isHighest = true;

            // if the joint moved
            if (notOriginalPos)
            {
             // BDebug.print (" highestMovedJointsV " + highestMovedJointsV);

              // if the joint  is not a descendant of a joint already in the list then add it
              // the list is assumed to be in breathfirst order
              for (int innerC =  0; innerC < highestMovedJointsV.size(); innerC++ )
              {

                BJoint high =  (BJoint) highestMovedJointsV.get(innerC);
              //  BDebug.print (" high " + high);
              //  BDebug.print (" getTreeNode " + high.getTreeNode());
              //
                isHighest = isHighest && !high.getTreeNode().isNodeDescendant(currentJoint.getTreeNode());

              }

              if (isHighest)
                highestMovedJointsV.add(currentJoint);
            }

            // DJG June 2001 - Used for capping rollbacks
            //currentJoint.setMoveMinMaxPositions( thisTransform.getJointPosition() );

          }
      }




      highestMovedJoints = new BJoint[highestMovedJointsV.size()];
      for (int i = highestMovedJoints.length - 1; i >=0; i-- )
        highestMovedJoints[i] = (BJoint) highestMovedJointsV.get(i);

      movedJoints = new BJoint[movedJointsV.size()];
      for(int i = movedJoints.length - 1; i >= 0 ; i-- )
        movedJoints[i] = (BJoint) movedJointsV.get(i);
     }
     catch (Exception e)
     {
        BDebug.print ("BMove " + this );
        BDebug.print ("snapshotCollection" + snapshotCollection );
        if (getChassis() != null)
          BDebug.print ("BMove gretBot " + getChassis().getBot() );


        e.printStackTrace();

     }
  }

  public BJoint[] getMovedJoints (boolean getHighest)
  {
    if ( getHighest )
      return highestMovedJoints;
    return movedJoints;
  }


 /* public Vector getMovedJoints(BSnapshot s)
  {
    Vector transforms = s.getComponentTransforms();
    Vector result = new Vector();

    BComponentTransform t;
    BJoint j;

    if (!transforms.isEmpty())
    {
      for (int i=0; i<transforms.size(); i++)
      {
        t = (BComponentTransform) transforms.get(i);
        j = t.getJoint();

        if (hasJointMoved(s, t, null, false))
          result.add(j);
      }
    }

    return result;
  }         */

  /*private boolean hasJointMoved(BSnapshot s, BComponentTransform t )
  {
    int snapIndex = snapshotCollection.indexOf(s);
    BJoint j = t.getJoint();

    if (snapIndex == 0 )
      return false;
    else if (snapIndex == 1 )
    {
      BJointPosition currentPos = t.getJointPosition();
      BJointPosition defaultPos = j.getDefaultJointPosition();
      return !currentPos.equals(defaultPos);
    }
    else
    {
      BJointPosition currentPos = t.getJointPosition();
      BJointPosition defaultPos = j.getDefaultJointPosition();
      boolean found = !currentPos.equals(defaultPos);

      if (found)
      {
        BSnapshot prev =  (BSnapshot) snapshotCollection.get(snapIndex-1) ;
        if ( ! prev.contains(j) )
          return true;
        else
        {
          prev.getComponentTransforms()
        }
      }
      return found;

    }



  }     */

  public void setMovedJointsHighlight(BSnapshot s, boolean isHighlight)
  {
    Vector transforms = s.getComponentTransforms();
    BComponentTransform t;
    BJoint j;

    if (!transforms.isEmpty())
    {
      for (int i=0; i<transforms.size(); i++)
      {
        t = (BComponentTransform) transforms.get(i);
        j = t.getJoint();


        if (  hasJointMoved(s, t, null,false)  )
        {
          // Set joint appearance...
          if (isHighlight)
            j.setMovedHighlight();
          else
            j.setNormalAppearance();
        }
      }
    }
  }

  public boolean hasJointMoved(BSnapshot thisSnapshot, BComponentTransform thisTransform, BJoint thisJoint, boolean undo)
  {
    // Determines whether a Joint has moved from a previous snapshot. Either BComponent Transform OR BJoint
    // can be passed. Code is faster with BComponentTransform. Undo will cause the code to reset the
    // transform for the current Joint back to the previous snapshot.

    boolean moved = false;
    //
    if ( thisTransform != null )
    {
      thisJoint = thisTransform.getJoint();
    }
    else
    {
      boolean found = false;
      for (int i = 0 ; i < thisSnapshot.getComponentTransforms().size() && !found; i++ )
      {
        thisTransform = (BComponentTransform) thisSnapshot.getComponentTransforms().get(i);
        if ( thisJoint == thisTransform.getJoint() )
        {
          found = true;
        }
      }
    }

    BSnapshot prevSnapshot;
    BComponentTransform prevTransform;
    BJoint prevJoint;
    int j = snapshotCollection.indexOf(thisSnapshot);

    if (j == 0 )
      ;
    else if ( j > 1 )
    {
      prevSnapshot = (BSnapshot) snapshotCollection.get(j-1);
      boolean found = false;
      for (int k = 0 ; k < prevSnapshot.getComponentTransforms().size() && !found; k++ )
      {
        prevTransform = (BComponentTransform) prevSnapshot.getComponentTransforms().get(k);
        prevJoint = prevTransform.getJoint();
        if ( prevJoint == thisJoint )
        {
          found = true;
          if ( !prevTransform.getJointPosition().equals(thisTransform.getJointPosition()) )
          {
            moved = true;
            if (undo)
              thisTransform.setJointPosition( prevTransform.getJointPosition() );
          }
        }
      }
      if ( !found )
      {
        //BDebug.print("Joint not found in previous Snapshot: "+thisJoint.toString());
        moved = true;
      }
    }
    else
    {
      // SJS [12/03/2000]: If looking at first snapshot, compare to default pos.
      BJointPosition currentPos = thisTransform.getJointPosition();
      BJointPosition defaultPos = thisJoint.getDefaultJointPosition();
      return !currentPos.equals(defaultPos);
    }

    //
    return moved;
  }


  public boolean isStartMove (BMove move)
  {
    return (move == this);
  }

  public boolean isVictoryMove (BMove move)
  {
    return (move == this);
  }


  private transient boolean jump = false;
  private transient int frameToJumpTo = 0;
  private transient BBot botToJumpSnap = null;

  public void setCurrentSnapshot(int snapshotIndex, BBot jumpBot)
  {
    jump = true;
    frameToJumpTo =   snapshotIndex;
    botToJumpSnap =   jumpBot;

  }

  public int getCurrentSnapshotIndex()
  {
    // SJS [12/04/2000] - If jumping, return the jump target.

    if (jump)
      return frameToJumpTo;
    else
      return currentSnapshotIndex;
  }

  public int indexOf(BSnapshot s)
    { return snapshotCollection.indexOf(s); }

  public BSnapshot getSnapshot(int i)
  {
      return (BSnapshot) snapshotCollection.get(i);
  }

  public BSnapshot getCurrentSnapshot()
  {
    return currentSnapshot;

  }

  public int getSnapshotCount()
  {
    return snapshotCollection.size();
  }

  public void addSnapshot(BSnapshot s)
    { snapshotCollection.add(s); }

  public void insertSnapshot(BSnapshot s, int i)
  {
    int n = size();
    if (n > 0)
    {
      if (i == n)
        snapshotCollection.add(s);
      else
        snapshotCollection.add(i, s);
    }
    else
      snapshotCollection.add(s);
  }

  public BSnapshot removeSnapshot(int i)
  {
    return (BSnapshot) snapshotCollection.remove(i);
  }

  public boolean isValidSnapshot(int i)
    { return (i >= 0) && (i < snapshotCollection.size()); }

  public boolean isFirstSnapshot(int i)
    { return i == 0; }

  public boolean isLastSnapshot(int i)
    { return i == (getSnapshotCount()-1); }

  public boolean isFirstSnapshot(BSnapshot s)
  {
    int i = snapshotCollection.indexOf(s);
    return i == 0;
  }

  public boolean isLastSnapshot(BSnapshot s)
  {
    int i = snapshotCollection.indexOf(s);
    return i == (getSnapshotCount()-1);
  }


  public int getSnapshotIndex(BSnapshot s)
  {
    return snapshotCollection.indexOf(s);
  }



  public void getNextSnapshot(boolean isChassisMove)
  {
    if (currentSnapshotIndex < (snapshotCollection.size()-1))
      setCurrentSnapshot(currentSnapshotIndex + 1, isChassisMove);
    else
      currentSnapshot = null;

    firstPausedTime = 0.0f;
    pauseTime = 0.0f;
    
  }



  private void setCurrentSnapshot(int snapshotIndex, boolean isChassisMove)
  {

    currentSnapshotIndex = snapshotIndex;

    //if (!isChassisMove)
    //  BDebug.print("BMove::" + getName() + " setCurrentSnapshot  currentSnapshotIndex " +currentSnapshotIndex);
   // BDebug.print("BMove.setCurrentSnapshot  currentSnapshotIndex " + currentSnapshotIndex);
    //BDebug.print("   --------    ");


    Vector joints;
    BComponentTransform bTransform;
    BJoint bJoint;
    //BJointPosition position;

    //float maxTime = 0;

    // if the global flag is set, then this is a chassis bot.
    // This flag is set in BBot.calculateStatistics if no ChassisMoveList is present;

    if (snapshotIndex < snapshotCollection.size())
    {
         currentSnapshot = (BSnapshot)snapshotCollection.get(snapshotIndex);
         joints = currentSnapshot.getComponentTransforms();

         for (int i = 0 ; i < joints.size() ; i++ )
         {
            bTransform = (BComponentTransform) joints.get(i);
            bJoint = bTransform.getJoint();

           // if (currentSnapshot.getDelayJoints())
            //{
            //  maxTime = java.lang.Math.max(maxTime, bJoint.workOutProcessingTime(bTransform.getJointPosition(), currentSnapshot.getPercentPowerRatio()));
            //  bJoint.storeCurrentPositionAsTarget();
            //}
           // else
              bJoint.setDelay(0);
              bJoint.pauseTime = 0f;



           // BDebug.print ("BMove.setCurrentSnapshot processing BJoint " + bJoint );
            if ( ( !isResultantMove ) && (!( bJoint.getComponentType() == BAutoRotateJoint.COMPONENT_TYPE )) )
                bJoint.currentVelocity = 0f;

         }

        /* if (currentSnapshot.getDelayJoints())
          for (int i = 0 ; i < joints.size() ; i++ )
          {
            bTransform = (BComponentTransform) joints.get(i);
            bJoint = bTransform.getJoint();
            bJoint.setDelay(maxTime);
          }  */
         //BDebug.print("BMove. setCurrentSnapshot  theChassis " + theChassis);

         // post chassis move
         if (!isChassisMove)
          if (  currentSnapshot.getChassisMove() != null     )
          {
            theChassis.lastExecTime = 0;
            waitForChassisMove = theChassis.setNewChassisMove(currentSnapshot.getChassisMove(), currentSnapshot.getMetersToMove(), BChassis.ADD_MODE_PASSIVE, this);
          }



    }
    else
    {
      currentSnapshot = null;
    }





   if ( isChassisMove && currentSnapshot != null )
   {
     theChassis.distanceMovedThisSnapshot = 0f;
     theChassis.distanceThisSnapshot =  currentSnapshot.getMetersToMove();
     if ( theChassis.distanceThisSnapshot == 0.0f )
       theChassis.distanceThisSnapshot = 1.0f;
   }

  }


   public boolean execute(float timeSlice  )
   {

    return execute(timeSlice, false);

   }



  public void clearChassisWait()
  {
    waitForChassisMove = false;
  }

  public void setBotOrientation(int snapIndex)
    { setBotOrientation(theChassis.getBot(), snapIndex); }

  public void setBotOrientation(BBot bot, int snapIndex)
  {
    Vector3d t = getCumulativeTranslation(snapIndex);
    float    r = getCumulativeRotation(snapIndex);

    float scale = BGlobal.WORLD_TO_CANVAS_SCALE;

    bot.setArenaPosition(new BArenaObjectPosition(
      (float) t.x * scale,
      (float) t.y * scale,
      (float) t.z * scale,
      r));

    bot.getChassis().setOrientation(getCumulativeOrientation(snapIndex));
    bot.getChassis().setHeight();

  }


  private float getConstructingChassisBotDistance(int index)
  {

       float distance = 0f;

       Vector snapshots = getSnapshots();
       if ( snapshots != null )
       {
         for ( int i=index; i<snapshots.size(); i++)
         {
           BSnapshot thisSnapshot = (BSnapshot)snapshots.get(i);
           distance += thisSnapshot.getMetersToMove();
         }
       }

       distance = distance * BPhysics.CONSTRUCTING_CHASSIS_MOVE_EXTENSION;

       return distance;



  }



  public boolean getIsResultantMove()
  {
    return isResultantMove;
  }

  public void setIsResultantMove( boolean b)
  {
    isResultantMove = b;
  }

  public boolean execute(float timeSlice, boolean isChassisMove )
  // Executes the move
  // PRE .. Calculate Bot Stats has been executed
  // returns wether the move was executed
  {
   // if (!isChassisMove)
    //  BDebug.print("BMove::" + getName() + " execute  currentSnapshotIndex " +currentSnapshotIndex);

    if (jump)
    {
     // BDebug.print ("BMove.jump called ");

      if (!BGlobal.CONSTRUCTING_CHASSISBOT )
        theChassis.terminateChassisMove(true);
      else
        botToJumpSnap.getChassis().setConstructionChassisMove(this,java.lang.Math.max(getConstructingChassisBotDistance(frameToJumpTo),0.1f),  botToJumpSnap.getMoveList().indexOf(this)   );

      setCurrentSnapshot(frameToJumpTo, isChassisMove);

      if (frameToJumpTo > 0)
      {
        BSnapshot lastSnapshot =  (BSnapshot) snapshotCollection.get ( frameToJumpTo - 1);
        botToJumpSnap.applySnapshot(this, frameToJumpTo - 1);

        //lastSnapshot.applyToBot();
        setBotOrientation(frameToJumpTo - 1);
      }
     // BDebug.print("BMove.jump activated ");

      jump = false;
      botToJumpSnap = null;
      return true;
    }
    /* Now Pause Joints Only
    else if ( standDown(pauseTime) ) // Paused after a collision
    {
       return true; // Still executing
    }
    */
    else if (currentSnapshot == null )
    {
       //BDebug.print("BMove.execute(): curSnap = null. [finished joints]");

      if (waitForChassisMove)
        return true;

      return false;
    }
    else  // normal move
    {

        // DJG June 2001
    		if ( theChassis != null )
      		lastActiveTime = theChassis.getTimer();  // For Move Standoff
    		else
    			lastActiveTime += timeSlice;

        Vector joints = currentSnapshot.getComponentTransforms();
      	//  BDebug.print ( this + ".execute  " + joints);

        BComponentTransform bTransform;
        BJoint bJoint;
        BJointPosition position;

        boolean stillExecutingThisSnap = false;

        //float slowestJ = 1;
        //float fastestJ = 0;

        // process Agregate
       /* if (currentSnapshot.getDelayJoints())
          for (int i = 0 ; i < joints.size() ; i++ )
          {
            BDebug.print ( this + " delayedJoint ");

            bTransform = (BComponentTransform) joints.get(i);
            bJoint = bTransform.getJoint();
            position =  bTransform.getJointPosition();

            if (!bJoint.getDestroyed() && !(bJoint instanceof BAutoRotateJoint )   )
            {
              float currentJ = bJoint.workoutPercentToTarget(position);

              if (currentJ != -1)
              {
                if (  currentJ < slowestJ )
                  slowestJ = currentJ;
                if (currentJ > fastestJ )
                  fastestJ = currentJ;
              }
            }
          }*/

        // Move the Joints
        for (int i = 0 ; i < joints.size() ; i++ )
        {


          bTransform = (BComponentTransform) joints.get(i);
        //  BDebug.print("BMove.execute(): bTransform "+bTransform);

          bJoint = bTransform.getJoint();
          position =  bTransform.getJointPosition();

          //BDebug.print("BMove.execute(): bJoint "+ bJoint);
         // BDebug.print("BMove.execute(): position "+ position);
         // BDebug.print("BMove.bJoint.getBot() "+ bJoint.getBot() );

          boolean moved = false;
          if ( bJoint.getBot().getEnergy() > 0f || isResultantMove || bJoint.getChassisLock() )
          {

            if (!bJoint.getDestroyed())
            {
              //BDebug.print ( this + ".execute currentSnapshot " + currentSnapshot );
             // BDebug.print ("BMove.execute  currentSnapshot.getPercentPower() " + currentSnapshot.getPercentPower() );
              float power =  currentSnapshot.getPercentPowerRatio();

             // if (currentSnapshot.getDelayJoints())
              //  if (fastestJ > 0.4)
               //   power = power * bJoint.getPercentToTargetScaleFactor(fastestJ, slowestJ);

              moved =  bJoint.move(position,  timeSlice, power , currentSnapshot.getPowerVelocityMultiplier(), isResultantMove);

            }


          }
          else
            return false;


        //  if (   ! isChassisMove )
        //    BDebug.print ( bJoint + " moved " + moved );

          stillExecutingThisSnap = stillExecutingThisSnap ||  moved;

        }    // for loop


        // Execute the chassis move
        if (isChassisMove)
        {
                //BDebug.print("BMove.execute(): executing chassis move.");
                boolean chassisMoved = theChassis.move(currentSnapshot.getChassisX(), currentSnapshot.getChassisY(), timeSlice, 1.0f);
        }

        // Skip to next snapshot if this one is "stuck"
        if ( pausedToLong() )
        {
          stillExecutingThisSnap = false;
          waitForChassisMove = false;
        }


        if (!stillExecutingThisSnap)
        {

            if (!waitForChassisMove)
              getNextSnapshot(isChassisMove);
        }

      return true;
    }
  }


  // BAiOperator methods
  //TODO REMOVE WQHEN WE HAVE THE NEW AI

  public String getCurrentValue()
  {
    return "";
  }


  public boolean isZonesTest()
  {
    return false;
  }


  public Vector3d getCumulativeTranslation(int snapshot)
  {
    Transform3D trans = new Transform3D();
    Transform3D temp  = new Transform3D();

    BSnapshot cur = null;
    Vector3d t = null;
    float r = 0.0f;

    for (int i=1; i<=snapshot; i++)
    {
      cur = getSnapshot(i);

      r = cur.getRotation(theChassis, this);
      if (r != 0.0f)
      {
        temp.rotY(r);
        trans.mul(trans, temp);
      }

      t = cur.getTranslation(theChassis, this);
      if (t != null)
      {
        temp.set(t);
        trans.mul(trans, temp);
      }

      // BDebug.print("val:"+ cur.getMetersToMove());
    }


    // Return the net translation...
    t = new Vector3d();
    trans.get(t);

    // BDebug.print("Cumulative translation to snapshot["+snapshot+"] = "+t);
    return t;
  }

  private static Matrix3f identity = new Matrix3f();

  public Matrix3f getCumulativeOrientation(int snapshot)
  {
    // return ID at the moment because we only use orientation for tipping.
    identity.setIdentity();
    return identity;

  }

  public float getCumulativeRotation(int snapshot)
  {
    BSnapshot cur = null;
    float rot = 0.0f;

    for (int i=1; i<=snapshot; i++)
    {
      cur = getSnapshot(i);
      rot += cur.getRotation(theChassis, this);
      // BDebug.print("val:"+ cur.getMetersToMove());
    }

    // BDebug.print("Cumulative rotation to snapshot["+snapshot+"] = "+rot);
    return rot;
  }


  private boolean contains (BJoint j )
  // pre :: calculateMovedJoints has been called
  {

    BJoint[] myJoints = getMovedJoints(false);

    for (int i = myJoints.length - 1; i >=0; i--)
    {
     // BDebug.print("against " + getName() +" " +   myJoints[i] );
      if (j == myJoints[i])
        return true;
    }
    return false;


  }


  public boolean conflictsWith(BAction a)
  // PRE CalculatedMovedJoints Has Been Called
  {

    boolean c = false;

    if (a == this)
      c = true;
    else if (a instanceof BMove )
    {

    //  BDebug.print ("BMove.conflictsWith checking " + this + " and " +  a);


      BMove aMove = (BMove) a;

      // if either is limblocked then they conflict
      if (aMove.getContainsChassisMove() && this.getContainsChassisMove()  )
        c = true;
      else if (aMove.getLimbLocked() && this.getLimbLocked() )
        c = true;
      else if (aMove.getIsResultantMove())
      {
          c = true;
      }
      // DJG June 2001
      else if ( standDown(lastActiveTime) )
    	{
    		// DJG June 2001
      	// Stand down move
      	c = true;
      	//BDebug.print("BMove.conflictsWith(): bump due to stand down - "+toString());
      }
      else
      {
        BJoint[] aJoints = aMove.getMovedJoints(false);
        //BDebug.print ("BMove.conflictsWith " + this + " and " +  aMove);
    //    BDebug.print ("BMove.conflictsWith " + aMove + " and " + aJoints);
      //  BDebug.print ("BMove.conflictsWith " + this + " and " + getMovedJoints(false));

        for (int i = aJoints.length - 1; ( i >= 0 ) && !c; i-- )
        {
          //BDebug.print("checking " + aMove + " " + aJoints[i] );
          if (this.contains(aJoints[i] ) )
            c = true;

        }
      }

    //  BDebug.print ("BMove.conflictsWith result " + c);

    }
    else if ( (a instanceof BOrientAction)  ||  (a instanceof BChassisAction) )
    {
      if (getContainsChassisMove())
        c = true;
      else if ( getIsResultantMove() )
      {
        c = containsChassisJoints();
      }

    }
    else if ( a instanceof BBotFallingAction)
    {
      c = getContainsChassisMove();
    }



    return c;

  }

  public boolean containsChassisJoints()
  {
    boolean c = false;

    BJoint[] myJoints = getMovedJoints(false);

    for (int i = myJoints.length - 1 ; i>= 0 && !c; i -- )
      if (myJoints[i].getChassisLock() )
            c = true;

    return c;
  }

  public boolean overrides(BAction a)
  {
    boolean result = false;

    if (a == this)
      result = false;
    else if ( a instanceof BBotFallingAction)
      result = false;
    else if ( this.getIsResultantMove() )
    {
      // Resultant moves override everything
      result = true;
      //result = false;
    }
    // DJG June 2001
    else if ( standDown(lastActiveTime) )
    {
    	// DJG June 2001
    	// Stand down move
    	result = false;
    	//BDebug.print("BMove.overrides(): bump due to stand down - "+toString());
    }
    else if ( getOverriding() )
    {
      if ( a instanceof BMove )
      {
        BMove m = (BMove) a;
        if (m.getIsResultantMove() )
          result = false;
        else if (  m.getOverriding() )
          result = false;
        else
          result = true;
       }
       else
        result = true;

    }
    else  if (a instanceof BOrientAction )
      result = true;
    else if  (a instanceof BChassisAction)
    {
      result = getOverriding();
    }


    return result;
  }

  public void setOverriding(boolean o)
    { overRiding = o; }

  public boolean getOverriding()
  { return overRiding; }


  public void stop()
  {
    //BDebug.print("BMove.stop() called for "+toString());
    if (!isResultantMove )
    {
      lastC_damageDone = theChassis.getBot().getOpponentBot().getCurrentDamage() - start_oppBotDam;
      lastC_damageTaken = theChassis.getBot().getCurrentDamage() - start_myBotDam;
      theChassis.getBot().getMoveList().setAnyMoveCompleted(true);
      theChassis.getBot().getMoveList().setAnyMoveActive(false);

      if (this.containsChassisMove )
        theChassis.returnToStand();

    }

    moveComplete = true;
    isActive = false;


  }

  public boolean getIsActive()
  { return isActive; }

  public boolean getIsMoveComplete()
  {
    boolean wasComplete = moveComplete;
    moveComplete = false;
    return wasComplete;
  }

  public double getDamageDone()
  { return lastC_damageDone; }

  public double getDamageTaken()
  { return lastC_damageTaken; }


  public boolean isValid()
  {
   // BDebug.print ("BMove " + this + this.hashCode() +" isValid " + !(parentList == null) );

    if (parentList == null)
      return false;
    return true;

  }

  public void replaceBot (BBot bot)
  {

    BSnapshot s;
    for (int i = snapshotCollection.size() - 1; i >=0; i--)
    {
      s = (BSnapshot) snapshotCollection.get(i);
      s.replaceBot(bot);

    }
  }


  public void upgrade( BMove other, BBot bot )
  {
    snapshotCollection = other.snapshotCollection;
    replaceBot(bot);
  }

  private boolean standDown(float testTime)
  {
    if ( theChassis == null )
      return false;
    else
  		return ( ((theChassis.getTimer() - testTime) * 1000) < BGlobal.STAND_DOWN_MILLISECS && (theChassis.getTimer()*1000) > BGlobal.STAND_DOWN_MILLISECS );
  }

  public void pause()
  {
    if ( theChassis != null )
    {
      if ( standDown( (pauseTime + (BGlobal.STAND_DOWN_MILLISECS / 1000f)) ) ) // Allow delay in move overlap
      {
        // Effectively still paused
        if ( firstPausedTime == 0.0f )
          firstPausedTime = pauseTime;
      } else {
        firstPausedTime = 0.0f;
        /*
        // Halt Joint Velocities
        if ( currentSnapshot != null )
        {

          Vector joints = currentSnapshot.getComponentTransforms();
          BComponentTransform bTransform;
          BJoint bJoint;

          for (int i = 0 ; i < joints.size() ; i++ )
          {
            bTransform = (BComponentTransform) joints.get(i);
            bJoint = bTransform.getJoint();
            bJoint.currentVelocity = 0f;
          }
        }
        */

      }

      pauseTime = theChassis.getTimer();

      if ( firstPausedTime == 0.0f )
        firstPausedTime = pauseTime;

    }

    if ( containsChassisMove && theChassis != null )
      theChassis.pause();
  }

  private boolean pausedToLong()
  {
    if ( theChassis == null || firstPausedTime == 0.0f )
      return false;
    else
    {
      float totalPausedTime = pauseTime - firstPausedTime;
      return ( totalPausedTime > 2.0 ); // time out snapshot if stuck for 2 seconds
    }
  }

  



}///:~



