package roboforge;

import roboforge.animation.*;
import roboforge.animation.alpha.*;
import roboforge.animation.alpha.map.*;
import roboforge.animation.arena.*;
import roboforge.animation.time.*;

import roboforge.math.*;



public class BRotatingArenaOrnament
  extends    BMobileArenaOrnament
{

  // Serialization ID.
  // -------------------------------------------------------------

  static final long serialVersionUID = -1L;


  // Constants.
  // -------------------------------------------------------------

  protected static final float FRAMERATE = 15f;
    // Target framerate for ornament animation.

  protected static final float DEFAULT_PERIOD = 1f;
    // Period of revolution.


  // Members...
  // -------------------------------------------------------------

  protected float period;
    // Time (in seconds) to complete one full revolution.




  // Constructors...
  // -------------------------------------------------------------

  public BRotatingArenaOrnament(String name, String filename)
  {
    super(name, filename);
    setPeriod(DEFAULT_PERIOD);
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setPeriod(float period)
    { this.period = period; }

  public float getPeriod()
    { return period; }



  // BAnimatedArenaOrnament implementation...
  // -------------------------------------------------------------

  public BAnimation getAnimation()
  {
    // Instantiate the animation..
    BArenaOrnamentAnimation anim
      = new BArenaOrnamentAnimation(this);

    // Setup target arena positions...
    BArenaObjectPosition p  = getArenaPosition();
    BArenaObjectPosition lo = new BArenaObjectPosition(p);
    BArenaObjectPosition hi = new BArenaObjectPosition(p);

    // Specify a full 360 degree rotation...
    hi.facing = BMath3D.TWO_PI;

    // Set up alpha map...
    BAlphaMap rev = new BLinearMap(period);
    BAlphaMap map = new BPeriodicMap(rev, period);

    // Set up animation channel...
    BLerpChannel rot = new BLerpChannel(lo, hi);

    // Assign the alpha map/channel to animation.
    anim.setAlphaMap(map);
    anim.setChannel(rot);

    // Throttle the animation to FRAMERATE fps.
    return new BScheduledAnimation(anim, FRAMERATE);
  }

}