package roboforge;

import javax.media.j3d.*;
import javax.vecmath.*;
//import java.awt.Color;
//import java.awt.event.*;
//import java.util.*;


public class BSelectionBox extends Shape3D
{

  protected Point3d centerPoint = null;
  protected double cornerLength = 0.0;
  protected float  lineWidth = 1.0f;

  public BSelectionBox(
    double xmin, double ymin, double zmin,
    double xmax, double ymax, double zmax,

    Color3f color, double cornerLength, float lineWidth, float alpha)
  {
    super();

    this.cornerLength = cornerLength;
    this.lineWidth = lineWidth;

    this.centerPoint = new Point3d
     (xmin + ((xmax - xmin) / 2.0d),
      ymin + ((ymax - ymin) / 2.0d),
      zmin + ((zmax - zmin) / 2.0d));

    // Number of vertices.
    int numVertices = 48; // 8 * 3 * 2;


    // Create Geometry...
    LineArray lines = new LineArray(
      numVertices,
      LineArray.COORDINATES);

    Point3d[] corners = new Point3d[8];

    corners[0] = new Point3d(xmax, ymax, zmax);
    corners[1] = new Point3d(xmax, ymax, zmin);
    corners[2] = new Point3d(xmin, ymax, zmin);
    corners[3] = new Point3d(xmin, ymax, zmax);
    corners[4] = new Point3d(xmax, ymin, zmax);
    corners[5] = new Point3d(xmax, ymin, zmin);
    corners[6] = new Point3d(xmin, ymin, zmin);
    corners[7] = new Point3d(xmin, ymin, zmax);

    int vertexIndex = 0;

    for (int i=0; i<8; i++)
    {
      makeCorner(lines, corners[i], vertexIndex);
      vertexIndex += 6;
    }

    // Set Geometry...
    setGeometry(lines);
    setAppearance(color, alpha);
  }

  protected void makeCorner(LineArray l, Point3d corner, int v)
  {
    double dx = corner.x;
    double dy = corner.y;
    double dz = corner.z;

    dx = (corner.x < centerPoint.x) ? (dx + cornerLength) : (dx - cornerLength);
    dy = (corner.y < centerPoint.y) ? (dy + cornerLength) : (dy - cornerLength);
    dz = (corner.z < centerPoint.z) ? (dz + cornerLength) : (dz - cornerLength);

    l.setCoordinate(v++, corner);
    l.setCoordinate(v++, new Point3d(dx, corner.y, corner.z));
    l.setCoordinate(v++, corner);
    l.setCoordinate(v++, new Point3d(corner.x, dy, corner.z));
    l.setCoordinate(v++, corner);
    l.setCoordinate(v++, new Point3d(corner.x, corner.y, dz));
  }

  public void setAppearance(Color3f color, float alpha)
  {
    Appearance a = new Appearance();

    ColoringAttributes ca =
      new ColoringAttributes(color, ColoringAttributes.FASTEST);
    a.setColoringAttributes(ca);

    if (alpha > 0.0f)
    {
      TransparencyAttributes ta = new TransparencyAttributes(
        TransparencyAttributes.BLENDED, alpha);

      a.setTransparencyAttributes(ta);
    }

    LineAttributes la = new LineAttributes(lineWidth, LineAttributes.PATTERN_SOLID, false);
    a.setLineAttributes(la);

    setAppearance(a);
  }

  public void cleanse()
  {
    setAppearance(null);
    setGeometry(null);
  }

  /*
   protected void finalize() throws Throwable
  {
    BDebug.print ( "BSelectionBox.finalize called" );
  }    */



}