package roboforge;

import roboforge.ai.worldstate.*;
import roboforge.ai.values.*;
//import roboforge.ai.*;

import java.io.Serializable;
import java.util.*;
import javax.vecmath.*;
//import javax.swing.*;
//import javax.swing.tree.*;

//import roboforge.math.*;

import roboforge.arenaSystems.*;

public class BSimulation implements Serializable
{
  /*
  * Description:
  Controls battles waged in an Arena

  * References:
  *
  * Modification History ( Name, Date, Notes ):
                            HSC 21-May-1999
                            YLY 23-Jun-1999 - Implemented a few methods,
                            plus changed to to Implement 'Runnable'
                            DJG 2001 - Massive Changes to Improve performance and Accuracy
  */
  static final long serialVersionUID = -1L;
  private BArena arena;
  private float battleDuration;
  private BBattleRecorder battleRecorder;
  //private BBattleResults battleResults;
  private boolean recordBattle; // whether or not to record the battle

  //private BArenaObjects arenaObjects;

  private float time;
  private float lastDeadlockTime;

  private transient BSimulationCanvas universeCanvas;
  private boolean battleRunning;
  //private BProgressBar pbar;
  //private JLabel plabel;

  private BBot botA = null;
  private BBot botB = null;
  //private BBot opponentBot;
  //private BBot selfBot;
  //boolean allBotsAlive = true;
  public boolean performStartVictoryMoves = true;

  private transient Vector componentsToDetach = new Vector(); // A list of components to detach from the universe at the end of the cycle
  private transient Vector componentsToDetachTotal= new Vector(); // A list of components for the battleRecorder .
  private transient Vector collisionsTotal = new Vector(); // A list of components to detach from the universe at the end of the cycle
  private transient BTimer simTimer;

  // DJG June 2001
  //private transient BTimer botExecTimer;
  //public transient BTimer movesAITimer;
  //public transient BTimer movesMVTimer;
  //public transient BTimer movesCSTimer;
  //public transient BTimerStub movesINTimer;
  //public transient BTimerStub movesTQTimer;
  //private transient BTimer collisionTimer;
  //private transient BTimer rollbackTimer;
  //private transient BTimer recordTimer;
  //

  private transient Vector lastCycleCollisions = new Vector();

  public static String WSTATE_ENEMYBOT = "enemy";
  public static String WSTATE_OPPDETECTED = WSTATE_ENEMYBOT + BWorldState.DELIMITER + "isDetected";
  public static String WSTATE_OPPANGLE = BBot.stateCode + BWorldState.DELIMITER + "angleToEnemy";
  public static String WSTATE_OPPDISTANCE = BBot.stateCode + BWorldState.DELIMITER+  "distanceToEnemy";
  public static String WSTATE_MEDETECTED = BBot.stateCode + BWorldState.DELIMITER+  "detectedByEnemy";
  public static String WSTATE_OPPFACING = WSTATE_ENEMYBOT + BWorldState.DELIMITER + "angleToMe";
  public static String WSTATE_OPPREACH= WSTATE_ENEMYBOT + BWorldState.DELIMITER + "reach";
  public static String WSTATE_MYREACH= BBot.stateCode + BWorldState.DELIMITER + "reach";

  public BSimulation(BBot botA, BBot botB, BArena arenaIn, float battleDurationIn, BSimulationCanvas nArena3dCanvas, boolean newrecordBattle)
  {
    this( botA, botB, arenaIn,  battleDurationIn,  nArena3dCanvas,  newrecordBattle,  true);

  }

  public BSimulation(BBot botA, BBot botB, BArena arenaIn, float battleDurationIn, BSimulationCanvas nArena3dCanvas, boolean newrecordBattle, boolean recordBothBots)
  {
    //BDebug.print("BSimulation: Created.");

    lastDeadlockTime = -180.0f; // init the last deadlock time

    simTimer = new BTimer( "Simulation Time" );
    //botExecTimer = new BTimer( "Bot Execute Total" );
    //movesAITimer = new BTimer( "  ... AI" );
    //movesMVTimer = new BTimer( "  ... Moves" );
    //movesCSTimer = new BTimer( "  ... CalcBotStats" );
    //movesINTimer = new BTimerStub( "      ... CalcInertia" );
    //movesTQTimer = new BTimerStub( "      ... CalcTorque" );
    //collisionTimer = new BTimer( "Collisions" );
    //rollbackTimer = new BTimer( "Rollbacks" );
    //recordTimer = new BTimer( "Recording" );
    simTimer.start();

    universeCanvas =  nArena3dCanvas;
    arena = arenaIn;
    arena.setSimulation(this);

    battleDuration = battleDurationIn;
    recordBattle = newrecordBattle;

    this.botA = botA;
    this.botB = botB;

    if (botA != null)
      botA.initArenaDetails(arena, botB, this);
    if (botB != null)
      botB.initArenaDetails(arena, botA, this );

    // Set up the battle recorder
    if (recordBattle)
      battleRecorder = new BBattleRecorder(botA, botB, arena.getFileName(),BGlobal.CALC_TIMESLICE, recordBothBots );

      // Stub for BBot Timers
      // One of the bots may be null for playing moves in the moves panel


  }

  public BBattleRecorder getBattleRecorder()
  {
    return battleRecorder;
  }

  public void cleanse()
  {

    arena = null;

    battleRecorder = null;
    universeCanvas = null;

       // Should be replaced with a more elegant solution to handle more than two bots
    botA = null;
    botB = null;

    //opponentBot = null;
    //selfBot = null;
    //boolean allBotsAlive = true;

    componentsToDetach = null; // A list of components to detach from the universe at the end of the cycle
    componentsToDetachTotal= null; // A list of components for the battleRecorder .
    collisionsTotal = null; // A list of components to detach from the universe at the end of the cycle
    simTimer = null;

    //BDebug.print ("BSimulation: Cleansed.");

  }

  /*protected void finalize() throws Throwable
  {
    BDebug.print ( "[ Simulation.finalize ]");
  }  */


  public boolean startMoveSlice( float timeSlice )
  // for the moves panel
  {
    return startMoveSlice(  timeSlice, botA );

  }

  public boolean startMoveSlice( float timeSlice , BBot runningBot)
  // PRE, THE TIME SLICE SHOULD BE normal calslice * multiple
  {

      boolean moved = false;

      moved = runningBot.runStartMove(timeSlice);

      // stop bot moving off the start position
      BArenaObjectPosition start = runningBot.getStartArenaPosition();
      BArenaObjectPosition newPos = runningBot.getArenaPosition();
      newPos.x =  start.getX();
      newPos.z =  start.getZ();
      newPos.facing = start.facing;

      if (recordBattle)
          battleRecorder.add(null, null);

      return moved;

  }

   public boolean victoryMoveSlice( float timeSlice , BBot runningBot)
  // PRE, THE TIME SLICE SHOULD BE normal calslice * multiple
  {

      boolean moved = false;

      BArenaObjectPosition newPos = runningBot.getArenaPosition();
      float x = newPos.getX();
      float z = newPos.getZ();
      float facing = newPos.facing;

      moved = runningBot.runVictoryMove(timeSlice);

      // stop bot moving off the start position
      newPos = runningBot.getArenaPosition();
      newPos.x =  x;
      newPos.z =  z;
      newPos.facing = facing;


      if (recordBattle)
          battleRecorder.add(null, null);

      return moved;

  }



  public void initStartMoves()
  {
    time = 0;
    botA.resetStartMove();
    if (botB != null)
      botB.resetStartMove();


  }

  public void endStartMoves()
  {
  }

  public void victoryMoves()
  // PRE: This is called after initArenaDetails

  {

    // special case to enter the loop, set finished to false on every loop execution
    BBot winner = battleRecorder.getVictoriousBot();
    BDebug.print ("BSimulation Victory Move victorious bot " + winner);



    if (winner != null)
    {
      BArenaEvent vicMoves = new BArenaEvent()
      {
        static final long serialVersionUID = 6898423788819294506L ;
        public int getStateModifier()
          { return 0; }

        public String getArenaEventType()
         {  return VICTORY_MOVES_START; }

      };

      battleRecorder.addArenaEvent(vicMoves);
      
      winner.resetVictoryMove();
      while( victoryMoveSlice(BGlobal.CALC_TIMESLICE, winner) );

    }
    else
    {
      //BDebug.print ("BSimulation TEST HACK adding extra frames if draw");
      if (recordBattle)
      {
         // battleRecorder.add(null, null);
          //battleRecorder.add(null, null);
          battleRecorder.add(null, null);

      }
    }


  }

  public float getElapsedTime()
    { return time; }

  public float getRemainingTime()
    { return battleDuration - time; }




  public void setBattleRunning (boolean battleRunningIn)
  {
    battleRunning = battleRunningIn;
  }

  public boolean getBattleRunning ()
  {
    return battleRunning;
  }



  public void stopBattle()
  {


    BDebug.print("===== Ending Battle =====");
    if (battleRunning)
    {

        //pbar.setValue(pbar.getMaximum());
        simTimer.stop();

        BDebug.print( simTimer );
        BDebug.print(simTimer.printRatio(battleSlice));

        /*
        botExecTimer.stop();
        //movesAITimer.stop();
        //movesMVTimer.stop();
        //movesCSTimer.stop();
        //movesINTimer.stop();
        //movesTQTimer.stop();
        collisionTimer.stop();
        rollbackTimer.stop();
        recordTimer.stop();
        BDebug.print( "--- Bot Moves Timers ---");
        BDebug.print( botExecTimer + " - " + botExecTimer.pctOf(simTimer.getTotalTime()));
        //BDebug.print( movesAITimer + " - " + movesAITimer.pctOf(simTimer.getTotalTime()) );
        //BDebug.print( movesMVTimer + " - " + movesMVTimer.pctOf(simTimer.getTotalTime()) );
        //BDebug.print( movesCSTimer + " - " + movesCSTimer.pctOf(simTimer.getTotalTime()) );
        //BDebug.print( movesINTimer + " - " + movesINTimer.pctOf(simTimer.getTotalTime()) );
        //BDebug.print( movesTQTimer + " - " + movesTQTimer.pctOf(simTimer.getTotalTime()) );

        BDebug.print( "--- Other Sim Timers ---");
        BDebug.print( collisionTimer + " - " + collisionTimer.pctOf(simTimer.getTotalTime()) );
        BDebug.print( rollbackTimer + " - " + rollbackTimer.pctOf(simTimer.getTotalTime()) );
        BDebug.print( recordTimer + " - " + recordTimer.pctOf(simTimer.getTotalTime()) );
        */
    }



    calculateWinner();
    if (performStartVictoryMoves)
      victoryMoves();



   
  }

  float battleSlice;
  float progressTimeCounter = 1f;

  public boolean runBattleSlice()
  {

      run();

      //BDebug.print("Time " + battleSlice);
      time = battleSlice;
      battleSlice += BGlobal.CALC_TIMESLICE ;

      return battleNotFinished() ;

  }

  protected static class BEndEvent implements  BArenaEvent
  {
    static final long serialVersionUID = -1L ;

    public BEndEvent()
    {}

    public int getStateModifier()
      { return 0; }

    public String getArenaEventType()
      {  return BArenaEvent.END; }

  }
  

  private int framesSinceBotDied = 0;
  private static int EXTRAFRAMES = BGlobal.FRAMES_PER_SEC  ;

  private boolean battleNotFinished()
  {
    if (! (botA.getBotAlive() && botB.getBotAlive() )  )
    {
      if (recordBattle)
      {

        battleRecorder.setWinString(botA.getTerminationString() + botB.getTerminationString() );

        // add end battle frame
        if (framesSinceBotDied == 0)
        {
          BArenaEvent endBattleMoves = new BEndEvent();

          battleRecorder.addArenaEvent(endBattleMoves);
        }

      }


      framesSinceBotDied ++;

    }
    return   battleSlice<=battleDuration  &&  ( framesSinceBotDied < EXTRAFRAMES );
  }

  private void preBattleDestructions()
  {
    //BDebug.print ("BSimulation.preBattleDestructions called");
    Vector v = new Vector(1);
    componentsToDetach.clear();

    // DJG June 2001
    // Taken out due to erratic explosions (some thread timing issue)
    // Checked in main Simulation Loop (after a period of time)
    //this.botA.testSelfCollisions(v, universeCanvas.getLocale3d(), botA.getChassis() );
    //this.botB.testSelfCollisions(v, universeCanvas.getLocale3d(), botB.getChassis() );
    
   // BDebug.print ("BSimulation.preBattleDestructions v" + v);


    Vector botALength = botA.getComponentsPastMaxDist();
    if (botALength.size() != 0)
    {
      for (int i = botALength.size()-1; i >= 0 ; i--)
      {
        BComponent comp = (BComponent) botALength.get(i);
        destroyComponent(botA, comp);
        botA.setArenaComments(comp + " violates bot size restrictions, component removed");

      }
    }

    Vector botBLength = botB.getComponentsPastMaxDist();
    if (botBLength.size() != 0)
    {
      for (int i = botBLength.size()-1; i >= 0 ; i--)
      {
        BComponent comp = (BComponent) botBLength.get(i);
        destroyComponent(botB, comp);
        botB.setArenaComments(comp + " violates bot size restrictions, component removed");

      }
    }



    if (v.size() != 0)
    {
      for (int i = v.size() -1; i>=0; i--)
      {
         BCollisionRecord colRec = (BCollisionRecord) v.get(i);
         colRec.armComponent.getBot().setArenaComments("Collided: "+ colRec.armComponent.toString()+" & "+  colRec.triggerComponent.toString());

         if (!colRec.armComponent.getChassisLock() )
             destroyComponent(colRec.armComponent.getBot(), colRec.armComponent);
         if (!colRec.triggerComponent.getChassisLock() )
            destroyComponent(colRec.triggerComponent.getBot(), colRec.triggerComponent);
      }

    }


    if (recordBattle)
      if (componentsToDetach.size() != 0)
      {
          battleRecorder.add(componentsToDetach,v);
          detachFromUniverse( componentsToDetach );
      }
    // recurseSelfCollisionTest(myChassis, myChassis, collisionList, myLocale, true);
   // recurseSelfCollisionTest(otherChassis, otherChassis, collisionList, myLocale, true);

  }

  public void startBattle ()
  {
	  //MDF - to fix the energy bug?
	  botA.validate(false);
	  botB.validate(false);	  
    battleRunning = true;

    //BDebug.print("BSimulation.StartBattle Called");

    BDebug.print("===== Starting Battle =====");

    if (performStartVictoryMoves)
      startMoves();

    // DJG July 2001
    // Need to reset everything after Start Moves
    resetBotPositions();
    botA.calculateBotStats(true, false, BGlobal.CALC_TIMESLICE, true);
    botB.calculateBotStats(true, false, BGlobal.CALC_TIMESLICE, true);
    botA.resetForSim();
    botB.resetForSim();

    preBattleDestructions();

    // Record the start of the fight timeSlice
    battleRecorder.setBattleStarted();

    battleSlice = 0;
    progressTimeCounter = 1f;
    maxInboundsVelocity = 1.0f;
    maxAllVelocity = 1.0f;

  }

  public void startMoves()
  {
    BArenaEvent botAstart = new BArenaEvent()
    {
       static final long serialVersionUID =5408342248482267961L;

       public int getStateModifier()
       { return 0; }

      public String getArenaEventType()
      { return BOTA_STARTMOVE; }

    };

    BArenaEvent botBstart = new BArenaEvent()
    {
      static final long serialVersionUID =7050284635653950695L;
      
       public int getStateModifier()
       { return 0; }

      public String getArenaEventType()
      { return BOTB_STARTMOVE; }

    };

    BArenaEvent fin = new BArenaEvent()
    {
      static final long serialVersionUID = 8211608205746005959L;
      
       public int getStateModifier()
       { return 0; }

      public String getArenaEventType()
      { return STARTMOVES_FIN; }

    };

    initStartMoves();

    battleRecorder.addArenaEvent(botAstart);
    while( startMoveSlice(BGlobal.CALC_TIMESLICE, botA) );
    battleRecorder.addArenaEvent(botBstart);
    while( startMoveSlice(BGlobal.CALC_TIMESLICE, botB) );
    battleRecorder.addArenaEvent(fin);

    endStartMoves();

  }


  public void calculateWinner()
  {
    
    if ((botA.isAlive && botB.isAlive) || (!botA.isAlive && !botB.isAlive))
    {
      // Either both bots are alive or both are dead
      float botAPctDamage = botA.getPercentageDamageLevel();
      float botBPctDamage = botB.getPercentageDamageLevel();

      if (botAPctDamage != botBPctDamage)
      {
        if (botAPctDamage < botBPctDamage)
          battleRecorder.setVictoriousBots(botA);
        else
          battleRecorder.setVictoriousBots(botB);

        if (recordBattle)
          if (battleRecorder.getWinString().equals("") )
            battleRecorder.setWinString("Time over. Points victory.");

      }
      else if (botA.getPercentageDamageLevel() == 0f && botB.getPercentageDamageLevel() == 0f)
      {
        // No damage was done
        // so they are both made victorious ie a Draw
        battleRecorder.setVictoriousBots(null);
      }
      else if (botA.getTotalPrice() != botB.getTotalPrice())
      {
        if (botA.getTotalPrice() < botB.getTotalPrice())
          battleRecorder.setVictoriousBots(botA);
        else
          battleRecorder.setVictoriousBots(botB);

        if (recordBattle)
          if (battleRecorder.getWinString().equals("") )
            battleRecorder.setWinString("Time over. Cost Victory.");

      }
      else
      {
        // OK so they both have the same damage and they both cost the same
        // so they are both made victorious
        battleRecorder.setVictoriousBots(null);
       // battleRecorder.setVictoriousBots(botB);
      }
    }
    else
    {
      //  Find the first bot alive and make it victorious
      if (botA.isAlive)
        battleRecorder.setVictoriousBots(botA);
      else if (botB.isAlive)
        battleRecorder.setVictoriousBots(botB);
      else
        battleRecorder.setVictoriousBots(null);
    }

  }

  private void resetBotPositions()
  {
      botA.resetBotPosition();
      botB.resetBotPosition();

  }

  private void checkDeadlockBreak()
  {
    if (BGlobal.DEADLOCKCOUNT > BGlobal.DEADLOCKS_TO_RESETFIGHT)
    {
      resetBotPositions();
      botA.setArenaComments( "--- DEADLOCK BREAK ---" );
      botB.setArenaComments( "--- DEADLOCK BREAK ---" );

      BGlobal.DEADLOCKCOUNT = 0;

      lastDeadlockTime = time;
    }
  }

  private float getTimeSinceDeadlock()
  {
    return time - lastDeadlockTime;
  }

  private float getDeadlockPercent()
  {
    return getBattleRecorder().getDeadlockPercent();
  }

  private boolean addedChassis = false;

  private void checkChassisDeath(BBot bot, Vector componentsToDetach)
  {                             
    if ( bot.getChassis().getDestroyed() )
    {
         // BDebug.print ("BChassis.execute: Chassis Destroyed ");
          bot.setBotAlive(false);
          bot.setArenaComments("Chassis destroyed");
          bot.getOpponentBot().setArenaComments(bot + " chassis destroyed " );

          if (recordBattle)
            bot.setTerminationString("Chassis destroyed");

          if (!addedChassis)
          {
            addedChassis = true;
            BDebug.print ("BChassis.execute: Chassis Destroyed ");
            componentsToDetach.add( bot.getChassis() );

          }
    }
  }

  //public transient float sliceKeeper = 0.0f; // controls slice 'deceleration'
  //public transient int prevSlices = 0; // Debug - To show change in slices

  /* DJG June 2001 - No Longer Used
  protected int calculateSubCycles(boolean botsBoundsCollide)
  {

    if (!BGlobal.USE_VARIABLE_TIMESLICE )
      return 1;
    else
    {
      if (framesSinceBotDied != 0 )
        return 1;

      int slices = 1;

      float fastestC = botA.fastestInboundsComponentSpeed;
      fastestC = Math.max(fastestC, botB.fastestInboundsComponentSpeed);

      maxInboundsVelocity = Math.max(maxDetectedVelocity,fastestC);

      if ( botsBoundsCollide )
      {

        slices = Math.round(fastestC / BPhysics.MAX_VELOCITY_PER_SLICE);

        if (slices <2)
          slices = 2;
        else if (slices > BGlobal.USER_MAX_SIM_SLICES)
          slices =  BGlobal.USER_MAX_SIM_SLICES;

        /* DJG June 2001 - No need as engine now checks instantly
        if ( sliceKeeper < slices ) {
          sliceKeeper = slices; // always goes up as fast as possible
        } else {
          sliceKeeper -= 0.05; // will only go down 0.75 slices every second
          slices = java.lang.Math.max(java.lang.Math.round(sliceKeeper), slices); // not totally accurate but s/b good enough (within 0.5 slices)
        }


        /*
        if (slices != prevSlices)
        	BDebug.print("BSim.calcSubCycles: slices="+slices+"  keeper="+sliceKeeper);
        prevSlices = slices;


      /*
       //if (fastestC < 10)
         // slices = 1;
        if (fastestC < 10)
          slices = 2;
        else if (fastestC < 20)
          slices = 3;
        else if (fastestC < 25)
          slices = 4;
        else
          slices = 5;

      //  BDebug.print ("BSimulation inBounds slices " + slices + " getFastestComponentSpeed: " +  fastestC );


      }
      else
      {



        if (BGlobal.USER_MAX_SIM_SLICES != BPhysics.MAX_SLICES )
          slices = 1;
        else
          slices = Math.round(fastestC / BPhysics.MAX_VELOCITY_PER_SLICE) / 3;

        if (slices <1)
          slices = 1;
        else if (slices > 3)
          slices =  3;

      /*
        if (fastestC < 25)
          slices = 1;
        else if (fastestC < 35)
          slices = 2;
        else if (fastestC < 60)
          slices = 3;

       // BDebug.print ("BSimulation outofbounds slices " + slices + " getFastestComponentSpeed: " +  fastestC );


      }


      return slices;
    }
  }
  */

  // Used for Sim Engine Scaling
  //private static Point3f run_botPosTemp = new Point3f();
  private static transient Vector collisions = new Vector();

  // DJG June 2001
  private transient float maxInboundsVelocity = 1.0f;
  private transient float maxAllVelocity = 1.0f;
  private transient float lastInboundsVelocityTimer = 0.0f;
  private transient float lastAllVelocityTimer = 0.0f;
  private transient int prevCyclesPerTimeSlice = 0;
  private transient boolean botsBoundsCollide = false;
  private transient float lastVelocityTimer = 0.0f;
  private transient boolean selfCollisionsProcessed = false; // Delayed processing of starting self collisions

  private void run()
  {

    //BDebug.print("Time: "+battleSlice);

    botA.setBattleTime(battleSlice);
    botB.setBattleTime(battleSlice);
    // clear cache
    componentsToDetach.clear();

    //recordCounter -= 1;

    boolean worldStateCycle = false;
    boolean botStatsCycle = false;

    // Scaling of Engine Due to Max Velocity Detection

    // Inbounds Components Velocity Check
    float fastestC = Math.max(botA.fastestInboundsComponentSpeed, botB.fastestInboundsComponentSpeed);
    maxInboundsVelocity = Math.max(maxInboundsVelocity,fastestC);

    if ( ( battleSlice - lastInboundsVelocityTimer ) > 1.0f ) // Max not detected for 1 seconds
    {
       maxInboundsVelocity = Math.max(botA.fastestInboundsComponentSpeed, botB.fastestInboundsComponentSpeed);
       maxInboundsVelocity = Math.max(maxInboundsVelocity,1.0f);
       lastInboundsVelocityTimer = battleSlice;
    }

    // All Components Velocity Check
    fastestC = Math.max(botA.fastestComponentSpeed, botB.fastestComponentSpeed);
    maxAllVelocity = Math.max(maxAllVelocity,fastestC);

    if ( ( battleSlice - lastAllVelocityTimer ) > 1.0f ) // Max not detected for 1 seconds
    {
       maxAllVelocity = Math.max(botA.fastestComponentSpeed, botB.fastestComponentSpeed);
       maxAllVelocity = Math.max(maxAllVelocity,1.0f);
       lastAllVelocityTimer = battleSlice;
    }

    // Now calculate engine cycles
    int cyclesPerTimeSlice = BPhysics.MAX_SLICES;
    int velocityCheckCycle = BPhysics.VELOCITY_CHECK_CYCLE;

    if ( BGlobal.USER_MAX_SIM_SLICES < BPhysics.MAX_SLICES )
    {
      cyclesPerTimeSlice = BGlobal.USER_MAX_SIM_SLICES;
      velocityCheckCycle = BGlobal.USER_VELOCITY_CHECK_CYCLE;
    }
    else if ( maxInboundsVelocity < BPhysics.MAX_VELOCITY_CAP )
    {
    	for (int i = BPhysics.MAX_SLICES-BPhysics.VELOCITY_CHECK_CYCLE; i >= BPhysics.VELOCITY_CHECK_CYCLE; i-=BPhysics.VELOCITY_CHECK_CYCLE )
    	{
      	float scale = BPhysics.MAX_SLICES / i;
      	if ( maxInboundsVelocity < ( BPhysics.MAX_VELOCITY_CAP / scale ) )
         	cyclesPerTimeSlice = i;
    	}
      velocityCheckCycle = cyclesPerTimeSlice * (BPhysics.VELOCITY_CHECK_CYCLE / BPhysics.MAX_SLICES);
    }

    // Must process Velocity Cap at regular intervals
    if ( !botsBoundsCollide )
    {
    	if ( maxAllVelocity > BPhysics.MAX_VELOCITY_CAP )
      {
        cyclesPerTimeSlice = BPhysics.MAX_SLICES / BPhysics.VELOCITY_CHECK_CYCLE;
        velocityCheckCycle = 1;
      }
      else
        cyclesPerTimeSlice = Math.max(1,(int)prevCyclesPerTimeSlice / 2);
    }

    //int subCycles = calculateSubCycles();

    //int lastCollisionCtr = 0;
    int lastVelocityCtr = velocityCheckCycle - 1; // Velocity is checked in first cycle

    if ( cyclesPerTimeSlice != prevCyclesPerTimeSlice )
    {
      //BDebug.print ("BSimulation.run(): ["+battleSlice+"] Cycles=" + cyclesPerTimeSlice + " MaxVelocity=" + maxInboundsVelocity  );
      prevCyclesPerTimeSlice = cyclesPerTimeSlice;
      lastInboundsVelocityTimer = battleSlice;
      lastAllVelocityTimer = battleSlice;
    }

    // DJG June 2001 - Must do this until I can ensure component movement can be detected
    //velocityCheckCycle = java.lang.Math.min(velocityCheckCycle,2);


    // MAIN LOOP
    //for (int subCounter = 0; subCounter < subCycles; subCounter++ )
    for (int subCounter = 0; subCounter < cyclesPerTimeSlice; subCounter++ )
    {

      float thisTimeSlice = BGlobal.CALC_TIMESLICE / cyclesPerTimeSlice;
      lastVelocityTimer += thisTimeSlice;

      // clear cache
      //componentsToDetach.clear();

      // DJG June 2001
      // The following is only done on the WorldStateCycle (every 1/15th sec)
      // AI Processed
      // Torque Calculated
      // Zone Collisions
      // Self Collisions
      worldStateCycle = (subCounter == 0);  // Used for AI, Torque
      //worldStateCycle = (subCounter == (cyclesPerTimeSlice - 1) );  // Used for AI, Torque
      // Should really be (but haven't got time to test)
      //worldStateCycle = (subCounter == (cyclesPerTimeSlice - velocityCheckCycle + 1) );

      // Primarily for Velocity Calculations
      lastVelocityCtr += 1;
      if ( lastVelocityCtr >= velocityCheckCycle )
      {
        botStatsCycle = true;
        botsBoundsCollide = BCollision.boundsIntersect(botA.getChassis().getLiveNode(), botB.getChassis().getLiveNode());
      }
      else
        botStatsCycle = false;

      //botExecTimer.start();

      //if (worldStateCycle)
      //{
        //botA.execute(BGlobal.CALC_TIMESLICE / subCycles, worldStateCycle,universeCanvas.getLocale3d()   );
        //botB.execute(BGlobal.CALC_TIMESLICE / subCycles, worldStateCycle, universeCanvas.getLocale3d() );

      botA.execute(thisTimeSlice, worldStateCycle, lastVelocityTimer, botStatsCycle);
      botB.execute(thisTimeSlice, worldStateCycle, lastVelocityTimer, botStatsCycle);

      // for the lastCollisionTime
      // DJG June 2001 - Already done once this cycle...
      //botA.setBattleTime(battleSlice);
      //botB.setBattleTime(battleSlice);

      //}
      //else
      //{
        //botA.execute(BGlobal.CALC_TIMESLICE / subCycles, worldStateCycle, null);
        //botB.execute(BGlobal.CALC_TIMESLICE / subCycles, worldStateCycle, null);
      //}

      //botExecTimer.stop();

      // Test to see if collision test required
      //int collisionCycle = (int) cyclesPerTimeSlice / calculateSubCycles(botsBoundsCollide);
      //lastCollisionCtr += 1;

      //if (lastCollisionCtr >= collisionCycle || worldStateCycle)
      if ( (botsBoundsCollide || worldStateCycle)
            && (botA.movedInLastTimeslice || botB.movedInLastTimeslice) && battleSlice > 0.5f )
      {
       //collisionTimer.start();

       //lastCollisionCtr = 0;

       botA.setCollided(false, time);
       botB.setCollided(false, time);

       // COLLISION DETECTION
       // Returns Vector of BCollisionRecord objects
       //Vector collisions = arena.testCollisions(universeCanvas.getLocale3d(), false, botA, botB);

       collisions.clear();
       botA.testCollisions(botB,collisions,universeCanvas.getLocale3d(),worldStateCycle,botsBoundsCollide,false);

       // Process self collisions 0.5 seconds into fight
       if ( !selfCollisionsProcessed )
       {
       		if ( collisions.isEmpty() )
       		{
          	botA.testSelfCollisions(collisions, universeCanvas.getLocale3d());
          	botB.testSelfCollisions(collisions, universeCanvas.getLocale3d());
          	selfCollisionsProcessed = true;
       		}
       }

       //collisionTimer.stop();
       
       //rollbackTimer.start();

       if ( !collisions.isEmpty() )
       {

        //BDebug.print("Processing Collisions: "+collisions.size());

        // DJG June 2001 - Rollback moved to stop "rubbing" damage
        botA.rollBack(universeCanvas.getLocale3d(), collisions, botsBoundsCollide);
        botB.rollBack(universeCanvas.getLocale3d(), collisions, botsBoundsCollide);

        for ( int i= collisions.size()- 1; i>= 0; i-- )
        {
          BCollisionRecord colRec = (BCollisionRecord)collisions.get(i);

          // Scale Inertia due to new rollback position.
          colRec.armComponent.calcAdjustedInertia(lastVelocityTimer);
          colRec.triggerComponent.calcAdjustedInertia(lastVelocityTimer);

          calculateCollisionVectors(colRec);

          BBot armBot = colRec.armComponent.getBot();
          BBot trgBot = colRec.triggerComponent.getBot();

          float impactInertia = rTrgNetInertia.length() * BPhysics.PERCENT_INERTIA_AFFECTS_DAMAGE;
          //BDebug.print("Impact Inertia = "+impactInertia);

          // check if this collisions occured last round
          if (lastCycleCollisions.indexOf(colRec) != -1 )
          {
            colRec.setInContact(true);
            colRec.setNullified(true);

            // Pause all moves if thrashing
            if ( armBot != null )
            {
              if ( impactInertia > BPhysics.DAMAGE_REQD_TO_STOP_MOVE )
                armBot.stopMoves(colRec.armComponent);
              else
                armBot.pauseMoves(colRec.armComponent);
              armBot.pauseChassisMove();
              //armBot.setArenaComments("-- NULL collision -- "+impactInertia);
            }

            if ( trgBot != null )
            {
              if ( impactInertia > BPhysics.DAMAGE_REQD_TO_STOP_MOVE )
                trgBot.stopMoves(colRec.triggerComponent);
              else
                trgBot.pauseMoves(colRec.triggerComponent);
              trgBot.pauseChassisMove();
              //trgBot.setArenaComments("-- NULL collision -- "+impactInertia);
            }

          }
          else if (!colRec.getNullified() )
          {
            processCollision(colRec, impactInertia);

          }

          collisionsTotal.add(colRec);

        }

        // Check to see if either collided with Wall. Recheck collisions.
        if ( ! (botA.completelyRolledBack && botB.completelyRolledBack) )
        {
          //BDebug.print("BSimulation.run(): Retesting Collisions after an Arena Impact");
          botA.testCollisions(botB,collisions,universeCanvas.getLocale3d(),worldStateCycle,botsBoundsCollide,false);
        }

        // DJG June 2001
        // Spiffy Rollback Thingy
      	boolean stillColliding = false;
        boolean botAinCollision = false;
        boolean botBinCollision = false;
      	if (BGlobal.ADVANCED_PASSTHROUGH_CORRECTION && (botA.completelyRolledBack || botB.completelyRolledBack))
      	{
        	stillColliding = true;
      		for ( int i=1; i <= BGlobal.FULL_ROLLBACK_DEPTH && stillColliding; i++ )
        	{
          	stillColliding = false;
            botAinCollision = false;
        		botBinCollision = false;
        		for ( int j= collisions.size()- 1; j>= 0 && !stillColliding; j-- )
        		{
          		BCollisionRecord colRec = (BCollisionRecord)collisions.get(j);
          		stillColliding = BCollision.intersect(colRec.getArmComponent(),colRec.getTriggerComponent(), universeCanvas.getLocale3d(), false, false);
              if ( colRec.containsBot(botA) )
                botAinCollision = true;
              if ( colRec.containsBot(botB) )
                botBinCollision = true;
        		}
          	if ( stillColliding )
            {
              if ( botAinCollision )
                botA.completelyRollback(collisions);
              if ( botBinCollision )
                botB.completelyRollback(collisions);
            }
        	}
      	}


       } // End Collisions Empty


       // update the last CycleCollisions Vector
       //lastCycleCollisions.clear();
       lastCycleCollisions.addAll(collisions);


       // recorder stubb
       componentsToDetachTotal.addAll(componentsToDetach);

       if ( !collisions.isEmpty() )
       {
        // if deadlock detected then create clearance Moves
          botA.createResultantMove(universeCanvas.getLocale3d(), collisions);
          botB.createResultantMove(universeCanvas.getLocale3d(), collisions);

       }

       detachFromUniverse( componentsToDetach );

       // clear cache
       componentsToDetach.clear();

       //rollbackTimer.stop();
      } // End Collision Processing

      if (botStatsCycle == true)
      {
        lastVelocityTimer = 0.0f;
        lastVelocityCtr = 0;
      }

    } // End Sub Cycles Loop

    //recordTimer.start();

    checkDeadlockBreak();
    checkChassisDeath(botA, componentsToDetach);
    checkChassisDeath(botB, componentsToDetach);

    if (recordBattle)
         battleRecorder.add(componentsToDetachTotal, collisionsTotal);

    componentsToDetachTotal.clear();
    collisionsTotal.clear();
    lastCycleCollisions.clear();

    // DJG June 2001
    botA.getChassis().setTimer(battleSlice);
    botB.getChassis().setTimer(battleSlice);

    /*
    if ( battleSlice >= debugTimer && battleSlice < debugTimer + 0.1 )
    {
      BDebug.print("---"+botA.toString()+"---");
      botA.debugDump();
      BDebug.print("---"+botB.toString()+"---");
      botB.debugDump();
      botB.setBotAlive(false);
    }
    */

    //recordTimer.stop();

  }

  private void detachFromUniverse(Vector componentsToDetach )
  {
    // remove all components that need to be removed from the scene graph
    // we do this here because we previously needed them in for resultant force calculations
    for (int i = 0 ; i < componentsToDetach.size(); i++)
    {
        BComponent destroyedComponent =    (BComponent)componentsToDetach.get(i);
        //BDebug.print("Detaching Component: "+destroyedComponent);
        universeCanvas.removeComponent( destroyedComponent );
    }

  }


  // net working vectors
  private static Vector3f nTrgNetInertia = new Vector3f();
  private static Vector3f nArmNetInertia = new Vector3f();

  private static Vector3f momentumSum = new Vector3f();

  // resultant working vectors
  private static Vector3f rTrgNetInertia = new Vector3f();
  private static Vector3f rArmNetInertia = new Vector3f();

  private static Vector3f armNetInertia = new Vector3f();
  private static Vector3f trgNetInertia = new Vector3f();

  //private static Vector3f directionDotTest = new Vector3f();


  private void calculateCollisionVectors(BCollisionRecord colRec)
  {
    armNetInertia.set(colRec.armComponent.getCollisionMomentum());

   // the floor will always be the trigger component
   if ( colRec.triggerComponent == BArena.arenaComponent)
   {
      trgNetInertia.x = 0;
      trgNetInertia.y = 0;
      trgNetInertia.z = 0;
   }
   else
    trgNetInertia.set(colRec.triggerComponent.getCollisionMomentum());


   //float armInertia = armNetInertia.length();
   //float trgInertia = trgNetInertia.length();

   //momentumSum.sub(armNetInertia, armNetInertia);

   // Only used for sound and sparks
   //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
   //colRec.setCollisionMomentum(armNetInertia.length() + armNetInertia.length() );
   // DJG June 2001 - changed second to TRG (ARM may not have moved ... therefore no sound)
   //colRec.setCollisionMomentum(armNetInertia.length() + trgNetInertia.length());
   momentumSum.sub(armNetInertia, trgNetInertia);
   colRec.setCollisionMomentum(momentumSum.length());
   //XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX



   //botA.setArenaComments("BCollision collisionMomentum " + ( armInertia +  trgInertia ) );
   //botB.setArenaComments("BCollision collisionMomentum " + ( armInertia +  trgInertia ) );
   
   // check inertia directions


   // Create Vector for collision impact on each component
   // Used to calculate resultant force and movement
   // inert inertia
   nTrgNetInertia.set(trgNetInertia);
   nArmNetInertia.set(armNetInertia);

   nTrgNetInertia.negate();
   nArmNetInertia.negate();

   // resultant direction inertia on a component is  the inverse of it's interia +  the collisions inertia
   rTrgNetInertia.add(nTrgNetInertia, armNetInertia );
   rArmNetInertia.add(nArmNetInertia, trgNetInertia);

   if (colRec.armComponent.collisionInertia == null)
    colRec.armComponent.collisionInertia = new Vector3f(rArmNetInertia);
   else
    colRec.armComponent.collisionInertia.set( rArmNetInertia );

   if ( colRec.triggerComponent.collisionInertia == null)
    colRec.triggerComponent.collisionInertia = new Vector3f(rTrgNetInertia);
   else
    colRec.triggerComponent.collisionInertia.set( rTrgNetInertia);


  }

  private void processCollision(BCollisionRecord colRec, float impactInertia)
  {

   // Calculate Damage

   BBot armBot = colRec.armComponent.getBot();
   BBot trgBot = colRec.triggerComponent.getBot();

   String colRecBotName;
    // check names for nullolRecBotName;
   if (colRec.armComponent.getBot() == null)
    colRecBotName = "";
   else
    colRecBotName = colRec.armComponent.getBot().toString();

   String colRecTrigBotName;
   if (colRec.triggerComponent.getBot() == null)
    colRecTrigBotName ="";
   else
    colRecTrigBotName = colRec.triggerComponent.getBot().toString();

   if ( botA != null )
    botA.setArenaComments("-- collision --");
   if ( botB != null )
    botB.setArenaComments("-- collision --");

   //BDebug.print(colRecBotName + "( " + colRec.armComponent.toString() + " ) collides with " + colRecTrigBotName + " ( " + colRec.triggerComponent.toString() + ")");
   //BDebug.print(armBot+" - "+trgBot);
   //botB.setArenaComments(colRecBotName + "( " + colRec.armComponent.toString() + " ) collides with "  + colRecTrigBotName + " ( " + colRec.triggerComponent.toString() + ")");

   /*
   // CALCULATE THE DAMAGE
   float impactInertia = rTrgNetInertia.length();
   // BDebug.print ("BSimulation impact Inertia " + impactInertia );

   // root the inertia to reduce the range and amount
   impactInertia = impactInertia * BPhysics.PERCENT_INERTIA_AFFECTS_DAMAGE;
   // BDebug.print ("BSimulation impact Inertia after * " + impactInertia );
   */

   // DJG July 2001 - Make Damage Cap be Inertia Cap
   impactInertia = java.lang.Math.min(impactInertia,BPhysics.COMPONENT_DAMAGE_CAP);

   float tmpInertia = impactInertia;

   // Calculate Damage to Arm Component
   float armDamage = colRec.armComponent.resistDamage(colRec.triggerComponent.applyDamage(tmpInertia));

   tmpInertia = impactInertia;

   // Calculate Damage to Trigger Component
   float trgDamage = colRec.triggerComponent.resistDamage(colRec.armComponent.applyDamage(tmpInertia));

   // DJG July 2001
   // Damage Caps
   //armDamage = java.lang.Math.min(armDamage,BPhysics.COMPONENT_DAMAGE_CAP);
   //trgDamage = java.lang.Math.min(trgDamage,BPhysics.COMPONENT_DAMAGE_CAP);

   // DJG Sep 2001
   // Limit damage if other component is destroyed.
   if ( armBot != trgBot && trgBot != null )
   {
     if ( armDamage > colRec.armComponent.getHealthCurrent() )
     {
       float dmgRatio = colRec.armComponent.getHealthCurrent() / armDamage;
       trgDamage = trgDamage * dmgRatio;
       armDamage = colRec.armComponent.getHealthCurrent();
       //BDebug.print("Arm Component Destroyed - Trg Damage="+trgDamage+" Ratio="+dmgRatio);
     } else if ( trgDamage > colRec.triggerComponent.getHealthCurrent() ) {
       float dmgRatio = colRec.triggerComponent.getHealthCurrent() / trgDamage;
       armDamage = armDamage * dmgRatio;
       trgDamage = colRec.triggerComponent.getHealthCurrent();
       //BDebug.print("Trg Component Destroyed - Arm Damage="+armDamage+" Ratio="+dmgRatio);
     }
   }


   armBot.setCollided( true, time );
   armBot.setArenaComments(armBot.getName() + " ( " + colRec.armComponent.toString() + " ) " +" takes "+ Math.round(armDamage)+" damage");
   if ( armBot != trgBot )
   {
      if (trgBot != null)
        armBot.setArenaComments(trgBot.getName() + " ( " + colRec.triggerComponent.toString() + " ) " +" takes "+ Math.round(trgDamage)+" damage");

      // armBot.setArenaComments(trgBot.toString()+" takes "+BMath.format(trgDamage,1)+" damage");
   }

   colRec.armComponent.addToDamageCurrent((int)armDamage);

   if ( colRec.armComponent.getDestroyed() || armBot.isTipped()  )
    if ( !colRec.armComponent.getChassisLock())
         destroyComponent(armBot, colRec.armComponent);


   // Apply Damage to Trigger Component

   // trgBot can be null if it is the floor
   if (trgBot != null)
   {
    //trgBot.setTotalBotDamage( trgDamage) ;
    trgBot.setCollided(true, time);
    trgBot.setArenaComments(trgBot.getName() + " ( " + colRec.triggerComponent.toString() + " ) " +" takes "+ Math.round(trgDamage)+" damage");

    //trgBot.setArenaComments(colRec.triggerComponent.toString()+" takes "+BMath.format(trgDamage,1)+" Damage");
    if ( trgBot != armBot )
       trgBot.setArenaComments(armBot.getName() + " ( " + colRec.armComponent.toString() + " ) " +" takes "+ Math.round(armDamage)+" damage");

       //trgBot.setArenaComments(armBot.toString()+" takes "+BMath.format(armDamage,1)+" Damage");

    //BDebug.print(trgBot.getName() + " ( " + colRec.triggerComponent.toString() + " ) " +" takes "+ Math.round(trgDamage)+" damage");
    colRec.triggerComponent.addToDamageCurrent((int)trgDamage);

    if (  colRec.triggerComponent.getDestroyed()  || trgBot.isTipped()  )
     if ( !colRec.triggerComponent.getChassisLock())
         destroyComponent(trgBot, colRec.triggerComponent);


   }

   // Nullify Zero Collisions
   if ( Math.round(armDamage) <= 0 && Math.round(trgDamage) <= 0 )
   {
      colRec.setNullified(true);
   }

   /*
   // DJG June 2001
   // If chassis component and big damage then stop current chassis move
   if ( armDamage > 1f && colRec.armComponent.getChassisLock() )
   {
     //BDebug.print("BBot.processCollisions: Pausing Chassis Move for "+armBot.toString());
     armBot.pauseChassisMove();
   }
   if ( trgDamage > 1f && colRec.triggerComponent.getChassisLock() )
   {
     //BDebug.print("BBot.processCollisions: Pausing Chassis Move for "+trgBot.toString());
     trgBot.pauseChassisMove();
   }
   */


 }





 private void destroyComponent(BBot bot, BComponent component)
 {
     //BDebug.print ("BSimulation .destroyedComponent.called for "+bot+" - "+component);

     // DO NOT REMOVE THE COMPONENTS FROM THE SCENE GRAPH BECAUSE WE NEED THEM IN THE SCENE GRAPH
     // FOR CALCULATING RESULTANT MOVES
     // REMOVED AT END OF SIMULATION CYCLE

     if ( componentsToDetach.indexOf(component) == -1 )
     {
      componentsToDetach.add(component);

      // Destroy Bot Component
      // This controls ALL stuff related to damage calculation of dependant components
      bot.destroyComponent(component);

      //bot.setArenaComments(bot.getName() + ":" +component.getName()+" Destroyed");
      if ( botA != null )
        botA.setArenaComments("... "+component.getName()+" destroyed.");
      if ( botB != null )
        botB.setArenaComments("... "+component.getName()+" destroyed.");

     }
     // BDebug.print ("BSimulation .destroyedComponent.called ");



 }



 public static BWorldStateNode getWorldState(BBot theBot)
 {

    BWorldStateNode root = new  BWorldStateNode();

    char dot = BWorldState.DELIMITER;

    //root.put( new BWorldCategory(BBot.stateCode, "Me") );
    //root.put( new BWorldCategory(BBot.stateCode + dot+ BZones.stateCode, "Zones"));
    //root.put( new BWorldCategory(BBot.stateCode + dot+  BComponent.stateCode , "Components"));
    //root.put( new BWorldCategory(WSTATE_ENEMYBOT, "Enemy") );


    // my stats
    // root.put( new BWorldItem(BBot.stateCode + BWorldState.KEY_DELIMETER +  "name", "Name", new BString(theBot.getName()), new BString(theBot.getName()), new BString(theBot.getName()), "") );
    root.put(
              new BBotNumberWorldItem(BBot.stateCode+ dot+  "weight", "Weight", new BNumber(theBot.getTotalWeight()), new BNumber(0.0), new BNumber (100000), "kg", "My Bots weight.", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  setValue(getBot().getTotalWeight());
                  return getValue();
                }
              }
            );

    root.put(
              new BBotNumberWorldItem(BBot.stateCode+ dot+  "cost", "Cost", new BNumber(0.0), new BNumber(0.0), new BNumber (100000.0), "cred","The cost of my Bot", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  setValue(getBot().getTotalPrice());
                  return getValue();
                }
              }
            );



    root.put(
              new BBotNumberWorldItem(BBot.stateCode + dot+  "health", "Health", new BNumber(100.0), new BNumber(0.0), new BNumber (100.0), "%", "The current health of my Bot", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  setValue(100.0 - getBot().getPercentageDamageLevel() );
                  return getValue();
                }
              }
            );

    root.put(
              new BBotNumberWorldItem(BBot.stateCode + dot+ "energy", "Energy", new BNumber(100.0), new BNumber(0.0), new BNumber (100.0), "%", "The remaining energy of my Bot", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {

                  setValue(getBot().getPercentageEnergyLevel() );
                  return getValue();
                }
              }
            );


   root.put(
              new BBotNumberWorldItem(WSTATE_MYREACH, "Reach", new BNumber(0.0), new BNumber(0.0), new BNumber (100.0), "m","The distance between the center of my Bot and it's furthest component", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  setValue(getBot().getBotReach() );
                  return getValue();
                }
              }
            );

    root.put(
              new BBotNumberWorldItem(WSTATE_OPPANGLE, "Angle to enemy", new BNumber(0.0), new BNumber(0.0), new BNumber (360.0), "deg","From above the Bot, this is the clockwise angle between my Bots front and the enemy",  theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  //BDebug.print("Angle to enemy getBot().getArenaPosition()" + getBot().getArenaPosition() + " getBot().getLastDetectedOpponentPosition() " + getBot().getLastDetectedOpponentPosition());
                  setValue(BPhysics.angleFrom(getBot().getArenaPosition(), getBot().getLastDetectedOpponentPosition()  ));
                  return getValue();
                }
              }
            );

    root.put(
              new BBotNumberWorldItem(WSTATE_OPPDISTANCE, "Distance to enemy", new BNumber(0.0), new BNumber(0.0), new BNumber (10000.0), "m","The distance between my Bot and the enemy", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  setValue(BPhysics.distanceBetween(getBot().getArenaPosition(),getBot().getLastDetectedOpponentPosition()));
                  return getValue();
                }
              }
            );

    root.put(
              new BBotNumberWorldItem(BBot.stateCode + dot+  "angleToArenaCenter", "Angle to center of arena", new BNumber(0.0), new BNumber(0.0), new BNumber (360.0), "deg","From above the Bot, this is the clockwise angle between my Bots front and the center of the arena", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  //BDebug.print("angleToArenaCentergetBot().getArenaPosition()" + getBot().getArenaPosition());

                  setValue(BPhysics.angleFrom(getBot().getArenaPosition(), BPhysics.CENTREOBJECTPOSITION  )     );
                  return getValue();
                }
              }
            );

    root.put(
              new BBotNumberWorldItem(BBot.stateCode + dot+  "distanceToArenaCenter", "Distance to center of arena", new BNumber(0.0), new BNumber(0.0), new BNumber (10000.0), "m","The distance between my Bot and the center of the arena",  theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  setValue(BPhysics.distanceBetween( getBot().getArenaPosition() ,BPhysics.CENTREOBJECTPOSITION));
                  return getValue();
                }
              }
            );
    root.put(
              new BBotBooleanWorldItem(BBot.stateCode+ dot +"isBlind", "isBlind", new BBoolean(), "","This becomes true if my bot has no working sensors", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  setValue(getBot().isBlind() );

                  return getValue();
                }
              }
            );

    root.put(
              new BBotNumberWorldItem(BBot.stateCode + dot+ "distanceToArenaEdge", "Distance to edge of arena", new BNumber(0.0), new BNumber(0.0), new BNumber (10000.0), "m","The distance between my Bot and the edge of the arena", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  //BDebug.print(  "distanceToArenaEdge getBot().getArena()" + getBot().getArena());
                  //BDebug.print ("BBot.name" + getBot()) ;

                  setValue(( (double)getBot().getArena().getSize() / 2.0 ) - BPhysics.distanceBetween(getBot().getArenaPosition(),BPhysics.CENTREOBJECTPOSITION));
                  return getValue();
                }
              }
            );



    root.put(
              new BBotNumberWorldItem(BBot.stateCode + dot+ "timeSinceCollision", "Time since last collision", new BNumber(0.0), new BNumber(0.0), new BNumber (10000.0), "s","The time elapsed since my bot collided", theBot)

            //  new BBotWorldItem(BBot.stateCode + dot+ "lastCollidedTime", "Last collision time", new BNumber(), new BNumber(), new BNumber(), "", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {

                  //BNumber theValue = (BNumber)getValue();
                  setValue( getBot().getTimeSinceLastCollision() );

                  return getValue();
                }
              }
            );

   /*
    root.put(
              new BBotNumberWorldItem(BBot.stateCode + dot+ "lastCollisionTime", "Last collision time", new BNumber(0.0), new BNumber(0.0), new BNumber (10000.0), "s","The recorded last time my Bot collided", theBot)

            //  new BBotWorldItem(BBot.stateCode + dot+ "lastCollidedTime", "Last collision time", new BNumber(), new BNumber(), new BNumber(), "", theBot)
              {
                public BValue evaluate()
                {

                  //BNumber theValue = (BNumber)getValue();
                  setValue( getBot().getLastCollidedTime() );

                  return getValue();
                }
              }
            );   */

    root.put(
              new BSimNumberWorldItem( "timeElapsed", "Time elapsed", new BNumber(0.0), new BNumber(0.0), new BNumber (10000.0), "secs", "How much time has passed since the beginning of the fight", null)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  //BDebug.print(  "timeElapsed getSim()" + getSim());

                  setValue(getSim().getElapsedTime() );
                  return getValue();
                }
              }
            );

/////////////////////////// Added by Andrew ////////////////////////////////////
    root.put(
              new BSimNumberWorldItem( "timeSinceDeadlock", "Time since last deadlock", new BNumber(180.0), new BNumber(0.0), new BNumber (10000.0), "secs", "How much time has passed since the last deadlock", null)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  float time = getSim().getTimeSinceDeadlock();
                  //BDebug.print(  "BSimulation - time since deadlock = " + time);

                  setValue(time);
                  return getValue();
                }
              }
            );
    root.put(
              new BSimNumberWorldItem( "deadlockPercent", "Imminence of deadlock", new BNumber(180.0), new BNumber(0.0), new BNumber (100.0), "%", "How close a deadlock is to occurring, expressed as a percentage.", null)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  float percent = getSim().getDeadlockPercent();
                 // BDebug.print(  "BSimulation - deadlock percent = " + percent );
                  //BDebug.print(  "timeElapsed getSim()" + getSim());

                  setValue(percent);
                  return getValue();
                }
              }
            );
    root.put(
              new BBotNumberWorldItem(WSTATE_ENEMYBOT + dot + "velocity", "Speed (scalar velocity)", new BNumber(0.0), new BNumber(0.0), new BNumber (100.0), "m/s", "The speed of the enemy bot.", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  double velocity = (double)getBot().getOpponentVelocity();

                 // BDebug.print(  "BSimulation - enemy's absolute velocity = " + velocity);

                  if (getBot().getCurrentlyDetectingEnemy() )
                    setValue(velocity);
                  return getValue();
                }
              }
            );
    root.put(
              new BBotNumberWorldItem(WSTATE_ENEMYBOT + dot + "velocityAngleToMe", "Angle of velocity to me", new BNumber(0.0), new BNumber(0.0), new BNumber (360.0), "deg", "The angle of the enemy bot's velocity in relation to my bot.", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  double angle = getBot().getOpponentVelocityAngle();
                 // BDebug.print(  "BSimulation - enemy's absolute velocity angle to me = " + angle);

                  if (getBot().getCurrentlyDetectingEnemy() )
                    setValue(angle);
                  return getValue();
                }
              }
            );
    root.put(
              new BBotNumberWorldItem(BBot.stateCode + dot+ "velocity", "Speed (scalar velocity)", new BNumber(0.0), new BNumber(0.0), new BNumber (100.0), "m/s", "The speed of my bot.", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  double velocity = (double)getBot().getOpponentBot().getOpponentVelocity();

                //  BDebug.print(  "BSimulation - my absolute velocity = " + velocity);

                  setValue(velocity);
                  return getValue();
                }
              }
            );
    root.put(
              new BBotBooleanWorldItem(WSTATE_MEDETECTED, "Enemy's sensors have detected me", new BBoolean(), "","True if my bot is currently detected by one or more of the enemy's sensors.", theBot)
              {
                /**
				 * 
				 */
				private static final long serialVersionUID = -1L;

				public BValue evaluate()
                {
                  boolean detects =  getBot().getOpponentBot().getCurrentlyDetectingEnemy();
              //    BDebug.print(  "BSimulation - enemy has detected me = " + detects);
                  setValue( detects );

                  return getValue();
                }
              }
            );
    root.put(
              new BBotNumberWorldItem(WSTATE_ENEMYBOT + dot + "height", "Height", new BNumber(0.0), new BNumber(0.0), new BNumber (100.0), "m", "The current height of the enemy bot.", theBot)
              {
                public BValue evaluate()
                {
                  double height = getBot().getOpponentHeight();
                //  BDebug.print(  "BSimulation - enemy height = " + height);

                  if (getBot().getCurrentlyDetectingEnemy() || getValueDouble() == 0 )
                    setValue(height);
                  return getValue();
                }
              }
            );
    root.put(
              new BBotNumberWorldItem(WSTATE_ENEMYBOT + dot + "width", "Width", new BNumber(0.0), new BNumber(0.0), new BNumber (100.0), "m", "The maximum distance from the centre of the enemy to the left or right of the enemy ", theBot)
              {
                public BValue evaluate()
                {
                  double width = getBot().getOpponentWidth();
                //  BDebug.print(  "BSimulation - enemy width = " + width);

                  if (getBot().getCurrentlyDetectingEnemy() || getValueDouble() == 0 )
                    setValue(width);
                  return getValue();
                }
              }
            );
    root.put(
              new BBotNumberWorldItem(WSTATE_ENEMYBOT + dot + "depth", "Depth", new BNumber(0.0), new BNumber(0.0), new BNumber (100.0), "m", "The maximum distance from the centre of the enemy to the front or back of the enemy", theBot)
              {
                public BValue evaluate()
                {
                  double depth = getBot().getOpponentDepth();
                //  BDebug.print(  "BSimulation - enemy depth = " + depth);

                  if (getBot().getCurrentlyDetectingEnemy() || getValueDouble() == 0 )
                    setValue(depth);
                  return getValue();
                }
              }
            );


////////////////////////////////////////////////////////////////////////////////
    root.put(
              new BSimNumberWorldItem( "timeRemaining", "Time remaining", new BNumber(0.0), new BNumber(0.0), new BNumber (10000.0), "secs","How much time is remaining in this fight", null)
              {
                public BValue evaluate()
                {
                  setValue(getSim().getRemainingTime() );
                  return getValue();
                }
              }
            );

    root.put(
              new BBotBooleanWorldItem(WSTATE_OPPDETECTED, "Enemy detected by my sensors", new BBoolean(), "","True if the enemy is currently detected by 1 or more of my Bots sensors", theBot)
              {
                public BValue evaluate()
                {
                  boolean detects =  BCollision.detects(getBot(), getBot().getOpponentBot());
                  setValue( detects );
                  getBot().setCurrentlyDetectingEnemy(detects);

                  return getValue();
                }
              }
            );

    root.put(
              new BBotNumberWorldItem(WSTATE_ENEMYBOT + dot +"weight", "Weight", new BNumber(theBot.getTotalWeight()), new BNumber(0.0), new BNumber (100000), "kg","The current weight of the enemy Bot", theBot)
              {
                public BValue evaluate()
                {
                  setValue(getBot().getOpponentBot().getTotalWeight());

                  return getValue();
                }
              }
            );

    root.put(
              new BBotNumberWorldItem(WSTATE_ENEMYBOT + dot +"cost", "Cost", new BNumber(0.0), new BNumber(0.0), new BNumber (100000.0), "cred","The total cost of the enemy Bot", theBot)
              {
                public BValue evaluate()
                {
                  setValue(getBot().getOpponentBot().getTotalPrice());
                  return getValue();
                }
              }
            );



    root.put(
              new BBotNumberWorldItem(WSTATE_ENEMYBOT + dot +"health", "Health", new BNumber(100.0), new BNumber(0.0), new BNumber (100.0), "%","The current health of the enemy Bot", theBot)
              {
                public BValue evaluate()
                {
                  setValue(100.0 - getBot().getOpponentBot().getPercentageDamageLevel() );
                  return getValue();
                }
              }
            );

    root.put(
              new BBotNumberWorldItem(WSTATE_ENEMYBOT + dot +"energy", "Energy", new BNumber(100.0), new BNumber(0.0), new BNumber (100.0), "%","The remaining energy of the enemy Bot", theBot)
              {
                public BValue evaluate()
                {
                  if (getBot().getCurrentlyDetectingEnemy() )
                    setValue( getBot().getOpponentBot().getPercentageEnergyLevel() );

                  return getValue();
                }
              }
            );



    root.put(
              new BBotNumberWorldItem(WSTATE_OPPREACH , "Reach", new BNumber(0.0), new BNumber(0.0), new BNumber (100.0), "m","The distance between the center of the enemy and it's furthest component", theBot)
              {
                public BValue evaluate()
                {
                  if (getBot().getCurrentlyDetectingEnemy() )
                    setValue(getBot().getOpponentBot().getBotReach() );

                  return getValue();
                }
              }
            );


    root.put(
              new BBotNumberWorldItem(WSTATE_OPPFACING, "Angle to me", new BNumber(0.0), new BNumber(0.0), new BNumber (360.0), "deg","From above the enemy, this is the clockwise angle between the enemys front and my Bot", theBot)
              {
                public BValue evaluate()
                {
                  setValue(BPhysics.angleFrom(getBot().getLastDetectedOpponentPosition(), getBot().getArenaPosition()   ));

                  return getValue();
                }
              }
            );


    root.put(
              new BBotNumberWorldItem(WSTATE_ENEMYBOT + dot +"velocityTowardsMe", "Velocity towards me", new BNumber(0.0), new BNumber(-100.0), new BNumber (100.0), "m/s","Relative velocity between my Bot and the enemy, + values indicate the enemy is moving closer to my Bot", theBot)
              {
                public BValue evaluate()
                {
                  // need to call this every time slice
                  double velocity = getBot().getOpponentRelativeVelocity();

                  if (getBot().getCurrentlyDetectingEnemy() )
                    setValue(velocity);
                  //else
                  //  setUndefined();


                  return getValue();
                }
              }
            );

    root.put(
              new BBotBooleanWorldItem(WSTATE_ENEMYBOT + dot +"isBlind", "isBlind", new BBoolean(), "","This becomes true if the enemy bot has no working sensors", theBot)
              {
                public BValue evaluate()
                {
                  setValue(getBot().getOpponentBot().isBlind() );

                  return getValue();
                }
              }
            );


    root.put(
              new BBotBooleanWorldItem(WSTATE_ENEMYBOT + dot +"chassis" + dot + "isHover", "Hovers", new BBoolean(), "", "True if the enemy bot uses hover technology for movement", theBot)
              {
                public BValue evaluate()
                {
                  setValue(getBot().getOpponentBot().getChassis().isHover());

                  return getValue();
                }
              }
            );
    root.put(
              new BBotBooleanWorldItem(WSTATE_ENEMYBOT + dot + "chassis" + dot +"isWheeled", "Is wheeled", new BBoolean(), "","True if the enemy bot is wheeled", theBot)
              {
                public BValue evaluate()
                {
                  setValue(getBot().getOpponentBot().getChassis().isWheeled());

                  return getValue();
                }
              }
            );
     root.put(
              new BBotBooleanWorldItem(WSTATE_ENEMYBOT + dot + "chassis" + dot +"isLegged", "Is legged", new BBoolean(), "","True if the enemy bot uses legs for movement", theBot)
              {
                public BValue evaluate()
                {
                  setValue(!getBot().getOpponentBot().getChassis().isWheeled()  && !getBot().getOpponentBot().getChassis().isHover() );

                  return getValue();
                }
              }
            );



    return root;
 }





}///:~
