package roboforge;

import java.util.*;
import java.io.*;
//import javax.media.j3d.*;
//import java.applet.AudioClip;  // For Audio Clip playing
//import java.net.*;
//import javax.swing.table.*;
//import javax.swing.*;
//import javax.swing.tree.*;

import roboforge.ai.values.*;
import roboforge.ai.worldstate.*;
import roboforge.ai.*;
import roboforge.util.io.*;

import roboforge.ai.memory.*;
//import roboforge.math.*;

public class BStats implements Serializable
{

  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
//Vector staticChannels;
  private transient BWorldItem[] worldItems;
  private transient BValue[] lastValues;

  private transient Vector worldValues;
  private transient Vector actionNames;
  private transient Vector comments;
  private transient Vector chassisMoves;
  private transient Vector memoryItems;
  private transient Vector visitedNodes;

  private transient BMemoryBank memoryBank;

  private transient Vector headings;
  private transient Vector memHeadings;

  // random read variables
  private String ownerID;
  private transient long[] worldSlicePointers;
  private transient String fileName;
  private transient BValue[] values;
  private int noOfSlices;
  private transient Vector cacheActionNames;
  private transient Vector cacheComments;
  private transient String cacheChassisMove;
  private transient Vector cacheMemory;
  private transient Vector cacheVisitedNodes;

  private transient BDecisionTreeAI ai;

  public BStats()
  {
  }

  // public BStats (BWorldState state, BMemoryBank mBank )
  public BStats (BWorldState state, BBot owner)
  {
    //BDebug.print ("setting stats for " + owner + " ID " + owner.getBotSerialID() );
    ownerID = owner.getBotSerialID();
    memoryBank = ((BDecisionTreeAI)owner.getAI()).getMemoryBank();
    fileName = null;

    Vector processVector = new Vector();
    for (Enumeration e = state.getWorldItems() ; e.hasMoreElements() ;)
      processVector.add(e.nextElement() ) ;

    // create preocessing worldItems cache
    worldItems = new BWorldItem[processVector.size()];
    lastValues = new BValue[worldItems.length];

    // place world items in backwards to get alphabetical order
    for (int i= 0; i < processVector.size(); i++)
      worldItems[( worldItems.length - 1) - i] = (BWorldItem) processVector.get(i);

      // create Headings Vector
    headings = new Vector();

    for (int i = 0; i < worldItems.length; i++)
      headings.add(   worldItems[i].getKey()  );

    ai = (BDecisionTreeAI) owner.getAI();
    //visitedNodes.add(ai.getAndCleanVisitedNodes() );


    worldValues = new Vector();
    actionNames = new Vector();
    comments = new Vector();
    chassisMoves = new Vector();
    memoryItems = new Vector();
    visitedNodes = new Vector();
  }

  private static class WorldItemDataPair
  {
    public BWorldItem item;
    public BValue value;

    public WorldItemDataPair(BWorldItem item, BValue value)
    {
      this.item = item;
      this.value = value;
    }

    public boolean equals(Object o)
    {
      if (o == item)
        return true;
      else if (o instanceof  WorldItemDataPair )
        return ((WorldItemDataPair)o).item == this.item;


      return false;
    }

    public String toString()
    { return item.toString() + " " + value.toString(); }

  }

  public void addSlice(BBot theBot)
  {

    Vector aChannelSlice = new Vector();

    for (int i = 0; i < worldItems.length ;i++)
    {
      if ( !worldItems[i].getValue().equals( lastValues[i] ) )
      {
        lastValues[i] = (BValue) worldItems[i].getValue().clone();
        aChannelSlice.add( new WorldItemDataPair( worldItems[i], lastValues[i]  ) );

      }

      //aChannelSlice.add( worldItems[i].getValue().clone() );


    }
    worldValues.add(aChannelSlice);

    actionNames.add( theBot.getCurrentActionNames() );

    comments.add(theBot.getArenaComments().clone() );
    theBot.getArenaComments().clear();

    chassisMoves.add(theBot.getCurrentChassisMoveName() );

    Vector memValues = new Vector();
    for (int i = 0; i < memoryBank.getSize() ;i++)
      memValues.add( memoryBank.getItem(i).getValue().clone() );

    memoryItems.add(memValues);
    visitedNodes.add(ai.getAndCleanVisitedNodes() );


  }

  private void copyTo(String targetFile)
  {

   try
   {
     BFileSystem.copyFile(this.fileName, targetFile);
   }
   catch (Exception e) { e.printStackTrace();  }



  }


  public String saveToTempFile() throws IOException
  // returns the fileName
  {
   // BDebug.print ("XXXXXXX TEMP TESTING CODE IN BSTATS REMOVE REMOVE REMOVE ");
   // BDebug.print ("XXXXXXX TEMP TESTING CODE IN BSTATS REMOVE REMOVE REMOVE ");

    //File tempFile = File.createTempFile("tempStats", "tmp");
    //String fileName = tempFile.getParent() + "/tempStats.tmp";
    String name;
    if (BGlobal.IS_FARMCLIENT)
      name = "c:/temp/tempStats.sta";
    else
      name = BGlobal.USERPATH + "tempStats.sta";

    saveToFile(name);
    return name;

  }


  public byte[] toByteArray() throws IOException
  {

    // a null stats file
    if (headings == null)
    {

      byte[] stub = new byte[1];
      return stub;
    }

    String tempFileName =  saveToTempFile();

    RandomAccessFile saveFile = new RandomAccessFile ( tempFileName, "r" );

    if (saveFile.length()  > Integer.MAX_VALUE )
      BDebug.print("ERROR ERROR BStats.toByteArray File Largeer Than Byte Array ");

    byte[] data = new byte[(int)saveFile.length()];

    saveFile.readFully(data) ;
    saveFile.close();

    return (data);

  }

  private static  int reverseIndexOf(Vector v, Object o)
  {
    for (int i = v.size() - 1; i >=0; i--)
      if (v.get(i).equals(o) )
        return i;

    return -1;


  }
   public void saveToFile (String filename)
  {

    // delete the old one
    File tester = new File(filename);
    if (tester.exists() )
          tester.delete();


    if (this.fileName != null)
    {
      copyTo(filename);
      //BDebug.print ("BStats copyingFile to filename " + filename);
    }
    else
    {
     try
     {

        // open a new one
        RandomAccessFile saveFile = new RandomAccessFile ( filename, "rw" );

        int sliceSize = ((Vector)worldValues.get(0)).size() ;

       // BDebug.print ("BStats.saveToFile " + filename + " stats file started botID " + ownerID );

        saveFile.writeUTF(   ownerID);
        saveFile.writeInt(   sliceSize );
        saveFile.writeInt(   worldValues.size() );

        Vector firstSlice = (Vector) worldValues.get(0);

        for (int i = 0; i<  sliceSize; i++ )
        {
          saveFile.writeUTF((String)headings.get(i));
         ((WorldItemDataPair)firstSlice.get(i)).value.writeDataType(saveFile);

        }

        // memory header
        saveFile.writeInt(memoryBank.size() );
        for (int i = 0; i < memoryBank.size(); i++ )
        {
        //  BDebug.print ("BStats writting heading for memory item ::" + memoryBank.getItem(i).getName()  );
          saveFile.writeUTF(memoryBank.getItem(i).getName() );
        }

        // pre pass writing the double pointers in
        long[] slicePointers = new long[worldValues.size()];
        long slicePointerHeader = saveFile.getFilePointer();

       // BDebug.print ("BStats.saveToFile sliceSize() " + slicePointers.length );
        for (int i = 0; i < slicePointers.length; i++)
          saveFile.writeLong(slicePointers[0]);


        lastValues = new BValue[worldItems.length];


        //-------------------------------------
        // WRITE AI

        ai.setBot(null);
        ByteArrayOutputStream aiFile = new ByteArrayOutputStream();
        ObjectOutputStream objOut = new ObjectOutputStream(aiFile);
        objOut.writeObject(ai);

        objOut.flush();
        objOut.close();

        byte[] aiBytes = aiFile.toByteArray();

        saveFile.writeInt(aiBytes.length );
        saveFile.write(aiBytes);

        aiFile.flush();
        aiFile.close();



        //----------------------------------------

        // write the values out
        for (int i = 0; i < worldValues.size(); i++)
        {
          slicePointers[i] = saveFile.getFilePointer();
         // BDebug.print ("slicePointers[i] " + i + " " +slicePointers[i] );
          Vector theSlice = (Vector) worldValues.get(i);

         // BDebug.print ("BStats.saveToFile  worldItems.length " + worldItems.length);

          for (int innerLoop = 0; innerLoop < worldItems.length; innerLoop ++ )
          {
          //  BDebug.print ("BStats.saveToFile  worldItems[innerLoop] " + worldItems[innerLoop]);

            int indexof = reverseIndexOf(theSlice,worldItems[innerLoop]);

           // BDebug.print ("BStats.saveToFile  theSlice " + theSlice);
           // BDebug.print ("BStats.saveToFile  indexof " + indexof);

            if ( indexof != -1 )
            {
            //  BDebug.print ("BStats.saveToFile saving theSlice.get(indexof) " + theSlice.get(indexof) );
              lastValues[innerLoop] = ((WorldItemDataPair)theSlice.get(indexof)).value;
              lastValues[innerLoop].writeValue(saveFile);
            }
            else
              lastValues[innerLoop].writeValue(saveFile);
          }

          /*
          for (int innerLoop = 0; innerLoop < theSlice.size(); innerLoop ++ )
          {
            ((BValue)theSlice.get(innerLoop)).writeValue(saveFile);
           // BDebug.print ("writing" + theSlice.get(innerLoop) );
          }  */

          // action Names
          Vector actionSlice = (Vector) actionNames.get(i);
          saveFile.writeInt( actionSlice.size()     );
          for (int innerLoop = 0; innerLoop < actionSlice.size(); innerLoop++ )
            saveFile.writeUTF(  (String)actionSlice.get(innerLoop));

          // arena Comments
          Vector commentSlice = (Vector) comments.get(i);
          saveFile.writeInt( commentSlice.size()     );
          for (int innerLoop = 0; innerLoop < commentSlice.size(); innerLoop++ )
            saveFile.writeUTF(  (String)commentSlice.get(innerLoop));

          // chassisMoveNames
          saveFile.writeUTF(  (String) chassisMoves.get(i) );

          // memoryItems
          Vector memSlice =  (Vector) memoryItems.get(i);
          for (int innerLoop = 0; innerLoop < memSlice.size(); innerLoop ++ )
          {

            BValue value = (BValue) memSlice.get(innerLoop);
          //  BDebug.print ("BStats.saveToFile saving memoryItem " + value.toString() );
            value.writeDataType(saveFile);
            value.writeValue(saveFile);

          }

          Vector visitSlice =  (Vector) visitedNodes.get(i);
          saveFile.writeInt( visitSlice.size() );
          for (int innerLoop = 0; innerLoop < visitSlice.size(); innerLoop ++ )
          {

            Integer value = (Integer) visitSlice.get(innerLoop);
            saveFile.writeInt(value.intValue());

          }




        }

        //writer the header with the new data
        saveFile.seek(slicePointerHeader);
        for (int i = 0; i < slicePointers.length; i++)
        {
          saveFile.writeLong(slicePointers[i]);
         // BDebug.print ("saving slicePointers[i] "+ i + " " + slicePointers[i] );
        }
        saveFile.close();

       // BDebug.print ("BStats.saveToFile " + filename + " complete with no errors");

    }
    catch (Exception e1)
    {
        BDebug.print("BStats.saveToFile  ERROR " + e1.getMessage());
    }

   }

  }

  public Vector getMemoryHeadings()
    { return memHeadings; }

  public Vector getCacheMemValues()
    { return cacheMemory; }

  public Vector getCacheVisitedValues()
    { return cacheVisitedNodes;}

  public Vector getHeadings()
  { return headings;}

  public int getSliceCount()
  { return noOfSlices;}

  public Vector getCacheActionNames()
  { return cacheActionNames ; }

  public Vector getCacheComments()
  { return cacheComments ; }

  public String getCacheChassisMove()
  { return cacheChassisMove; }

  public Vector getWorldAndFillCache(int i)
  {
    try
     {
     // BDebug.print ("XXXXX BStats.getWorldAndFillCache for " + i);

      RandomAccessFile loadFile = new RandomAccessFile ( fileName, "r" );

     // BDebug.print ("BStats.getWorldAndFillCache filePointer " + worldSlicePointers[i]);
      loadFile.seek(worldSlicePointers[i]);

      Vector tempVec = new Vector();

      for (int innerLoop = 0; innerLoop < values.length ; innerLoop++ )
      {
           // BDebug.print ("reading " + values[innerLoop]);
            values[innerLoop].readValue( loadFile );
            if ( values[innerLoop].isNumber() )
              tempVec.add(   BPhysics.roundToString(   ((BNumber)values[innerLoop]).getValue(),2   )      );
            else
              tempVec.add(values[innerLoop].toString() );

      }


       // read ActionNames
       if ( cacheActionNames == null)
        cacheActionNames = new Vector();
       else
        cacheActionNames.clear();

       int noOfActions = loadFile.readInt();
      // BDebug.print ("BStats.getWorldAndFillCache noOfActions "  + noOfActions );

       for (int innerLoop = 0; innerLoop < noOfActions; innerLoop++ )
        cacheActionNames.add(loadFile.readUTF());

       // read comments into cache
       if ( cacheComments == null)
        cacheComments = new Vector();
       else
        cacheComments.clear();

       int noOfComments= loadFile.readInt();
       for (int innerLoop = 0; innerLoop < noOfComments; innerLoop++ )
        cacheComments.add(loadFile.readUTF());

       cacheChassisMove = loadFile.readUTF();

      // read memory
      if (cacheMemory == null)
        cacheMemory = new Vector();
      else
        cacheMemory.clear();

     // BDebug.print ("memHeadings.size() " + memHeadings.size() );

      for (int memLoop = 0; memLoop < memHeadings.size(); memLoop++ )
      {
      //  BDebug.print ("BStats.getWorldAndFillCache received mem item " + memHeadings.get(memLoop)  );

        BValue m = BValue.createValue(loadFile.readInt());
        m.readValue(loadFile);

        if ( m.isNumber() )
          tempVec.add(   BPhysics.roundToString(   ((BNumber)m).getValue(),2   )      );
        else
          tempVec.add( m.toString() );
         

      }

      // visited Nodes

      if (cacheVisitedNodes == null)
        cacheVisitedNodes = new Vector();
      else
        cacheVisitedNodes.clear();

      int visits = loadFile.readInt();
      for (int visiti = 0; visiti < visits; visiti++ )
      {
        int v = loadFile.readInt();
        cacheVisitedNodes.add( new Integer(v));


      }   

      //BDebug.print ("BStats.getWorldAndFillCache cacheVisitedNodes ::" + cacheVisitedNodes);
      loadFile.close();

      return tempVec;

     }
     catch (Exception e1)
    {
        BDebug.print("BStats.get  ERROR " + e1.getMessage());
    }

    return null ;


  }

  public String getOwnerID()
    { return ownerID; }




  public static BStats loadFromFile(String filename)
  {
    File tester = new File(filename);
    if (!tester.exists())
    {

      filename = filename.substring(0, filename.length() - 4) + ".stb";
   //   BDebug.print ("BStats.trying stb :: " + filename);
    }

    BStats newStats = new BStats();

    try
     {


        RandomAccessFile loadFile = new RandomAccessFile ( filename, "r" );

        newStats.ownerID = loadFile.readUTF();
        //BDebug.print ("BStats.loadFromFile " + filename + " stats file started ownerID " + newStats.ownerID );

        int sliceSize = loadFile.readInt();
        //BDebug.print("sliceSize " + sliceSize);

        newStats.noOfSlices = loadFile.readInt();

        newStats.headings = new Vector(sliceSize);
        newStats.values = new BValue[sliceSize];

        for (int i = 0; i<  sliceSize; i++ )
        {
          newStats.headings.add( new String( loadFile.readUTF() ) );
          newStats.values[i] = BValue.createValue(loadFile.readInt() );
          //BDebug.print( " created " + newStats.values[i] );

        }

        int memSize = loadFile.readInt();
        newStats.memHeadings = new Vector();
        for (int i= 0; i < memSize; i++)
        {
          String memTitle =  loadFile.readUTF();
         // BDebug.print ("BStats reading in heading for " + memTitle);
          newStats.headings.add("me.memory." + memTitle);
          newStats.memHeadings.add(memTitle);
        }
        //BDebug.print( "BStats reading in pointers"  );

        // pre pass reading the double Pointers
        newStats.worldSlicePointers = new long[newStats.noOfSlices];

        for (int i = 0; i < newStats.noOfSlices; i++)
          newStats.worldSlicePointers[i] = loadFile.readLong();

       // long aiPointer = loadFile.getFilePointer();
        //loadFile.close();
        //BDebug.print( " worldValues read "  );
        // read in


         //-------------------------------------
        // WRITE AI

        //try
        //{
          //BDebug.print ("BStats.loadFromFile newStats.worldSlicePointers[i]" + newStats.worldSlicePointers[0] );
         //BDebug.print ("BStats.loadFromFile getFilePointer" + loadFile.getFilePointer() );

          int aiBytesLength = loadFile.readInt();

          byte[] aiBytes =  new byte[aiBytesLength];
          loadFile.read(aiBytes);

          ByteArrayInputStream aiFile = new ByteArrayInputStream(aiBytes);
          ObjectInputStream objIn = new ObjectInputStream(aiFile);

          newStats.ai = (BDecisionTreeAI) objIn.readObject();

          aiFile.close();
          objIn.close();

          
       /* }
         catch (Exception e)
         {
            BDebug.print ("BStats.loadFromFile old stats format detected complete with no errors");
         }     */

        loadFile.close();

        //----------------------------------------

        newStats.fileName = filename;
        newStats.cacheChassisMove = "";


       // BDebug.print ("BStats.loadFromFile " + filename + " complete with no errors");

        return newStats;

    }
    catch (Exception e1)
    {
        //BDebug.print("BStats.loadFromFile  ERROR " + e1.getMessage());
    }

    return null;


  }

  public BDecisionTreeAI getAI()
  {
    return ai;
  }

  public BStats convertToRandomFile()
  {
    try
      {
        //File tempFile = File.createTempFile("tempStats", "tmp");
        //String name = BGlobal.USERPATH + "tempStats.tmp";
        String name;
        if (BGlobal.IS_FARMCLIENT)
          name = "c:/temp/tempStats.sta";
        else
          name = BGlobal.USERPATH + "tempStats.sta";

        saveToFile(name);

        return BStats.loadFromFile(name);
      }
      catch (Exception e)
        { BDebug.print ("BStats.convertToRandomFile error creating temp Stats file " + e.getMessage() );}

      return null;
  }




}