package roboforge;

//: BTextures.java

import java.util.*;
//import javax.swing.*;
import com.sun.j3d.utils.image.*;
import javax.media.j3d.*;
import java.awt.Component;

//import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;

import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;

import java.io.File;

import roboforge.util.text.BStrings;


public class BTextures
{

/*

* Description: Collection of Textures and related filepaths. For performance and minimising memory usage.

* Example:
* Modification History ( Name, Date, Notes ):
                        DJG 08-Apr-99
                        YLY           , Modified to be a Completely static class
*/

  private static Hashtable textures = new Hashtable();
  private static Component  listener = null;


  public static Texture getTexture(String fileName)
  {
    return getTexture(fileName, false);
  }

  public static Texture getTexture(String fileName, boolean mipmap)
  {
    //BDebug.print ("BTextures.getTexture called ");
    fileName = BStrings.convertToFilePath(fileName);

    if (textures.containsKey(fileName))
      { return (Texture) textures.get(fileName); }

    try
    {

     if (listener == null)
        BDebug.print("Btextures.getTexture() LISTENER NOT INITIALISED ");
     else
     {

        Texture t;

       // BDebug.print ("BTextures.getTexture getting texture " + fileName);


        //if (BGlobal.USE_MIPMAP && mipmap)
        if (BGlobal.USE_MIPMAP || mipmap)
          t = getCompoundMipMapped(fileName);
        else
          t =  getCompoundTexture(fileName);

        textures.put(fileName, t);
        return t;
      }
     }
     catch (Exception e)
      { BDebug.print("BTextures.getTexture() ERROR "+ e+ " in file: "+ fileName); }

     return null;

  }

  public static void setListener(Component newL)
  {
    if (listener == null)
      listener = newL;
  }

  public static void clear()
    { textures.clear(); }



    public static java.awt.image.BufferedImage getImage(String fileName)
    // Dont use this unless you will be showing the image in the 3d world.
  {

    //java.awt.image.BufferedImage

    try
    {


    if (textures.containsKey(fileName) )
    {
        return ((ImageComponent2D)  ((Texture)textures.get(fileName)).getImage(0)).getImage() ;
    }


     if (listener == null)
        BDebug.print("Btextures.getImage LISTENER NOT INITIALISED ");
     else
     {
        // TextureLoader texLoad = new TextureLoader( fileName,listener);
        // Texture thisTexture = texLoad.getTexture();

        Texture thisTexture = getCompoundTexture(fileName);
        textures.put(fileName, thisTexture);

        return ((ImageComponent2D)  thisTexture.getImage(0)).getImage();
      }

     }catch (Exception e) { BDebug.print("Btextures.getImage ERROR "  + e.getMessage());}


     return null;
  }


  public static Texture getTexture(BufferedImage source)
  {

   try {
    TextureLoader loader = new TextureLoader(source);
    return loader.getTexture();
   } catch (Exception ex) {
    BDebug.print("BTextures.getTexture Exception: "+ex);
   }

   return null;
  }

  public static Texture getCompoundTexture(String fileName)
  {
    BufferedImage image = getCompoundImage(fileName);

    //BDebug.print ( fileName + " type " + image.getType() );
    //BDebug.print ( " Buffered Image custom " + BufferedImage.TYPE_CUSTOM );
    try {
      TextureLoader loader = new TextureLoader(image );

      return loader.getTexture();
    } catch (Exception ex) {
      BDebug.print("BTextures.getCompoundTexture Exception: "+ex);
    }

    return null;
  }


  public static Texture getCompoundMipMapped(String fileName)
  {
    //BDebug.print("BTextures.getMipMappedImage " + fileName);
    BufferedImage image = getCompoundImage(fileName);

    return getMipMapped(image);
  }


  public static List getTextureSet(String root, int digits, String ext)
  {
    // Loads all textures with names beginning with 'root' and
    // ending in a numeric ID then extension 'ext'.
    // e.g. ui/images/pic001.jpg, ui/images/pic002.jpg...

    String filename, num, padded;
    File   file;
    int current     = 0;
    boolean isDone  = false;

    Texture t;
    ArrayList ts = new ArrayList();

    while (!isDone)
    {
      num      = Integer.toString(current++);
      padded   = BStrings.padLeftWithChar(num, '0', digits);
      filename = root + padded + ext;

      // Determine if texture file exists...
      file = new File(filename);

      if (file.exists())
        t = getTexture(filename);
      else
        t = null;

      isDone  = (t == null);
      if (!isDone) ts.add(t);
    }

    ts.trimToSize();
    return ts;
  }


  public static BufferedImage getCompoundImage(String fileName)
  {

   try {
    BufferedImage image = getLoadedImage(fileName);
    BufferedImage alpha = getAlphaImage(fileName);

    if (alpha != null)
      setAlphaChannel(image, alpha);



    if (image != null)
      if (image.getWidth() > BGlobal.MAX_TEXTURE_SIZE ||  image.getHeight() > BGlobal.MAX_TEXTURE_SIZE )
      {
        int widthDiff = image.getWidth() /  BGlobal.MAX_TEXTURE_SIZE;
        int heightDiff = image.getHeight() /  BGlobal.MAX_TEXTURE_SIZE;
        int scaleDiff = java.lang.Math.max  (widthDiff, heightDiff);

        int width =  image.getWidth() /  scaleDiff;
        int height =  image.getHeight() /  scaleDiff;


        TextureLoader loader =
        new TextureLoader(image );

        BDebug.print ("BTextures reducing image " + fileName + " to " + width + "x" +  height );
        image = loader.getScaledImage(width, height).getImage();


      }



    return image;
   } catch (Exception ex) {
    BDebug.print("BTextures.getCompoundImage Exception: "+ex);
   }

   return null;
  }

  protected static void setAlphaChannel
  (
    BufferedImage image,
    BufferedImage alpha
  )
  {
    WritableRaster dst = image.getAlphaRaster();
    WritableRaster src = alpha.getRaster();

    int w = image.getWidth();
    int h = image.getHeight();

    BDebug.assert(
      (w == alpha.getWidth()) && (h == alpha.getHeight()),
      "BTextures.setAlphaChannel(): alpha & texture image size mismatch.");

    int[] a = src.getSamples(0, 0, w, h, 0, (int[]) null);
    dst.setSamples(0, 0, w, h, 0, a);
  }

  protected static BufferedImage getAlphaImage(String fileName)
  {
    return getLoadedImage(getFileNameNoExt(fileName)+ ".alp");
  }


  protected final static Toolkit toolkit = Toolkit.getDefaultToolkit();
  protected final static MediaTracker tracker = new MediaTracker(null);



  public static BufferedImage getLoadedImage(String fileName)
  {
   try {
    // Check if file actually exists...
    File f = new File(fileName);
    if (!f.exists())
      return null;

    TextureLoader loader = new TextureLoader(fileName, listener);

    return loader.getImage().getImage();
   } catch (Exception ex) {
    BDebug.print("BTextures.getLoadedImage Exception: "+ex);
   }

   return null;

  }


/*
  public static BufferedImage getLoadedImage(String fileName)
  {
    // Check if file actually exists...
    File f = new File(fileName);
    if (!f.exists())
      return null;

    TextureLoader loader = new TextureLoader(fileName, listener);

    return loader.getImage().getImage();

  }*/

  /*
  public static Image getAWTImage(String fileName)
  {
    // Begin loading...
    Image image = toolkit.createImage(fileName);

    // Wait for loading to complete...
    synchronized(tracker)
    {
	    tracker.addImage(image, 0);

      try
        { tracker.waitForID(0, 5000); }
      catch (InterruptedException e)
        {	BDebug.print("BTextures.getLoadedImage(): image load interrupted."); }

      // loadStatus = tracker.statusID(0, false);
	    tracker.removeImage(image, 0);
  	}

    // Return the loaded image.
    return image;
  }
  */

  protected static String getFileNameNoExt(String f)
  {
    String result;

    int i = f.lastIndexOf(BGlobal.IMAGESUFFIX);

    if (i == -1)
      i = f.lastIndexOf(BGlobal.JPEGSUFFIX);

    if (i > 0)
      result = f.substring(0, i);
    else
      result = f;

    return result;
  }




  protected static Texture getMipMapped(BufferedImage source)
  {

   try {
    TextureLoader loader =
      new TextureLoader(source );

    ImageComponent2D image =
      loader.getImage();


    int w = image.getWidth();
    int h = image.getHeight();

    Texture2D result =
      new Texture2D(Texture.MULTI_LEVEL_MIPMAP, Texture.RGBA, w, h);

    int level = 0;
    result.setImage(0, image);

    while ((w > 1) || (h > 1))
    {
      level++;

      // Halve image size.
      if (w > 1) w /= 2;
      if (h > 1) h /= 2;

      image = loader.getScaledImage(w, h);
      result.setImage(level, image);
    }

    result.setMagFilter(Texture.NICEST); // BASE_LEVEL_POINT);
    result.setMinFilter(Texture.NICEST); // MULTI_LEVEL_POINT);

    return result;
   } catch (Exception ex) {
    BDebug.print("BTextures.getLoadedImage Exception: "+ex);
   }

   return null;
  }



}

///:~
