package roboforge;

import java.io.*;
import java.util.*;
//import javax.swing.tree.*;
//import javax.swing.event.*;

//import javax.vecmath.*;
//import javax.media.j3d.*;
//import javax.swing.text.*;

import roboforge.ai.*;

////////////////////////////////////////////////////////////////////////////////
//
//  BWizardBlockOptions is a class to store the AI block options
//  associated with a particular wizard bot.
//
//
public class BWizardBlockOptions implements Serializable
{

  static final long serialVersionUID = -1L;

  private Vector blockOptions;
  private String name;
  private String description;

  private transient BWizardTemplateBot bot;
  private transient BAIBlock selectedBlock;
  //private boolean allowNone = false;

  public BWizardBlockOptions(Vector blockOptions)
  {
    this.blockOptions = blockOptions;
    name =   blockOptions.size() + " blockOptions ";
    description = "";
  }

  // adds a block the the vector of available blocks
  public void addBlock (BAIBlock b)
  { blockOptions.add(b); }

  // removes a block from the vector of available blocks
  public void removeBlock (BAIBlock b)
  { blockOptions.remove(b); }


  // add the currently selected ai block to the bot
  public void addSelectedBlockToBot()
  {
    if (selectedBlock != null)
    {
      BDecisionTreeAI ai = (BDecisionTreeAI) bot.getAI();
      ai.getRoot().add(selectedBlock.getRoot());

      selectedBlock.setBot(bot);
    }
  }

  public void setParentBot(BWizardTemplateBot bot)
    { this.bot = bot; }

  public BWizardTemplateBot getParentBot()
    { return this.bot; }


  // get all the blocks in this options list
  public Vector getBlocks()
    { return blockOptions; }

  // get a block at a specific location
  public BAIBlock getBlock(int i)
    { return (BAIBlock) blockOptions.get(i); }

  // the number of blocks in the vector
  public int size()
    { return blockOptions.size(); }

  // sets the selected block by index
  public void setSelected(int i)
  { selectedBlock = (BAIBlock) blockOptions.get(i); }

  // sets the selected block
  public void setSelected(BAIBlock s)
  {
    if (blockOptions.indexOf(s) == -1)
      selectedBlock = null;
    else
      selectedBlock = s;
  }

  // returns the selected block
  public BAIBlock getSelectedBlock()
  { return selectedBlock; }

  // sets the name
  public void setName (String name)
  { this.name = name; }

  // returns the name
  public String getName()
  { return name; }

  // sets the description
  public void setDescription(String description)
  { this.description = description; }

  // gets the description
  public String getDescription()
  { return description; }

  public String toString()
  { return name; }

}
