package roboforge;

import javax.vecmath.*;
//import java.util.*;
import javax.media.j3d.*;
//import com.sun.j3d.utils.geometry.ColorCube;
//import com.sun.j3d.utils.image.*;
//import javax.swing.*;
import java.io.*;
import com.sun.j3d.utils.geometry.Text2D;

//import roboforge.geometry.*;
import java.awt.Font;
//import java.awt.Dimension;



public class BZone
  implements Serializable, BNameable
{

  static final long serialVersionUID = -1L;

  private Vector3f position;  // position in 3d space relative to the bot  (Canvas co-ordinates )

  private double scale;  // the size of the box
  private String name;
  private int zoneNumber;

  private transient boolean detectsController = false;
  private transient boolean detectsEnergy = false;
  private transient boolean detectsSensor = false;
  private transient boolean detectsShield = false;
  private transient boolean detectsWeapon = false;
  private transient boolean detectsJoint  = false;
  private transient boolean detectsChassis  = false;


  private transient BZones parentList; // set up in init
  private transient TransformGroup zoneTransGroup;
  private transient TransformGroup zoneScaleGroup;
  private transient TransformGroup textTransGroup;

  private transient Transform3D zoneTrans;
  private transient Transform3D zoneScale;
  private transient Transform3D textTrans;

  private transient Group       attachGroup;
  private transient BranchGroup detachGroup;
  //private transient OrientedShape3D zoneName;
  private transient Shape3D zoneName;

  private transient Appearance zoneAppearance;
  private transient Appearance textAppearance;
  private transient BZoneShape theShape;

  private transient boolean detected = false;
  // the zone has detected a component
  private transient BComponent detectedComponent;
  // the component that the zone ahs currently detected

  private transient boolean active = false;
  // indicates that the zone has has a sensor covering it.

  protected transient Tuple3f minDisp;
  protected transient Tuple3f maxDisp;

  protected transient double minScale;
  protected transient double maxScale;


  // static
  private static double ZONE_NAME_X_OFFSET = 0;
  //private static double ZONE_NAME_Y_OFFSET = 0;
 // private static double ZONE_NAME_X_OFFSET = -0.1;
  private static double ZONE_NAME_Y_OFFSET = -0.1;



  private static Color3f NORMALCOLOR =
    BGlobal.colors.getDarkShade3f();

  // YLY -temp active color for me to test
  // please change to fit into colour scheme
  private static Color3f ACTIVECOLOR = new Color3f(0.8f,0.8f,1f);

  //private static Color3f TEXTCOLOR =
  //  BGlobal.colors.getDarkShade3f();

  private static Color3f SELECTEDCOLOR =
    BGlobal.colors.getWhiteShade3f();

  private static PolygonAttributes pa = null;

  private transient ColoringAttributes  cAtrib = null;
  private transient RenderingAttributes rAtrib = null;

  private transient TransparencyAttributes tAtrib = null; // For alternate display
  private transient TextureAttributes texAtrib = null;
  private transient TexCoordGeneration texGen= null;
  private transient Material rangeMaterial = null;


  private static transient final LineAttributes zoneLine =
    new LineAttributes(1.0f, LineAttributes.PATTERN_SOLID, false);

  private static int TEXT_POINT_SIZE = 64;

  public transient float currentX = 0f;
  public transient float currentY = 0f;
  public transient float currentZ = 0f;
  public transient float boundingRadius = 0f;


  public BZone(BZone other)
  {
    this(new Vector3f(other.position), other.zoneNumber, other.name, other.scale);
  }

  public BZone(Vector3f newPosition, int zoneNumbernew, String namenew, double scalenew)
  {

    position = newPosition;
    zoneNumber = zoneNumbernew;

    name = namenew;

    if (name == null)
      name = zoneNumber + "";


    scale = scalenew;
    setDetected(false);

  }

  public int getZoneNumber()
    { return zoneNumber; }

  public void setName(String name)
  {
    if (name.equals("") )
      return;

    if (!this.name.equals(name) )
    {
      String oldName = this.name;
      this.name = name;
      parentList.zoneNameChanged(this, oldName);
    }
  }

  public String getName()
    { return name; }

  public String toString()
  {
    return getName();
  }

  public void detach()
  {
    if (detachGroup != null)
      detachGroup.detach();
      
    //BDebug.print ("BZone.detach called zoneTransGroup.isLive : " + zoneTransGroup.isLive() );

    cleanse();
  }


  public void deltaScale( double delta )
    { setScale(scale + delta); }

  public void deltaX( float delta )
    { setDisplacement( position.x + delta, position.y, position.z); }

  public void deltaZ( float delta )
    { setDisplacement( position.x, position.y, position.z + delta); }

  public void deltaY(float dy)
    { setDisplacement( position.x, position.y + dy, position.z); }

  public void deltaXZ(float dx, float dz)
    { setDisplacement(position.x + dx, position.y, position.z + dz); }

  public void setXZ(float x, float z)
    { setDisplacement(x, position.y, z); }

  public void setDisplacement(Tuple3f d)
    { position.set(d); }

  public void setDisplacement(float x, float y, float z )
  {
    position.x = x;
    position.y = y;
    position.z = z;

    if (minDisp != null)
    {
      if (position.x < minDisp.x) position.x = minDisp.x;
      if (position.y < minDisp.y) position.y = minDisp.y;
      if (position.z < minDisp.z) position.z = minDisp.z;
    }

    if (maxDisp != null)
    {
      if (position.x > maxDisp.x) position.x = maxDisp.x;
      if (position.y > maxDisp.y) position.y = maxDisp.y;
      if (position.z > maxDisp.z) position.z = maxDisp.z;
    }

    if (zoneTransGroup != null )
    {
      zoneTrans.setTranslation(position);
      zoneTransGroup.setTransform(zoneTrans);
    }
  }

  public void setDisplacementRange(Tuple3f min, Tuple3f max)
  {
    this.minDisp = min;
    this.maxDisp = max;
  }

  public void setNoDisplacementRange()
  {
    this.minDisp = null;
    this.maxDisp = null;
  }

  public void setScaleRange(double min, double max)
  {
    this.minScale = min;
    this.maxScale = max;
  }


  public Vector3f getDisplacement()
    { return position; }


  public void setScale(double newScale )
  {
    scale = newScale;

    if (scale < minScale)
      scale = minScale;
    else if (scale > maxScale)
      scale = maxScale;

    if (zoneScaleGroup != null )
    {
      zoneScale.setScale(scale);
      zoneScaleGroup.setTransform(zoneScale);
    }
  }

  public double getScale()
    { return scale; }


  public void attach (Group attachGroup, boolean optimise, boolean visible)
  // optimse is flag for optimising for the player
  {
    this.attachGroup = attachGroup;

    // displacement and size group
    zoneTrans = new Transform3D();
    zoneTrans.set(position);
    zoneTransGroup = new TransformGroup(zoneTrans);

    zoneScale = new Transform3D();
    zoneScaleGroup = new TransformGroup(zoneScale);

    textTrans = new Transform3D();
    textTrans.setTranslation(new Vector3d(ZONE_NAME_X_OFFSET, ZONE_NAME_Y_OFFSET, 0));
    textTrans.setScale(1.5);

    if ( BGlobal.SOLID_ZONES )
      textTrans.rotX(-0.7);

    textTransGroup = new TransformGroup(textTrans);
    textTransGroup.setTransform(textTrans);

    if (!optimise)
    {
      zoneTransGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
      zoneTransGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);

      zoneScaleGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
      zoneScaleGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
    }

    initStructure3D(visible);

    zoneTransGroup.addChild(zoneScaleGroup);
    zoneScaleGroup.addChild(theShape);

    // name box
    Text2D txt =
      new Text2D(name, BGlobal.WHITE_3F, BGlobal.FONT_NAME, TEXT_POINT_SIZE, Font.PLAIN);

    Geometry txtGeo = txt.getGeometry();
    if (txtGeo instanceof QuadArray )
    {
     // BDebug.print ("BZone.iniStructure3D txt.getGeometry() being processed" );
      BRectangle3D.alignXLeft( (QuadArray)txtGeo);
    }

    if (zoneName != null)
      zoneName.setAppearance(null);


    if ( BGlobal.SOLID_ZONES )
      zoneName = new Shape3D(txtGeo, txt.getAppearance());
    else
      zoneName = new OrientedShape3D(txtGeo, txt.getAppearance(), OrientedShape3D.ROTATE_ABOUT_POINT, BPhysics.CENTEROFUNIVERSE) ;

    zoneName.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
    textAppearance = zoneName.getAppearance();

    textAppearance.setColoringAttributes(cAtrib);
    textAppearance.setRenderingAttributes(rAtrib);
    zoneName.setPickable(false);

    textTransGroup.addChild(zoneName);
    zoneTransGroup.addChild(textTransGroup);

    if (optimise)
    {
      attachGroup.addChild(zoneTransGroup) ;
     // BDebug.print ("BZone.attach zone " + this + " to " + attachGroup );

    }
    else
    {
      detachGroup = new BranchGroup();
      detachGroup.setCapability(BranchGroup.ALLOW_DETACH );
      detachGroup.setCapability(Node.ALLOW_PICKABLE_WRITE);

      detachGroup.addChild(zoneTransGroup);
      attachGroup.addChild(detachGroup);
    }

    // BDebug.print ("BZone.attach called ");

    // Set zone position and scale...
    setScale(scale);
    setDisplacement(position);
  }

  public void setPickable(boolean p)
    { detachGroup.setPickable(p); }


  public void init(BZones parent)
  {
    parentList = parent;
  }

  private void initStructure3D(boolean visibility)
  {
    minDisp  = null;
    maxDisp  = null;
    minScale = BGlobal.ZONE_MIN_SIZE;
    maxScale = BGlobal.ZONE_MAX_SIZE;

     if (pa == null)
     {
          pa = new PolygonAttributes();

          if ( BGlobal.SOLID_ZONES )
          {
            pa.setPolygonMode(PolygonAttributes.POLYGON_FILL);
            pa.setCullFace(PolygonAttributes.CULL_NONE);
            NORMALCOLOR = BGlobal.LIGHT_GREY_3F;
            ACTIVECOLOR = BGlobal.HIGHLIGHT_COLOR_3F;
            SELECTEDCOLOR = BGlobal.ALT_HIGHLIGHT_COLOR_3F;
          } else {
            pa.setPolygonMode(PolygonAttributes.POLYGON_LINE);
            pa.setCullFace(PolygonAttributes.CULL_BACK);
          }
     }

     float size = BGlobal.ZONE_DEFAULT_SIZE;


     // Pre clean (just in case)
     if (theShape != null)
      theShape.cleanse();

     theShape = new BZoneShape(this, size, size, size);

     theShape.setCapability(Node.ALLOW_BOUNDS_READ);
     theShape.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
     theShape.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
     theShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);

     // Appearance
     zoneAppearance = new Appearance();

     // Colour
     cAtrib = new ColoringAttributes();
     cAtrib.setColor(NORMALCOLOR);
     cAtrib.setCapability(ColoringAttributes.ALLOW_COLOR_WRITE);
     cAtrib.setShadeModel(ColoringAttributes.SHADE_FLAT);

     rAtrib = new RenderingAttributes();
     rAtrib.setCapability(RenderingAttributes.ALLOW_VISIBLE_WRITE);
     rAtrib.setVisible(visibility);

     if ( BGlobal.SOLID_ZONES )
     {
       zoneAppearance.setColoringAttributes(cAtrib);
       zoneAppearance.setRenderingAttributes(rAtrib);
     	 tAtrib = new TransparencyAttributes(TransparencyAttributes.BLENDED , 0.7f, TransparencyAttributes.BLEND_SRC_ALPHA, TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA);
       texAtrib = new TextureAttributes();
       texAtrib.setTextureMode(TextureAttributes.MODULATE);
       if ( BGlobal.USE_AUTOGEN )
         texGen= new TexCoordGeneration(TexCoordGeneration.SPHERE_MAP, TexCoordGeneration.TEXTURE_COORDINATE_2);
       //else
       //  texGen= new TexCoordGeneration(TexCoordGeneration.OBJECT_LINEAR, TexCoordGeneration.TEXTURE_COORDINATE_2);
       zoneAppearance.setTextureAttributes(texAtrib);
       //zoneAppearance.setMaterial(new Material());
       zoneAppearance.setTransparencyAttributes(tAtrib);
       if ( BGlobal.USE_AUTOGEN )
         zoneAppearance.setTexCoordGeneration(texGen);
       zoneAppearance.setPolygonAttributes(pa);
       rangeMaterial = new Material();
       rangeMaterial.setLightingEnable(false);
       zoneAppearance.setMaterial(rangeMaterial);
       if ( BGlobal.USE_AUTOGEN )
         zoneAppearance.setTexture(BTextures.getTexture(BGlobal.TEXTUREPATH + BGlobal.ZONETEXTURE));
     } else {
       zoneAppearance.setLineAttributes(zoneLine);
       zoneAppearance.setColoringAttributes(cAtrib);
       zoneAppearance.setRenderingAttributes(rAtrib);
       //zoneAppearance.setLineAttributes(zoneLine);  // new LineAttributes(1.0f, LineAttributes.PATTERN_SOLID ,false)
       zoneAppearance.setPolygonAttributes(pa);
     }




     theShape.setAppearance(zoneAppearance);

  }



  public void cleanse()
    { cleanse(false); }

  public void cleanse(boolean reattaching)
  {



    if (detachGroup != null)
      detachGroup.detach();


    if (!reattaching)
    {
      if (theShape != null)
        theShape.cleanse();

      if (zoneName != null)
        zoneName.setAppearance(null);

      if (theShape != null)
        theShape.setAppearance(null);

      tAtrib = null; // For alternate display
  		texAtrib = null;
  		texGen= null;
 			rangeMaterial = null;

      attachGroup = null;

      zoneAppearance = null;
      theShape = null;

      //position = null;  // position in 3d space relative to the bot  (Canvas co-ordinates )
      detectedComponent = null;

      activatedImmobileSensor = null;

      parentList = null; // set up in init
      zoneTransGroup = null;
      zoneScaleGroup = null;;
      textTransGroup = null;

      zoneTrans = null;
      zoneScale = null;
      textTrans = null;

      attachGroup = null;
      detachGroup = null;
      zoneName = null;

      zoneAppearance = null;
      textAppearance = null;
      theShape = null;

      minDisp = null;
      maxDisp = null;

      tempT = null;
      currentVWorld = null;

    }

    /*
    detectedComponent = null;
    activatedImmobileSensor = null;

    parentList = null; // set up in init
    zoneTransGroup = null;
    zoneScaleGroup = null;;
    textTransGroup = null;

    zoneTrans = null;
    zoneScale = null;
    textTrans = null;

    attachGroup = null;
    detachGroup = null;
    zoneName = null;

    zoneAppearance = null;
    textAppearance = null;
    theShape = null;



    minDisp = null;
    maxDisp = null;   */



  }

  public void setHighlight(boolean turnOn)
  {
     if (turnOn)
        setHighlight(SELECTEDCOLOR);
     else
        setHighlight(NORMALCOLOR);
  }

  public void setVisible(boolean v)
  {
    if (rAtrib != null)
       rAtrib.setVisible(v);
  }

  private void setHighlight(Color3f highlightColour)
  {
    if (cAtrib != null)
       cAtrib.setColor(highlightColour);
  }


  public BZoneShape getShape()
    { return theShape; }


  private void updateDetectingBooleans(BComponent armingComponent)
  {
        if (armingComponent.getChassisLock())
         detectsChassis = true;
        else if (armingComponent.getComponentType() == BCpu.COMPONENT_TYPE)
         detectsController = true;
        else if (armingComponent.getComponentType() == BEnergyGenerator.COMPONENT_TYPE)
         detectsEnergy = true;
        else if (armingComponent.getComponentType() == BSensor.COMPONENT_TYPE)
          detectsSensor = true;
        else if (armingComponent.getComponentType() == BShielding.COMPONENT_TYPE )
          detectsShield = true;
        else if (armingComponent.getComponentType() == BWeapon.COMPONENT_TYPE)
          detectsWeapon = true;
        else if (armingComponent.getComponentType() == BJoint.COMPONENT_TYPE)
          detectsJoint = true;
        else if (armingComponent.getComponentType() == BAutoRotateJoint.COMPONENT_TYPE)
          detectsJoint = true;
  }


  public boolean testCollision(BComponent armingComponent, Node compNode, javax.media.j3d.Locale myLocale)
  // PRE :: armingComponent has been checked for 'destroyed' flag
  {
    //if (!detected)
    //{
      //BDebug.print ("BZone checking " + armingComponent + " and " + this );
    //if ( BCollision.boundsIntersect(compNode, theShape) )
    //{
      //if ( BCollision.bBoundsIntersect(armingComponent.getLiveNode(), theShape,  myLocale) )

    float diffX = java.lang.Math.abs(armingComponent.currentX - this.currentX);
    float diffY = java.lang.Math.abs(armingComponent.currentY - this.currentY);
    float diffZ = java.lang.Math.abs(armingComponent.currentZ - this.currentZ);
    float colDistance = armingComponent.boundingRadius + this.boundingRadius;
    float innerRadius = (float) (scale / 2d);

    if ( diffX <= colDistance && diffY <= colDistance && diffZ <= colDistance)
    {

      //if ( armingComponent.toString().startsWith("C180 ") && this.toString().startsWith("Front") )
    	//  BDebug.print("Zone Col Test: "+armingComponent.toString()+" "+this.toString()+" - x="+diffX+" - y="+diffY+" z="+diffZ+" dist="+colDistance);

      if ( diffX <= innerRadius && diffY <= innerRadius && diffZ <= innerRadius)
      {
        //BDebug.print("BZone sphere bounds: "+armingComponent.toString()+" "+this.toString()+" - x="+diffX+" - y="+diffY+" z="+diffZ+" dist="+colDistance);
        setDetected(true);
        updateDetectingBooleans(armingComponent);
      }
      //else if ( BCollision.bBoundsIntersect(compNode, theShape,  myLocale) )
      //{
      //  BDebug.print ("BZone middle point detected between " + armingComponent + " and " + this );
      //  setDetected(true);
      //  updateDetectingBooleans(armingComponent);
      //}
      else if (BCollision.bBoundsIntersect(armingComponent, theShape, myLocale))
      {
        //BDebug.print("BZone Poly Collide: "+armingComponent.toString()+" "+this.toString());
        setDetected(true);
        updateDetectingBooleans(armingComponent);
      }
    }
    //}



    //}

    return detected;
  }

  public String getDetectedComponentType()
  {
    if (detectedComponent == null)
      return BComponent.UNDEFINED_COMPONENT_TYPE;
    else
      return detectedComponent.getComponentType();

  }



  private void setDetected(boolean newDetected)
  {
    boolean diff =   newDetected != detected;
    detected = newDetected;

    if (diff )
      highlightDetected();

    if(!detected)
      detectedComponent = null;
  }

  private void highlightDetected()
  {
    if (detected)
      highlightActive(true);
    else
      highlightActive(false);
  }

  public String getDetectedChar()
  {
    if (detected)
      return "Y";
    else
      return "N";
  }


  public boolean getDetected()
  { return detected; }

  public void reattach(boolean optimize)
  {
    if (attachGroup != null)
    {
      detachGroup.detach();
      cleanse(true);
      attach(attachGroup, optimize, true);
    }
  }

  public void clearDetected()
  {
    detectsController = false;
    detectsEnergy = false;
    detectsSensor = false;
    detectsShield = false;
    detectsWeapon = false;
    detectsJoint = false;
    detectsChassis = false;

    setDetected(false);


  }

  private transient BSensor activatedImmobileSensor;
  private transient Transform3D tempT = new Transform3D();
  private transient Vector3f currentVWorld = new Vector3f();

  public boolean activate (BSensor[] sensors )
  // returns true if this zone was activated by an immobile sensor
  {

    if ( tempT == null )
    {
    	tempT = new Transform3D();
      currentVWorld = new Vector3f();
    }

    theShape.getLocalToVworld(tempT);
    tempT.get(currentVWorld);
    currentX = currentVWorld.x * BGlobal.CANVAS_TO_WORLD_SCALE;
    currentY = currentVWorld.y * BGlobal.CANVAS_TO_WORLD_SCALE;
    currentZ = currentVWorld.z * BGlobal.CANVAS_TO_WORLD_SCALE;

    if ( boundingRadius == 0f )
    {
      //float tempX = (float)(((BBox)theShape).getUpper().x * scale) * BGlobal.CANVAS_TO_WORLD_SCALE;
      //float tempZ = (float)(((BBox)theShape).getUpper().z * scale) * BGlobal.CANVAS_TO_WORLD_SCALE;
      //boundingRadius = (float) java.lang.Math.sqrt((double)(( tempX * tempX ) + (tempZ * tempZ)));
      float tempX = (float) (scale / 2d);
      boundingRadius = (float) java.lang.Math.sqrt((double)(( tempX * tempX ) + (tempX * tempX)));
    }

    //BDebug.print("Zone: "+toString()+" - radius="+boundingRadius+" scale="+scale+" x="+currentX+" y="+currentY+" z="+currentZ);

    if (activatedImmobileSensor!= null)
    {
       if (activatedImmobileSensor.getDestroyed())
        activatedImmobileSensor  = null;
       else
       {
        setActive(true);
        return true;
       }
    }


    setActive(false);

    for (int i =  0; i < sensors.length  && !active ; i++ )
    	if (  sensors[i].detects(this) )
      {
          setActive(true);
          if (sensors[i].isImmobile() )
          {
            activatedImmobileSensor = sensors[i];
            return true;

          }
      }


    return false;
  }

  public void setActive( boolean b)
  {
    //boolean changeHighlight = (b!= active);

    active = b;

  //  if (changeHighlight)
    //    highlightActive();

  }

  public boolean getActive( )
  { return active ;}

  public void highlightActive(boolean isActive)
  {
    if (isActive)
      setHighlight(ACTIVECOLOR);
    else
      setHighlight(NORMALCOLOR);
  }


  public boolean isDetectingController()
    { return detectsController; }

  public boolean isDetectingEnergy()
    { return detectsEnergy; }

  public boolean isDetectingSensor()
    { return detectsSensor; }

  public boolean isDetectingShielding()
    { return detectsShield; }

  public boolean isDetectingWeapon()
    { return detectsWeapon; }

  public boolean isDetectingJoint()
    { return detectsJoint; }

  public boolean isDetectingChassis()
    { return detectsChassis; }

  /////////////// BOX //////////////////////////////////




}
