package roboforge;

import javax.vecmath.*;
import java.util.*;
import javax.media.j3d.*;
import java.io.*;

import roboforge.ai.worldstate.*;
import roboforge.ai.values.*;

public class BZones extends BMonitoredList
  implements Serializable
{

 static final long serialVersionUID = -1L;
 private static Vector3f DEFAULT_ZONE_POSITION = new Vector3f(0f, 0f, 2.0f);

 private transient BranchGroup    zonesGroup = null;
 private transient TransformGroup worldScale = null;
 public transient boolean anyZoneDetected = false;
 private transient BWorldState lastWorldState;

 public static final int MAX_ZONES = 50;
 public static transient String stateCode = "zones";
 public static String FULL_WORLDSTATE_CODE =  BBot.stateCode + BWorldStateNode.DELIMITER+  BZones.stateCode + BWorldStateNode.DELIMITER;


 private transient boolean inOptimiseMode;


  // Deprecated variables...
  // -------------------------------------------------------------

  //private Vector zoneList;
    // SJS [29/05/2000] - Deprecated.
    // Transfer code for zoneList -> list is in init().


 // Constructor...
 // -------------------------------------------------------------

 public BZones()
   { }

 public void init()
 {

  for( int i = 0; i < list.size(); i++)
    ((BZone) list.get(i)).init(this);

 }



 public BZone addZone()
   { return addZone(new Vector3f(DEFAULT_ZONE_POSITION), "Zone"); }

 public BZone addZone(String name)
   { return addZone(new Vector3f(DEFAULT_ZONE_POSITION), name); }



 public BZone addZone(Vector3f posOfZone, String name)
 {
    int n = list.size() + 1;  // start adding at 1, only  programmers start counting at 0

    name = "Zone "+ n;
    boolean nameFound = true;

    while (nameFound)
    {
      nameFound = false;

      for(int i = 0; i < list.size(); i++)
        if (name.equalsIgnoreCase(list.get(i).toString() ) )
        {
          nameFound = true;
          n++;
          name = "Zone "+ n;

        }
    }



    BZone z = new BZone(posOfZone, n, name, 1f);
    return addZone(z);

 }

 public BZone addZone(BZone z)
 {
   if (zonesGroup != null)
     { z.attach(worldScale, inOptimiseMode, true); }

   add(z);
   super.setSelectedItem(z);

   return z;
 }


  public BZone getSelectedZone()
    { return (BZone) selected; }


  public BZone getZoneAt(int i)
    { return (BZone) list.get(i); }

  /*
  public int indexOf(BZone z)
    { return list.indexOf(z); }
  */

  public void cleanseLiveNodes()
  {
    try
    {
      clearDetected();

      if (zonesGroup != null)
        detach();

      zonesGroup = null;
    }
    catch (Exception e)
    {
      BDebug.print ("BZones.cleanseLiveNodes error " + e.getMessage() );

    }
  }

  public void cleanse()
  {
  // BDebug.print ("BZones.cleanse called ");
    cleanseLiveNodes();

    if (list!= null)
      for (int i = 0 ; i < list.size() ; i++ )
        { ((BZone)list.get(i)).cleanse()  ;}

    worldScale = null;
    lastWorldState = null;

    super.cleanse();
  }


  public void removeAll()
  {
    int n = list.size();

    for (int i=0; i<n; i++)
      ((BZone) list.get(i)).detach();

    super.removeAll();
  }

  /*
  public void remove(Object o)
  {
    int i = list.indexOf(o);
    if (i != -1)
      removeAt(i);
  }
  */

  public void removeAt(int i)
  {
    BZone z = (BZone) list.get(i);
    z.detach();

    super.removeAt(i);
  }


  public Vector getZones()
    { return list; }

  public Vector getZoneNames()
  {
    Vector v = new Vector();

    for (int i = 0; i < list.size(); i++)
      v.add(((BZone) list.get(i)).getName()  );

    return v;
  }

  public void attach(Group parentNode, boolean optimise, boolean visible)
  {
    inOptimiseMode = optimise;

    if (zonesGroup == null)
    {
      zonesGroup = new BranchGroup();
      zonesGroup.setCapability(BranchGroup.ALLOW_DETACH);

      // zonesGroup.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);

      Transform3D t = new Transform3D();
      t.setScale(BGlobal.WORLD_TO_CANVAS_SCALE);

      worldScale = new TransformGroup();
      worldScale.setTransform(t);

      if (!optimise)
      {
        worldScale.setCapability (Group.ALLOW_CHILDREN_EXTEND );
        worldScale.setCapability (Group.ALLOW_CHILDREN_WRITE );
        zonesGroup.setCapability (Node.ALLOW_BOUNDS_READ);
        zonesGroup.setCapability (Node.ALLOW_LOCAL_TO_VWORLD_READ);
      }
      else
        worldScale.setPickable(false);

      //BDebug.print ("BZones.attach list.size() " + list.size() );

      for (int i = 0; i < list.size(); i ++ )
        ((BZone) list.get(i)).attach(worldScale, optimise, visible);

      zonesGroup.addChild(worldScale);
    }

    parentNode.addChild(zonesGroup);



  }

  public void setPickable(boolean p)
  {
    for (int i = 0; i<list.size(); i++ )
    {
        ((BZone)list.get(i)).setPickable(p);
    }

  }

  public void detach()
  {
    //BDebug.print("BZones.detach(): Removing zonesGroup : live :." + zonesGroup.isLive() );
    if (zonesGroup != null)
      if (zonesGroup.isLive() )
          zonesGroup.detach();

  
  }

 public void setVisible(boolean v)
 {
   BZone z;

   int n = list.size();

   for (int i = 0; i < n; i++ )
   {
     z = (BZone) list.get(i);
     z.setVisible(v);
   }
 }



 public void clearDetected()
 {
    anyZoneDetected = false;

   if (list!= null)
    for (int i = 0 ; i < list.size() ; i++ )
     { ((BZone)list.get(i)).clearDetected()  ;}

 }

 public boolean testChannelBoolean ( String channelName, String channelString)
 {

  //BDebug.print ("BBot.testChannelBoolean channelString :" + channelString );
  //BDebug.print ("BBot.testChannelBoolean channelName :" + channelName );

  //boolean zoneIntersected = false;

  for (int i = 0 ; i < list.size() ; i++ )
  {
    if (  channelName.equals(list.get(i).toString() )  )
    {
      return channelString.charAt(i) == 'Y';
    }

  }


  return false ;






 }


 public String toString()
 {
    String channelString = "";

    for (int i = 0 ; i < list.size() ; i++ )
      channelString = channelString +  ((BZone)list.get(i)).getDetectedChar();

    return channelString;

 }

   // zone component type

  class BZoneBooleanWorldItem  extends BZoneWorldItem
  {
    static final long serialVersionUID = -1L;
    BBoolean theValue;

    public BZoneBooleanWorldItem(String key, String name ,BBoolean defaultValue, String units, String description, BZone theZone)
    {
      super( key,  name,  defaultValue,  defaultValue,  defaultValue,  units, description,  theZone) ;
      theValue = defaultValue;
    }

    public void setValue(boolean b)
    {
      theValue.setValue(b) ;

    }
  }

  /*
    class BZoneComponentWorldItem  extends BZoneWorldItem
  {

    static final long serialVersionUID = -1L;
    BComponentType theValue;

    public BZoneComponentWorldItem  (String key, String name ,BComponentType defaultValue, BZone theZone)
    {
      super(key, name ,defaultValue, "",  theZone) ;
      theValue = defaultValue;
    }

    public BValue evaluate()
    {
      theValue.setValue(getZone().getDetectedComponentType() ) ;
      return getValue();

    }
  }  */



  public void zoneNameChanged(BZone z, String oldName)
 {
    if (lastWorldState != null)
    {
      char dot = BWorldState.DELIMITER;
      
      // remove the last
      lastWorldState.remove(  BBot.stateCode+ dot+
                              BZones.stateCode+ dot+
                              oldName
                            );

      addWorldItemsForZone (lastWorldState, z);



    }
 }
  
  public void remove(Object o)
  {
    // clean the world state before we call super . remove so that the
    // ai can be cleaned correctly
    char dot = BWorldState.DELIMITER;

    if (lastWorldState != null)
      lastWorldState.remove(  BBot.stateCode+ dot+
                              BZones.stateCode+ dot+
                              o.toString()
                            );


    super.remove(o);



  }

   public void add(Object o)
  {
    // add worldstatefirst so that any zone list listeners that parse decisions evaluate correctly
    if (lastWorldState != null)
    {
      BZone z = (BZone) o;
      addWorldItemsForZone (lastWorldState, z);
      z.init(this);

    }

    super.add(o);



  }


 private void addWorldItemsForZone (BWorldState root, BZone currentZ)
 {
    char dot = BWorldStateNode.DELIMITER;

    String catName =
        FULL_WORLDSTATE_CODE+
        currentZ.getName();

      //root.put( new BWorldCategory(catName , currentZ.getName() ) );

      // zone is active
      BZoneBooleanWorldItem myZoneItem =
        new BZoneBooleanWorldItem(
          catName + dot+ "isActive",
          "Is active",
          new BBoolean(), "","'yes' if this zone detects an enemy", currentZ)
      {
        /**
		 * 
		 */
		private static final long serialVersionUID = -1L;

		public BValue evaluate()
        {
            setValue(getZone().getDetected());
            return getValue();
        }
      } ;

      root.put(myZoneItem );

       // Chassis
       root.put
      (
          new BZoneBooleanWorldItem( catName + dot+ "detectsChassis", "Detects chassis",
          new BBoolean(), "", "'yes' if this zone detects the enemy chassis", currentZ)
          {
            /**
			 * 
			 */
			private static final long serialVersionUID = -1L;

			public BValue evaluate()
            {
                setValue(getZone().isDetectingChassis());
                return getValue();
            }
          }

       );

       // controller
      root.put
      (
          new BZoneBooleanWorldItem( catName + dot+ "detectsController", "Detects controller",
          new BBoolean(), "", "'yes' if this zone detects an enemy controller", currentZ)
          {
            /**
			 * 
			 */
			private static final long serialVersionUID = -1L;

			public BValue evaluate()
            {
                setValue(getZone().isDetectingController());
                return getValue();
            }
          }

       );

       // energy
      root.put
      (
          new BZoneBooleanWorldItem( catName + dot+ "detectsEnergy", "Detects energy generator",
          new BBoolean(), "","'yes' if this zone detects an enemy energy generator", currentZ)
          {
            /**
			 * 
			 */
			private static final long serialVersionUID = -1L;

			public BValue evaluate()
            {
                setValue(getZone().isDetectingEnergy());
                return getValue();
            }
          }

       );

        // sensor
      root.put
      (
          new BZoneBooleanWorldItem( catName + dot+ "detectsSensor", "Detects sensor",
          new BBoolean(), "","'yes' if this zone detects an enemy sensor", currentZ)
          {
            /**
			 * 
			 */
			private static final long serialVersionUID = -1L;

			public BValue evaluate()
            {
                setValue(getZone().isDetectingSensor());
                return getValue();
            }
          }

       );

       // shielding
       root.put
      (
          new BZoneBooleanWorldItem( catName + dot+ "detectsShielding", "Detects shields",
          new BBoolean(), "","'yes' if this zone detects an enemy shield", currentZ)
          {
            /**
			 * 
			 */
			private static final long serialVersionUID = -1L;

			public BValue evaluate()
            {
                setValue(getZone().isDetectingShielding());
                return getValue();
            }
          }

       );

        // weapon
       root.put
      (
          new BZoneBooleanWorldItem( catName + dot+ "detectsWeapon", "Detects weapon",
          new BBoolean(), "","'yes' if this zone detects an enemy weapon",  currentZ)
          {
            /**
			 * 
			 */
			private static final long serialVersionUID = -1L;

			public BValue evaluate()
            {
                setValue(getZone().isDetectingWeapon());
                return getValue();
            }
          }

       );

         // Joint
       root.put
      (
          new BZoneBooleanWorldItem( catName + dot+ "detectsJoint", "Detects joint",
          new BBoolean(), "", "'yes' if this zone detects an enemy joint", currentZ)
          {
            /**
			 * 
			 */
			private static final long serialVersionUID = -1L;

			public BValue evaluate()
            {
                setValue(getZone().isDetectingJoint());
                return getValue();
            }
          }

       );





 }

 public void addWorldItems(BWorldState root)
 {
    lastWorldState = root;
    
    //char dot = BWorldStateNode.DELIMITER;

    for (int i = 0 ; i < list.size() ; i++ )
    {
      BZone currentZ =   (BZone)list.get(i);
      addWorldItemsForZone(root, currentZ);
    }

   // anyZones
    String catName =
        FULL_WORLDSTATE_CODE;

    BZoneBooleanWorldItem myZoneItem =
        new BZoneBooleanWorldItem(
          catName + "anyActive",
          "Any active",
          new BBoolean(), "","'yes' if any of my zones are active", null)
      {
        /**
		 * 
		 */
		private static final long serialVersionUID = -1L;

		public BValue evaluate()
        {
            //BDebug.print ( "BZones.anyActive evaluate " + anyZoneDetected );
            setValue(anyZoneDetected);
            return getValue();
        }

       /* public  BValue getValue()
        {
          BValue superV = super.getValue();
          BDebug.print ( "BZones.world.getValue() " + superV );
          return superV;
        }       */

      } ;

   root.put(myZoneItem );




 }



 public void highlightChannelInfo(String zoneChannel)
 {
    //BDebug.print ("BZones.highlightZoneChannel " + zoneChannel);
    if ( zoneChannel.length() == list.size() )
        for (int i = 0 ; i < list.size() ; i++ )
        {

                if (zoneChannel.charAt(i) == 'Y')
                     ((BZone)list.get(i)).setHighlight(true) ;
                else
                      ((BZone)list.get(i)).setHighlight(false) ;
        }
   else

   BDebug.print ("The Channel Information in this Battle is Incorrect :: " + zoneChannel );

 }


 public void activateZones(BSensor[] sensors )
 {
    //BDebug.seedTimer();

    BZone currentZone;


    for (int i = 0 ; i < list.size() ; i++ )
    {
      currentZone = (BZone)list.get(i);
      currentZone.activate(sensors);

    }

   // BDebug.printTime(" BZones.activateZones ");

 }

 /* DJG June 2001
 public void testCollision(BBot opponent, javax.media.j3d.Locale myLocale)
 {
       // BDebug.print("BZone.testCollision");

        boolean detected = false;

        clearDetected();

        // check the opponents shapes vs the zones
        if (BCollision.boundsIntersect(opponent.getChassis().getLiveNode(),zonesGroup ))
        {
                BZone currentZone;

                // activate my zones
                opponent.getOpponentBot().activateZones();

                for (int i = 0 ; i < list.size() ; i++ )
                {
                  currentZone = (BZone)list.get(i);

                  if (currentZone.getActive())
                    if ( BCollision.boundsIntersect(opponent.getChassis().getLiveNode(true),currentZone.getShape() ))
                       opponent.recurseCollisionTest(opponent.getChassis(), currentZone, myLocale);

                  if (currentZone.getDetected() )
                    anyZoneDetected = true;

                }




        }
 }
 */



  public BZone getZoneForName(String name)
  {

    for (int i = list.size() -1 ; i >=0; i-- )
    {
      BZone z = (BZone)list.get(i);

      if (name.equals( z.getName() )  )
        return z;
    }

    return null;
  }

  
}
