package roboforge;

import java.io.*;
import java.util.*;
import java.sql.*;
import javax.servlet.*;
                                 
import oracle.sql.*;
import oracle.jdbc.driver.*;

import roboforge.net.*;
import roboforge.net.data.*;
import roboforge.net.client.*;
import roboforge.net.server.*;
import roboforge.net.services.*;
import roboforge.util.io.*;
import roboforge.util.data.*;
import roboforge.server.util.*;

import java.awt.event.*;


public class SChallengeRoomServlet extends SGatewayAbstract implements ActionListener
{

  // Member variables...
  // -------------------------------------------------------------

  // source acts as the distributor of new messages
  //static MessageSource source = new MessageSource();
  static Map challengeRooms;

  static int challengeRoomID;

  static int maxNumberOfChallengeRooms;
  static int maxNumberOfParticipants;
  static float maxFightTime;
  static float maxEntryCost;

  javax.swing.Timer timer;

  // Servlet info method...
  // -------------------------------------------------------------

  public String getServletInfo()
    { return "roboforge.SChallengeRoomServlet"; }

  // Request Handler initialization...
  // -------------------------------------------------------------

  protected void initializeHandlers()
  {
    handlers = new HashMap();
    
    //register(new AccountCreationHandler());
    register(new LoginHandler());
    register(new GetMessage());
    register(new SendMessage());
    register(new EnterChallengeRoom());
    register(new GetChallengeRooms());
    register(new GetConstantData());
    register(new ExitChallengeRoom());
    register(new CreateChallengeRoom());
    register(new GetMembers());
    register(new Challenge());
    register(new AcknowledgeChallenge());
    register(new GetChallengeRooms());
    register(new GetTournaments());
    register(new CancelChallenge());
    register(new SetAFK());
  }

  protected static synchronized int getNextChallengeID()
  {
    return challengeRoomID++;
  }


  // Servlet Initialization...
  // -------------------------------------------------------------

  public void init(ServletConfig config) throws ServletException
  {
    super.init(config);

    try
    {
      //timer = new javax.swing.Timer(90000,this);
      timer = new javax.swing.Timer(15000,this);
      timer.start();

      // Create the challenge rooms
      challengeRooms = Collections.synchronizedMap(new HashMap());


      //challengeRooms.put(new Long(99),new SChallengeRoom(99,"Challenge One","Dust off",20,"volker_mechcropolis",35000,180,BChallengeRoomData.SIMULATION_ACCURACY_HIGH,null));
      //challengeRooms.put(new Long(2),new SChallengeRoom(2,"Challenge Two","Rumble Time",20,"volker_mechcropolis",35000,180,BChallengeRoomData.SIMULATION_ACCURACY_HIGH,null));
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
      throw new RuntimeException();
    }
  }

public void actionPerformed (ActionEvent ae)
  {

    Connection con = null;
    try
    {

      //BDebug.print("Doing the loop thing");
      con = getDatabaseConnection();
      cleanUpAccounts(con);
      updateTournaments(con);

      // TODO: Need to write code that goes through and deletes
      // all tournaments and related fights, tournament entries and log files
      // for Challenge room tournaments that doesn't exist in any challenge rooms 
      //deleteOrphanTournaments(con);


    }

    catch (Exception e)
    {
      System.err.println(e.getMessage());
    }
    finally
    {
      try
      {
        if (con!= null)
        {
          //Close the connection no matter what
          myBroker.freeConnection(con);
        }

      }
      catch(Exception sqle1)
      {
        System.err.println(sqle1.getMessage());
      }
    }
  }

public void updateTournaments (Connection con)
{

  Statement statement = null;
  ResultSet rs = null;
  String sql = "";

  try
  {
    // Go through all the tournaments and update their results from the database
    Iterator roomsIt = challengeRooms.values().iterator();
    while(roomsIt.hasNext())
    {
        SChallengeRoom room = (SChallengeRoom)roomsIt.next();
        Iterator tournamentsIt = room.tournamentCollection.values().iterator();
        while (tournamentsIt.hasNext())
        {
          // Loop through updating all the tournaments
          STournament t = (STournament)tournamentsIt.next();

          if (!t.tournamentStatus.equals("FINISHED"))
          {
            if (t.tournamentStatus.equals("WAITING"))
            {
              // Check to see if it's exceeded its waiting time
              // If so delete it
              long timeElapsed = (System.currentTimeMillis() - t.timeTournamentCreated);
              if (timeElapsed > 120000)
              {
                // Delete the tournament and related records
                deleteTournament(con, t.tournamentID.longValue(),
                                  t.fightID,t.challengerTEID, t.challengedTEID);
                // Inform the room members
                 BTournamentUpdateData tournamentUpdate = new BTournamentUpdateData(t.tournamentID.longValue(),
                                                                          BTournamentUpdateData.DELETED,
                                                                          t.challengerID,
                                                                          t.challengedID,
                                                                          t.challengerDesignerName,
                                                                          t.challengerDesignerName,
                                                                          t.tournamentStatus,
                                                                          t.fightProgress,
                                                                          t.tournamentName,
                                                                          t.fightID,
                                                                          t.challengerBotName,
                                                                          t.challengedBotName);

                // Delete the tournament
                room.tournamentCollection.remove(t.tournamentID);

                // Inform the members
                room.notifyObservers(null,tournamentUpdate);
              }
            }
            else
            {
              // It must be 'FIGHTING'

              // Check to see if it hasn't FINISHED
              sql = "SELECT Count(id) As fin FROM TOURNAMENT";
              sql += " WHERE status = 'FINISHED' AND id = " + t.tournamentID.longValue();
              statement = con.createStatement();
              rs = statement.executeQuery(sql);

              rs.next();

              int count = rs.getInt("fin");
              rs.close();
              statement.close();

              if (count == 1)
              {
                // Tournament Finished
                // Set the status of the tournament to 'FINISHED'
                t.tournamentStatus = "FINISHED";
                t.fightProgress = 100f;

                // Inform the room members
                BTournamentUpdateData tournamentUpdate = new BTournamentUpdateData(t.tournamentID.longValue(),
                                                                          BTournamentUpdateData.UPDATED,
                                                                          t.challengerID,
                                                                          t.challengedID,
                                                                          t.challengerDesignerName,
                                                                          t.challengerDesignerName,
                                                                          t.tournamentStatus,
                                                                          t.fightProgress,
                                                                          t.tournamentName,
                                                                          t.fightID,
                                                                          t.challengerBotName,
                                                                          t.challengedBotName);


                //BDebug.print("Tournament Finished:");
                //BDebug.print("Tournament Finished: Challenger Bot Name: " + tournamentUpdate.challengerBotName);
                //BDebug.print("Tournament Finished: Challenged Bot Name: " + tournamentUpdate.challengedBotName);


                room.notifyObservers(null,tournamentUpdate);
              }
              else
              {
                // Tournament not finished

                // So find the log entry and get the progress
                sql = "SELECT fight_progress FROM FIGHT_CLIENT_LOG";
                sql += " WHERE fight_id = " + t.fightID;
                statement = con.createStatement();
                rs = statement.executeQuery(sql);

                //BDebug.print("Update Tournament Fighting: " + t.fightID);


                if (rs.next())
                {
                  //BDebug.print("Update Tournament/Found Fighting Tournament: "+ t.fightID);
                  float fightProgress = rs.getFloat("fight_progress");

                  if (t.fightProgress == fightProgress)
                  {
                    // The fight progress hasn't changed

                    long timeElapsed = (System.currentTimeMillis() - t.timeLastUpdated);
                    // If the tournament/fight hasn't progressed for over 5 mins (300 secs)
                    // then delete it if it's not at 100% (probably sending it up)
                    // Or if it's taken longer then 15mins then delete it

                    if ((timeElapsed > 300000 && t.fightProgress != 100) || (timeElapsed > 900000))
                    {
                      // Delete the tournament
                      deleteTournament(con, t.tournamentID.longValue(),
                                  t.fightID,t.challengerTEID, t.challengedTEID);

                      // Inform the room members
                      BTournamentUpdateData tournamentUpdate = new BTournamentUpdateData(t.tournamentID.longValue(),
                                                                          BTournamentUpdateData.DELETED,
                                                                          t.challengerID,
                                                                          t.challengedID,
                                                                          t.challengerDesignerName,
                                                                          t.challengerDesignerName,
                                                                          t.tournamentStatus,
                                                                          t.fightProgress,
                                                                          t.tournamentName,
                                                                          t.fightID,
                                                                          t.challengerBotName,
                                                                          t.challengedBotName);

                      // Delete the tournament
                      room.tournamentCollection.remove(t.tournamentID);

                      // Inform the members
                      room.notifyObservers(null,tournamentUpdate);

                      //BDebug.print("Update Tournament/Fight not progressed: Deleting Fight "+ t.fightID + " Tournament: " + t.tournamentID);

                    }

                  }
                  else
                  {
                    // The fight has progressed
                    // Update the progression
                    t.fightProgress = fightProgress;
                    t.timeLastUpdated = System.currentTimeMillis();

                    // Inform the room members
                    BTournamentUpdateData tournamentUpdate = new BTournamentUpdateData(t.tournamentID.longValue(),
                                                                          BTournamentUpdateData.UPDATED,
                                                                          t.challengerID,
                                                                          t.challengedID,
                                                                          t.challengerDesignerName,
                                                                          t.challengerDesignerName,
                                                                          t.tournamentStatus,
                                                                          t.fightProgress,
                                                                          t.tournamentName,
                                                                          t.fightID,
                                                                          t.challengerBotName,
                                                                          t.challengedBotName);
                    // Inform the members
                    room.notifyObservers(null,tournamentUpdate);

                    //BDebug.print("Update Tournament/Told Members of Room: "+ room.roomName);
                    //BDebug.print("Update Tournament/Tournament Name: "+ t.tournamentName);
                    //BDebug.print("Update Tournament/Fight ID: "+ t.fightID);
                    //BDebug.print("Update Tournament/Progress: "+ t.fightProgress);

                  }

                }
                else
                {
                  // Strange can't find log for fight
                  BDebug.print("SChallengeRoomServlet/updateTournaments: Can't find log for fight: " + t.fightID);

                }
                rs.close();
                statement.close();
              }
            }
          }
        }
    }
  }
  catch (Exception e)
  {
    BDebug.print("Error: SChallengeRoomServlet/updateTournaments: " + e.getMessage());
  }
  finally
  {
    closeResultSet(rs,"SChallengeRoomServlet/updateTournaments");
    closeStatement(statement,"SChallengeRoomServlet/updateTournaments");
  }
}

public static void deleteTournament(Connection con, long tournamentID, long fightID,
                              long challengerTEID, long challedgedTEID)
{
  Statement statement = null;
  String sql = "";

  try
  {
    //BDebug.print("***** Deleting Tournamnets *****");
    //BDebug.print("TournamentID: "+ tournamentID);
    //BDebug.print("FightID: "+ fightID);
    //BDebug.print("ChallengerID: " + challengerTEID);
    //BDebug.print("ChallengedTEID: "+ challedgedTEID);

    // Delete any related log files for validation tournament entries
    sql = "DELETE FROM FIGHT_CLIENT_LOG WHERE (tournament_entry_id = " + challengerTEID;
    sql += " OR tournament_entry_id = " + challedgedTEID + ")";
    statement = con.createStatement();
    statement.executeUpdate(sql);
    statement.close();

    // Delete any related log files for fight tournament entries
    sql = "DELETE FROM FIGHT_CLIENT_LOG WHERE fight_id = " + fightID;
    statement = con.createStatement();
    statement.executeUpdate(sql);
    statement.close();

    // Delete any tournament entries
    sql = "DELETE FROM TOURNAMENT_ENTRY WHERE (id = " + challengerTEID;
    sql += " OR id = " + challedgedTEID + ")";
    statement = con.createStatement();
    statement.executeUpdate(sql);
    statement.close();

    // Delete any fights
    sql = "DELETE FROM FIGHT WHERE id = " + fightID;
    statement = con.createStatement();
    statement.executeUpdate(sql);
    statement.close();

    // Finally delete the tournament
    sql = "DELETE FROM TOURNAMENT WHERE id = " + tournamentID;
    statement = con.createStatement();
    statement.executeUpdate(sql);
    statement.close();
  }
  catch (Exception e)
  {
    BDebug.print("Error: SChallengeRoomServlet/deleteTournament: " + e.getMessage());
  }
  finally
  {
    closeStatement(statement,"SChallengeRoomServlet/updateTournaments");
  }
}

public void cleanUpAccounts (Connection con)
{
  Iterator challengeRoomsIt = challengeRooms.values().iterator();

    while(challengeRoomsIt.hasNext())
    {
      SChallengeRoom challengeRoom = (SChallengeRoom)challengeRoomsIt.next();
      if (challengeRoom.getNoOfPeopleInRoom() == 0)
      {
        if (!challengeRoom.justCreated)
        {
          // Delete the challenge room

          // First go through cleaning up all database files
          // related to tournaments
          Iterator tIt = challengeRoom.tournamentCollection.values().iterator();

          while (tIt.hasNext())
          {
            STournament t = (STournament)tIt.next();
            deleteTournament(con, t.tournamentID.longValue(),
                                  t.fightID,t.challengerTEID, t.challengedTEID);
          }

          challengeRoom.destroy();
          challengeRooms.remove(new Long(challengeRoom.roomNumber));
        }
        else
        {
          challengeRoom.justCreated = false;
        }
      }
      else
      {
        // Go through all the accounts in the room marking any
        // that haven't connected for greater than 30 seconds as afk
        // Or if the account is marked as afk, they are then deleted if they haven't connected
        // for more than 10 minutes.
        Iterator accountsIt = challengeRoom.accountCollection.values().iterator();
        while(accountsIt.hasNext())
        {
          SAccount account = (SAccount)accountsIt.next();

          if (account.afk)
          {

            if ((System.currentTimeMillis() - account.timeLastConnected) > 600000)
            {
              // Account is logged out due to inactivity and the room is informed
              challengeRoom.logoutAccount(account.accountID);

              // Inform everyone else in the room that someone has exited
              BExitRoomData exitRoomData = new BExitRoomData(account.accountID.longValue());
              exitRoomData.challengeRoomID = challengeRoom.roomNumber;

              //BDebug.print("logging out due to afk of the account: " + exitRoomData.getID());
              //BDebug.print("logging out due to afk CR: " + exitRoomData.challengeRoomID);

              challengeRoom.notifyObservers(null,exitRoomData);

            }
          }
          else
          {
            if ((System.currentTimeMillis() - account.timeLastConnected) > 30000)
            {

              // Account is no longer deleted but marked as afk
              account.afk = true;
              /*
              // Account is logged out due to inactivity and the room is informed
              challengeRoom.logoutAccount(account.accountID);

              // Inform everyone else in the room that someone has exited
              BExitRoomData exitRoomData = new BExitRoomData(account.accountID.longValue());
              exitRoomData.challengeRoomID = challengeRoom.roomNumber;

              BDebug.print("logging out due to time out of the account: " + exitRoomData.getID());
              BDebug.print("logging out due to time out CR: " + exitRoomData.challengeRoomID);
              challengeRoom.notifyObservers(null,exitRoomData);
              */
            }
          }
        }

        // Send everyone in the room some test data, to make sure they
        // are really there
        BTestAliveData testData = new BTestAliveData(-1);
        challengeRoom.notifyObservers(null,testData);
      }
    }
}


  // Get challenge Rooms
  // -------------------------------------------------------------

  protected static class GetChallengeRooms extends ValidatedRequestHandler
  {
    public String getType()
      { return "GCR"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      // Loop through the challenge Room collection
      // getting the challenge room data

      Iterator challengeRoomsIt = challengeRooms.values().iterator();
      BDataSet dataSet = new BDataSet();

      while(challengeRoomsIt.hasNext())
      {
        SChallengeRoom challengeRoom = (SChallengeRoom)challengeRoomsIt.next();

        BChallengeRoomData challengeRoomData      = new BChallengeRoomData(challengeRoom.roomNumber);
        challengeRoomData.roomName                = challengeRoom.roomName;
        challengeRoomData.roomDescription         = challengeRoom.roomDescription;
        challengeRoomData.maxEntryCost            = challengeRoom.maxEntryCost;
        challengeRoomData.maxFightTime            = challengeRoom.maxFightTime;
        challengeRoomData.maxNumberOfParticipants = challengeRoom.maxNumberOfParticipants;
        challengeRoomData.simulationSpeed         = challengeRoom.simulationSpeed;
        challengeRoomData.currentNoOfPeople       = challengeRoom.accountCollection.size();
        challengeRoomData.arena                   = challengeRoom.arena;

        if (challengeRoom.password != null)
          challengeRoomData.isPassworded = true;
        else
          challengeRoomData.isPassworded = false;

        dataSet.add(challengeRoomData);
      }

      BMessage response = BMessage.ACK("GCR");

      response.setData(dataSet);

      // Zip and encrypt the response...
      services.add(BServices.zip());
      //services.add(BServices.encrypt());

      return response;
    }
  }

  // Get Members of a challenge room
  // -------------------------------------------------------------

  protected static class CancelChallenge extends ValidatedRequestHandler
  {
    public String getType()
      { return "CCHAL"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      BCRAuthenticationData authData = (BCRAuthenticationData)request.getData();

      // Get the challenge room
      SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(new Long(authData.challengeRoomID));

      // Find the tournament
      STournament t = (STournament)challengeRoom.tournamentCollection.get(new Long(authData.tournamentID));

      // Check to see if this account is the challenged of this tournament
      if (!t.challengedID.equals(authData.getIDLong()))
      {
        // Not the challenged so can't delete it
        BMessage response = BMessage.error("CCHAL","Only the challenged can cancel a fight");
        return response;
      }

      // Check the status
      if (t.tournamentStatus.equals("FINISHED"))
      {
        // The fight has finished so can't delete it
        BMessage response = BMessage.error("CCHAL","Can't delete a Finished fight");
        return response;
      }

      // Create the tournament notification data object before the tournament is deleted
      BTournamentUpdateData tournamentUpdate = new BTournamentUpdateData(t.tournamentID.longValue(),
                                                                          BTournamentUpdateData.DELETED,
                                                                          t.challengerID,
                                                                          t.challengedID,
                                                                          t.challengerDesignerName,
                                                                          t.challengedDesignerName,
                                                                          t.tournamentStatus,
                                                                          t.fightProgress,
                                                                          t.tournamentName,
                                                                          t.fightID,
                                                                          t.challengerBotName,
                                                                          t.challengedBotName);


      // Delete the tournament
      challengeRoom.tournamentCollection.remove(t.tournamentID);

      // Delete the tournament and related records
      deleteTournament(reqState.con, t.tournamentID.longValue(),
                                  t.fightID,t.challengerTEID, t.challengedTEID);

      // Inform the members
      challengeRoom.notifyObservers(null,tournamentUpdate);

      BMessage response = BMessage.ACK("CCHAL");

      // Zip and encrypt the response...
      //services.add(BServices.zip());
      //services.add(BServices.encrypt());

      return response;
    }
  }
  // Get Members of a challenge room
  // -------------------------------------------------------------

  protected static class GetMembers extends ValidatedRequestHandler
  {
    public String getType()
      { return "GMEM"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      BCRAuthenticationData authData = (BCRAuthenticationData)request.getData();

      // Get the challenge room
      SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(new Long(authData.challengeRoomID));

      // Get all the members of the challenge rooms
      Iterator challengeRoomIt = challengeRoom.accountCollection.values().iterator();
      BDataSet dataSet = new BDataSet();

      while(challengeRoomIt.hasNext())
      {
        SAccount account = (SAccount)challengeRoomIt.next();

        BCRMemberData memberData      = new BCRMemberData(account.accountID.longValue());
        memberData.designerName       = account.designerName;
        memberData.designerBlurb      = account.designerBlurb;
        memberData.afk                = account.afk;
        memberData.proStatus          = account.proStatus;
        dataSet.add(memberData);
      }

      BMessage response = BMessage.ACK("GMEM");

      response.setData(dataSet);

      // Zip and encrypt the response...
      services.add(BServices.zip());
      //services.add(BServices.encrypt());

      return response;
    }

  public BMessage validate (BRequest request, RequestState reqState)
    {

      try
      {
        // Get the user information
        BCRAuthenticationData authData = (BCRAuthenticationData)request.getData();

        // Does the challenge room exist
        if (!challengeRooms.containsKey(new Long(authData.challengeRoomID)))
          return BMessage.error(request.getType(), "Unknown Challenge Room");

        SChallengeRoom chalRoom = (SChallengeRoom)challengeRooms.get(new Long(authData.challengeRoomID));

        // Is the challenge room password correct?
        if (chalRoom.password != null)
        {
          if (authData.password == null)
            return BMessage.error(request.getType(), "Incorrect Null Password");

          if (!chalRoom.password.equals(authData.password))
            return BMessage.error(request.getType(), "Incorrect Password");
        }

        // Is the person logged into the challenge room
        if (!chalRoom.accountCollection.containsKey(authData.getIDLong()))
        {
          System.err.println("Get Members: Account doesn't exist in room");
          return BMessage.error(request.getType(), "Account doesn't exist in room");
        }

        // Check to see if the account password  and loginID is correct
        String user = request.getRequestHeader().getUser();
        String password = request.getRequestHeader().getPass();

        if (user == null || user.equals("") || password == null || password.equals(""))
          return BMessage.error(request.getType(), "Empty login/password");


        SAccount account = (SAccount)chalRoom.accountCollection.get(authData.getIDLong());

        if (!account.loginID.equals(user)|| !account.password.equals(password))
          return BMessage.error(request.getType(), "Incorrect login/password combination");

        reqState.accountID = authData.getID();
        String type = account.type;

        if (type != null)
          reqState.type = type;

        reqState.designerName   = account.designerName;
        reqState.designerBlurb  = account.designerBlurb;
        reqState.proStatus      = account.proStatus;
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }

      // All is ok
      return null;
    }
  }

  // Get the Tournaments(Fights) of a challenge room
  // -------------------------------------------------------------

  protected static class GetTournaments extends ValidatedRequestHandler
  {
    public String getType()
      { return "GTRN"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      BCRAuthenticationData authData = (BCRAuthenticationData)request.getData();

      // Get the challenge room
      SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(new Long(authData.challengeRoomID));

      // Get all the tournaments of the challenge rooms
      Iterator tournamentsIT = challengeRoom.tournamentCollection.values().iterator();
      BDataSet dataSet = new BDataSet();

      while(tournamentsIT.hasNext())
      {
        STournament t = (STournament)tournamentsIT.next();

        BCRTournamentData tournamentData  = new BCRTournamentData(t.tournamentID.longValue(),t.tournamentName,
                                                                  t.tournamentStatus, t.fightProgress,
                                                                  t.challengerID.longValue(),t.challengedID.longValue(),
                                                                  t.challengerDesignerName,t.challengedDesignerName,t.fightID);

        tournamentData.challengedBotName = t.challengedBotName;
        tournamentData.challengerBotName = t.challengerBotName;

        dataSet.add(tournamentData);
      }

      BMessage response = BMessage.ACK("GTRN");

      response.setData(dataSet);

      // Zip and encrypt the response...
      services.add(BServices.zip());
      //services.add(BServices.encrypt());

      return response;
    }

  public BMessage validate (BRequest request, RequestState reqState)
    {

      try
      {
        // Get the user information
        BCRAuthenticationData authData = (BCRAuthenticationData)request.getData();

        // Does the challenge room exist
        if (!challengeRooms.containsKey(new Long(authData.challengeRoomID)))
          return BMessage.error(request.getType(), "Unknown Challenge Room");

        SChallengeRoom chalRoom = (SChallengeRoom)challengeRooms.get(new Long(authData.challengeRoomID));

        // Is the challenge room password correct?
        if (chalRoom.password != null)
        {
          if (authData.password == null)
            return BMessage.error(request.getType(), "Incorrect Null Password");

          if (!chalRoom.password.equals(authData.password))
            return BMessage.error(request.getType(), "Incorrect Password");
        }

        // Is the person logged into the challenge room
        if (!chalRoom.accountCollection.containsKey(authData.getIDLong()))
        {
          System.err.println("Get Tournaments: Account doesn't exist in room");
          return BMessage.error(request.getType(), "Account doesn't exist in room");
        }

        // Check to see if the account password  and loginID is correct
        String user = request.getRequestHeader().getUser();
        String password = request.getRequestHeader().getPass();

        if (user == null || user.equals("") || password == null || password.equals(""))
          return BMessage.error(request.getType(), "Empty login/password");


        SAccount account = (SAccount)chalRoom.accountCollection.get(authData.getIDLong());

        if (!account.loginID.equals(user)|| !account.password.equals(password))
          return BMessage.error(request.getType(), "Incorrect login/password combination");

        reqState.accountID = authData.getID();
        String type = account.type;

        if (type != null)
          reqState.type = type;

        reqState.designerName   = account.designerName;
        reqState.designerBlurb  = account.designerBlurb;
        reqState.proStatus      = account.proStatus;
      }
      catch (Exception e)
      {
        System.err.println("SChallengeRoomServelt/GetTournaments/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }

      // All is ok
      return null;
    }
  }

  // Get constant data
  // -------------------------------------------------------------

  protected static class GetConstantData extends ValidatedRequestHandler
  {
    public String getType()
      { return "GCD"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      String sql;
      Statement statement = null;
      ResultSet rs = null;

      try
      {

        Connection con = reqState.con;


        // Return List of the Different Types of Featured Fights
        sql = "SELECT max_no_of_cr, max_cr_entries, max_cr_fight_duration, max_cr_entry_cost";
        sql += " FROM CONSTANTS";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);

        rs.next();

        BChallengeConstantData chalConstData = new BChallengeConstantData(-1);

        chalConstData.maxNumberOfChallengeRooms = rs.getInt("max_no_of_cr");
        chalConstData.maxNumberOfParticipants   = rs.getInt("max_cr_entries");
        chalConstData.maxFightTime              = rs.getFloat("max_cr_fight_duration");
        chalConstData.maxEntryCost              = rs.getFloat("max_cr_entry_cost");

        maxNumberOfChallengeRooms = chalConstData.maxNumberOfChallengeRooms;
        maxNumberOfParticipants   = chalConstData.maxNumberOfParticipants;
        maxFightTime              = chalConstData.maxFightTime;
        maxEntryCost              = chalConstData.maxEntryCost;

        rs.close();
        statement.close();

        BMessage response = BMessage.ACK("GCD");

        response.setData(chalConstData);

        // Zip the response...
        services.add(BServices.zip());

        return response;

      }
      catch (Exception e)
      {
        System.err.println("SChallengeRoomServlet/GetConstantData: " + e.getMessage());
        return BMessage.error("GCD", e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SChallengeRoomServlet/GetConstantData");
        closeStatement(statement,"SChallengeRoomServlet/GetConstantData");
      }
    }
  }

protected static void closeResultSet(ResultSet rs, String location)
  {
    try
    {
      if (rs != null)
        rs.close();
    }
    catch (Exception e)
    {
      System.err.println("SChallengeRoomServlet/closeResultSet: "+ location + ": " + e);
    }

  }

  protected static void closeStatement(Statement statement, String location)
  {
    try
    {
      if (statement != null)
        statement.close();
    }
    catch (Exception e)
    {
      System.err.println("SChallengeRoomServlet/closeStatement: "+ location + ": " + e);
    }

  }
  // Create challenge Rooms
  // -------------------------------------------------------------

  protected static class CreateChallengeRoom extends ValidatedRequestHandler
  {
    public String getType()
      { return "CCR"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      // Make sure we haven't exceeded out max number of Challenge Rooms
      if (challengeRooms.size() >= maxNumberOfChallengeRooms)
        return BMessage.error(request.getType(), "Can't create any more challenge rooms");

      BChallengeRoomData cRD = (BChallengeRoomData)request.getData();

      SChallengeRoom challengeRoom = new SChallengeRoom(getNextChallengeID(),
                                                        cRD.roomName,
                                                        cRD.roomDescription,
                                                        cRD.maxNumberOfParticipants,
                                                        cRD.arena,
                                                        cRD.maxEntryCost,
                                                        cRD.maxFightTime,
                                                        cRD.simulationSpeed,
                                                        cRD.password);

      challengeRooms.put(new Long(challengeRoom.roomNumber), challengeRoom);
      BMessage response = BMessage.ACK("CCR");

      BIdentData identData = new BIdentData(challengeRoom.roomNumber);
      response.setData(identData);
      // Zip and the response...
      services.add(BServices.zip());

      return response;
    }
  }


  // Exit challenge Rooms
  // -------------------------------------------------------------

  protected static class ExitChallengeRoom extends ValidatedRequestHandler
  {
    public String getType()
      { return "EXCR"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      BCRAuthenticationData cRAuthData = (BCRAuthenticationData)request.getData();

      SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(new Long(cRAuthData.challengeRoomID));




      // Inform everyone else in the room that someone has exited
      BExitRoomData exitRoomData = new BExitRoomData(cRAuthData.getID());
      exitRoomData.challengeRoomID = challengeRoom.roomNumber;

      challengeRoom.notifyObservers(null,exitRoomData);

      challengeRoom.logoutAccount(cRAuthData.getIDLong());

      BMessage response = BMessage.ACK("EXCR");
      // Zip and the response...
      services.add(BServices.zip());

      return response;
    }
  }

  // Set AFK...
  // -------------------------------------------------------------

  protected static class SetAFK extends ValidatedRequestHandler
  {
    public String getType()
      { return "AFK"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
     /*
      // Find the challenge room where the account exists
      Iterator cRooms = challengeRooms.values().iterator();
      boolean found = false;
      SChallengeRoom challengeRoom = null;

      while(cRooms.hasNext() && found == false)
      {
        SChallengeRoom cRoom = (SChallengeRoom)cRooms.next();
        Map map = cRoom.accountCollection;
        if(map.containsKey(new Long(reqState.accountID)))
        {
          // Found the challenge room for this account
          found = true;
          challengeRoom = cRoom;
        }
      }

      if (challengeRoom == null)
      {
        BDebug.print("Client not logged into any challenge room");
        return BMessage.NAK("Client not logged into any challenge room");
      } */

      // Get the user information
      BCRAuthenticationData authData = (BCRAuthenticationData)request.getData();

      SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(new Long(authData.challengeRoomID));
      SAccount account = (SAccount)challengeRoom.accountCollection.get(authData.getIDLong());

      account.afk = authData.afk;

      //BDebug.print("*** Setting account: " + account.accountID.longValue() + " to afk state: " + account.afk);

      // Tell the group person afk state
      BAFKData afkData = new BAFKData(account.accountID.longValue());
      afkData.afk = account.afk;

      challengeRoom.notifyObservers(null, afkData);

      BMessage response = BMessage.ACK("AFK");

      // Zip the response...
      //services.add(BServices.zip());

      return response;
    }

    public BMessage validate (BRequest request, RequestState reqState)
    {

      try
      {
        // Get the user information
        BCRAuthenticationData authData = (BCRAuthenticationData)request.getData();

        // Does the challenge room exist
        if (!challengeRooms.containsKey(new Long(authData.challengeRoomID)))
          return BMessage.error(request.getType(), "Unknown Challenge Room");

        SChallengeRoom chalRoom = (SChallengeRoom)challengeRooms.get(new Long(authData.challengeRoomID));

        // Is the challenge room password correct?
        if (chalRoom.password != null)
        {
          if (authData.password == null)
            return BMessage.error(request.getType(), "Incorrect Null Password");

          if (!chalRoom.password.equals(authData.password))
            return BMessage.error(request.getType(), "Incorrect Password");
        }

        // Is the person logged into the challenge room
        if (!chalRoom.accountCollection.containsKey(authData.getIDLong()))
        {
          System.err.println("Set AFK: Account doesn't exist in room");
          return BMessage.error(request.getType(), "Account doesn't exist in room");
        }

        // Check to see if the account password  and loginID is correct
        String user = request.getRequestHeader().getUser();
        String password = request.getRequestHeader().getPass();

        if (user == null || user.equals("") || password == null || password.equals(""))
          return BMessage.error(request.getType(), "Empty login/password");


        SAccount account = (SAccount)chalRoom.accountCollection.get(authData.getIDLong());

        if (!account.loginID.equals(user)|| !account.password.equals(password))
          return BMessage.error(request.getType(), "Incorrect login/password combination");

        reqState.accountID = authData.getID();
        String type = account.type;

        if (type != null)
          reqState.type = type;

        reqState.designerName   = account.designerName;
        reqState.designerBlurb  = account.designerBlurb;
        reqState.proStatus      = account.proStatus;
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }

      // All is ok
      return null;
    }
  }

  // GetMessage...
  // -------------------------------------------------------------

  protected static class GetMessage extends ValidatedRequestHandler
  {
    public String getType()
      { return "GMES"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
     /*
      // Find the challenge room where the account exists
      Iterator cRooms = challengeRooms.values().iterator();
      boolean found = false;
      SChallengeRoom challengeRoom = null;

      while(cRooms.hasNext() && found == false)
      {
        SChallengeRoom cRoom = (SChallengeRoom)cRooms.next();
        Map map = cRoom.accountCollection;
        if(map.containsKey(new Long(reqState.accountID)))
        {
          // Found the challenge room for this account
          found = true;
          challengeRoom = cRoom;
        }
      }

      if (challengeRoom == null)
      {
        BDebug.print("Client not logged into any challenge room");
        return BMessage.NAK("Client not logged into any challenge room");
      } */

      // Get the user information
      BCRAuthenticationData authData = (BCRAuthenticationData)request.getData();

      SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(new Long(authData.challengeRoomID));
      MessageSink messageSink = new MessageSink(reqState.accountID);

      // Test to see if the client is already listening in this challenge room
      // should be a very unusual situation for this to happen
      if (challengeRoom.observerExists(messageSink))
      {
        BDebug.print("Client already logged on in another session");

        // Don't treat this as a fatal error any more
        // Just send a message to the thread to clear it and add the new thread
        challengeRoom.notifyObservers(new Long(reqState.accountID),new BTestAliveData(-1));
        //return BMessage.NAK("Client already logged on in another session");
      }


      BMessage response = BMessage.ACK("GMES");

      // The getNextMessage() method of the MessageSink object returns the next new message(s).
      // or it blocks until there is one.
      //response.setData(new MessageSink(reqState.accountID).getNextMessage(challengeRoom));
      response.setData(messageSink.getNextMessage(challengeRoom));

      // Zip the response...
      services.add(BServices.zip());

      return response;
    }

    public BMessage validate (BRequest request, RequestState reqState)
    {

      try
      {
        // Get the user information
        BCRAuthenticationData authData = (BCRAuthenticationData)request.getData();

        // Does the challenge room exist
        if (!challengeRooms.containsKey(new Long(authData.challengeRoomID)))
          return BMessage.error(request.getType(), "Unknown Challenge Room");

        SChallengeRoom chalRoom = (SChallengeRoom)challengeRooms.get(new Long(authData.challengeRoomID));

        // Is the challenge room password correct?
        if (chalRoom.password != null)
        {
          if (authData.password == null)
            return BMessage.error(request.getType(), "Incorrect Null Password");

          if (!chalRoom.password.equals(authData.password))
            return BMessage.error(request.getType(), "Incorrect Password");
        }

        // Is the person logged into the challenge room
        if (!chalRoom.accountCollection.containsKey(authData.getIDLong()))
        {
          System.err.println("Get Message: Account doesn't exist in room");
          return BMessage.error(request.getType(), "Account doesn't exist in room");
        }
        // Check to see if the account password  and loginID is correct
        String user = request.getRequestHeader().getUser();
        String password = request.getRequestHeader().getPass();

        if (user == null || user.equals("") || password == null || password.equals(""))
          return BMessage.error(request.getType(), "Empty login/password");


        SAccount account = (SAccount)chalRoom.accountCollection.get(authData.getIDLong());

        if (!account.loginID.equals(user)|| !account.password.equals(password))
          return BMessage.error(request.getType(), "Incorrect login/password combination");

        reqState.accountID = authData.getID();
        String type = account.type;

        if (type != null)
          reqState.type = type;

        reqState.designerName   = account.designerName;
        reqState.designerBlurb  = account.designerBlurb;
        reqState.proStatus      = account.proStatus;
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }

      // All is ok
      return null;
    }
  }

 
  // Challenge...
  // -------------------------------------------------------------

  protected static class Challenge extends ValidatedRequestHandler
  {
    public String getType()
      { return "CHAL"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      Connection con = reqState.con;;
      Statement statement = null;
      PreparedStatement pstmt = null;
      ResultSet rs = null;
      try
      {
        con.setAutoCommit(false);
        
        // Get the user information
        BCreateChallengeData challengeData = (BCreateChallengeData)request.getData();

        SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(new Long(challengeData.challengeRoomID));

        // Get both the designer names
        SAccount challengerAcc = (SAccount)challengeRoom.accountCollection.get(challengeData.getIDLong());
        SAccount challengedAcc = (SAccount)challengeRoom.accountCollection.get(new Long(challengeData.challengedAccountID));

        String c1Name = challengerAcc.designerName;
        String c2Name = challengedAcc.designerName;

        String tournamentName =  c1Name + " vs " + c2Name;

        // Make the Tournament Name Oracle Safe
        tournamentName = SServerAccess.parseString(tournamentName);

        // Create a new tournament for the challenge
        String sql = "INSERT INTO TOURNAMENT";
        sql += " (name, description, arena_file_name, status, fight_duration, auto_fight";
        sql += " ,type, clientID, unique_item, current_round)";
        sql += " VALUES ('" + tournamentName + "','Challenge Fight','" + challengeRoom.arena + ".are'";
        sql += " ,'WAITING'," + challengeRoom.maxFightTime + ",'N','EXCFC'," + challengeData.challengedAccountID;
        sql += " ,-1,1)";

        statement = con.createStatement();
        statement.executeUpdate(sql);
        statement.close();

        // Use the name to get the tournament ID
        sql = "SELECT MAX(id) As max_id FROM TOURNAMENT WHERE";
        sql += " name = '" + tournamentName + "' AND type = 'EXCFC'";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);

        rs.next();

        int tournamentID = rs.getInt("max_id");
        rs.close();
        statement.close();

        // Create the challenger tournament Entry
        // Get the next tournament entry ID
        sql = "SELECT TOURNAMENT_ENTRY_IDS.nextval As ID FROM DUAL";

        statement = con.createStatement();
        rs = statement.executeQuery(sql);
        rs.next();
        long tournament_entry_id = rs.getLong("id");
        rs.close();
        statement.close();

        // Create the record in the Tournament_Entry table
        pstmt = con.prepareStatement("INSERT INTO TOURNAMENT_ENTRY (id, tournament_id, bot_name, bot_file, account_id, status, current_round) values (?, ?, ?, empty_blob(), ?,'Y',0)");
        pstmt.setLong(1,tournament_entry_id);
        pstmt.setLong(2,tournamentID);
        pstmt.setString(3,SServerAccess.parseString(challengeData.botName));
        pstmt.setLong(4,challengeData.getID());

        pstmt.execute();
        pstmt.close();

        // Create and add the Tournament
        STournament tournament = new STournament(new Long(tournamentID),challengeData.getIDLong(),
                                                  new Long(challengeData.challengedAccountID), "WAITING",
                                                  c1Name,c2Name, tournament_entry_id,tournamentName,"EXCFC");

        tournament.challengerBotName = challengeData.botName;

        challengeRoom.tournamentCollection.put(tournament.tournamentID,tournament);

        // Create and send the Accept challenge object to the challenged
        BAcceptChallengeData accept = new BAcceptChallengeData(challengeData.getID()
                                                              ,reqState.designerName
                                                              ,challengeData.challengerBot
                                                              ,tournamentID);

        // Send the challenged the challenge
        challengeRoom.notifyObservers(new Long(challengeData.challengedAccountID),accept);

        // Notify everyone in the room of the added challenge
        BTournamentUpdateData tournamentUpdate = new BTournamentUpdateData(tournament.tournamentID.longValue(),
                                                                          BTournamentUpdateData.ADDED,
                                                                          tournament.challengerID,
                                                                          tournament.challengedID,
                                                                          tournament.challengerDesignerName,
                                                                          tournament.challengedDesignerName,
                                                                          tournament.tournamentStatus,
                                                                          tournament.fightProgress,
                                                                          tournament.tournamentName,
                                                                          tournament.fightID,
                                                                          tournament.challengerBotName,
                                                                          tournament.challengedBotName);

        challengeRoom.notifyObservers(null,tournamentUpdate);

        BMessage response = BMessage.ACK("CHAL");

        // Zip the response...
        //services.add(BServices.zip());

        con.commit();
        return response;
      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SChallengeRoomServlet/Challenge: " + sqle1);
          return BMessage.error("CHAL", sqle1.getMessage());
        }

        System.err.println("SChallengeRoomServlet/Challenge: " + e.getMessage());
        return BMessage.error("CHAL", "SChallengeRoomServlet/Challenge: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SChallengeRoomServlet/Challenge:");
        closeStatement(statement,"SChallengeRoomServlet/Challenge:");
        closeStatement(pstmt,"SChallengeRoomServlet/Challenge:");
      }


    }

    public BMessage validate (BRequest request, RequestState reqState)
    {

      try
      {
        //BDebug.print("Challenge!!!");
        // Get the user information
        BCreateChallengeData challengeData = (BCreateChallengeData)request.getData();

        // Does the challenge room exist
        if (!challengeRooms.containsKey(new Long(challengeData.challengeRoomID)))
          return BMessage.error(request.getType(), "Unknown Challenge Room");

        SChallengeRoom chalRoom = (SChallengeRoom)challengeRooms.get(new Long(challengeData.challengeRoomID));

        // Is the challenge room password correct?
        if (chalRoom.password != null)
        {
          if (challengeData.password == null)
            return BMessage.error(request.getType(), "Incorrect Null Password");

          if (!chalRoom.password.equals(challengeData.password))
            return BMessage.error(request.getType(), "Incorrect Password");
        }

        // Is the person logged into the challenge room
        if (!chalRoom.accountCollection.containsKey(challengeData.getIDLong()))
        {
          System.err.println("Challenge: Account doesn't exist in room");
          return BMessage.error(request.getType(), "Account doesn't exist in room");
        }

        // Check to see if the account password  and loginID is correct
        String user = request.getRequestHeader().getUser();
        String password = request.getRequestHeader().getPass();

        if (user == null || user.equals("") || password == null || password.equals(""))
          return BMessage.error(request.getType(), "Empty login/password");

        SAccount account = (SAccount)chalRoom.accountCollection.get(challengeData.getIDLong());

        if (!account.loginID.equals(user)|| !account.password.equals(password))
          return BMessage.error(request.getType(), "Incorrect login/password combination");

        // Are the challenger and the challenged in the same room
        if (!chalRoom.accountCollection.containsKey(new Long(challengeData.challengedAccountID)))
          return BMessage.error(request.getType(), "Challenger not in same room as challenged");

        // Determine if either the challenger or the challenged are already involved in a fight
        Long challengedID = new Long(challengeData.challengedAccountID);
        Long challengerID = challengeData.getIDLong();

        // Loop through all the Tournaments in this challenge room

        Iterator tournamentIt = chalRoom.tournamentCollection.values().iterator();

        while(tournamentIt.hasNext())
        {
          STournament tournament = (STournament)tournamentIt.next();

          // Changed so only the challenged can't take part in a fight
          /*if ((tournament.challengedID.equals(challengedID)
              || tournament.challengedID.equals(challengerID)
              || tournament.challengerID.equals(challengedID)
              || tournament.challengerID.equals(challengerID)) && !tournament.tournamentStatus.equals("FINISHED"))*/

          if ((tournament.challengedID.equals(challengedID)
              || tournament.challengedID.equals(challengerID)) && !tournament.tournamentStatus.equals("FINISHED"))
          {
            return BMessage.error(request.getType(), "Challenger or Challenged already running a Fight");
          }
        }

        reqState.accountID = challengeData.getID();
        String type = account.type;

        if (type != null)
          reqState.type = type;

        reqState.designerName   = account.designerName;
        reqState.designerBlurb  = account.designerBlurb;
        reqState.proStatus      = account.proStatus;
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }

      // All is ok
      return null;
    }
  }

// Challenge...
  // -------------------------------------------------------------

  protected static class AcknowledgeChallenge extends ValidatedRequestHandler
  {
    public String getType()
      { return "ACHA"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      Connection con = reqState.con;;
      Statement statement = null;
      PreparedStatement pstmt = null;
      ResultSet rs = null;
      try
      {
        con.setAutoCommit(false);
        
        // Get the user information
        BAcknowledgeChallengeData ackChallengeData = (BAcknowledgeChallengeData)request.getData();

        SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(new Long(ackChallengeData.challengeRoomID));
        STournament tournament = (STournament)challengeRoom.tournamentCollection.get(ackChallengeData.getIDLong());

        // See if the challenge is ok
        if(ackChallengeData.challengeAccepted && ackChallengeData.botsValid)
        {
          // The challenge is ok
          tournament.challengedBotName = ackChallengeData.challengedBotName;


          // Create the challenged tournament Entry
          // Get the next tournament entry ID
          String sql = "SELECT TOURNAMENT_ENTRY_IDS.nextval As ID FROM DUAL";

          statement = con.createStatement();
          rs = statement.executeQuery(sql);
          rs.next();
          long tournament_entry_id = rs.getLong("id");
          rs.close();
          statement.close();

          // Create the record in the Tournament_Entry table
          pstmt = con.prepareStatement("INSERT INTO TOURNAMENT_ENTRY (id, tournament_id, bot_name, bot_file, account_id, status, current_round) values (?, ?, ?, empty_blob(), ?,'Y',1)");
          pstmt.setLong(1,tournament_entry_id);
          pstmt.setLong(2,tournament.tournamentID.longValue());
          pstmt.setString(3,SServerAccess.parseString(ackChallengeData.challengedBotName));
          pstmt.setLong(4,reqState.accountID);

          pstmt.execute();
          pstmt.close();

          // Create update the tournament
          sql = "UPDATE TOURNAMENT SET status = 'FIGHTING' WHERE id = " + ackChallengeData.getID();
          statement = con.createStatement();
          statement.executeUpdate(sql);
          statement.close();

          // Update the ChallengeRoom tournemant entry
          tournament.tournamentStatus = "Fighting";
          tournament.challengedTEID = tournament_entry_id;

          // Create the fight and set it to fighting
          sql = "INSERT INTO FIGHT (tournament_id, tournament_entry1_id, tournament_entry2_id, round, status,battle_recording,battle_stats1,battle_stats2,fight_client_id)";
          sql = sql + " VALUES (" + tournament.tournamentID + ", " + tournament.challengerTEID + ", " + tournament.challengedTEID + ", 1, 'FIGHTING',empty_blob(),empty_blob(),empty_blob()," + reqState.accountID + ")";
          //System.out.println(sql);
          statement = con.createStatement();
          statement.executeUpdate(sql);
          statement.close();

          // Find the fight ID
          sql = "SELECT id FROM FIGHT WHERE tournament_id = " + tournament.tournamentID;
          statement = con.createStatement();
          rs = statement.executeQuery(sql);
          rs.next();
          long fightID = rs.getLong("id");
          rs.close();
          statement.close();


          // Update the tournament with the fightID
          tournament.fightID = fightID;
          tournament.tournamentStatus = "FIGHTING";
          tournament.timeLastUpdated = System.currentTimeMillis() + 60000;

          // Create the fight Log
          sql = "INSERT INTO FIGHT_CLIENT_LOG (fight_client_id, task_type, fight_id)";
          sql = sql + " VALUES(" + reqState.accountID + ",'FGT'," + fightID + ")";
          statement = con.createStatement();
          statement.executeUpdate(sql);
          statement.close();

          // Inform the members of the challenge room
          BTournamentUpdateData tournamentUpdate = new BTournamentUpdateData(tournament.tournamentID.longValue(),
                                                                          BTournamentUpdateData.UPDATED,
                                                                          tournament.challengerID,
                                                                          tournament.challengedID,
                                                                          tournament.challengerDesignerName,
                                                                          tournament.challengedDesignerName,
                                                                          tournament.tournamentStatus,
                                                                          tournament.fightProgress,
                                                                          tournament.tournamentName,
                                                                          tournament.fightID,
                                                                          tournament.challengerBotName,
                                                                          tournament.challengedBotName);
          challengeRoom.notifyObservers(null,tournamentUpdate);


          // Send the data back to the client
          BStartFightData sf = new BStartFightData(fightID, tournament.tournamentID.longValue(),tournament.challengerDesignerName,
                                                    tournament.challengedDesignerName,tournament.challengerTEID,
                                                    tournament.challengedTEID);

          BMessage response = BMessage.ACK("ACHA");
          response.setData(sf);

          // Zip the response...
          services.add(BServices.zip());

          con.commit();
          return response;
        }
        else
        {
          // Fight can't start for some reason
          BTournamentUpdateData tournamentUpdate = new BTournamentUpdateData(tournament.tournamentID.longValue(),
                                                                          BTournamentUpdateData.DELETED,
                                                                          tournament.challengerID,
                                                                          tournament.challengedID,
                                                                          tournament.challengerDesignerName,
                                                                          tournament.challengerDesignerName,
                                                                          tournament.tournamentStatus,
                                                                          tournament.fightProgress,
                                                                          tournament.tournamentName,
                                                                          tournament.fightID,
                                                                          tournament.challengerBotName,
                                                                          tournament.challengedBotName);

          // Tell the members about to delete the tournament
          challengeRoom.notifyObservers(null,tournamentUpdate);


          // Delete the tournament
          challengeRoom.tournamentCollection.remove(tournament.tournamentID);

          // Delete the tournament from the database
          deleteTournament(con, tournament.tournamentID.longValue(),
                                  tournament.fightID,tournament.challengerTEID, tournament.challengedTEID);

          // Inform the room/parties
          BMessage response = BMessage.ACK("ACHA");
          // Zip the response...
          services.add(BServices.zip());

          con.commit();
          return response;
        }
      }
      catch (Exception e)
      {
        try
        {
          con.rollback();
        }
        catch (SQLException sqle1)
        {
          System.err.println("SChallengeRoomServlet/AcknowledgeChallenge: " + sqle1);
          return BMessage.error("CHAL", sqle1.getMessage());
        }

        System.err.println("SChallengeRoomServlet/AcknowledgeChallenge: " + e.getMessage());
        return BMessage.error("SCHALTI", "SChallengeRoomServlet/AcknowledgeChallenge: " + e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SChallengeRoomServlet/AcknowledgeChallenge:");
        closeStatement(statement,"SChallengeRoomServlet/AcknowledgeChallenge:");
        closeStatement(pstmt,"SChallengeRoomServlet/AcknowledgeChallenge:");
      }


    }

    public BMessage validate (BRequest request, RequestState reqState)
    {

      try
      {
        // Get the user information
        BAcknowledgeChallengeData ackChallengeData = (BAcknowledgeChallengeData)request.getData();

        // Does the challenge room exist
        if (!challengeRooms.containsKey(new Long(ackChallengeData.challengeRoomID)))
          return BMessage.error(request.getType(), "Unknown Challenge Room");

        SChallengeRoom chalRoom = (SChallengeRoom)challengeRooms.get(new Long(ackChallengeData.challengeRoomID));

        // Is the challenge room password correct?
        if (chalRoom.password != null)
        {
          if (ackChallengeData.password == null)
            return BMessage.error(request.getType(), "Incorrect Null Password");

          if (!chalRoom.password.equals(ackChallengeData.password))
            return BMessage.error(request.getType(), "Incorrect Password");
        }

        // Is the person logged into the challenge room
        if (!chalRoom.accountCollection.containsKey(new Long(ackChallengeData.accountID)))
        {
          System.err.println("Acknowledge Challenge: Account doesn't exist in room");
          return BMessage.error(request.getType(), "Account doesn't exist in room");
        }

        // Check to see if the account password  and loginID is correct
        String user = request.getRequestHeader().getUser();
        String password = request.getRequestHeader().getPass();

        if (user == null || user.equals("") || password == null || password.equals(""))
          return BMessage.error(request.getType(), "Empty login/password");


        SAccount account = (SAccount)chalRoom.accountCollection.get(new Long(ackChallengeData.accountID));

        if (!account.loginID.equals(user)|| !account.password.equals(password))
          return BMessage.error(request.getType(), "Incorrect login/password combination");

        // Are the challenger and the challenged in the same room
        if (!chalRoom.accountCollection.containsKey(new Long(ackChallengeData.accountID)))
          return BMessage.error(request.getType(), "Challenger not in same room as challenged");


        reqState.accountID = ackChallengeData.accountID;
        String type = account.type;

        if (type != null)
          reqState.type = type;

        reqState.designerName   = account.designerName;
        reqState.designerBlurb  = account.designerBlurb;
        reqState.proStatus      = account.proStatus;
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }

      // All is ok
      return null;
    }
  }

  // Send Message
  // -------------------------------------------------------------

  protected static class SendMessage extends ValidatedRequestHandler
  {
    public String getType()
      { return "SMES"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      BCRWrapperData wrapperData = (BCRWrapperData)request.getData();

      if (wrapperData.challengeData.getType() == BDataTypes.MESSAGE)
      {
        // Set the DesignerName
        ((BMessageData)wrapperData.challengeData).designerName = reqState.designerName;
        ((BMessageData)wrapperData.challengeData).senderAccountID = reqState.accountID;

        if (wrapperData.destination == wrapperData.TO_ACCOUNT)
          ((BMessageData)wrapperData.challengeData).isWhisper = true;
      }

      // Send the message to the correct recipient(s)

      if (wrapperData.destination == wrapperData.SPECIFIED_CHALLENGE_ROOM)
      {
        // Send the message to the specific challenge room
        SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(wrapperData.challengeRoomID);
        challengeRoom.notifyObservers(null,wrapperData.challengeData);
      }
      else if (wrapperData.destination == wrapperData.TO_ACCOUNT)
      {
        SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(wrapperData.challengeRoomID);
        challengeRoom.notifyObservers(wrapperData.getIDLong(), wrapperData.challengeData);
      }
      else if (wrapperData.destination == wrapperData.ALL_CHALLENGE_ROOMS)
      {
        // Broadcast to all challenge rooms
        Iterator cRooms = challengeRooms.values().iterator();

        while(cRooms.hasNext())
        {
          SChallengeRoom cRoom = (SChallengeRoom)cRooms.next();
          cRoom.notifyObservers(null,wrapperData.challengeData);
        }
      }
      else
      {
        // Unknown destination specified
        BMessage response = BMessage.NAK("Unknown destination specified");
        return response;
      }

      // Zip the response...
      //services.add(BServices.zip());

      BMessage response = BMessage.ACK("SMES");
      return response;
    }

    public BMessage validate (BRequest request, RequestState reqState)
    {
      try
      {
        // Get the user information
        BCRWrapperData wrapperData = (BCRWrapperData)request.getData();

        String errorMessage = null;

        // Determine if there are any error messages

        // Does the challenge room exist

        if (!challengeRooms.containsKey(wrapperData.challengeRoomID))
        {
          errorMessage = "Unknown Challenge Room";
        }
        else if (wrapperData.destination == wrapperData.ALL_CHALLENGE_ROOMS)
        {
          errorMessage = "No permission to talk to all Rooms";
        }
        else if (wrapperData.destination == wrapperData.TO_ACCOUNT)
        {
          // If the recipient and the sender are in the same challenge
          // room
          SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(wrapperData.challengeRoomID);

          if (!challengeRoom.accountCollection.containsKey(wrapperData.getIDLong()))
          {
            errorMessage = "Sender not in same room as recipient";
          }
        }

        if (errorMessage == null)
        {
          SChallengeRoom chalRoom = (SChallengeRoom)challengeRooms.get(wrapperData.challengeRoomID);
          String user = request.getRequestHeader().getUser();
          String password = request.getRequestHeader().getPass();

          // Is the challenge room password correct?
          if (chalRoom.password != null)
          {
            if (wrapperData.password == null)
            {
              errorMessage = "Incorrect Null Password";
            }
            else if (!chalRoom.password.equals(wrapperData.password))
            {
              errorMessage = "Incorrect Password";
            }
          }
          else if (!chalRoom.accountCollection.containsKey(wrapperData.accountID))
          {
            System.err.println("Send Message: Account doesn't exist in room");
            errorMessage = "Account doesn't exist in room";
          }
          else if (user == null || user.equals("") || password == null || password.equals(""))
          {
            errorMessage = "Empty login/password";
          }

          if(errorMessage == null)
          {
            SAccount account = (SAccount)chalRoom.accountCollection.get(wrapperData.accountID);

            if (!account.loginID.equals(user)|| !account.password.equals(password))
            {
              errorMessage = "Incorrect login/password combination";
            }
            else
            {
              reqState.accountID = wrapperData.accountID.longValue();
              String type = account.type;

              if (type != null)
                reqState.type = type;

              reqState.designerName   = account.designerName;
              reqState.designerBlurb  = account.designerBlurb;
              reqState.proStatus      = account.proStatus;
            }
          }
        }

        // If there are any validation issues then see if the user has admin permissions
        if (errorMessage != null)
        {
          // Verify the account details
          if (super.validate (request,reqState) == null)
          {
            // Account details ok see they have ADMIN permissions
            if (hasPermission("ADMIN",reqState.con,wrapperData.accountID.longValue()))
              return null;
            else
              return BMessage.error(request.getType(), errorMessage);
          }
          else
          {
            // Can't validate user
            return BMessage.error(request.getType(), "Incorrect login/password combination");
          }
        }
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }

      // All is ok
      return null;
    }
 }


  // Enter Challenge Room handler
  // -------------------------------------------------------------

  protected static class EnterChallengeRoom extends ValidatedRequestHandler
  {
    public String getType()
      { return "ECR"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

      BCRAuthenticationData authData = (BCRAuthenticationData)request.getData();

      //  Find the challenge room to enter into
      SChallengeRoom challengeRoom = (SChallengeRoom)challengeRooms.get(new Long(authData.challengeRoomID));

      // Create a new account and place it into the correct room
      challengeRoom.addAccount(new SAccount(new Long(reqState.accountID),request.getPassword(),request.getUser(),reqState.designerName,reqState.type,reqState.designerBlurb, reqState.proStatus));

      // Inform everyone else in the room that someone has entered
      BEnterRoomData enterRoomData  = new BEnterRoomData(authData.getID());
      enterRoomData.challengeRoomID = challengeRoom.roomNumber;
      enterRoomData.designerName    = reqState.designerName;
      enterRoomData.designerBlurb   = reqState.designerBlurb;
      enterRoomData.proStatus       = reqState.proStatus;


      //BDebug.print("Entering the challenge Room: " + enterRoomData.challengeRoomID);
      //BDebug.print("Account entering the challenge Room: " + enterRoomData.getID());
      //BDebug.print("Designer Blurb entering the challenge Room: " + enterRoomData.designerBlurb);

      challengeRoom.notifyObservers(null,enterRoomData);

      BMessage response = BMessage.ACK("ECR");
      // Zip and encrypt the response...
      //services.add(BServices.zip());

      return response;
    }

 public BMessage validate (BRequest request, RequestState reqState)
    {

      try
      {

        if (super.validate(request, reqState) != null)
          return BMessage.error(request.getType(), "Incorrect Login Password Combination");

        // Get the user information
        BCRAuthenticationData authData = (BCRAuthenticationData)request.getData();

        // Make sure that the user doesn't already exist in another challenge room

        Iterator cRooms = challengeRooms.values().iterator();
        boolean found = false;
        SChallengeRoom challengeRoom = null;

        while(cRooms.hasNext() && !found)
        {
          SChallengeRoom cRoom = (SChallengeRoom)cRooms.next();
          Map map = cRoom.accountCollection;
          if(map.containsKey(authData.getIDLong()))
          {
            // Found the challenge room for this account
            found = true;
            challengeRoom = cRoom;
          }
        }

        if (found)
        {
          // If the user is already logged into a challenge room different from the
          // one they were logged into
          // then don't return an error just log them out automatically
          // and inform the logout
          // challenge room

          if (challengeRoom.roomNumber != authData.challengeRoomID)
          {
            challengeRoom.logoutAccount(authData.getIDLong());
            BExitRoomData exitRoomData = new BExitRoomData(authData.getID());
            exitRoomData.challengeRoomID = challengeRoom.roomNumber;

            challengeRoom.notifyObservers(null,exitRoomData);
          }

          //return BMessage.error(request.getType(), "Already Logged into Challenge Room : " + challengeRoom.roomNumber);
        }

        // Does the challenge room exist
        if (!challengeRooms.containsKey(new Long(authData.challengeRoomID)))
          return BMessage.error(request.getType(), "The room you tried to enter no longer exists.  Click Ok to refresh the room list.");

        SChallengeRoom chalRoom = (SChallengeRoom)challengeRooms.get(new Long(authData.challengeRoomID));

        // Is the challenge room full?
        if (chalRoom.getNoOfPeopleInRoom() >= maxNumberOfParticipants)
          return BMessage.error(request.getType(), "Challenge Room Full");

        // Is the challenge room password correct?
        if (chalRoom.password != null)
        {
          if (authData.password == null)
            return BMessage.error(request.getType(), "The password you have entered is incorrect - please try again.");

          if (!chalRoom.password.equals(authData.password))
            return BMessage.error(request.getType(), "The password you have entered is incorrect - please try again.");
        }

      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }

      // All is ok
      return null;
    }
  }

  // Login request handler...
  // -------------------------------------------------------------

  protected static class LoginHandler extends ValidatedRequestHandler
  {
    public String getType()
      { return "LOG"; }

    public BMessage respond
      (BRequest request, RequestState reqState, BServiceSet services)
    {

       BLoginData loginData = new BLoginData(-1);
       loginData.accountID = reqState.accountID;

       Calendar cal = Calendar.getInstance(TimeZone.getDefault());
       cal.setTime(new java.util.Date(System.currentTimeMillis()));
    

       loginData.serverTime = cal;

       return BMessage.ACK("LOG", loginData);

      /*
      BRequestHeader header =
        request.getRequestHeader();

      // TODO: Lookup user in database...
      boolean isValidUser =
        header.getUser().equals("hc");

      if (isValidUser)
        return BMessage.ACK("LOG");
      else
        return BMessage.error("LOG", "Not a valid user."); */
    }
  }

 

  private static boolean hasPermission(String permission, Connection con, long accountID)
  {

    Statement statement = null;
    ResultSet rs = null;
    try
    {
      String sql = "SELECT count(*) As count";
      sql += " FROM PERMISSION P, ACCOUNT_PERMISSION AP";
      sql += " WHERE AP.permission_id = P.id";
      sql += " AND P.permission = '" + permission + "'";
      sql += " AND AP.account_ID = " + accountID;

      statement = con.createStatement();

      rs = statement.executeQuery(sql);
      rs.next();
      int count = rs.getInt("count");
      rs.close();
      statement.close();

      if (count !=0)
      {
        // Has the required permission
        return true;
      }
      else
      {
        // Doesn't have the required permission
        return false;
      }
    }
    catch (Exception e)
    {
      System.err.println(e.getMessage());
      return false;
    }
    finally
    {
      closeResultSet(rs,"SGateway/hasPermission");
      closeStatement(statement,"SGateway/hasPermission");
    }

  }

// MessageSource acts as the source for new messages.
// Clients interested in receiving new messages can
// observe this object.
static class MessageSource extends Observable
{
  public void sendMessage(BData message)
  {
    setChanged();
    notifyObservers(message);
  }
}

// MessageSink acts as the receiver of new messages.
// It listens to the source.
static class MessageSink implements Observer
{

  // set by update() and read by getNextMessage()
  BData message = null;

  Long accountID;
  // Called by the message source when it gets a new message

  public MessageSink (long accountID)
  {
    this.accountID = new Long(accountID);
  }

  synchronized public void update(Observable o, Object arg)
  {
    // Get the new message
    message = (BData)arg;

    // Wake up our waiting thread
    notify();
  }

  public boolean equals(Object o)
  {
    if ((o != null) && (o instanceof SChallengeRoomServlet.MessageSink))
      return accountID.equals(((SChallengeRoomServlet.MessageSink)o).accountID);
    else
      return false;

  }

  // Gets the next message sent out from the message source
  synchronized public BData getNextMessage(SChallengeRoom source)
  {

    // If there are messages waiting to be delivered for this
    // account deliver them immediately


    SAccount account = (SAccount)source.accountCollection.get(accountID);
    if (!account.isEmptyOfMessages())
    {
      BDataSet dataSet = new BDataSet();

      dataSet.add(account.getAndClearMessages());
      message = dataSet;

      return message;

    } 

    // Tell source we want to be told about new messages
    source.addObserver(this);

    // Wait until our update() method receives a message
    while (message == null)
    {
      try
      {
        wait();
      }
      catch (Exception ignored)
      {
      }
    }

    // Tell source to stop telling us about new messages
    // Don't delete it here, rather do it at the source
    // cause doing it here will result in lost messages
    // source.deleteObserver(this);

    // Now return the message we received
    // But first set the message instance variable to null
    // so update() and getNextMessage() can be called again.
    //String messageCopy = message;
    //message = null;
    //return messageCopy;

    return message;
  }
}
  /*
  // getNextMessage() returns the next new message.
  // It blocks until there is one.
  public static BData getNextMessage(SChallengeRoom challengeRoom)
  {

    // Find the challenge room where the account exists

    Iterator cRooms = challengeRooms.values().iterator();
    boolean found = false;
    SChallengeRoom challengeRoom = null;

    while(cRooms.hasNext() && found == false)
    {
      SChallengeRoom cRoom = (SChallengeRoom)cRooms.next();
      Map map = cRoom.accountCollection;
      if(map.containsKey(new Long(accountID)))
      {
        // Found the challenge room for this account
        found = true;
        challengeRoom = cRoom;
      }

    }

    if (challengeRoom == null)
      BDebug.print("**Couldn't Find Challenge Room");

    // Create a message sink to wait for a new message from the
    // message source.
    return new MessageSink(accountID).getNextMessage(challengeRoom);
  }*/

  /*
  // broadcastMessage() informs all currently listening clients that there
  // is a new message.  Causes all calls to getNextMessage() to unblock.
  public static void broadcastMessage(BData message)
  {


    // Send the message to all the HTTP-connected clients by giving the
    // message to the message source
    source.sendMessage(message);
  } */
}



