package roboforge;

import roboforge.net.*;
import roboforge.net.data.*;
import roboforge.net.client.*;
import roboforge.net.server.*;
import roboforge.net.services.*;

import roboforge.util.io.*;
import roboforge.util.data.*;

import iaik.utils.CryptoUtils;
import iaik.security.provider.IAIK;

import oracle.sql.*;
import oracle.jdbc.driver.*;

//import java.lang.reflect.Array;
import java.io.*;
import java.util.*;
//import java.security.*;
import java.sql.*;
import java.util.zip.*;

//import javax.crypto.Cipher;
import javax.servlet.*;
import javax.servlet.http.*;
//import javax.swing.ImageIcon;
//import roboforge.server.util.*;




public abstract class SGatewayAbstract extends SHttpServletJXGB
{

  // Member variables...
  // -------------------------------------------------------------

  protected BServerStack stack;
    // The server-side network stack; provides messaging services
    // for this servlet.

  protected HashMap handlers;
    // A collection of objects that respond to specific request types.
    // For a valid request-type, e.g. "GOT", there is exactly one
    // handler available to process that request.

  protected static TimeZone tz =
    TimeZone.getTimeZone(SGlobal.TIME_ZONE);



  // Servlet info method...
  // -------------------------------------------------------------

  public abstract String getServletInfo();



  // Servlet Initialization...
  // -------------------------------------------------------------

  public void init(ServletConfig config) throws ServletException
  {
    super.init(config);

    System.out.println("USER DIRECTORY: " + System.getProperty("user.dir"));

    try
    {
      // Add IAIK as the provider
      IAIK.addAsProvider(false);

      // Initialize the network stack.
      stack = new BServerStack();

      // Setup the handler collection.
      initializeHandlers();
    }
    catch(Exception ex)
    {
      ex.printStackTrace();
      throw new RuntimeException();
    }
  }



  // Request Handler initialization...
  // -------------------------------------------------------------

  protected abstract void initializeHandlers();

  protected void register(RequestHandler h)
    { handlers.put(h.getType(), h); }



  // Request Service method...
  // -------------------------------------------------------------

  public void service(HttpServletRequest HTTPRequest, HttpServletResponse HTTPResponse)
    throws ServletException, IOException
  {
    // Declare local variables...
    Connection    con       = null;
    BClientTarget client    = null;
    BRequest      request   = null;
    BMessage      response  = null;
    BServiceSet   services  = null;

    try
    {

      // Test code as in the future we way use the
      // type to redirect a request to another URL
      //String enum = HTTPRequest.getQueryString();
      //System.out.println(enum);
      
      // Retrieve a database connection...
      con = getDatabaseConnection();

      // Set the request State
      RequestState reqState = new RequestState(con);

      // Initialize client messaging target...
      client = new BClientTarget(HTTPRequest, HTTPResponse);

      // Receive request message from client...
      request = stack.receive(client);

      // Determine server response...
      services = new BServiceSet();
      response = generateResponse(request, reqState, services);
    }
    catch (Exception e)
    {
      // TODO: Improve error handling...
      String type = (request != null) ? request.getType() : "ERR";
      response    = BMessage.error(type, e.getMessage());
    }
    finally
    {
      try
      {
        if (con!= null)
        {
          con.commit();
          con.setAutoCommit(true);
          myBroker.freeConnection(con);
        }

      }
      catch(Exception sqle)
      {
        System.err.println(sqle.getMessage());
        String type = (request != null) ? request.getType() : "ERR";
        response    = BMessage.error(type, sqle.getMessage());
      }
    }

    try
    {
      // Send response to the client...
      /*if (response.getType().equals("HIS"))
        try
        {
          Thread.sleep(120000);
        }
        catch (InterruptedException cie)
        {
          System.err.println("Got InterruptedException Error");
        }*/



      stack.send(client, response, services);
    }
    catch (Exception e)
    {
      // TODO: Handle unexpected error during response transmission...
      System.err.println("Error during response");
      System.err.println(e.getMessage());
    }
  }


  // Database connection retrieval method...
  // -------------------------------------------------------------

  Connection getDatabaseConnection()
  {
    try
      { return myBroker.getConnection(); }
    catch (Exception e)
      { System.err.println(e.getMessage()); }

    return null;
  }


  // Response method...
  // -------------------------------------------------------------

  public BMessage generateResponse
    (BRequest request, RequestState reqState, BServiceSet services)
  {
    // Get the type of request...
    String type = (request != null) ? request.getType() : null;

    // Locate an appropriate handler... [O(1) lookup hopefully]
    RequestHandler handler =
      (RequestHandler) handlers.get(type);

    // Pass off request to handler.
    if (handler == null)
      return BMessage.error(type, "Unrecognized request.");
    else
      return handler.handle(request, reqState, services);
  }



  // Request Handler base class...
  // -------------------------------------------------------------

  protected static abstract class RequestHandler
  {
    // NB: Handlers are shared across multiple service()
    //     threads, so they should probably be stateless.

    public abstract String getType();
      // Returns the type of request that this handler deals with.

    public abstract BMessage handle
      (BRequest request, RequestState reqState, BServiceSet services);
      // For the given request, return a response message.
      // Also add any required data transmission services
      // to the supplied service collection, 'services'.
  }


  // Class to hold the connection and the id of the client
  // -------------------------------------------------------------
  protected static class RequestState
  {
    public Connection con;
    public long accountID;
    public String designerName = "";
    public String designerBlurb = "";
    public String proStatus;

    // Used by SFarmFeeder to determine the type of fight that can be sent to
    // the SFarmClient default value is EXCFC - External Challenge Farm Client
    // Other possible values are:       INCFC - Internal Challenge Farm client
    //                                  INTFC - Internal Tournament Farm Client
    // This one is unlikely to be used due to security issues
    //                                  EXTFC - External Tournament Farm Client
    // These types dictate the type of fights and verifications that can be given to
    // the farm client
    public String type = "EXCFC";

    public RequestState (Connection con)
    {
      this.con = con;
    }
  }

  protected static abstract class ValidatedRequestHandler extends RequestHandler
  {
    public BMessage handle
      (BRequest request, RequestState reqState, BServiceSet services)
    {
      BMessage response =
        validate(request, reqState);

      if (response == null)
        response = respond(request, reqState , services);

      return response;
    }

    public BMessage validate (BRequest request, RequestState reqState)
    {
      Statement statement = null;
      ResultSet rs        = null;

      try
      {
        String user = request.getRequestHeader().getUser();
        String password = request.getRequestHeader().getPass();

        if (user == null || user.equals("") || password == null || password.equals(""))
          return BMessage.error(request.getType(), "Empty login/password");

        statement = reqState.con.createStatement();

        String sql =
          "Select status, id, type, designer_name, blurb, professional_status FROM ACCOUNT WHERE LOWER(loginID) = LOWER('" +
          user + "') " + "AND LOWER(password) = LOWER('"+ password + "')";

        rs = statement.executeQuery(sql);

        if (!rs.next())
          // No rows returned invalid login/password combination
          return BMessage.error(request.getType(), "Incorrect login/password combination");
        else
        {
          String status = rs.getString("status");

          if(!status.equals("OPEN"))
            // Correct login/password combination but account doesn't have open
            // status so can't login
            return BMessage.error(request.getType(), "Account Not Open. Account Status: " + status);
        }
        reqState.accountID = rs.getInt("id");
        String type = rs.getString("type");
        String blurb = rs.getString("blurb");
        String proStatus = rs.getString("professional_status");

        if (type != null)
          reqState.type = type;

        if (blurb != null)
          reqState.designerBlurb = blurb;

        if (proStatus != null)
          reqState.proStatus = proStatus;

        reqState.designerName = rs.getString("designer_name");
      }
      catch (Exception e)
      {
        System.err.println("BMessage/Validate: "+ e);
        return BMessage.error(request.getType(), e.getMessage());
      }
      finally
      {
        closeResultSet(rs,"SGateway/ValidatedRequestHeader");
        closeStatement(statement,"SGateway/ValidatedRequestHeader");
      }

      // All is ok
      return null;
    }

    public abstract BMessage respond(BRequest request, RequestState reqState, BServiceSet services);

  }



  protected static void closeResultSet(ResultSet rs, String location)
  {
    try
    {
      if (rs != null)
        rs.close();
    }
    catch (Exception e)
    {
      System.err.println("SGateway/closeResultSet: "+ location + ": " + e);
    }

  }

  protected static void closeStatement(Statement statement, String location)
  {
    try
    {
      if (statement != null)
        statement.close();
    }
    catch (Exception e)
    {
      System.err.println("SGateway/closeStatement: "+ location + ": " + e);
    }

  }
}



