package roboforge.ai.actions;

import javax.vecmath.*;
import roboforge.*;

public class BBotFallingAction implements BAction
{
  BBot bot;
  Vector3f axis;

  float currentVelocity = 0f;  // rad per sec
  float acceleration = 0.8f;  // rad/s^2
  float totalTilted = 0f;

  public BBotFallingAction(BBot bot, Vector3f axis)
  {
    this.bot = bot;
    this.axis = axis;
  }

  public void start()
  {
    bot.getChassis().terminateChassisMove(true);
  }

  public boolean execute(float s)
  {
    currentVelocity = currentVelocity + (acceleration  * s);
    totalTilted = totalTilted + (currentVelocity * s);
    bot.getChassis().setOrientation(axis, totalTilted);

    return (totalTilted < BPhysics.PI);

  }

  public void stop()
  {
    bot.setBotAlive(false);
    bot.setTerminationString(bot + " tipped.");
    /*bot.clearActions();
    bot.clearTippingFlag();
    bot.getChassis().terminateChassisMove(true);  */


  }

  public boolean conflictsWith(BAction a)
  {
    if (a == this)
      return true;
    else if (a instanceof BBotFallingAction)
      return true;
    else if (a instanceof BOrientAction)
      return true;
    else if (a instanceof BChassisAction )
      return true;
    else if ( a instanceof BMove )
    {

      BMove m = (BMove) a;
      if ( m.getContainsChassisMove() )
       return true;


    }

    return false;

  }

  public boolean overrides(BAction a)
  {
    if (a == this)
      return false;
    else if (a instanceof BBotFallingAction)
      return false;
    else
      return true;
  }


  public String getName()
    { return "Bot Falling "; }

  public boolean isValid()
    { return true;}


}