package roboforge.ai.actions;

import roboforge.*;
import java.io.*;

public class BChassisAction implements BAction, Serializable
{

  static final long serialVersionUID = -1L;

  BMove chassisMove;
  BChassis theChassis;
  float distance;  // meters
  private transient boolean active = false;

  public BChassisAction(BChassisAction other)
  {
    this (other.chassisMove, other.theChassis, other.distance);
  }

  public BChassisAction(BMove chassisMove, BChassis theChassis, float distance)
  {
    this.chassisMove = chassisMove;
    this.theChassis = theChassis;
    setDistance(distance);
  }

  public void start()
  {
    theChassis.setNewChassisMove(chassisMove, distance, BChassis.ADD_MODE_PASSIVE, null);
    active = true;
  }

  public boolean getActive()
    { return active; }

  public void setChassisMove(BMove chassisMove)
  { this.chassisMove = chassisMove;  }

  public BMove getChassisMove()
  { return chassisMove;  }

  public void setDistance(float distance)
  {  this.distance = java.lang.Math.max(distance,0);   }

  public float getDistance()
  { return distance;  }

  public BChassis getChassis()
  { return theChassis; }

  public void setChassis(BChassis chassis)
  { this.theChassis = chassis; }

  public boolean execute(float s)
  {
    // if the chassis is currently conflicting then it is executing this move
    
    return theChassis.conflictsWith(chassisMove);
  }

  public void stop()
  {
    active = false;
    if ( theChassis.getCurrentChassisMove() ==  chassisMove )
      theChassis.returnToStand();
  }

  public boolean conflictsWith(BAction a)
  {
    //BDebug.print("BChassisAction.conflictsWith: Start "+toString());
    boolean c = false;

    if (a == this)
      c = true;
    else if (a instanceof BChassisAction)
      c = true;
    else if (a instanceof BBotFallingAction )
      c = true;
    else if ( a instanceof BMove )
    {

      BMove m = (BMove) a;
      if ( m.getContainsChassisMove() )
        c = true;
      else if ( m.getIsResultantMove() )
        if (m.containsChassisJoints() )
          c = true;

    }
    else if (a instanceof BOrientAction)
      return true;

   // if (!c)
    //  c =  theChassis.conflictsWith(chassisMove);


    return c;
  }

  public boolean overrides(BAction a)
  {
    if (a instanceof BOrientAction)
      return true;

    return false;
  }

  public String getName()
  { return  "Chassis: " + chassisMove;  }

  public String toString()
  { return getName() ; }

   public boolean isValid()
  {
    return chassisMove.isValid();
  }



}