package roboforge.ai.decisions;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BNameable;
import roboforge.ai.actions.*;

import roboforge.ui.components.HSBImageIcon;
import roboforge.util.graphics.*;
import roboforge.ui.components.BBorderedIcon;
import roboforge.ui.ai.BDecisionTreeIcon;


import java.awt.Color;
import java.io.Serializable;

import javax.swing.Icon;



public abstract class BDecisionTreeNode
  extends    BVisitableTreeNode
  implements BNameable, Cloneable
{

  // Class variables...
  // -----------------------------------------------------------

  // protected static TreeCellRenderer renderer
  //   = new DefaultTreeCellRenderer();



  // Internal variables...
  // -----------------------------------------------------------

  protected String name;

  // Transient variables...
  // -------------------------------------------------------------

  protected transient String defaultName;
  protected boolean closed = false;
  protected transient Integer primaryKey;
  protected boolean visited; // only valid in the battle screen



  // Constructors...
  // -----------------------------------------------------------

  public BDecisionTreeNode()
  {
    super();
    init();
  }

  public BDecisionTreeNode(BDecisionTreeNode node)
  {
    super();

    if (node.name != null)
      this.name = new String(node.name);
    this.closed = node.closed;

    init();
  }


  // Initialization method.
  // -----------------------------------------------------------

  public void init()
    // Initializes the tree rooted at this node.
    // Must be called after de-serialization.
  {
    int n = getChildCount();

    BDecisionTreeNode node;

    for (int i=0; i<n; i++)
    {
      node = (BDecisionTreeNode) getChildAt(i);
      node.init();
    }
  }


  // Accessor methods...
  // -----------------------------------------------------------
  public void setPrimaryKey(Integer i)
  {
    primaryKey = i;
    //BDebug.print ("BDecisionTreeNode.setPrimaryKey " + this.toString() + " : " + i);

  }

  public void setVisited(boolean b)
  {
    visited = b;
  }

  public boolean getVisited()
  {
    return visited;
  }


  public Integer getPrimaryKey()
    { return primaryKey; }

  public void setName(String name)
  {
    this.name = name;

    if (name != null)
      if (name.equals(""))
        this.name = null;

    setUserObject(name);

  //  BDebug.print ("BDecisionTreeNode.setName " + this.name);
  }

  public String getName()
    { return getName(true); }

  public String getName(boolean useDefaultWhenNull)
  {
    if (name != null)
      return name;
    else if (useDefaultWhenNull)
      return getDefaultName();
    else
      return null;
  }

  public String toString()
    { return getName(); }


  protected void setDefaultName(String name)
  {
    this.defaultName = name;

    if (name == null)
      setUserObject(defaultName);
  }

  protected String getDefaultName()
  {
    if (defaultName != null)
      return defaultName;
    else
      return generateDefaultName();
  }

  protected void setDefaultName()
    { setDefaultName(generateDefaultName()); }

  protected String generateDefaultName()
    { return ""; }

  public void prune()
  {
    // method to be overridden by subclasses.
    // Validate yourself, and take the appropriate action if you are no longer valid
  }

  public boolean isValid()
  {
    return true;
    // method to be overridden by subclasses.
    // returns wether this node is still Valid.
  }


  // Child maintenance methods...
  // -----------------------------------------------------------

  public boolean isRemoveable()
  {
    BDecisionTreeNode parent =
      (BDecisionTreeNode) getParent();

    if (parent != null)
      return parent.canRemoveChildren();
    else
      return false;
  }

  public boolean canAddChildren()
    { return false; }

  public boolean canRemoveChildren()
    { return false; }




  // UI methods...
  // -----------------------------------------------------------

  // public TreeCellRenderer getCellRenderer()
  //   { return renderer; }

  public abstract Icon getIcon();

  public abstract Icon getSelectedIcon();




  // UI helper methods...
  // -----------------------------------------------------------
  private static int BORDER_GAP = 6;

  protected static BBorderedIcon createTreeIcon(String name, Color borderColor)
  {
    if (borderColor == null)
      borderColor = BGlobal.colors.getDarkShade();
      
    if (name == null)
      return  new BDecisionTreeIcon( borderColor, null,  BORDER_GAP, true );
    else
      return new BDecisionTreeIcon( borderColor, BGraphics.getHSBIcon(name),  BORDER_GAP, true ) ;
  }

  protected static BBorderedIcon createTreeIcon(String name, Color iconColor, Color borderColor)
    {return new BDecisionTreeIcon( borderColor, BGraphics.getHSBIcon(name, iconColor), BORDER_GAP, true ) ;}



  // State storage / retrieval...
  // -----------------------------------------------------------

  public void setState(Object o)
  {
    DecisionTreeNodeState state =
      (DecisionTreeNodeState) o;

    //BDebug.print ( "BDecisionNode.setState name " + state.name);

    this.name = state.name;
    this.closed = state.closed;
    setUserObject(name);

  }

  public Object getState()
    { return new DecisionTreeNodeState(this); }

  protected static class DecisionTreeNodeState implements Serializable
  {
    static final long serialVersionUID = -1L;
    protected String name;
    protected boolean closed;

    public DecisionTreeNodeState(BDecisionTreeNode node)
    {
        this.name = node.name;
        this.closed = node.closed;
    }
  }


  public boolean isClosed()
    { return closed; }

  public void setClosed(boolean b )
    { closed = b; }

  public boolean isCloseable()
  	{ return false; }
  	// SJS [09/04/2001] Returns whether this node can be closed.


   public void setActor(BActor actor)
    {}

}

