package roboforge.ai.decisions;

import roboforge.BGlobal;
import roboforge.BZones;
import roboforge.BDebug;
import roboforge.BZone;
import roboforge.BBot;
import roboforge.ai.values.BString;
import roboforge.ui.components.HSBImageIcon;
import roboforge.ai.*;
import roboforge.ai.actions.*;
import roboforge.util.parser.*;
import roboforge.ai.decisions.*;
import java.util.Enumeration;
import javax.swing.Icon;
import roboforge.ui.components.BBorderedIcon;
import javax.swing.event.*;




public class BZoneGroupNode extends BGroupNode
{
   protected static BBorderedIcon icon =
    createTreeIcon("ai/zone_group_18.gif", BGlobal.colors.getAIGroupColor());

   protected BZones zones;
   protected BDecisionTreeAI ai;

  public BZoneGroupNode()
  {

    super();
    setName("Zone auto-group");

  }

  public BZoneGroupNode(BZones zones, BDecisionTreeAI ai)
  {
    super();
    setZones( zones);
    this.ai = ai;
    setName("Zone auto-group");

  }

  public BZoneGroupNode(BZoneGroupNode node)
  {
    super(node);
    setZones(node.zones);
    this.ai = node.ai;
    setName("Zone auto-group");
  }

  public Object clone()
  {
    BZoneGroupNode gz = new BZoneGroupNode(this);
    gz.createAllChildren();

    return gz;
  }

  protected void setZones(BZones zones)
  {

     this.zones = zones;
     if (this.zones != null)
      this.zones.addListDataListener(new ZonesListener() );

  }
  public String getDescription()
  { return  "This node type will automatically add, remove and update when you edit this Bots zones"; }


  public boolean canAddChildren()
    { return false; }


  protected void createAllChildren()
  {
    if (zones != null)
      for (int i = 0; i < zones.size() ; i++)
      {

        BZone z = (BZone) zones.getElementAt(i);
        addChildForZone(z);

      }

  }

  protected void addChildForZone(BZone z)
    {  addChildForZone(z, this.getChildCount()) ;}

  protected void addChildForZone(BZone z, int index)
  {
      BSimpleDecisionNode zNode = new BSimpleDecisionNode(".isActive", ".isActive", new BString(z.getName()), "ai/zone_18.gif", "Zone Auto Group");
      String exp = "[" + BZones.FULL_WORLDSTATE_CODE + z.getName() + ".isActive]";
      zNode.setName(z.getName() );
      zNode.add( new BNOPNode() );

      this.insert(zNode, index);

    // BDebug.print ("BZoneNodeGroup addChildForZone exp " + exp);

      try
        { zNode.setQuestion(ai.getParser().parse(exp));}
      catch
        ( BParseException ex )
          {   ex.printStackTrace();  }

  }

  private void syncGroup()
  {
    for (int i = 0; i < zones.size(); i++ )
    {
      BZone z = zones.getZoneAt(i);
      boolean found = false;

      for (int n = 0; n < this.getChildCount(); n++)
      {
        BSimpleDecisionNode zNode = (BSimpleDecisionNode) this.getChildAt(n);
        if (  z.toString().equals(     ((BString)zNode.getValueRef()).getValue()         ) )
          found = true;
      }

      if (!found)
        addChildForZone(z, i);
    }


  }

  private void updateNodeForZone(int i)
  {


    BSimpleDecisionNode zNode = (BSimpleDecisionNode) this.getChildAt(i);
    BZone z = zones.getZoneAt(i);
    zNode.setValueRef( new BString(z.getName()) );
    zNode.setName(z.getName() );
    String exp = "[me.zones." + z.getName() + zNode.getWorldItemRef()+ "]";

    BDebug.print ("BZoneGroup.updateNodeForZone exp" + exp);
    try
    {
      zNode.setQuestion(ai.getParser().parse(exp));
    }
    catch
    ( BParseException ex )
      { ex.printStackTrace();  }

  }


  public Icon getIcon()
  {
    if (this.getVisited())
      icon.setBorderColor(BGlobal.colors.getLightestShade() );
    else
      icon.setBorderColor( BGlobal.colors.getAIGroupColor() );

    return icon;
  }

  public Icon getSelectedIcon()
  {
    return getIcon();
  }


    public void setState(Object o)
  {
    super.setState(o);

    // test for old bots.
    // pre.. feb 2001
    if (o instanceof ZoneGroupNodeState)
    {
      ZoneGroupNodeState state = (ZoneGroupNodeState) o;
      this.setZones(state.zones);
      this.ai = state.ai;
    }


  }

  public Object getState()
    { return new ZoneGroupNodeState(this); }

  protected static class ZoneGroupNodeState  extends GroupNodeState
  {
    static final long serialVersionUID = -1L;
    BZones zones;
    BDecisionTreeAI ai;

    public ZoneGroupNodeState(BZoneGroupNode node)
    {
      super(node);
      this.zones = node.zones;
      this.ai = node.ai;

    }
  }


  protected class ZonesListener implements ListDataListener
  {

        public void contentsChanged(ListDataEvent e)
        {

          try
          {
            if (getParent() != null)
              if (e.getIndex0() != -1)
                updateNodeForZone(e.getIndex0() );
          }
          catch (Exception ex)
          { BDebug.print ("BZoneGroup listener contentsChanged exception: " + ex.getMessage());    }
        }

        public void intervalAdded(ListDataEvent e)
        {
          try
          {
            if (getParent() != null)
              addChildForZone(zones.getZoneAt(e.getIndex0()));
           }
          catch (Exception ex)
          { BDebug.print ("BZoneGroup listener interval added exception: " + ex.getMessage());    }
        }

        public void intervalRemoved(ListDataEvent e)
        {
        //  BDebug.print ("BZoneGroupNode:Listener interval removed fired for " + e.getIndex0());

          try
          {
            if (getParent() != null)
              BZoneGroupNode.this.remove(e.getIndex0());
          }
          catch (Exception ex)
            { BDebug.print ("BZoneGroup listener intervalRemoved exception: " + ex.getMessage());    }

        }

  }


  public void setActor(BActor bot)
  {

    if (bot == null)
    {
      zones = null;
      ai = null;
    }
    else
    {
      setZones(  ((BBot)bot).getZones()   ) ;
      ai = (BDecisionTreeAI) ((BBot)bot).getAI();
      syncGroup();
    }

    super.setActor(bot);

  }



} 