package roboforge.ai.expressions;

import roboforge.ai.values.*;


public abstract class BBinaryOp extends BNonTerminal
{

  // Serialization ID.
  // -----------------------------------------------------------
  static final long serialVersionUID = -1L;


  protected BExpression left;
  protected BExpression right;

  public BBinaryOp()
    { super(); }

  public BBinaryOp(BBinaryOp o)
  {
    super();
    
    this.left  = (o.left  != null) ?
      (BExpression) o.left.clone()  : null;

    this.right = (o.right != null) ?
      (BExpression) o.right.clone() : null;
  }

  public BBinaryOp(BExpression l, BExpression r)
  {
    super();
    this.left  = l;
    this.right = r;
  }

  public void init()
  {
    super.init();

    if (left  != null) left.init();
    if (right != null) right.init();
  }

  public void setLeft(BExpression left)
    { this.left = left; }

  public BExpression getLeft()
    { return this.left; }


  public void setRight(BExpression right)
    { this.right = right; }

  public BExpression getRight()
    { return this.right; }


  public BValue evaluate()
  {
    BValue l = left.evaluate();
    BValue r = right.evaluate();

    return combine(l, r);
  }

  protected abstract BValue combine(BValue l, BValue r);


  public int getType()
  {
    int l = left.getType();
    int r = right.getType();
    return combineType(l, r);
  }

  protected abstract int combineType(int l, int r);


  public String toString()
  {
    if ((left == null) || (right == null))
      return getName();
    else
      return left+ " "+ getName()+ " "+ right;
  }

  public abstract String getName();

}