package roboforge.ai.orient;

import roboforge.*;
import roboforge.ai.worldstate.*;
import roboforge.ai.values.*;

import java.util.Vector;


public class BOrientFacingOnly extends BOrient
{
  static final long serialVersionUID = -1L;
  
  private float facing;
  private static final float ANGLE_EQUALS_THRESHOLD = 0.4f;
  private transient BNumber oppAng;


  public BOrientFacingOnly( float facing )
    { this.facing = facing; }

  public Object clone()
    { return new BOrientFacingOnly(this.facing); }

  public boolean equals(Object o)
    { return (o instanceof BOrientFacingOnly); }


  public void setFacing(float facing )
    { this.facing = facing; }

  public float getFacing()
    { return facing; }


  public boolean executeOrient( BBot bot)
  {

    BWorldState theState = bot.getWorldState();

    BWorldItem oppAngItem = theState.get(BSimulation.WSTATE_OPPANGLE);
    oppAng = (BNumber) oppAngItem.getValue();

    float angleToTurn = (float)oppAng.getValue() - facing;
    float smallestAngle;

    if (angleToTurn < -180 )
      smallestAngle = 360 + angleToTurn;
    else  if (angleToTurn < 0 )
       smallestAngle = -angleToTurn;
    else if (angleToTurn < 180 )
        smallestAngle = angleToTurn;
    else
        smallestAngle = 360 - angleToTurn ;

    if ( smallestAngle < ANGLE_EQUALS_THRESHOLD )
    {
      //bot.setArenaComments(" BCpu.orient achieved " +getName() + ".orientBot orient achieved ");
      bot.getChassis().terminateChassisMove(false);
      return true;
    }
    else
    {
       int rotateDir;

       if (angleToTurn < -180 )
        rotateDir =  BMoveList.CHASSIS_ROTATE_RIGHT;
       else if (angleToTurn < 0 )
        rotateDir =  BMoveList.CHASSIS_ROTATE_LEFT;
        
       else if ( angleToTurn < 180 )
        rotateDir =  BMoveList.CHASSIS_ROTATE_RIGHT;
       else
        rotateDir =  BMoveList.CHASSIS_ROTATE_LEFT;

        bot.getChassis().setNewChassisMove(bot.getChassis().getChassisMoveList().getMove(rotateDir), smallestAngle,   BChassis.ADD_MODE_UPDATE_BUT_NOT_OVERIDE, null );

    }
    
     return false;


  }


  /*
  public String getName()
  {
    return "Orienting to face " + BPhysics.roundToString(facing,1) + " degrees clockwise from opponent" ;
  }
  */

  public String getName()
    { return "Orient facing only "; } //+  java.lang.Math.round(facing)  + " deg"; }


  

}
/*
public boolean orientBot( Vector channelsIn, BMove orientMove, BChassis theChassis, float timeSlice, BBot opponentBot)
  {


    orientMove.increaseTimeSpentOrienting(timeSlice);

  // init working variables
    for (int i = 0; i < channelsIn.size(); i++)
    {
      BChannel tempChan =(BChannel)channelsIn.get(i);

      if (tempChan.getId().equalsIgnoreCase( "OPPDET")  )
        oppDetChannel = tempChan;
      else if ( tempChan.getId().equalsIgnoreCase( "OPPDST")   )
        oppDstChannel  = tempChan;
      else if ( tempChan.getId().equalsIgnoreCase( "OPPANG"   )  )
        oppAngChannel  = tempChan;
      else if ( tempChan.getId().equalsIgnoreCase( "OPPFACING"   ) )
        oppFacingChannel  = tempChan;
      else if ( tempChan.getId().equalsIgnoreCase( "OPPREACH"   ) )
        oppReachChannel  = tempChan;
      else if ( tempChan.getId().equalsIgnoreCase( "BOTREACH"   ) )
        myReachChannel  = tempChan;




    }



   // BDebug.print ("BCpu " + getName() + " executing orientBot");

    // OPPDET channel is set by BSimulator to YES only if the opponent Bot can
    // been seen by this bot (using Sensors)

    // 1 or 3 second memory for last known location if Bot cannot see opponent
    boolean falseLocation = false;

    // fill in memory
    if ( (timeSinceLastKnown < memoryTime)
         && oppDetChannel.getChannel().equals("NO")  )
    {

     oppDstChannel.setChannel(BPhysics.distanceBetween(this.getBot(),opponentBot) + "");
     oppAngChannel.setChannel(     BPhysics.angleFrom(this.getBot(), opponentBot.getArenaPosition())  + "");
     oppFacingChannel.setChannel(         BPhysics.angleFrom( opponentBot, this.getBot().getArenaPosition() )  + "");
     oppReachChannel.setChannel( opponentBot.getBoundingRadius() * BGlobal.CANVAS_TO_WORLD_SCALE + "" );

     //oppDstChannel.setChannel(lastKnownDist+"");
     //oppAngChannel.setChannel(lastKnownAngle+"");
     //oppFacingChannel.setChannel(lastKnownOppFacing+"");
     //oppReachChannel.setChannel(lastKnownOppFacing+"");

     oppDetChannel.setChannel("YES");
     falseLocation = true;


    }

    // if no
    if ( oppDetChannel.getChannel().equals("NO")  )
    {

      float angleToCenter = BPhysics.angleFrom( getBot(), BPhysics.CENTREOBJECTPOSITION );

      int rotateDir;
      if (angleToCenter > 180)
        rotateDir = BMoveList.CHASSIS_ROTATE_LEFT;
      else
        rotateDir = BMoveList.CHASSIS_ROTATE_RIGHT;

      theChassis.setNewChassisMove(theChassis.getChassisMoveList().getMove(rotateDir), 360,   BChassis.ADD_MODE_PASSIVE, null );



    }
    else
    {
      //BDebug.print("BCpu.transposeAngleDist oppBotAngle :" + oppAngChannel.getChannel());
      //BDebug.print("BCpu.transposeAngleDist distToGo :" + oppDstChannel.getChannel());
      //BDebug.print("BCpu.transposeAngleDist oppFacingChannel.getChannel() :" + oppFacingChannel.getChannel());
      //BDebug.print("BCpu.transposeAngleDist oppReachChannel.getChannel() :" + oppReachChannel.getChannel());

      if (!falseLocation )
      {

        lastKnownDist = Float.parseFloat(oppDstChannel.getChannel());
        lastKnownAngle = Float.parseFloat(oppAngChannel.getChannel());

        lastKnownOppFacing = Float.parseFloat(oppFacingChannel.getChannel());
        lastKnownOppFacing = Float.parseFloat(oppReachChannel.getChannel());

      }



      float oppBotAngle = Float.parseFloat(oppAngChannel.getChannel());
      distToGo = Float.parseFloat(oppDstChannel.getChannel());
      angleToTurn = Float.parseFloat(oppAngChannel.getChannel());
      //BDebug.print("BCpu.orientBot distToGo :" + distToGo);
      //BDebug.print("BCpu.orientBot angleToTurn:" + angleToTurn);





      transposeAngleDist( orientMove);

      float smallestAngle;

      if (angleToTurn < 180 )
        smallestAngle = angleToTurn;
      else
        smallestAngle = 360 - angleToTurn ;








      // precalculate the target pos so we can use it to test orient achieved
      float targetAngleToOpp = orientMove.getOrientAngle();
      float smallTargetAngle;
      int targetRotateDir;

      //BDebug.print("BCpu.orient oppBotAngle " + oppBotAngle);
      //BDebug.print("BCpu.orient targetAngleToOpp " + targetAngleToOpp);

      if (oppBotAngle >  targetAngleToOpp )
        smallTargetAngle =  oppBotAngle -  targetAngleToOpp;
      else
        smallTargetAngle =  targetAngleToOpp - oppBotAngle;

      if (smallTargetAngle > 180 )
      {
        smallTargetAngle = 360 - smallTargetAngle;
        targetRotateDir = BMoveList.CHASSIS_ROTATE_LEFT;

      }
      else
        targetRotateDir = BMoveList.CHASSIS_ROTATE_RIGHT;

       //BDebug.print("BCpu.orient smallTargetAngle " + smallTargetAngle);
       //BDebug.print("BCpu.orient distToGo " + distToGo);

        // testing orient achieved

       if (  ( distToGo <= orientMove.getOrientMargin() )
          &&   ( smallTargetAngle < ANGLE_EQUALS_THRESHOLD )  )
       {
          BDebug.print(" BCpu.orient achieved " +getName() + ".orientBot orient achieved ");
          theChassis.terminateChassisMove(false);
          return true;
       }




      if (distToGo >  orientMove.getOrientMargin() )
      {
        // get to the position

        if (smallestAngle > STOPANDTURN_THRESHOLD )
        {
          int rotateDir;

          if ( angleToTurn < 180 )
            rotateDir =  BMoveList.CHASSIS_ROTATE_RIGHT;
          else
            rotateDir =  BMoveList.CHASSIS_ROTATE_LEFT;

          theChassis.setNewChassisMove(theChassis.getChassisMoveList().getMove(rotateDir), smallestAngle,   BChassis.ADD_MODE_OVERRIDE, null );


         }
        else
        {


          smallestAngle = java.lang.Math.min(MAX_FORWARD_BEND, smallestAngle);

          if ( angleToTurn < 180 )
            smallestAngle = -smallestAngle;
          else
            smallestAngle = smallestAngle;





          theChassis.setNewChassisMove(theChassis.getChassisMoveList().getMove(BMoveList.CHASSIS_MOVE_FORWARD), distToGo,   BChassis.ADD_MODE_UPDATE_BUT_NOT_OVERIDE , smallestAngle, null);
          //setArenaComments("BBot.orientBot: Set Move Forward "+distToGo+" metres ");
        }






      }
      else
      {
         // achieved the position



          BDebug.print(" BCpu.orient position achieved, rotating bot ");
          theChassis.setNewChassisMove(theChassis.getChassisMoveList().getMove(targetRotateDir), smallTargetAngle,  BChassis.ADD_MODE_PASSIVE , null);



      }





    }





    if ( falseLocation  && oppDetChannel.getChannel().equals("YES") )
      timeSinceLastKnown = timeSinceLastKnown + timeSlice;
    else
      timeSinceLastKnown =0;








   return false;


  }*/