package roboforge.animation;

import roboforge.*;
import java.util.*;

public class BTimerAnimator extends BAbstractAnimatorAdapter
{
  private int waitTime;  // the time between animation frames
  private Timer timer;  // the timer
  private int numberOfFrames; // how many frames of the animation to play

  // *********** class for the timer task *************
  private class BTimerTask extends TimerTask
  {

    BTimerAnimator ta;
    int numberOfFrames;

    public BTimerTask(BTimerAnimator ta)
    {
      super();
      this.ta = ta;
      this.numberOfFrames = ta.getNumberOfFrames();
    }

    public void run()
    {
       if(numberOfFrames > 0 || numberOfFrames == -1)
       {
         ta.animate();
         if (numberOfFrames > -1)
           numberOfFrames--;
       }
       else
         ta.stop();
    }
  }
  // ***************************************************

  // Constructors
  public BTimerAnimator( int waitTime, int numberOfFrames )
  {
    this(waitTime, numberOfFrames, false);
  }

  public BTimerAnimator( int waitTime, int numberOfFrames , boolean autoActive )
  {
    super(autoActive);
    this.waitTime = waitTime;
    this.numberOfFrames = numberOfFrames;
  }

  // returns the number of frames to animate
  protected int getNumberOfFrames()
  {
    return numberOfFrames;
  }

  // sets the number of frames to animate
  public void setNumberOfFrames(int numberOfFrames)
  {
    this.numberOfFrames = numberOfFrames;
  }

  // sets the delay time between frames
  public void setWaitTime(int waitTime)
  {
    this.waitTime = waitTime;
  }

  // start the animation
  public void start()
  {
    if(!active && numberOfFrames > 0)
    {
      super.start();
      timer = new Timer();
      BTimerTask bTimerTask = new BTimerTask(this);
      timer.schedule(bTimerTask, 0, waitTime);
    }
  }

  // stop the animation
  public void stop()
  {
    if(active)
    {
      super.stop();
      timer.cancel();
    }
  }
}
