package roboforge.animation.alpha;

import roboforge.animation.time.*;
import roboforge.animation.alpha.map.*;



public abstract class BAlphaAnimation extends BTimeAnimation
{

  // A BAlphaAnimation takes time, maps it to an
  // alpha value using a BAlphaMap, then peforms
  // animation through a set of abstract methods.


  // Members...
  // -------------------------------------------------------------

  protected BAlphaMap map;
    // The map used to map time onto alpha.

  protected float alpha;
    // Current alpha value.



  // Constructors...
  // -------------------------------------------------------------

  public BAlphaAnimation(BAlphaMap map)
  {
    super();
    setAlphaMap(map);
  }

  public BAlphaAnimation()
    { super(); }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setAlphaMap(BAlphaMap map)
    { this.map = map; }

  public BAlphaMap getAlphaMap()
    { return map; }

  public float getAlpha()
    { return alpha; }



  // BTimeAnimation implementation...
  // -------------------------------------------------------------

  public final void start(float t)
  {
    alpha = map.alpha(t);
    alphaStart(alpha);
  }

  public final boolean animate(float t, float dt)
  {
    // Stop if we're out of the time-map's domain.
    if (!map.isValidTime(t))
      return false;

    alpha = map.alpha(t);
    alphaAnimate(alpha);
    return true;
  }

  public final void stop(float t)
  {
    alpha = map.alpha(t);
    alphaStop(alpha);
  }


  // Abstract time-animation methods...
  // -------------------------------------------------------------

  public abstract void alphaStart(float alpha);
    // Start animation.

  public abstract void alphaAnimate(float alpha);
    // Animates for alpha = 'alpha'.

  public abstract void alphaStop(float alpha);
    // Stop animation.

}