package roboforge.animation.alpha;

import roboforge.animation.alpha.map.*;



public class BAlphaChannelAnimation extends BAlphaAnimation
{

  // Members...
  // -------------------------------------------------------------

  protected BAlphaAcceptor acceptor;
    // The alpha-value acceptor.

  protected BAlphaChannel channel;
    // The alpha-value channel.

  protected BAlphaValue value;
    // The current alpha-value.


  // Constructors...
  // -------------------------------------------------------------

  public BAlphaChannelAnimation()
    { super(); }
    

  public BAlphaChannelAnimation
  (
    BAlphaMap map,
    BAlphaAcceptor acceptor,
    BAlphaChannel channel
  )
  {
    super(map);
    setAcceptor(acceptor);
    setChannel(channel);
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setAcceptor(BAlphaAcceptor acceptor)
    { this.acceptor = acceptor; }

  public BAlphaAcceptor getAcceptor()
    { return acceptor; }

  public void setChannel(BAlphaChannel channel)
    { this.channel = channel; }

  public BAlphaChannel setChannel()
    { return channel; }



  // BAlphaAnimation implementation...
  // -------------------------------------------------------------

  public void alphaStart(float alpha)
  {
    // If acceptor is null, abort.
    if (acceptor == null) return;

    // Get starting value. This provides a
    // writable value for ensuing animate() calls.
    if (value == null)
      value = channel.getStartValue();

    // Evaluate start value.
    channel.evaluate(value, alpha);
    acceptor.setAlphaValue(value);
  }

  public void alphaAnimate(float alpha)
  {
    // If acceptor is null, abort.
    if (acceptor == null) return;

    channel.evaluate(value, alpha);
    acceptor.setAlphaValue(value);
  }

  public void alphaStop(float alpha)
  {
    // If acceptor is null, abort.
    if (acceptor == null) return;

    // Perform one final animation step.
    channel.evaluate(value, alpha);
    acceptor.setAlphaValue(value);
  }


}