package roboforge.animation.animation.b3d;

import roboforge.*;
import roboforge.animation.animation.*;
import roboforge.animation.*;
import roboforge.btree.*;
import roboforge.util.timing.*;

import java.util.*;

public class BDemoAutoJoints implements BAnimation
{
  BBot parent;
  BAutoRotateJoint[] myJoints;

  double lastTime;
  boolean notinited = false;


  public BDemoAutoJoints(BBot parent)
  {
    this.parent = parent;

  }


  public void addLimb(BLimb limb)
  {
   //BDebug.print ("BDemoAutoJoints addLimb Called " );

    Vector tempV = new Vector();

    Enumeration cEnum = limb.getBreathComponents();
    while (cEnum.hasMoreElements() )
    {
      BComponent c = ((BComponentNode)  cEnum.nextElement()).getComponent();
      //BDebug.print ("BDemoAutoJoints checking " + c + " " + c.getClass());
      if (c instanceof BAutoRotateJoint )
      {
       // BDebug.print ("BDemoAutoJoints detects " + c);
        tempV.add(c);
      }
    }

    if (tempV.size() != 0)
    {
     // BDebug.print ("BDemoAutoJoints creating Array " + (myJoints.length + tempV.size()) );
      BAutoRotateJoint[] processArray = new BAutoRotateJoint[myJoints.length + tempV.size()];

      System.arraycopy(myJoints, 0, processArray, 0, myJoints.length);
      for (int i =  myJoints.length;  i < processArray.length; i++)
        processArray[i] =  (BAutoRotateJoint) tempV.get(i - myJoints.length );

      myJoints = processArray;


    }



  }

  public void removeLimbFromEnd(BLimb limb)
  {
    int noOfAutoJoints = 0;

    Enumeration cEnum = limb.getBreathComponents();
    while (cEnum.hasMoreElements() )
    {
      BComponent c = ((BComponentNode)  cEnum.nextElement()).getComponent();
      if (c instanceof BAutoRotateJoint )
        noOfAutoJoints++;
    }

    if (noOfAutoJoints != 0)
    {

      BAutoRotateJoint[] processArray = new BAutoRotateJoint[myJoints.length - noOfAutoJoints];

      System.arraycopy(myJoints, 0, processArray, 0, processArray.length);

      myJoints =  processArray;
    }

  }



  private void extractAutoJoints()
  {
    BComponent[] components = parent.getBreathComponents();

    Vector tv = new Vector();

    for (int i = components.length - 1 ; i >= 0 ; i-- )
    {
      components[i].calculateLimbLoad(BGlobal.CALC_TIMESLICE, true);

      if (components[i] instanceof BAutoRotateJoint)
        tv.add( components[i] );
    }
    myJoints = new BAutoRotateJoint[tv.size()];

    for (int i = 0; i < tv.size(); i++)
      myJoints[i] = (BAutoRotateJoint) tv.get(i);



  }



  public void start()
  {

    extractAutoJoints();

    for (int i = 0 ; i < myJoints.length ; i++ )
      myJoints[i].clearVelocity();

    lastTime = BClock.time();
    notinited = true;

  }

  public void stop()
  {
    for (int i = 0 ; i < myJoints.length ; i++ )
          myJoints[i].clearVelocity();
  }

  public boolean animate()
  {
   try
   {
    
    double newTime =  BClock.time();
    double deltaT = BClock.time() - lastTime;

    if (deltaT != 0)
    {
        lastTime = newTime;
        if ( myJoints != null)
          for (int i = myJoints.length - 1 ; i >= 0  ; i-- )
            myJoints[i].move((float)deltaT , 1.0f, true);

        parent.calculateBotStats(false, true, (float)deltaT, true);

    }

    }
    catch (Exception e)
    {
      BDebug.print ("BDemoJoints.animate exception");
      return false;
    }

    return true;
  }


}

/*
public class BAutoJointAction implements BAction
{
  BAutoRotateJoint[] myJoints;

  public BAutoJointAction(BAutoRotateJoint[] newJoints)
  {
    myJoints = newJoints;

  }

  public void start()
  {
        for (int i = 0 ; i < myJoints.length ; i++ )
          myJoints[i].clearVelocity();
  }

  public boolean execute(float s)
  {
    //BDebug.print("BAutoJointAction execute called ");

    for (int i = 0 ; i < myJoints.length ; i++ )
      myJoints[i].move(s, 1.0f);

    return  true;
  }

  public void stop()
  {
    for (int i = 0 ; i < myJoints.length ; i++ )
          myJoints[i].clearVelocity();
  }
*/