package roboforge.arenaSystems;


import roboforge.*;
import roboforge.sound.*;
import roboforge.animation.*;


public class BCrowdModel  implements BArenaSystem
{
  private BFadingSound ambientLoop = null;
  private BSound smallCrowdRise = null;
  private BSound smallCrowdRise2 = null;

  private BFadingSound maxCrowdRoar = null;
  private BSound upsetEnd = null;

  private static int UPSET_CROWD = 1000;
  private static int CROWD_EVENT_THRESHOLD = 1500;
  private static int TIMESLICE_PENALTY = (int)( BGlobal.CALC_TIMESLICE * 100) ;


  int crowdState = 0;
  private BSound randomSound = null;
  private float randomFrequency = 0.005f;  // 0 .. 1  (0 - never, 1 - 100%chance every frame

  private boolean crowdActive = false;
  private boolean battleEnded = false;
  private boolean forceCrowdEvent = false;

  private transient BAnimator animator;


  public BCrowdModel( BAnimator animator)
  {
    this.animator = animator;
      try
        {
            BDebug.print ("Time Slice Penalty");
            String path =  BGlobal.SOUNDPATH + "/Arena/";

            //ambientStart = BSound.getSound(path+ "crowdAmbientStart.wav", 0, BSound.SOUND_TYPE_BACKGROUND);
            //BSound ambientEnd  = BSound.getSound(path+ "crowdAmbientEnd.wav", 0, BSound.SOUND_TYPE_BACKGROUND);

            //ambientLoop.setLoopPoints(11206, 64076);
            //ambientStart.chainSound(ambientLoop);
            //ambientLoop.chainSound(ambientEnd);


            // preload the cache
            //ambientStart.reset();
            ambientLoop = BFadingSound.getFadingSound(path+ "crowdAmbientLoop.wav", BSound.LOOP_CONTINUOUSLY, BSound.SOUND_TYPE_BACKGROUND, 2000 );
            ambientLoop.reset();
            //ambientEnd.reset();

            // crowd Rise
            smallCrowdRise = BSound.getSound(path + "crowdRise.wav");
            smallCrowdRise2 = BSound.getSound(path + "crowdRise2.wav");

            maxCrowdRoar   = BFadingSound.getFadingSound(path+ "crowdwildloop.wav", BSound.LOOP_CONTINUOUSLY, BSound.SOUND_TYPE_FOREGROUND, 2000 );

            //BSound.getSound(path+ "crowdwildloop.wav");
            randomSound  = BSound.getSound( path + "crowdRandom.wav",0, BSound.SOUND_TYPE_BACKGROUND);
            randomSound.setPriority(BSound.PRIORITY_MIN);

            upsetEnd = BSound.getSound(path+ "crowdUpsetEnd.wav");


        }
        catch (Exception e)
        { e.printStackTrace();}



  }



  private void playCrowdSound()
  {
    BSound crowdSound;

    if (java.lang.Math.random() > 0.5f )
      crowdSound = smallCrowdRise;
    else
      crowdSound = smallCrowdRise2;

    forceCrowdEvent = false;
    if (!crowdSound.isRunning() )
      crowdSound.play();
    else if (crowdSound.getFramePosPercent() > 50 )
      crowdSound.play();
  }

  public void executeActions()
  // called every time slice
  {
    if (forceCrowdEvent)
      playCrowdSound();

    if (battleEnded)
      endBattle();

    if ( Math.random() < randomFrequency )
      randomSound.play();

    crowdState -= TIMESLICE_PENALTY;

  }

  public void processEvent(BArenaEvent event)
  {
    updateState(event);

    if (event.getArenaEventType() == BArenaEvent.END)
      battleEnded = true;
    else if (event.getArenaEventType().equals(BArenaEvent.VICTORY_MOVES_START) )
      battleEnded = true;
    else if (event.getArenaEventType() == BArenaEvent.DESTRUCTION )
      forceCrowdEvent = true;
    else if (event.getArenaEventType().equals( BArenaEvent.DEADLOCK) )
      upsetEnd.play();
    else if (event.getStateModifier() >= CROWD_EVENT_THRESHOLD )
      forceCrowdEvent = true;



  }

  private void endBattle()
  {
    battleEnded = false;

    if (!maxCrowdRoar.isRunning())
    {
      ambientLoop.stop();
      randomSound.stop();

      maxCrowdRoar.fadeIn(animator);
    }
  }


  public void updateState(BArenaEvent event)
  // update the crowdState Without performing any resulant actions
  {
    crowdState += event.getStateModifier();
  }

  public void start()
  {
    if (!crowdActive )
    {
      ambientLoop.fadeIn(animator);
      crowdActive = true;
    }
  }

  public void stop()
  {
    ambientLoop.fadeOut(animator);
    randomSound.stop();
    maxCrowdRoar.fadeOut(animator);
    crowdActive = false;
  }
} 