package roboforge.geometry;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.Color;
import java.awt.event.*;
import java.util.*;

import roboforge.BDebug;


public class BPieSlice3D extends Shape3D
{
  public BPieSlice3D(double maxAngle, double minAngle, double radius, int segments)
  {
    this( maxAngle,  minAngle,  radius,  segments, new Vector3d(0,-1,0));
  }

  public BPieSlice3D(double maxAngle, double minAngle, double radius, int segments, Vector3d axis)
  {

    super();

    double arc = maxAngle -  minAngle;
    if (arc > java.lang.Math.PI * 2.0)
      arc = java.lang.Math.PI * 2.0;

    TriangleArray geometry = createGeometry( maxAngle, minAngle, radius, segments, axis, arc);

    setGeometry ( geometry );

  }

  private static void setTransform (Transform3D tran, Vector3d axisOfRotation, double amount)
  {
    if (axisOfRotation.x == 1f)
      tran.rotX(amount);
    else if (axisOfRotation.x == -1f)
      tran.rotX(-amount);
    else if (axisOfRotation.y == 1f)
      tran.rotY(amount);
    else if (axisOfRotation.y == -1f)
      tran.rotY(-amount);
    else if (axisOfRotation.z == 1f)
      tran.rotZ(amount);
    else if (axisOfRotation.z == -1f)
      tran.rotZ(-amount);
    else
      BDebug.print ("ERROR BPieSliceD.setTransform Incorrect axisOfRotation " +  axisOfRotation );


  }

  private static Vector3f getUpVector(Vector3d axisOfRotation )
  {
    Vector3f up = new Vector3f();

    if (axisOfRotation.x == 1f)
      up.set(1,0,0);
    else if (axisOfRotation.x == -1f)
      up.set(-1,0,0);
    else if (axisOfRotation.y == 1f)
      up.set(0,1,0);
    else if (axisOfRotation.y == -1f)
      up.set(0,-1,0);
    else if (axisOfRotation.z == 1f)
      up.set(0,0,1);
    else if (axisOfRotation.z == -1f)
      up.set(0,0,-1);
    else
      BDebug.print ("ERROR BPieSliceD.getUpVector Incorrect axisOfRotation " +  axisOfRotation );

    return up;

  }

    private static Point3d getEdge(Vector3d axisOfRotation, double edgeSize )
  {
    Point3d edge = new Point3d();

    if (axisOfRotation.x == 1f)
      edge.set(0,edgeSize,0);
    else if (axisOfRotation.x == -1f)
      edge.set(0,-edgeSize,0);
    else if (axisOfRotation.y == 1f)
      edge.set(0,0,edgeSize);
    else if (axisOfRotation.y == -1f)
      edge.set(0,0,edgeSize);
    else if (axisOfRotation.z == 1f)
      edge.set(edgeSize,0,0);
    else if (axisOfRotation.z == -1f)
      edge.set(-edgeSize,0,0);
    else
      BDebug.print ("ERROR BPieSliceD.getEdge Incorrect axisOfRotation " +  axisOfRotation );

    return edge;

  }


  private static TriangleArray createGeometry( double maxAngle, double minAngle, double radius, int segments, Vector3d axis , double arc)
  {

    TriangleArray geometry = new TriangleArray(segments * 3, TriangleArray.COORDINATES | TriangleArray.NORMALS);

    // create vert and normal array
    Point3d verts[] = new Point3d[segments * 3];
    Vector3f[] normals = new Vector3f[segments];

    //BDebug.print (" fillPrimaryArrays  axis " + axis);
    //BDebug.print (" fillPrimaryArrays  maxAngle " + maxAngle);
    //BDebug.print (" fillPrimaryArrays  minAngle " + minAngle);

    //BDebug.print (" fillPrimaryArrays  radius " + radius);
    //BDebug.print (" fillPrimaryArrays  arc " + arc);

    // create processing transforms
    Transform3D totalRot = new Transform3D();
    setTransform(totalRot, axis, minAngle);
    //BDebug.print (" fillPrimaryArrays  totalRot " + totalRot);

    //totalRotY.rotY(minAngle); // current transform

    Transform3D rot = new Transform3D();
    setTransform(rot, axis, arc / segments);
    //BDebug.print (" fillPrimaryArrays  rot " + rot);

    //rotY.rotY(arc / segments);  // delta transform

    // create the 3 points that will make up all the triangle parts
    Vector3f up = getUpVector( axis );
    // BDebug.print (" fillPrimaryArrays  up " + up);
    Point3d center = new Point3d (0,0,0);

    Point3d edge = getEdge(axis, radius);
    // start at the minimumn
    totalRot.transform(edge);
   // BDebug.print (" fillPrimaryArrays  edge " + edge);

    for (int i = 0 ; i < segments; i++)
    {
      // centre
      verts[i * 3] = new Point3d(center);

      // edges
      verts[(i * 3)+ 1] = new Point3d(edge);
      rot.transform(edge);
      verts[(i * 3)+ 2] = new Point3d(edge);

      normals[i] = up;
    }


    // set normal and edges
    geometry.setCoordinates(0, verts);
    geometry.setNormals(0, normals);

    return geometry;
  }






}