package roboforge.geometry;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.awt.Color;
import java.awt.event.*;
import java.util.*;

public class BRectangle3D extends Shape3D
{

   protected Point2d lowerPt;
   protected Point2d upperPt;

   protected QuadArray box;

   public BRectangle3D(double xsize, double ysize)
   {
        this( -xsize/2.0,  -ysize/2.0,  xsize/2.0,  ysize/2.0);
   }

   public BRectangle3D(double xmin, double ymin, double xmax, double ymax)
   {
        super();

        // the 2 diagonals
        this.lowerPt = new Point2d(xmin, ymin);
        this.upperPt = new Point2d(xmax, ymax);

        box = getGeometry( xmin,  ymin,  xmax,  ymax);

        setGeometry(box);
        box.setCapability(Geometry.ALLOW_INTERSECT);
        box.setCapability(QuadArray.ALLOW_COUNT_READ);
        box.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
    }

  public void setTextureCoords(float l, float t, float r, float b)
  {
   setTextureCoords( l,t,r,b,box);
  }


  private static void  setTextureCoords(float l, float t, float r, float b, QuadArray box )
  {
    box.setTextureCoordinate(0, 0, new TexCoord2f (r, b));
    box.setTextureCoordinate(0, 1, new TexCoord2f (r, t));
    box.setTextureCoordinate(0, 2, new TexCoord2f (l, t));
    box.setTextureCoordinate(0, 3, new TexCoord2f (l, b));
  }


  public static QuadArray getGeometry(double xsize, double ysize)
   {
        return getGeometry( -xsize/2.0,  -ysize/2.0,  xsize/2.0,  ysize/2.0);
   }

  public static QuadArray getGeometry(double xmin, double ymin, double xmax, double ymax)
  {
        QuadArray tempBox = new QuadArray(4, QuadArray.COORDINATES | QuadArray.TEXTURE_COORDINATE_2 | QuadArray.NORMALS);

	      Point3f verts[] = new Point3f[4];
        Vector3f[] normals = new Vector3f[1];

        tempBox.setCoordinate(0, new Point3f((float) xmax, (float) ymin, 0) );
        tempBox.setCoordinate(1, new Point3f((float) xmax, (float) ymax, 0) );
        tempBox.setCoordinate(2, new Point3f((float) xmin, (float) ymax, 0) );
        tempBox.setCoordinate(3, new Point3f((float) xmin, (float) ymin, 0) );

        normals[0] = new Vector3f(0,0,1f);
        tempBox.setNormals(0, normals);

        setTextureCoords(0f, 1f, 1f, 0f, tempBox);

        return tempBox;

  }

  private static Point3f centerXAxis_P = new Point3f();

  public static void alignXLeft( QuadArray quad)
  {
    float xMax = -Float.MAX_VALUE ;
    float xMin = Float.MAX_VALUE;

    // work out middle
    for (int i = 0; i < quad.getVertexCount(); i++ )
    {
      quad.getCoordinate(i,centerXAxis_P );
      xMax = java.lang.Math.max(xMax, centerXAxis_P.x);
      xMin = java.lang.Math.min(xMin, centerXAxis_P.x);
    }

    float length = xMax - xMin;
    length = length / 2f;

    // update fields
    for (int i = 0; i < quad.getVertexCount(); i++ )
    {
      quad.getCoordinate(i,centerXAxis_P );
      centerXAxis_P.x = centerXAxis_P.x - length;
      quad.setCoordinate(i,centerXAxis_P );
    }




  }


  public void cleanse()
  {
    setAppearance(null);
    box = null;
  }



/*
public class BBox extends Shape3D
{

   double diagonalDistance;

   // 2 diagonals
   Point3d lowerPt ;
   Point3d upperPt ;

   public BBox(double xsize, double ysize, double zsize)
   {
        this( -xsize/2.0,  -ysize/2.0,  -zsize/2.0,xsize/2.0,  ysize/2.0,  zsize/2.0);
   }


    public BBox(double xmin, double ymin, double zmin, double xmax, double ymax, double zmax)
    {
	      super();

        // the 2 diagonals
        lowerPt = new Point3d(xmin, ymin, zmin);
        upperPt = new Point3d(xmax, ymax, zmax);


	      QuadArray box = new QuadArray(24, QuadArray.COORDINATES | QuadArray.TEXTURE_COORDINATE_2 | QuadArray.NORMALS);

        Point2f texCoord[] = {
                          new Point2f(0.0f, 0.0f),
                          new Point2f(1.0f, 0.0f),
                          new Point2f(0.0f, 1.0f),
                          new Point2f(1.0f, 1.0f) };

	      Point3d verts[] = new Point3d[24];
        Vector3f[] normals = new Vector3f[6];


	      // front face
	      verts[0] = new Point3d(xmax, ymin, zmax);
	      verts[1] = new Point3d(xmax, ymax, zmax);
	      verts[2] = new Point3d(xmin, ymax, zmax);
	      verts[3] = new Point3d(xmin, ymin, zmax);
        normals[0] = new Vector3f(0,0,1f);

	        // back face
	      verts[4] = new Point3d(xmin, ymin, zmin);
	      verts[5] = new Point3d(xmin, ymax, zmin);
	      verts[6] = new Point3d(xmax, ymax, zmin);
	      verts[7] = new Point3d(xmax, ymin, zmin);
        normals[1] = new Vector3f(0,0,-1f);
	      // right face
        verts[8] = new Point3d(xmax, ymin, zmin);
	      verts[9] = new Point3d(xmax, ymax, zmin);
	      verts[10] = new Point3d(xmax, ymax, zmax);
	      verts[11] = new Point3d(xmax, ymin, zmax);
        normals[2] = new Vector3f(1f,0,0);
	      // left face
	      verts[12] = new Point3d(xmin, ymin, zmax);
	      verts[13] = new Point3d(xmin, ymax, zmax);
	      verts[14] = new Point3d(xmin, ymax, zmin);
	      verts[15] = new Point3d(xmin, ymin, zmin);
        normals[3] = new Vector3f(-1f,0,0);

	      // top face
	      verts[16] = new Point3d(xmax, ymax, zmax);
      	verts[17] = new Point3d(xmax, ymax, zmin);
	      verts[18] = new Point3d(xmin, ymax, zmin);
	      verts[19] = new Point3d(xmin, ymax, zmax);
        normals[4] = new Vector3f(0,1f,0);

	      // bottom face
	      verts[20] = new Point3d(xmin, ymin, zmax);
	      verts[21] = new Point3d(xmin, ymin, zmin);
	      verts[22] = new Point3d(xmax, ymin, zmin);
	      verts[23] = new Point3d(xmax, ymin, zmax);
        normals[5] = new Vector3f(0,-1f,0);

	      box.setCoordinates(0, verts);
        box.setNormals(0, normals);


        setGeometry(box);
        box.setCapability(Geometry.ALLOW_INTERSECT);
        box.setCapability(QuadArray.ALLOW_COUNT_READ);
        box.setCapability(GeometryArray.ALLOW_COORDINATE_READ);

        for (int i = 0; i < 23; i++)
           box.setTextureCoordinate(i, texCoord[i%4]);


    }


    public void cleanse()
      { setAppearance(null); }

    public double getDiagonalDistance()
    {

      return diagonalDistance;

    }

}*/
}