
// DG 1/9/00
// Utility to search through all components or chassis and
// output statistics to a CSV file for analysis in Excel

package roboforge.internal;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.tree.*;
import java.util.*;
import java.io.*;
import roboforge.*;

public class BComponentAnalyzer extends JFrame
{static final long serialVersionUID = -1L;
  JLabel label1 = new JLabel("Output File:");
  JLabel label2 = new JLabel("Component Type:");
  JLabel label3 = new JLabel("Process Type:");

  JTextField outputFile = new JTextField("z:/test/components.csv");
  JComboBox compType;
  JComboBox procType;

  JButton doIt = new JButton("Do It!");
  JLabel processText = new JLabel("");

  String filter;
  boolean hideSuffix = false;
  String fileSeparator = "/";

  public BComponentAnalyzer()
  {
    try
    {
      jbInit();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  //Overriden so we can exit on System Close
  protected void processWindowEvent(WindowEvent e)
  {
    super.processWindowEvent(e);
    if(e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      System.exit(0);
    }
  }

  public static void main(String[] args)
  {
    BComponentAnalyzer mainFrame = new BComponentAnalyzer();
    mainFrame.setSize(500,350);
    mainFrame.setVisible(true);
  }

  private void jbInit() throws Exception
  {
    //
    BGlobal.init();
    BTextures.setListener(this);

    // Set up the Frame
    this.getContentPane().setLayout(null);
    this.setSize(new Dimension(800, 600));
    this.setTitle("Component Analyzer");
    this.getContentPane().setBackground(BGlobal.BATTLEPLAYER_COLOR);

    label1.setSize(100,25);
    label1.setLocation(70,100);
    this.getContentPane().add(label1);
    outputFile.setSize(200,25);
    outputFile.setLocation(200,100);
    this.getContentPane().add(outputFile);

    Vector typeVector = new Vector();
    typeVector.addElement("All");
    typeVector.addElement("Component");
    typeVector.addElement("Rotator");
    typeVector.addElement("TeleJoint");
    typeVector.addElement("Joint");
    typeVector.addElement("Controller");
    typeVector.addElement("Sensor");
    typeVector.addElement("Energy");
    typeVector.addElement("Shielding");
    typeVector.addElement("Weapon");
    typeVector.addElement("Chassis");
    compType = new JComboBox(typeVector);

    Vector procVector = new Vector();
    procVector.addElement("Polys");
    procVector.addElement("MaxMin");
    procType = new JComboBox(procVector);

    label2.setSize(100,25);
    label2.setLocation(70,140);
    this.getContentPane().add(label2);
    compType.setSize(100,25);
    compType.setLocation(200,140);
    this.getContentPane().add(compType);

    label3.setSize(100,25);
    label3.setLocation(70,180);
    this.getContentPane().add(label3);
    procType.setSize(100,25);
    procType.setLocation(200,180);
    this.getContentPane().add(procType);

    doIt.setSize(200,30);
    doIt.setLocation(200,230);
    this.getContentPane().add(doIt);
    doIt.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        processText.setText("Processing...");

        int c = 0;
        String prevComponentType = "";
        String prevStatDesc1 = "";
        String prevStatDesc2 = "";
        float statMax1 = 0;
        float statMin1 = 0;
        float statMax2 = 0;
        float statMin2 = 0;

        String fType = BGlobal.COMPONENTSUFFIX;
        String path = BGlobal.COMPONENTPATH;
        String rootName = "Component Library";

        if ( compType.getSelectedItem().equals("Chassis") )
        {
          fType = BGlobal.CHASSISSUFFIX;
          path = BGlobal.CHASSISPATH;
          rootName = "Chassis Library";
        }

        filter = fType.toLowerCase();
        if (path == "") {path = "C:";}
        if (rootName == "") {rootName = path;}
        if (path.endsWith(fileSeparator)) {path = path.substring(0,(int) path.length() - 1);}

        try
        {

         // Create the first node
         FileTreeNode rootNode = new FileTreeNode(null, path, rootName);

         try {

          FileWriter fileOut = new FileWriter(outputFile.getText(),false);

          // Populate the root node with its subdirectories
		      boolean addedNodes = rootNode.populateDirectories(true);

          //DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode)componentTree.getRoot();
          for (Enumeration children = rootNode.depthFirstEnumeration(); children.hasMoreElements() ;)
          {
            FileTreeNode child = (FileTreeNode)children.nextElement();
            //Object o = child.getUserObject();
            if ( child != null )
            {
              if ( child.isLeaf() )
              {

                String fullPath = child.getFullPath();
                BComponent component = BComponent.loadFromFile(fullPath);
                //
                String componentType;
                String statDesc1;
                String statDesc2;
                float stat1 = 0f;
                float stat2 = 0f;

                if ( component instanceof BChassis ) {
                  componentType = "Chassis";
                  stat1 = ((BChassis)component).loadCapacity;
                  statDesc1 = "Load";
                  statDesc2 = "";
                } else if ( component instanceof BAutoRotateJoint ) {
                  componentType = "Rotator";
                  stat1 = ((BAutoRotateJoint)component).jointPowerOutput;
                  statDesc1 = "Power";
                  statDesc2 = "";
                } else if ( component instanceof BTelescopicJoint ) {
                  componentType = "TeleJoint";
                  stat1 = ((BTelescopicJoint)component).jointPowerOutput;
                  statDesc1 = "Power";
                  statDesc2 = "";
                } else if ( component instanceof BJoint ) {
                  componentType = "Joint";
                  stat1 = ((BJoint)component).jointPowerOutput;
                  statDesc1 = "Power";
                  statDesc2 = "";
                } else if ( component instanceof BCpu ) {
                  componentType = "Controller";
                  stat1 = ((BCpu)component).cpuCyclesPerSecond;
                  statDesc1 = "Cycles/Sec";
                  statDesc2 = "";
                } else if ( component instanceof BSensor ) {
                  componentType = "Sensor";
                  stat1 = ((BSensor)component).sensorRange;
                  statDesc1 = "Range";
                  statDesc2 = "";
                } else if ( component instanceof BEnergyGenerator ) {
                  componentType = "Energy";
                  stat1 = ((BEnergyGenerator)component).energyCapacity;
                  statDesc1 = "Capacity";
                  statDesc2 = "";
                } else if ( component instanceof BShielding ) {
                  componentType = "Shielding";
                  stat1 = ((BShielding)component).getDamageResist();
                  statDesc1 = "Resist";
                  statDesc2 = "";
                } else if ( component instanceof BWeapon ) {
                  componentType = "Weapon";
                  stat1 = ((BWeapon)component).getDamageBonus();
                  statDesc1 = "Damage";
                  statDesc2 = "";
                } else {
                  componentType = "Component";
                  stat1 = component.damageMax;
                  statDesc1 = "Hitpoints";
                  statDesc2 = "";
                }

                if ( compType.getSelectedItem().equals(componentType) ||
                     compType.getSelectedItem().equals("All")     )
                {
                  if ( fullPath.indexOf("/internal/") < 0 )
                  {
                    c += 1;
                    BDebug.print("Count="+c+" = "+child.getFullPath());

                    // Output To File Here
                    String outString;
                    if ( procType.getSelectedItem().equals("MaxMin") )
                    {

                      if ( !componentType.equals(prevComponentType) )
                      {

                        outString = ""+prevComponentType+","+prevStatDesc1+","+statMin1+","+statMax1+","+prevStatDesc2+","+statMin2+","+statMax2;
                        fileOut.write(outString+'\n');

                        statMin1 = stat1;
                        statMax1 = stat1;
                        statMin2 = stat2;
                        statMax2 = stat2;
                        prevComponentType = componentType;
                        prevStatDesc1 = statDesc1;
                        prevStatDesc2 = statDesc2;

                      }

                      if ( stat1 < statMin1 )
                        statMin1 = stat1;
                      if ( stat1 > statMax1 )
                        statMax1 = stat1;
                      if ( stat2 < statMin2 )
                        statMin2 = stat2;
                      if ( stat2 > statMax2 )
                        statMax2 = stat2;

                    } else {
                      // Polys
                      outString = ""+componentType+","+child.toString()+","+component.price+","+component.damageMax+","+component.getPolyCount();
                      fileOut.write(outString+'\n');
                    }
                  } else {
                    //BDebug.print(""+fullPath.indexOf("/internal/"));
                  }
                }

              }
            }
          }

          if ( procType.getSelectedItem().equals("MaxMin") )
          {
            String outString = ""+prevComponentType+","+prevStatDesc1+","+statMin1+","+statMax1+","+prevStatDesc2+","+statMin2+","+statMax2;
            fileOut.write(outString+'\n');
          }

          fileOut.close();

         } catch (IOException ioe) { BDebug.print("IO Exception Error: "+ioe.toString()); }

        } catch ( FileNotFoundException exf ) { BDebug.print("File Not Found Error: "+exf.toString()); }

        processText.setText("Done. "+c+" exported.");
      }
    });

    processText.setSize(300,25);
    processText.setLocation(100,300);
    this.getContentPane().add(processText);

  }



	// Inner class that represents a node in this file system tree
	protected class FileTreeNode extends DefaultMutableTreeNode {
		static final long serialVersionUID = -1L;
		public FileTreeNode(String parent, String name, String dspName) throws SecurityException,
															FileNotFoundException {
			this.name = name;
      if (dspName.toLowerCase().endsWith(filter) && hideSuffix)
      {
        dspName = dspName.substring(0,(int) dspName.length() - filter.length());
      }
      this.dispName = dspName;

			// See if this node exists and whether it is a directory
			fullName = parent == null ? name : parent + fileSeparator + name;

			File f = new File(fullName);
			if (f.exists() == false) {
				throw new FileNotFoundException("File " + fullName + " does not exist");
			}

			isDir = f.isDirectory();

			// Hack for Windows which doesn't consider a drive to be a directory!
			if (isDir == false && f.isFile() == false) {
				isDir = true;
			}
		}

		// Override isLeaf to check whether this is a directory
		public boolean isLeaf() {
			return !isDir;
		}

		// Override getAllowsChildren to check whether this is a directory
		public boolean getAllowsChildren() {
			return isDir;
		}

		// For display purposes, we return our own name
		public String toString() {
			return dispName;
		}

    public String getFullPath() {
			return fullName;
		}

		// If we are a directory, scan our contents and populate
		// with children. In addition, populate those children
		// if the "descend" flag is true. We only descend once,
		// to avoid recursing the whole subtree.
		// Returns true if some nodes were added
		boolean populateDirectories(boolean descend)
    {
			boolean addedNodes = false;

			// Do this only once
			if (populated == false)
      {
				File f;
				try
        {
					f = new File(fullName);
				}
        catch (SecurityException e)
        {
					populated = true;
					return false;
				}

				if (interim == true)
        {
					// We have had a quick look here before:
					// remove the dummy node that we added last time
					removeAllChildren();
					interim = false;
				}

				String[] names = f.list();    // Get list of contents

				// Process the directories
				for (int i = 0; i < names.length; i++)
        {
					String name = names[i];
					File d = new File(fullName, name);
					try
          {
						if (d.isDirectory())
            {
							FileTreeNode node = new FileTreeNode(fullName, name, name);
							this.add(node);
							if (descend)
              {
                //node.populateDirectories(false);
                // HSC 8 March 2000 Changed from above as need to get all
                // sub directories at start - not just as required
								node.populateDirectories(true);
							}
							addedNodes = true;
							if (descend == false) {
								// Only add one node if not descending
								break;
							}
						}
            else
            {
              if (filter == "" || d.getName().toLowerCase().endsWith(filter))
              {
                this.add(new FileTreeNode(fullName, name, name));
                addedNodes = true;
              }
            }
					} catch (Throwable t)
          {
						// Ignore phantoms or access problems
					}
				}

				// If we were scanning to get all subdirectories,
				// or if we found no subdirectories, there is no
				// reason to look at this directory again, so
				// set populated to true. Otherwise, we set interim
				// so that we look again in the future if we need to
				if (descend == true || addedNodes == false) {
					populated = true;
				} else {
					// Just set interim state
					interim = true;
				}
			}
			return addedNodes;
		}

		protected String name;		// Name of this component
		protected String fullName;	// Full pathname
    protected String dispName; // Display Name
		protected boolean populated;// true if we have been populated
		protected boolean interim;	// true if we are in interim state
		protected boolean isDir;	// true if this is a directory
	}

}