package roboforge.internal;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.table.*;
import javax.swing.UIManager;
import java.util.*;
import javax.swing.tree.*;
import javax.swing.border.*;
import javax.swing.plaf.metal.*;

import roboforge.ui.*;
import roboforge.ui.components.*;
import roboforge.ui.components.text.*;
import roboforge.ui.modes.*;
import roboforge.ui.fusion.*;
import roboforge.util.graphics.*;
import roboforge.BComponentMaterial;
import roboforge.BFileFilter;
import roboforge.BGlobal;
import roboforge.BDebug;


public class BMaterialEditor extends JFrame
{

/*
* Description:
  Allows the modification, creation of Materials.
  DJG Feb 2000

* Example:
* Modification History ( Name, Date, Notes )
                         AWH, 11 Jan 01, updated the UI

*/

  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "mainBox.opaque",         Boolean.TRUE,

    "titleBox.opaque",        Boolean.TRUE,

    "buttonBox.opaque",       Boolean.TRUE,

    "box1.opaque",            Boolean.TRUE,

    "box2.opaque",            Boolean.TRUE,

    "box3.opaque",            Boolean.TRUE,

    "loadButton.text",        "Load",

    "saveButton.text",        "Save",

    "newButton.text",         "New",

    "mainLabel.text",         "Roboforge Material Editor",
    "mainLabel.fontname",     "Arial",
    "mainLabel.fontsize",     new Integer(24),
    "mainLabel.fontstyle",    new Integer(2),
    "mainLabel.width",        new Integer(500),
    "mainLabel.height",       new Integer(28),
    "mainLabel.antialiased",  Boolean.TRUE,

    "fileLabel.text",         "[empty]",
    "fileLabel.width",        new Integer(150),
    "fileLabel.hAlign",       BLabel.ALIGN_LEFT,

    "nameLabel.text",         "Name:",
    "nameLabel.width",        new Integer(150),

    "matCodeLabel.text",      "Material Code:",
    "matCodeLabel.width",     new Integer(150),

    "soundTypeLabel.text",    "Type (Sound):",
    "soundTypeLabel.width",   new Integer(150),

    "densityLabel.text",      "Density (gm/cm3):",
    "densityLabel.width",     new Integer(150),

    "dtwrLabel.text",         "Dmg to Weight Ratio:",
    "dtwrLabel.width",        new Integer(150),

    "baseDmgLabel.text",      "Base Damage",
    "baseDmgLabel.width",     new Integer(150),

    // "destSeqLabel.text",      "Destruct Sequence:",
    // "destSeqLabel.width",     new Integer(150),

  });

  private BPanel bPanel = new BPanel(new FlowLayout());

  // Buttons
  private BButton loadButton = new BButton(res, "loadButton");
  private BButton saveButton = new BButton(res, "saveButton");
  private BButton newButton = new BButton(res, "newButton");

  // Fields
  private BTextField nameField = new BTextField();
  private BTextField matCodeField = new BTextField();
  private BComboBox soundTypeCombo = new BComboBox();
  private BTextField densityField = new BTextField();
  private BTextField dtwrField = new BTextField();
  private BTextField baseDmgField = new BTextField();
  // private BTextField destSeqField = new BTextField();

  // Labels
  private BLabel mainLabel = new BLabel(res, "mainLabel");
  private BLabel nameLabel = new BLabel(res, "nameLabel");
  private BLabel matCodeLabel = new BLabel(res, "matCodeLabel");
  private BLabel soundTypeLabel = new BLabel(res, "soundTypeLabel");
  private BLabel densityLabel = new BLabel(res, "densityLabel");
  private BLabel dtwrLabel = new BLabel(res, "dtwrLabel");
  private BLabel baseDmgLabel = new BLabel(res, "baseDmgLabel");
  private BLabel destSeqLabel = new BLabel(res, "destSeqLabel");
  private BLabel fileLabel = new BLabel(res, "fileLabel");

  // Other
  private BComponentMaterial thisMaterial = new BComponentMaterial("",0f,0,0f,0f,"");
  private String filePath;
  private Border paneBorder;

  // File Stuff
  private BFileFilter filter;
  private File startFilePath;

  private JFileChooser chooser;

  private String matCode = "";

  //Main method
  static public void main(String[] args)
  {

    // Set the look and feel

    /*
    try
    {
     Border paneBorder = BorderFactory.createLineBorder(BGlobal.BUTTONCOLOR2,1);


     MetalLookAndFeel.setCurrentTheme(new BTheme());
     UIManager.put("TextField.border",paneBorder);
     UIManager.put("TextArea.border",paneBorder);
     UIManager.put("TextField.font",BGlobal.SMALLFONT);
     UIManager.put("TextArea.font",BGlobal.SMALLFONT);


    }
    catch(Exception e)
    {
    }
    */

  //   BWindow win = new BWindow();
     new BMaterialEditor();
     //win.push(bMe);
     //win.setVisible(true);
   }

  // Construct the frame.
  public BMaterialEditor()
  {
   super();
   BGlobal.init();
//   boolean success = FusionLookAndFeel.install();
//   BDebug.print("BMaterialEditor - success? " + success);
   enableEvents(AWTEvent.WINDOW_EVENT_MASK);
   try
    {
      this.setSize(new Dimension(560, 440));
      this.setResizable(false);
     // this.getContentPane().setBackground(BGlobal.BATTLEPLAYER_COLOR);


      layoutMaterialEditorChildren();
      configMaterialEditorChildren();

      this.validate();

      this.setVisible(true);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  // layout the controls
  protected void layoutMaterialEditorChildren()
  {
    BBoxPanel mainBox = new BBoxPanel(res, "mainBox", BoxLayout.Y_AXIS);
    BBoxPanel titleBox = new BBoxPanel(res, "titleBox", BoxLayout.Y_AXIS);
    BBoxPanel buttonBox = new BBoxPanel(res, "buttonBox", BoxLayout.X_AXIS);

    BBoxPanel box1 = new BBoxPanel(res, "box1", BoxLayout.Y_AXIS);
    BBoxPanel box2 = new BBoxPanel(res, "box2", BoxLayout.Y_AXIS);
    BBoxPanel box3 = new BBoxPanel(res, "box3", BoxLayout.X_AXIS);

    // filler size
    int strutSize = 15;

    // setup the title
    titleBox.addVStrut(strutSize);
    titleBox.add(mainLabel);
    titleBox.addVStrut(strutSize);

    // setup the labels
    box1.addVStrut(strutSize);
    box1.add(nameLabel);
    box1.addVStrut(strutSize);
    box1.add(matCodeLabel);
    box1.addVStrut(strutSize);
    box1.add(soundTypeLabel);
    box1.addVStrut(strutSize);
    box1.add(densityLabel);
    box1.addVStrut(strutSize);
    box1.add(dtwrLabel);
    box1.addVStrut(strutSize);
    box1.add(baseDmgLabel);
    box1.addVStrut(strutSize);
    box1.add(destSeqLabel);
    box1.addVStrut(strutSize);
    box1.addVStrut(strutSize);
    box1.addVStrut(strutSize);

    // setup the fields
    box2.addVStrut(strutSize);
    box2.add(nameField);
    box2.addVStrut(strutSize);
    box2.add(matCodeField);
    box2.addVStrut(strutSize);
    box2.add(soundTypeCombo);
    box2.addVStrut(strutSize);
    box2.add(densityField);
    box2.addVStrut(strutSize);
    box2.add(dtwrField);
    box2.addVStrut(strutSize);
    box2.add(baseDmgField);
    box2.addVStrut(strutSize);
    // box2.add(destSeqField);
    // box2.addVStrut(strutSize);
    box2.add(fileLabel);
    box2.addVStrut(strutSize);

    // setup the load and save buttons
    buttonBox.addBigMargin();
    buttonBox.add(newButton);
    buttonBox.addHStrut(strutSize);
    buttonBox.add(loadButton);
    buttonBox.addHStrut(strutSize);
    buttonBox.add(saveButton);
    // add the labels and fields
    box3.addBigMargin();
    box3.add(box1);
    box3.add(box2);
    box3.addBigMargin();

    // add everything to a main box
    mainBox.addBigMargin();
    mainBox.add(titleBox);
    mainBox.add(box3);
    mainBox.add(buttonBox);

    // and the box to the frame
    //this.getContentPane().add(mainBox);
    bPanel.add(mainBox);
    bPanel.setEnabled(true);
    bPanel.setVisible(true);
    this.getContentPane().add(bPanel);
  }

  // configure the controls
  protected void configMaterialEditorChildren()
  {
    // set the combo box items
    soundTypeCombo.addItem("DENSE_METAL");
    soundTypeCombo.addItem("HOLLOW_METAL");
    soundTypeCombo.addItem("PLASTIC");
    soundTypeCombo.addItem("GLASS");
    soundTypeCombo.addItem("ENERGY");
    soundTypeCombo.addItem("FLESH");
    soundTypeCombo.addItem("STONE");

    // set some initial values for density and damage to weight ratio
    densityField.setText("4.0");
    dtwrField.setText("5.0");
    baseDmgField.setText("20.0");

    // add focus listener to the name textfield
    nameField.addFocusListener( new FocusAdapter()
    {
       public void focusLost(FocusEvent e)
       {
          if ( (thisMaterial != null) && ( matCodeField.getText().equals("") ) )
          {
            String matDesc =  nameField.getText();
            matDesc = matDesc.toUpperCase();
            char[] cA = matDesc.toCharArray();

            // replace all non alpha numeric
            for (int i = 0; i < cA.length; i++ )
            {
              if (cA[i] == ':' )
                cA[i] = '_';
              else if (    ! ((cA[i] >= 'A') && (cA[i] <= 'Z' ))        )
                cA[i] = ' ';
            }

            // remove all spaces
            String endProduct = "";
            for (int i = 0; i < cA.length; i++ )
            {
              if (    cA[i] != ' '      )
                endProduct = endProduct + cA[i];
            }

            // only take first 10 characters or less
            if(endProduct.length() >= 10)
              matCodeField.setText( endProduct.substring(0, 10)  );
            else
              matCodeField.setText( endProduct.substring(0, endProduct.length())  );
            matCodeField.requestFocus();
         }
       }
    });

    // add focus listener to the material code field
    matCodeField.addFocusListener( new FocusAdapter()
    {
       public void focusLost(FocusEvent e)
       {
         if (thisMaterial != null)
         {
            thisMaterial.setCode( matCodeField.getText().toUpperCase() );
            matCodeField.setText(thisMaterial.getCode());
            File ifExists = new File( thisMaterial.getFileName() );

            if (ifExists.exists() && !matCode.toUpperCase().equals(thisMaterial.getCode().toUpperCase()))
            {
              int selection = JOptionPane.showConfirmDialog(null,"WARNING A Material With This Code Exists" ,"MATERIAL WARNING",JOptionPane.OK_OPTION );
            }
          }
       }
    });

    // add a mouse listener to the destruction sequence field
    /*
    destSeqField.addMouseListener(new java.awt.event.MouseAdapter()
    {
      public void mouseClicked(MouseEvent e)
      {
          File startFilePath = new File(BGlobal.ANIMATIONPATH);
          JFileChooser chooser = new JFileChooser(startFilePath);

          // Open the Chooser
          int option = chooser.showOpenDialog(null);

          if (option == JFileChooser.APPROVE_OPTION)
          {
            String filePath = chooser.getSelectedFile().getPath();
            destSeqField.setText(filePath.substring(BGlobal.ANIMATIONPATH.length()));
          }
      }
    });
    */

    // add the new button action listner
    newButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        buttonNew_actionPerformed(e);
      }
    });

    // add the load button action listner
    loadButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        buttonLoad_actionPerformed(e);
      }
    });

    // add the save button action listner
    saveButton.addActionListener(new java.awt.event.ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        buttonSave_actionPerformed(e);
      }
    });
  }

  // setup the file chooser
  private void setUpChooser (String suffix, String desc, String path)
  {
    // Remove the dot from the suffix.
    suffix = suffix.substring(1,suffix.length());

    filter = new BFileFilter(new String[] {suffix},desc,true);

    startFilePath = new File(path);

    chooser = new JFileChooser(startFilePath);
    chooser.setFileFilter(filter);
  }

  // overriden so we can exit on System Close
  protected void processWindowEvent(WindowEvent e)
  {
    //super.processWindowEvent(e);
    if(e.getID() == WindowEvent.WINDOW_CLOSING)
    {
      System.exit(0);
    }
  }


  // save material
  void buttonSave_actionPerformed(ActionEvent e)
  {
    // Update Members

    thisMaterial.name = nameField.getText();
    thisMaterial.type = soundTypeCombo.getSelectedIndex()+1;
    thisMaterial.density = Float.valueOf(densityField.getText()).floatValue();
    thisMaterial.damageToWeightRatio = Float.valueOf(dtwrField.getText()).floatValue();
    thisMaterial.baseDamage = Float.valueOf(baseDmgField.getText()).floatValue();
    thisMaterial.setCode(matCodeField.getText());

    // Create the File Chooser
    setUpChooser (BGlobal.MATERIALSUFFIX,"Material",BGlobal.MATERIALPATH);

    // Open the Chooser
    if (thisMaterial != null)
    {
       thisMaterial.saveToFile();
       fileLabel.setText(thisMaterial.getFileName().substring(BGlobal.MATERIALPATH.length()));
       matCode = thisMaterial.getCode();
    }
  }

  // new material
  void buttonNew_actionPerformed(ActionEvent e)
  {
    thisMaterial.name = "";
    thisMaterial.type = 0;
    thisMaterial.density = 4.0f;
    thisMaterial.damageToWeightRatio = 5.0f;
    // thisMaterial.setDestructSequenceFile("");
    thisMaterial.setCode("");

    nameField.setText("");
    soundTypeCombo.setSelectedIndex(0);
    densityField.setText("4.0");
    dtwrField.setText("5.0");
    baseDmgField.setText("20.0");
    // destSeqField.setText("");
    matCodeField.setText("");
    matCode = "";
  }

  // load material
  void buttonLoad_actionPerformed(ActionEvent e)
  {
    setUpChooser (BGlobal.MATERIALSUFFIX,"Material",BGlobal.MATERIALPATH);

    // Open the Chooser
    int option = chooser.showOpenDialog(this);

    if (option == JFileChooser.APPROVE_OPTION)
    {
      // Create the Material
      filePath = chooser.getSelectedFile().getPath();
      fileLabel.setText(filePath.substring(BGlobal.MATERIALPATH.length()));
      thisMaterial = BComponentMaterial.loadFromFile(filePath);

      // Update Fields
      nameField.setText(thisMaterial.name);
      soundTypeCombo.setSelectedIndex(thisMaterial.type-1);
      densityField.setText(""+thisMaterial.density);
      dtwrField.setText(""+thisMaterial.damageToWeightRatio);
      baseDmgField.setText(""+thisMaterial.baseDamage);
      // destSeqField.setText(thisMaterial.getDestructSequenceFile());
      matCodeField.setText(thisMaterial.getCode() );

      // save a copy of the file code for testing later
      matCode = thisMaterial.getCode();
    }
  }
}
