package roboforge.math;

import javax.vecmath.*;
import javax.media.j3d.Transform3D;


public class BMath3D
{

  // Angle Constants...
  // -------------------------------------------------------------

  public static final float RAD_CONVERT
  	= (float) java.lang.Math.PI / 180;

  public static final float DEGREE_CONVERT
  	= 180 / (float)java.lang.Math.PI;


  public static  final float PI
  	= (float) java.lang.Math.PI;

  public static  final float HALF_PI
  	= (float) (java.lang.Math.PI / 2.0);

  public static  final float QUARTER_PI
  	= (float) (java.lang.Math.PI / 4.0);

  public static  final float TWO_PI
  	= (float) (java.lang.Math.PI * 2.0);


  // Tuple Constants...
  // -------------------------------------------------------------

  public static final Point3f ORIGIN3f
  	= new Point3f(0, 0, 0);

  public static final Point3d ORIGIN3d
  	= new Point3d(0, 0, 0);

  public static final Vector3f UP
  	= new Vector3f(0, 1, 0);



  // Static working variables...
  // -------------------------------------------------------------

  // TODO: Ensure that the functions that use these working vars.
  // are never called by multiple threads simultaneously.
  // If this is required, then place synchronized on all
  // methods that use these vars.

	protected static Transform3D transform
  	= new Transform3D();




  // Angle conversion...
  // -------------------------------------------------------------

  public static final float degToRad(float deg)
    { return deg * RAD_CONVERT; }

  public static final float radToDeg(float rad)
    { return rad * DEGREE_CONVERT; }



  // Vector math...
  // -------------------------------------------------------------

  public static boolean isAxis(Vector3d check)
  // return true if the vector is a axis
  {
    Vector3d v = new Vector3d(check);

    v.normalize();
    v.absolute();

    if (v.x == 1.0)
      return true;
    if (v.y == 1.0)
      return true;
    if(v.z == 1.0)
      return true;

    return false;
  }



  // Random vectors...
  // -------------------------------------------------------------

  public static void randomize
    // Sets 'result' to base + (random variance).
	  (Tuple3f base, Tuple3f variance, Tuple3f result)
  {
    result.x = base.x + ((float) Math.random() * variance.x) - (variance.x / 2f);
    result.y = base.y + ((float) Math.random() * variance.y) - (variance.y / 2f);
    result.z = base.z + ((float) Math.random() * variance.z) - (variance.z / 2f);
  }


  public static void randomize
    // Sets 'result' to base + (random variance).
	  (Tuple4f base, Tuple4f variance, Tuple4f result)
  {
    result.x = base.x + ((float) Math.random() * variance.x) - (variance.x / 2f);
    result.y = base.y + ((float) Math.random() * variance.y) - (variance.y / 2f);
    result.z = base.z + ((float) Math.random() * variance.z) - (variance.z / 2f);
    result.w = base.w + ((float) Math.random() * variance.w) - (variance.w / 2f);
  }


  // Random points...
  // -------------------------------------------------------------

  public static void randomPtOnSphere(float r, Point3f result)
	  { randomPtOnSphere(ORIGIN3f, r, result); }

  public static void randomPtOnSphere(Point3f origin, float r, Point3f result)
  {
  	// Sets result to be a random point on the sphere
    // centered at origin with radius r.

    // Initialize result to be +z, length r.
    result.set(0, 0, r);

    // Rotate randomly about y.
    transform.rotY(BMath.random(-PI, PI));
    transform.transform(result);

    // Rotate randomly about x.
    transform.rotX(BMath.random(-PI, PI));
    transform.transform(result);

    // Finally, translate result relative to origin.
    result.add(origin);
  }


}