package roboforge.particles;

import roboforge.scalability.*;



public class BParticleSystemScalable
  extends    BParticleSystemPooled
  implements BScalable
{

  protected BScalabilityMetric metric;
    // Metric used to scale this particle system.

  protected int min, target, max;
    // Range of particle counts allowable.


  public BParticleSystemScalable(int min, int target, int max)
    { this(null, min, target, max); }

  public BParticleSystemScalable
  (
    BScalabilityMetric metric,
    int min, int target, int max
  )
  {
    super(max);

    this.metric = metric;
    setScalabilityRange(min, target, max);
  }



  public void setScalabilityMetric(BScalabilityMetric metric)
    { this.metric = metric; }

  public BScalabilityMetric getScalabilityMetric()
    { return metric; }



  protected void updateParticles(float t, float dt)
  {
    int max = getScaledParticleLimit();

    if (alive > max)
    {
      // Number of particles to kill.
      int n = alive - max;

      // Kill off the first n live particles...
      for (int i=used-1; i>=0; i--)
        if (isAlive[i])
        {
          killParticle(i);
          if (--n == 0) break;
        }
    }

    super.updateParticles(t, dt);
  }


  public BParticle emitParticle()
    throws BParticleSystemException
  {
    int max = getScaledParticleLimit();

    if (alive < max)
      return super.emitParticle();
    else
      throw CREATE_EXCEPTION;
  }


  protected int getScaledParticleLimit()
  {
    // Returns the maximum number of particles in this system.
    // This value scales according to the supplied metric.

    if (metric == null)
      return target;

    float m = metric.getMetric() * 2f;

    if (m <= 1f)
      m = (min + (target - min) * m);
    else
      m = target + (max - target) * (m - 1f);

    return (int) m;
  }


  public void setScalabilityRange(int min, int target, int max)
  {
    this.min    = min;
    this.target = target;
    this.max    = max;

    setPoolSize(max);
  }
}
