package roboforge.particles.emitters;

import roboforge.particles.*;


public abstract class BSystemEmitterAbstract
  extends     BSystemAspectAdapter
  implements  BSystemEmitter
{

  protected BParticleSystem  system;

  protected BEmissionPattern emissionPattern;
    // Controls how particles are emitted over time.



  public BSystemEmitterAbstract(BParticleSystem s)
    { this(s, null); }

  public BSystemEmitterAbstract(BParticleSystem s, BEmissionPattern ep)
  {
    this.system = s;
    setEmissionPattern(ep);
  }


  public int getEmissionCount(float t, float dt)
  {
    if (emissionPattern != null)
      return emissionPattern.getEmissionCount(t, dt);
    else
      return 0;
  }


  public void setEmissionPattern(BEmissionPattern ep)
    { this.emissionPattern = ep; }

  public BEmissionPattern getEmissionPattern()
    { return emissionPattern; }


  public void update(float t, float dt)
    { emitParticles(getEmissionCount(t, dt)); }



  protected void emitParticles(int n)
  {
    for (int i=0; i<n; i++)
      try
        { system.emitParticle(); }
      catch (BParticleSystemException e)
        { /* Ignore emission exceptions. */ }
  }



}
