package roboforge.particles.forces;

import roboforge.math.*;
import roboforge.particles.*;

import javax.vecmath.*;



public class BGravityPointForce extends BPointForce
{
  // BGravityPointForce applies the gravity force of an
  // imaginary body of mass M, centered at 'origin'.

  protected static final float G = 6.67E-11f;
  	// Gravitational constant...


  protected float M;
  	// The mass of this imaginary gravity source...

  protected float g;
  	// The coefficient of gravity force.
    // Used to fudge the magnitude.


    
  public BGravityPointForce()
  	{ this(BMath3D.ORIGIN3f, 1f, G); }

  public BGravityPointForce(Point3f o)
  	{ this(o, 1f, G); }

  public BGravityPointForce(Point3f o, float M)
  	{ this(o, M, G); }

  public BGravityPointForce(float M, float G)
  	{ this(BMath3D.ORIGIN3f, M, G); }

  public BGravityPointForce(Point3f o, float M, float G)
  {
  	super(o);
    setMass(M);
    setG(G);
  }


  protected float getMagnitude(BParticle p, float r)
  {
  	if (r == 0)
    	return 0;
    else
	  	return (g * p.m * M) / (r * r);
  }


  public void setMass(float M)
  	{ this.M = M; }

  public float getMass()
  	{ return M; }

  public void setG(float G)
  	{ this.g = G; }

  public float getG()
  	{ return g; }


}