package roboforge.particles.forces;

import roboforge.math.*;
import roboforge.particles.*;

import javax.vecmath.*;



public abstract class BPointForce implements BParticleForce
{
  // BPointForce applies force directed towards
  // a given point to particles.


  protected Point3f origin
    = new Point3f();


  protected Vector3f force
    = new Vector3f();



  public BPointForce()
    { this(BMath3D.ORIGIN3f); }

  public BPointForce(Point3f o)
    { origin.set(o); }

  public void setOrigin(Point3f o)
    { origin.set(o); }


  public void apply(BParticle p, float t, Vector3f f)
  {
  	force.sub(origin, p.p);

    float d = force.length();
    float m = getMagnitude(p, d);

    // Check for 0 distance...
    if (d <= 0f) return;

    force.scale(m / d);
    f.add(force);
  }

  public void update(float t)
    { /* Do nothing. */ }


  protected abstract float getMagnitude(BParticle p, float d);
  	// Returns the magnitude of force exerted
    // on particle p at distance d.
}