package roboforge.particles.forms;

import roboforge.particles.*;
import javax.vecmath.*;
import javax.media.j3d.*;


public abstract class BSystemFormGeomArray extends BSystemForm3D
{

  // Members...
  // -------------------------------------------------------------

  protected int n;
    // Total indices available.

  protected int used;
    // The number of indices used by client form objects.


  // Java3D Members...
  // -------------------------------------------------------------

  protected Shape3D shape;
    // Shape3D - holds points geometry.

  protected GeometryArray geometry;
    // Array of geometry.


  // Constructor...
  // -------------------------------------------------------------

  public BSystemFormGeomArray(BranchGroup parent, int n)
  {
    super(parent);

    this.n        = n;
    this.used     = 0;
    this.geometry = createGeometry();

    createStructure3D();
    configStructure3D();
  }


  // Abstract Methods...
  // -------------------------------------------------------------

  protected abstract GeometryArray createGeometry();
    // Instantiate & initialize the geometry array.



  // Methods...
  // -------------------------------------------------------------

  protected void createStructure3D()
  {
    shape = new Shape3D(geometry);
    root.addChild(shape);

    // Compile the scene subgraph...
    root.compile();
  }

  protected void configStructure3D()
  {
    Appearance a = new Appearance();

    TransparencyAttributes ta
      = new TransparencyAttributes(
        TransparencyAttributes.BLENDED,
        0.1f);

    a.setTransparencyAttributes(ta);
    shape.setAppearance(a);
  }


}