package roboforge.particles.systems;

import roboforge.animation.*;
import roboforge.particles.*;

import java.util.*;



public class BCompoundSystem
  extends    BAbstractAnimation
  implements BParticleSystem
{

  protected static BParticleSystemException CREATE_EXCEPTION
    = new BParticleSystemException("Can't emit particles from BCompoundSystem.");

  protected List systems;


  public BCompoundSystem()
    { this(2); }

  public BCompoundSystem(int n)
  {
    systems = new ArrayList(n);
  }


  public void add(BParticleSystem s)
    { systems.add(s); }

  public void remove(BParticleSystem s)
    { systems.remove(s); }

  public BParticleSystem get(int i)
    { return (BParticleSystem) systems.get(i); }

    

  public boolean isAlive()
  {
    BParticleSystem s;
    int n = systems.size();

    for (int i=0; i<n; i++)
    {
      s = (BParticleSystem) systems.get(i);
      if (s.isAlive())
        return true;
    }

    return false;
  }


  public boolean isDead()
    { return !isAlive(); }

  public BParticle emitParticle()
    throws BParticleSystemException
      { throw CREATE_EXCEPTION; }



  public void start()
  {
  	if (isAnimating())
    	return;

    super.start();

    int n = systems.size();
    for (int i=0; i<n; i++)
      get(i).start();
  }

  public boolean animate()
  {
    boolean isAnyActive = false;

    int n = systems.size();
    for (int i=0; i<n; i++)
      if (get(i).animate())
        isAnyActive = true;

    return isAnyActive;
  }

  public void stop()
  {
    super.stop();

    int n = systems.size();
    for (int i=0; i<n; i++)
      get(i).stop();
  }


  public String toString()
  {
  	String result = "BCompoundSystem[";

    int n = systems.size();

    if (n > 0)
    	result += get(0).toString();

    for (int i=1; i<n; i++)
      result += ", "+ get(i).toString();

    result += "]";
    return result;
  }
}