package roboforge.particles.systems;

import roboforge.BComponent;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BTextures;

import roboforge.particles.*;
import roboforge.particles.emitters.*;
import roboforge.particles.forces.*;
import roboforge.particles.forms.*;
import roboforge.particles.looks.*;
import roboforge.particles.physics.*;

import roboforge.scalability.*;

import java.util.*;
import javax.vecmath.*;
import javax.media.j3d.*;



public class BSmokeSystem extends BParticleSystemScalable
{

  protected static Texture smoke
    = BTextures.getTexture(BGlobal.TEXTUREPATH +
      "internal/particles/particle_smoke.gif");


  protected static float SCALE
    = BGlobal.WORLD_TO_CANVAS_SCALE;


  protected Point3f origin = new Point3f();

  protected float intensity = 1f;
  protected float size;
  protected float duration;

  protected BEmissionPatternPeriodic pattern;



  public BSmokeSystem(BranchGroup parent, BScalabilityMetric metric)
  {
    super(metric, 0, 1, 2);

    BSystemEmitterShared emitter =
      new BSystemEmitterShared(this);

    pattern = new BEmissionPatternPeriodic(0.05f);
    emitter.setEmissionPattern(pattern);
    emitter.setSharedEmitter(new BSmokeEmitter());

    BSystemPhysicsForces physics = new BSystemPhysicsForces();
    physics.setForces(new BSmokeForces());

    BSystemForm form;

    if (BGlobal.isParticleQualityHigh())
      form = new BSystemFormCard(parent, BSystemFormCard.SUBTRACTIVE);
    else
      form = new BSystemFormCluster(parent, BSystemFormCluster.SUBTRACTIVE);

    BSystemLookShared look = new BSystemLookShared();
    look.setSharedLook(new BSmokeParticleLook());

    addAspect(physics);
    addAspect(form);
    addAspect(look);
    addAspect(emitter);

    // Preload particles into system...
    preloadParticles();
    
  }


  public String getName()
  	{ return "BSmokeSystem"; }

	 public void setOrigin(Point3d o)
    { origin.set(o); }

  public void setIntensity(float intensity)
  {
    size     = intensity / 2f;
    duration = Math.max(0f, (intensity - 0.5f) * 2f);

    pattern.setDuration(duration);
  }


  protected class BSmokeEmitter
    extends    BParticleAspectAdapter
    implements BParticleEmitter
  {

    public void start(BParticle p)
    {
      p.p.set(origin);
      p.oldp.set(p.p);

      float v = size / 10f * SCALE;
      p.v.set(rnd(v), rnd(v*2f), rnd(v));

      p.setMass(1f);
      p.setLifeSpan(duration);
    }

    protected float rnd(float range)
      { return ((float) Math.random() - 0.5f) * range; }
  }



  protected class BSmokeParticleLook
    extends    BParticleAspectAdapter
    implements BParticleLook
  {

    protected float maxsize;
    
    public void start(BParticle p)
    {
      maxsize = size * SCALE;
      p.form.setSize(0.5f * maxsize);
      p.form.setTexture(smoke);
      p.form.setColor(BGlobal.WHITE_4F);
    }

    public void update(BParticle p, float t, float dt)
    {
      if (p.alpha < 0.5f)
        p.form.setSize(maxsize * (0.5f + p.alpha));
      else
        p.form.setSize(maxsize * (2f - p.alpha * 2f));
    }
  }


  protected class BSmokeForces extends BRandomForce
  {
    public BSmokeForces()
    {
      super();

      float f = SCALE / 2f;
      setBase(new Vector3f(0, f/2f,0));
      setVariance(new Vector3f(f,f,f));
    }
  }

}
