package roboforge.particles.systems;

import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BTextures;

import roboforge.particles.*;
import roboforge.particles.emitters.*;
import roboforge.particles.forces.*;
import roboforge.particles.forms.*;
import roboforge.particles.looks.*;
import roboforge.particles.physics.*;

import roboforge.math.*;
import roboforge.scalability.*;

import javax.vecmath.*;
import javax.media.j3d.*;



public class BSparkSystem extends BParticleSystemScalable
{

  protected static Texture spark
    = BTextures.getTexture(BGlobal.TEXTUREPATH +
      "internal/particles/particle_spark.gif");

  protected static float SCALE
    = BGlobal.WORLD_TO_CANVAS_SCALE;
    // = 0.5f;

  protected Point3f origin = new Point3f();

  protected float intensity = 1f;
  protected float duration  = 1f;

  protected BEmissionPatternPeriodic pattern;


  public BSparkSystem
    (BranchGroup parent, BScalabilityMetric metric)
  {
    super(metric, 1, 4, 10);

    BSystemEmitterShared emitter =
      new BSystemEmitterShared(this);

    pattern = new BEmissionPatternPeriodic(1f);
    emitter.setEmissionPattern(pattern);
    emitter.setSharedEmitter(new BSparkEmitter());

    BSystemPhysicsForces physics = new BSystemPhysicsForces();
    physics.setForces(new BSparkForces());

    BSystemForm form;

    if (BGlobal.isParticleQualityHigh())
      form = new BSystemFormCard(parent);
    else
      form = new BSystemFormCluster(parent);

    BSystemLookShared look = new BSystemLookShared();
    look.setSharedLook(new BSparkLook());

    addAspect(physics);
    addAspect(form);
    addAspect(look);
    addAspect(emitter);

    // Preload particles into system...
    preloadParticles();
    
  }

  public String getName()
  	{ return "BSparkSystem"; }

  public void setOrigin(Point3d o)
    { origin.set(o); }

  public void setIntensity(float intensity)
  {
    this.intensity = intensity;

    // Determine how many particles to release...
    int n = 2 + (int) Math.round(8 * intensity);
    pattern.setN(n);

    // Determine particle lifespan...
    duration = 0.5f + (intensity / 2f);
  }


  protected class BSparkEmitter
    extends    BParticleAspectAdapter
    implements BParticleEmitter
  {
    protected float BASE_SPEED = 2f;

    public void start(BParticle p)
    {
      p.p.set(origin);
      p.oldp.set(p.p);

      // Calculate initial velocity.
      // Influenced by intensity.
      float v = (2f + intensity * BASE_SPEED) * SCALE;

      p.v.set(rnd(v), rnd(v), rnd(v));
      p.setMass(1f);
      p.setLifeSpan(duration);
    }

    protected float rnd(float range)
      { return ((float) Math.random() - 0.5f) * range; }
  }



  protected class BSparkLook
    extends    BParticleAspectAdapter
    implements BParticleLook
  {
    protected float size;

    protected Color4f base
      = new Color4f(0.9f, 0.8f, 0.7f, 1);

    protected Color4f vary
      = new Color4f(0.2f, 0.4f, 0, 0);

    protected Color4f color
    	= new Color4f();


    public void start(BParticle p)
    {
      size = (0.1f + intensity * 0.05f) * SCALE;
      p.form.setSize(size);
      p.form.setTexture(spark);

      BMath3D.randomize(base, vary, color);
      p.form.setColor(color);
    }

    public void update(BParticle p, float t, float dt)
      { p.form.setSize(size * (1f - p.alpha)); }
  }


  protected class BSparkForces extends BNetForce
  {
    public BSparkForces()
    {
      super();
      add(new BGravityForce(new Vector3f(0, -2f * SCALE, 0)));
      add(new BDragForce(1.0f));
    }
  }

  public boolean isAlive()
   { return (time < duration); }

  // public boolean isAlive()
  //  { return true; }
}
