package roboforge.ui;

import roboforge.*;
import roboforge.geometry.*;
import roboforge.sound.*;
import roboforge.ui.components.*;
import roboforge.ui.dnd.*;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

import java.io.*;
import java.util.*;

import javax.vecmath.*;
import javax.media.j3d.*;

import java.beans.PropertyChangeListener;

import javax.swing.tree.*;
import javax.swing.event.SwingPropertyChangeSupport;

import com.sun.j3d.utils.universe.*;
import com.sun.j3d.utils.behaviors.*;
import com.sun.j3d.utils.behaviors.mouse.*;
import com.sun.j3d.utils.geometry.*;
import roboforge.btree.*;



public class B3DBotCanvas extends B3DCanvas
  implements BSimulationCanvas
{
  
  // Member variables.
  // -------------------------------------------------------------
  
  protected BBot bot;

  protected boolean        isFloorVisible;
  protected TransformGroup floorGroup;

  protected BPlanarGrid3D  smallGrid;
  protected BPlanarGrid3D  largeGrid;
  protected BranchGroup    smallGridGroup;
  protected BranchGroup    largeGridGroup;
  protected BranchGroup    currentFloor;

  protected TransformGroup arenaRotateGroup;
  protected TransformGroup arenaDisplaceGroup;
  protected TransformGroup chassisTilt;

  protected BSound selectSound =
    BSoundSystem.getUISound("8-2.wav");

  
  // Dran n' Drop variables...
  // -------------------------------------------------------------

  protected DropTarget dropTarget;
  protected DnDTargetListener dropTargetListener;

  protected static final DataFlavor OBJECT =
    ObjectSelection.createObjectFlavor();

  protected BSound addSound =
      BSoundSystem.getUISound("8-6.wav");

  
  
  
  // Constructor.
  // -------------------------------------------------------------

  public B3DBotCanvas(BUIResources res, String key)
  { 
    super(res, key); 
    
    dropTargetListener = new DnDTargetListener();
    dropTarget = new DropTarget(this, DnDConstants.ACTION_MOVE, dropTargetListener);
  }

  public B3DBotCanvas()
    { this(null, null); }

  protected Color3f getBackground3f()
    { return BGlobal.colors.getBlackShade3f(); }

  
  
  // Initialization helper methods.
  // -------------------------------------------------------------

  public void initializeUI(BUIResources res, String key)
  {
    super.initializeUI(res, key);

    isFloorVisible = false;

    if (res != null)
    {
      Boolean b = (Boolean) res.get(key+ ".showfloor");
      if (b != null)
        isFloorVisible = b.booleanValue();
    }
  }


  // Scene graph creation.
  // -------------------------------------------------------------

  protected void createSceneGraph()
  {
    super.createSceneGraph();

    floorGroup = new TransformGroup();
    floorGroup.setCapability(Group.ALLOW_CHILDREN_WRITE);
    floorGroup.setCapability(Group.ALLOW_CHILDREN_EXTEND);

    Transform3D t = new Transform3D();
    t.setScale(BGlobal.WORLD_TO_CANVAS_SCALE);
    floorGroup.setTransform(t);

    // Get canvas' UI foreground color...
    Color canvasFGColor = getForeground();
    float[] canvasFGComp = canvasFGColor.getColorComponents(null);
    Color3f fg = new Color3f(canvasFGComp);

    float smallGridSize = 10.0f; // * BGlobal.WORLD_TO_CANVAS_SCALE;
    float largeGridSize = 20.0f; // * BGlobal.WORLD_TO_CANVAS_SCALE;

    smallGrid = new BPlanarGrid3D(smallGridSize, 2, 5, fg, fg, true);
    largeGrid = new BPlanarGrid3D(largeGridSize, 4, 5, fg, fg);

    // Setup the small floor-grid.
    smallGridGroup = new BranchGroup();
    smallGridGroup.setCapability(BranchGroup.ALLOW_DETACH);
    smallGridGroup.addChild(smallGrid);

    // Setup the large floor-grid.
    largeGridGroup = new BranchGroup();
    largeGridGroup.setCapability(BranchGroup.ALLOW_DETACH);
    largeGridGroup.addChild(largeGrid);

    currentFloor = null;

    root.addChild(floorGroup);


  }

  public void setFloor(BranchGroup floor)
  // YLY - 4 Sep 2000 Changed to delete the floor if a null is passed in
  {
    if ( (currentFloor != null) && (currentFloor != floor) )
      currentFloor.detach();

    if (isFloorVisible)
    {
      //if (currentFloor == null)
      //  floorGroup.addChild(floor);
      //else
        if ( (floor != null) && (currentFloor != floor) )
        {
          //currentFloor.detach();
          floorGroup.addChild(floor);
        }

      currentFloor = floor;
    }
  }

  public BranchGroup getSmallGrid()
    { return smallGridGroup; }

  public BranchGroup getLargeGrid()
    { return largeGridGroup; }


  public void addBot(BBot bot)
  {
    this.bot = bot;

    BComponent[] breadth =
      bot.getBreathComponents();

    int n = breadth.length;

    // Each call to addComponent will clone the components shape3Ds, Then add them to the canvas universe
    for (int i =0; i < n; i++ )
      addComponent(breadth[i], breadth[i].getParent());

    BChassis chassis = bot.getChassis();

    
    chassis.setLiveNodes(arenaDisplaceGroup, arenaRotateGroup);
    chassis.setOrientationGroup(chassisTilt);
    chassis.setHeight();

  }

  public BBot getBot()
    { return bot; }

  public void removeBot()
  {
   // BDebug.print ("B3DBotCanvas.removeBot Called XXXX");
   // if (bot != null)
   //   removeComponent(bot.getChassis());

    if (scene != null)
    {
      scene.detach();

      BComponent c;
      for (int a=0; a<componentList.size(); a++)
      {
        c = (BComponent) componentList.get(a);
        c.cleanseLiveNodes();
      }

      BShape3D s;
      for (int a=0; a<shapeList.size(); a++)
      {
        s = (BShape3D) shapeList.get(a);
        s.cleanse();
      }

      shapeList.removeAllElements();
      componentList.removeAllElements();
      scene = null;

      System.gc();
    }
  }

  public void addComponentTree(BComponent baseComponent)
  {
    DefaultMutableTreeNode  root, node, parentNode;
    BComponent              c, parent;

    root = baseComponent.getTreeNode();

    Enumeration e = root.breadthFirstEnumeration();

    while (e.hasMoreElements())
    {
      node       = (DefaultMutableTreeNode) e.nextElement();
      parentNode = (DefaultMutableTreeNode) node.getParent();

      c          = (BComponent) node.getUserObject();
      parent     = (parentNode == null) ? null : (BComponent) parentNode.getUserObject();

      addComponent(c, parent);
    }
  }

  public void addComponentTree(BComponent baseComponent, BComponent parentComponent)
  {
    addComponent(baseComponent, parentComponent);

    DefaultMutableTreeNode d = baseComponent.getTreeNode();
    Enumeration enumerator = d.children();

    while (enumerator.hasMoreElements())
    {
      DefaultMutableTreeNode node = (DefaultMutableTreeNode)enumerator.nextElement();
      BComponent component = (BComponent)node.getUserObject();
      addComponentTree(component);
    }
  }

  public void addLimbToBot(BLimb limb, BComponent parentComponent)
  {
    BComponent base = limb.getBaseComponent();

    addComponent ( base,base.getConnectPointSelected(), parentComponent, parentComponent.getConnectPointSelected(), false  );

    Enumeration e = limb.getBreathComponents();
    while (e.hasMoreElements())
    {
      BComponent c = ((BComponentNode) e.nextElement()).getComponent() ;
      if (c != base)
        addComponent ( c,c.getConnectPointBase(), c.getParent(), c.getConnectPointParent() , false );

    }

    setSelectedComponent(base);

  }

  public void addLimb(BLimb limb, BComponent parentComponent)
  {
    addComponentTree(limb.getBaseComponent(), parentComponent);
  }

  public void removeLimb(BLimb limb)
    { removeComponent(limb.getBaseComponent()); }

  private void showTooBigDialog()
  {
      BPromptDialog tooLongDialog =  BPromptDialog.createNotifyDialog("Illegal addition",
        "Adding this component would result in your bot being disqualified from tournaments due to the Adjudicator size restrictions." );

      BApplication.showDialog(tooLongDialog);

  }

  public boolean addComponent
  (
    BComponent child,
    BConnectPoint childPt,
    BComponent parent,
    BConnectPoint parentPt
  )
  {
    return addComponent
    (
      child,
      childPt,
      parent,
      parentPt,
      true
    )   ;

  };


  public boolean addComponent
  (
    BComponent child,
    BConnectPoint childPt,
    BComponent parent,
    BConnectPoint parentPt,
    boolean select
  )
  {
    float ptDistance = parentPt.getLocalToVWorldDistance() * BGlobal.CANVAS_TO_WORLD_SCALE;
    //BDebug.print ( "BBodyPanel.addAction ptDistance" + ptDistance + " MAX: " + BPhysics.MAX_BOT_RADIUS );

    if (parentPt.getLocalToVWorldDistance() * BGlobal.CANVAS_TO_WORLD_SCALE > BPhysics.MAX_BOT_RADIUS )
    {
      showTooBigDialog();
      return false;

    }
    else
    {

      setSelectedComponent(null);
      child.setConnectPointSelected(null);
      parent.setConnectPointSelected(null);

      bot.addComponent(child, childPt, parent, parentPt);
      addComponent(child, parent);

      if(select)
        setSelectedComponent(child);

      return true;
    }
  }


  public void addComponent(BComponent c, BComponent parentComponent)
  {
  //  try
   // {

      if (parentComponent == null )
      {

        // HACK AROUND BUG
        // ATTACHING ARENA GROUPS HERE INSTEAD OF IN B3DCanvas TO GET AROUND THE
        // BUG IN j3d1.2 WHERE Only The Chassis was moving instead of the Bot

        arenaDisplaceGroup = new TransformGroup();
        arenaDisplaceGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        arenaDisplaceGroup.setUserData("Arena Displacement");

        arenaRotateGroup   = new TransformGroup();
        arenaRotateGroup.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        arenaRotateGroup.setUserData("Arena Rotation");

        chassisTilt = new TransformGroup();
        chassisTilt.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        chassisTilt.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
        chassisTilt.setUserData("Chassis Tilt");

        // END OF HACK

        // If c has no parent, it must be a chassis.
        //super.addComponent(c);

            // Place component in the clearance list.
        componentList.add(c);

        // Cache Shapes for disposal (BUG HACK)
        tempLiveNode = c.cloneStructure3D();
        tempLiveNode.setCapability(Group.ALLOW_CHILDREN_WRITE);
        tempLiveNode.setCapability(BranchGroup.ALLOW_DETACH);
        tempLiveNode.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
        tempLiveNode.setCapability(Group.ALLOW_CHILDREN_EXTEND);
        tempLiveNode.setCapability(Group.ALLOW_BOUNDS_READ);

        // Get the Shape3D's, for garbage collection later.
        extractShapes(tempLiveNode);

        // New Live Node
        c.setLiveBotNode(tempLiveNode);

        // Add component to the scene.
        chassisTilt.addChild(tempLiveNode);
        chassisTilt.setCapability(Group.ALLOW_CHILDREN_EXTEND);
        chassisTilt.setCapability(Group.ALLOW_CHILDREN_WRITE);

        arenaRotateGroup.addChild(chassisTilt);
        arenaDisplaceGroup.addChild(arenaRotateGroup);
        BranchGroup wrapperBranch = new BranchGroup();
        wrapperBranch.addChild(arenaDisplaceGroup);

        pickGroup.addChild(wrapperBranch);


        // Remember the component's BG [for detaching]...
        scene = tempLiveNode;

      }
      else
      {

        // place component in clearance in the list
        componentList.add(c);

        // Cache Shapes for disposal (BUG HACK)
        tempLiveNode = c.cloneStructure3D();

        tempLiveNode.setCapability(Group.ALLOW_CHILDREN_WRITE);
        tempLiveNode.setCapability(BranchGroup.ALLOW_DETACH);
        tempLiveNode.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
        tempLiveNode.setCapability(Group.ALLOW_CHILDREN_EXTEND);
        tempLiveNode.setCapability(Group.ALLOW_BOUNDS_READ);


        extractShapes(tempLiveNode);


        c.attach(parentComponent, tempLiveNode, true, false);
      };


      // SJS [13/07/2000] -
      // Add sensor range display to canvas, if appropriate.
      // NB: This is hacky; we should think of a better way.

      if (c.getComponentType() == BSensor.COMPONENT_TYPE)
      {
        if ((bot != null) && bot.areRangesDisplayed())
          ((BSensor) c).addRangeDisplay();
      }


   /* }
    catch (Exception e)
    {
      BDebug.print("ERROR: B3DBotCanvas.addComponent removing " + c   );
      //c.getBot().removeComponent(c);
      BDebug.print("ERROR: B3DBotCanvas.addComponent parent " + parentComponent);


      BDebug.print("ERROR: B3DBotCanvas.addComponent:" + e.getMessage());


    }; */    }    


  public void removeComponent(BComponent toRemove)
  {
    // Set cursor to hourglass...
    //Cursor oldCursor = getCursor();
    BApplication.setCursor(new Cursor(Cursor.WAIT_CURSOR));

    //try
    //{
      DefaultMutableTreeNode d = toRemove.getTreeNode();
     // BDebug.print("Canvas.removeComponent: toRemove="+toRemove+"  root="+d);
      Enumeration e = d.depthFirstEnumeration();

      BComponent  c = null;

      while (e.hasMoreElements())
      {
        DefaultMutableTreeNode node =
          (DefaultMutableTreeNode) e.nextElement();

        // Retrieve component from the tree.
        c = (BComponent) node.getUserObject();

        // TODO: Connect points?

        // Invoke superclass method for removing a single component (no tree).
        super.removeComponent(c);
      }

      //BDebug.print ("BBotCanvas.A GC TEMP REMOVED ");
      //System.gc();
    //}
    //catch (Exception ex)
    //  { BDebug.print("B3DCanvas.removeComponent: Error : " + ex.getMessage()); }

    BApplication.setCursor(defaultCursor);
  }



  // Accessors for bot orientation..
  // ----------------------------------------------------

  // TODO: Assumes bot != null - need error checking here?


  public void setViewRotation(Vector3d r)
  { bot.setViewRotation(r); }

  public Vector3d getViewRotation()
    { return bot.getViewRotation(); }

  public void setViewTranslation(Vector3d t)
  {
    //super.setViewTranslation(t);
    bot.setViewTranslation(t);
  }

  public Vector3d getViewTranslation()
    { return bot.getViewTranslation(); }

  public Vector3d getViewOrigin()
    { return bot.getViewOrigin(); }



  // Selection stuff
  // -------------------------------------------------------------


   protected Transferable getDragData(DragGestureEvent e)
  {
    BComponent old =
      pickComponent(e.getDragOrigin(), true);

    if (old == null)
      return null;

    BConnectPoint cp =
      old.getConnectPointSelected();

    if (old.getChassisLock() )
      return null;

    if ( (cp == null) && (old.getConnectPointBase() == null))
        return null;

    setSelectedComponent(null);

    BComponentNode compLimb = (BComponentNode)old.getTreeNode().cloneTree();
    BLimb limb = new BLimb( compLimb.getComponent() );
    //BComponent c =
    //  (BComponent) old.clone();

    // HACK
    // TELESCOPIC JOINTS CAN ONLY BE ATTACHED BY THE BASE POINT
    if ( (cp == null) || (compLimb.getComponent() instanceof BTelescopicJoint) )
    {
      // make sure the connect point base is correct.
      cp = compLimb.getComponent().getConnectPointBase();
      cp.setActive(true);
    }
    else
    {
      compLimb.getComponent().setConnectPointBase( compLimb.getComponent().getConnectPointBase() );
      compLimb.getComponent().setConnectPointBase(null);

    }

    setSelectedComponent(old);
    old.setConnectPointSelected(cp);

    compLimb.getComponent().setConnectPointSelected(cp);
    lastDragData =  new CleansableObjectSelection(limb);
    return lastDragData;
  }



  public void setSelectedComponent(BComponent c)
    { setSelectedComponent(c, true, true, true, false, BGlobal.SELECTED_COLOR_3F); }

  public void setSelectedComponent(
    BComponent c,
    boolean showSelectBox,
    boolean showSelectColor,
    boolean showConnectPts,
    boolean showInactiveConnectPoints,
    Color3f selectBoxColor)
  {

    if ((c != null) && (c != selectedComponent))
      selectSound.play();
    
    super.setSelectedComponent(
      c, showSelectBox, showSelectColor, showConnectPts, 
      showInactiveConnectPoints, selectBoxColor);
  }



  public void removeComponent  (BComponent theComponent, boolean playSound, boolean playAnimation) 
  // AN  SUPERFLUOUS METHOD FOR THE BSimulationCanvas Interface
  {
    removeComponent(theComponent);
    
  }


  // Activation methods.
  // -------------------------------------------------------------

  public void onClose()
  {
    removeBot();
    bot = null;

    floorGroup = null;

    smallGrid = null;
    largeGrid = null;
    smallGridGroup = null;
    largeGridGroup = null;
    currentFloor = null;

    arenaRotateGroup = null;
    arenaDisplaceGroup = null;
    chassisTilt = null;

    addSound = null;
    selectSound = null;

    dropTarget.setComponent(null);
    dropTarget = null;
    dropTargetListener = null;

    setDefaultSelectedConnectPoint_Tran = null;
    setDefaultSelectedConnectPoint_v = null;

      
    super.onClose();
  }



  
  // -------------------------------------------------------------
  // Drop Target Listener Class...
  // -------------------------------------------------------------

  protected class DnDTargetListener implements DropTargetListener
  {

    public void dragEnter(DropTargetDragEvent e)
      // { e.acceptDrag(DnDConstants.ACTION_MOVE); }
      { e.rejectDrag(); }

    public void dragExit (DropTargetEvent e)
      { }

    public void dragOver (DropTargetDragEvent e)
    {
      BComponent c = pickComponent(e.getLocation(), true);

      if (c == null)
        e.rejectDrag();
      else if (c.getConnectPointSelected() == null)
      {
        setDefaultSelectedConnectPoint(c);

      }
      else
        e.acceptDrag(DnDConstants.ACTION_MOVE);

      propertyManager.firePropertyChange("selectedComponent", selectedComponent, c);

    }

    public void dropActionChanged ( DropTargetDragEvent e )  
      { }
    

    public void drop (DropTargetDropEvent e)
    {
      boolean accepted = false;

      try
      {
        Transferable t    = e.getTransferable();
        Object dropped   = getDropped(t);
        BComponent target = pickComponent(e.getLocation(), true);

        if ((dropped == null) || (target == null))       return;
        if (target.getConnectPointSelected() == null) return;

        accepted = true;


        if (dropped instanceof BComponent)
        {
          BComponent droppedC = (BComponent) dropped ;
          addComponent( droppedC, droppedC.getConnectPointSelected(),  target, target.getConnectPointSelected());
        }
        else
        {
          addLimbToBot((BLimb)dropped, target);
        }



        addSound.play();

      }
      catch (Exception ex)
      {
        BDebug.print ("B3DBotCanvas ERROR accepting Drop  " + ex.getMessage() );
      }
      finally
      {
        e.acceptDrop(DnDConstants.ACTION_MOVE);
        e.dropComplete(true);


      }
    }

    
    protected Object getDropped(Transferable t)
    {
      if (!t.isDataFlavorSupported(OBJECT))
        return null;
     
      try
      {
        Object o = t.getTransferData(OBJECT);
       // if (!(o instanceof BComponent))
        //  return null;

        return o;
      }
      catch (Exception ex)
        { return null; }
    }
  }
  

}







