package roboforge.ui;

import roboforge.*;
import roboforge.ui.components.BUIResources;
import roboforge.ui.dnd.*;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

import java.util.*;

import javax.vecmath.*;
import javax.media.j3d.*;



public class B3DComponentCanvas extends B3DCanvas
{
  protected DnDTargetListener dropTargetListener;


  // Constructor.
  // -------------------------------------------------------------

  public B3DComponentCanvas(BUIResources res, String key)
    {

      super(res, key);

      setDragActive(true);

      dropTargetListener = new DnDTargetListener();

    }

  
  protected BranchGroup  cloneStructure3D(BComponent c)
    { return c.cloneStructure3D(true, true); }

  

  // Component mgmt. methods.
  // -------------------------------------------------------------

  public void addComponent(BComponent c)
  {
    super.addComponent(c);

    setSelectedComponent(c);
    setViewFromParameters();

    setDefaultSelectedConnectPoint(c);
    
  }

  public void setSelectedComponent(BComponent c)
    { setSelectedComponent(c, false, false, true, true, BGlobal.SELECTED_COLOR_3F); }

  public void removeComponent(BComponent c)
  {
    super.removeComponent(c);

    // Reset view parameters.
    setViewRotation(null);
    setViewTranslation(null);
  }

  public void setSelectedConnectPt(BConnectPoint c, boolean moveArc)
  {
    if (moveArc)
      setSelectedConnectPt(c);
    else
      super.setSelectedConnectPt(c);
      
  }
  public void setSelectedConnectPt(BConnectPoint c)
  {
    boolean moveArc = (c != null);
    if (moveArc)
      moveArc = (c != this.getSelectedConnectPt());

    super.setSelectedConnectPt( c);

    if (moveArc)
      if ( this.getSelectedComponent() instanceof BRotateJoint )
        ((BRotateJoint)this.getSelectedComponent()).moveArcToSelectedConnectPoint();


  }

  
  protected void setupMouse()
  {
    mouseBehavior = new BMouse2DBehavior(this);
    //addBehavior(mouseBehavior);
  }

  // Accessor methods.
  // -------------------------------------------------------------


  private static double VIEW_TRANSLATION_MAX = 170;

     // zoom accesor Methods
  public double getViewTranslationMax()
    { return VIEW_TRANSLATION_MAX; } 

  public Vector3d getViewTranslation()
  {
    if (viewTranslation == null)
    {
      float dist = 75.0f;

      if (scene != null)
      {
        Bounds sceneBounds = scene.getBounds();
        if (sceneBounds instanceof BoundingSphere)
        {
          BoundingSphere s = (BoundingSphere) sceneBounds;
          dist = ((float) s.getRadius()) * 5.0f;
        }
      }

      viewTranslation = new Vector3d(0.0, 0.0, dist);
    }

    return viewTranslation;
  }

  public Vector3d getViewRotation()
  {
    double angle = -BPhysics.degToRad(30.0f);

    if (viewRotation == null)
      viewRotation = new Vector3d(angle, angle, 0f);

    return viewRotation;
  }



  // Activation methods.
  // -------------------------------------------------------------

  public void onClose()
  {
    if (selectedComponent != null)
    {
      removeComponent(selectedComponent);
      selectedComponent = null;
    }

    super.onClose();
  }

  public void onActivate()
  {
    super.onActivate();

    // Hack to stop the textures of a component dissappearing.
    if (selectedComponent != null)
    {
      BComponent s = selectedComponent;
      removeComponent(s);
      addComponent(s);

    }

  }

  protected class DnDTargetListener implements DropTargetListener
  {

    public void dragEnter(DropTargetDragEvent e)
      { e.rejectDrag(); }

    public void dragExit (DropTargetEvent e)
      {  }

    public void dragOver (DropTargetDragEvent e)
      { e.rejectDrag(); }

    public void dropActionChanged ( DropTargetDragEvent e )
      { e.rejectDrag(); }

    public void drop(DropTargetDropEvent e)
      { }

  }



}