package roboforge.ui;

import roboforge.*;
import roboforge.ui.components.BUIResources;
import roboforge.ui.dnd.*;
import roboforge.btree.*;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

import java.util.*;

import javax.vecmath.*;
import javax.media.j3d.*;



public class B3DLimbCanvas extends B3DComponentCanvas
{
  protected static final double   VIEW_TRANSLATION_MAX  = 6000.0;

  BLimb activeLimb;
  protected BLimb clonedLimb;

  public B3DLimbCanvas(BUIResources res, String key)
    {
      super(res, key);

    }


   public void clear()
   {
      if (activeLimb!= null)
      {
        Enumeration e = activeLimb.getBaseComponent().getTreeNode().breadthFirstEnumeration();

        while (e.hasMoreElements())
        {
          BComponent c = ((BComponentNode)e.nextElement() ).getComponent();
          super.removeComponent(c);
        }

        activeLimb = null;
      }

   }

   public void addLimb(BLimb limb)
   {
      clear();

      activeLimb = limb;

      Enumeration e = activeLimb.getBaseComponent().getTreeNode().breadthFirstEnumeration();

      while (e.hasMoreElements())
        {
          BComponent c = ((BComponentNode)e.nextElement() ).getComponent();
          if (c == activeLimb.getBaseComponent())
          {
            if (c instanceof BTelescopicJoint )
              ((BTelescopicJoint)c).enforceBaseConnectPoint();
              
            super.addComponent(c);
          }
          else
          {
            BComponent parentComponent = c.getParent();
            // place component in clearance in the list
            componentList.add(c);

            // Cache Shapes for disposal (BUG HACK)
            BranchGroup tempLiveNode = c.cloneStructure3D();

            tempLiveNode.setCapability(Group.ALLOW_CHILDREN_WRITE);
            tempLiveNode.setCapability(BranchGroup.ALLOW_DETACH);
            tempLiveNode.setCapability(Node.ALLOW_LOCAL_TO_VWORLD_READ);
            tempLiveNode.setCapability(Group.ALLOW_CHILDREN_EXTEND);
            tempLiveNode.setCapability(Group.ALLOW_BOUNDS_READ);
            tempLiveNode.setPickable(false);

            extractShapes(tempLiveNode);


            c.attach(parentComponent, tempLiveNode, true, false);
          };

        }



        // set default connectPoint
        setDefaultSelectedConnectPoint(activeLimb.getBaseComponent());

        // set base view
        setViewTranslation(null);
        setViewFromParameters();

   }

   public void setSelectedConnectPt(BConnectPoint c)
  {
    //boolean moveArc = (c != null);
   // if (moveArc)
     // moveArc = (c != this.getSelectedConnectPt());

    super.setSelectedConnectPt( c, false);

   // if (moveArc)
   //   if ( this.getSelectedComponent() instanceof BRotateJoint )
    //    ((BRotateJoint)this.getSelectedComponent()).moveArcToSelectedConnectPoint();
   //

  }


   protected Transferable getDragData(DragGestureEvent e)
  {
    BComponent old =
      pickComponent(e.getDragOrigin(), true);

    if (old == null)
      return null;

    BConnectPoint cp =
      old.getConnectPointSelected();

    if (cp == null)
      return null;

    if (!cp.isActive() )
      return null;

    setSelectedComponent(null);

    clonedLimb = (BLimb) activeLimb.clone();

    setSelectedComponent(old);
    old.setConnectPointSelected(cp);

    clonedLimb.getBaseComponent().setConnectPointSelected(cp);
    lastDragData =  new CleansableObjectSelection(clonedLimb);

    return lastDragData;
  }


    public Vector3d getViewTranslation()
  {
    if (viewTranslation == null)
    {
      float dist = 75.0f;

      if (scene != null)
      {
        Bounds sceneBounds = scene.getBounds();
        if (sceneBounds instanceof BoundingSphere)
        {
          BoundingSphere s = (BoundingSphere) sceneBounds;
          dist = ((float) s.getRadius()) * 3.0f;
        }
      }

      viewTranslation = new Vector3d(0.0, 0.0, dist);
    }

    return viewTranslation;
  }

  protected BLimb getClonedLimb()
  {
    return clonedLimb;
  }

  public double getViewTranslationMax()
    { return VIEW_TRANSLATION_MAX; }

  public void onClose()
  {
    BDebug.print("BLimbCanvas.onClose");
    clear();
    if (clonedLimb != null)
      clonedLimb.cleanse();
    clonedLimb = null;

    if (activeLimb != null)
      activeLimb.cleanse();
    activeLimb = null;



    super.onClose();

  }


  public void onActivate()
  {
    super.onActivate();

    // Hack to stop the textures of a component dissappearing.
    if (activeLimb != null)
    {
      BDebug.print("B3DCanvas.onActivate Called ");
      addLimb(activeLimb);
    }

  }

  public void onDeactivate()
  {
    //BDebug.print("BLimbCanvas.onDeactivate");
    if (activeLimb!= null)
    {
        Enumeration e = activeLimb.getBaseComponent().getTreeNode().breadthFirstEnumeration();

        while (e.hasMoreElements())
        {
          BComponent c = ((BComponentNode)e.nextElement() ).getComponent();
          super.removeComponent(c);
        }
        activeLimb.cleanse();
        activeLimb = null;
    }

    super.onDeactivate();

  }

}