package roboforge.ui;

import roboforge.B3dShapeManager;
import roboforge.BArena;
import roboforge.BArenaCanvas;
import roboforge.BArenaObject;
import roboforge.BArenaObjectPosition;
import roboforge.BBattleRecorder;
import roboforge.BBattleFX;
import roboforge.BBot;
import roboforge.BComponentMaterial;
import roboforge.BChassis;
import roboforge.BDebug;
import roboforge.BFileFilter;
import roboforge.BGlobal;
import roboforge.BPhysics;
import roboforge.BProgressMonitorable;
import roboforge.BTextures;

import roboforge.ai.*;
import roboforge.ai.decisions.*;

import roboforge.animation.*;
import roboforge.animation.alpha.*;
import roboforge.animation.alpha.map.*;
import roboforge.animation.animation.*;
import roboforge.animation.animation.b2d.*;
import roboforge.animation.animation.b3d.*;
import roboforge.animation.arena.*;
import roboforge.animation.time.*;

import roboforge.arenaSystems.*;
import roboforge.geometry.BRectangle3D;
import roboforge.math.*;
import roboforge.particles.systems.*;
import roboforge.scalability.*;
import roboforge.sound.*;

import roboforge.ui.ai.*;
import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.console.*;
import roboforge.ui.events.*;
import roboforge.ui.modes.*;

import roboforge.util.io.*;
import roboforge.util.graphics.*;
import roboforge.util.processors.*;
import roboforge.util.timing.*;

import java.util.*;
import java.io.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import javax.vecmath.*;
import javax.media.j3d.*;
import javax.swing.table.*;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;

import com.sun.j3d.utils.geometry.*;




public class BBattleScreen extends BScreen implements  BProgressMonitorable, BBattlePlayerAnimation, ActionListener
{

  private BBattleRecorder battleRecording;
  private BArenaCanvas arenaCanvas;
  private BArena arena;

  private BAnimator battleAnimator;
    // The player behaviour.
    // (was BArenaAnimator)

  protected BArenaAnimator canvasRefresher;
  	// HACK: Keeps arena canvas repainting.

  private BBattleFX fx;
    // Audiovisual fx for battle player.


  // Frame Rate members
  private int currentFrame = 0;
  private int framesToPlay;
  private int lastFrame;
  private int lastFramePlayed;

  // Frame timing members...
  private double currentTime;
  private double startPlayTime;
  private int skippedStartFrame;
  private int startPlayFrame;


  private float timeScale = 1f;
    // Time scale (ratio of playback to real-time.)


  // SJS [19/03/2001] - Whether to use interpolation
  // for bot animations...
  private boolean isInterpolating;

  // SJS [08/05/2001] - Whether slow-motion is engaged...
  private boolean isSlowMotion = false;

  private boolean skipped = false;
  private float frameRate = 0f;
  private float framesPerSec = 0f;
  private float framesPerProgressUpdate = 0f;
  private int progressDisplayTimer = 99;

  private BArenaSystem crowd;
  private BConsolePopupRef ref;
  private BArenaEvent endEvent;
  private BArenaEvent clearEvent;
  private BSound idleSound;

  private BAnimation actionsAnimation;
    // Animation for executing periodic actions.
    // e.g. updating the virtual ref.

  boolean winnerScreenVisible = false;
  private static int END_FRAME_OFFSET = 1;


  // UI Constants...
  // -------------------------------------------------------------

  protected static final int CONTROL_BAR_HEIGHT = 60;

  protected int CANVAS_WIDTH  = 800;
  protected int CANVAS_HEIGHT = 600 - CONTROL_BAR_HEIGHT - 1;


  // UI Resources...
  // -------------------------------------------------------------

  protected BUIColorScheme colors = BGlobal.colors;

  protected BUIResources res = new BUIResources( new Object[]
  {
    "control.bounds",
      new Rectangle(0, CANVAS_HEIGHT+1, CANVAS_WIDTH, CONTROL_BAR_HEIGHT),

    "baseline.bounds",
      new Rectangle(0, CANVAS_HEIGHT, CANVAS_WIDTH, 1),

    "back.icon",                BIconCache.getHSB("exit_icon.gif"),
    "back.tooltip",             "Back to previous screen.",
    "back.accelerator",         KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),

    "play.icon",                BIconCache.getHSB("play_icon.gif"),
    "play.foreground",          colors.getDarkShade(),
    "play.tooltip",             "Play",

    "stop.icon",                BIconCache.getHSB("stop_icon.gif"),
    "stop.foreground",          colors.getDarkShade(),
    "stop.tooltip",             "Stop",

    "next.icon",                BIconCache.getHSB("next_icon.gif"),
    "next.foreground",          colors.getDarkShade(),
    "next.tooltip",             "Skip forward",

    "prev.icon",                BIconCache.getHSB("prev_icon.gif"),
    "prev.foreground",          colors.getDarkShade(),
    "prev.tooltip",             "Skip backward",

    "progress.value",           new Integer(0),
    "progress.ticks",           Boolean.FALSE,
    "progress.labels",          Boolean.FALSE,
    "progress.snap",            Boolean.FALSE,

    "snap.icon",                BIconCache.getHSB("snapshot_icon.gif"),
    "snap.foreground",          colors.getDarkShade(),
    "snap.tooltip",             "Capture screenshot",

    "view.size",                new Dimension(30, 30),
    "view.foreground",          colors.getDarkShade(),
    "view.tooltip",             "Current viewing mode: ",

    "slowmo.icon",              BIconCache.getHSB("slowmo.gif"),
    "slowmo.foreground",        colors.getDarkShade(),
    "slowmo.tooltip",           "Toggle slow motion",

    "info.icon",                BIconCache.getHSB("info_icon.gif"),
    "info.foreground",          colors.getDarkShade(),
    "info.tooltip",             "Show/hide bot information",

    "fps.size",                 new Dimension(50, 20),
    "timeElapsed.size",         new Dimension(50, 20),
    "timeElapsed.text",         "0s",

    "walkNavMode.icon",         BIconCache.getHSB("drag_xz_icon.gif"),
    "studyNavMode.icon",        BIconCache.getHSB("rotb_icon.gif"),
    "sensorNavMode.icon",       BIconCache.getHSB("sensors_icon.gif"),
    "fixedCamNavMode.icon",      BIconCache.getHSB("fixed_cam.gif"),
    "autoCamNavMode.icon",       BIconCache.getHSB("fixed_cam.gif"),

    "save.icon",                BIconCache.getHSB("save_icon.gif"),
    "save.tooltip",             "Save this battle.",
    "save.keystroke",           KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK),

  });


  // UI models...
  // -------------------------------------------------------------


  // Child panels...
  // -------------------------------------------------------------

  protected BInfoDialog infoDialog;


  // Visual elements...
  // -------------------------------------------------------------

  protected BLine baseline          = new BLine(res, "baseline");
  protected BBoxPanel control       = new BBoxPanel(res, "control", BoxLayout.X_AXIS);

  protected BButton play            = new BButton(res, "play");
  protected BButton stop            = new BButton(res, "stop");
  protected BButton next            = new BButton(res, "next");
  protected BButton prev            = new BButton(res, "prev");

  SaveListener saveListener = new SaveListener();
  PlayListener playListener = new PlayListener();
  StopListener stopListener = new StopListener();
  NextListener nextListener = new NextListener();


  protected PrevListener prevListener;

  protected BSlider progress        = new BSlider(res, "progress");

  protected BButton snap            = new BButton(res, "snap");
  protected SnapListener snapListener;    // keep a reference to the snapListener so we can remove it for onClose GC Cleanup (OptimiseIT showed handle from event manager);

  protected BToggleButton slowmo    = new BToggleButton(res, "slowmo");
  protected SlowMotionListener slowmoListener;

  protected BMultiModalButton view  = new BMultiModalButton(res, "view");
  protected BToggleButton info      = new BToggleButton(res, "info");
  protected InfoListener infoListener;


  protected BSystemButton back      = new BSystemButton();
  protected BSystemButton save      = new BSystemButton(res, "save");

  protected BLabel fps              = new BLabel(res, "fps");
  protected BLabel timeElapsed      = new BLabel(res, "timeElapsed");

  protected int modeIndex = 0;

  protected MouseInputAdapter progressDrag;

  protected BattleFocusManager focusManager;
    // Handles focus for the battle screen.



  // UI Actions...
  // -------------------------------------------------------------

  protected BackAction backAction;

    BMode walk;
    BMode study;
    BMode botAmode;
    BMode botBmode;
    BMode fixedCamMode;
    BMode autoCamMode;

    BMode currentMode;

  // Console elements...
  // -------------------------------------------------------------

  protected BBotConsole botAConsole;
  protected BBotConsole botBConsole;

  protected StartAnimation     startAnimation;
  protected EndBattleAnimation endAnimation;



  // Constructor...
  // -------------------------------------------------------------
  public BBattleScreen
  (
    BBattleRecorder battleRecordingIn,
    BArena theArena
  )
  { this(battleRecordingIn, theArena, true); }


  public BBattleScreen
  (
    BBattleRecorder battleRecordingIn,
    BArena theArena , boolean init
  ) // boolean standAlonenew
  {
    super();

    BIconCache.setHardCache(true);
    VirtualUniverse.setJ3DThreadPriority(Thread.MIN_PRIORITY );
    BGlobal.J3DENGINESTARTED = true;

    // SJS [19/03/2001] - Decide whether to use
    // interpolation in battle animation...
		isInterpolating = BGlobal.USE_INTERPOLATION;

    if (init)
      init( battleRecordingIn,  theArena);

  }


  private transient String lastProgressMessage = "Initializing BattleScreen";

  private static int LOAD_OBJECT 					= 10;
  private static int LOAD_CROWD 					= 20;
  private static int LOAD_ARENA					  = 40;
  private static int LOAD_ARENA_HALF 			= 50;
  private static int LOAD_POPARENA 			  = 60;
  private static int LOAD_POPCANVAS 			= 80;
  private static int LOAD_INIT_UI 				= 85;
  private static int LOAD_BATTLE 					= 90;

  private transient int progressPercent = LOAD_OBJECT;


  public String  getProgressMessage()
  { return lastProgressMessage; }


  public int  getProgressPercent()
  { return progressPercent; }

  public void init(
    BBattleRecorder battleRecordingIn,
    BArena theArena)
  {
    lastProgressMessage = "Initializing player...";

    currentFrame = 0;
    battleRecording = battleRecordingIn;

    frameRate       = battleRecording.getTimeSlice();
    framesPerSec    = 1 / frameRate;
    framesToPlay    = battleRecording.size();
    lastFrame       = framesToPlay - 1;
    lastFramePlayed = -1;

    framesPerProgressUpdate = framesPerSec / 3f;

    progressPercent = LOAD_CROWD;
    lastProgressMessage = "Creating Crowd..." ;

    endEvent = new BArenaEvent()
    {
      public int getStateModifier()
      { return 0; }

      public String getArenaEventType()
      { return END; }
    };

    clearEvent = new BArenaEvent()
    {
      public int getStateModifier()
      { return 0; }

      public String getArenaEventType()
      { return CLEAR_EVENTS; }

    };

    progressPercent = LOAD_ARENA;
    lastProgressMessage = "Loading Arena..." ;

    if (theArena == null)
      // First create the arena
      arena = BArena.loadFromFile(BGlobal.ARENAPATH+ battleRecording.getArenaName(), false);
    else
      arena = theArena;

    progressPercent = LOAD_ARENA_HALF;
    lastProgressMessage = "Initializing Arena..." ;

    arena.init(true);

    progressPercent = LOAD_POPARENA;
    lastProgressMessage = "Populating Arena..." ;


    // call this here to stop the expansion of working memory
    BGlobal.clearAllHardCaches();
    System.gc();

    // Initialize the 3D canvas.
    init3DCanvas();

    // Get an instance of the arena postSwap() animator.
    battleAnimator = arenaCanvas.getAnimator();

    // Create the canvas refresher animator.
    canvasRefresher = new BArenaAnimator(arenaCanvas, true);
      

    if ( (arena.ambientSoundName == null) || (arena.ambientSoundName.equals("")))
      crowd = new BCrowdModel(battleAnimator );
    else
      crowd = new BAmbientSound(battleAnimator, arena.ambientSoundName, arena.randomSoundName );


    // SJS [12/03/2001] - Audiovisual effects for player...
    fx = new BBattleFX(arenaCanvas);

    if (!BGlobal.isParticleQualityLow())
      // Use full-speed fx.
      battleAnimator.add(fx);
    else
      // Throttle down fx to 15 fps...
      battleAnimator.add(new BScheduledAnimation(fx, 15f));

    battleRecording.setFX(fx);


    // Populate the 3D canvas with objects.
    populate3DCanvas(true);

    progressPercent = LOAD_POPCANVAS;
    lastProgressMessage = "Initializing UI..." ;

    // Layout and configure UI components...
    layoutBattleScreenChildren();
    configBattleScreenChildren();

    // Layout the 3d console...
    layoutConsole();

    res.cleanse();
    res = null;


    ref = new BConsolePopupRef(arena, arenaCanvas, battleRecording);


    // Set up animations...
    actionsAnimation = new BScheduledAnimation(new ActionsAnimation(), 30f);
    // refAnimation     = new BAnnounceVictorAnimation();
    // victoryAnimation = new BVictoryAnimation();
    startAnimation   = new StartAnimation();
    endAnimation     = new EndBattleAnimation();

    progressPercent = LOAD_INIT_UI;
    lastProgressMessage = "Loading battle..." ;

    // Apply the initial battle state.
    battleRecording.preprocessForPlayer(crowd, ref, this, arenaCanvas);
    battleRecording.apply(BBattleRecorder.APPLYMODE_PLAY, 0,  0);


    arenaCanvas.lookAt(battleRecording.getBotA(), battleRecording.getBotB(), true);


    // Perform a final GC...
    progressPercent = LOAD_BATTLE;
    lastProgressMessage = "Cleaning memory..." ;

    idleSound = BSound.getSound(BGlobal.SOUNDPATH + "Arena/bgrlp.wav", BSound.LOOP_CONTINUOUSLY, BSound.SOUND_TYPE_BACKGROUND );
    //idleSound.play();

    // Start the crowd up...
    resetAmbientSounds();

    BGlobal.clearAllHardCaches();
    System.gc();

    BDebug.printFreeMem();
    initStartAnimation();

    battleAnimator.start();
    battleAnimator.add(actionsAnimation);

    this.repaint();
    arenaCanvas.repaint();
  }

  private void initStartAnimation()
  {
    battleAnimator.add(startAnimation);

  }


  // Initialization helper methods...
  // -------------------------------------------------------------


  protected void init3DCanvas()
  {
    String floorString = arena.getFloorObjectName();

    if (floorString == null)
      floorString = BGlobal.FLOOROBJECT;

    arenaCanvas = new BArenaCanvas
    (
      this, null, arena,
      false, false, true,
      true, false, false
    );

    arenaCanvas.setLocation(0, 0);
    arenaCanvas.setSize(CANVAS_WIDTH, CANVAS_HEIGHT);

  }


  protected void populate3DCanvas(boolean loadBots)
  {
    Vector arenaObjects =
      arena.getArenaObjects(false);

      for (int i = 0 ; i < arenaObjects.size(); i++)
      {
        BArenaObject a = (BArenaObject)arenaObjects.get(i);
        String adID = a.getAdvertID();

        if (adID != null)
          a.setAdvert(battleRecording.getAdvert(adID));

        arenaCanvas.loadArenaObject(a);
      }

      // remove attached zones from the test panel
      battleRecording.getBotA().setZonesAttached(false);
      battleRecording.getBotA().setZonesAttached(false);

	    // Init BBot arena objects and add them to the canvas
      arenaCanvas.loadArenaObject(battleRecording.getBotA(), battleRecording);
      arenaCanvas.loadArenaObject(battleRecording.getBotB(), battleRecording);

    	// TODO: Hack!
  	  initShadows();
  }




  private static float SHADOWSCALE = 1f;

  protected void initShadows()
  {
    // Set up bot shadows...
    {

      // alighning and orienting shadow
      Transform3D alignTransform = new Transform3D();
      alignTransform.rotX(-BPhysics.HALF_PI);
      alignTransform.setTranslation(new Vector3d(0,2,0));
      //alignTransform.setTranslation(new Vector3d(0,50,0));

      // the texture
      Texture texture = BTextures.getTexture
        (BGlobal.TEXTUREPATH +"Internal/bot_shadow.gif");


      // the appearance
      Appearance appearance = new Appearance();
      appearance.setTexture(texture);

      Material material = new Material();
      material.setLightingEnable(false);
      appearance.setMaterial(material);


      TransparencyAttributes tra =  new TransparencyAttributes(TransparencyAttributes.BLENDED,
                              0.1f,
                              TransparencyAttributes.BLEND_SRC_ALPHA,
                              TransparencyAttributes.BLEND_ONE_MINUS_SRC_ALPHA);

      appearance.setTransparencyAttributes(tra);

      RenderingAttributes renderAttr = new RenderingAttributes();
	    renderAttr.setDepthBufferWriteEnable( false );
      appearance.setRenderingAttributes(renderAttr);



      // shadow a
      alignTransform.setScale(battleRecording.getBotA().getBoundingRadius() * SHADOWSCALE);
      TransformGroup sga = new TransformGroup(alignTransform);
      BRectangle3D shadowA = new BRectangle3D(1, 1);
      shadowA.setAppearance(appearance);
      sga.addChild(shadowA);

      BranchGroup wrapA = new BranchGroup();
      wrapA.addChild(sga);

      BranchGroup botAShadow = battleRecording.getBotA().getLiveNode(false);
      botAShadow.addChild(wrapA);


      // shadow B
      alignTransform.setScale(battleRecording.getBotB().getBoundingRadius() * SHADOWSCALE );
      alignTransform.setTranslation(new Vector3d(0,2,0));
      TransformGroup sgb = new TransformGroup(alignTransform);

      BRectangle3D shadowB = new BRectangle3D(1, 1);
      shadowB.setAppearance(appearance);
      sgb.addChild(shadowB);

      BranchGroup wrapB = new BranchGroup();
      wrapB.addChild(sgb);

      BranchGroup botBShadow = battleRecording.getBotB().getLiveNode(false);
      botBShadow.addChild(wrapB);


    }
  }


  // UI Layout / Configation methods...
  // -------------------------------------------------------------

  protected void layoutBattleScreenChildren()
  {
    addOnTop(arenaCanvas);
    addOnTop(control);
    addOnTop(baseline);

    control.setEmptyBorder(15);

    control.add(play);
    control.addSmallMargin();
    control.add(stop);
    control.addSmallMargin();
    control.add(prev);
    control.addSmallMargin();
    control.add(next);

    control.addSmallMargin();
    control.addVGlued(progress);

    control.addSmallMargin();
    control.add(timeElapsed);

    if (BGlobal.DEVELOPER_MODE)
    {
      control.addSmallMargin();
      control.add(fps);
    }

    control.add(view);
    control.addSmallMargin();
    control.add(slowmo);
    control.addSmallMargin();
    control.add(info);
    control.addSmallMargin();
    control.add(snap);

    control.addBigMargin();
    control.add(save);
    control.addSmallMargin();
    control.add(back);
  }

  protected void configBattleScreenChildren()
  {
    backAction = new BackAction(res);
    back.setAction(backAction);

    save.addActionListener(saveListener);
    play.addActionListener(playListener);
    stop.addActionListener(stopListener);
    next.addActionListener(nextListener);

    prevListener =  new PrevListener();
    prev.addActionListener(prevListener);

    progress.setMaximum(lastFrame);

    snapListener =  new SnapListener();
    snap.addActionListener(snapListener);

    walk   = new BWalkNavigationMode();
    study  = new BStudyNavigationMode();
    botAmode = new BBotNavigationMode(battleRecording.getBotA() );
    botBmode = new BBotNavigationMode(battleRecording.getBotB() );
   // fixedCamMode = new BFixedCamMode();
    autoCamMode = new BAutoCamMode(battleRecording.getBotA(), battleRecording.getBotB());

    view.addMode(autoCamMode);
    view.addMode(walk);
    view.addMode(botAmode);
    view.addMode(botBmode);
    view.addMode(study);
   // view.addMode(fixedCamMode);
    view.setSelectedMode(study);
    modeIndex = 4;

    view.addActionListener(this);

    // YLY retained slowmoe listener so we can remove it onClose
    slowmoListener =  new SlowMotionListener();
    slowmo.addActionListener(slowmoListener);

    infoListener = new InfoListener();
    info.addActionListener(infoListener);

    info.setEnabled(battleRecording.getBotToShowStatsFor() != null);

    progressDrag = new ProgressDragListener();
    progress.addMouseListener(progressDrag);
    progress.addMouseMotionListener(progressDrag);

    // Instantiate focus manager...
    focusManager = new BattleFocusManager();

    // Initialize UI button enabled status...
    setUIPlaying(false);

  }

  // keep track of the current camera mode
  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource() == view)
    {
      modeIndex ++;
      if(modeIndex > (view.getModeCount() - 1))
        modeIndex = 0;
      currentMode = view.getMode(modeIndex);
    }
  }

  protected void layoutConsole()
  {
    botAConsole = new BBotConsole(battleRecording.getBotA());
    botBConsole = new BBotConsole(battleRecording.getBotB());

    botAConsole.setBounds(25,  25, 200, 50);
    botBConsole.setBounds(575, 25, 200, 50);

    arenaCanvas.add(botAConsole);
    arenaCanvas.add(botBConsole);
  }




  protected void updateUIForFrame(int frame)
  {
    updateMetrics(frame);
    updateProgress(frame);
    updateBotInfo(frame);
  }


  protected void updateProgress(int frame)
    { progress.setValue(frame); }

  protected void updateMetrics(int frame)
  {
    timeElapsed.setText(BMath.format(getBattleTimeElapsed(), 1)+ "s");

    if (BGlobal.DEVELOPER_MODE)
      fps.setText(arenaCanvas.getFPSMetric().getFPS()+ " fps");
  }

  protected void updateBotInfo(int frame)
  {
    botAConsole.update();
    botBConsole.update();
  }




  // Static load method...
  // -------------------------------------------------------------

  public static void load(String filename)
  {
    LoadAction loader =
      new LoadAction(filename);

    loader.perform();
  }


  public static void load(BBattleRecorder r)
  {
    LoadAction loader =
      new LoadAction(r);

    loader.perform();
  }
  


  // Progress drag Action...
  // -------------------------------------------------------------

  protected class ProgressDragListener extends MouseInputAdapter
  {
    boolean playerLoopActive;

    public void mousePressed(MouseEvent e)
    {
      playerLoopActive = battleAnimator.contains(BBattleScreen.this);
      battleAnimator.remove(BBattleScreen.this);
      updateProgress(e);
    }

    public void mouseDragged(MouseEvent e)
      { updateProgress(e); }

    public void mouseReleased(MouseEvent e)
    {
      updateProgress(e);

      playFrame(currentFrame, BBattleRecorder.APPLYMODE_RANDOM, currentFrame);

      if (playerLoopActive)
      {
        battleAnimator.start();
        battleAnimator.add(BBattleScreen.this);
      }
    }

    protected void updateProgress(MouseEvent e)
    {
      int x = e.getX();
      int xLen = progress.getWidth();
      float proportionAcross = ((float) x / (float) xLen);

      progress.setValue((int) (proportionAcross * framesToPlay));


      currentFrame = progress.getValue();
      //BDebug.print ("ProgressDragListener updateProgress currentFrame set top" +currentFrame );
    }
  };


  // Loading Action...
  // -------------------------------------------------------------

  protected static class LoadAction extends BThreadedAction
    implements BProgressMonitorable
  {
    protected String            filename;
    protected String            progressMessage;
    protected int               progressPercent;
    
    protected BBattleRecorder   recording;
    protected BBattleScreen     screen;

    
    public LoadAction(String filename)
    { 
      super(); 
      this.filename = filename;
    }
    
    public LoadAction(BBattleRecorder r)
    { 
      super(); 
      this.recording = r;
    }
    

    public void threadedAction(ActionEvent e)
    {
      update("Loading battle...", 0);

      BLoadingScreen.setSource(this);
      BLoadingScreen.setBackdrop(BGraphics.getHSBIcon("background_bplayer.jpg"));
      BLoadingScreen.open();

      if (recording == null)
      {
        // Load battle recording...
        try
          { recording = BBattleRecorder.loadFromFile(filename, false); }
        catch (Exception ex)
          { }
      }
        
      if (recording != null)
      {
        recording.init();
        update("Preparing battle...", 50);

        screen = new BBattleScreen(recording, null, false);
        screen.init(recording, null);

        update("Opening battle...", 100);
        BLoadingScreen.forceUpdate();
      }

      try
        { Thread.currentThread().sleep(100); }
      catch (InterruptedException ex)
        { }

      BLoadingScreen.setBackdrop(null);
     // BLoadingScreen.close();  // moved to onClose to stop the ressurection of the WorkshopScreen

    }

    public void swingCleanupAction(ActionEvent e)
    {
      if (screen != null)
      {
        BApplication.openScreen(screen);
        screen    = null;
        recording = null;
      }
    }
    
    
    // Progress update methods...
    // ......................................................
    
    protected void update(String msg, int percent)
    { 
      progressMessage = msg;  
      progressPercent = percent; 
    }

    public String getProgressMessage()
    {
      if (progressPercent < 50)
        { if (recording != null) return recording.getProgressMessage(); }
      else
        { if (screen != null) return screen.getProgressMessage(); }

      return progressMessage;
    }

    public int getProgressPercent()
    {
      if (progressPercent < 50)
        { if (recording != null) return recording.getProgressPercent() / 2; }
      else
        { if (screen != null) return 50 + (screen.getProgressPercent() / 2); }

      return progressPercent;
    }
   
  }


  // Focus Management...
  // -------------------------------------------------------------

  protected class BattleFocusManager extends DefaultFocusManager
  {

    protected boolean isInstalled = false;
      // Whether this manager is currently installed.

    protected FocusManager previousManager;
      // The focus manager which is replaced by this one.

    public void processKeyEvent(Component c, KeyEvent k)
    {
      super.processKeyEvent(c, k);

      if (arenaCanvas != null)
        if (c != arenaCanvas)
          arenaCanvas.handleKeyEvent(k);
    }

    public void setInstalled(boolean install)
    {
      // Installs/uninstalls this manager.
      if (install && !isInstalled)
      {
        previousManager = FocusManager.getCurrentManager();
        FocusManager.setCurrentManager(this);
      }
      else if (!install && isInstalled)
      {
        FocusManager.setCurrentManager(previousManager);
        previousManager = null;
      }
      isInstalled = install;
    }
  }



  // UI Actions...
  // -------------------------------------------------------------

  protected class BackAction extends BAction
  {
    public BackAction(BUIResources res)
      { super(res, "back"); }

    public void actionPerformed(ActionEvent e)
      { BApplication.closeScreen(); }
  }


  protected class SaveListener extends BAsyncActionAdapter
  {
    protected BFileDialog saveDlg;
    protected String filename;

    public void preActionApproved(ActionEvent e)
    {
      saveDlg = BFileDialog.createSaveDialog(
        "Save battle", "Save battle as:");

      try
      {
        saveDlg.setOKDefault();
        saveDlg.setFilter(new BRoboforgeFilter(BGlobal.RECORDINGSUFFIX));
        saveDlg.setPath(BGlobal.BATTLERECORDINGSPATH);
        System.gc();
      }
      catch (Exception ex)
        { BApplication.notifyUser("Could not save battle."); }

    }

    public void asyncActionApproved(ActionEvent e)
    {
      BApplication.showDialog(saveDlg);

      filename = saveDlg.getSelectedFile();

      // Check for user cancel...
      if (filename != null)
      {
        File f = new File(filename);

        if (f.exists())
        {
          BPromptDialog replaceDlg = BPromptDialog.createYesNoDialog(
            "replace battle?", "A battle named "+
              saveDlg.getSelectedName()+ " already exists. \n\n"+
            "Do you want to replace it with the current battle?");

          BApplication.showDialog(replaceDlg);
          if (!replaceDlg.getResult())
            filename = null;
        }
      }
    }

    public void postActionApproved(ActionEvent e)
    {
      if (filename != null)
        battleRecording.saveToFile(filename, battleRecording.getBotToShowStatsFor(), null);
    }
  }


  protected class PlayListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      // Can't do real-time info, so ensure info-dialog is hidden.
      //if (infoDialog != null)
      //  if (infoDialog.isActive())
       //   BApplication.hideDialog(infoDialog);

      // Start playing the battle.
      play();
    }
  }

  protected class StopListener implements ActionListener
  {

    public void actionPerformed(ActionEvent e)
    {

      stopBattle();
      //BDebug.print ("BBattleScreen.stop is animating Mode");
      //arenaCanvas.animateMode();

    }
  }

  protected class NextListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      { playNextFrame(); }
  }

  protected class PrevListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      { playPreviousFrame(); }
  }

  protected void setUIPlaying(boolean e)
  {
    play.setEnabled(!e);
    prev.setEnabled(!e);
    next.setEnabled(!e);

    // stop.setEnabled(e);
  }


  protected class SnapTakenListener implements ActionListener
  {
     public void actionPerformed(ActionEvent e)
     {
        BApplication.notifyUser("Screenshot taken successfully.");

        // sneaky time to do a GC
        System.gc();

     }
  }


  protected class SnapListener extends BAsyncActionAdapter
  {

    public void asyncActionApproved(ActionEvent e)
    {
      BBattleScreen.this.stopBattle();

      BFileDialog saveDlg = BFileDialog.createSaveDialog
        ("screenshot capture", "Please enter a name for this screenshot.");

      try
      {
        saveDlg.setPath(BGlobal.IMAGEPATH);
        saveDlg.setFilter(new BBasicFilter(BGlobal.JPEGSUFFIX));
        // saveDlg.setMask(BGlobal.JPEGSUFFIX);

        BApplication.showDialog(saveDlg);

        String filename = saveDlg.getSelectedFile();

        if (filename != null)
        {
          arenaCanvas.takeSnapshot(filename, new  SnapTakenListener() );

        }
      }
      catch (FileNotFoundException fe)
        { BDebug.print("BBattleScreen.SnapListener: couldn't find path: "+ BGlobal.USERPATH); }
    }
  }


  protected class SlowMotionListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      // Determine whether slowmo should be on or not...
      isSlowMotion = slowmo.isSelected();

      // Set the time scale...
      timeScale = isSlowMotion ? 0.25f : 1f;

      // Propagate changes to a/v fx...
      fx.setTimeScale(timeScale);

      // Propagate changes to arena modes...
      arenaCanvas.setModeRates(timeScale);

      // Set the starting point (needed for timing.)
      resetTimingInfo();
    }
  }

  protected class InfoListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      if (info.isSelected())
        BApplication.showDialog(getInfoDialog());
      else
        BApplication.hideDialog(getInfoDialog());
    }
  }


  protected class InfoDialogListener implements BActivationListener
  {
    public void activationChanged(BActivationEvent e)
    {
      if (e.getType() == BActivationEvent.DEACTIVATE)
        info.setSelected(false);
    }
  }




  // Activation methods...
  // -------------------------------------------------------------

  public void onClose()
  {
    BLoadingScreen.close();

    if (focusManager != null)
    {
      focusManager.setInstalled(false);
      focusManager = null;
    }
    if (battleAnimator != null)
    {
      // battleAnimator.cleanse();
      battleAnimator.clear();
      battleAnimator = null;
    }

    if (canvasRefresher != null)
    {
    	canvasRefresher.stop();
      canvasRefresher = null;
    }

    if (idleSound != null)
    {
      idleSound.stop();
      idleSound.close();
      idleSound = null;

    }

    if (infoDialog != null)
      if (infoDialog.isActive())
        BApplication.hideDialog(infoDialog);



    if (endAnimation != null)
      endAnimation = null;

    /*
    if (refAnimation != null)
      refAnimation = null;

    if (victoryAnimation != null)
      victoryAnimation = null;
    */

    battleRecording.getBotA().cleanse();
    battleRecording.getBotB().cleanse();

    if (battleRecording != null)
      battleRecording.cleanse();
    battleRecording = null;
    arena = null;

    // Remove everything from the canvas...
    if (arenaCanvas != null)
    {
      arenaCanvas.clear();
      arenaCanvas.onClose();
      remove(arenaCanvas);
      arenaCanvas = null;
    }

    if (ref != null)
    {
      ref.cleanse();
      ref = null;
    }
    
    battleAnimator = null;
    crowd = null;
    endEvent = null;
    colors = null;
    res = null;

    if (infoDialog != null)
      infoDialog.onClose();
    infoDialog = null;


    //BLine = null;
    control = null;
    save.removeActionListener(saveListener);
    play.removeActionListener(playListener);
    stop.removeActionListener(stopListener);
    next.removeActionListener(nextListener);

    save            = null;
    play            = null;
    stop            = null;
    next            = null;

    prev.removeActionListener(prevListener);
    prev            = null;

    progress.removeMouseListener(progressDrag);
    progress.removeMouseMotionListener(progressDrag);
    progress        = null;

    snap.removeActionListener(snapListener);
    snap            = null;

  /*  view.removeMode(walk);
    view.removeMode(study);
    view.removeMode(fixedCamMode);
    view.removeMode(botAmode);
    view.removeMode(botBmode);
    view.setSelectedMode(null);  */
    view.onClose();
    view.removeActionListener(this);
    view            = null;

    info.removeActionListener(infoListener);
    info            = null;

    back            = null;
    fps              = null;
    backAction       = null;


    //logo = null;
    /*
    botAHealth = null;
    botBHealth = null;
    botAHealthIcon = null;
    botBHealthIcon = null;
    botAName = null;
    botBName = null;
    */

    botAConsole.cleanse();
    botBConsole.cleanse();
    botAConsole = null;
    botBConsole = null;
    
    ref = null;

    //winnerLabel = null;

    walk = null;
    study = null;
    botAmode = null;
    botBmode = null;
   // fixedCamMode = null;
    autoCamMode = null;

   // soundManager = null;


   slowmo.removeActionListener(slowmoListener);
   slowmo = null;
   slowmoListener = null;

   // clearing the iconcache will happen in  BGlobal.clearAllHardCaches();
    BGlobal.clearAllHardCaches();
    super.onClose();

    BDebug.print ("BBattleScreen.onClose gc");
    System.gc();



  }


  // Canvas viewing modes...
  // -------------------------------------------------------------
  /*
  protected class BFixedCamMode extends BMode
  {
    public BFixedCamMode()
    {
      name = "Fixed camera";
      icon = res.getIcon("fixedCamNavMode.icon");
    }

    public void onActivate()
    {
      super.onActivate();
      arenaCanvas.setMode(BGlobal.CANVAS_MODE_ARENA_FIXEDCAM);

    }
  }    */


  protected class BWalkNavigationMode extends BMode
  {
    public BWalkNavigationMode()
    {
      name = "Walking";
      icon = res.getIcon("walkNavMode.icon");
    }

    public void onActivate()
    {
      super.onActivate();
      arenaCanvas.setMode(BGlobal.CANVAS_MODE_ARENA_HUMAN);

      // arenaCanvas.setCameraXZ(arena.getCameraX(), arena.getCameraZ());
      // arenaCanvas.lookAt(false);

    }
  }

  protected class BStudyNavigationMode extends BMode
  {
    public BStudyNavigationMode()
    {
      name = "Study";
      icon = res.getIcon("studyNavMode.icon");
    }

    public void onActivate()
    {
      super.onActivate();
      arenaCanvas.setMode(BGlobal.CANVAS_MODE_ARENA_AUTOCENTER);
      arenaCanvas.lookAt(battleRecording.getBotA(), battleRecording.getBotB(), true);
    }
  }

  protected class BBotNavigationMode extends BMode
  {
    private BBot theBot;

    public BBotNavigationMode(BBot theBot)
    {
      this.theBot = theBot;
      name = theBot.getName() + " cam";
      icon = res.getIcon("sensorNavMode.icon");
    }

    public void onActivate()
    {
      super.onActivate();
      arenaCanvas.setBotCam(theBot);
      arenaCanvas.setMode(BGlobal.CANVAS_MODE_ARENA_BOTCAM);
    }
  }

  protected class BAutoCamMode extends BMode
  {
    private BBot bot1;
    private BBot bot2;

    public BAutoCamMode(BBot bot1, BBot bot2)
    {
      this.bot1 = bot1;
      this.bot2 = bot2;

      name = "TV cam";

      icon = res.getIcon("autoCamNavMode.icon");
    }

    public void onActivate()
    {
      super.onActivate();
      arenaCanvas.setAutoCam(bot1, bot2);
      arenaCanvas.setMode(BGlobal.CANVAS_MODE_ARENA_AUTOCAM);
    }
  }


  // Information dialog class...
  // -------------------------------------------------------------

  protected BInfoDialog getInfoDialog()
  {
    if (infoDialog == null)
    {
      infoDialog = new BInfoDialog();
      infoDialog.addActivationListener(new InfoDialogListener());
    }

    return infoDialog;
  }

 


    protected class BInfoDialog extends BSystemDialog
  {

    // UI Resources...
    // -------------------------------------------------------------

    protected BUIResources res = new BUIResources( new Object[]
    {
      "chassisMove.height",             new Integer(20),
      "chassisMove.foreground",         colors.getMidShade(),

      "commentsScroller.height",        new Integer(100),

      "zones.showText",                 Boolean.FALSE,
      "zones.icon",                     BGraphics.getHSBIcon("zones_icon_small.gif", colors.getMidShade()),
      "zones.tooltip",                  "Show zones",
      "zones.height",                   new Integer(20),
      "zones.width",                    new Integer(20),


      "sensors.showText",               Boolean.FALSE,
      "sensors.icon",                   BIconCache.getHSB("sensors_icon_small.gif"),
      "sensors.tooltip",                "Show sensor coverage",
      "sensors.height",                 new Integer(20),
      "sensors.width",                  new Integer(20),


      "labels.showText",               Boolean.FALSE,
      "labels.icon",                   BIconCache.getHSB("ai/show_labels.gif"),
      "labels.tooltip",                "Show labels in the ai tree",
      "labels.height",                 new Integer(20),
      "labels.width",                  new Integer(20),

    });

    protected InfoDialogClient infoClient;


    public BInfoDialog()
    {
      super("battle information", true);

      int dialogHeight = 370;
      int dialogWidth = 370;
      setSize(new Dimension(dialogWidth, dialogHeight));

      int diay = (arenaCanvas.getHeight() - dialogHeight) / 2;
      int diax = arenaCanvas.getWidth() - diay - dialogWidth;
      setLocation(diax, diay);

      infoClient = new InfoDialogClient();
      setClient(infoClient);

      modal    = false;
      blocking = false;
    }

    public void onActivate()
    {
      super.onActivate();
      onFrameChanged(currentFrame);
    }

     public void onDeactivate()
    {
      super.onDeactivate();
      // signal repaint
      arenaCanvas.repaint();
    }

    public void onFrameChanged(int i)
      { infoClient.onFrameChanged(i); }


    protected class InfoDialogClient extends SystemDialogClient
    {
      protected TableModel channelModel =
        battleRecording.getChannelsModel();

      
      protected BTabbedPane tabs             = new BTabbedPane();
      protected BTable channels              = new BTable(channelModel);
      protected BDisplayOnlyDecisionTree aiTree;
      protected BList  moves                 = new BList();
      protected BList  comments              = new BList();
      protected BLabel chassisMove           = new BLabel(res, "chassisMove");

      protected BScrollPane channelScroller  = new BScrollPane(channels);
      //protected BScrollPane stateScroller    = new BScrollPane(states);
      protected BScrollPane movesScroller    = new BScrollPane(moves);
      protected BScrollPane commentsScroller = new BScrollPane(res, "commentsScroller", comments);

      protected BToggleButton zones   = new BToggleButton(res, "zones");
      protected BToggleButton sensors = new BToggleButton(res, "sensors");
      protected BToggleButton labels = new  BToggleButton(res, "labels");


      public InfoDialogClient()
      {
        super(BoxLayout.X_AXIS);

        layoutInfoChildren();
        configInfoChildren();
      }

      protected void layoutInfoChildren()
      {
        BBoxPanel toolBar = new BBoxPanel(BBoxPanel.Y_AXIS);
          toolBar.addBigMargin();
          toolBar.add(labels);
          toolBar.addSmallMargin();
          toolBar.add(zones);
          toolBar.addSmallMargin();
          toolBar.add(sensors);
          toolBar.addGlue();
        add(toolBar);

        addSmallMargin();

        BDecisionTreeAI ai = (BDecisionTreeAI) battleRecording.getStats(battleRecording.getBotToShowStatsFor() ).getAI();

        if(ai != null)
        {
          battleRecording.getBotToShowStatsFor().setAI(ai);
          ai.setBot(battleRecording.getBotToShowStatsFor());

        }
        BRootNode root;

        if (ai != null)
          root = ai.getRoot();
        else
          root = new BRootNode();

        aiTree =  new BDisplayOnlyDecisionTree( root );

        BScrollPane treePane = new BScrollPane(aiTree);
        treePane.getVerticalScrollBar().addMouseListener( new MouseAdapter()
        {
            public void mouseReleased(MouseEvent e)
            {
              aiTree.repaint();
            }
        }

        );
        treePane.getHorizontalScrollBar().addMouseListener( new MouseAdapter()
        {
            public void mouseReleased(MouseEvent e)
            {
              aiTree.repaint();
            }
        }

        );

        BBoxPanel moveTab = createVBoxPanel();
        moveTab.setEmptyBorder(MARGIN);

        moveTab.addHGlued(new BLabel("Chassis move:"));
        moveTab.addHGlued(chassisMove);
        moveTab.addSmallMargin();

        moveTab.addHGlued(new BLabel("Current moves:"));
        moveTab.add(movesScroller);

        tabs.addTab("ai", treePane);
        tabs.addTab("world", channelScroller);
        tabs.addTab("action", moveTab);
        tabs.addTab("comment", commentsScroller);

        add(tabs);



        /*
        BBoxPanel toggleBox = new BBoxPanel(BBoxPanel.Y_AXIS);
        toggleBox.addBigMargin();
        toggleBox.add(labels);
        toggleBox.addSmallMargin();
        toggleBox.add(zones);
        toggleBox.addSmallMargin();
        toggleBox.add(sensors);
        toggleBox.addGlue();

        BBoxPanel toggleAndComments = new BBoxPanel(BBoxPanel.X_AXIS);
        toggleAndComments.add(toggleBox);
        toggleAndComments.addTinyMargin();

        BBoxPanel comments = new BBoxPanel(BBoxPanel.Y_AXIS);
        comments.addHGlued(new BLabel("Comments:"));
        comments.add(commentsScroller);
        toggleAndComments.add(comments);

        add(toggleAndComments);    */

      }

      protected void configInfoChildren()
      {
        channelScroller.setVerticalScrollBarPolicy(BScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        channelScroller.setBorder(null);

        channels.setRowSelectionAllowed(true);
        channels.setColumnSelectionAllowed(false);
        channels.setCellSelectionEnabled(false);
        channels.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
        channels.setEnabled(false);
        channels.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        channels.getColumnModel().getColumn(1).setMaxWidth(100);

        zones.addActionListener(new ShowZonesListener());
        sensors.addActionListener(new ShowSensorsListener());
        labels.addActionListener(new ShowLabelsListener() );

        labels.setSelected(BGlobal.SHOW_AI_LABELS);
        // show flow through, but it does not.
        aiTree.setTextVisible(BGlobal.SHOW_AI_LABELS);

        labels.setBorderPainted(false);

        // Set Fixed size
        // XXX To fix nullpointer on SetPreferedSize();
        moves.setFixedCellWidth(100);
        moves.setFixedCellHeight(20);

        comments.setFixedCellWidth(100);
        comments.setFixedCellHeight(20);

      }

      protected void onFrameChanged(int i)
      {
          // YLY onFrameChanged, stats is out by 1;
          battleRecording.updateStatsModels(i);
          chassisMove.setText(battleRecording.getChassisMove(i));
          moves.setListData(battleRecording.getMoves(i));
          comments.setListData(battleRecording.getComments(i));
          BDecisionTreeAI ai = battleRecording.getStats(battleRecording.getBotToShowStatsFor()).getAI();
          if (ai != null)
          {
            ai.updateVisited(battleRecording.getStats(battleRecording.getBotToShowStatsFor()).getCacheVisitedValues());
            aiTree.repaint();
          }
       // }
        
      }


       class ShowLabelsListener implements ActionListener
      {
        public void actionPerformed(ActionEvent e)
        {
            aiTree.setTextVisible(labels.isSelected());

        }
      }


      class ShowZonesListener implements ActionListener
      {
        public void actionPerformed(ActionEvent e)
        {
          if (battleRecording.getBotToShowStatsFor() != null)
          {
            if (zones.isSelected() )
            {
              battleRecording.getBotToShowStatsFor().attachZones(true, true);
              battleRecording.getBotToShowStatsFor().setZonesVisible(true);
            }
            else
              battleRecording.getBotToShowStatsFor().detachZones();
          }

        }
      }

      class ShowSensorsListener implements ActionListener
      {
        public void actionPerformed(ActionEvent e)
        {
          if (battleRecording.getBotToShowStatsFor() != null)
          {
            if (sensors.isSelected() )
            {
              battleRecording.getBotToShowStatsFor().addRangeDisplay();
            }
            else
              battleRecording.getBotToShowStatsFor().removeRange();
          }


        }
      }

    }

  }


  //--------------------------------------------------------------------------------------------


  private final void playFrame(float frame, int mode, int skipStartFrame)
  {
    if (frame < -1)
    {
      BDebug.print("BBAttleScreen.playFrame frame start" + frame );

    }

    int frameToPlay = (int) Math.floor(frame);

      if (frameToPlay < -1)
    {
      BDebug.print("BBAttleScreen.playFrame frameToPlay floored" + frameToPlay );

    }

    if ((frameToPlay ==0) || (lastFramePlayed != frameToPlay))
    {
      if (winnerScreenVisible && (mode == BBattleRecorder.APPLYMODE_RANDOM))
        if (frameToPlay < battleRecording.getEndBattleIndex())
          removeEndBattleExtras();

      if (startAnimation != null)
      {
        battleAnimator.remove(startAnimation);
        startAnimation = null;
      }

      if (blockingAnimation != null)
      {
        if(!blockingAnimation.animate() )
          clearInterrupt();
        arenaCanvas.updateCamera();
        return;

      }

      // play the frame
      battleRecording.apply(mode, frameToPlay,  skipStartFrame);

      arenaCanvas.updateCamera();

      lastFramePlayed = frameToPlay;
      progressDisplayTimer += 1;

      boolean isTimeToUpdate =
        (progressDisplayTimer >= framesPerProgressUpdate) ||
        (frameToPlay >= framesToPlay-1) ||
        (mode == BBattleRecorder.APPLYMODE_RANDOM);

      if (isTimeToUpdate)
      {
        updateUIForFrame(frameToPlay);
        progressDisplayTimer = 0;
      }

	    if (infoDialog != null)
  	    if (infoDialog.isActive())
    	    infoDialog.onFrameChanged(currentFrame);

    	if (frameToPlay == framesToPlay - END_FRAME_OFFSET)
      	addEndBattleExtras();
	   }

     else if (isInterpolating || isSlowMotion)
     {
     		// Calculate current (fractional) frame position...
        float df = frame - frameToPlay;

        // Range check on df...
        if ((df > 0f) && (df < 1f))
        {
	        // Apply interpolated frame...
    	    battleRecording.interpolate(frameToPlay+1, df);

      	  // Update the camera...
        	arenaCanvas.updateCamera();
        }
     }
   }



  public void playNextFrame()
  {
    if (framesToPlay-1 > currentFrame)
    {


      java.lang.Math.min(framesToPlay-1,  currentFrame++);
      if (currentFrame < -1)
      {
        BDebug.print("BBAttleScreen.playNextFrame currentFrame " + currentFrame );
      }
      
      playFrame(currentFrame, BBattleRecorder.APPLYMODE_PLAY, currentFrame);
    }
  }

  public void playPreviousFrame()
  {
    if (currentFrame > 0)
    {
      currentFrame--;
      if (currentFrame < -1)
      {
        BDebug.print("BBAttleScreen.playPreviousFrame currentFrame " + currentFrame );
      }
      playFrame(currentFrame,BBattleRecorder.APPLYMODE_RANDOM, currentFrame);
    }
  }


  public void play()
  {

    progressDisplayTimer = 99;

    // Skip to start of battle, if necessary.
    if (currentFrame == lastFrame)
    {
      currentFrame = 0;
      
      playFrame(0,  BBattleRecorder.APPLYMODE_RANDOM, 0);

    }

    // Continuous play from the currentFrame position.
    battleAnimator.start();
    battleAnimator.add(this);
  }

  public void stopBattle()
  {
    progressDisplayTimer = 99;

    crowd.stop();
    if(!idleSound.isRunning() )
      idleSound.play();

    battleAnimator.remove(this);
    battleAnimator.remove(endAnimation);

    endAnimation.clear();

    if (startAnimation != null)
    {
      battleAnimator.remove(startAnimation);
      startAnimation = null;
    }
  }


  protected class ActionsAnimation implements BAnimation
  {

    public ActionsAnimation()
    {
    }

    public void start()
      { arena.start(); }

    public boolean animate()
    {
      // Crowd model...
      crowd.executeActions();

       // SJS 20/02/2001 - Added to give ref opportunity to act each frame.
      ref.executeActions();

      // SJS 22/02/2001 - Animate the arena...
      arena.animate();

      return true;
    }

    public void stop()
      { arena.stop(); }
  }


  public void addEndBattleExtras()
  {

    if (!winnerScreenVisible)
    {
      winnerScreenVisible = true;

      crowd.processEvent(endEvent);
      ref.processEvent(endEvent);

      // Start up the end-battle animation...
      this.interrupt(endAnimation);
      // this.interrupt(refAnimation);
    }

  }

  public void removeEndBattleExtras()
  {
    clearInterrupt();

    battleAnimator.remove(endAnimation);

    // Cleanup any shit left by the ending animation.
    endAnimation.cleanup();


    if (battleRecording.getVictoriousBot() != null)
    {
        BBot loser;
        if (battleRecording.getVictoriousBot() == battleRecording.getBotA() )
          loser = battleRecording.getBotB();
        else
          loser = battleRecording.getBotA();

        loser.restoreTransparency();
    }

    winnerScreenVisible = false;

    if (!battleAnimator.contains (this) )
    {
      // stop the crowd sounds and stuff
      stopBattle();
    }

  }


  public void start()
  // implementation of inteface BAnimation
  // start of the animation
  {
    resetTimingInfo();
    resetAmbientSounds();

    setUIPlaying(true);
 }


 protected void resetTimingInfo()
 {
    currentTime 	= BClock.time();
    startPlayTime = currentTime;

    startPlayFrame = currentFrame;
    skippedStartFrame = currentFrame;
 }

 protected void resetAmbientSounds()
 {
    idleSound.stop();
    crowd.start();
 }


 public void stop()
	 // implementation of BAnimation.stop
 {
   setUIPlaying(false);
 }


 public boolean animate()
 {
  //soundManager.update();

  if (blockingAnimation != null)
  {

    if(!blockingAnimation.animate() )
      clearInterrupt();

    arenaCanvas.updateCamera();

    return true;

  }
  else
  {

    boolean continueExecuting = true;

    skippedStartFrame = currentFrame + 1;
    skippedStartFrame = Math.min(framesToPlay - 1, skippedStartFrame);
    currentTime       = BClock.time();
    float dt          = (float) (currentTime - startPlayTime) * timeScale;
    float frame       = startPlayFrame + (dt * framesPerSec);

    if (frame > framesToPlay-1)
      frame = framesToPlay-1;
    // frame can go below 0 on computers that do not support QuadWords.
    // These computers will flip when currentTime exceeds 32000
    else if ( frame < 0)
    {
      resetTimingInfo();
      BDebug.print ("BBattleScreen.animate resetTimingInfo Called");
      frame =  currentFrame;
    }

    currentFrame      = (int) Math.floor(frame);
         

     if ((currentFrame >=0)  && (currentFrame < framesToPlay))
    {

      playFrame(frame, BBattleRecorder.APPLYMODE_PLAY, skippedStartFrame);

      // stop at the end
      if (currentFrame >= framesToPlay -1)
        continueExecuting = false;
    }

    return continueExecuting;
  }
 }

 public float getBattleTimeElapsed()
   { return battleRecording.getTimeElapsed(currentFrame); }



  BAnimation blockingAnimation;

  public boolean interrupt (BAnimation a)
  {
    if (blockingAnimation == null)
    {
      blockingAnimation = a;
      blockingAnimation.start();
      return true;

    }
    else
      return false;
  }



  public void clearInterrupt()
  {
    if (blockingAnimation != null)
    {
      blockingAnimation.stop();
    }
    blockingAnimation = null;

    // restart things
    ref.processEvent(clearEvent);

    if (battleAnimator.contains(this) )
      start();


  }

  public void onActivate()
  {
    super.onActivate();
    arenaCanvas.onActivate();


    focusManager.setInstalled(true);

    canvasRefresher.start();

    this.repaint();
    arenaCanvas.repaint();

  }

  public void onDeactivate()
  {
    super.onDeactivate();
    focusManager.setInstalled(false);
  }


  boolean playedBotA = false;
  boolean playedBotB = false;
  boolean playedStartMoves = false;
  boolean playedVictoryMoves = false;


  public void processEvent(BArenaEvent a)
  {
    //BDebug.print ("BBattleScreen processEvent " + a.getArenaEventType() );

    view.removeActionListener(this);
    if (a.getArenaEventType().equals(BArenaEvent.BOTA_STARTMOVE) )
    {
      view.setSelectedMode(botAmode);
      arenaCanvas.translateToBotCam(
        BPhysics.PI + BPhysics.QUARTER_PI / 2,
        BPhysics.QUARTER_PI / 2 );
    }
    else if (a.getArenaEventType().equals(BArenaEvent.BOTB_STARTMOVE) )
    {
      view.setSelectedMode(botBmode);
      arenaCanvas.translateToBotCam(
        BPhysics.PI + BPhysics.QUARTER_PI / 2,
        BPhysics.QUARTER_PI / 2);
    }
    else if (a.getArenaEventType().equals(BArenaEvent.STARTMOVES_FIN) )
    {
//      if (!playedVictoryMoves)
//      {
//        view.setSelectedMode(study);
        if(!playedStartMoves)
        {
          view.setSelectedMode(autoCamMode);
          currentMode = autoCamMode;
          modeIndex = 0;
        }
        else
          view.setSelectedMode(currentMode);
   //        arenaCanvas.setFov(BPhysics.degToRad(50) );
        arenaCanvas.setFov(BPhysics.degToRad(75) );
        arenaCanvas.setConsoleVisible(true);
        playedStartMoves =true;
        playedBotA = true;
        playedBotB = true;
//      }
    }

    else if (a.getArenaEventType().equals(BArenaEvent.VICTORY_MOVES_START) )
    {
      addEndBattleExtras();
    }

    view.addActionListener(this);

  }



  // Start animation class...
  // -------------------------------------------------------------

  protected class StartAnimation implements BAnimation
  {
    BArenaEvent startEvent;
    public StartAnimation()
    {
      startEvent = new BArenaEvent()
      {
        public int getStateModifier()
        { return 0; }

        public String getArenaEventType()
        { return START ; }
      } ;
    }

    public boolean animate()
    {
      arenaCanvas.animateMode();
      return true;
    }

    public void start()
      { ref.processEvent(startEvent); }

    public void stop()
      { ref.processEvent (clearEvent); }
  }


  // End animation classes...
  // -------------------------------------------------------------

  protected class EndBattleAnimation extends BAnimationSequence
  {

    protected BArenaObjectPosition center
      = new BArenaObjectPosition();

    protected VictoryAnimation victory;
      // Victory sequence for the winning bot.
      // May be null in the case of a draw.

    public EndBattleAnimation()
    {
      super(3);

      // Wait for the ref to stop talking...
      add(new EndBattleAnnouncementAnimation());

      // Was there a victor?
      BBot victor = battleRecording.getVictoriousBot();

      if (victor != null)
      {
        // Teleport bots to center of arena...
        BAnimationSet teleport = new BAnimationSet(3);
        teleport.add(new BTeleportBotAnimation(victor, center, fx, 4f));
        teleport.add(new TeleportLoserAnimation());
        teleport.add(new BDelayedAnimation(new SelectStudyCameraAnimation(), 2f));
        add(teleport);

        // Add Victory animation..
        victory = new VictoryAnimation(victor);
        add(victory);
      }
    }

    public void clear()
      { if (victory != null) victory.clear(); }

    public void cleanup()
      { if (victory != null) victory.cleanup(); }
  }


  protected class TeleportLoserAnimation extends BTimeAnimation
  {
    protected BArenaObjectPosition origin
      = new BArenaObjectPosition(0f, 0f, 0f, 0f);

    public void start(float t)
      { }

    public boolean animate(float t, float dt)
      { return (t < 2f); }

    public void stop(float t)
    {
      BBot loser = battleRecording.getLoserBot();
      loser.setArenaPosition(origin);
    }
  }


  protected class EndBattleAnnouncementAnimation extends BAbstractAnimation
  {
    public boolean animate()
      { return ref.isBusy(); }
  }

  protected class VictoryAnimation extends BAbstractAnimation
  {
    protected BAnimationSet     animations;
    protected PedestalAnimation pedestal;

    public VictoryAnimation(BBot victor)
    {
      super();

      animations = new BAnimationSet(4);

      // Standard victory animations.
      animations.add(new EndCameraAnimation());
      animations.add(new EndMusicAnimation());

      // Tournament animation - a rising pedestal...
      if (battleRecording.isTournament())
      {
        pedestal = new PedestalAnimation(victor, 5f);
        animations.add(pedestal);
      }

      // Tournament final animation - fireworks!
      if (battleRecording.isFinal())
        animations.add(new BDelayedAnimation(new FireworksAnimation(), 5f));
    }

    public void start()
    {
      super.start();
      battleAnimator.add(animations);
    }

    public boolean animate()
      { return false; }

    public void clear()
      { battleAnimator.remove(animations); }

    public void cleanup()
    {
      battleAnimator.remove(animations);
      if (pedestal != null)
        pedestal.cleanup();
    }
  }


  protected class PedestalAnimation extends BAnimationSet
  {

    protected float duration = 5f;
      // Duration of raising effect.

    protected BArenaOrnamentAnimation pedestal;
      // Pedestal raising animation.

    protected BArenaOrnamentAnimation ring;
      // Outer ring raising animation.

    protected BAlphaChannelAnimation raise;
      // Bot raising animation.


    protected BArenaObjectPosition origin
      = new BArenaObjectPosition(0f, 0f, 0f, 0f);

    protected BArenaObjectPosition pedestalEnd
      = new BArenaObjectPosition(0f, 107.893f, 0f, 0f);

    protected BArenaObjectPosition ringEnd
      = new BArenaObjectPosition(0f, 18.446f, 0f, 0f);

    protected BArenaObjectPosition botEnd
      = new BArenaObjectPosition(0f, 107.893f, 0f, 0f);


    public PedestalAnimation(BBot bot, float duration)
    {
      super();
      this.duration = duration;

      pedestal = new BArenaOrnamentAnimation("stan_ped_in.cmp", arenaCanvas);
      pedestal.setChannel(new BLerpChannel(origin, pedestalEnd));
      pedestal.setAlphaMap(new BLinearMap(duration, BLinearMap.INFINITE));
      pedestal.setAutoUnloading(false);

      ring = new BArenaOrnamentAnimation("stan_ped_out.cmp", arenaCanvas);
      ring.setChannel(new BLerpChannel(origin, ringEnd));
      ring.setAlphaMap(new BLinearMap(duration/5f, BLinearMap.INFINITE));
      ring.setAutoUnloading(false);

      raise = new BAlphaChannelAnimation();
      raise.setAcceptor(bot);
      raise.setChannel(new BLerpChannel(origin, botEnd));
      raise.setAlphaMap(new BLinearMap(duration, BLinearMap.INFINITE));

      BAlphaChannelAnimation loser = new BAlphaChannelAnimation();
      loser.setAcceptor(battleRecording.getLoserBot());
      loser.setChannel(new BLerpChannel(origin, botEnd));
      loser.setAlphaMap(new BLinearMap(duration, BLinearMap.INFINITE));

      add(pedestal);
      add(ring);
      add(raise);
      add(loser);
    }

    public void cleanup()
    {
      pedestal.setLoaded(false);
      ring.setLoaded(false);
    }
  }


  protected class FireworksAnimation extends BAbstractAnimation
  {
    public boolean animate()
      { fx.spawnFireworks(); return false; }
  }

  protected class SelectStudyCameraAnimation extends BAbstractAnimation
  {
    public boolean animate()
    {
    	arenaCanvas.setSphereCamParams(BModeArenaSphereCam.ZENITH_MIN, 0.0);
    	view.setSelectedMode(study);
      return false;
    }
  }

  protected class EndCameraAnimation extends BAbstractAnimation
  {
    public boolean animate()
      { arenaCanvas.animateMode(); return true; }
  }


  protected class EndMusicAnimation implements BAnimation
  {
    protected BFadingSound music;

    public EndMusicAnimation()
    {
      music = BFadingSound.getFadingSound(
        arena.getVictorySoundFullPath(),
        BSound.LOOP_CONTINUOUSLY,
        BSound.SOUND_TYPE_FOREGROUND,
        4000, 2000);
    }

    public void start()
      { music.play(); }

    public boolean animate()
      { return true; }

    public void stop()
      { music.fadeOut(battleAnimator); }
  }


}
