package roboforge.ui;

import roboforge.B3dShapeManager;
import roboforge.BComponent;
import roboforge.BChassis;
import roboforge.BConnectPoint;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BPhysics;

import roboforge.ui.components.*;
import roboforge.util.io.*;
import roboforge.ui.modes.*;
import roboforge.math.BMath;
import roboforge.util.graphics.BGraphics;

import roboforge.util.text.BStrings;

import java.io.*;
import java.awt.BorderLayout;
import java.awt.event.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.Vector;
import java.awt.Cursor;


import javax.swing.*;
import javax.swing.event.*;



public class BComponentLibrary extends BToolPanel
  implements SwingConstants
{

  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "genreLabel.text",        "Race: ",
    "genreLabel.h_align",     new Integer(RIGHT),
    "genreLabel.width",       new Integer(35),
    "genre.height",           new Integer(18),

    "compTypeLabel.text",     "Type: ",
    "compTypeLabel.h_align",  new Integer(RIGHT),
    "compTypeLabel.width",    new Integer(35),
    "compType.height",        new Integer(18),

    "scroller.background",    colors.getVeryDarkShade(),
    "inspector.tooltip",      "Component inspector",
    "inspector.height",       new Integer(210),
    "attribs.background",     colors.getVeryDarkShade(),
    "blank.background",       colors.getVeryDarkShade(),

    "3dcanvas.background",    BGlobal.colors.getExtraDarkShade(),
    "3dcanvas.foreground",    BGlobal.colors.getWhiteShade(),

    "costLabel.width",        new Integer(70),
    "healthLabel.width",      new Integer(50),
    "weightLabel.width",      new Integer(50),

    "infoBox.height",         new Integer(20),
    "infoBox.background",     BGlobal.colors.getVeryDarkShade(),
  });


  // Visual elements...
  // -------------------------------------------------------------

  protected BFileList components      = new BFileList(res, "components");

  protected BLabel genreLabel         = new BLabel(res, "genreLabel");
  protected BFileComboBox genre       = new BFileComboBox(res, "genre");

  protected BLabel compTypeLabel      = new BLabel(res, "compTypeLabel");
  protected BFileComboBox compType    = new BFileComboBox(res, "compType");

  protected BScrollPane scroller      = new BScrollPane(res, "scroller", components);
  protected BTabbedPane inspector     = new BTabbedPane(res, "inspector");

  protected ComponentViewer viewer    = new ComponentViewer();
  protected BComponentAttributeViewer attribs   = new BComponentAttributeViewer(res, "attribs");


  // Constructor.
  // -------------------------------------------------------------

  public BComponentLibrary (B3DEditingPanel parent)
  {
    super(parent, "components");

    layoutComponentLibChildren();
    configComponentLibChildren();

    res.cleanse();
    res = null;
  }


  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutComponentLibChildren()
  {
    addLinear(compTypeLabel, getHStrut(TINY_MARGIN), compType);
    addSmallMargin();

    addLinear(genreLabel, getHStrut(TINY_MARGIN), genre);
    addSmallMargin();

    scroller.setVerticalScrollBarPolicy
      (BScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

    addVGlued(scroller);
    addMargin();

    inspector.addTab("3d view", viewer);
    inspector.addTab("attributes", attribs);

    add(inspector);
  }

  protected void configComponentLibChildren()
  {
    String[] extensions = new String[2];
    extensions[0] = BGlobal.COMPONENTSUFFIX;
    extensions[1] = BGlobal.CHASSISSUFFIX;


    if (BGlobal.DEVELOPER_MODE)
    {
      compType.setFilter(new BBasicFilter());
      genre.setFilter(new BBasicFilter());
      components.setFilter(new BBasicFilter(extensions));
    }
    else
      components.setFilter(new BRoboforgeFilter(extensions));


    try
    {
      // Initialize the component category combo-boxes...
      compType.setSourcePath(BGlobal.COMPONENTPATH);

      if (!compType.isEmpty())
      {
        compType.setSelectedIndex(0);
        String firstType = compType.getFileAt(0);
        genre.setSourcePath(BGlobal.COMPONENTPATH+ firstType+ "/");

        // Select first genre, by default.
        if (!genre.isEmpty())
          genre.setSelectedIndex(0);
      }
    }
    catch (FileNotFoundException e)
      { BDebug.print("BAppearanceChooser.<init>: "+e); }

    genre.addActionListener(new GenreListener());
    compType.addActionListener(new TypeListener());
    components.addListSelectionListener(new ComponentsListener());

    // load a component
 //   components.setSelectedIndex(0);
  //  loadSelectedComponent();
    // Initialize component inspector...
    // inspector.addChangeListener(new InspectorListener());
  }



  // Accessor methods...
  // -------------------------------------------------------------

  public void setVisible(boolean v)
  {
    boolean old = this.isVisible();

    if (viewer != null)
      viewer.setCanvasVisible(v);

    super.setVisible(v);
    if (propertyManager != null)
      propertyManager.firePropertyChange("componentLibrary", old, v);


  }

  public void lookup(BComponent c)
  {
    try
    {
      String type = c.getType();

      if (c.isInternal())
        return;

      if (c instanceof BChassis)
      {
        // TODO: Remove...
        BDebug.print("WARNING: Remove BChassis from component list!");
      }
      else
      {
        //BDebug.print ("BComponentLibrary lookup type " + type );
        //BDebug.print ("BComponentLibrary genre " + c.getGenre() );
        //BDebug.print ("BComponentLibrary c.getFileName() " + c.getFileName().replace(BStrings.SPACE,BStrings.UNDERSCORE) );

       
        compType.setSelectedFile(type);
        genre.setSelectedFile(c.getGenre());
        components.setSelectedFile(c.getFileName().replace(BStrings.SPACE,BStrings.UNDERSCORE) );

        

      }
    }
    catch (Exception e)
      { };
  }

  protected String getSelectedComponentPath()
  {
    return
      BGlobal.COMPONENTPATH +
      compType.getSelectedFile() + "/" +
      genre.getSelectedFile()+ "/";
  }

  protected void updateComponentList()
  {
    try
    {
      if (genre.isEmpty())
        components.removeAllFiles();
      else
        components.setSourcePath(getSelectedComponentPath());

    }
    catch (FileNotFoundException e)
      { BDebug.print("BComponentLibrary.updateComponentList: "+e); }
  }

  protected void loadComponent(String fileName)
  {
      File f = new File(fileName);

      if (f.isFile())
      {
        // Instantiate Viewable Component Object
        BComponent c = BComponent.loadFromFile(fileName);

        attribs.setComponent(c);
        viewer.setComponent(c);
      }
  }


  // UI Listeners...
  // -------------------------------------------------------------

  protected class TypeListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      try
      {
        String type    = compType.getSelectedFile();
        String current = genre.getSelectedFile();

        genre.setSourcePath(BGlobal.COMPONENTPATH+ type);
        genre.setSelectedFile(current);
        updateComponentList();
      }
      catch (FileNotFoundException ex)
        { BDebug.print("BAppearanceChooser.TypeListener: "+ex); }
    }
  }

  protected class GenreListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      { updateComponentList(); }
  }

  protected class ComponentsListener implements ListSelectionListener
  {
    public void valueChanged(ListSelectionEvent e)
    {
      loadSelectedComponent();
    }
  }


  protected void loadSelectedComponent()
  {
      String path      = getSelectedComponentPath();
      String component = components.getSelectedFile();
      loadComponent(path+ component);

  }


  // Selection stuff
  // -------------------------------------------------------------

  public void setSelectedComponent(BComponent c)
    { viewer.canvas.setSelectedComponent(c); }

  public BComponent getSelectedComponent()
    { return viewer.canvas.getSelectedComponent(); }

  public void setSelectedConnectPt(BConnectPoint c)
    { viewer.canvas.setSelectedConnectPt(c); }

  public BConnectPoint getSelectedConnectPt()
    { return viewer.canvas.getSelectedConnectPt(); }


  // Property change stuff
  // -------------------------------------------------------------

  public void addPropertyChangeListener(PropertyChangeListener c)
  {    
    viewer.canvas.addPropertyChangeListener(c);
    super.addPropertyChangeListener(c);
  }


  // Activation methods.
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();

    if (isVisible())
      viewer.onActivate();

    updateComponentList();
    
  }

  public void onDeactivate()
  {
    viewer.onDeactivate();
    super.onDeactivate();
  }

  public void onClose()
  {
    viewer.onClose();
    super.onClose();
  }


  // -------------------------------------------------------------
  // Inner class: Component viewer panel...
  // -------------------------------------------------------------

  protected class ComponentViewer extends BPanel implements MouseMotionListener
  {

    // Internal variables...
    // -------------------------------------------------------------

    protected BComponent oldComponent   = null;
    protected B3DComponentCanvas canvas = null;
    protected BLabel costLabel          = new BLabel(res, "costLabel");
    protected BLabel healthLabel        = new BLabel(res, "healthLabel");
    protected BLabel weightLabel        = new BLabel(res, "weightLabel");
    BModeMouse componentRot;

    protected Cursor dragCursor = BGraphics.getCursor("drag_cursor.gif", 0,0);

    public ComponentViewer()
    {
      super();

      // Initialize canvas.
      canvas = new B3DComponentCanvas(res, "3dcanvas");

      // Set the canvas mode to View Rotation.
      componentRot = new BModeComponentStudy(canvas);
      canvas.setMode(componentRot);

      BBoxPanel infoBox =
        new BBoxPanel(res, "infoBox", BoxLayout.X_AXIS);

      infoBox.addGlue();
      infoBox.addVGlued(costLabel);
      infoBox.addVGlued(weightLabel);
      infoBox.addVGlued(healthLabel);

      setLayout(new BorderLayout());
      add("Center", canvas);
      add("South",  infoBox);

      canvas.addMouseMotionListener(this);
    }

    public void setComponent(BComponent c)
    {
      if (oldComponent != null)
      {
        canvas.removeComponent(oldComponent);
        oldComponent.cleanse();
      }

      canvas.addComponent(c);

      // set a default connect point as active
     // BConnectPoint cp = c.getFreeConnectPoint();
      //c.setConnectPointSelected(cp);
      setSelectedComponent(c);
      //setSelectedConnectPt(cp);

      costLabel.setText(BMath.format(c.getPrice())+ " " + BGlobal.MONEY_UNIT);
      healthLabel.setText(c.getDamageMax()+ " hp");
      weightLabel.setText(BMath.format(c.getWeight())+ " kg");

      oldComponent = c;
    }

    public void mouseDragged(MouseEvent e)
    {
    }

    public void mouseMoved(MouseEvent e)
    {
      //BDebug.print("BComponentLibrary - mouseMoved over canvas");
      if (canvas.pickComponent(e.getPoint(), false) != null)
        canvas.setCursor(dragCursor);
      else
        canvas.setCursor(Cursor.getDefaultCursor());
    }

    // Activation methods.
    // -------------------------------------------------------------

    public void onActivate()
    {
      if (isVisible())
      {
        canvas.setVisible(true);
        canvas.requestActivate();
      }

      this.repaint(0,0,800,600);
      canvas.repaint();

    }

    public void onDeactivate()
    {
      canvas.setVisible(false);
      canvas.requestDeactivate();
    }

    public void onClose()
    {
      canvas.requestClose();
      remove(canvas);
      canvas = null;
    }

    public void setCanvasVisible(boolean v)
    {
      if (canvas != null)
        if (isVisible())
          canvas.setVisible(v);
    }

    public void setVisible(boolean v)
    {
      super.setVisible(v);

      if (canvas != null)
        canvas.setVisible(v);

      if (v)
        onActivate();
      else
        onDeactivate();   
    }
  }





}
