package roboforge.ui;

/*
* File:
* BMainScreen.java
*
* Description:
* Main screen; the first interface screen displayed by the application.
*
* Modification History: ( Name, Date, Notes )
* Created SJS, 27/01/2000.
*
*/

import roboforge.BBattleRecorder;
import roboforge.BBot;
import roboforge.BChassis;
import roboforge.BDebug;
import roboforge.BGhost;
import roboforge.BGlobal;
import roboforge.BProgressMonitorable;
import roboforge.BWizardTemplateBot;

import roboforge.net.BNetException;
import roboforge.net.client.*;

import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.ui.net.*;

import roboforge.util.graphics.*;
import roboforge.util.io.*;
import roboforge.util.processors.*;

import java.awt.*;
import java.awt.event.*;
import java.io.File;

import java.util.Vector;
import javax.swing.*;
import javax.swing.border.*;
//import javax.media.j3d.VirtualUniverse;



public class BMainScreen extends BSystemScreen
{

  // Internal variables...
  // -------------------------------------------------------------

  // UI Resources ...
  // -------------------------------------------------------------

  protected static int BUTTONS_TOP = 40;
  

  protected BUIResources res = new BUIResources( new Object[]
  {
    "titleIcon.icon",             BIconCache.getHSB("logo_icon.gif"),
    "titleLabel.icon",            BIconCache.getHSB("main_title.gif"),

    "background.icon",            BGraphics.getHSBIcon("background_main_new_07.jpg"), // background_main.jpg"),
    "background.foreground",      colors.getMidShade(),
    "background.bounds",          new Rectangle(0, 1, 800, 450),

    "subTitleLabel.text",         "robotic combat sport v"+ BGlobal.APP_VERSION ,
    "subTitleLabel.font",         new Font(BGlobal.FONT_NAME, Font.PLAIN, 18),
    "subTitleLabel.antialiased",  new Boolean(true),
    "subTitleLabel.border",       new EmptyBorder(0, 0, 15, 0),

    "workshop.bounds",            new Rectangle(50, 50, 355, 70),
    "workshop.icon",              BGraphics.getUIImage("workshop_icon.gif"),
    "workshop.title",             BGraphics.getUIImage("workshop_small_title.gif"),
    "workshop.subtitle",          "Build and maintain combat bots...",
    
    "tournaments.bounds",         new Rectangle(50, 145, 300, 70),
    "tournaments.icon",           BGraphics.getUIImage("tournament_icon.gif"),
    "tournaments.title",          BGraphics.getUIImage("tournament_small_title.gif"),
    "tournaments.subtitle",       "Enter bots in online tournaments...",

    "archives.bounds",            new Rectangle(50, 240, 235, 70),
    "archives.icon",              BGraphics.getUIImage("archives_icon.gif"),
    "archives.title",             BGraphics.getUIImage("archives_small_title.gif"),
    "archives.subtitle",          "View recorded bot battles...",

    "options.bounds",             new Rectangle(50, 335, 330, 70),
    "options.icon",               BGraphics.getUIImage("options_icon.gif"),
    "options.title",              BGraphics.getUIImage("options_small_title.gif"),
    "options.subtitle",           "Configure Roboforge...",

//    "creditsAction.name",         "Credits",
//    "creditsAction.icon",         BGraphics.getHSBIcon("credits.gif"),
//    "creditsAction.tooltip",      "Credits screen",

    "credits.bounds",             new Rectangle(620, 410, 660, 425),
    "credits.icon",               BGraphics.getUIImage("net_fight_icon.gif"),
    "credits.title",              BGraphics.getUIImage("credits.gif"),
    "credits.subtitle",           "",

  });


  // Visual elements ...
  // -------------------------------------------------------------
  protected JLabel  subTitleLabel       = new BLabel(res, "subTitleLabel");
  protected JLabel  background          = new BLabel(res, "background");

  protected ButtonGroup    buttons      = new ButtonGroup();
  
  protected AbstractButton none         = new JToggleButton();
  protected BSectionButton workshop     = new BSectionButton(res, "workshop");
  protected BSectionButton tournaments  = new BSectionButton(res, "tournaments");
  protected BSectionButton archives     = new BSectionButton(res, "archives");
  protected BSectionButton options      = new BSectionButton(res, "options");

//  protected CreditsAction creditsAction = new CreditsAction(res, "creditsAction");
//  protected BActionButton creditsButton = new BActionButton(creditsAction);
  protected BSectionButton credits      = new BSectionButton(res, "credits");

  protected String W2K_TOKEN = "Windows 2000";

  // Constructor.
  // -------------------------------------------------------------

  public BMainScreen()
  {
    super(false);

    //VirtualUniverse.setJ3DThreadPriority(Thread.MIN_PRIORITY );
    BGlobal.J3DENGINESTARTED = true;


    layoutMainScreenChildren();
    configMainScreenChildren();

    res.cleanse();
    res = null;
    runBLTThingy();

        /*
    String os = System.getProperty("os.name");
    if (W2K_TOKEN.equals(os) )
      if (BGlobal.VIDEOMODE.equalsIgnoreCase(BGlobal.VIDEOMODE_OPENGL) )
      {
       // BPromptDialog vidWarning = BPromptDialog.

        createNotifyDialog("WARNING", "Roboforge has detected that you are a windows 2000 user running Roboforge with the openGL video option. Please switch to the direct X video mode, ( press the 'Options' button, then select Direct X )");

        //BApplication.showDialog(vidWarning);

      }

    BDebug.print ("BMainScreen Operating system=" + os); */



  }

  private static final String introPath =  "btl/intro.exe";

  private void runBLTThingy()
  {
      try
          {


            File introTester =  new File( BGlobal.SYSTEMPATH + introPath);
            BDebug.print("testing " + introTester);


            if (introTester.exists() )
            {
              BDebug.print("Executing " + BGlobal.SYSTEMPATH + introPath);
              Runtime.getRuntime().exec(BGlobal.SYSTEMPATH + introPath);
            }
          }
          catch (Exception ex)
            { BDebug.print(ex.getMessage() );  }

  }


  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutMainScreenChildren()
  {
    titleIcon.setIcon(res.getIcon("titleIcon.icon"));
    titleLabel.setIcon(res.getIcon("titleLabel.icon"));

    subTitle.addHVGlued(subTitleLabel);

    body.setLayout(null);

    body.addOnTop(background);

    body.addOnTop(workshop);
    body.addOnTop(tournaments);
    body.addOnTop(archives);
    body.addOnTop(options);    
    body.addOnTop(credits);
  }

  protected void configMainScreenChildren()
  {
    workshop.setAction(new WorkshopAction());
    tournaments.setAction(new TournamentsAction());
    archives.setAction(new ArchivesAction());
    options.setAction(new OptionsAction());
    credits.setAction(new CreditsAction());

    buttons.add(workshop);
    buttons.add(tournaments);
    buttons.add(archives);
    buttons.add(options);
    buttons.add(credits);
    buttons.add(none);
  }


  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();
    setStatus("");

    // Reload background graphic...
    if (background.getIcon() == null)
      background.setIcon(
        BIconCache.getHSB("background_main_new_07.jpg"));



  }


  public void onDeactivate()
  {
    //BDebug.print ("onDeactivate Called ");
    none.setSelected(true);

    // Deallocate background graphic...
    background.setIcon(null);

    super.onDeactivate();
  }
  
  
  

  // Workshop Action classes...
  // -------------------------------------------------------------

  protected class WorkshopAction extends BAction
  {
    public void actionPerformed(ActionEvent e)
    {
      setBusyWorking(true);

      BGarageScreen garage =
        new BGarageScreen();

      BApplication.openScreen(garage);
      setBusyWorking(false);

    }
  }
 
  
  // Tournaments Action class...
  // -------------------------------------------------------------

  protected class TournamentsAction extends BAction
  {
    public void actionPerformed(ActionEvent e)
    {
      BApplication.setBusy(true);
      BClientStack server = null;
      
      try
        { server = new BClientStack(); }
      catch (BNetException ex)
        { BApplication.abort("Could not init net-stack: "+ ex); }

       BLicenceAgreementScreen lScreen = new  BLicenceAgreementScreen(server);
       BApplication.setBusy(false);
       BApplication.openScreen(lScreen);
       //BApplication.closeScreen(this);

      /*
      BPatchScreen patch =
        new BPatchScreen(server);

      BApplication.setBusy(false);
      BApplication.openScreen(patch);   */
    }
  }


  // Credits Action class...
  // -------------------------------------------------------------

  private static final String creditsPath =  "btl/credits.exe";



  private void showAndWaitForbtlCredits()
  {
     try
          {


            File introTester =  new File( BGlobal.SYSTEMPATH + creditsPath);
            BDebug.print("testing " + creditsPath);


            if (introTester.exists() )
            {
              BDebug.print("Executing " + BGlobal.SYSTEMPATH + creditsPath);
              Process flashPlayer = Runtime.getRuntime().exec(BGlobal.SYSTEMPATH + creditsPath);
              flashPlayer.waitFor();


            }
          }
          catch (Exception ex)
            { BDebug.print(ex.getMessage() );  }

  }

  protected class CreditsAction extends BAction
  {
//    public CreditsAction(BUIResources res, String key)
//    { super(res, key); }

    public void actionPerformed(ActionEvent e)
    {
      BCreditsScreen credits =
        new BCreditsScreen();

      showAndWaitForbtlCredits();

      BApplication.openScreen(credits);
    }
  }


  // Archives Action class...
  // -------------------------------------------------------------


  protected class ArchivesAction extends BAction
  {
    public void actionPerformed(ActionEvent e)
    {
      BArchiveScreen archive =
        new BArchiveScreen();

      BApplication.openScreen(archive);
    }
  }
  /*
  protected class ArchivesAction extends BThreadedAction
  {
    public void threadedAction(ActionEvent e)
    {
      String filename = 
        getRecordingFileNameFromUser();

      if (filename != null)
        BBattleScreen.load(filename);
    }
  }  */


  // Options Action class...
  // -------------------------------------------------------------
  
  protected class OptionsAction extends BAction
  {
    public void actionPerformed(ActionEvent e)
    {
      BApplication.setBusy(true);

      BPreferenceScreen options =
        new BPreferenceScreen();

      BApplication.openScreen(options);
      BApplication.setBusy(false);
    }
  }
  


  // Helper methods...
  // -------------------------------------------------------------

  protected String getRecordingFileNameFromUser()
  {
    try
    {
      BFileDialog openDlg = BFileDialog.
        createOpenDialog("open battle", "Please choose a battle to view:");

      openDlg.setFilter(new BRoboforgeFilter(BGlobal.RECORDINGSUFFIX));
      openDlg.setPath(BGlobal.BATTLERECORDINGSPATH);

      BApplication.showDialog(openDlg);

      return openDlg.getSelectedFile();
    }
    catch (Exception e)
      { return null; }
  }
  
}
