package roboforge.ui;

import java.awt.*;
import java.awt.event.*;
import javax.media.j3d.*;
import javax.vecmath.*;
import java.lang.Math.*;
import java.util.*;

import roboforge.ui.*;
import roboforge.ui.events.*;
import roboforge.ui.modes.*;
import roboforge.*;


public class BMouseListenerBehavior extends BMouseMultiBehavior implements BActivateable, MouseMotionListener, MouseListener
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BActivationSupport activation
    = new BActivationSupport(this);

  protected BModeMouse mode;
  protected boolean isMouseInCanvas;
  protected boolean isMousePressed;
  
  protected WakeupOr mouseCritereon;
  protected LinkedList mouseq;



  // Constructor...
  // -------------------------------------------------------------

  // conrtuctor to hack the mouse not waking up bug from j3d beta 1.2.1
  public BMouseListenerBehavior(Component c)
  {
    super();
    c.addMouseListener(this);
	  c.addMouseMotionListener(this);

    setDefaultParams();
  }

  /*
  public BMouseListenerBehavior()
  {
    super();

    setDefaultParams();

  }     */

  private void setDefaultParams()
  {
    this.setSchedulingBounds(BGlobal.INFINITE_BOUNDS);
    this.setMode(null);
    this.isMouseInCanvas = false;
    this.isMousePressed  = false;
  }

  public void initialize()
  {
  /*
    mouseMotionCritereon =
      new WakeupOnAWTEvent(AWTEvent.MOUSE_MOTION_EVENT_MASK);
    
    mouseCritereon =
      new WakeupOnAWTEvent(AWTEvent.MOUSE_EVENT_MASK);
    
    wakeUpConditions =
      new WakeupOr(new WakeupCriterion[]
        { mouseMotionCritereon, mouseCritereon });     */

    WakeupCriterion[] mouseEvents = new WakeupCriterion[3];

    mouseEvents[0] = new WakeupOnBehaviorPost(this,
						  MouseEvent.MOUSE_DRAGGED);
	  mouseEvents[1] = new WakeupOnBehaviorPost(this,
						  MouseEvent.MOUSE_PRESSED);
	  mouseEvents[2] = new WakeupOnBehaviorPost(this,
						  MouseEvent.MOUSE_RELEASED);
	  mouseq = new LinkedList();

    mouseCritereon = new WakeupOr(mouseEvents);
    wakeupOn (mouseCritereon);

   // this.wakeupOn(wakeUpConditions);
  }

  
  
  // Accessor methods...
  // -------------------------------------------------------------
  
  public boolean isMouseInCanvas()
    { return isMouseInCanvas; }

  public BModeMouse getMode()
    { return mode; }

  public void setMode(BModeMouse m)
  {
    BModeMouse old = mode;

    if (old != null)
      old.requestDeactivate();
    
    this.mode = m;
    
    if (isActive())
    {
      boolean isAMode = 
        mode != null;
      
      setEnable(isAMode);
      
      if (isAMode) 
        mode.requestActivate();
    }
  }
  
  // Activation methods...
  // -------------------------------------------------------------

  public boolean isActive()
    { return activation.isActive(); }

  public boolean isClosed()
    { return activation.isClosed(); }

  public void requestActivate()
    { activation.requestActivate(); }

  public void requestDeactivate()
    { activation.requestDeactivate(); }

  public void requestClose()
    { activation.requestClose(); }

  public void onActivate()
  {
    if (mode != null)
    {
      setEnable(true);
      mode.requestActivate();
    }
   
    activation.fireActivate();
  }

  public void onDeactivate()
  {
    if (mode != null)
    {
      setEnable(false);
      mode.requestDeactivate();
    }

    if (activation != null)
      activation.fireDeactivate();
  }

  public void onClose()
    { activation.fireClose(); }

  public void addActivationListener(BActivationListener l)
    { activation.addActivationListener(l); }

  public void removeActivationListener(BActivationListener l)
    { activation.addActivationListener(l); }
  
  
  
  // Implementation methods...
  // -------------------------------------------------------------
  
  public void handleMouseEvent(MouseEvent m)
  {
    int id = m.getID();

    switch (id)
    {
      case MouseEvent.MOUSE_DRAGGED:
        if (isMousePressed)
          mode.mouseDragged(m);
        break;

      case MouseEvent.MOUSE_PRESSED:
        isMousePressed = true;
        mode.mousePressed(m);
        break;

      case MouseEvent.MOUSE_RELEASED:
        isMousePressed = false;
        mode.mouseReleased(m);
        break;

      case MouseEvent.MOUSE_ENTERED:
        isMouseInCanvas = true;
        break;

      case MouseEvent.MOUSE_EXITED:
        isMouseInCanvas = false;
        break;

      case MouseEvent.MOUSE_MOVED:
        break;
    }
  }

  

  public void processStimulus(Enumeration criteria)
  {
    //AWTEvent[] es;

    // BDebug.print("processStimulus()");

    if (mode == null)
      return;


    synchronized (mouseq)
    {
      while(!mouseq.isEmpty())
      {
        MouseEvent e = (MouseEvent) mouseq.getFirst();
        mouseq.removeFirst();
        handleMouseEvent(e);
      }

    }

      /*
    if (mouseCritereon.hasTriggered())
    {
      es = mouseCritereon.getAWTEvent();

      int n = es.length;
      for (int i=0; i<n; i++)
        handleMouseEvent((MouseEvent) es[i]);
    }

    if (mouseMotionCritereon.hasTriggered())
    {
      es = mouseMotionCritereon.getAWTEvent();

      int n = es.length;
      for (int i=0; i<n; i++)
        handleMouseEvent((MouseEvent) es[i]);
    }   */

    wakeupOn(mouseCritereon);
  }



  // mouse listener methods

    public void mouseClicked(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}

    public void mousePressed(MouseEvent e)
    {
     // System.out.println("mousePressed");

	    // add new event to the queue
	    // must be MT safe
	    synchronized (mouseq)
      {
        // post a mouse Release for the to fix the drag and drop bug

	      mouseq.add(e);
	      // only need to post if this is the only event in the queue
	      if (mouseq.size() == 1) postId(MouseEvent.MOUSE_PRESSED);
	    }
    }

    public void mouseReleased(MouseEvent e)
    {
    //  System.out.println("mouseRealeased");
      // add new event to the queue
	    // must be MT safe
	    synchronized (mouseq)
      {
	      mouseq.add(e);
	      // only need to post if this is the only event in the queue
	      if (mouseq.size() == 1) postId(MouseEvent.MOUSE_RELEASED);
	    }
    }

    public void mouseDragged(MouseEvent e)
    {
     // System.out.println("mouseDragged");

	    // add new event to the to the queue
	    // must be MT safe.
	    synchronized (mouseq)
      {
	      mouseq.add(e);
	      // only need to post if this is the only event in the queue
	      if (mouseq.size() == 1) postId(MouseEvent.MOUSE_DRAGGED);
	    }
    }

    public void mouseMoved(MouseEvent e) {}

    public void cleanse()
    {
        activation = null;

        mode = null;

        mouseCritereon = null;
        mouseq = null;


    }


}