package roboforge.ui;

import roboforge.BArena;
import roboforge.BArenaObjectPosition;
import roboforge.BCollisionRecord;
import roboforge.BGlobal;
import roboforge.BBot;
import roboforge.BDebug;
import roboforge.BMoveList;
import roboforge.BMove;
import roboforge.BSnapshot;
import roboforge.BSimulation;

import roboforge.ui.events.*;

import roboforge.util.graphics.*;
import roboforge.util.processors.*;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.*;
import com.sun.image.codec.jpeg.*;


import java.util.Vector;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import javax.vecmath.*;
import javax.media.j3d.*;

public class BMoveController
  extends
    AbstractTableModel

  implements
    ChangeListener,
    BActivateable
{

  // Internal variables...
  // -------------------------------------------------------------

  protected B3DBotCanvas  canvas            = null;
  protected BBot          bot               = null;
  protected BArena        moveArena         = null;
  protected BSimulation   sim               = null;
  protected BMoveBehavior moveBehavior      = null;
  protected boolean       isStopping        = false;


  // Support class for handling activation listeners...
  protected BActivationSupport activationSupport
    = new BActivationSupport(this);

  // Support class for handling property-change listeners...
  protected SwingPropertyChangeSupport propertyManager
    = new SwingPropertyChangeSupport(this);

  protected SelectionRefresher selectionRefresh
    = new SelectionRefresher();

  

  // Member variables...
  // -------------------------------------------------------------

  protected BMove         move              = null;
  protected int           targetIndex       = 0;
  protected boolean       isSimulating      = false;
  protected boolean       isPaused          = false;

  //protected BMove originalStartMove         = null;
  protected BSnapshot highlightedSnap       = null;


  // Constructor...
  // -------------------------------------------------------------

  public BMoveController(BBot bot, B3DBotCanvas canvas)
  {
    this.canvas = canvas;
    this.bot = bot;
  }

  // Accessor methods...
  // -------------------------------------------------------------

  public void setMove(BMove move)
  {
    // If we're currently playing a move, stop that move.
    if (isSimulating)
      stop();

    // Clear any existing highlighting...
    if (this.move != null)
      setSnapshotHighlights(-1);

    // Change current move.
    this.move = move;

    if ((move != null) && isActive())
    {
      // Set bot's start move to the current move...
      //bot.getMoveList().setStartMove(move);
      //sim.initStartMoves();
      bot.resetSingleMove(move);
      
      fireTableDataChanged();
      skipTo(0);
    }
    else
      fireTableDataChanged();
  }

  public void setNewMove(BMove move)
  {
    setMove(move);

    // HACK: This sets up any new moves for arena shit...
    bot.initArenaDetails(moveArena);
  }

  public BMove getMove()
    { return move; }


  // Activation methods...
  // -------------------------------------------------------------

  public boolean isActive()
    { return activationSupport.isActive(); }

  public boolean isClosed()
    { return activationSupport.isClosed(); }


  public void requestActivate()
    { activationSupport.requestActivate(); }

  public void requestDeactivate()
    { activationSupport.requestDeactivate(); }

  public void requestClose()
    { activationSupport.requestClose(); }

    
  public void onActivate()
  {
    // Set this global to ensure heights are correct.
    BGlobal.HEIGHT_SCALE = 1.0f; // BGlobal.CHASSIS_SCALE;

    // Tell bot to ignore the view navigation node...
    // bot.setNegateNode(canvas.getNavigationNode());

    // Remember the bot's original start move...
    //originalStartMove = bot.getMoveList().getStartMove();

    // Create an arena in which to simulate moves.
    moveArena = new BArena();
    moveArena.addArenaObject(bot);

    // Create an simulator to run moves.
    sim = new BSimulation(bot, null, moveArena, 1.0f, canvas, false);

    if (move != null)
    {
      // Set bot's start move to the current move...
      //bot.getMoveList().setStartMove(move);
      //sim.initStartMoves();
      bot.resetSingleMove(move);
      skipTo(0);
    }

    // Create the move behavior...
    moveBehavior = new BMoveBehavior(canvas, bot);
    moveBehavior.addChangeListener(this);

    // Notify listeners that we're active.
    activationSupport.fireActivate();
  }

  public void onDeactivate()
  {
    if (move != null)
    {
      doSafeCleanup();

      // Clear any existing highlighting...
      setSnapshotHighlights(-1);
    }

    if (bot != null)
      // Reset bot position...
      resetBotOrientation();


    // Notify listeners that we're deactivated.
    activationSupport.fireDeactivate();
  }

  public void onClose()
  {
    if (this.isActive())
      doSafeCleanup();


    //moveBehavior.removeChangeListener(this);

    // Notify listeners that we're close.
    activationSupport.fireClose();

      // Support class for handling activation listeners...
    activationSupport  = null;


    // Support class for handling property-change listeners...
     selectionRefresh = null;

    canvas            = null;
    bot               = null;
    moveArena         = null;
    sim               = null;
    moveBehavior      = null;

    activationSupport = null;

    propertyManager   = null;

    selectionRefresh  = new SelectionRefresher();

    move              = null;
    highlightedSnap   = null;

    listenerList = null;
    BDebug.print ("BMoveController.onClose Called !");

  }

  protected void doSafeCleanup()
  {
    // Ask controller to stop playing.
    stop();

    // Clean-up simulator...
    //sim.endStartMoves();

    // Deallocate arena and simulation objects.
    moveArena = null;
    sim = null;

    // Reset bot's start move to its old value...
    //bot.getMoveList().setStartMove(originalStartMove);

    // Request a garbage collection...
    System.gc();
  }

  public void addActivationListener(BActivationListener l)
    { activationSupport.addActivationListener(l); }

  public void removeActivationListener(BActivationListener l)
    { activationSupport.addActivationListener(l); }



  // Table model methods...
  // -------------------------------------------------------------

  public int getRowCount()
    { return getSize(); }

  public int getColumnCount()
    { return 1; }
  
  public Class getColumnClass(int column)
    { return BSnapshot.class; }

  public Object getValueAt(int row, int column)
  {
    if (move != null)
      return move.getSnapshot(row);
    else
      return null;
  }
  
  public boolean isCellEditable(int row, int column)
    { return (row > 0) && !isPlaying(); }



  // Selection model methods...
  // -------------------------------------------------------------

  protected SnapshotSelectionModel selectionModel;
    // A singleton instance of the snapshot selection
    // model is held, so that we can update it easily
    // during playback.

  public ListSelectionModel getSelectionModel()
  {
    if (selectionModel == null)
      selectionModel = new SnapshotSelectionModel();

    return selectionModel;
  }

  protected void refreshSelection()
  {
    if (selectionModel != null)
      selectionModel.refresh();
  }


  // Selection Model Inner class...
  // -------------------------------------------------------------

  protected class SnapshotSelectionModel
    extends DefaultListSelectionModel
  {
    public SnapshotSelectionModel()
    {
      super();
      super.setSelectionMode(SINGLE_SELECTION);
    }

    protected void refresh()
      { super.setSelectionInterval(targetIndex, targetIndex); }

    public void setSelectionInterval(int index0, int index1)
    {
      skipTo(index0, false);
      super.setSelectionInterval(targetIndex, targetIndex);
    }

    public void setLeadSelectionIndex(int leadIndex)
    {
      // skipTo(leadIndex, false);
      // super.setLeadSelectionIndex(leadIndex);
    }
  }



  // Player commands...
  // -------------------------------------------------------------

  public void play()
  {
    if (!isSimulating && (move != null))
    {
      if (!move.isLastSnapshot(targetIndex))
        targetIndex++;

      if (BGlobal.CONSTRUCTING_CHASSISBOT )
        bot.resetSingleMove(move);
      else
        bot.clearActions();


      move.setCurrentSnapshot(targetIndex, bot);

      canvas.addBehavior(moveBehavior);
      moveBehavior.start(move);
      isSimulating = true;

      propertyManager.firePropertyChange("playing", false, true);
    }
  }

  public boolean isPlaying()
    { return (isSimulating && !isPaused); }

  public void stop()
  {
    if (isSimulating)
    {
      // Signal that we wish to deallocate resources.
      isStopping = true;

      // Request behaviour stoppage.
      moveBehavior.stop();

      // Block until stop is complete.
      while (isStopping)
        try { Thread.sleep(50); }
        catch
          (InterruptedException e)
            { BDebug.print("stop(): blocking was interrupted..."); }

      // Behavior will generate a ChangeEvent when it's ready to stop.
      // We listen for ChangeEvents, calling onBehaviorStopped() upon receipt.
    }
  }

  public void onBehaviorStopped()
  {
    // Update our status on this end.
    isSimulating = false;
    isPaused  = false;

    // Signal that stopping is complete...
    // [used by stop().]
    if (isStopping && isActive())
      isStopping = false;

    canvas.removeBehavior(moveBehavior);
    propertyManager.firePropertyChange("playing", true, false);
  }



  public BSnapshot getSelectedSnapshot()
    { return getSnapshot(targetIndex); }
  
  public int getSelectedIndex()
    { return targetIndex; }


  public BSnapshot getSnapshot(int row)
  {
    if (move != null)
      if (move.isValidSnapshot(row))
        return move.getSnapshot(row);
    
    return null;
  }

  protected int getSize()
  {
    if (move != null)
      return move.getSnapshotCount();
    else
      return 0;
  }

  protected int indexOf(BSnapshot s)
    { return move != null ? move.indexOf(s) : -1; }


  public boolean canRemoveSnap()
    { return targetIndex > 0; }
  
  public void remove()
  {
    if (canRemoveSnap())
      removeAt(targetIndex);
  }


  protected void addAt(Object o, int i)
  {
    targetIndex = i;
    BSnapshot s = (BSnapshot) o;
    move.insertSnapshot(s, i);

    fireTableRowsInserted(i, i);
    skipTo(targetIndex);
  }


  protected void removeAt(int i)
  {
    targetIndex = i;
    
    if (targetIndex == getSize()-1)
      targetIndex--;

    move.removeSnapshot(i);

    fireTableRowsDeleted(i, i);
    skipTo(targetIndex);
  }


  /*
  public void remove(Object o)
  {
    int i = move.indexOf((BSnapshot) o);

    if (i != -1)
      removeAt(i);
  }

  public boolean isRemoveable(Object o)
    { return (indexOf(o) != 0); }

  public boolean isRemoveableAt(int i)
    { return (i != 0); }

  public void notifyChange(Object o)
  {
    int i = move.indexOf((BSnapshot) o);
    if (i != -1)
      super.fireContentsChanged(this, i, i);
  }

  public void notifyChange()
    { fireContentsChanged(this, -1, -1); }
  */



  // Skip methods...
  // -------------------------------------------------------------

  protected void skipTo(int i)
    { skipTo(i, true); }

  protected void skipTo(BSnapshot s)
  { 
    int i = indexOf(s);
    skipTo(i, true); 
  }
  
  protected void skipTo(int i, boolean fireChange)
  {
    if (move != null && !isSimulating)
      if (move.isValidSnapshot(i))
      {
        setSnapshotHighlights(i);

        BSnapshot s = move.getSnapshot(i);

        move.setBotOrientation(bot, i);
        canvas.setViewFromParameters();
        bot.applySnapshot(move, i);

        if (s.getThumbnail() == null)
          updateSnapshotThumb(s);

        int old = targetIndex;
        targetIndex = i;

        if (fireChange)
          refreshSelection();
      }
  }




  // Recording methods...
  // -------------------------------------------------------------

  public void record()
  {
    if (!isSimulating)
    {
      targetIndex++;

      // SJS [10/08/2000] - Set limb-locked to false always..
      boolean isLimbLocked = false;
        // move.getLimbLocked();

      int n = move.getSnapshotCount();

      BSnapshot cur =
        bot.getSnapshot("Snapshot "+ n, false, false, isLimbLocked, move, targetIndex);

      updateSnapshotThumb(cur);

      addAt(cur, targetIndex);
    }
  }

  public void recordOver()
  {
    if (!isSimulating)
    {
      BSnapshot old = getSelectedSnapshot();

      // SJS [10/08/2000] - Set limb-locked to false always..
      boolean isLimbLocked = false;
        // move.getLimbLocked();
      
      // TODO: [HACK] Used toString instead of getName().
      BSnapshot cur = bot.getSnapshot(old.toString(), false, false, isLimbLocked, move, move.indexOf(old));

      // Copy over the component transforms...
      old.setComponentTransforms(cur.getComponentTransforms());

      updateSnapshotThumb(old);

      fireTableRowsUpdated(targetIndex, targetIndex);
    }
  }


  protected void updateSnapshotThumb(BSnapshot snap)
    { new ImageUpdater(snap).start(); }


  protected class ImageUpdater extends BProcessor
  {
    protected BSnapshot snap;

    public ImageUpdater(BSnapshot snap)
      { this.snap = snap; }

    public void process()
    {
      BufferedImage thumb =
        BGraphics.scale(canvas.getImage(), 64, 64);

      snap.setThumbnail(thumb);

      //YLY - 5-4-01, Commented out because it is hanging onto memory
      /*SwingUtilities.invokeLater(
        new Runnable()
        {
          public void run()
          { */
            int i = indexOf(snap);
            if (i != -1)
              fireTableRowsUpdated(i, i);
      //    }
        //});
    }
  }
  


  // Internal methods...
  // -------------------------------------------------------------

  protected void resetBotOrientation()
  {
    bot.resetBotPosition();
    bot.setArenaPosition(new BArenaObjectPosition());
  }

  protected void setSnapshotHighlights(int i)
  {

    // Remove existing highlights...
    
    if (highlightedSnap != null)
    {
      move.setMovedJointsHighlight(highlightedSnap, false);
      highlightedSnap = null;
    }

    // Add new highlights...
    if (move.isValidSnapshot(i))
    {
      // i = (i < (move.size()-1)) ? i+1 : i;
      BSnapshot toHighlight = move.getSnapshot(i);

      move.setMovedJointsHighlight(toHighlight, true);
      highlightedSnap = toHighlight;
    }  

  }


  // Listeners...
  // -------------------------------------------------------------

  public void stateChanged(ChangeEvent e)
  {
    // Check if snapshot has changed...
    int i = moveBehavior.getSnapshotIndex();

    targetIndex = i;
    setSnapshotHighlights(targetIndex);

    // Update selection...
    SwingUtilities.invokeLater(selectionRefresh);

    // Check if behavior has stopped simulating...
    if (!moveBehavior.isSimulating())
      onBehaviorStopped();
  }


  protected class SelectionRefresher implements Runnable
  {
    public void run()
      { refreshSelection(); }
  }
  

  // Property change listeners
  // -------------------------------------------------------------

  public void addPropertyChangeListener(PropertyChangeListener listener)
    { propertyManager.addPropertyChangeListener(listener); }

  public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
    { propertyManager.addPropertyChangeListener(propertyName, listener); }

  public void removePropertyChangeListener(PropertyChangeListener listener)
    { propertyManager.removePropertyChangeListener(listener); }

  public void removePropertyChangeListener( String propertyName, PropertyChangeListener listener)
   { propertyManager.removePropertyChangeListener(propertyName, listener); }

}
