package roboforge.ui;

import roboforge.BBot;
import roboforge.BChassis;
import roboforge.BComponent;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BMove;
import roboforge.BMoveList;

import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.ui.modes.*;
import roboforge.util.graphics.*;
import roboforge.animation.animation.*;

import java.awt.Event;
import java.awt.event.*;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import javax.swing.*;
import javax.vecmath.*;

import java.util.Vector;


public class BMovesPanel extends B3DEditingPanel
  // implements PropertyChangeListener
{

  // Internal variables...
  // -------------------------------------------------------------


  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "addMove.icon",               BIconCache.getHSB("add_move_icon.gif"),
    "addMove.tooltip",            "Add New Move. [Ins]",
    "addMove.keystroke",          KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, 0),

    "removeMove.icon",            BIconCache.getHSB("remove_move_icon.gif"),
    "removeMove.tooltip",         "Remove Current Move. [Del]",
    "removeMove.keystroke",       KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0),

    "zones.showText",             Boolean.FALSE,
    "zoneAction.icon",            BGraphics.getHSBIcon("zones_icon.gif", colors.getMidShade()),
    "zoneAction.tooltip",         "Show Zones",
    
    "sensors.showText",           Boolean.FALSE,
    "sensorAction.icon",          BIconCache.getHSB("sensors_icon.gif"),    
    "sensorAction.tooltip",       "Show Sensor coverage",

    "setHeight.icon",             BIconCache.getHSB("snap_to_floor.gif"),
    "setHeight.tooltip",          "Snap Bot to Floor",

    "setDefaultPos.icon",         BIconCache.getHSB("set_default_icon.gif"),
    "setDefaultPos.tooltip",      "Set current position as Default",
  });


  // Visual elements...
  // -------------------------------------------------------------

  protected BMovesLibrary library;
  protected BZoneEditor zoneEditor;

  protected BToggleButton zones   = new BToggleButton(res, "zones");
  protected BToggleButton sensors = new BToggleButton(res, "sensors");

  protected BButton setHeight;
  protected BButton setDefaultPos;

  protected BModeMouse mRotJ;

  protected BZoomAnimation zoomAni;



  // Constructor.
  // -------------------------------------------------------------

  public BMovesPanel
  (
    BWorkshopScreen parent,
    B3DBotCanvas canvas,
    BBot bot
  )
  {
    super(parent, "Moves", canvas, bot);

    if (BGlobal.CONSTRUCTING_CHASSISBOT)
    {
      setHeight = new BButton(res, "setHeight");
      setDefaultPos = new BButton(res, "setDefaultPos");
    }

    layoutMovesPanelChildren();
    configMovesPanelChildren();

    // Initialize component library...
    library = new BMovesLibrary(this);
    openToolPanel(library);

    zoneEditor = new BZoneEditor(this);
    openToolPanel(zoneEditor);

    selectToolPanel(library);
    
    res.cleanse();
    res = null;

    // HACK FOR TESTING
   /* botName.addMouseListener( new MouseAdapter()
    {
            public void mouseClicked(MouseEvent e)
            {
              BMovesPanel.this.bot.activateZones();
            }
    }
    ); */
  }


  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutMovesPanelChildren()
  {
    layoutMovesPanelToolbar();
  }

  protected void layoutMovesPanelToolbar()
  {
    toolbar.addBigMargin();

    toolbar.add(zones);
    toolbar.addSmallMargin();
    toolbar.add(sensors);
    
    if (BGlobal.CONSTRUCTING_CHASSISBOT)
    {
      toolbar.add(setHeight);
      toolbar.addSmallMargin();
      toolbar.add(setDefaultPos);
    }

    toolbar.addGlue();
  }


  protected void configMovesPanelChildren()
  {
    mRotJ = new BModeViewer(
      canvas, new BModeJointRot(canvas, BModeJointRot.TRANSIENT));

    setupModeButton(rotj, mRotJ);

    if (BGlobal.CONSTRUCTING_CHASSISBOT)
    {
      setHeight.addActionListener(new SetHeightListener());
      setDefaultPos.addActionListener(new SetDefaultPosListener());
    }

    zones.setAction(new ZoneAction(res));
    sensors.setAction(new SensorAction(res));
    
    // Disable select and component rotate modes..
    select.setEnabled(false);
    rotc.setEnabled(false);
  }


  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    ((BWorkshopScreen)parent).autosave();

    super.onActivate();

    bot.setZonesAttached(zones.isSelected());
    bot.setSensorsAttached(sensors.isSelected()  );

    canvas.setFloor(canvas.getLargeGrid());
  }

  public void onDeactivate()
  {

    super.onDeactivate();
    selectToolPanel(library);
    bot.setZonesAttached(false);
    bot.setSensorsAttached(false);


  }

  public void onSaveBegin()
    { library.onSaveBegin(); }

  public void onSaveEnd()
    { library.onSaveEnd(); }

  protected void setDefaultMode()
    { canvas.setMode(mRotJ); }

  public void onClose()
  {
    library = null;
    setHeight = null;
    setDefaultPos = null;
    mRotJ = null;
    zoneEditor = null;

    zones.setAction(null);
    sensors.setAction(null);

    zones   =  null;
    sensors = null;

    zoomAni = null;


    super.onClose();
  }

  
  // UI Listeners...
  // -------------------------------------------------------------

  protected class ZoneAction extends BAction
  {
    public ZoneAction(BUIResources res)
      { super(res, "zoneAction"); }

    public void actionPerformed(ActionEvent e)
    {
      boolean s = zones.isSelected();

      bot.setZonesAttached(s);
      setShortDescription((s ? "Hide" : "Show") + " Zones");
    }
  }


  protected class SensorAction extends BAction
  {
    public SensorAction(BUIResources res)
      { super(res, "sensorAction"); }

    public void actionPerformed(ActionEvent e)
    {
      boolean s = sensors.isSelected();

      bot.setSensorsAttached(s);
      setShortDescription((s ? "Hide" : "Show") + " Sensor coverage");

      attachZoom();

    }

    double lastPreZoomViewTranslate = 300.0;

    private void attachZoom()
    {
      if (zoomAni == null)
        zoomAni = new BZoomAnimation( 0, 1, canvas.getViewTranslation() );

      if (sensors.isSelected() && getBot().getContainsSensor())
      {
        lastPreZoomViewTranslate =  getCanvas().getViewTranslation().z;
        zoomAni.setPath( lastPreZoomViewTranslate, getCanvas().getViewTranslationMax() );
        workshop.getAnimator().add(zoomAni);

      }
      else if (  getBot().getContainsSensor() )
      {
        if (getCanvas().getViewTranslation().z > lastPreZoomViewTranslate)
        {
          zoomAni.setPath(  getCanvas().getViewTranslation().z, lastPreZoomViewTranslate);
          workshop.getAnimator().add(zoomAni);
        }
      }
    }
  }

  
  protected class SetHeightListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      BChassis c = bot.getChassis();
      c.setHeight();
    }
  }

  protected class SetDefaultPosListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      { bot.setDefaultJointTransformsToCurrent(); }
  }
  

  
  
  // Accessor methods...
  // -------------------------------------------------------------
  
  protected boolean wasViewingZones;
  
  public void setEditingZones(boolean e)
  {
    if (e) 
      wasViewingZones = zones.isSelected();
    
    boolean select =
      e || wasViewingZones;
    
    zones.setSelected(select);
    zones.setEnabled(!e);
    
    bot.setZonesAttached(select);
  }



  // Animation class...
  // -------------------------------------------------------------

  protected class BZoomAnimation extends BLinearAcceleration
  {
    private Vector3d translation;

    public BZoomAnimation( double start, double fin, Vector3d translation)
    {
      super(start, fin, 4.0);
      this.translation = translation;
    }

    public boolean animate()
    {
       super.animate();

       translation.z =  current;
       getCanvas().setViewTranslation(translation);
       getCanvas().setViewFromParameters();

       return continueExecuting();
    }
  }

}
