package roboforge.ui;

import roboforge.BArena;
import roboforge.BArenaCanvas;
import roboforge.BArenaObject;
import roboforge.BArenaObjectPosition;
import roboforge.BBattleRecorder;
import roboforge.BBot;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BPlaceHolder;
import roboforge.BSimulation;
import roboforge.BSimulationBehaviour;
import roboforge.BStats;
import roboforge.BZones;
import roboforge.BComponent;
import roboforge.math.*;

import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.ui.events.*;
import roboforge.ui.modes.*;

import roboforge.util.graphics.*;
import roboforge.util.io.*;

import roboforge.ui.console.*;

import roboforge.BTextures;

import java.io.*;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.*;

import javax.media.j3d.*;
import javax.vecmath.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.util.Vector;

import javax.swing.*;

import com.sun.j3d.utils.image.TextureLoader;


public class BTestPanel extends BTooledPanel
  implements PropertyChangeListener
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BBot bot                          = null;
  protected BBot enemy                        = null;
  protected BArenaObjectPosition botStartPos  = null;
  protected BArenaObjectPosition enemyStartPos= null;

  protected BTestController control           = null;
  protected BTestSetup setup                  = null;
  protected BTestVitals vitals                = null;

  protected boolean isViewingBattle           = false;

 // protected boolean isValidPanel = false;

  protected BConsoleImage simRunningImage;

  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "zones.icon",              BIconCache.getHSB("zones_icon.gif"),
    "zones.tooltip",           "Show / Hide Zones",

    "sensors.icon",            BIconCache.getHSB("sensors_icon.gif"),
    "sensors.tooltip",         "Show / Hide Sensor Views",

    "arenaSetup.icon",         BIconCache.getHSB("arena_setup.gif"),
    "arenaSetup.tooltip",      "Arena setup / Arena view",

    "3dcanvas.background",     colors.getExtraDarkShade(),
    "3dcanvas.foreground",     colors.getWhiteShade(),

    "3dborder.border",         BUIResources.getLineBorder(colors.getDarkerShade()),
    // "3dborder.width",          new Integer(400),

    "picker.size",                new Dimension(400, 391),
   // "picker.size",                new Dimension(230, 230),
    "picker.background",          colors.getVeryDarkShade(),
    "picker.meters",              new Double(20),
    "picker.metersPerMajor",      new Double(5),
    "picker.minorsPerMajor",      new Integer(5),
    "picker.icon",                BIconCache.getHSB("bot_icon.gif"),

  });


  // Visual elements...
  // -------------------------------------------------------------

  protected BTitlebarLabel botName        = new BTitlebarLabel();
  protected BToggleButton zones           = new BToggleButton(res, "zones");
  protected BToggleButton sensors         = new BToggleButton(res, "sensors");

  protected BArenaCanvas canvas           = null;
  protected BPanel canvasBorder           = new BPanel(res, "3dborder");

  protected BToggleButton arenaSetup  = new BToggleButton(res, "arenaSetup");

  protected BArenaSetupPicker picker = new BArenaSetupPicker(res, "picker");

  // has the simulation been stopped manually?
  protected boolean manualStop = false;

  // should the arena be reset after the controller has stopped?
  protected boolean resetAfterStop = true;

  protected boolean isActive = false;


  // Constructor.
  // -------------------------------------------------------------

  public BTestPanel(BWorkshopScreen parent, BBot bot)
  {
    super(parent, "Test");

    this.bot      = bot;
    bot.addPropertyChangeListener(this);

    canvas = new BArenaCanvas(res, "3dcanvas", this);
    canvas.setFloor(canvas.getGrid());
    canvas.setLocation(0, 0);

    //canvasBorder.setSize(230, 230);
    // HACK FOR THE CONSOLE
    canvas.setSize(398, 389);
    canvas.setLocation(61, 46);
    //canvas.setBounds(61, 46, 398, 389);

    control = new BTestController(canvas);
    control.setBot(bot);

    control.setArenaFileName(
      BGlobal.ARENAPATH + "Close" + BGlobal.ARENASUFFIX);

    setup  = new BTestSetup(this);
    vitals = new BTestVitals(this);

    layoutTestPanelChildren();
    configTestPanelChildren();

    openToolPanel(setup);
    openToolPanel(vitals);
    selectToolPanel(setup);

    res.cleanse();
    res = null;

  }


  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutTestPanelChildren()
  {
    // contentArea.addGlue();
    layoutCanvas();
  }

  protected void layoutCanvas()
  {

    canvasBorder.setLayout(new BorderLayout());
    canvasBorder.add(picker, BorderLayout.CENTER);

    picker.setVisible(false);

    canvasBorder.add(canvas, BorderLayout.CENTER);

   // add(picker);
    toolbar.add(zones);
    toolbar.addSmallMargin();
    toolbar.add(sensors);
//    toolbar.addBigMargin();
//    toolbar.add(arenaSetup);

    contentArea.addHGlued(botName);
    contentArea.addVStrut(9);
    contentArea.add(canvasBorder);
  }


  protected void configTestPanelChildren()
  {
    botName.setText(bot.getName());

    zones.addActionListener(new ZonesListener());
    sensors.addActionListener(new SensorsListener());
   // arenaSetup.addActionListener(new ArenaSetupListener());

    control.addPropertyChangeListener(this);

    canvas.setEnabled(true);
    canvas.addMouseListener(new CanvasMouseListener());

    picker.setArena(control.getArena());
    picker.setViewScale(0.35);

    /*Texture healthTextureR =
      BTextures.getTexture(BGlobal.TEXTUREPATH +"Internal/heart_hud_iconR.gif");

    BConsoleImage botAHealthIcon = new BConsoleImage(healthTextureR);
    //botAHealthIcon.setTransparency(0.2f);
    botAHealthIcon.setLocation(200, 200);
    botAHealthIcon.setColor( BGlobal.colors.getLightShade3f());
    canvas.add(botAHealthIcon); */



       // setting up the 'simulation running' image
    Color3f simRunningCol = BGlobal.colors.getLightShade3f();

    // Directly get the sim run image so that it does not get reduced

    Texture simTexture = null;
    try {
      TextureLoader loader = new TextureLoader(BGlobal.IMAGEPATH + "ui/sim_run.gif", this);
      simTexture = loader.getTexture();
    } catch (Exception ex) {
    	BDebug.print("BTestPanel.configTestPanelChildren Exception: "+ex);
    }


    simRunningImage = new BConsoleImage(simTexture);
    simRunningImage.setSize(256, 32);
    simRunningImage.setTransparency(0.2f);
    simRunningImage.setLocation(176, 14);

    simRunningImage.setColor(simRunningCol);
    simRunningImage.setRemoveable(true);

   // canvas.setCameraXYZ(400.0f, 400.0f, 400.0f);
    ////////////////////////////////////////////



  }


  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {

    ((BWorkshopScreen)parent).autosave();

    resetAfterStop = true;
    control.requestActivate();

    // Remove bot from the editing canvas.
    B3DBotCanvas editingCanvas =
      ((BWorkshopScreen) parent).getCanvas();

    editingCanvas.removeBot();

    //zones.setSelected(bot.areZonesAttached());
    bot.setZonesVisible(zones.isSelected());


   // BDebug.print("BTestPanel - sensors selected? - " + sensors.isSelected());

    if(arenaSetup.isSelected())
    {
      picker.setBots(bot, enemy);
      canvas.setVisible(false);
      picker.setVisible(true);
    }

    //picker.setViewScale(0.5);
    super.onActivate();

    bot.setSensorsAttached(sensors.isSelected());

    //XXX SAFE FIX FOR PRE_RELEASE
    // [YLY] 4-May-01.
    selectToolPanel(setup);
    vitals.viewAction.setEnabled(false);

    isActive = true;

  }

  public void onDeactivate()
  {
    resetAfterStop = false;

    //BDebug.print("BTestPanel - onDeativate - ending the simulation");

    control.stop();

    // wait for the controller to stop;
    try
    {
      while(control.isPlaying())
        {
          Thread.currentThread().sleep(10);
        }
    }
    catch (Exception e) {};

    //BDebug.print("BTestPanel - onDeativate - simulation stopped");

    resetAfterStop = false;
    control.requestDeactivate();

    if (!isViewingBattle && isActive )
    {
      // Add bot to the editing canvas.
      B3DBotCanvas editingCanvas =
        ((BWorkshopScreen) parent).getCanvas();

      bot.resetBotPosition();
      bot.setArenaPosition(new BArenaObjectPosition());


      editingCanvas.addBot(bot);

    }

    // remove the sensor visual
    bot.setSensorsAttached(false);
    bot.setZonesVisible(true);

    canvas.clear();

    enemy = null;
    control.setEnemy(null);
    //viewAction.setEnabled(false);
    super.onDeactivate();

    isActive = false;

  }

  public void onClose()
  {
    onDeactivate();

    //BDebug.print("BTestPanel - onClose called");

    bot.removePropertyChangeListener(this);

    hideToolPanel(setup);
    hideToolPanel(vitals);

    bot = null;

    if (enemy != null)
        enemy.cleanse();
    enemy = null;


    setup.onClose();
    setup = null;

    vitals.onClose();
    vitals = null;


    // Remove canvas from panel (allows GC).
    canvas.onClose();
    remove(canvas);
    canvas = null;

    botStartPos = null;
    enemyStartPos = null;
    //control = null;
    
    super.onClose();
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setVisible(boolean b)
  {
    super.setVisible(b);
    canvas.setVisible(b);
  }



  // UI Listeners...
  // -------------------------------------------------------------
  protected class CanvasMouseListener implements MouseListener
  {
    public void mousePressed(MouseEvent e)
    {
      control.pauseSim(true);
    }
    public void mouseClicked(MouseEvent e) {}
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mouseReleased(MouseEvent e)
    {
      control.pauseSim(false);
    }

  }

  protected class ArenaSetupListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      if(arenaSetup.isSelected())
      {
        picker.setBots(bot, enemy);
        canvas.setVisible(false);
        picker.setVisible(true);
      }
      else
      {
        picker.setVisible(false);
        canvas.setVisible(true);
        canvas.updateCamera();
      }
    }
  }

  protected class ZonesListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      { bot.setZonesVisible(zones.isSelected()); }
  }

  protected class SensorsListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
      { bot.setSensorsAttached(sensors.isSelected()); }
  }

  public void propertyChange(PropertyChangeEvent e)
  {
    if (e == null)
      return;

    if (e.getPropertyName().equals("name"))
    {
      if (botName != null)
        botName.setText((String) e.getNewValue());
    }
    else if (e.getPropertyName().equals("playing"))
    {
      if (control != null)
      {
        boolean playing = control.isPlaying();

        if (playing)
        {
          if (vitals != null)
          {
            selectToolPanel(vitals);
            setStatus("Running test battle...");
          }
        }
        else
          setStatus("Test battle completed.");

        setWorking(playing);

        if (setup != null)
          setup.setEnabled(!playing);
      }
    }
  }
 
  
  
  // -------------------------------------------------------------
  // Test setup panel class...
  // -------------------------------------------------------------

  protected class BTestSetup extends BToolPanel
  {

    // UI Resources...
    // ...............................................................

    protected Object[] resourceData =
    {
      "duration.min",                new Integer(0),
      "duration.max",                new Integer(180),
      "duration.value",              new Integer(30),
      "duration.majorSpacing",       new Integer(60),
      "duration.minorSpacing",       new Integer(15),

      "play.size",                   new Dimension(80, 30),

      "playAction.icon",             BIconCache.getHSB("play_icon_small.gif"),
      "playAction.name",             "Start",
      "playAction.tooltip",          "Start testing.",

      "botDesc.tooltip",             "enemy Information.",

    };

    protected BUIResources res
      = new BUIResources(resourceData);


    // UI controls...
    // ...............................................................

    protected BLabel enemyBotsLabel     = new BLabel("Bot to test against:");
    protected BFileComboBox enemyBots   = new BFileComboBox();

    protected BMultiLabel botDescription   = new BMultiLabel(this.res, "botDesc");
    protected BScrollPane botDescScroller  = new BScrollPane(botDescription);

    protected BLabel arenaLabel         = new BLabel("Arena:");
    protected BFileComboBox arenas      = new BFileComboBox();

    protected BLabel durationLabel      = new BLabel("Duration of test (seconds):");
    protected BSlider duration          = new BSlider(res, "duration");

    protected BLabel collisionsLabel    = new BLabel("Accuracy of battle simulation:");
    protected BComboBox collisions      = new BComboBox();

    protected BButton play              = new BButton(this.res, "play");



    // UI actions...
    // ...............................................................

    protected PlayAction playAction     = new PlayAction(this.res, "playAction");

    protected boolean processBotCombo = true; // false to ingore a actionPerformed on the bot combo
    // Constructor...
    // ...............................................................

    public BTestSetup(BTooledPanel parent)
    {
      super(parent, "setup");

      layoutSetupChildren();
      configSetupChildren();
    }

    public void layoutSetupChildren()
    {
      addGlued(durationLabel);
      addTinyMargin();
      add(duration);
      //addMargin();
      addSmallMargin();

      addGlued(enemyBotsLabel);
      addTinyMargin();
      add(enemyBots);

      addTinyMargin();
      addVGlued( botDescScroller) ;
      addSmallMargin();

      addGlued(arenaLabel);
      addTinyMargin();
      addLinear(arenas, getHStrut(SMALL_MARGIN), arenaSetup);
      addSmallMargin();

      addGlued(collisionsLabel);
      addTinyMargin();
      add(collisions);

      addMargin();
      addLinear(getHGlue(), play);
    }

    public void configSetupChildren()
    {
      initCollisionsUI();
      arenaSetup.addActionListener(new ArenaSetupListener());

      try
      {
        enemyBots.setFilter(new BRoboforgeFilter(BGlobal.TESTBOTSUFFIX));
        enemyBots.setSourcePath(BGlobal.TESTBOTPATH);

        if (!enemyBots.isEmpty())
          enemyBots.setSelectedIndex(0);
      }
      catch (IOException e)
        { /* TODO: Handle error. */ }

      enemyBots.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          if(processBotCombo)
          {
            loadEnemyBot(enemyBots.getSelectedFileWithPath());
            botDescription.setText(enemy.getBotDescription() );

            canvas.updateCamera();
            picker.setBots(bot, enemy);
          }
        }
      });

      try
      {
        arenas.setFilter(new BRoboforgeFilter(BGlobal.ARENASUFFIX));
        arenas.setSourcePath(BGlobal.ARENAPATH);

        if (!arenas.isEmpty())
          arenas.setSelectedIndex(0);
      }
      catch (IOException e)
        { /* TODO: Handle error. */ }

      arenas.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          control.setArenaFileName(arenas.getSelectedFileWithPath());
          control.init();
          bot.setZonesVisible(zones.isSelected());
          picker.setArena(control.getArena());
          picker.setBotOrient();
        }
      });

      play.setAction(playAction);
    }

    protected void initCollisionsUI()
    {
      collisions.addItem("Low (fast)");
      collisions.addItem("Medium");
      collisions.addItem("High (slow)");

      int i = 0;

      switch (BGlobal.getBattleAccuracy() )
      {
        case BGlobal.BATTLE_ACCURACY_LOW:
          i = 0; break;
        case BGlobal.BATTLE_ACCURACY_MEDIUM:
          i = 1; break;
        case BGlobal.BATTLE_ACCURACY_HIGH:
          i = 2; break;
      }



      collisions.addActionListener ( new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          setCollisionMode();
        }

      });

      collisions.setSelectedIndex(i);

    }

    protected void setCollisionMode()
    {
      switch (collisions.getSelectedIndex())
      {
        case 0:
          BGlobal.setBattleAccuracy(BGlobal.BATTLE_ACCURACY_LOW);     break;
        case 1:
          BGlobal.setBattleAccuracy(BGlobal.BATTLE_ACCURACY_MEDIUM);  break;
        case 2:
          BGlobal.setBattleAccuracy(BGlobal.BATTLE_ACCURACY_HIGH);    break;
      }

      BGlobal.save();
    }

    public void onActivate()
    {
      super.onActivate();

      // refresh the bot combo
      try
      {
        processBotCombo = false;
        enemyBots.setFilter(new BRoboforgeFilter(BGlobal.TESTBOTSUFFIX));
        enemyBots.setSourcePath(BGlobal.TESTBOTPATH);

        if (!enemyBots.isEmpty() || enemyBots.getSelectedIndex() == -1)
          enemyBots.setSelectedIndex(0);
        processBotCombo = true;
      }
      catch (IOException e)
        { /* TODO: Handle error. */ }

     // play.doClick();
      loadEnemyBot(enemyBots.getSelectedFileWithPath());
      control.setArenaFileName(arenas.getSelectedFileWithPath());
      setCollisionMode();

      control.init();
      bot.setZonesVisible(zones.isSelected());
      picker.setArena(control.getArena());
      picker.setBots(bot, enemy);
     // control.stop();

    }

    public void onClose()
    {
      enemyBotsLabel     = null;
      enemyBots   =       null;

      botDescription   = null;
      botDescScroller  = null;

      arenaLabel         = null;
      arenas      = null;

      durationLabel      = null;
      duration          = null;

      collisionsLabel    = null;
      collisions      = null;

      play.removeActionListener(playAction);
      playAction     = null;

      super.onClose();
    }

    // UI listeners...
    // -------------------------------------------------------------

    protected class PlayAction extends BAction
    {
      public PlayAction(BUIResources res, String key)
        { super(res, key); }

      public void actionPerformed(ActionEvent e)
      {
       // loadEnemyBot(enemyBots.getSelectedFileWithPath());
       // control.setArenaFileName(arenas.getSelectedFileWithPath());


        manualStop = false;

        arenaSetup.setSelected(false);
        arenaSetup.setEnabled(false);

        picker.setVisible(false);
        canvas.setVisible(true);

        control.play(duration.getValue());

        //zones.setSelected(true);
        bot.setSensorsAttached(sensors.isSelected());

        botStartPos = new BArenaObjectPosition(bot.getArenaPosition());
        enemyStartPos = new BArenaObjectPosition(enemy.getArenaPosition());

        canvas.add(simRunningImage);
       // BDebug.print("BTestPanel - play action - image added");
      }
    }
  }




  // -------------------------------------------------------------
  // Test setup panel class...
  // -------------------------------------------------------------

  protected class BTestVitals extends BToolPanel
  {

    // UI Resources ...
    // -------------------------------------------------------------

    protected Object[] resourceData =
    {
     // "chassisMoveLabel.height",     new Integer(10),
     // "chassisMoveLabel.foreground", colors.getMidShade(),

      "timeLabel.width",             new Integer(100),
      "timeLabel.text",              "Test progress: 0s",

      "stopAction.icon",             BIconCache.getHSB("stop_icon_small.gif"),
      "stopAction.name",             "Stop",
      "stopAction.tooltip",          "Stop testing.",

      "stop.size",                   new Dimension(80, 30),

      "viewAction.name",             "View",
      "viewAction.tooltip",          "View test battle.",

      "view.size",                   new Dimension(80, 30),
    };

    protected BUIResources res = new BUIResources(resourceData);


    // UI controls...
    // ...............................................................

    protected BProgressBar progress     = new BProgressBar();

    protected BBotVitals heroVitals     = new BBotVitals();
    protected BBotVitals enemyVitals    = new BBotVitals();

    protected BButton stop              = new BButton(this.res, "stop");
    protected BButton view              = new BButton(this.res, "view");


//    protected BLabel chassisMoveLabel   = new BLabel(this.res, "chassisMoveLabel");

    protected BList movesList           = new BList();

    protected BScrollPane movesScroll   = new BScrollPane(movesList);

    protected BLabel timeLabel          = new BLabel(res, "timeLabel");

    // UI actions...
    // ...............................................................

    protected StopAction stopAction     = new StopAction(this.res, "stopAction");
    protected ViewAction viewAction     = new ViewAction(this.res, "viewAction");

    protected int timeElapsed = 0;

    public BTestVitals(BTooledPanel parent)
    {
      super(parent, "progress");

      layoutVitalsChildren();
      configVitalsChildren();
    }

    public void onClose()
    {
        progress     =null;

        heroVitals     = null;
        enemyVitals    = null;

        stop.removeActionListener(stopAction);
        stop = null;

        view.removeActionListener(viewAction);
        view = null;

        stopAction     = null;
        //viewAction     = null;

        super.onClose();
    }

    public void layoutVitalsChildren()
    {
      addHGlued(timeLabel);
      addTinyMargin();
      add(progress);
      addMargin();

      add(heroVitals);
      addMargin();
      add(enemyVitals);
      addMargin();

  //    addHGlued(new BLabel("Chassis move:"));
  //    addHGlued(chassisMoveLabel);
  //    addSmallMargin();
      addHGlued(new BLabel("Current actions: " + bot.getName()));
      add(movesScroll);
      addMargin();
      BBoxPanel box = createHBoxPanel();

      box.addHGlue();
      box.add(stop);
      box.addSmallMargin();
      box.add(view);

      addGlue();
      add(box);
    }

    public void configVitalsChildren()
    {
      control.addPropertyChangeListener(new ControlListener());
      control.addProgressListener(new ProgressListener());

      heroVitals.setBot(bot);
      enemyVitals.setTitle("Bot: (none)");

      stop.setAction(stopAction);
      view.setAction(viewAction);

      stopAction.setEnabled(false);
      viewAction.setEnabled(false);
    }


    // Activation methods...
    // -------------------------------------------------------------

    public void onDeactivate()
    {
      enemyVitals.setBot(null);
      viewAction.setEnabled(false);
      super.onDeactivate();
    }


    // UI Actions...
    // -------------------------------------------------------------

    protected class StopAction extends BAction
    {
      public StopAction(BUIResources res, String key)
        { super(res, key); }

      public void actionPerformed(ActionEvent e)
      {
        manualStop = true;
        canvas.remove(simRunningImage);
        control.stop();
        //BDebug.print("BTestPanel: Stop pressed");

      }
    }

    protected class ViewAction extends BAction
    {
      public ViewAction(BUIResources res, String key)
        { super(res, key); }

      public void actionPerformed(ActionEvent e)
        { viewCurrentBattle(); }
    }



    // Control listener...
    // -------------------------------------------------------------

    protected class ControlListener implements PropertyChangeListener
    {
      boolean playing;
      
      protected void reloadBots()
      {
        BApplication.setBusy(true);

        bot.setArenaPosition(botStartPos);
        enemy.setArenaPosition(enemyStartPos);
        canvas.remove(bot);
        canvas.remove(enemy);
        canvas.loadArenaObject(bot);
        canvas.loadArenaObject(enemy);
        enemy.setZonesVisible(false);
        bot.resetBotPosition();
        enemy.resetBotPosition();
        canvas.updateCamera();

        BApplication.setBusy(false);

      }

      protected void stopCleanup()
      {
         canvas.remove(simRunningImage);
         arenaSetup.setEnabled(true);
         if(resetAfterStop)
          reloadBots();

          bot.setZonesVisible(zones.isSelected());


          if(!manualStop)
          {
            viewAction.setEnabled(!playing);
            view.doClick();
          }
      }

      public void propertyChange(PropertyChangeEvent e)
      {
        //try
        //{
          if (e.getPropertyName().equals("playing"))
          { //
            //if (control == null)
            //  playing = false;
            //else
            playing = control.isPlaying();

            //if (stopAction != null)
            stopAction.setEnabled(playing);

            if(!playing)
            {
              stopCleanup();

            }

            //if (viewAction != null)
              viewAction.setEnabled(!playing);
          }

         /*}
         catch (Exception ex)
         {
            BDebug.print ("BTestPanel.BTestVitals.ControlListener exception " + ex.getMessage());

            BApplication.setBusy(false);
         } */


      }
    }

    // Progress listener...
    // -------------------------------------------------------------

    protected class ProgressListener implements BProgressListener
    {
      public void progressUpdated(BProgressEvent e)
      {
        if(control != null)                                                                 
        {
          if(progress != null)
            progress.progressUpdated(e);

          if(control.isPlaying())
          {
            if (movesList != null)
              if (bot != null)
                movesList.setListData(bot.getCurrentActionNames());

            if (timeLabel != null)
              timeLabel.setText("Test progress: " + BMath.format(control.getElapsedTime(), 1) + "s");
          }


          if (heroVitals != null)
            heroVitals.updateVitalsUI();

          if (enemyVitals != null)
            enemyVitals.updateVitalsUI();
        }

      //  chassisMoveLabel.setText(bot.getCurrentChassisMoveName());
      }
    }


  }

  // -------------------------------------------------------------
  // Bot Vitals Panel class...
  // -------------------------------------------------------------

  protected class BBotVitals extends BBoxPanel
  {

    // Internal vars ...
    // -------------------------------------------------------------

    protected BBot bot;


    // UI Resources ...
    // -------------------------------------------------------------

    protected Object[] resourceData =
    {
      "health.icon",                 BIconCache.getHSB("health_icon.gif"),
      "health.width",                new Integer(20),
      "health.foreground",           colors.getMidShade(),

      "energy.icon",                 BIconCache.getHSB("energy_icon.gif"),
      "energy.width",                new Integer(20),
      "energy.foreground",           colors.getMidShade(),
    };

    protected BUIResources res
      = new BUIResources(resourceData);


    protected BLabel botName         = new BLabel(res, "botName");
    protected BLine  line            = new BLine();
    protected BLabel healthLabel     = new BLabel(res, "health");
    protected BLabel energyLabel     = new BLabel(res, "energy");
    protected BProgressBar health    = new BProgressBar();
    protected BProgressBar energy    = new BProgressBar();

    protected BBotVitals()
    {
      super(BoxLayout.Y_AXIS);
      res.initComponent(this, "this");

      layoutVitalsChildren();
      configVitalsChildren();
    }

    protected void layoutVitalsChildren()
    {
      addHGlued(botName);
      addTinyMargin();

      addLinear(healthLabel, getHStrut(5), health);
      addTinyMargin();
      addLinear(energyLabel, getHStrut(5), energy);
    }

    protected void configVitalsChildren()
    {
      health.setValue(100);
      energy.setValue(100);
    }

    protected void setBot(BBot bot)
    {
      this.bot = bot;

      if (bot != null)
      {
        botName.setText("Bot: "+ bot.getName());
        updateVitalsUI();
      }
    }

    public void updateVitalsUI()
    {
      if (bot != null)
      {
        int h = (int) Math.round(100.0 - bot.getPercentageDamageLevel());
        int e = (int) Math.round(bot.getPercentageEnergyLevel());

        health.setValue(h);
        energy.setValue(e);
      }
    }

    public void setEnabled(boolean e)
    {
      super.setEnabled(e);
      botName.setEnabled(true);
    }

    public void setTitle(String s)
      { botName.setText(s); }
  }



  // -------------------------------------------------------------
  // Battle viewer methods...
  // -------------------------------------------------------------
  
  protected void loadEnemyBot(String fileName)
  {
    BBot oldEnemy = enemy;

    //if (enemy != null)
    //  { enemy.cleanse(); }

    // Create the Opponent Bot
    String enemyFileName = fileName;
    enemy = BBot.loadFromFile(enemyFileName);

    // enemy.setBotFilePath(enemyFileName);
    if (enemy != null)
      enemy.removeThumbnails();

    control.setEnemy(enemy);
    vitals.enemyVitals.setBot(enemy);

    if (oldEnemy != null)
      oldEnemy.cleanse();
  }

  
  protected void viewCurrentBattle()
  {
    setBusy(true);
    setStatus("Loading battle - please wait...");

    // cache bot for the return to the workshopScreen
    try
      { bot.saveToFile(BGlobal.USERPATH + "bCache"); }
    catch (Exception e)
      { BDebug.print(" BWorkshopGhost.constructor bot load error " ); }


    BBattleRecorder recorder =
      control.getBattleRecorder();

    if (recorder != null)
    {
      /*
      isViewingBattle = true;

      // Deactivate controller, thereby removing bots from test canvas.
      control.requestDeactivate();

      // Sort out battle stats...
      BStats stats = recorder.getStats(bot);
      BStats randomStats = stats.convertToRandomFile();

      recorder.convertToSaveFormat();
      if (randomStats != null)
        recorder.setStats(randomStats, bot);

      // Load the battle screen.
      BBattleScreen.load(recorder);
      isViewingBattle = false;    */

      isViewingBattle = true;

      // Deactivate controller, thereby removing bots from test canvas.
      control.requestDeactivate();

      // Sort out battle stats...
      BStats stats = recorder.getStats(bot);
      BStats randomStats = stats.convertToRandomFile();

      recorder.convertToSaveFormat();
      if (randomStats != null)
        recorder.setStats(randomStats);

      recorder.saveToFile(BGlobal.USERPATH + "recordingCache.rec",null,null);
      randomStats.saveToFile(BGlobal.USERPATH + "recordingCache.sta");

      // Load the battle screen.
      BBattleScreen.load(BGlobal.USERPATH + "recordingCache.rec");
      isViewingBattle = false;




    }
  }

}
