package roboforge.ui;

import roboforge.BBot;
import roboforge.BDebug;
import roboforge.BGlobal;
import roboforge.BZone;
import roboforge.BZones;

import roboforge.ui.components.*;
import roboforge.ui.components.text.*;
import roboforge.ui.modes.*;
import roboforge.util.graphics.*;

import java.awt.*;
import java.awt.event.*;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;

import java.util.Vector;

import javax.swing.*;
import javax.swing.event.*;

import javax.vecmath.*;


public class BZoneEditor extends BToolPanel
{

  // Internal variables...
  // -------------------------------------------------------------

  protected B3DBotCanvas canvas;
  protected BBot bot;
  protected BZones zones;
  
  // protected BButtonGroup modeGroup;
  protected ZoneModelListener zonesListener;


  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "zonesLabel.text",        "Zone:",
    "zonesLabel.width",       new Integer(50),

    "addZone.icon",           BIconCache.getHSB("add_icon_small.gif"),
    "addZone.tooltip",        "Add a new Zone. [Ins]",
    "addZone.keystroke",      KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, 0),

    "delZone.icon",           BIconCache.getHSB("del_icon_small.gif"),
    "delZone.tooltip",        "Remove this zone. [Del]",
    "delZone.keystroke",      KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0),

    "drag_xz.icon",           BIconCache.getHSB("drag_xz_icon.gif"),
    "drag_xz.tooltip",        "Zone location tool [X]",
    "drag_xz.keystroke",      KeyStroke.getKeyStroke(KeyEvent.VK_X, 0),

    "drag_y.icon",            BIconCache.getHSB("drag_y_icon.gif"),
    "drag_y.tooltip",         "Zone height tool [Y]",
    "drag_y.keystroke",       KeyStroke.getKeyStroke(KeyEvent.VK_Y, 0),

    "drag_scale.icon",        BIconCache.getHSB("drag_scale_icon.gif"),
    "drag_scale.tooltip",     "Zone size tool [S]",
    "drag_scale.keystroke",   KeyStroke.getKeyStroke(KeyEvent.VK_S, 0),
  });


  // Visual elements...
  // -------------------------------------------------------------

  protected BLabel zonesLabel         = new BLabel(res, "zonesLabel");
  protected BComboBox zonesCombo      = new BComboBox();
  protected BNamedEditor editor       = null;
  protected BButton addZone           = new BButton(res, "addZone");
  protected BButton delZone           = new BButton(res, "delZone");

  protected BModeButton xz            = new BModeButton(res, "drag_xz");
  protected BModeButton y             = new BModeButton(res, "drag_y");
  protected BModeButton scale         = new BModeButton(res, "drag_scale");

  protected BModeViewer xzMode;
  protected BModeViewer yMode;
  protected BModeViewer scaleMode;


  // Constructor.
  // -------------------------------------------------------------

  public BZoneEditor(B3DEditingPanel parent)
  {
    super(parent, "zones");

    this.bot         = parent.getBot();
    this.canvas      = parent.getCanvas();
    this.zones       = bot.getZones();
    
    // this.modeGroup   = parent.getModeButtonGroup();

    setupModes();

    layoutZoneEditorChildren();
    configZoneEditorChildren();

    res.cleanse();
    res = null;
  }


  // Children initialization methods...
  // -------------------------------------------------------------

  protected void layoutZoneEditorChildren()
  {
    BBoxPanel box = createHBoxPanel();
    box.add(zonesLabel);
    box.add(zonesCombo);
    box.addSmallMargin();
    box.add(addZone);
    box.addSmallMargin();
    box.add(delZone);

    add(box);

    addSmallMargin();

    box = createHBoxPanel();
    box.addHStrut(50);
    box.add(xz);
    box.addSmallMargin();
    box.add(y);
    box.addSmallMargin();
    box.add(scale);
    box.addGlue();
    add(box);
  }

  protected void configZoneEditorChildren()
  {
    zonesListener = new ZoneModelListener();
    zones.addListDataListener(zonesListener);

    editor = new BNamedEditor(zones);
    editor.setGuard(new BFilenameGuard());


    zonesCombo.setEditable(true);
    zonesCombo.setEditor(editor);
    zonesCombo.setModel(zones);
    zonesCombo.setEnabled(false);

    addZone.addActionListener(new AddZoneListener());
    delZone.addActionListener(new DelZoneListener());
    editor.addActionListener(new BZoneRenameListener());

    setupModeButton(xz,    xzMode);
    setupModeButton(y,     yMode);
    setupModeButton(scale, scaleMode);

  }

  protected void setupModes()
  {
    xzMode    = new BModeViewer(canvas, new BModeZoneXZDrag(canvas));
    yMode     = new BModeViewer(canvas, new BModeZoneYDrag(canvas));
    scaleMode = new BModeViewer(canvas, new BModeZoneScaleDrag(canvas));
  }

  protected void setupModeButton(BModeButton b, BModeMouse mode)
  {
    b.setMode(mode, canvas);
    // modeGroup.add(b);
  }

  
  // Activation methods...
  // -------------------------------------------------------------

  public void onClose()
  {
    zones.removeListDataListener(zonesListener);

    bot    = null;
    canvas = null;
    zones  = null;

    zonesLabel         = null;
    zonesCombo         = null;
    editor            = null;
    addZone           =  null;
    delZone           = null;

    xz            = null;
    y             = null;
    scale         = null;

    xzMode.onClose();
    yMode.onClose();
    scaleMode.onClose();

    xzMode = null;
    yMode = null;
    scaleMode = null;

    super.onClose();
  }
  

  public void setVisible(boolean v)
  {
    if (isActive())
    {
      BMovesPanel p = 
        (BMovesPanel) parent;
    
      p.setEditingZones(v);
      if(zones.size() > 0)
        zonesCombo.setEnabled(true);
      else
        zonesCombo.setEnabled(false);
    }
    
    super.setVisible(v);
  }


  // UI Listeners...
  // -------------------------------------------------------------

  protected class ZoneModelListener implements ListDataListener
  {
    public void contentsChanged(ListDataEvent e)
    {
      BZone z = zones.getSelectedZone();

      if (z != canvas.getSelectedZone())
        canvas.setSelectedZone(z);
    }

    public void intervalAdded(ListDataEvent e)
      { }

    public void intervalRemoved(ListDataEvent e)
      { }
  }

  protected class BZoneRenameListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      BZone z = zones.getSelectedZone();

      if (z != null)
      {
        z.setName(editor.getText());
        z.reattach(false);
      }
    }
  }

  protected class AddZoneListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      if (zones.size() < BZones.MAX_ZONES )
        zones.addZone("Zone");
      else
      {
        BPromptDialog tooLongDialog =  BPromptDialog.createNotifyDialog("Too many zones",
          "A bot may have a maximum of " + BZones.MAX_ZONES + " zones." );

        BApplication.showDialog(tooLongDialog);

      }
      zonesCombo.setEnabled(true);
    }
  }

  protected class DelZoneListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      BZone z = zones.getSelectedZone();

      if (z != null)
      {
        zones.remove(z);
        if(zones.size() == 0)
          zonesCombo.setEnabled(false);
      }
    }
  }




  // -------------------------------------------------------------
  // Zone manipulation modes...
  // -------------------------------------------------------------

  protected class BModeZoneDrag extends BModePlanePickDrag
  {
    protected BZone zone;
    protected Tuple3f min, max;

    protected final float TO_WORLD  = BGlobal.CANVAS_TO_WORLD_SCALE;
    protected final float TO_CANVAS = BGlobal.WORLD_TO_CANVAS_SCALE;


    // Constructors...
    // -------------------------------------------------------------

    public BModeZoneDrag(B3DCanvas canvas)
    {
      super(canvas);

      float r = BGlobal.ZONE_RANGE;

      min = new Point3f(-r, -r, -r);
      max = new Point3f(+r, +r, +r);
    }

    protected Vector3d ref = new Vector3d();


    // Activation methods.
    // -------------------------------------------------------------

    public void onActivate()
    {
      super.onActivate();
      zone = zones.getSelectedZone();

      bot.setPickable(false);
      zones.setPickable(true);
    }

    public void onDeactivate()
    {
      bot.setPickable(true);
      zones.setPickable(false);

      super.onDeactivate();
    }



    // Mouse methods.
    // -------------------------------------------------------------

    public void mousePressed(MouseEvent e)
    {
      super.mousePressed(e);
      if (zone != null)
        { zone.setDisplacementRange(min, max); }
    }


    public void mouseReleased(MouseEvent e)
    {
      super.mouseReleased(e);
      if (zone != null)
        { zone.setNoDisplacementRange(); }
    }


    // Pick methods.
    // -------------------------------------------------------------

    protected void onPickZone(BZone z)
    {
      zone = z;

      Vector3f d = zone.getDisplacement();
      ref.set(d);
      ref.scale(TO_CANVAS);
      plane.setReferencePoint(ref);

      zones.setSelectedItem(zone);
    }
  }


  protected class BModeZoneXZDrag extends BModeZoneDrag
  {

    protected Point3d offset = new Point3d();

    // Constructor.
    // -------------------------------------------------------------

    public BModeZoneXZDrag(B3DCanvas canvas)
      { super(canvas); }


    // Mouse-event methods.
    // -------------------------------------------------------------

    public void mousePressed(MouseEvent e)
    {
      super.mousePressed(e);

      // Remember difference between pick point and zone center...
      offset.sub(intersection, plane.getReferencePoint());
    }

    public void mouseDragged(MouseEvent e)
    {
      super.mouseDragged(e);

      if (zone != null)
      {
        float zoneX = (float) (intersection.x - offset.x) * TO_WORLD;
        float zoneZ = (float) (intersection.z - offset.z) * TO_WORLD;
        zone.setXZ(zoneX, zoneZ);
      }
    }
  }


  protected class BModeZoneYDrag extends BModeZoneDrag
  {
    protected final float MOUSE_TO_METRES = 0.025f;

    // Constructors...
    // -------------------------------------------------------------

    public BModeZoneYDrag(B3DCanvas canvas)
    {
      super(canvas);
      plane.setNormal(new Vector3d(0,0,1));
    }

    // Mouse-event methods.
    // -------------------------------------------------------------

    public void mouseDragged(MouseEvent e)
    {
      super.mouseDragged(e);

      if (zone != null)
      {
        float dZoneY = (float) -dy * MOUSE_TO_METRES; // (float) delta.y * TO_WORLD; // / BGlobal.CHASSIS_SCALE;
        zone.deltaY(dZoneY);
      }
    }
  }


  protected class BModeZoneScaleDrag extends BModeZoneDrag
  {
    protected final double MOUSE_TO_SCALE = 0.02;

    // Constructors...
    // -------------------------------------------------------------

    public BModeZoneScaleDrag(B3DCanvas canvas)
      { super(canvas); }

    // Mouse-event methods.
    // -------------------------------------------------------------

    public void mouseDragged(MouseEvent e)
    {
      super.mouseDragged(e);

      if (zone != null)
      {
        double dZoneY = ((double) -dy) * MOUSE_TO_SCALE;
        double dZoneX = ((double) +dx) * MOUSE_TO_SCALE;
        double dScale = dZoneY + dZoneX / 2.0;

        zone.deltaScale(dScale);
      }
    }
  }

}