package roboforge.ui.ai;

import roboforge.ai.BDecisionTreeAI;
import roboforge.ai.decisions.*;
import roboforge.ai.expressions.*;
import roboforge.ai.expressions.parser.*;
import roboforge.ai.memory.*;
import roboforge.ai.values.*;

import roboforge.ui.BToolPanel;
import roboforge.ui.components.*;
import roboforge.ui.components.text.*;
import roboforge.ui.events.*;
import roboforge.util.graphics.*;

import java.awt.*;
import java.awt.event.*;

import java.util.Vector;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;



public class BMemoryBankPanel extends BToolPanel
  implements ListSelectionListener
{

  // Internal variables...
  // -------------------------------------------------------------

  protected BMemoryBank bank;
  protected BMemoryItem item;

  protected UpdateStateListener updater;
 // protected BExpressionParser parser;


  // UI Resources ...
  // -------------------------------------------------------------

  protected static final int LABEL_W = 100;

  protected BUIResources res = new BUIResources(new Object[]
  {
    "memoryLabel.text",           "memory items",
    "memoryLabel.fontsize",       new Integer(14),
    "memoryLabel.height",         new Integer(20),
    "memoryLabel.fontstyle",      new Integer(Font.BOLD),
    "memoryLabel.v_align",        new Integer(SwingConstants.BOTTOM),

    "memory.border",              new EmptyBorder(5, 5, 5, 5),

    "currentLabel.text",          "current item",
    "currentLabel.fontsize",      new Integer(14),
    "currentLabel.fontstyle",     new Integer(Font.BOLD),

    "addMemory.icon",             BIconCache.getHSB("add_icon_small.gif"),
    "addMemory.tooltip",          "Add a new memory item.",

    "delMemory.icon",             BIconCache.getHSB("del_icon_small.gif"),
    "delMemory.tooltip",          "Remove the selected memory item.",

    "nameLabel.text",             "Name:",
    "nameLabel.h_align",          new Integer(SwingConstants.RIGHT),
    "nameLabel.width",            new Integer(LABEL_W),

    "dataTypeLabel.text",         "Data Type:",
    "dataTypeLabel.h_align",      new Integer(SwingConstants.RIGHT),
    "dataTypeLabel.width",        new Integer(LABEL_W),

    "defaultLabel.text",          "Starting Value:",
    "defaultLabel.h_align",       new Integer(SwingConstants.RIGHT),
    "defaultLabel.width",         new Integer(LABEL_W),
  });


  // Visual elements...
  // -------------------------------------------------------------

  protected BLabel memoryLabel      = new BLabel(res, "memoryLabel");
  protected BList  memory           = new BList(res, "memory");
  protected BScrollPane scroller    = new BScrollPane(memory);

  protected BLabel currentLabel     = new BLabel(res, "currentLabel");
  protected BLine  currentLine      = new BLine();

  protected BButton addMemory       = new BButton(res, "addMemory");
  protected BButton delMemory       = new BButton(res, "delMemory");

  protected BLabel nameLabel        = new BLabel(res, "nameLabel");
  protected BNamedField name        = new BNamedField();

  protected BLabel dataTypeLabel    = new BLabel(res, "dataTypeLabel");
  protected BLabel defaultLabel     = new BLabel(res, "defaultLabel");

  protected BComboBox dataType      = new BComboBox(BValue.getTypeNames());
  protected BTextField defaultVal   = new BTextField();

  protected BDecisionTreeAI ai;

  // Constructor...
  // -------------------------------------------------------------

  public BMemoryBankPanel(BDecisionTreeAI ai)
  {
    super ("memory");

    this.bank = ai.getMemoryBank();
    this.ai = ai;
    //this.parser = new BExpressionParser(ai);

    layoutBankChildren();
    configBankChildren();
  }


  protected void layoutBankChildren()
  {
    // setEmptyBorder(MARGIN);

    BBoxPanel box = createHBoxPanel();
    box.addHGlued(memoryLabel);
    box.addSmallMargin();
    box.add(addMemory);
    box.addSmallMargin();
    box.add(delMemory);

    add(box);

    addTinyMargin();
    addHGlued(scroller);

    addVStrut(20);

    addHGlued(currentLabel);
    addTinyMargin();
    addHGlued(currentLine);
    addMargin();

    addLinear(nameLabel,     getHStrut(TINY_MARGIN), name);
    addSmallMargin();

    addLinear(dataTypeLabel, getHStrut(TINY_MARGIN), dataType);
    addSmallMargin();

    addLinear(defaultLabel,  getHStrut(TINY_MARGIN), defaultVal);
    addSmallMargin();
  }

  protected void configBankChildren()
  {
    // memoryModel = new BListModel(bot.getStates());

    memory.setModel(bank);
    // memory.setRowsDeleteable(true);
    // memory.setItemName("memory item");

    memory.addListSelectionListener(this);

    memory.setCellRenderer(
      new BNamedIconicListCellRenderer());


    addMemory.addActionListener(new AddMemoryListener());
    delMemory.addActionListener(new DelMemoryListener());

    name.setEnabled(false);
    name.setGuard(new BIdentifierGuard());

    updater = new UpdateStateListener();
    dataType.addActionListener(updater);
    defaultVal.addActionListener(updater);

    dataType.setEnabled(false);
    defaultVal.setEnabled(false);
  }


  // UI Listeners...
  // -------------------------------------------------------------

  protected class AddMemoryListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      BMemoryItem m = new BMemoryItem("New_Memory_Item");
      bank.add(m);
      memory.setSelectedValue(m, true);
    }
  }

  protected class DelMemoryListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      Object m = memory.getSelectedValue();
      if (m != null)
        bank.remove(m);
    }
  }

  public void valueChanged(ListSelectionEvent e)
  {
    item = (BMemoryItem) memory.getSelectedValue();
    name.setNameable(item);

    boolean isItemSelected = (item != null);

    name.setEnabled(isItemSelected);
    dataType.setEnabled(isItemSelected);
    defaultVal.setEnabled(isItemSelected);

    updater.setEnabled(false);

    if (isItemSelected)
    {
      dataType.setSelectedIndex(item.getType());
      defaultVal.setText(item.getInitialValue().toString());
    }
    else
    {
      dataType.setSelectedItem(null);
      defaultVal.setText("");
    }

    updater.setEnabled(true);
  }

  public class UpdateStateListener extends BActionAdapter
  {
    public void actionApproved(ActionEvent e)
    {
      if (item != null)
      {
        int type = dataType.getSelectedIndex();

        if (type < 0)
          type = BType.UNDEFINED;

        item.setType(type);

        BValue initial;

        BExpressionParser parser = ai.getParser();

        if (parser != null)
          initial = parser.parseForValue(defaultVal.getText());
        else
          initial = BValue.UNDEFINED;

        item.setInitialValue(initial);
      }
    }
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public boolean isAlwaysOnTop()
    { return true; }
    
}


