package roboforge.ui.components;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

import javax.swing.*;


public abstract class BAnimatedLabel extends BLabel
  implements ActionListener
{

  // Class constants...
  // -------------------------------------------------------------

  public static final int LOOP  = 0;


  // Member variables...
  // -------------------------------------------------------------

  protected Timer  timer;
  protected int    delay;

  protected int    current;
  protected int    frames;

  protected AnimationStop stopper;


  // Constructors...
  // -------------------------------------------------------------

  public BAnimatedLabel()
    { this(null, null); }

  public BAnimatedLabel(BUIResources res, String key)
  {
    super(res, key);
    stopper = new AnimationStop();
  }

  public void initializeUI(BUIResources res, String key)
  {
    super.initializeUI(res, key);

    delay = res.getInt(key+ ".delay");
    if (delay == 0)
      delay = 100;

    frames = res.getInt(key+ ".frames");
  }


  // Implementation methods...
  // -------------------------------------------------------------

  protected void setupTimer()
    { timer = new Timer(delay, this); }

  public void actionPerformed(ActionEvent e)
  {
    if (current == 0)
      animationStart();

    animation();
    current++;

    if (frames != LOOP)
      if (current >= frames)
        stop();
  }



  // Accessor methods...
  // -------------------------------------------------------------

  public void start()
  {
    if (timer == null)
      setupTimer();

    if (!timer.isRunning())
    {
      current = 0;
      timer.start();
    }
  }

  public void stop()
  {
    if (timer != null)
    {
      timer.stop();
      SwingUtilities.invokeLater(stopper);
    }
  }

  protected class AnimationStop implements Runnable
  {
    public void run()
      { animationStop(); }
  }


  // Abstract methods...
  // -------------------------------------------------------------

  public void animationStart()
    { }

  public void animation()
    { }

  public void animationStop()
    { }


}