package roboforge.ui.components;

import roboforge.BGlobal;
import roboforge.util.io.*;
import roboforge.util.text.*;

import java.io.*;
import javax.swing.DefaultListModel;
import javax.swing.ListSelectionModel;
import java.util.*;


// import javax.swing.event.*;

public class BFileList extends BList
{

  protected static final BFilenameFilter DEFAULT_FILTER =
    new BInternalFilter();

  protected String path;
  protected String[] files;

  // protected String mask;

  protected BFilenameFilter filter;
  protected boolean isMultiSelectable;
  protected DefaultListModel model = null;

  public BFileList()
    { this(null, null, DEFAULT_FILTER); }

  public BFileList(BUIResources res, String key)
    { this(res, key, DEFAULT_FILTER); }

  public BFileList(BUIResources res, String key, BFilenameFilter filter)
  {
    super(res, key);

    this.filter = filter;

    model = new DefaultListModel();
    setModel(model);

    setMultiSelectable(false);
  }

  /*
  public void setFilteredExtension(String mask)
  {
    filter.addExtension(mask);
    this.mask = mask;
  }
  */

  public void setFilter(BFilenameFilter filter)
  {
    this.filter = filter;

    try
    {
      if (path != null)
        setSourcePath(path);
    }
    catch (FileNotFoundException e)
      { }
  }

  public BFilenameFilter getFilter()
    { return filter; }


  public void setSourcePath(String path)
    throws FileNotFoundException
  {
    this.path = path;

    // Load the files in this path.
    File file = new File(path);

    if (!file.canRead())
      throw new FileNotFoundException("BFileList.setSourcePath: path ["+path+"] does not exist!");

    files = file.list(filter);

    model.removeAllElements();

    // sort the files alphabetically
    TreeSet sortSet = new TreeSet() ;

    for (int i = files.length-1 ; i >= 0 ; i-- )
      sortSet.add(files[i]);

    int i = 0;
    for (Iterator it = sortSet.iterator(); it. hasNext()   ; i++ )
    {
      String name = (String)it.next();
      model.addElement(toDisplayable(name));
      files[i] = name;

    }
  //  for (int i=0; i<files.length; i++)
    //  model.addElement(toDisplayable(files[i]));
  }

  public void updateFile(String oldFile, String newFile)
  {
    int n = model.getSize();

    for (int i=0; i<n; i++)
      if (files[i].equals(oldFile))
      {
        files[i] = newFile;
        int oldSelectedIndex = this.getSelectedIndex();

        model.removeElementAt(i);
        model.insertElementAt(toDisplayable(newFile), i);

        this.setSelectedIndex(oldSelectedIndex);
        return;
      }

  }


  public String getSourcePath()
    { return path; }


  protected String toDisplayable(String file)
    { return filter.toDisplayable(file); }

  protected String toFileName(String display)
    { return filter.toFilename(display); }

  protected String toFileNameNoExt(String display)
    { return BStrings.stripExtension(filter.toFilename(display)); }

  protected String toFileName(int index)
  {
    if (index == -1)
      return null;
    else if (files != null)
      return files[index];
    else
      return null;
  }


  public boolean isEmpty()
    { return (getModel().getSize() == 0); }

  public String getFileAt(int index)
    { return toFileName(index); }


  public void setMultiSelectable(boolean enabled)
  {
    this.isMultiSelectable = enabled;

    if (enabled)
      setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
    else
      setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
  }

  public boolean isMultiSelectable()
    { return isMultiSelectable; }


  public String getSelectedFile()
    { return toFileName(getSelectedIndex()); }

  public String getSelectedFileWithPath()
  {
    if (getSelectedFile() == null)
      return null;
    else
      return path + getSelectedFile();
  }
  
  public String getSelectedFileNoExt()
    { return toFileNameNoExt(getSelectedValue().toString()); }
  
  public String getSelectedFileWithPathNoExt()
    { return path+ toFileNameNoExt(getSelectedValue().toString()); }

  public void setSelectedFile(String file)
  {
    int n = model.getSize();

    for (int i=0; i<n; i++)
      if (files[i].equals(file))
      {
        setSelectedIndex(i);
        return;
      }
  }


  public void removeAllFiles()
    { model.removeAllElements(); }
}