package roboforge.ui.components;

import roboforge.BGlobal;

import java.awt.*;
import javax.swing.JLabel;
import javax.swing.Icon;
import javax.swing.SwingConstants;




public class BLabel extends JLabel
{

  // Public constants...
  // -------------------------------------------------------------

  public static final Integer ALIGN_LEFT =
    new Integer(SwingConstants.LEFT);
      
  public static final Integer ALIGN_CENTER =
    new Integer(SwingConstants.CENTER);

  public static final Integer ALIGN_RIGHT =
    new Integer(SwingConstants.RIGHT);
  
  public static final Integer ALIGN_TOP =
    new Integer(SwingConstants.TOP);

  public static final Integer ALIGN_BOTTOM =
    new Integer(SwingConstants.BOTTOM);

    
  
  // Internal variables...
  // -------------------------------------------------------------

  private static RenderingHints hints = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );

  protected boolean antiAliased = false;


  
  // Constructor.
  // -------------------------------------------------------------

  public BLabel()
  {
    super();
    initializeUI(null, null);
  }

  public BLabel(BUIResources res, String resourceKey)
  {
    super();
    initializeUI(res, resourceKey);
  }

  public BLabel(String text)
  {
    super(text);
    initializeUI(null, null);
  }

  public BLabel(Icon icon)
  {
    super(icon);
    initializeUI(null, null);
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setAntiAliased(boolean aa)
    { this.antiAliased = aa; }

  public boolean isAntiAliased()
    { return this.antiAliased; }

  public void setIcon(Icon icon)
  {
    super.setIcon(icon);

    if (icon instanceof HSBImageIcon)
      setHSBIcon((HSBImageIcon) icon);
  }

  protected void setHSBIcon(HSBImageIcon icon)
    { setDisabledIcon(icon); }


  // Visual Initialization methods...
  // -------------------------------------------------------------

  protected static final int PREF_HEIGHT = 19;

  public void initializeUI(BUIResources res, String key)
  {
    // Set centered alignment, for correct BoxLayout display...
    setAlignmentX(Component.CENTER_ALIGNMENT);
    setAlignmentY(Component.CENTER_ALIGNMENT);

    setMinimumSize(new Dimension(getMinimumSize().width, PREF_HEIGHT));
    setPreferredSize(new Dimension(getPreferredSize().width, PREF_HEIGHT));
    setMaximumSize(new Dimension(getMaximumSize().width, PREF_HEIGHT));

    if (res != null)
    {
      String text = res.getString(key+ ".text");
      if (text != null)
        setText(text);

      Icon icon = res.getIcon(key+ ".icon");
      if (icon != null)
        setIcon(icon);

      Integer alignment = (Integer) res.get(key+ ".h_align");
      if (alignment != null)
        setHorizontalAlignment(alignment.intValue());

      alignment = (Integer) res.get(key+ ".v_align");
      if (alignment != null)
        setVerticalAlignment(alignment.intValue());

      String   fontname = res.getString(key+ ".fontname");
      Integer fontstyle = (Integer) res.get(key+ ".fontstyle");
      Integer  fontsize = (Integer) res.get(key+ ".fontsize");

      if ((fontname != null) || (fontstyle != null) || (fontsize != null))
      {
        Font f = getFont();

        String fn = (fontname  != null) ? fontname             : f.getFontName();
        int    fs = (fontstyle != null) ? fontstyle.intValue() : f.getStyle();
        int    ft = (fontsize  != null) ? fontsize.intValue()  : f.getSize();

        setFont(new Font(fn, fs, ft));
      }

      Font font = res.getFont(key+ ".font");
      if (font != null)
        setFont(font);

      Boolean a = (Boolean) res.get(key+ ".antialiased");
      if (a != null)
        antiAliased = a.booleanValue();

     /* Color c = (Color) res.get(key+ ".foreground");
      if (c != null)
        setForeground( c ); */

      res.initComponent(this, key);
    }
  }

  public void paint(Graphics g)
  {
    if (antiAliased)
    {
      Graphics2D g2d = (Graphics2D) g;
      g2d.setRenderingHints(hints);
    }

    super.paint(g);
  }

}
