package roboforge.ui.components;

import roboforge.BGlobal;

import java.awt.*;
import javax.swing.JLabel;
import javax.swing.UIManager;

public class BLine extends JLabel
{

  // Internal variables...
  // -------------------------------------------------------------

  private static RenderingHints hints = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );


  // Member variables...
  // -------------------------------------------------------------

  protected Color lineColor = Color.white;

  // Constructor.
  // -------------------------------------------------------------

  public BLine(BUIResources res, String resourceKey)
  {
    super();
    initializeUI(res, resourceKey);
  }

  public BLine()
  {
    super();
    initializeUI(null, null);
  }

  // Visual Initialization methods...
  // -------------------------------------------------------------

  public void initializeUI(BUIResources res, String key)
  {
    // Set centered alignment, for correct BoxLayout display...
    setAlignmentX(Component.CENTER_ALIGNMENT);
    setAlignmentY(Component.CENTER_ALIGNMENT);

    Color fg = null;

    if (res != null)
    {
      res.initComponent(this, key);
      fg = res.getColor(key+ ".foreground");
    }

    if (fg != null)
      setForeground(fg);
    else
      setForeground(UIManager.getColor("Line.foreground"));
  }

  // Action methods...
  // -------------------------------------------------------------

  public void paint(Graphics g)
  {
    Graphics2D g2D = (Graphics2D) g;
    g2D.addRenderingHints(hints);

    g2D.setColor(getForeground());
    Dimension d = getSize();

    // TODO: Fix problem of 0 height...
    if (d.width == 1)
      d.width = 0;
    else if (d.height == 1)
      d.height = 0;

    g2D.drawLine(0, 0, d.width, d.height);
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public Dimension getMinimumSize()
  {
    Dimension d = super.getMinimumSize();
    if (d.width  == 0) d.width  = 1;
    if (d.height == 0) d.height = 1;
    return d;
  }

  public Dimension getPreferredSize()
  {
    Dimension d = super.getPreferredSize();
    if (d.width  == 0) d.width  = 1;
    if (d.height == 0) d.height = 1;
    return d;
  }

  public Dimension getMaximumSize()
  {
    Dimension d = super.getMaximumSize();
    if (d.width  == 0) d.width  = 1;
    if (d.height == 0) d.height = 1;
    return d;
  }


}