package roboforge.ui.components;

import roboforge.ui.modes.*;

import java.awt.event.*;
import javax.swing.*;

public class BMultiModalButton extends BButton
{

  protected BModeManager manager = new BModeManager();
  //YLY added in listener intoMember to remove onClose. (was holding handle)
  protected ModeSwitchListener switchListener;

  // Constructor.
  // -------------------------------------------------------------

  public BMultiModalButton(BUIResources res, String key)
  {
    super(res, key);

    switchListener =  new ModeSwitchListener();
    addActionListener(switchListener);

    manager.requestActivate();
  }

  public BMultiModalButton(String text)
  {
    super(text);

    switchListener =  new ModeSwitchListener();
    addActionListener(switchListener);

    manager.requestActivate();
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void addMode(BMode m)
    { manager.addMode(m); }

  public void removeMode(BMode m)
    { manager.removeMode(m); }

  public BMode getMode(int i)
    { return manager.getMode(i); }

  public int getModeCount()
    { return manager.getModeCount(); }

  public BMode getSelectedMode()
    { return manager.getSelectedMode(); }

  public void setSelectedMode(BMode m)
  {
    manager.setSelectedMode(m);

    if (m != null)
      setIcon(m.getIcon());
  }



  // UI Listener...
  // -------------------------------------------------------------

  protected class ModeSwitchListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      int i = manager.getSelectedModeIndex();

      // Determine next selected mode...
      if (i == -1)
      {
        if (manager.getModeCount() > 0)
          i = 0;
      }
      else
      {
        i++;
        if (i >= manager.getModeCount())
          i = 0;
      }

      if (i > -1)
      {
        manager.setSelectedModeIndex(i);
        BMode m = manager.getSelectedMode();

        if (m != null)
          setIcon(m.getIcon());
      }
      else
        setIcon(null);
    }
  }


  public String getToolTipText()
  {
    BMode m = manager.getSelectedMode();

    if (m != null)
      return super.getToolTipText()+ m.getName();
    else
      return super.getToolTipText();
  }

  public void onClose()
  {
    manager.onClose();
    manager = null;

    removeActionListener(switchListener);
    switchListener = null;
  }

}