package roboforge.ui.components;

import roboforge.BDebug;
import roboforge.BNameable;

import javax.swing.text.*;


public class BNamedField extends BGuardedTextField
{

  // Member variables...
  // -------------------------------------------------------------

  protected BNameable named;

  
  // Constructor.
  // -------------------------------------------------------------

  public BNamedField(BUIResources res, String key)
    { super(res, key); }

  public BNamedField()
    { super(); }


  // Accessor methods.
  // -------------------------------------------------------------

  public void setNameable(BNameable n)
  {
    this.named = n;

    if (named != null)
      setText(named.getName());
    else
      setText("");
  }

  public BNameable getNameable()
    { return named; }

  

  // Document model stuff.
  // -------------------------------------------------------------

  protected Document createDefaultModel()
    { return new NameDocument(); }

  protected class NameDocument extends GuardedTextDocument
  {
    public void insertString(int o, String s, AttributeSet a)
      throws BadLocationException
    {
      super.insertString(o, s, a);
      updateName();
    }

    public void remove(int o, int l) throws BadLocationException
    {
      super.remove(o, l);
      updateName();
    }

    protected void updateName()
    {
      if (named == null)
        return;
      
      try
        { named.setName(getText(0, getLength())); }
      catch (BadLocationException e)
        { BDebug.print("ERROR: Could not edit named item '"+ named+ "'."); }

    }
  }

}