package roboforge.ui.components;

import roboforge.BGlobal;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


public class BPromptDialog extends BOKCancelDialog
{

  // Constructor...
  // -------------------------------------------------------------

  protected PromptClient promptClient;

  public BPromptDialog
  (
    String title,
    String msg,
    boolean showOK,
    boolean showCancel,
    String OKName,
    String cancelName
  )
  {
    super(title, showOK, showCancel, OKName, cancelName);

    promptClient = new PromptClient(msg);
    setClient(promptClient);
  }



  public static BPromptDialog createYesNoDialog(String title, String message)
    { return new BPromptDialog(title, message, true, true, "Yes", "No"); }

  public static BPromptDialog createYesNoDialog(String title)
    { return new BPromptDialog(title, "", true, true, "Yes", "No"); }
  
  public static BPromptDialog createOKDialog(String title, String message)
    { return new BPromptDialog(title, message, true, false, "OK", ""); }

  public static BPromptDialog createMessageDialog(String title, String message)
  { return new BPromptDialog(title, message, true, true, "OK", "Cancel"); }

  public static BPromptDialog createNotifyDialog()
    { return createNotifyDialog("", ""); }

  public static BPromptDialog createNotifyDialog(String title, String message)
  {
    BPromptDialog result = createOKDialog(title, message);
    result.setBlocking(false);
    return result;
  }


  public void setMessage(String msg)
    { promptClient.setMessage(msg); }

  public void setScrollable(boolean b)
  {
    showOK = false;
    promptClient = new PromptClient("", b);
    setClient(promptClient);
    showOK = true;

  }


  protected class PromptClient extends OKCancelClient
  {
    // Visual elements...
    // -------------------------------------------------------------

    protected BMultiLabel message   = new BMultiLabel();
    protected BScrollPane scroller = new BScrollPane(res, "scroller", message);
    protected boolean scrollable = false;

    public PromptClient(String msg)
      { this(msg, false);}

    public PromptClient(String msg, boolean scrollable)
    {
      super();
      message.setText(msg);
      this.scrollable = scrollable;
      // message.setVerticalAlignment(BLabel.TOP);

      layoutPromptChildren();
      configPromptChildren();
    }

    protected void layoutPromptChildren()
    {
      if (scrollable)
      {
        setHVGlue(scroller);
        add(scroller);
      }
      else
      {
        setHVGlue(message);
        add(message);
      }

      addSmallMargin();


    }

    protected void configPromptChildren()
    {
      message.setForeground(BGlobal.colors.getMidShade());
      scroller.setBorder(null);
  
    }

    public void setMessage(String msg)
    {
      message.setText(msg);


    }
  }
}