package roboforge.ui.components;

import roboforge.BGlobal;
import roboforge.util.graphics.*;

import java.awt.*;
//import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;



public class BSectionButton extends BToggleButton
{

  // protected boolean selected;
  // protected ActionListener action;

  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
protected Color back      = BGlobal.colors.getDarkShade();
  protected Color normal    = BGlobal.colors.getMidShade();
  protected Color rollover  = BGlobal.colors.getLightShade();
  protected Color select    = BGlobal.colors.getWhiteShade();
  protected Color backsel   = BColors.blendHSB(back, normal, 0.25f);

  protected BUIResources res = new BUIResources( new Object[]
  {
    "client.opaque",        Boolean.FALSE,
    
    "icon.foreground",      normal,
    "icon.size",            new Dimension(70, 70),
    "icon.opaque",          Boolean.FALSE,
      
    "title.foreground",     normal,
    "title.border",         new EmptyBorder(4, 8, 4, 0),
    "title.height",         new Integer(33),

    "line.height",          new Integer(1),
    "line.foreground",      back,
      
    "subtitle.border",      new EmptyBorder(8, 8, 0, 0),
    "subtitle.v_align",     BLabel.ALIGN_TOP,
  });
    
  protected ImageIcon icon_normal;
  protected ImageIcon icon_select;
  protected ImageIcon title_normal;
  protected ImageIcon title_select;

  protected BBoxPanel client   = new BBoxPanel(res, "client", BBoxPanel.X_AXIS);
  protected BLabel    icon     = new BFilledLabel(res, "icon");
  protected BLabel    title    = new BLabel(res, "title");
  protected BLine     line     = new BLine(res, "line");
  protected BLabel    subtitle = new BLabel(res, "subtitle");

    
  protected Cursor hand = 
    Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
      
      
    
  public BSectionButton(BUIResources res, String key)
  {
    super(res, key); // , BBoxPanel.X_AXIS);

    initializeSectionUI(res, key);
      
    layoutSectionChildren();
    configSectionChildren();
  }
    

  /*
  
  public void addActionListener(ActionListener a)
    { this.action = a; }

  public void setAction(Action a)
    { this.action = a; }
  */
  
  
  protected void initializeSectionUI(BUIResources res, String key)
  {
    setCursor(hand);
    setOpaque(false);
    setRolloverEnabled(true);
      
    if (res != null)
    {
      Image icon_image  = (Image) res.get(key+ ".icon");
      Image title_image = (Image) res.get(key+ ".title");
        
      icon_select = new ImageIcon(icon_image);
      icon_normal = new HSBImageIcon(icon_image, normal);
        
      title_select = new ImageIcon(title_image);
      title_normal = new HSBImageIcon(title_image, normal);
        
      icon.setIcon(icon_normal);
      title.setIcon(title_normal);

      String s = res.getString(key+ ".subtitle");
      subtitle.setText(s);
    }
  }
    
  protected void layoutSectionChildren()
  {
    BBoxPanel text = 
      new BBoxPanel(BBoxPanel.Y_AXIS);
      
    text.setOpaque(false);
    text.addHGlued(title);
    text.addHGlued(line);
    text.addHVGlued(subtitle);
      
    client.add(icon);
    client.add(text);
    
    setLayout(new BorderLayout());
    add(client, "Center");
  }

  protected void configSectionChildren()
  {
    /*
    MouseHandler handler = 
      new MouseHandler();
      
    addMouseListener(handler);
    */
    
    ButtonModel model = getModel();
    model.addChangeListener(new ModelListener());
  }
  
  
  protected class ModelListener implements ChangeListener
  {
    public void stateChanged(ChangeEvent e)
      { setAppearance(); }
  }

  
  protected void setAppearance()
  {
    ButtonModel model = getModel();
    
    if (model.isSelected() || model.isPressed())
    {
      icon.setBackground(backsel);
      icon.setIcon(icon_select);
      title.setIcon(title_select);
      subtitle.setForeground(select);
      line.setForeground(select);
    }
    else if (model.isRollover())
    {
      icon.setBackground(backsel);
      icon.setIcon(icon_select);
      title.setIcon(title_normal);
      subtitle.setForeground(normal);
      line.setForeground(normal);
    }
    else
    {
      icon.setBackground(back);
      icon.setIcon(icon_normal);
      title.setIcon(title_normal);
      subtitle.setForeground(back);
      line.setForeground(back);
    }
  }
    
  
  /*
  protected class MouseHandler extends MouseAdapter
  {
    public void mouseEntered(MouseEvent e)
      { setHighlighted(true); }
      
    public void mouseExited(MouseEvent e)
    {
      if (!isSelected()) // (!selected)
        setHighlighted(false); 
    }

    public void mousePressed(MouseEvent e)
      { setSelected(true); }
    
    public void mouseReleased(MouseEvent e)
    {
      // Check if released inside active area..
      Point p = e.getPoint();
      if (!contains(p.x, p.y))
        return;
          
      if (action != null)
        action.actionPerformed(new ActionEvent(
          BSectionButton.this,
          ActionEvent.ACTION_PERFORMED, 
          null));
    }
  }
  */
 
  
  protected void paintComponent(Graphics g)
  {
    // Override, to avoid the default toggle-button
    // appearance being painted.
  }
  
  protected void paintBorder(Graphics g)
  {
    // Override, to avoid the default toggle-button
    // border appearance being painted.
  }
  
  
}
