package roboforge.ui.console;

import roboforge.geometry.BRectangle3D;

import javax.media.j3d.*;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

public class BTexturePortion
{

  // Member variables...
  // -------------------------------------------------------------

  protected BManagedTexture texture;
  protected int x, y, w, h;


  // Constructors...
  // -------------------------------------------------------------

  public BTexturePortion()
    { }

  public BTexturePortion(BManagedTexture texture, int x, int y, int w, int h)
  {
    setTexture(texture);
    setBounds(x, y, w, h);
  }


  // Accessor methods...
  // -------------------------------------------------------------

  public void setTexture(BManagedTexture texture)
    { this.texture = texture; }

  public BManagedTexture getTexture()
    { return texture; }

  public void setBounds(int x, int y, int w, int h)
  {
    this.x = x;
    this.y = y;
    this.w = w;
    this.h = h;
  }

  public void setLocation(int x, int y)
  {
    this.x = x;
    this.y = y;
  }

  public void setSize(int w, int h)
  {
    this.w = w;
    this.h = h;
  }

  public void setImage(BufferedImage image)
  {
    Graphics2D g = createGraphics();
    g.drawImage(image, 0, 0, null);

    texture.update();
  }

  public Graphics2D createGraphics()
  {
    Graphics2D g = texture.createGraphics();
    g.translate(x, y);
    g.setClip(0, 0, w, h);

    return g;
  }

  public void update()
    { texture.update(); }

  public int getWidth()
    { return w; }

  public int getHeight()
    { return h; }


  // Texture coord methods...
  // -------------------------------------------------------------

  public float getLeft()
    { return (float) x / (float) texture.getWidth(); }

  public float getTop()
    { return (float) y / (float) texture.getHeight(); }

  public float getRight()
    { return (float) (x + w - 1) / (float) texture.getWidth(); }

  public float getBottom()
    { return (float) (y + h - 1) / (float) texture.getHeight(); }


  public BRectangle3D getRectangle()
  {
    BRectangle3D result = new BRectangle3D(w, h);

    float tw = (float) texture.getWidth()  - 1f;
    float th = (float) texture.getHeight() - 1f;

    float l = (float) x / tw;
    float t = (float) 1.0f - (y / th);
    float r = (float) (x + w - 1) / tw;
    float b = (float) 1.0f - ((y + h - 1) / th);

    result.setTextureCoords(l, t, r, b);

    return result;
  }

  public void cleanse()
    { texture.returnPortion(this); }
}