package roboforge.ui.events;

import roboforge.BDebug;
import java.util.Vector;


public class BActivationSupport
{

  // Class Variables...
  // -------------------------------------------------------------

  protected static final int EXPECTED_LISTENERS = 1;
    // Expected number of listeners to be registered...

  protected static final boolean DEBUG_ACTIVATION = false;
    // Issues debug messages on activation events.


  // Member Variables...
  // -------------------------------------------------------------

  protected BActivateable source = null;
    // Supported object..

  protected Vector listeners = null;
    // Vector of activation listeners..

  protected boolean isActive = false;
    // Whether the object being supported is currently active..

  protected boolean isClosed = false;
    // Whether the object being supported is currently closed..




  // Constructor...
  // -------------------------------------------------------------

  public BActivationSupport(BActivateable source)
  {
    this.source = source;
  }


  // Listener mgmt. methods...
  // -------------------------------------------------------------

  public void addActivationListener(BActivationListener l)
  {
    // Lazily initialize listener vector (saves memory if no listeners)
    if (listeners == null)
      listeners = new Vector(EXPECTED_LISTENERS);

    listeners.add(l);
  }

  public void removeActivationListener(BActivationListener l)
    { listeners.remove(l); }



  // Status inquiry methods...
  // -------------------------------------------------------------

  public boolean isActive()
    { return this.isActive; }

  public boolean isClosed()
    { return this.isClosed; }



  // Activation request methods...
  // -------------------------------------------------------------

  public void requestActivate()
    { if (!isActive) source.onActivate(); }

  public void requestDeactivate()
    { if (isActive) source.onDeactivate(); }

  public void requestClose()
    { if (!isClosed) source.onClose(); }



  // Activation methods...
  // -------------------------------------------------------------

  public void fireActivate()
  {
    isActive = true;
    fireActivationChanged(BActivationEvent.ACTIVATE);

    if (DEBUG_ACTIVATION)
      debug("[ACTIVATE]  ");
  }

  public void fireDeactivate()
  {
    if (DEBUG_ACTIVATION)
      debug("[DEACTIVATE]");

    isActive = false;
    fireActivationChanged(BActivationEvent.DEACTIVATE);


  }

  public void fireClose()
  {
    if (DEBUG_ACTIVATION)
      debug("[CLOSE]     ");

    isActive = false;
    isClosed = true;
    fireActivationChanged(BActivationEvent.CLOSE);


  }


  // Internal methods...
  // -------------------------------------------------------------

  protected void fireActivationChanged(int type)
  {
    if (listeners != null)
    {
      BActivationEvent e = new BActivationEvent(source, type);
      BActivationListener l;

      for (int i=0; i<listeners.size(); i++)
      {
        l = (BActivationListener) listeners.get(i);
        l.activationChanged(e);
      }
    }
  }

  protected void debug(String type)
  {
    Class c = source.getClass();
    BDebug.print("Activation "+ type+ " - "+ c);
  }

}