package roboforge.ui.fusion;

import roboforge.BGlobal;
import roboforge.ui.components.HSBImageIcon;

import java.awt.*;
import javax.swing.AbstractButton;

public class HSBCheckBoxIcon extends HSBImageIcon
{
  public static final int STYLE_SQUARE = 0;
  public static final int STYLE_ROUND  = 1;

  protected int style;

  protected static RenderingHints hints = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_ON );

  protected static RenderingHints hintsOff = new RenderingHints (
    RenderingHints.KEY_ANTIALIASING,
    RenderingHints.VALUE_ANTIALIAS_OFF );


  protected static Color activeColor =
    BGlobal.colors.getDarkestShade();

  protected static Color focusColor =
    BGlobal.colors.getDarkShade();

  protected static Color checkColor =
    BGlobal.colors.getMidShade();

  protected static Color borderColor
    = BGlobal.colors.getDarkerShade();


  public HSBCheckBoxIcon(String name)
    { this(name, STYLE_SQUARE); }

  public HSBCheckBoxIcon(String name, int style)
  {
    super(name, checkColor);
    this.style = style;
  }


  public synchronized void paintIcon(Component c, Graphics g, int x, int y)
  {
    Graphics2D g2D = (Graphics2D) g;
    g2D.addRenderingHints(hints);

    AbstractButton b = (AbstractButton) c;

    int w = getIconWidth() -1;
    int h = getIconHeight() -1;

    switch (style)
    {
      case STYLE_SQUARE:
        // Draw a check box...
        g.setColor(getFillColor(c));
        g.fillRect(x, y, w, h);

        g.setColor(getBorderColor(c));
        g.drawRect(x, y, w, h);
        break;

      case STYLE_ROUND:
        // Draw a check box...
        g.setColor(getFillColor(c));
        g.fillOval(x, y, w, h);

        g.setColor(getBorderColor(c));
        g.drawOval(x, y, w, h);
        break;
    }

    if (b.isSelected())
      super.paintIcon(c, g, x, y);

    g2D.addRenderingHints(hintsOff);
  }

  protected Color getFillColor(Component c)
  {
    AbstractButton b = (AbstractButton) c;

    if (b.isSelected())
      return activeColor;
    else
      return b.getBackground();
  }

  protected Color getBorderColor(Component c)
  {
    if (c.hasFocus())
      return focusColor;
    else
      return borderColor;
  }


}
