package roboforge.ui.layers;

import roboforge.BGhost;
import roboforge.BGhostable;

import java.awt.*;
import javax.swing.*;



public class BSingleLayerStack extends BAbstractLayerStack
{

  // Member variables...
  // -------------------------------------------------------------
  
  protected BLayer top;

  
  // Constructor...
  // -------------------------------------------------------------
  
  public BSingleLayerStack(JComponent holder)
  {
    super(holder);
    top = null;
  }
  
  
  // Interface methods...
  // -------------------------------------------------------------
  
  public void push(BLayer l)
  {
    if (top != null)
    {
      hide(top);
      stack.push(top.createGhost());
    }
    
    top = l;
    show(top);
  }

  public BLayer peek()
    { return top; }

  public BLayer pop()
    { return pop(false); }
  
  public BLayer pop(boolean close)
  {
    if (top != null)
    {
      BLayer l = top;
      hide(l, close);
      
      BGhost g = stack.isEmpty() ? 
        null : (BGhost) stack.pop();
      
      if (g != null)
      {
        top = (BLayer) g.resurrect();
        show(top);
      }

      return l;
    }
    else
      return null;
  }
  
  
  public boolean remove(BLayer l)
    { return remove(l, false); }
  
  public boolean remove(BLayer l, boolean close)
  {
    // NB: If a layer has been ghosted, there
    // is no guarantee that the object in the
    // stack will be a layer.  Use this method
    // with caution!!
    
    if (l == null)
      return false;
    
    if (l == top)
      { pop(close); return true; }
    else
      return stack.remove(l);
  }
  
  public void moveToTop(BLayer l)
  {
    // NB: Same caveat as remove().

    if ((l == null) || (l == top))
      return;
    
    boolean removed =
      stack.remove(l);
    
    if (removed)
      push(l);
  }

  public boolean isOpen(BLayer l)
    { return ((l != null) && (l == top)); }
  
  public boolean contains(BLayer l)
    { return isOpen(l) || stack.contains(l); }
  
  public int size()
    { return (top == null) ? 0 : (stack.size() + 1); }
  
  
  
  // Activation support methods...
  // -------------------------------------------------------------

  protected void activateLayers()
  { 
    if (top != null)
      top.requestActivate(); 
  }
  
  protected void deactivateLayers()
  { 
    if (top != null)
      top.requestDeactivate(); 
  }
  
  protected void closeLayers()
  {
    if (top != null)
      top.requestClose();

    int n = stack.size();

    BGhost g;
    
    for (int i=0; i<n; i++)
    {
      g = (BGhost) stack.get(i);
      g.banish();
    }

    stack.clear();
  }
  
}
