package roboforge.ui.modes;

import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;

import javax.media.j3d.*;
import javax.vecmath.*;
import java.lang.Math.*;

import roboforge.*;
import roboforge.ui.*;


public abstract class BModeFOVZoom extends BModeArena
{

  protected BArenaCanvas canvas;
  protected double defaultFOV;
  protected boolean consoleHidden = false;

  protected static double minFOV = 0.05;
  protected static double zoomDelta = 0.01;




  public BModeFOVZoom(BArenaCanvas canvas)
  {
    super();
    this.canvas = canvas;

  }

  public void onActivate()
  {
    super.onActivate();
    defaultFOV = canvas.getFov();

    //BDebug.print ( this + " mouse Activate called " );

  }

  public void onDeactivate()
  {
    super.onDeactivate();

    canvas.setFov(defaultFOV);
    canvas.setConsoleVisible(true);

    //BDebug.print ( this + " mouse Deactivate called " );


  }

  public void mouseReleased(MouseEvent e)
  {
    super.mouseReleased(e);
    
    consoleHidden =false;
    canvas.setConsoleVisible(true);
  }

   protected void setFOV(double fov)
   {
        if (fov > defaultFOV)
          fov = defaultFOV;
        else if (fov < minFOV)
          fov = minFOV;

        canvas.setFov(fov);
   }

  protected void incrementFOV(int delta)
  {
    if (!consoleHidden)
    {
      consoleHidden  = true;
      canvas.setConsoleVisible(false);
    }

    double fov = canvas.getFov();
    fov += delta * zoomDelta;

    setFOV(fov);
  }



  public void mouseDragged(MouseEvent e)
  {
    super.mouseDragged(e);

    if (isMouseActive)
      if (e.isShiftDown()) // for zoom
        incrementFOV(dy);
  }



  protected static final int KEY_ZOOM_DELTA = 5;

  
  public void keyUpPressed(KeyEvent e)
  {
    if (e.isShiftDown())
      incrementFOV(-KEY_ZOOM_DELTA);
  }

  public void keyDownPressed(KeyEvent e)
  {
    if (e.isShiftDown())
      incrementFOV(+KEY_ZOOM_DELTA);
  }

  public void keyReleased(KeyEvent e)
  {
    super.keyReleased(e);

    if (consoleHidden)
    {
      consoleHidden = false;
      canvas.setConsoleVisible(true);
    }
  }


}