package roboforge.ui.net;

import roboforge.BBot;
import roboforge.BDebug;
import roboforge.BGlobal;
//import roboforge.STransferDetail;

import roboforge.ui.*;
import roboforge.ui.action.*;
import roboforge.ui.components.*;
//import roboforge.ui.events.*;
import roboforge.util.BSystemBrowser;
import roboforge.util.graphics.*;
import roboforge.util.io.*;
import roboforge.util.text.BStrings;

import roboforge.net.*;
import roboforge.net.client.*;
import roboforge.net.data.*;
import roboforge.net.services.*;
import roboforge.animation.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.text.*;
import java.util.*;



public class BEnterPanel extends BOnlinePanel
{

  // Internal variables...
  // -------------------------------------------------------------

  /**
	 * 
	 */
	private static final long serialVersionUID = -1L;
protected boolean               isRefreshRequired;
  protected DefaultTreeModel      treeModel;
  protected TreeSelectionModel    selectionModel;
  protected SelectionListener     selectionListener;

  protected BTournamentData       tournament;
  protected BTournamentEntryData  entry;

  protected BPromptDialog errorDialog = BPromptDialog.createNotifyDialog("Bot submission Error", "");

  public BSleepingThreadAnimator clockAnimator =
    new BSleepingThreadAnimator(1000);
    // every second


  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "treeTitle.text",     "tournament entry view",

    "clock.text",         "Server time:",
    "clock.width",        new Integer(250),
    "clock.h_align",      BLabel.ALIGN_RIGHT,
    "clock.font",         new Font("Dialog", Font.PLAIN, 10),

    "submit.name",        "Submit bot",
    "submit.icon",        BIconCache.getHSB("net_submit_icon.gif"),
    "submit.tooltip",     "Submit a bot to the selected tournament.",

    "remove.name",        "Withdraw bot",
    "remove.icon",        BIconCache.getHSB("net_remove_icon.gif"),
    "remove.tooltip",     "Withdraw selected bot from tournament.",

    "refresh.name",       "Refresh",
    "refresh.icon",       BIconCache.getHSB("net_refresh_icon.gif"),
    "refresh.tooltip",    "Refresh Tournament view.",

    "stopRef.name",       "Cancel refresh",
    "stopRef.icon",       BIconCache.getHSB("stop_icon.gif"),
    "stopRef.tooltip",    "Cancel refresh",
  });



  // Visual elements...
  // -------------------------------------------------------------

  protected BTitlebarLabel      treeTitle   = new BTitlebarLabel(res, "treeTitle");
  protected BTitlebarLabel      clock       = new BTitlebarLabel(res, "clock");

  protected BEntryTree          tree        = new BEntryTree();
  protected BScrollPane         scroller    = new BScrollPane(tree);
  protected TournamentDetails   details     = new TournamentDetails();

  protected BActionButton       submit      = new BActionButton();
  protected BActionButton       remove      = new BActionButton();
  protected BActionButton       refresh     = new BActionButton();


  // Actions...
  // -------------------------------------------------------------

  protected RefreshAction refreshAction = new RefreshAction(res, "refresh");
  protected StopRefreshAction stopRefAction = new StopRefreshAction(res, "stopRef");
  protected SubmitAction  submitAction  = new SubmitAction(res, "submit");
  protected RemoveAction  removeAction  = new RemoveAction(res, "remove");



  // Constructor.
  // -------------------------------------------------------------

  public BEnterPanel(BTournamentScreen parent)
  {
    super(parent, "enter");

    this.isRefreshRequired  = true;
    this.selectionModel     = tree.getSelectionModel();
    this.treeModel          = (DefaultTreeModel) tree.getModel();

    layoutEnterChildren();
    configEnterChildren();
  }


  // Initialization helper methods...
  // -------------------------------------------------------------

  protected void layoutEnterChildren()
  {
    layoutToolbar();

    BBoxPanel titleBar = new BBoxPanel(X_AXIS);
    titleBar.addHGlued(treeTitle);
    titleBar.add(clock);

    contentArea.add(titleBar);
    contentArea.addStrut(8);
    contentArea.add(scroller);

    scroller.setViewportBorder(new EmptyBorder(10,10,10,10));

  }

  protected void layoutToolbar()
  {
    toolbar.add(submit);
    toolbar.addSmallMargin();
    toolbar.add(remove);
    toolbar.addSmallMargin();
    toolbar.add(refresh);
  }

  protected void configEnterChildren()
  {
    clock.setHorizontalAlignment(BTitlebarLabel.RIGHT);
    clockAnimator.add(new ClockAnimation() );

    selectionListener = new SelectionListener();
    selectionModel.addTreeSelectionListener(selectionListener);

    openToolPanel(details);

    submit.setAction(submitAction);
    remove.setAction(removeAction);
    refresh.setAction(refreshAction);

    setActionStatus();
  }


  public class ClockAnimation implements BAnimation
  {
    public boolean animate()
    {
      if (server.getServerTimeZone()  != null )
      {
       // int minuteOffset = server.getServerMinuteOffset();

        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date(System.currentTimeMillis()));
        //cal.add (Calendar.MINUTE, -minuteOffset );
        //cal.setTimeZone( server.getServerTimeZone() );

        // SJS [07/04/2001] - Made the time update every second.
        DateFormat df =   DateFormat.getDateTimeInstance(DateFormat.MEDIUM,  DateFormat.LONG );
       // df.setTimeZone( server.getServerTimeZone() );

        //cal.setTimeZone(server.getServerTimeZone() );
        clock.setText("Time: "+ df.format(cal.getTime())+ "  ");
      }
      
      return true;
    }

    public void stop()
      {}

    public void start()
      { animate() ;}
  }

  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();
    clockAnimator.start();

    if (isRefreshRequired)
    {
      refreshAction.perform();
      isRefreshRequired = false;
    }
  }

  public void onDeactivate()
  {
    clockAnimator.stop();
    super.onDeactivate();
  }

  public void onClose()
  {
    if( !refreshAction.isStopped() )
      refreshAction.stop();

    // busy wait until the refresh thread has stopped
    while ( !refreshAction.isStopped() )
    {
      try {
        wait((long)20);
      }catch (InterruptedException e) {}
    }
    super.onClose();
  }

  // UI Listeners...
  // -------------------------------------------------------------
  // action for stopping a refresh that is currently in progress
  protected class StopRefreshAction extends BThreadedAction
  {
    public StopRefreshAction(BUIResources res, String key)
    { super(res, key); }
    public void swingPrepAction(ActionEvent e)
    { }
    public void threadedAction(ActionEvent e)
    { }
    public void swingCleanupAction(ActionEvent e)
    {
      setEnabled(false);
      refreshAction.stop();
    }
  }

  // refresh the entry panel
  protected class RefreshAction extends BThreadedAction
  {
    protected BMessage response;
    BTransmission transmission;
    protected boolean manualStop = false;

    public RefreshAction(BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
    {
      parent.setStatus("Refreshing tournament listings...");
      submitAction.setEnabled(false);
      removeAction.setEnabled(false);
      refresh.setAction(stopRefAction);
      parent.setWorking(true);
    }

    public void threadedAction(ActionEvent e)
    {
      try
      {
        transmission = new BTransmission(
          new BRequest(server, "GOT"));
        parent.setMonitored(transmission);
        response = server.request(transmission);
      }
      catch (BNetException ex)
      { 
        if(!manualStop)
          parent.setStatus("ERROR: "+ ex.getMessage());
        else
          parent.setStatus("Refresh terminated");

        response = null;
      }
      finally
      {
        parent.setMonitored(null);
      }
    }

    // terminate the refresh
    public void stop()
    {
      manualStop = true;          // the download has been interrupted by the user
      transmission.stop();        // stop the download thread
      tree.setEnabled(true);
      submitAction.setEnabled(false);
      removeAction.setEnabled(false);
      refresh.setAction(refreshAction);  // reset the play button action
    }
    // true if the refresh has stopped
    public boolean isStopped()
    {
      if (transmission != null)
        return transmission.isStopped();
      else
        return true;
    }
    public void swingCleanupAction(ActionEvent e)
    {
      if (response != null)
      {
        if (response.isAcknowledgement())
        {
          if(tree != null)
            tree.reload(response.getData());
          if(parent != null)
            parent.setStatus("Tournaments listings refreshed.");
        }
        else
        {
          if (parent != null)
            parent.setStatus("ERROR: "+ response);
        }
      }
      if(parent != null)
        parent.setWorking(false);

      submitAction.setEnabled(false);
      removeAction.setEnabled(false);
      refresh.setAction(refreshAction);  // reset the play button action

    }

  }


  protected class SubmitAction extends BThreadedAction
  {
    protected BMessage response;
    
    protected BFileDialog botDialog = new BFileDialog(
      "Submit bot",
      "Please select a bot to submit",
      "Submit", "Cancel", true);


    public SubmitAction(BUIResources res, String key)
    {
      super(res, key);

      botDialog.setFilter(
        new BRoboforgeFilter(BGlobal.BOTSUFFIX));

      try
        { botDialog.setPath(BGlobal.BOTPATH); }
      catch (FileNotFoundException ex)
        { }
    }


    public boolean validateBot(BTournamentSubmitData data)
    {
      if (data.botCost > tournament.botPriceCap)
      {
          errorDialog.setMessage
          (     data.botName + " is overpriced for this tournament. \n\n" +
                "Tournament has a price limit of " + (int)tournament.botPriceCap + "cred.\n" +
                data.botName + " costs " + (int)data.botCost + "cred."

          );

          BApplication.showDialog(errorDialog);

          parent.setStatus(data.botName + " costs " + (int)data.botCost + "cred. Tournament has a price limit of " + (int)tournament.botPriceCap + "cred.");
          return false;
      }

      Vector v = data.bot.validate(true);
      if (v.size() != 0)
      {

        String sText = "";


        for (int i = 0 ; ( i < v.size() && i < 3 ); i++)
          sText = sText + v.get(i) + "\n";

        errorDialog.setMessage
          (     data.botName + " contains the following errors which prevent it from entering internet tournaments \n\n" +
                sText

          );

          BApplication.showDialog(errorDialog);

          parent.setStatus(data.botName + " contains errors which prevent it from entering a internet tournament.");
          return false;
      }



      return true;

    }


    protected void displayPaymentOptions(boolean futurePayActive, BBalanceData balance)
    {

        BPromptDialog noMoneyDialog = BPromptDialog.createYesNoDialog("Insufficent funds", "Sorry you do not enough money in your account to enter this tournament.\n\nWould you like RF to take you to the accounts screen where you can add money to your account?" );
        BApplication.showDialog(noMoneyDialog);
        try
        {

          if (noMoneyDialog.getResult() )
          {
            if (futurePayActive)
              ((BTournamentScreen)parent).showBalancePanel(balance);
            else
              BSystemBrowser.displayURL( BGlobal.WORLDPAY_URL + "?M_ACCTID=" + server.getAccountID() );
          }


        }
        catch(Exception e)
        {
          setStatus("ERROR connecting to account, please visit the following web site: " + BGlobal.WORLDPAY_URL );
          BDebug.print ("BEnterPanel validateAccountBalance " + e.getMessage() );

        }

    }


    protected boolean validateAccountBalance(BTournamentData tournamentToEnter)
    {
      boolean feeOK = false;

      if (tournamentToEnter.entryFee == 0 )
        feeOK = true;
      else
      {

        BRequest r = new BRequest(server, "GATB");

        try
        {
          BTransmission t = new BTransmission(r);
          parent.setMonitored(t);

          BBalanceData balance = (BBalanceData)
            server.request(t).getData();

          float totalBalance = balance.currentBalance + balance.winningBalance;

          if (totalBalance <  tournamentToEnter.entryFee )
          {
            displayPaymentOptions(balance.isFuturePayActive, balance);
            feeOK = false;

          }
          else
            feeOK = true;


        }
        catch (BNetException ex)
        {
          errorDialog.setMessage ( ex.getMessage() );
          BApplication.showDialog(errorDialog);
          parent.setStatus("ERROR: "+ ex.getMessage());

        }

        parent.setMonitored(null);


      }

      return feeOK;


    }

    public void threadedAction(ActionEvent e)
    {
      response = null;

      if (tournament == null)
        return;

      if (!validateAccountBalance(tournament ) )
        return;

      BApplication.showDialog(botDialog);

      String file =
          botDialog.getSelectedFile();

      if (file != null)
      {
        parent.setStatus("Submitting entry to "+ tournament+ "...");
        parent.setBusyWorking(true);
      
        BTournamentSubmitData
          data = getBotData(file, tournament.getIDLong());

        if (data == null)
        {
          parent.setStatus("ERROR: Could not load bot.");
          return;
        }

        if (!validateBot(data))
          return;




        BRequest r = new BRequest(server, "SUB", data);

        try
        {
          BTransmission t = new BTransmission(r);
          t.setSendService(BServices.zip());

          parent.setMonitored(t);
          response = server.request(t);

        }
        catch (BNetException ex)
        {
          errorDialog.setMessage ( ex.getMessage() );
          BApplication.showDialog(errorDialog);
          parent.setStatus("ERROR: "+ ex.getMessage());

        }
        finally
          { parent.setMonitored(null); }
      }
    }


    public void swingCleanupAction(ActionEvent e)
    {
      parent.setBusyWorking(false);

      if (response != null)
      {
        if (response.isAcknowledgement())
        {
          // Store the new entry...
          response.getData().store(tree);
          parent.setStatus("Entry submitted successfully.");
        }
        else
        {
          if (errorDialog != null)
          {
            errorDialog.setMessage ( response.getData() + "" );
            BApplication.showDialog(errorDialog);
          }
          
          if (parent != null)
            parent.setStatus("ERROR: "+ response);
        }
      }
    }
    
    protected BTournamentSubmitData getBotData(String file, Long ID)
    {
      BBot bot = BBot.loadFromFile(file, false);
      
      if (bot == null)
        return null;
      
      bot.removeThumbnails();
      return new BTournamentSubmitData(bot, ID);
    }
  }

  
  protected class RemoveAction extends BThreadedAction
  {
    protected boolean  isRemoving;
    protected BMessage response;

    protected BPromptDialog removeDialog =
      BPromptDialog.createYesNoDialog("Withdraw entry?");

        
    public RemoveAction(BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
    {
      parent.setStatus("Withdrawing "+ entry+ " from tournament...");
      parent.setWorking(true);
    }

    public void threadedAction(ActionEvent e)
    {
      removeDialog.setMessage(
        "Do you wish to withdraw "+ entry+ " from the tournament?");

      BApplication.showDialog(removeDialog);
      isRemoving = removeDialog.getResult();
        
      if (isRemoving)
        try
        {
          parent.setBusy(true);
          
          BRequest r = new BRequest(server, "RBT", 
            new BIdentData(entry.getID()));

          BTransmission t = new BTransmission(r);
          response = server.request(t);
        }
        catch (BNetException ex)
        { 
          parent.setStatus("ERROR: "+ ex.getMessage());
          response = null;
        }
        finally
        { 
          parent.setBusy(false); 
        }
    }

    public void swingCleanupAction(ActionEvent e)
    {
      if (isRemoving)
      {
        if ((response != null) && response.isAcknowledgement())
        {
          remove.setEnabled(false);
          tree.removeEntry(entry.getIDLong());
          parent.setStatus(entry+ " withdrawn successfully.");
        }
        else
          parent.setStatus("ERROR: "+ response);
      }
      else
        parent.setStatus("Withdrawal cancelled.");
        
      parent.setWorking(false);
    }
  }


  // UI Listeners...
  // -------------------------------------------------------------

  protected class SelectionListener implements TreeSelectionListener
  {
    protected boolean enabled = true;

    public void setEnabled(boolean e)
      { enabled = e; }

    public boolean getEnabled()
      { return enabled; }

    public void valueChanged(TreeSelectionEvent e)
    {
      if (enabled)
      {
        TreePath path =
          selectionModel.getSelectionPath();

        if (path != null)
        {
          BDataNode node = (BDataNode)
            path.getLastPathComponent();

          tournament = null;
          entry      = null;

          if (node.isOfType(BDataTypes.TOURNAMENT))
          {
            tournament = (BTournamentData) node.getData();
            details.setTournament(tournament);
          }

          else if (node.isOfType(BDataTypes.TOURNAMENT_ENTRY))
            entry = (BTournamentEntryData) node.getData();
        }

        setActionStatus();
      }
    }
  }


  protected void setActionStatus()
  {
    submitAction.setEnabled(tournament != null);
    removeAction.setEnabled(entry != null);
  }




  // Tournament details panel...
  // -------------------------------------------------------------

  protected class TournamentDetails extends BToolPanel
  {

    // Member variables.
    // -----------------------------------------------------------

    protected BTournamentData tournament;


    // UI Resources ...
    // -----------------------------------------------------------

    protected BUIResources res = new BUIResources(new Object[]
    {
      "name.font",                new Font(BGlobal.FONT_NAME, Font.PLAIN, 14),
      "name.antialiased",         Boolean.TRUE,
      "name.height",              new Integer(20),

      // "desc.height",              new Integer(50),

      "image.height",             new Integer(50),

    });


    // Visual elements.
    // -----------------------------------------------------------

    protected BLabel name       = new BLabel(res, "name");
    protected BMultiLabel desc  = new BMultiLabel(res, "desc");
    protected BLabel image      = new BLabel(res, "image");



    // Constructor.
    // -----------------------------------------------------------

    public TournamentDetails()
    {
      super("selected tournament");

      layoutSelectedTournamentChildren();
      configSelectedTournamentChildren();
    }


    // Initialization helper methods...
    // -----------------------------------------------------------

    protected void layoutSelectedTournamentChildren()
    {
      addHGlued(name);
      addSmallMargin();

      addHGlued(image);

      addSmallMargin();
      add(desc);
    }

    protected void configSelectedTournamentChildren()
    {
    }


    // Accessor methods...
    // -----------------------------------------------------------

    public void setTournament(BTournamentData tournament)
    {
      this.tournament = tournament;


      name.setText(tournament.name);

      if ( tournament.description == null)
        tournament.description = "";


      String arenaName = BStrings.convertFromFilename(tournament.arena);

      if(arenaName.substring(0, 10).equals("/" + BGlobal.HIDDENPATH))
        arenaName = arenaName.substring(10, arenaName.length());

      String firstLetter = arenaName.substring(0,1);
      firstLetter = firstLetter.toUpperCase();

      arenaName = firstLetter + arenaName.substring(1, arenaName.length());


      desc.setText(tournament.description + "\n\n");

      String typeString = "";

      if(tournament.tournamentType != null)
      {
        if(tournament.tournamentType.equals("PN"))
          typeString = "Professional (Nonranked)";
        if(tournament.tournamentType.equals("AN"))
          typeString = "Amateur (Nonranked)";
        if(tournament.tournamentType.equals("PR"))
          typeString = "Professional (Ranked)";
        if(tournament.tournamentType.equals("AR"))
          typeString = "Amateur (Ranked)";
      }
      desc.append("Type:\t\t" + typeString + "\n");
      desc.append("Arena:\t\t" + arenaName + "\n");
      desc.append("Bot cost limit:\t\t" + (int)tournament.botPriceCap+"\n");
      //desc.append("Closing date:\t\t"+ tournament.closingDate+"\n");
      Calendar closingDate = (Calendar) tournament.closingDate.clone();


      //cal.setTime(new Date(System.currentTimeMillis()));
      int minuteOffset = server.getServerMinuteOffset();
      closingDate.add (Calendar.MINUTE, -(minuteOffset % 60) );

      DateFormat df = DateFormat.getDateTimeInstance();
      ///df.setTimeZone(closingDate.getTimeZone());

      //BDebug.print ("BEnterPanel " + closingDate.getTime() );

      desc.append("Closing date:\t\t"+ df.format(closingDate.getTime()) +"\n" );

      //desc.append("Closing date:\t\t"+ DateFormat.getDateTimeInstance().format(closingDate) +"\n");

      desc.append("Max. player entries:\t"+ tournament.allowedEntriesPerAccount + "\n");
      desc.append("Max. entries:\t\t"+ tournament.maximumEntries+"\n");
      desc.append("Entries to date:\t\t"+ tournament.currentEntries+"\n");
      desc.append("Length of fights:\t"+ tournament.fightDuration+" seconds\n");
      desc.append("Entry fee:\t\t$"+ tournament.entryFee+"\n");

      Image img = tournament.getImage();

      if (img != null)
        image.setIcon(new ImageIcon(img));

    }

  }

}
