package roboforge.ui.net;


import roboforge.BGlobal;
import roboforge.BProgressMonitorable;

import roboforge.net.*;
import roboforge.net.client.*;
import roboforge.net.data.*;

import roboforge.ui.*;
import roboforge.ui.action.*;
import roboforge.ui.components.*;
import roboforge.util.graphics.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import java.net.*;
import java.io.*;

import roboforge.BDebug;

import roboforge.util.processors.*;

import sun.net.www.content.text.PlainTextInputStream;


public class BPatchScreen extends BBasicScreen
{


  // Member variables...
  // -------------------------------------------------------------

  protected BClientStack server;
  protected BAutoPatcher patcher;
  protected Timer timer;


  // UI Resources ...
  // -------------------------------------------------------------

  protected BUIResources res = new BUIResources( new Object[]
  {
    "title.bounds",               new Rectangle(230, 294, 300, 30),
    "title.foreground",           colors.getMidShade(),
    "title.icon",                 BIconCache.getHSB("patch_title.gif"),


    "cogs.bounds",                new Rectangle(130, 294, 100, 100),
    "cogs.icon",                  BGraphics.getHSBIcon("anim_cogs.gif", colors.getMidShade()),

    "progress.bounds",            new Rectangle(230, 334, 440, 20),
    "message.bounds",             new Rectangle(230, 364, 440, 20),
    "message.foreground",         colors.getLighterShade(),

    "background.foreground",      colors.getMidShade(),
    "background.bounds",          new Rectangle(0, 1, 800, 450),

    "action.bounds",              new Rectangle(590, 364, 80, 30),
    
    "cancelAction.name",      "Cancel",
    "cancelAction.tooltip",   "Cancel the auto-patching process.",

    "restartAction.name",     "Restart",
    "restartAction.tooltip",  "Restart roboforge."

    /////

  });


  // Visual elements ...
  // -------------------------------------------------------------

  protected BLabel              title    = new BLabel(res, "title");
  protected BAnimatedIconLabel  cogs     = new BAnimatedIconLabel(res, "cogs");
  protected BProgressBar        progress = new BProgressBar(res, "progress");
  protected BLabel              message  = new BLabel(res, "message");

  // protected BLabel              number   = new BLabel(res, "number");
  protected BButton             action   = new BButton(res, "action");
  protected BLabel              background  = new BLabel(res, "background");
  
  
  // Actions...
  // -------------------------------------------------------------

  protected CancelAction  cancelAction    = new CancelAction(res, "cancelAction");
  protected RestartAction restartAction   = new RestartAction(res, "restartAction");
  

  
  // Constructor ...
  // -------------------------------------------------------------

  public BPatchScreen(BClientStack stack)
  {
    super();

    this.server  = stack;
    this.patcher = new BAutoPatcher(stack);
    
    layoutPatchChildren();
    configPatchChildren();


  }


  protected void layoutPatchChildren()
  {
    body.setLayout(null);

    body.addOnBottom(background);
    background.setIcon(BGraphics.getHSBIcon("background_patch.jpg"));

    body.addOnTop(title);
    body.addOnTop(cogs);
    body.addOnTop(progress);
    body.addOnTop(message);
    // body.addOnTop(number);
    body.addOnTop(action);


    //setBackgroundIcon(BGraphics.getHSBIcon("background_patch.jpg"));
  }

  protected void configPatchChildren()
  {
    timer = new Timer(50, new TimerListener());
    action.setAction(cancelAction);
  }



  // Activation methods...
  // -------------------------------------------------------------

  public void onActivate()
  {
    super.onActivate();

    try
      { patcher.start(); }
    catch (BAutoPatcherException e)
      { BApplication.abort("ERROR: "+ e); }
    
    timer.start();
  }



  // Action classes...
  // -------------------------------------------------------------
  
  protected class CancelAction extends BThreadedAction
  {
    protected BPromptDialog confirmation
      = BPromptDialog.createYesNoDialog(
        "Cancel upgrade?", 
        "You will not be able to enter roboforge.net "+
        "until you have updated to the most recent "+
        "version of our product.\n\n"+
        "Do you want to cancel the update process?");
      
    protected boolean isStopping;
      
    public CancelAction(BUIResources res, String key)
      { super(res, key); }

    public void swingPrepAction(ActionEvent e)
      { }

    public void threadedAction(ActionEvent e)
    {
      BApplication.showDialog(confirmation);
      isStopping = confirmation.getResult();
    }

    public void swingCleanupAction(ActionEvent e)
    {
      setEnabled(!isStopping);

      if (isStopping)
        patcher.stop();
    }
  }

  protected class RestartAction extends BAction
  {
    public RestartAction(BUIResources res, String key)
      { super(res, key); }

    public void actionPerformed(ActionEvent e)
      { BApplication.restart(); }
  }



  // UI Listeners...
  // -------------------------------------------------------------

  protected class TimerListener implements ActionListener
  {
    public void actionPerformed(ActionEvent e)
    {
      if (patcher.isBusy())
        updateDisplay();
      else
      {
        updateDisplay();
        timer.stop();

        if (patcher.isError())
        {
          BProcessor p = new BProcessor()
          {
            public void process()
              { patchError(); }
          };

          p.start();

          goBack();
        }
        else
          goOnline();
      }
    }

    protected void patchError()
    {
      BPromptDialog patchFail;
      String errorMessage = "";
      String line;

      try{
        URL u = new URL("http://www.roboforge.net/status/server.txt");
        PlainTextInputStream c = (PlainTextInputStream)u.getContent();
        InputStreamReader r = new InputStreamReader(c);

        BufferedReader br = new BufferedReader(r);

        line = br.readLine();
        while(line != null)
        {
          errorMessage = errorMessage + line + "\n";
          line = br.readLine();
        }

        // errorMessage = br.readLine();
        //BDebug.print("BPatchScreen - " + br.readLine());
        c.close();
        br.close();
      }
      catch(Exception e) {errorMessage = "Could not connect to the server.  Please try again later.";}

      patchFail = BPromptDialog.createOKDialog("Error connecting to the server",
                                           errorMessage);
      BApplication.showDialog(patchFail);
    }
  }

  public void notifyAndExit()
  {
          BPromptDialog restartD = BPromptDialog.createOKDialog("System Patch applied",
                                            "The RoboForge software has been updated, RoboForge will now restart." );
          BApplication.showDialog(restartD);
          //action.setAction(restartAction);
          try {
            BGlobal.swapInVideoMode();
          }
          catch(Exception e) {}
          BApplication.restart();

  }


  protected void updateDisplay()
  {
    if (patcher != null)
    {
      int    p = patcher.getProgressPercent();
      String s = patcher.getProgressMessage();
      
      /*
      String n = 
        "patch "+ patcher.getCurrentPatch()
        + " of "+ patcher.getPatchCount();

      number.setText(n);
      */
      
      progress.setValue(p);
      message.setText(s);

      cogs.animation();
    }
  }

  
  protected void goOnline()
  {

    if( patcher.isRestartRequired() )
    {
      BProcessor p = new BProcessor()
      {
        public void process()
        { notifyAndExit(); }
      };
      p.start();
    }

    else
    {
      BTournamentScreen tournaments =
        new BTournamentScreen(server);

      BApplication.openScreen(tournaments);
      BApplication.closeScreen(this);
    }
  }

  protected void goBack()
    { BApplication.closeScreen(); }

}