package roboforge.util.graphics;

import java.io.*;
import java.awt.image.*;
import roboforge.BDebug;
import com.sun.image.codec.jpeg.*;

public class BSerializableImage implements Serializable
{
  static final long serialVersionUID = -1L;

  private transient BufferedImage image;
  byte[] jpgForm;  // the serialized form

  private int height;
  private int width;
  private int imageType;

  boolean dirty = false;


  public BSerializableImage(BufferedImage image)
  {
    setImage( image);
  }

  public void setImage(BufferedImage image)
  {
    dirty = true;
    this.image = image;
  }

  private void createImage()
  {
    if ( jpgForm != null)
    {

     try
     {
      ByteArrayInputStream comvertStream = new ByteArrayInputStream(jpgForm);

      JPEGImageDecoder jdcoder =  JPEGCodec.createJPEGDecoder(comvertStream);
      JPEGDecodeParam param = jdcoder.getJPEGDecodeParam();
      image =  jdcoder.decodeAsBufferedImage();

      /* old decoder
      image = new  BufferedImage(width, height,imageType);
      WritableRaster raster = image.getRaster();
      raster.setPixels(0,0, width, height, serializedForm);
      image.setData(raster); */
     }catch (Exception e) {BDebug.print("BSerializedImage.createImage ERROR " + e.getMessage() );}
    }

  }

  public BufferedImage getImage()
  {
    if (image == null)
      createImage();

    return image;
  }

  private void writeObject (ObjectOutputStream stream) throws IOException
  {
    if (dirty && (image != null))
    {
     ByteArrayOutputStream convertStream = new ByteArrayOutputStream() ;

     JPEGImageEncoder jie = JPEGCodec.createJPEGEncoder(convertStream);
     JPEGEncodeParam param = jie.getDefaultJPEGEncodeParam(image);
     param.setQuality(1.0f,true);
     jie.setJPEGEncodeParam(param);
     jie.encode(image);

     jpgForm = convertStream.toByteArray();


    /*
    BufferedOutputStream bos = new BufferedOutputStream(fos);
     JPEGImageEncoder jie = JPEGCodec.createJPEGEncoder(bos);
     JPEGEncodeParam param = jie.getDefaultJPEGEncodeParam(botImage);
     param.setQuality(1.0f,true);
     jie.setJPEGEncodeParam(param);
     jie.encode(botImage);

     bos.flush();
     fos.close(); */


      /* Old Serialization
      dirty = false;
      Raster r = image.getRaster();
      width = r.getWidth();
      height = r.getHeight();
      imageType = image.getType();

      serializedForm = new int[width * height * r.getNumBands()  ];
      serializedForm = r.getPixels(r.getMinX() , r.getMinY() , width,height, serializedForm);
      //serializedForm = image.getRGB(0, 0, width, height,serializedForm, 0, 1) ;
      */
    }

    stream.defaultWriteObject();
  }

  private void readObject (ObjectInputStream stream) throws Exception
  {
    stream.defaultReadObject();  
  }




}