package roboforge.util.parser;

public class BSymbol
{

  // Member variables...
  // -------------------------------------------------------------

  protected int id;
    // Unique integer ID, indicating symbol-type.

  protected int state;
    // Index of the 'state' in which this symbol

  protected Object value;
    // The 'value' of this symbol.


  // Class variables...
  // -------------------------------------------------------------

  public static final BSymbol START_SYMBOL = new BStartSymbol();
    // The 'start symbol' for a parse-stack.


  // Symbol IDs
  // -------------------------------------------------------------

  public static final int NOTHING            = -1;
  public static final int START              = -2;



  // Constructors...
  // -------------------------------------------------------------

  public BSymbol(int id)
    { this(id, 0, null); }

  public BSymbol(int id, Object value)
    { this(id, 0, value); }

  public BSymbol(Object value)
    { this(NOTHING, 0, value); }

  public BSymbol(BSymbol s)
    { this(s.id, s.state, s.value); }

  public BSymbol(int id, int state, Object value)
  {
    this.id    = id;
    this.state = state;
    this.value = value;
  }


  // Accessors...
  // -------------------------------------------------------------

  public String getName()
    { return (value != null) ? value.toString()+":"+ state : ""; }

  public String toString()
    { return getName(); }


  public int getID()
    { return id; }

  public int getState()
    { return state; }

  public void setState(int state)
    { this.state = state; }

  public Object getValue()
    { return value; }

  public void setValue(Object value)
    { this.value = value; }



  // Start symbol class...
  // -------------------------------------------------------------

  protected static class BStartSymbol extends BSymbol
  {
    public BStartSymbol()
      { super(START, 0, (Object) null); }

    public String getName()
      { return "$"+":"+ state; }
  }


  // Start symbol class...
  // -------------------------------------------------------------

  protected static class BNonTerminal extends BSymbol
  {
    protected String name;

    public BNonTerminal(String name, Object value)
    {
      super(NOTHING, 0, value);
      this.name = name;
    }

    public String getName()
      { return name+":"+ state; }
  }

  public static BSymbol nonTerminal(String name, Object value)
    { return new BNonTerminal(name, value); }


}
