package roboforge.util.processors;

import roboforge.BDebug;


public class BProcessor
  implements Runnable
{

  public static final int FOREVER = -1;

  protected Thread   thread;
  protected int priority = Thread.NORM_PRIORITY;
  protected int n = 1;
  protected boolean isProcessing;

  public BProcessor()
  {
    thread = null;
    isProcessing = false;
  }

  public void start()
    { start(1); }

  public void start(int loopCount)
  {
    this.n = loopCount;

    thread = new Thread(this, "BProcessor.thread");
    thread.setPriority(priority);
    thread.start();

    isProcessing = true;
  }

  public void stop()
  {
    thread = null;
    isProcessing = false;
  }

  public synchronized boolean isProcessing()
    { return isProcessing; }


  public void startAndWait()
  {
    // Remember what thread invoked this method.
    Thread current = Thread.currentThread();

    // Begin processing.
    start();

    // Wait for processing to complete...
    while (isProcessing())
    {
      try
        { current.sleep(100); }

      catch (InterruptedException e)
        { BDebug.print("BProcessor.processAndWait(): ERROR - Waiting interrupted."); }
    }
  }



  public synchronized void setPriority(int p)
  {
    priority = p;

    if (thread != null)
      thread.setPriority(p);
  }

  public void process()
    // Does some processing.
  {
  }


  public void run()
    // Processing loop.
  {
    Thread t = Thread.currentThread();

    while (thread == t)
    {
      process();
      if (n != FOREVER)  n--;
      if (n == 0)        stop();
    }
  }

}