package roboforge.util.text;

import java.util.StringTokenizer;
import roboforge.BDebug;



public class BStrings
{

  public static final String WHITESPACE = " \t\n\r\f";
  public static final char   SPACE      = ' ';
  public static final char   UNDERSCORE = '_';
  public static final char   FORWARD_SLASH = '/';
  public static final char   BACK_SLASH = '\\';



  protected static final String VALID_FILE_CHARS =
    "abcdefghijklmnopqrstuvwxyz"+
    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"+
    "0123456789"+
    " ";




  public static String convertToFilename(String s)
  {
    if (s == null)
      return null;

    s = s.toLowerCase();
    s = filter(s, VALID_FILE_CHARS);
    s = s.replace(SPACE, UNDERSCORE);

    return s;
  }


  public static String convertFromFilename(String s)
  {
    if (s == null)
      return null;

    s = s.replace(UNDERSCORE, SPACE);
    s = toTitleCase(s);

    return s;
  }

  public static String convertToFilePath(String s)
  {
    if (s == null)
      return null;
    
    s = s.toLowerCase();
    s = s.replace(SPACE, UNDERSCORE);
    s = s.replace(BACK_SLASH, FORWARD_SLASH);

    return s;
  }


  public static String stripExtension(String s)
  {
    if (s == null)
      return null;
    
    int dot = s.lastIndexOf('.');
    
    if (dot == -1)
      return s;
    
    return s.substring(0, dot);
  }


  public static final String filter(String s, String validCharacters)
  {
    if (s == null)
      return null;

    char[] c = s.toCharArray();
    int n = c.length;

    StringBuffer buf = new StringBuffer();

    for (int i=0; i<n; i++)
      if (validCharacters.indexOf(c[i]) >= 0)
        buf.append(c[i]);

    return buf.toString();
  }


  public static String toTitleCase(String s)
  {
    if (s == null)
      return null;

    else if (s.length() == 0)
      return s;

    StringBuffer result =
      new StringBuffer(s.length());

    StringTokenizer splitter =
      new StringTokenizer(s, WHITESPACE, true);

    while (splitter.hasMoreTokens())
      result.append(toSentenceCase(splitter.nextToken()));

    return result.toString();
  }


  public static String toSentenceCase(String s)
  {
    if (s == null)
      return null;

    else if (s.length() == 0)
      return s;

    StringBuffer result = 
      new StringBuffer(s.toLowerCase());
    
    result.setCharAt(0, 
      Character.toUpperCase(result.charAt(0)));

    return result.toString();
  }

  public static String toLowerCase(String s)
  { 
    if (s == null)
      return null;

    return s.toLowerCase();
  }

  public static String toUpperCase(String s)
  {
    if (s == null)
      return null;

    return s.toUpperCase();
  }


  public static String padLeftWithChar(String s, char padder, int length)
    // Pads string s on the left with 'padder' char, such that the
    // resulting string is at least 'length' chars long.
    // If s.length >= length, nothing happens.

  {
    int sl = s.length();

    if (sl >= length)
      return s;

    int n = length - sl;

    // Append n padder chars to result.
    String result = repeatChar(padder, n);

    // Append the string to be padded.
    result += s;

    return result;
  }


  public static String repeatChar(char c, int n)
    // Creates a string corresp. to n reps of char c.
  {
    StringBuffer result = new StringBuffer();

    // Append string s n times to result.
    for (int i=0; i<n; i++) result.append(c);

    return result.toString();
  }


  public static String repeatString(String s, int n)
    // Creates a string corresp. to n reps of string s.
  {
    StringBuffer result = new StringBuffer();

    // Append string s n times to result.
    for (int i=0; i<n; i++) result.append(s);

    return result.toString();
  }


}